// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------

#ifndef __Complex__
#define	__Complex__
#include <math.h>
class	Cmplx {
	public:
		float	x ;
		float	y ;
		Cmplx(float rl=0.,float im=0.) : x(rl),y(im) {} ;
		float	Real() { return x ; } ;
		float	Imag() { return y ; } ;
		float	Modulus() { return sqrt(x*x+y*y) ; } ;
		float	Modsq() { return (x*x+y*y) ; } ;
		double	Phase() { return atan2(y,x) ; } ;
		Cmplx& operator=(const float v) {this->x=v;this->y=0.;return *this;};
		Cmplx& operator*=(const float v) {this->x*=v;this->y*=v;return *this;} ;
		Cmplx& operator/=(const float v) {this->x/=v;this->y/=v;return *this;} ;
		Cmplx& operator+=(const float v) {this->x+=v;this->y+=v;return *this;} ;
		Cmplx& operator-=(const float v) {this->x-=v;this->y-=v;return *this;} ;
		Cmplx& operator*=(const Cmplx &v) {Cmplx r(*this);this->x=(r.x*v.x-r.y*v.y);
				this->y=(r.x*v.y+r.y*v.x);return *this;} ;
		Cmplx& operator/=(const Cmplx &v) {Cmplx r(*this);float l=(v.x*v.x+v.y*v.y);
				this->x=(r.x*v.x+r.y*v.y)/l; this->y=(v.x*r.y-r.x*v.y)/l ; return *this;} ;
		Cmplx& operator+=(const Cmplx &v) {this->x+=v.x;this->y+=v.y;return *this;} ;
		Cmplx& operator-=(const Cmplx &v) {this->x-=v.x;this->y-=v.y;return *this;} ;
		
	// Friend function operators
		friend Cmplx operator*(const Cmplx& c,const float v) {Cmplx r(c); r*=v; return r;} ;
		friend Cmplx operator/(const Cmplx& c,const float v) {Cmplx r(c); r/=v; return r;} ;
		friend Cmplx operator+(const Cmplx& c,const float v) {Cmplx r(c); r+=v; return r;} ;
		friend Cmplx operator-(const Cmplx& c,const float v) {Cmplx r(c); r-=v; return r;} ;
		friend Cmplx operator*(const float v,const Cmplx& c) {Cmplx r(v); r*=c; return r;} ;
		friend Cmplx operator/(const float v,const Cmplx& c) {Cmplx r(v); r/=c; return r;} ;
		friend Cmplx operator+(const float v,const Cmplx& c) {Cmplx r(v); r+=c; return r;} ;
		friend Cmplx operator-(const float v,const Cmplx& c) {Cmplx r(v); r-=c; return r;} ;
		friend Cmplx operator*(const Cmplx& c1,const Cmplx& c2) {Cmplx r(c1); r*=c2; return r;} ;	
		friend Cmplx operator/(const Cmplx& c1,const Cmplx& c2) {Cmplx r(c1); r/=c2; return r;} ;
		friend Cmplx operator+(const Cmplx& c1,const Cmplx& c2) {Cmplx r(c1); r+=c2; return r;} ;	
		friend Cmplx operator-(const Cmplx& c1,const Cmplx& c2) {Cmplx r(c1); r-=c2; return r;} ;	
		
//		friend	Cmplx &complex(float x,float y) { Cmplx r(x,y) ; return r ; } ;
} ;
#endif
