// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------


#include "TBrightnessControl.h"
#include "TContrastControl.h"
#include "globals.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "XDrivers.h"
long near(float x) ;
extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;
void TBrightnessControl::Draw() {
	Rect tRect = mRect ;
	switch(mOrientation) {
		case kHorizontal:
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			OffsetRect(&tRect,0,1) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			DrawLine((tRect.left+tRect.right)/2,tRect.top-8,(tRect.left+tRect.right)/2,tRect.bottom) ;
			break ;
		case kVertical:
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			OffsetRect(&tRect,1,0) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			DrawLine(tRect.left,(tRect.top+tRect.bottom)/2,tRect.right+8,(tRect.top+tRect.bottom)/2) ;
			break ;
	}
	mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
	DrawMinValue() ;
	DrawMaxValue() ;
	DrawCurrentValue() ;
	DrawControl() ;
	mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
	if (mLabel != nil) DrawLabel() ;
	SetFgColor(gBlackPixel) ;
}

void	TBrightnessControl::DrawMinValue() {
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	char	text[32] ;
	sprintf(text,"-") ;
	
	switch(mOrientation) {
		case kHorizontal:
			// Find the position of the label ;
			valPosY = tRect.top - 4 ;
			valPosX = tRect.left - StringWidth(text)/2 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
		case kVertical:
			valPosX = tRect.right +4 ;
			valPosY = tRect.bottom+4 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
		case kDiagonal:
			break ;
	}
}

void	TBrightnessControl::DrawMaxValue() {
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	char	text[32] ;
	sprintf(text,"+") ;
	switch(mOrientation) {
		case kHorizontal:
			// Horizontal
			// Find the position of the label ;
			valPosY = tRect.top - 4 ;
			valPosX = tRect.right - StringWidth(text)/2 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
		case kVertical:
			// Vertical
			valPosX = tRect.right +4 ;
			valPosY = tRect.top+4 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
	}
}

void	TBrightnessControl::DrawControl()  {
	Rect tRect = mControlRect ;
	InsetRect(&tRect,1,1) ;
	short	valPosX,valPosY ;
	SetFgColor(gGrayPixel) ;
	FillRect(&tRect) ;
	switch(mOrientation) {
		case kHorizontal:
			valPosY = mRect.bottom ;
			valPosX = near(mRect.left+(mRect.right-mRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue)) ;
			SetFgColor(gWhitePixel) ;
			MoveTo(valPosX-mControlPartWidth/2,valPosY+mControlPartHeight/2) ;
			LineTo(valPosX-mControlPartWidth/2,valPosY-mControlPartHeight/2) ;
			LineTo(valPosX+mControlPartWidth/2,valPosY-mControlPartHeight/2) ;
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			LineTo(valPosX+mControlPartWidth/2,valPosY+mControlPartHeight/2) ;
			LineTo(valPosX-mControlPartWidth/2,valPosY+mControlPartHeight/2) ;
			SetFgColor(gWhitePixel) ;
			DrawLine((mControlRect.left+mControlRect.right)/2-1,mControlRect.top,
				(mControlRect.left+mControlRect.right)/2-1,mControlRect.bottom) ;
			DrawLine((mControlRect.left+mControlRect.right)/2+1,mControlRect.top,
				(mControlRect.left+mControlRect.right)/2+1,mControlRect.bottom) ;
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			DrawLine((mControlRect.left+mControlRect.right)/2,mControlRect.top,
				(mControlRect.left+mControlRect.right)/2,mControlRect.bottom) ;
			break ;
		case kVertical:
			valPosX = mRect.left-1 ;
			valPosY = near(mRect.top+(mRect.bottom-mRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue)) ;
			SetFgColor(gWhitePixel) ;
			MoveTo(valPosX-mControlPartHeight/2,valPosY+mControlPartWidth/2) ;
			LineTo(valPosX-mControlPartHeight/2,valPosY-mControlPartWidth/2) ;
			LineTo(valPosX+mControlPartHeight/2,valPosY-mControlPartWidth/2) ;
			mIsActive ? SetFgColor(gBlackPixel) : SetFgColor(gGrayPixel) ;
			LineTo(valPosX+mControlPartHeight/2,valPosY+mControlPartWidth/2) ;
			LineTo(valPosX-mControlPartHeight/2,valPosY+mControlPartWidth/2) ;
			break ;
		case kDiagonal:
			break ;
	}
}

void	TBrightnessControl::EraseControl() {
	short	valPosX,valPosY ;
	Rect	tRect = mControlRect ;
	switch(mOrientation) {
		case kHorizontal:
			// Erase the control at it's current position
			SetFgColor(gGrayPixel) ;
			DrawCurrentValue() ;
			FillRect(&mControlRect) ;
			FrameRect(&mControlRect) ;
			SetFgColor(gBlackPixel) ;
			break ;
		case kVertical:
			// Erase the control at it's current position
			SetFgColor(gGrayPixel) ;
			DrawCurrentValue() ;
			FillRect(&mControlRect) ;
			FrameRect(&mControlRect) ;
			SetFgColor(gBlackPixel) ;
			break ;
		case kDiagonal:
			break ;
	}
}

void	TBrightnessControl::SetControlRect() {
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	switch(mOrientation) {
		case kHorizontal:
			valPosY = tRect.bottom ;
			valPosX = tRect.left+(tRect.right-tRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetRect(&mControlRect,valPosX-mControlPartWidth/2,valPosY-mControlPartHeight/2,
				valPosX+mControlPartWidth/2,valPosY+mControlPartHeight/2) ;
			break ;
		case kVertical:
			valPosX = tRect.left ;
			valPosY = tRect.top+(tRect.bottom-tRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetRect(&mControlRect,valPosX-mControlPartHeight/2,valPosY-mControlPartWidth/2,
				valPosX+mControlPartHeight/2,valPosY+mControlPartWidth/2) ;
			break ;
		case kDiagonal:
			break ;
	}
}
