// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "TButton.h"
#include "globals.h"
#include "XDrivers.h"
#define EOS '\0'
#include "TButton.h"
#include "TWindow.h" 
Boolean	IsDoubleClick() ;

extern	long	near(float) ;
extern	short	gNumGrayLevels ;
extern	short	gDebugMode ;

#include "TDMApplicationWindow.h"
extern TMainPane	*gTheMainPane ;


TPushButton	*gCurrentTool = nil ;

TPushButton	*GetCurrentTool() ; 
void		SetCurrentTool(TPushButton	*) ;

void	SetCurrentTool(TPushButton *tool) {
	if (gCurrentTool != nil)
		gCurrentTool->SetState(false);
	tool->SetState(true);
	gCurrentTool=tool;
} 

TPushButton	*GetCurrentTool() { return  gCurrentTool ; } 

TObject::TObject(Rect *rect,char *title) {

	if (rect != nil)
		mRect = *rect ;
	else 
		SetRect(&mRect,0,0,0,0) ;
	mIsActive = true ;	// Default
	mState = OFF ;		// Default
	mIsVisible = true ;	// Default
	mLabel = nil ;
	mActionRoutine = nil ;
	mTrackingRoutine = nil ;
	mNumLabelLines = 0 ;
	mLabelPosition = kLabelInside ;
	mJustification = kLeftJustified ;
	if (title != nil) CreateLabel(title) ;
	mParentWindow = nil ;
}

TObject::TObject(short x1,short y1, short x2, short y2,char *title) {

	SetRect(&mRect,x1,y1,x2,y2) ;
	mIsActive = true ;	// Default
	mState = OFF ;		// Default
	mIsVisible = true ;	// Default
	mLabel = nil ;
	mActionRoutine = nil ;
	mTrackingRoutine = nil ;
	mNumLabelLines = 0 ;
	mLabelPosition = kLabelInside ;
	mJustification = kLeftJustified ;
	if (title != nil) CreateLabel(title) ;
	mParentWindow = nil ;
}

TObject::~TObject() {
	if (mLabel != nil) delete[] mLabel ;
}

void TObject::Update() {
	if (mParentWindow==nil) return ; 
	if (!mParentWindow->IsVisible()) return ;
	
	GrafPtr tPort ;
	GetPort(&tPort) ;
	SetPort(mParentWindow) ;
	Draw() ;
	SetPort(tPort) ;
}

void TObject::CreateLabel(char *title) {
	short	whichLine ;
	char	*cp,*cpStart ;

	mNumLabelLines = 1 ;
	//mLabel = new Str256[1] ;
	//strcpy(mLabel[0],title) ;

	
	// Count number of lines to draw
	for(cp=title; *cp != '\0' ; cp++) {
		if (*cp == '\n')
			mNumLabelLines++ ;
	}
	mLabel = new Str256[mNumLabelLines] ;
	if (mNumLabelLines == 1) {
		strcpy(mLabel[0],title) ;
	}
	else {
		whichLine = 0 ;
		cpStart=title ;
		short	numChars = 0 ;
		for(cp=title; *cp != '\0' ;cp++) {
			if (*cp == '\n') {
				// Store the line
				strncpy(mLabel[whichLine],cpStart,numChars) ;
				mLabel[whichLine][numChars] = '\0' ;
				cpStart=cp+1 ;
				whichLine++ ;
				numChars = 0 ;
			}
			else {
				numChars++ ;
			}
		}
		// We have reached the end of the line
		// Copy the last part
		strcpy(mLabel[whichLine],cpStart) ;
	}
}


void TObject::SetItemText(char *text) {
	if (mLabel != nil) delete[] mLabel ;
	CreateLabel(text) ;
	Draw() ;
}

void TObject::GetItemText(char *text) {
	// Only return the first line of the text
	if (mLabel == nil) 
		strcpy(text,"") ;
	else
		strcpy(text,mLabel[0]) ;
}


void	TObject::DoEvent(XEvent *theEvent) {
	if (!mIsActive) return ;
	// What type of event was it
	switch(theEvent->type) {
		case ButtonPress:
			Point	thePoint ;
			thePoint.x = theEvent->xbutton.x ;
			thePoint.y = theEvent->xbutton.y ;
			DoMouseDown(&thePoint) ;
			break ;
		case KeyPress:
			DoKeyDown(theEvent) ;
			break ;
	}
}



TObjectEntry::TObjectEntry() {
	mNextObjectEntry = nil ;
	mObject = nil ;
}

TObjectEntry::~TObjectEntry() {
	mNextObjectEntry = nil ;
	if (mObject != nil)
		delete mObject ;
}


TObjectList::TObjectList() {
	mNumObjects = 0 ;
	mObjectEntry = nil ;
}

TObjectList::~TObjectList() {
	TObjectEntry *entry ;
	TObjectEntry *nextEntry ;
	
	// For every object entry
	entry = mObjectEntry ;
	while(entry != nil) {
		nextEntry = entry->mNextObjectEntry ;
		delete entry ;
		entry = nextEntry ;
		mNumObjects-- ;
	} ;
}

void TObjectList::AddObject(TObject *theObject) {
	
	TObjectEntry *aNewEntry = new TObjectEntry ;
	aNewEntry->mObject = theObject ;
	
	// We save time by always adding to the beginning of the list
	aNewEntry->mNextObjectEntry = mObjectEntry ;
	mObjectEntry = aNewEntry ;
	mNumObjects++ ;
}

void TObjectList::DrawObjects() {
	TObjectEntry* entry;
	TObject	*theObject ; 
		
	for (entry = mObjectEntry ;entry != nil; entry = entry->mNextObjectEntry) {
		theObject = entry->mObject ;
		theObject->Draw() ;
	}
}

void TObjectList::RemoveObject(TObject *theObject) {
	TObjectEntry* entry;
	TObjectEntry* last;
	TObject	*currentObject ;
	
	last = nil;
	for (entry = mObjectEntry ;entry != nil; entry = entry->mNextObjectEntry) {
		currentObject = entry->mObject ;
		if ( currentObject == theObject ) {			
			// is the current window the one to be removed?
			// yes, remove it
			// if first item in list, just set first
				if ( last == nil )				
					mObjectEntry = entry->mNextObjectEntry;
				else
					last->mNextObjectEntry = entry->mNextObjectEntry ;
			
			// free the TObjectEntry
				delete entry ;
				mNumObjects-- ;
				
			// we're done; no need to continue onto the next object
				return;
		}
		else						// no, go onto the next object in the list
			last = entry;
	}
}

TObject *TObjectList::GetObject(int number) {
	TObjectEntry* entry;
	int i ;
	
	if (number >= mNumObjects || number < 0)
		return nil ;
		
	for (i = 0,entry = mObjectEntry ;entry != nil; entry = entry->mNextObjectEntry,i++) {
		if (i == number) {
			return entry->mObject ;
		}
	}
	return nil ;
}

TObject *TObjectList::GetObject(Point *pt) {
	TObjectEntry* entry;
	TObject *theObject = nil ;  
		
	for (entry = mObjectEntry ;entry != nil; entry = entry->mNextObjectEntry) {
		theObject = entry->mObject ;
		if (theObject->IsVisible() && theObject->InObject(pt)) {
			return theObject ;
		}
	}
	return nil ;
}



void TEditTextList::AddObject(TObject *theObject) {
	
	TEditTextEntry *aNewEntry = new TEditTextEntry ;
	aNewEntry->mObject = theObject ;
	
	// We save time by always adding to the beginning of the list
	aNewEntry->mNextEditTextEntry = mEditTextEntry ;
	mEditTextEntry = aNewEntry ;
	mNumObjects++ ;
}


void TEditTextList::RemoveObject(TObject *theObject) {
	TEditTextEntry* entry;
	TEditTextEntry* last;
	TObject	*currentObject ;
	
	last = nil;
	for (entry = mEditTextEntry ;entry != nil; entry = entry->mNextEditTextEntry) {
		currentObject = entry->mObject ;
		if ( currentObject == theObject ) {			
			// is the current window the one to be removed?
			// yes, remove it
			// if first item in list, just set first
				if ( last == nil )				
					mEditTextEntry = entry->mNextEditTextEntry;
				else
					last->mNextEditTextEntry = entry->mNextEditTextEntry ;
			
			// free the TObjectEntry
				delete entry ;
				mNumObjects-- ;
				
			// we're done; no need to continue onto the next object
				return;
		}
		else						// no, go onto the next object in the list
			last = entry;
	}
}

TObject *TEditTextList::GetObject(int number) {
	TEditTextEntry* entry;
	int i ;
	
	if (number >= mNumObjects || number < 0)
		return nil ;
		
	for (i = 0,entry = mEditTextEntry ;entry != nil; 
			entry = entry->mNextEditTextEntry,i++) {
		if (i == number) {
			return entry->mObject ;
		}
	}
	return nil ;
}

TObject *TEditTextList::GetObject(Point *pt) {
	TEditTextEntry* entry;
	TObject *theObject = nil ;  
		
	for (entry = mEditTextEntry ;entry != nil; entry = entry->mNextEditTextEntry) {
		theObject = entry->mObject ;
		if (theObject->IsVisible() && theObject->InObject(pt)) {
			return theObject ;
		}
	}
	return nil ;
}

int TEditTextList::GetObjectIndex(TObject *obj) {
	TEditTextEntry* entry;
	TObject *theObject ;
	int i ;
			
	for (i = 0,entry=mEditTextEntry ;entry != nil; entry=entry->mNextEditTextEntry,i++) {
		theObject = entry->mObject ;
		if (theObject == obj) {
			return i ;
		}
	}
	return 0 ;
}

TEditText *TEditTextList::GetNextField(TEditText *current) {
	int whichItem = GetObjectIndex(current) ;
	if (whichItem < mNumObjects-1) {
		whichItem++ ;
		return (TEditText *)GetObject(whichItem) ;
	}
	whichItem = 0 ;
	return (TEditText *)GetObject(whichItem) ;
}

TIcon::TIcon(short	width,short height,unsigned char *data,Drawable drawable) {
	mWidth = width ;
	mHeight = height ;
	mPixmap = nil ;
	if (data != nil && drawable != nil) {
		mPixmap = XCreateBitmapFromData(gDisplay,drawable,
					(const char *)data,width,height) ;
	}
}

TIcon::~TIcon() {
	if (mPixmap != nil)
		XFreePixmap(gDisplay,mPixmap) ;
}

void	TIcon::Draw(short x,short y) {
	XCopyPlane(gDisplay,mPixmap,gCurrentPort->GetDrawable(),
		gCurrentPort->GetGC(),0,0,mWidth,mHeight,x,y,1) ;
}

Boolean TAnnotationLine::DoMouseDown(Point *pt) {
        return true ;
} 

Boolean TAnnotationRect::DoMouseDown(Point *pt) {
        return true ;
} 

void TAnnotationTextRect::Draw() { 
	if (mParentWindow != nil && mParentWindow->IsVisible()) {
		Rect	tRect = mRect ;
		InsetRect(&tRect,-1,-1) ;
		SetFgColor(gBlackPixel) ;
		FrameRect(&tRect) ; 
		SetFgColor(mColorIndex) ;
		FrameRect(&mRect) ; 
		DrawText(mRect.right+4,mRect.bottom,mText) ;		
		SetFgColor(gBlackPixel) ;
	}
} 

TAnnotationCircle::TAnnotationCircle(short xc,short yc, short rad,unsigned long col) 
	: TObject(xc-rad,yc-rad,xc+rad,yc+rad), mXC(xc),mYC(yc),mRad(rad) {
	mColorIndex = col ;

}

Boolean TAnnotationCircle::InObject(Point *pt) {
	if (!mIsActive || !mIsVisible) return false ;
	float	radSq = mRad*mRad ;
	float distSq = (pt->x-mXC)*(pt->x-mXC) + (pt->y-mYC)*(pt->y-mYC) ;
	if (fabs(distSq-radSq) <= 9.) return true ;
	return false ;
}

Boolean TAnnotationCircle::DoMouseDown(Point *pt) {
        return true ;
}

void	TAnnotationCircle::Draw() {
	if (!mParentWindow->IsVisible()) return ;
	DrawCircle(mXC+1,mYC+1,mRad) ;
	SetFgColor(mColorIndex) ;
	DrawCircle(mXC,mYC,mRad) ;
	SetFgColor(gBlackPixel) ;
}



TButton::TButton(Rect *rect,char *title,ObjectActionRoutine actionRoutine)
		: TObject(rect,title) {
	mIsDefault = false ;
	mActionRoutine = actionRoutine ;
}


TButton::TButton(short x1,short y1, short x2, short y2,char *title,
		ObjectActionRoutine actionRoutine)
		: TObject(x1,y1,x2,y2,title) {
	mIsDefault = false ;
	mActionRoutine = actionRoutine ;
}

TButton::~TButton() {
}

void	TButton::DrawLabel() {
	short 	x,y,width,height,numLine,numChars ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect = mRect ;

	if (mNumLabelLines == 1) {
		width = StringWidth(mLabel[0]) ;
		x = (tRect.left+tRect.right)/2-width/2 ;
		y = (tRect.top+tRect.bottom)/2 + 4 ;
 		DrawText(x,y,mLabel[0]) ;
	}
	else {
		y = (int)((tRect.top+tRect.bottom)/2 - mNumLabelLines*lineHeight/2 - 
			(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
		for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
			// Write out the line
			width = StringWidth(mLabel[numLine]) ;
			x = (tRect.left+tRect.right)/2-width/2 ;
 			DrawText(x,y,mLabel[numLine]) ;
			y += (int)(lineHeight+lineSpacing) ;
		}
	}
}

Boolean	TButton::DoMouseDown(Point *pt) {
	Point	where ;
	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;
	mState = ON ;
	Draw() ;
	while (Button()) ;	/* Wait until the mouse comes up */
	GetMouse(&where) ;
	if (!PtInRect(&where,&mRect)) {    
 		mState = OFF ;
		Draw() ; 
		return false ;
	}
	else {
 		mState = OFF ;
		Draw() ;
		if (mActionRoutine != nil) {
			(*mActionRoutine)(this) ;
		}		
		return true ;
	}
}

void	TButton::Draw() {
	Rect	tRect ;
	extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;

	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;

	tRect = mRect ;
	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}

	SetClip(mRect) ;
	switch(mState) {
		case OFF:
			SetFgColor(GetBkColor()) ;
			InsetRect(&tRect,1,1) ;
			EraseRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (mLabel != nil) {
				DrawLabel() ;
 			}
			SetFgColor(gBlackPixel) ;
			break ;
		case ON:
			InsetRect(&tRect,1,1) ;
			SetFgColor(gDarkerGray) ;
			FillRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (mLabel != nil) {
				if (!mIsActive) SetFgColor(gGrayPixel) ;
				DrawLabel() ;
			}
			SetFgColor(gBlackPixel) ;
			break ;
	}
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	XFlush(gDisplay) ;
}							

TIconButton::TIconButton(Rect *rect,TIcon *icon,char *title,Boolean state) 
	: TButton(rect,title) {
	
	mState 	= state ;
	mIcon 	= icon ;
	if (mIcon != nil) {
		short iconWidth,iconHeight ;
		iconWidth = mIcon->GetWidth() ;
		iconHeight = mIcon->GetHeight() ;
		mIconPosX = rect->left + (rect->right-rect->left - iconWidth)/2 ;
		mIconPosY = rect->top + (rect->bottom-rect->top - iconHeight)/2 ;
	}
}

TIconButton::TIconButton(short x1,short y1, short x2, short y2,TIcon *icon,
			char *title,Boolean state) 
	: TButton(x1,y1,x2,y2,title) {

	mState 	= state ;
	mIcon 	= icon ;
	if (mIcon != nil) {
		short iconWidth,iconHeight ;
		iconWidth = mIcon->GetWidth() ;
		iconHeight = mIcon->GetHeight() ;
		mIconPosX = x1 + (x2-x1 - iconWidth)/2 ;
		mIconPosY = y1 + (y2-y1 - iconHeight)/2 ;
	}
}

void TIconButton::Draw() {
	Rect	tRect ;
	extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;

	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;

	if (mIcon != nil) {
		short iconWidth,iconHeight ;
		iconWidth = mIcon->GetWidth() ;
		iconHeight = mIcon->GetHeight() ;
		mIconPosX = mRect.left + (mRect.right-mRect.left - iconWidth)/2 ;
		mIconPosY = mRect.top + (mRect.bottom-mRect.top - iconHeight)/2 ;
	}
	tRect = mRect ;
	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}
	SetClip(mRect) ;
	switch(mState) {
		case OFF:
			SetFgColor(GetBkColor()) ;
			InsetRect(&tRect,1,1) ;
			EraseRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (mIcon != nil) {
				mIcon->Draw(mIconPosX,mIconPosY) ;
			}
			if (mLabel != nil) {
				DrawLabel() ;
			}
			SetFgColor(gBlackPixel) ;
			break ;
		case ON:
			InsetRect(&tRect,1,1) ;
			SetFgColor(gDarkerGray) ;
			FillRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (mIcon != nil) {
				if (!mIsActive) SetFgColor(gGrayPixel) ;
				mIcon->Draw(mIconPosX,mIconPosY) ;
			}
			if (mLabel != nil) {
				if (!mIsActive) SetFgColor(gGrayPixel) ;
				DrawLabel() ;
			}
			SetFgColor(gBlackPixel) ;
			break ;
	}
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	XFlush(gDisplay) ;

}

TIconButton::~TIconButton() {
	if (mIcon != nil) delete mIcon ;
}


TPushButton::TPushButton(Rect *rect,TIcon *icon,short id,char *title,Boolean state) 
	: TButton(rect,title) {
	
	mState 	= state ;
	mIcon 	= icon ;
	mToolID	= id ;
	if (mIcon != nil) {
		short iconWidth,iconHeight ;
		iconWidth = mIcon->GetWidth() ;
		iconHeight = mIcon->GetHeight() ;
		mIconPosX = rect->left + (rect->right-rect->left - iconWidth)/2 ;
		mIconPosY = rect->top + (rect->bottom-rect->top - iconHeight)/2 ;
	}
}

TPushButton::TPushButton(short x1,short y1, short x2, short y2,TIcon *icon,
			short id,char *title,Boolean state) 
	: TButton(x1,y1,x2,y2,title) {

	mState 	= state ;
	mIcon 	= icon ;
	mToolID	= id ;
	if (mIcon != nil) {
		short iconWidth,iconHeight ;
		iconWidth = mIcon->GetWidth() ;
		iconHeight = mIcon->GetHeight() ;
		mIconPosX = x1 + (x2-x1 - iconWidth)/2 ;
		mIconPosY = y1 + (y2-y1 - iconHeight)/2 ;
	}
}

Boolean TPushButton::DoMouseDown(Point *pt) {
	Point	where ;
	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;
	if (mState)	return false ;

	mState = ON ;
	Draw() ;
	while (Button()) ;	/* Wait until the mouse comes up */
	GetMouse(&where) ;
	if (!PtInRect(&where,&mRect)) {
 		mState = OFF ;
		Draw() ;
		return false ;
	}
	if (mActionRoutine != nil) {
		(*mActionRoutine)(this) ;
	}		
	if (mParentWindow == gTheMainPane) {
		if (gCurrentTool != nil) 
			gCurrentTool->SetState(false) ;
		gCurrentTool = this ;
	}
	return true ;
}

void TPushButton::Draw() {
	Rect	tRect ;
	extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;

	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;


	if (mIcon != nil) {
		short iconWidth,iconHeight ;
		iconWidth = mIcon->GetWidth() ;
		iconHeight = mIcon->GetHeight() ;
		mIconPosX = mRect.left + (mRect.right-mRect.left - iconWidth)/2 ;
		mIconPosY = mRect.top + (mRect.bottom-mRect.top - iconHeight)/2 ;
	}
	tRect = mRect ;
	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}
	SetClip(mRect) ;
	switch(mState) {
		case OFF:
			SetFgColor(GetBkColor()) ;
			InsetRect(&tRect,1,1) ;
			EraseRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (mIcon != nil) {
				mIcon->Draw(mIconPosX,mIconPosY) ;
			}
			if (mLabel != nil) {
				DrawLabel() ;
			}
			SetFgColor(gBlackPixel) ;
			break ;
		case ON:
			InsetRect(&tRect,1,1) ;
			SetFgColor(gGrayPixel) ;
			//SetFgColor(gDarkerGray) ;
			FillRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (mIcon != nil) {
				if (!mIsActive) SetFgColor(gGrayPixel) ;
				mIcon->Draw(mIconPosX,mIconPosY) ;
			}
			if (mLabel != nil) {
				if (!mIsActive) SetFgColor(gGrayPixel) ;
				DrawLabel() ;
			}
			SetFgColor(gBlackPixel) ;
			break ;
	}
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	XFlush(gDisplay) ;

}

TPushButton::~TPushButton() {
	if (mIcon != nil) delete mIcon ;
}

void TFrame::Draw() { 
	if (mParentWindow != nil && mParentWindow->IsVisible()) {
		Rect	tRect = mRect ;
		InsetRect(&tRect,-1,-1) ;
		SetFgColor(gBlackPixel) ;
		FrameRect(&tRect) ; 
		SetFgColor(mColorIndex) ;
		FrameRect(&mRect) ; 
		SetFgColor(gBlackPixel) ;
	}
} 


TCheckBox::TCheckBox(Rect *rect,char *title,Boolean state) 
		: TButton(rect,title) {
	mState = state ;
	
}

TCheckBox::TCheckBox(short x1,short y1, short x2, short y2,char *title,Boolean state) 
		: TButton(x1,y1,x2,y2,title) {
	mState = state ;
}



TCheckBox::~TCheckBox() {
}

void	TCheckBox::DrawLabel() {
	short 	x,y,width,height,numLine,numChars ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect = mRect ;

	if (mNumLabelLines == 1) {
		width = StringWidth(mLabel[0]) ;
		x = (tRect.left+tRect.right)/2-width/2 ;
		y = (tRect.top+tRect.bottom)/2 + 2 ;
 		DrawText(x,y,mLabel[0]) ;
	}
	else {
		y = (int)((tRect.top+tRect.bottom)/2 - 
			mNumLabelLines*lineHeight/2 - 
			(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
		for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
			// Write out the line
			width = StringWidth(mLabel[numLine]) ;
			x = (tRect.left+tRect.right)/2-width/2 ;
 			DrawText(x,y,mLabel[numLine]) ;
			y += (int)(lineHeight+lineSpacing) ;
		}
	}
}

void	TCheckBox::DrawPressed(Boolean upDwn) {
	Rect 	tRect ;
	int 	x1,y1,x2,y2,side=12 ;
    
	x1 = mRect.left+2 ;
	y1 = mRect.top+2 ;
	x2 = x1+side ;
	y2 = y1+side ;
	switch(upDwn) {
		case ON: 
			SetRect(&tRect,x1,y1,x2,y2) ;
			PenSize(2,2) ;
			FrameRect(&tRect) ;
			PenSize(1,1) ;
			break ;
		case OFF:
			SetRect(&tRect,x1,y1,x2,y2) ;
			InsetRect(&tRect,-1,-1) ;
			EraseRect(&tRect) ;
			InsetRect(&tRect,1,1) ;
			PenSize(2,2) ;
			FrameRect(&tRect) ;
			PenSize(1,1) ;
			break ;
	}
	XFlush(gDisplay) ;
}

void	TCheckBox::Draw() {
	Rect 	tRect ;
	short	x1,y1,x2,y2,side=12,xm,ym ;
 	extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;
   
	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;

	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}

	SetClip(mRect) ;
	EraseRect(&mRect) ;
	xm = (mRect.left+mRect.right)/2 ;
	ym = (mRect.top+mRect.bottom)/2 ;
	x1 = mRect.left+2 ;
	y1 = ym-side/2 ;
	x2 = x1+side ;
	y2 = y1+side ;
	xm = (x1+x2)/2 ;
	ym = (y1+y2)/2 ;
	SetRect(&tRect,x1,y1,x2,y2) ;
	if (!mIsActive) {
		SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
		DrawLine(x1+1,y1+1,x2-1,y1+1) ;
		DrawLine(x1+1,y1+1,x1+1,y2-1) ;
		SetFgColor(gColors[near(0.46875*gNumGrayLevels)].pixel) ;
		DrawLine(x1+1,y2-1,x2,y2-1) ;
		DrawLine(x2-1,y1+1,x2-1,y2) ;
		SetFgColor(gDarkerGray) ;
		if (mState) {
			PenSize(2,2) ;
			DrawLine(x1+2,ym,xm-1,y2-3) ;
			DrawLine(xm-1,y2-3,x2-2,y1+2) ;
			PenSize(1,1) ;
		}
		FrameRect(&tRect) ;
	}
	else {
		SetFgColor(gWhitePixel) ;
		DrawLine(x1+1,y1+1,x2-1,y1+1) ;
		DrawLine(x1+1,y1+1,x1+1,y2-1) ;
		SetFgColor(gColors[near(0.3125*gNumGrayLevels)].pixel) ;
		DrawLine(x1+1,y2-1,x2,y2-1) ;
		DrawLine(x2-1,y1+1,x2-1,y2) ;
		SetFgColor(gBlackPixel) ;
		if (mState) {
			PenSize(2,2) ;
			DrawLine(x1+2,ym,xm-1,y2-3) ;
			DrawLine(xm-1,y2-3,x2-2,y1+2) ;
			PenSize(1,1) ;
		}
		FrameRect(&tRect) ;
	}

	if (mLabel !=  nil)
		DrawText(x2+4,y2-1,mLabel[0]) ;
	SetFgColor(gBlackPixel) ;
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	XFlush(gDisplay) ;
}

Boolean	TCheckBox::DoMouseDown(Point *pt) {
	Point	where ;
	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;
	mState = 1-mState ;
	DrawPressed(mState) ;
	while (Button()) ;	/* Wait until the mouse comes up */
	GetMouse(&where) ;
	if (!PtInRect(&where,&mRect)) {
 		mState = 1-mState ;
		Draw() ;
		return false ;
	}
	Draw() ;
	if (mActionRoutine != nil) {
		(*mActionRoutine)(this) ;
	}		
	return true ;
}

TRadioButton::TRadioButton(Rect *rect,char *title,Boolean state) 
		: TButton(rect,title) {
	mState = state ;
	
}

TRadioButton::TRadioButton(short x1,short y1, short x2, short y2,char *title,Boolean state) 
		: TButton(x1,y1,x2,y2,title) {
	mState = state ;
}

TRadioButton::~TRadioButton() {
}

void	TRadioButton::DrawLabel() {
	if (!mParentWindow->IsVisible()) return ;
	short 	x,y,width,height,numLine,numChars ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect = mRect ;

	if (mNumLabelLines == 1) {
		width = StringWidth(mLabel[0]) ;
		x = (tRect.left+tRect.right)/2-width/2 ;
		y = (tRect.top+tRect.bottom)/2 + 4 ;
 		DrawText(x,y,mLabel[0]) ;
	}
	else {
		y = (int)((tRect.top+tRect.bottom)/2 - mNumLabelLines*lineHeight/2 - 
			(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
		for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
			// Write out the line
			width = StringWidth(mLabel[numLine]) ;
			x = (tRect.left+tRect.right)/2-width/2 ;
 			DrawText(x,y,mLabel[numLine]) ;
			y += (int)(lineHeight+lineSpacing) ;
		}
	}
}
void	TRadioButton::DrawPressed(Boolean upDwn) {
	Rect 	tRect ;
	int 	xc,yc,x1,y1,x2,y2,side=12,rad=6 ;
    
	if (!mParentWindow->IsVisible()) return ;
	x1 = mRect.left+2 ;
	y1 = mRect.top+2 ;
	x2 = x1+side ;
	y2 = y1+side ;
	xc = (x1+x2)/2 ;
	yc = (y1+y2)/2 ;
	switch(upDwn) {
		case ON:
			PenSize(2,2) ;
			DrawCircle(xc,yc,rad) ;
			PenSize(1,1) ;
			break ;
		case OFF:
			SetRect(&tRect,x1,y1,x2,y2) ;
			EraseRect(&tRect) ;
			PenSize(2,2) ;
			DrawCircle(xc,yc,rad) ;
			break ;
	}
	PenSize(1,1) ;
	XFlush(gDisplay) ;
}

void	TRadioButton::Draw() {
	Rect 	tRect ;
	int 	xc,yc,x1,y1,x2,y2,side=12,rad=6 ;
  	extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;
   
	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;

	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}
	SetClip(mRect) ;
	EraseRect(&mRect) ;
	x1 = mRect.left+2 ;
	y1 = mRect.top+2 ;
	x2 = x1+side ;
	y2 = y1+side ;
	xc = (x1+x2)/2 ;
	yc = (y1+y2)/2 ;
	SetRect(&tRect,x1,y1,x2,y2) ;
	EraseRect(&tRect) ;
	PenSize(1,1) ;
	if (!mIsActive) {
		if (mState) {
			SetFgColor(gDarkerGray) ;
			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad+1,yc-rad+1,2*rad-2,2*rad-2,45*64,180*64) ;
 			SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
   			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad,yc-rad,2*rad,2*rad,225*64,180*64) ;
			SetFgColor(gDarkerGray) ;
			FillCircle(xc,yc,rad-2) ;
			DrawCircle(xc,yc,rad) ;
		}
		else {
			SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad+1,yc-rad+1,2*rad-2,2*rad-2,45*64,180*64) ;
 			SetFgColor(gColors[near(0.625*gNumGrayLevels)].pixel) ;
   			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad,yc-rad,2*rad,2*rad,225*64,180*64) ;
 			SetFgColor(gColors[near(0.71875*gNumGrayLevels)].pixel) ;
			FillCircle(xc,yc,rad-4) ;
			SetFgColor(gDarkerGray) ;
			DrawCircle(xc,yc,rad) ;
		}
	}
	else {
		if (mState) {
			SetFgColor(gDarkerGray) ;
			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad+1,yc-rad+1,2*rad-2,2*rad-2,45*64,180*64) ;
 			SetFgColor(gColors[near(0.89*gNumGrayLevels)].pixel) ;
   			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad,yc-rad,2*rad,2*rad,225*64,180*64) ;
			SetFgColor(gBlackPixel) ;
			FillCircle(xc,yc,rad-2) ;
			DrawCircle(xc,yc,rad) ;
		}
		else {
			SetFgColor(gColors[near(0.84375*gNumGrayLevels)].pixel) ;
			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad+1,yc-rad+1,2*rad-2,2*rad-2,45*64,180*64) ;
 			SetFgColor(gColors[near(0.46875*gNumGrayLevels)].pixel) ;
   			XFillArc(gDisplay,gCurrentPort->GetDrawable(),gCurrentPort->GetGC(),
				xc-rad,yc-rad,2*rad,2*rad,225*64,180*64) ;
 			SetFgColor(gColors[near(0.78125*gNumGrayLevels)].pixel) ;
			FillCircle(xc,yc,rad-4) ;
			SetFgColor(gBlackPixel) ;
			DrawCircle(xc,yc,rad) ;
		}
	}
	PenSize(1,1) ;

	if (mLabel != nil)
		DrawText(x2+4,y2,mLabel[0]) ;
	SetFgColor(gBlackPixel) ;
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	XFlush(gDisplay) ;
}

Boolean	TRadioButton::DoMouseDown(Point *pt) {
	Point	where ;

	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;
	if (mState) return false ;

	mState = ON ;
	DrawPressed(mState) ;
	while (Button()) ;	// Wait until the mouse comes up
	GetMouse(&where) ;
	if (!PtInRect(&where,&mRect)) {
 		mState = OFF ;
	}
	Draw() ;
	if (mActionRoutine != nil) {
		(*mActionRoutine)(this) ;
	}		
	return mState ;
}

TMenuButton::TMenuButton(Rect *rect,char *text,char *label,LabelPosition pos) 
		: TButton(rect,label) {
	
	mLabelPosition = pos ;
	if (text != nil) strcpy(mMenuText,text) ;
}

TMenuButton::TMenuButton(short x1,short y1, short x2, short y2,char *text,char *label,LabelPosition pos) 
		: TButton(x1,y1,x2,y2,label) {
	mLabelPosition = pos ;
	if (text != nil) strcpy(mMenuText,text) ;
}

TMenuButton::~TMenuButton() {
}

void	TMenuButton::DrawLabel() {
	if (mLabel == nil) return ;
	short 	x,y,width,height,numLine,numChars ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect = mRect ;

	if (!mIsActive) SetFgColor(gDarkerGray) ;

	if (mLabelPosition == kLabelOutsideLeft && mJustification == kLeftJustified) {
		// This is default and we force it to be right justified 
		// with the left side
		// of the menu

		if (mNumLabelLines == 1) {
			width = StringWidth(mLabel[0]) ;
			x = tRect.left-4 - width ;
			y = (tRect.top+tRect.bottom)/2 + 4 ;
 			DrawText(x,y,mLabel[0]) ;
		}
		else {
			y = (int)((tRect.top+tRect.bottom)/2 - mNumLabelLines*lineHeight/2 - 
				(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
			for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
				// Write out the line
				width = StringWidth(mLabel[numLine]) ;
				x = tRect.left-4 - width ;
 				DrawText(x,y,mLabel[numLine]) ;
				y += (int)(lineHeight+lineSpacing) ;
			}
		}
	}
	else if (mLabelPosition == kLabelOutsideTop && mJustification == kLeftJustified) {
		if (mNumLabelLines == 1) {
			width = StringWidth(mLabel[0]) ;
			x = tRect.left + 2 ;
			y = tRect.top - 5 ;
 			DrawText(x,y,mLabel[0]) ;
		}
		else {
			y = (int)(tRect.top - 5 - mNumLabelLines*lineHeight/2 - 
				(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
			for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
				// Write out the line
				width = StringWidth(mLabel[numLine]) ;
				x = tRect.left + 2 ;
 				DrawText(x,y,mLabel[numLine]) ;
				y += (int)(lineHeight+lineSpacing) ;
			}
		}
	}
	// Don't feel like covering all the other cases right now
	SetFgColor(gBlackPixel) ;
}

void	TMenuButton::DrawMenuText() {
	short 	x,y,width,height,numLine,numChars ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect = mRect ;

	width = StringWidth(mLabel[0]) ;
	x = tRect.left+8 ;
	y = (tRect.top+tRect.bottom)/2 + 4 ;
 	DrawText(x,y,mMenuText) ;
}

void	TMenuButton::Draw() {
	GrafPtr	tPort ;

	Rect	tRect,menuRect ;
	extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;

	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;

	GetPort(&tPort) ;
	SetPort(mParentWindow) ;
	tRect = mRect ;
	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}

	SetClip(mRect) ;
	SetRect(&menuRect,mRect.right-18,(mRect.top+mRect.bottom)/2-3,
		mRect.right-4,(mRect.top+mRect.bottom)/2+3) ;
	switch(mState) {
		case OFF:
			SetFgColor(GetBkColor()) ;
			InsetRect(&tRect,1,1) ;
			EraseRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			DrawMenuText() ;
			SetFgColor(gWhitePixel) ;
			DrawLine(menuRect.left,menuRect.bottom,menuRect.left,menuRect.top) ;
			DrawLine(menuRect.left,menuRect.top,menuRect.right,menuRect.top) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(menuRect.right,menuRect.top,menuRect.right,menuRect.bottom) ;
			DrawLine(menuRect.right,menuRect.bottom,menuRect.left,menuRect.bottom) ;

			SetFgColor(gBlackPixel) ;
			break ;
		case ON:
			InsetRect(&tRect,1,1) ;
			SetFgColor(gDarkerGray) ;
			FillRect(&tRect) ;
			PenSize(2,2) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
			PenSize(1,1) ;
			if (!mIsActive) SetFgColor(gGrayPixel) ;
			DrawMenuText() ;
			SetFgColor(gWhitePixel) ;
			DrawLine(menuRect.left,menuRect.bottom,menuRect.left,menuRect.top) ;
			DrawLine(menuRect.left,menuRect.top,menuRect.right,menuRect.top) ;
			SetFgColor(gBlackPixel) ;
			if (!mIsActive) SetFgColor(gDarkerGray) ;
			DrawLine(menuRect.right,menuRect.top,menuRect.right,menuRect.bottom) ;
			DrawLine(menuRect.right,menuRect.bottom,menuRect.left,menuRect.bottom) ;
			SetFgColor(gBlackPixel) ;
			break ;
	}

	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	
	// Since the label is generally outside the button, we don't clip to the button
	SetFgColor(gBlackPixel) ;
	DrawLabel() ;

	XFlush(gDisplay) ;
	SetPort(tPort) ;
}

Boolean	TMenuButton::DoMouseDown(Point *pt) {
	Point	where ;

	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;
	if (mState) return false ;

	mState = ON ;
	Draw() ;
	while (Button()) ;	// Wait until the mouse comes up
	GetMouse(&where) ;
	if (!PtInRect(&where,&mRect)) {
 		mState = OFF ;
	}
	Draw() ;
	return mState ;
}

TLine::TLine(short x1,short y1, short x2, short y2,unsigned long col)
		 : TObject(x1,y1,x2,y2), mColorIndex(col) {
	float	dx = x2 - x1 ;
	float	dy = y2 - y1 ;
	float	r ;
	if (y2 == y1) {
		mCosTheta = 1. ;
		mSinTheta = 0. ;
	}
	else if (x2 == x1) {
		mCosTheta = 0. ;
		mSinTheta = 1. ;
	}	
	else {
		r = sqrt(dx*dx+dy*dy) ;
		mCosTheta = dx/r ;
		mSinTheta = dy/r ;
	}
}

void TLine::Draw() {
	if (!mParentWindow->IsVisible()) return ;
	Rect tRect = mRect ;
	SetFgColor(gBlackPixel) ;
	SetFgColor(mColorIndex) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
	SetFgColor(gWhitePixel) ;
	DrawLine(near(tRect.left+mSinTheta),near(tRect.top-mCosTheta),
		near(tRect.right+mSinTheta),near(tRect.bottom-mCosTheta)) ;
	SetFgColor(gBlackPixel) ;
}

void TSlider::Update() {
	if (mParentWindow==nil) return ;
	if (!mParentWindow->IsVisible()) return ;
	GrafPtr tPort ;
	GetPort(&tPort);
	
	SetPort(mParentWindow) ; 
	SetControlRect() ; 
	Draw() ;
	SetPort(tPort) ;
}

void TSlider::Draw() {
	if (!mParentWindow->IsVisible()) return ;
	Rect tRect = mRect ;
	switch(mOrientation) {
		case kHorizontal:
			SetFgColor(gBlackPixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			OffsetRect(&tRect,0,1) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			SetFgColor(gBlackPixel) ;
			break ;
		case kVertical:
			SetFgColor(gBlackPixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			OffsetRect(&tRect,1,0) ;
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			SetFgColor(gBlackPixel) ;
			break ;
	}
	DrawMinValue() ;
	DrawMaxValue() ;
	DrawCurrentValue() ;
	DrawControl() ;
	if (mLabel != nil) DrawLabel() ;
}

void	TSlider::DrawMinValue() {
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	char	text[32] ;
	sprintf(text,"%.1f",mMinValue) ;
	
	switch(mOrientation) {
		case kHorizontal:
			// Find the position of the label ;
			valPosY = tRect.top-4 ;
			valPosX = tRect.left - StringWidth(text)/2 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
		case kVertical:
			valPosX = tRect.right +4 ;
			valPosY = tRect.bottom+4 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
	}
}

void	TSlider::DrawMaxValue() {
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	char	text[32] ;
	sprintf(text,"%.1f",mMaxValue) ;
	switch(mOrientation) {
		case kHorizontal:
			// Horizontal
			// Find the position of the label ;
			valPosY = tRect.top-4 ;
			valPosX = tRect.right - StringWidth(text)/2 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
		case kVertical:
			// Vertical
			valPosX = tRect.right +4 ;
			valPosY = tRect.top+4 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
	}
}

void	TSlider::DrawCurrentValue()  {
	if (!mParentWindow->IsVisible()) return ;
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	char	text[32] ;
	sprintf(text,"%.1f",mCurrentValue) ;
	switch(mOrientation) {
		case kHorizontal:
			valPosY = tRect.top-4 ;
			valPosX = tRect.left+(tRect.right-tRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			valPosX -= StringWidth(text)/2 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
		case kVertical:
			valPosX = tRect.right +4 ;
			valPosY = tRect.top+(tRect.bottom-tRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue)+4 ;
			DrawText(valPosX,valPosY,text) ;
			break ;
	}
}

void	TSlider::DrawControl()  {
	Rect tRect = mRect ;
	short	valPosX,valPosY,width=10 ;
	short	height = near(width*sqrt(3.)/2) ;
	switch(mOrientation) {
		case kHorizontal:
			valPosY = tRect.bottom+1 ;
			valPosX = tRect.left+(tRect.right-tRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetFgColor(gWhitePixel) ;
			MoveTo(valPosX-width/2,valPosY+height) ;
			LineTo(valPosX,valPosY) ;
			SetFgColor(gBlackPixel) ;
			LineTo(valPosX+width/2,valPosY+height) ;
			LineTo(valPosX-width/2,valPosY+height) ;
			break ;
		case kVertical:
			valPosX = tRect.left-1 ;
			valPosY = tRect.top+(tRect.bottom-tRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetFgColor(gWhitePixel) ;
			MoveTo(valPosX-height,valPosY+width/2) ;
			LineTo(valPosX-height,valPosY-width/2) ;
			LineTo(valPosX,valPosY) ;
			SetFgColor(gBlackPixel) ;
			LineTo(valPosX-height,valPosY+width/2) ;
			break ;
	}
}

void	TSlider::DrawLabel() {
	Rect tRect = mRect ;
	short	valPosX,valPosY ;
	switch(mOrientation) {
		case kHorizontal:
			valPosY = tRect.bottom+2 ;
			valPosX = tRect.left - StringWidth(mLabel[0]) - 20 ;
			DrawText(valPosX,valPosY,mLabel[0]) ;
			break ;
		case kVertical:
			valPosX = tRect.right - StringWidth(mLabel[0])/2 ;
			valPosY = tRect.top-4 ;
			DrawText(valPosX,valPosY,mLabel[0]) ;
			break ;
	}
}

void	TSlider::Erase() {
	// We will erase the control
	if (mParentWindow==nil) return ;
	TGrafPort *tp ;
	GetPort(&tp) ;
	SetPort((TGrafPort *)mParentWindow) ;
	SetFgColor(gGrayPixel) ;
	
	Rect tRect = mRect ;
	switch(mOrientation) {
		case kHorizontal:
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			OffsetRect(&tRect,0,1) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			break ;
		case kVertical:
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			OffsetRect(&tRect,1,0) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.bottom) ;
			break ;
	}
	DrawMinValue() ;
	DrawMaxValue() ;
	
	EraseControl() ;

	SetFgColor(gBlackPixel) ;
	SetPort(tp) ;
}

void	TSlider::EraseControl() {
	short	valPosX,valPosY,width=10 ;
	short	height = near(width*sqrt(3.)/2) ;
	Rect	tRect = mRect ;
	switch(mOrientation) {
		case kHorizontal:
			// Erase the control at it's current position
			valPosY = tRect.bottom+1 ;
			valPosX = tRect.left+(tRect.right-tRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetFgColor(gGrayPixel) ;
			DrawCurrentValue() ;
			MoveTo(valPosX-width/2,valPosY+height) ;
			LineTo(valPosX,valPosY) ;
			LineTo(valPosX+width/2,valPosY+height) ;
			LineTo(valPosX-width/2,valPosY+height) ;
			SetFgColor(gBlackPixel) ;
			break ;
		case kVertical:
			// Erase the control at it's current position
			valPosX = tRect.left-1 ;
			valPosY = tRect.top+(tRect.bottom-tRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetFgColor(gGrayPixel) ;
			DrawCurrentValue() ;
			MoveTo(valPosX-height,valPosY+width/2) ;
			LineTo(valPosX-height,valPosY-width/2) ;
			LineTo(valPosX,valPosY) ;
			LineTo(valPosX-height,valPosY+width/2) ;
			SetFgColor(gBlackPixel) ;
			break ;
	}
}

void	TSlider::MoveControl(short pos) {
	Rect	tRect = mRect ;
	float	dValue = mOrientation == kHorizontal ? (mRect.right - mRect.left)/(mMaxValue-mMinValue) :
				(mRect.bottom - mRect.top)/(mMaxValue-mMinValue) ;
				
	EraseControl() ;
	mCurrentValue = mOrientation == kHorizontal ? 
							mMinValue + (pos - mRect.left)/dValue :
							mMinValue + (mRect.bottom - pos)/dValue ;
	if (mCurrentValue > mMaxValue) mCurrentValue = mMaxValue ;
	if (mCurrentValue < mMinValue) mCurrentValue = mMinValue ;
	SetControlRect() ;
	Draw() ;
	if (mTrackingRoutine != nil)
		(*mTrackingRoutine)(this) ;
}

void	TSlider::SetControlRect() {
	Rect tRect = mRect ;
	short	valPosX,valPosY,width=10 ;
	short	height = near(width*sqrt(3.)/2) ;
	switch(mOrientation) {
		case kHorizontal:
			valPosY = tRect.bottom+1 ;
			valPosX = tRect.left+(tRect.right-tRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetRect(&mControlRect,valPosX-width/2,valPosY,valPosX+width/2,valPosY+height) ;
			break ;
		case kVertical:
			valPosX = tRect.left-1 ;
			valPosY = tRect.top+(tRect.bottom-tRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			SetRect(&mControlRect,valPosX-height,valPosY-width/2,valPosX,valPosY+width/2) ;
			break ;
	}
}

Boolean TSlider::DoMouseDown(Point *pt) {
	float	dValue ;
	Point	currentPos,lastPos ;
	short	dd ;

	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;

	dValue = mOrientation == kHorizontal ? (mRect.right - mRect.left)/(mMaxValue-mMinValue) :
				(mRect.bottom - mRect.top)/(mMaxValue-mMinValue) ;
				
	GetMouse(&currentPos) ;
	lastPos = currentPos ;
	if (PtInRect(&currentPos,&mControlRect)) {
		while (Button()) {
			GetMouse(&currentPos) ;
			switch(mOrientation) {
				case kHorizontal:
					dd = currentPos.x - lastPos.x ;
					if (currentPos.x != lastPos.x) {
						MoveControl(currentPos.x) ;
						lastPos = currentPos ;
					}
					break ;
				case kVertical:
					dd = currentPos.y - lastPos.x ;
					if (currentPos.y != lastPos.y) {
						MoveControl(currentPos.y) ;
						lastPos = currentPos ;
					}
					break ;
			}
		}
		if (mActionRoutine != nil) {
			(*mActionRoutine)(this) ;
		}		
		Draw() ;
	}
	return true ;
}

TGroupBox::TGroupBox(Rect *rect,char *title) 
		: TObject(rect,title) {
	mIsActive = false ;
}

TGroupBox::TGroupBox(short x1,short y1, short x2, short y2,char *title) 
		: TObject(x1,y1,x2,y2,title) {
	mIsActive = false ;
}

TGroupBox::~TGroupBox() {
}

void	TGroupBox::DrawLabel() {
	short 	x,y,width,height,descent,numLine ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect  ;

	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;

	height = font->ascent ;	// Height of text
	descent = font->descent ;	// Descent below baseline

	if (mNumLabelLines == 1) {
		width = StringWidth(mLabel[0]) ;
		if (mJustification == kLeftJustified)
			x = mRect.left+10 ;
		else if (mJustification == kCenterJustified)
			x = (mRect.left+mRect.right)/2-width/2 ;
		else 
			x = mRect.right-width - 4 ;
		y = mRect.top+4 ;
 		SetRect(&tRect,x - 4, mRect.top-height, x+width+4,mRect.top+descent+2) ;
		EraseRect(&tRect) ;
		DrawText(x,y,mLabel[0]) ;
	}
	else {
		y = (int)(mRect.top - mNumLabelLines*lineHeight/2 - 
			(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
		for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
			// Write out the line
			width = StringWidth(mLabel[numLine]) ;
			if (mJustification == kLeftJustified)
				x = mRect.left+10 ;
			else if (mJustification == kCenterJustified)
				x = (mRect.left+mRect.right)/2-width/2 ;
			else 
				x = mRect.right-width - 4 ;
 			SetRect(&tRect,x - 4, (int)(y-lineHeight-1), x+width+4,y+descent+1) ;
			EraseRect(&tRect) ;
			DrawText(x,y,mLabel[numLine]) ;
			y += (int)(lineHeight+lineSpacing) ;
		}
	}
}
void TGroupBox::Draw() {
	Rect	tRect ;
 
	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;
  
	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}
	tRect = mRect ;
	//SetFgColor(gDarkerGray) ;
	SetFgColor(gBlackPixel) ;
	FrameRect(&tRect) ;
	SetFgColor(gWhitePixel) ;
	OffsetRect(&tRect,1,1) ;
	FrameRect(&tRect) ;
	SetFgColor(gBlackPixel) ;
	if (mLabel != nil)
		DrawLabel() ;
	DisposeRgn(clipRgn) ;
}

TStaticText::TStaticText(Rect *rect,char *title) 
		: TObject(rect,title) {
	mIsActive = true ;
}

TStaticText::TStaticText(short x1,short y1, short x2, short y2,char *title) 
		: TObject(x1,y1,x2,y2,title) {
	mIsActive = true ;
}

TStaticText::~TStaticText() {
}

void	TStaticText::DrawLabel() {
	short 	x,y,width,numLine ;
	float	lineHeight=9.,lineSpacing=3. ;
	Rect	tRect = mRect ;

	EraseRect(&mRect) ;
	if (!mIsActive) SetFgColor(gDarkGray) ;

	if (mNumLabelLines == 1) {
		width = StringWidth(mLabel[0]) ;
		if (mJustification == kLeftJustified)
			x = tRect.left ;
		else if (mJustification == kCenterJustified) {
			x = (tRect.left+tRect.right)/2-width/2 ;
		}
		else {
			x = tRect.right-width ;
		}
		y = (tRect.top+tRect.bottom)/2 + 4 ;
 		DrawText(x,y,mLabel[0]) ;
	}
	else {
		y = (int)((tRect.top+tRect.bottom)/2 - mNumLabelLines*lineHeight/2 - 
			(mNumLabelLines-1)*lineSpacing/2 + lineHeight) ;
		for(numLine = 0; numLine < mNumLabelLines ; numLine++) {
			// Write out the line
			width = StringWidth(mLabel[numLine]) ;
			if (mJustification == kLeftJustified)
				x = tRect.left ;
			else if (mJustification == kCenterJustified)
				x = (tRect.left+tRect.right)/2-width/2 ;
			else 
				x = tRect.right-width ;
 			DrawText(x,y,mLabel[numLine]) ;
			y += (int)(lineHeight+lineSpacing) ;
		}
	}
	SetFgColor(gBlackPixel) ;
}


void TStaticText::SetItemText(char *text) {
	if (mLabel != nil) delete[] mLabel ;
	CreateLabel(text) ;

	if (mParentWindow == nil) return ;
	GrafPtr tPort ;
	GetPort(&tPort) ;
	SetPort(mParentWindow) ;
	Draw() ;
	SetPort(tPort) ;
}

void TStaticText::GetItemText(char *text) {
	// Return only the first line of the label
	if (strlen(mLabel[0]) == 0) 
		strcpy(text,"") ;
	else
		strcpy(text,mLabel[0]) ;
}

void TStaticText::Draw() {
    
	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;
	if (mLabel == nil) return ;

	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	if (!RectInRgn(&mRect,clipRgn)) {
		DisposeRgn(clipRgn) ;
		return ;
	}
	SetClip(mRect) ;
	DrawLabel() ;
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
}

TEditText::TEditText(Rect *rect,char *title,char *text)
		: TObject(rect,title) {

	mText = new char[256] ;
	mText[0] = '\0' ;
	if (text != nil) {
		strcpy(mText,text) ;
	}
	mFirstSelChar = 0 ;
	mNumSelChars = 0 ;

	int	numChars ;
	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;

	mInsertionPt.x = mRect.left + 5 ; 	// Beginning point of the string 
	mInsertionPt.y = mRect.bottom - 5 ;	// Beginning point of the string
	numChars = strlen(mText) ;	// Current number of characters	
	if (numChars > 0)
		mInsertionPt.x += XTextWidth(font,mText,numChars) ;
	mIsFocus = false ;
	mInputType = kNumericInput ;
	mLabelPosition = kLabelOutsideLeft ;

}

TEditText::TEditText(short x1,short y1, short x2, short y2,char *title,char *text)
		: TObject(x1,y1,x2,y2,title) {
	mText = new char[256] ;
	mText[0] = '\0' ;
	if (text != nil) {
		strcpy(mText,text) ;
	}
	mFirstSelChar = 0 ;
	mNumSelChars = 0 ;
	int	numChars ;

	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;

	mInsertionPt.x = mRect.left + 5 ; 	// Beginning point of the string 
	mInsertionPt.y = mRect.bottom - 5 ;	// Beginning point of the string
	numChars = strlen(mText) ;	// Current number of characters	
	if (numChars > 0)
		mInsertionPt.x += XTextWidth(font,mText,numChars) ;
	mIsFocus = false ;
	mInputType = kNumericInput ;
	mLabelPosition = kLabelOutsideLeft ;
}

TEditText::~TEditText() {
	delete[] mText ;
}

void TEditText::SetFocus(Boolean focus) {
	mIsFocus = focus ; 
	if (!mIsFocus) {
		mFirstSelChar = 0 ;
		mNumSelChars = 0 ;
	}
	Draw() ; 
} 

void TEditText::SetEditText(char *text) {
	strcpy(mText,text) ;

	if (mParentWindow == nil) return ;

	GrafPtr	tPort ;
	GetPort(&tPort) ;
	SetPort(mParentWindow) ;
	mFirstSelChar = 0 ;
	mNumSelChars = 0 ;
	int	numChars ;

	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;

	mInsertionPt.x = mRect.left + 5 ; 	// Beginning point of the string 
	mInsertionPt.y = mRect.bottom - 5 ;	// Beginning point of the string
	numChars = strlen(mText) ;	// Current number of characters	
	if (numChars > 0)
		mInsertionPt.x += XTextWidth(font,mText,numChars) ;

	Draw() ;
	SetPort(tPort) ;
}

void TEditText::GetEditText(char *text) {
	if (strlen(mText) == 0) 
		strcpy(text,"") ;
	else
		strcpy(text,mText) ;
}

void	TEditText::SelectAll() {

	mNumSelChars = strlen(mText) ;
	mFirstSelChar = 0 ;
	SetFocus(true) ;
}

void	TEditText::Draw() {
	Rect tRect,clipRect,selectionRect ;
	unsigned long oldFg ;
	extern	uLong gBlackPixel,gWhitePixel ;
	int	numChars ;
	int 	labelPlacementX,labelPlacementY,textPlacement ;

	if (!mIsVisible) return ;
	if (mParentWindow == nil) return ;
	if (!mParentWindow->IsVisible()) return ;

	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	//if (!RectInRgn(&mRect,clipRgn)) {
	//	DisposeRgn(clipRgn) ;
	//	return ;
	//}
	oldFg = GetFgColor() ;
	tRect = mRect ;
	clipRect = mRect ;
	InsetRect(&tRect,1,1) ;
	InsetRect(&clipRect,2,2) ;
	EraseRect(&tRect) ;

	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;
	numChars = strlen(mText) ;
	textPlacement = (mRect.top+mRect.bottom)/2 + font->ascent/2 ;
	// We always assume left justified label
	switch(mLabelPosition) {
		case kLabelOutsideLeft:		
			labelPlacementY = (mRect.top+mRect.bottom)/2 + font->ascent/2 ;
			labelPlacementX = mRect.left - StringWidth(mLabel[0]) - 5 ;
			break ;
		case kLabelOutsideRight:
			labelPlacementY = (mRect.top+mRect.bottom)/2 + font->ascent/2 ;
			labelPlacementX = mRect.right + 5 ;
			break ;
		case kLabelOutsideTop:
			labelPlacementY = mRect.top - 5 ;
			labelPlacementX = mRect.left + 2 ;
			break ;
		case kLabelOutsideBottom:
			labelPlacementY = mRect.bottom + 15 ;
			labelPlacementX = mRect.left + 2 ;
			break ;
	}

	if (mIsFocus && mIsActive) {
		PenSize(2,2) ;
		SetFgColor(gBlackPixel) ;
		DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
		DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
		SetFgColor(gWhitePixel) ;
		DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
		DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
		PenSize(1,1) ;
		SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
		InsetRect(&tRect,2,2) ;
		FillRect(&tRect) ;
		SetFgColor(gBlackPixel) ;
		// If there is no selection we will draw a caret to indicate 
		// the insertion point
		PenSize(1,1) ;
		SetClip(clipRect) ;	// Set clip so we don't draw outside our area
		if (mNumSelChars == 0) {
			DrawText(mRect.left+5,textPlacement,mText) ; 
			DrawLine(mInsertionPt.x,mRect.top+4,mInsertionPt.x,mRect.bottom-4) ;
		}
		else {
			// Invert the area that is selected
			char firstString[256] ;
			int	x,y,left,right ;
			x = mRect.left+5 ;
			y = mRect.bottom-6 ;
			Rect	selRect ;
			DrawText(x,y,mText) ;
			SetFgColor(gBlackPixel) ;
			SetBkColor(gWhitePixel) ;
			XSetFunction(gDisplay,gCurrentPort->GetGC(),GXinvert) ;
			left = mRect.left+5+
					XTextWidth(font,mText,mFirstSelChar) ;			
			right = left+XTextWidth(font,mText+mFirstSelChar,
						mNumSelChars) ;
			if (mFirstSelChar == 0 && mNumSelChars == numChars) 
				SetRect(&selectionRect,mRect.left+4,mRect.top+4,
					mRect.right-4,mRect.bottom-4) ;
			else 
				SetRect(&selectionRect,left,mRect.top+4,right,mRect.bottom-4) ;
			FillRect(&selectionRect) ;
			PenMode(kCopyMode) ;
			SetFgColor(gBlackPixel) ;
			SetBkColor(gGrayPixel) ;
		}
		SetClip(gCurrentPort->GetPortRect()) ;
	}
	else {
		PenSize(2,2) ;
		if (!mIsActive) 
			SetFgColor(gDarkGray) ;
		else
			SetFgColor(gBlackPixel) ;
		DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
		DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
		if (!mIsActive) 
			SetFgColor(gLighterGray) ;
		else
			SetFgColor(gWhitePixel) ;
		DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
		DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
		if (!mIsActive) 
			SetFgColor(gDarkGray) ;
		else
			SetFgColor(gBlackPixel) ;
		PenSize(1,1) ;
		InsetRect(&tRect,2,2) ;
		SetClip(clipRect) ;
		
		if (mNumSelChars == 0) {
			DrawText(mRect.left+5,textPlacement,mText) ; 
		}
		else {
			// Invert the area that is selected
			char firstString[256],secondString[256] ;
			int	x,y,left,right ;
			x = mRect.left+5 ;
			y = mRect.bottom-6 ;
			Rect	selRect ;
			DrawText(x,y,mText) ;
			if (!mIsActive) {
				SetFgColor(gDarkGray) ;
				SetBkColor(gLighterGray) ;
			}
			else {
				SetFgColor(gBlackPixel) ;
				SetBkColor(gWhitePixel) ;
			}
			XSetFunction(gDisplay,gCurrentPort->GetGC(),GXinvert) ;
			left = mRect.left+5+
					XTextWidth(font,mText,mFirstSelChar) ;			
			right = left+XTextWidth(font,mText+mFirstSelChar,
						mNumSelChars) ;
			if (mFirstSelChar == 0 && mNumSelChars == numChars) 
				SetRect(&selectionRect,mRect.left+4,mRect.top+4,
					mRect.right-4,mRect.bottom-4) ;
			else 
				SetRect(&selectionRect,left,mRect.top+4,right,mRect.bottom-4) ;
			FillRect(&selectionRect) ;
			PenMode(kCopyMode) ;
			SetFgColor(gBlackPixel) ;
			SetBkColor(gGrayPixel) ;
		}
		
		SetClip(gCurrentPort->GetPortRect()) ;
	}
	PenSize(1,1) ;
	if (!mIsActive) 
		SetFgColor(gDarkGray) ;
	else
		SetFgColor(gBlackPixel) ;
	if (mLabel != nil) {
		DrawText(labelPlacementX,labelPlacementY,mLabel[0]) ;
	}
	SetFgColor(oldFg) ;
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
	XFlush(gDisplay) ;

}

short	TEditText::FindCharacterToLeft(short x) {
	char 	*cp = mText ;
	short	i,xt,dx,numChars,xCurrPos,xBegPos,EndStrXPos ;
	
	xBegPos 	= mRect.left+5 ;
	numChars	= strlen(mText) ;
	cp		= mText ;
	xCurrPos	= xBegPos ;
	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;
	EndStrXPos = xBegPos + XTextWidth(font,mText,numChars) ;
	if (x < xBegPos) return -1 ;
	if (x > EndStrXPos) return numChars-1 ;
	for (i=0;i<numChars;i++,cp++) {
		dx = XTextWidth(font,cp,1) ;
		xt = xCurrPos + dx/2 ; // half char
		if (xt > x) {
			return i-1 ;
		}
		xCurrPos += dx ;
	}
	return numChars-1 ;
}

short	TEditText::FindCharacterToRight(short x) {
	char 	*cp = mText ;
	short	i,xt,dx,numChars,xCurrPos,xBegPos,EndStrXPos ;

	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;
	
	xBegPos 	= mRect.left+5 ;
	numChars	= strlen(mText) ;
	cp		= mText ;
	xCurrPos	= xBegPos ;
	EndStrXPos = xBegPos + XTextWidth(font,mText,numChars) ;
	if (x < xBegPos) return 0 ;
	if (x > EndStrXPos) return numChars ;
	for (i=0;i<numChars;i++,cp++) {
		dx = XTextWidth(font,cp,1) ;
		xt = xCurrPos + dx - 2 ; // half char
		if (xt > x) {
			return i ;
		}
		xCurrPos += dx ;
	}
	return numChars ;
}

Boolean	TEditText::DoMouseDown(Point *pt) {
	Rect	tRect,clipRect ;
	int 	i,x,y,nInChars,dx,xt ;
	int	EndStrXPos,numChars ;
	int 	xBegPos,yBegPos,xCurrPos,yCurrPos ;
	char	*cp ;

	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;

	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;

	xBegPos = mRect.left + 5 ; 	// Beginning point of the string 
	yBegPos = mRect.bottom - 5 ;// Geginning point of the string
	numChars = strlen(mText) ;	// Current number of characters	
	EndStrXPos = xBegPos + XTextWidth(font,mText,numChars) ;
	xCurrPos = mInsertionPt.x ;		// Current Point to put character
	yCurrPos = mInsertionPt.y ;

	if (IsDoubleClick()) {
		SelectAll() ;
		return true ;
	}

	clipRect	= mRect ;
	InsetRect(&clipRect,3,3) ;
	SetClip(clipRect) ;
		
	int textPlacement = (mRect.top+mRect.bottom)/2 + font->ascent/2 ;
	if (mNumSelChars == numChars) {
		// The selection must be cleared
		// We do this whether we have 
		// selection or not	
		mFirstSelChar = 0 ;
		mNumSelChars = 0 ;
		tRect = mRect ;
		InsetRect(&tRect,3,3) ;
		SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
		FillRect(&tRect) ;
		SetFgColor(gBlackPixel) ;
		DrawText(xBegPos,textPlacement,mText) ;
	}
	if (numChars == 0) {	// There are no characters in this field yet.
		mInsertionPt.x = xBegPos ;
		mInsertionPt.y = yBegPos ;
		SetClip(gCurrentPort->GetPortRect()) ;
		return true ;
	}
	// Only need to find the insertion point 
	x = pt->x ;
	y = pt->y ;

	Point	currentPt=*pt,lastPt = *pt,startPt = *pt ;
	Rect	selRect ;
	SetRect(&selRect,x,mRect.top+4,x,mRect.bottom-4) ;
	int right,left ;
	SetFgColor(BlackPixel(gDisplay,gScreen)) ;
	SetBkColor(WhitePixel(gDisplay,gScreen)) ;
	XSetFunction(gDisplay,gCurrentPort->GetGC(),GXinvert) ;
	while(Button()) {
		GetMouse(&currentPt) ;
		if (currentPt.x != lastPt.x) {
			left = (currentPt.x < lastPt.x ? currentPt.x : lastPt.x) ;
			right = (currentPt.x > lastPt.x ? currentPt.x : lastPt.x) ;
			SetRect(&selRect,left,mRect.top+4,right,mRect.bottom-4) ;
			FillRect(&selRect) ;
			lastPt = currentPt ;
		}
	}
	PenMode(kCopyMode) ;
	SetFgColor(gBlackPixel) ;
	SetBkColor(gGrayPixel) ;
	SetClip(gCurrentPort->GetPortRect()) ;


	left = (currentPt.x < startPt.x ? currentPt.x : startPt.x) ;
	right = (currentPt.x > startPt.x ? currentPt.x : startPt.x) ;
	if (left != right) {
		mFirstSelChar = FindCharacterToRight(left) ;
		int	lastSelChar = FindCharacterToLeft(right) ;
		mNumSelChars = lastSelChar - mFirstSelChar + 1 ;
		if (mNumSelChars == 0) 
			mFirstSelChar = 0 ;
	}

	SetClip(gCurrentPort->GetPortRect()) ;
	x = left ;
	if (x <= xBegPos) {	// Change the x current position 
		xCurrPos = xBegPos ;
	}
	else if (x > xBegPos && x < EndStrXPos) { // Change x pos.
		xCurrPos = xBegPos + XTextWidth(font,
			mText,FindCharacterToRight(x)) ;
	}
	else {
		xCurrPos = EndStrXPos ; // Append at the end 
	}
	mInsertionPt.x = xCurrPos ;
	mInsertionPt.y = yBegPos ;
	Draw() ;
	return true ;
}

KeySym 	TEditText::DoKeyDown(XEvent *theEvent) {
	Rect	tRect,clipRect ;
	int 	i,x,y,height,width,descent,nInChars,dx,xt ;
	int		inString=false,EndStrXPos,numPrevChar,numChars ;
	int 	xBegPos,yBegPos,xCurrPos,yCurrPos,count,retcode ;
	KeySym 	keysym ;
	XComposeStatus compose ;
	extern	GrafPtr gCurrentPort ;
	char 	*cpst,inputChar,InputBuffer[80],lastCharString[2],
		savestring[80],endString[80],*cp ;
		
	if (!mIsActive) return 0 ;
		
	XFontStruct *font = (gCurrentPort == nil) ? gFontStructure : gCurrentPort->GetFont() ;

	xBegPos = mRect.left + 5 ; 	// Beginning point of the string 
	yBegPos = mRect.bottom - 5 ;// Geginning point of the string
	numChars = strlen(mText) ;	// Current number of characters	
	EndStrXPos = xBegPos + 
    		XTextWidth(font,mText,numChars) ;
	xCurrPos = mInsertionPt.x ;		// Current Point to put character
	yCurrPos = mInsertionPt.y ;
	height = font->ascent ;	// Height of text
	descent = font->descent ;	// Descent below baseline

	// Obviously this control is already the active
	// edit text field, so process characters

	int textPlacement = (mRect.top+mRect.bottom)/2 + font->ascent/2 ;

	// What is the character
	count = XLookupString((XKeyEvent *)theEvent,InputBuffer,80,&keysym,&compose) ; 
	
	if (gDebugMode) {
		fprintf(stderr,"\nkeysym = %c , Integer %d , Hex: %X",keysym,keysym,keysym) ;
	}
	
 	// Do we have a current selection
	clipRect = mRect ;
	InsetRect(&clipRect,2,2) ;
	
	if (mNumSelChars > 0 || mNumSelChars == numChars) {
		if (keysym == XK_Tab) {
			// The selection must be cleared */
			mFirstSelChar = 0 ;
			mNumSelChars = 0 ;
			tRect = mRect ;
			InsetRect(&tRect,3,3) ;
			SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
			FillRect(&tRect) ;
			SetFgColor(gBlackPixel) ;
			SetClip(clipRect) ;
			DrawText(xBegPos,textPlacement,mText) ;
			SetClip(gCurrentPort->GetPortRect()) ;
		}
		else if (keysym == XK_Return || 
				keysym == XK_KP_Enter || keysym == XK_Linefeed) {
				// don't do anything here
		}
// Trap here for Keypad Left/Right not being defined
#ifndef XK_KP_Left
#define XK_KP_Left XK_Left
#define XK_KP_Right XK_Right
#endif
		else if (keysym == XK_Left || keysym == XK_Right ||
			keysym == XK_KP_Left || keysym == XK_KP_Right	) {
				// don't do anything here, added by LDM
		}
		else { // Erase the selection and change the
    			// Insertion point */
			if (mNumSelChars == numChars) {	// The whole string is selected
				tRect = mRect ;
				InsetRect(&tRect,3,3) ;
				SetClip(clipRect) ;
				SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
				FillRect(&tRect) ;
 				SetFgColor(gBlackPixel) ;
  				mText[0] = EOS ;
				numChars = strlen(mText)  ;
				EndStrXPos = xBegPos + XTextWidth(font,mText,numChars) ;
				mInsertionPt.x = xBegPos ;
				mInsertionPt.y = yBegPos ;
				mFirstSelChar = 0 ;
				mNumSelChars = 0 ;
				SetClip(gCurrentPort->GetPortRect()) ;
			}
			else {	// The selection is not the entire string
				if (mFirstSelChar > 0) {
					strncpy(savestring,mText,mFirstSelChar) ;
					savestring[mFirstSelChar] = EOS ;
				}
				if (mFirstSelChar+mNumSelChars < numChars) {	// Save the end
					strcpy(endString,mText+mFirstSelChar+mNumSelChars) ;
					strcpy(mText,savestring) ;
					strcat(mText,endString) ;
				}
				else {
					strcpy(mText,savestring) ;
				}
				SetClip(clipRect) ;
				x = xBegPos ;
				width = XTextWidth(font,mText,strlen(mText)) ;
				y = yBegPos ;
				SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
				FillRect(&clipRect) ;
 				SetFgColor(gBlackPixel) ;
				mInsertionPt.x = x + XTextWidth(font,mText,
					mFirstSelChar) ;
				DrawText(xBegPos,textPlacement,mText) ;
				SetClip(gCurrentPort->GetPortRect()) ;
				mInsertionPt.y = yBegPos ;
				numChars = strlen(mText) ;
				EndStrXPos = xBegPos + XTextWidth(font,mText,numChars) ;
				mFirstSelChar = 0 ;
				mNumSelChars = 0 ;
			}
			if (keysym == XK_Delete || keysym == XK_BackSpace) {
				return(true)  ; // We are done 
			}
		}
	}
	// Is the insertion point within the string
	xCurrPos = mInsertionPt.x ;
	yCurrPos = mInsertionPt.y ;
	numPrevChar = -1 ; // Initialize to beginning of str
	if (xCurrPos == xBegPos) { // Beginning of string 
		inString = true ;
		numPrevChar = -1 ;
	}
	else if (xCurrPos < EndStrXPos) { /* Inside the string */
		for (i=0;i<numChars;i++) {
			x = xBegPos + XTextWidth(font,mText,i+1) ;
			if (x >= xCurrPos)
				break ;
		}
		inString = true ;
		// The number of the prev char (starting at 0)
		numPrevChar = i ; 
	}
	else {
		numPrevChar = numChars - 1;
	}

	if (keysym == XK_Tab || keysym == XK_Return || 
			keysym == XK_KP_Enter || keysym == XK_Linefeed) {
		// These characters are special, and we	
		// return the symbol 
		return(keysym) ;
	}
	else if (keysym == XK_Delete || keysym == XK_BackSpace) {
		if (numPrevChar >= 0) { // Erase a character
			lastCharString[0] = mText[numPrevChar];
    			lastCharString[1] = EOS ;
			SetClip(clipRect) ;
			if (inString) {
				// We must contract the string
				// The character to erase is # numPrevChar
				cpst = mText+numPrevChar ;
				x = xBegPos + XTextWidth(font,mText,numPrevChar) ;
				width = XTextWidth(font,cpst,numChars-numPrevChar) ;
				y = yBegPos ;
				SetRect(&tRect,x,y-height,x+width,y+descent) ;
				SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
				FillRect(&tRect) ;
 				SetFgColor(gBlackPixel) ;
				*cpst = EOS ; // Erases the char
				// Append the rest of the string
				strcat(mText,cpst+1) ;
				DrawText(xBegPos,textPlacement,mText);
   			}
			else {
				width = XTextWidth(font,lastCharString,1) ;
				mText[numPrevChar] = EOS ;
				x = EndStrXPos - width ;
				y = yBegPos ;
				SetRect(&tRect,x,y-height,x+width,y+descent) ;
				SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
				FillRect(&tRect) ;
 				SetFgColor(gBlackPixel) ;
			}
			SetClip(gCurrentPort->GetPortRect()) ;
			mInsertionPt.x = x ;
			mInsertionPt.y = y ;
		}
	}
	else if (keysym >= XK_Shift_L && keysym <= XK_Hyper_R) {
		// Nothing, Modifier keys
	}
	else if (keysym >= XK_F1 && keysym <= XK_F35) {
		// Nothing, function keys 
	}
	else {
		InputBuffer[1] = '\0' ;
		if (gDebugMode) {
			fprintf(stderr,"\nInput character = %s",InputBuffer) ;
		}
		if (inString) {
			// We must expand the string
			SetClip(clipRect) ;
			nInChars = 1 ; // always just one character // strlen(InputBuffer) ;
			cpst = mText+numPrevChar+1 ;
			strcpy(savestring,cpst) ; // Save the end
			if (numChars > 0) {
				x = xBegPos + XTextWidth(font,mText,numPrevChar+1) ;
				width = XTextWidth(font,cpst,numChars-numPrevChar-1) ;
				y = yBegPos ;
				SetRect(&tRect,x,y-height,x+width,y+descent) ;
				SetFgColor(gColors[near(0.8125*gNumGrayLevels)].pixel) ;
				FillRect(&tRect) ;
				SetFgColor(gBlackPixel) ;
			}
			*cpst = EOS ;
			// Append the input string
			strcat(mText,InputBuffer) ;
			strcat(mText,savestring) ;
			SetClip(clipRect) ;
			DrawText(xBegPos,textPlacement,mText);
			SetClip(gCurrentPort->GetPortRect()) ;
			x = xBegPos + XTextWidth(font,mText,
				numPrevChar+1+nInChars) ;
				mInsertionPt.x = x ;
		}
		else {
			nInChars = 1 ; // always just one character // strlen(InputBuffer) ;
			cpst = mText + numPrevChar + 1 ;
			SetClip(clipRect) ;
			for(i=0;i<nInChars;i++) {
				*cpst++ = InputBuffer[i] ;
				*cpst++ = EOS ;
				DrawChar(xCurrPos,textPlacement,InputBuffer[i]) ;
				xCurrPos = xBegPos + XTextWidth(font,mText,
				strlen(mText)) ;
			}
			SetClip(gCurrentPort->GetPortRect()) ;
			mInsertionPt.x = xCurrPos ;
		}
	}
	if (mActionRoutine != nil) {
		(*mActionRoutine)(this) ;
	}		
	Draw() ;
	return(true) ;
}

TScrollBar::TScrollBar(short x1,short y1, short x2, short y2,Orientation orientation,
			float minVal,float maxVal,float curVal)
				: TControl(x1,y1,x2,y2,nil,minVal,maxVal,curVal) { 
				
	float	fraction ;
	
	Rect	tRect = mRect ;
	InsetRect(&tRect,1,1) ;

	SetRect(&mRect,x1,y1,x2,y2) ;
	
	
	mOrientation = orientation ;
	short	availableSpace = mOrientation == kHorizontal ? (mRect.right - mRect.left - 32) :
				(mRect.bottom - mRect.top - 32) ;
	float	range = mMaxValue - mMinValue ;
	
	if (range < availableSpace) {
		mThumbSize = (availableSpace - range)/2 ;		
		mThumbSize *= 2 ;
	}
	else {
		fraction = availableSpace/(range+availableSpace) ;
		mThumbSize = near(availableSpace*fraction)/2 ;
		mThumbSize *= 2 ;
	}
	if (mThumbSize < 16) mThumbSize = 16 ;
	switch(mOrientation) {
		case kHorizontal:
			SetRect(&mPageUpRect,tRect.left,tRect.top,tRect.left+15,tRect.bottom) ;
			SetRect(&mPageDownRect,tRect.right-15,tRect.top,tRect.right,tRect.bottom) ;
			SetRect(&mScrollRect,tRect.left+16,tRect.top,tRect.right-16,tRect.bottom) ;
			InsetRect(&mScrollRect,mThumbSize/2,0) ; 
			mStepValue = range/(mScrollRect.right-mScrollRect.left) ;
			SetControlRect() ;
			break ;
		case kVertical:
			SetRect(&mPageUpRect,tRect.left,tRect.top,tRect.right,tRect.top+15) ;
			SetRect(&mPageDownRect,tRect.left,tRect.bottom-15,tRect.right,tRect.bottom) ;
			SetRect(&mScrollRect,tRect.left,tRect.top+16,tRect.right,tRect.bottom-16) ;
			InsetRect(&mScrollRect,0,mThumbSize/2) ;
			mStepValue = range/(mScrollRect.bottom-mScrollRect.top) ;
			SetControlRect() ;
			//fprintf(stderr,"\nTScrollBar::TScrollBar  : Rect = %d,%d,%d,%d",tRect.left,tRect.bottom-15,tRect.right,tRect.bottom) ;
			break ;
		default:
			break ;
	}
}  

void	TScrollBar::Resize(short x1,short y1, short x2, short y2,float maxValue) { 
	float	fraction ;

	SetRect(&mRect,x1,y1,x2,y2) ;
	mMaxValue = maxValue ;
	
	if (mCurrentValue > mMaxValue) mCurrentValue = mMaxValue ;
	
	Rect	tRect = mRect ;
	InsetRect(&tRect,1,1) ;
	
	short	availableSpace = mOrientation == kHorizontal ? (mRect.right - mRect.left - 32) :
				(mRect.bottom - mRect.top - 32) ;
	float	range = mMaxValue - mMinValue ;
	
	if (range < availableSpace) {
		mThumbSize = (availableSpace - range)/2 ;		
		mThumbSize *= 2 ;
	}
	else {
		fraction = availableSpace/(range+availableSpace) ;
		mThumbSize = near(availableSpace*fraction)/2 ;
		mThumbSize *= 2 ;
	}
	if (mThumbSize < 16) mThumbSize = 16 ;
	switch(mOrientation) {
		case kHorizontal:
			SetRect(&mPageUpRect,tRect.left,tRect.top,tRect.left+15,tRect.bottom) ;
			SetRect(&mPageDownRect,tRect.right-15,tRect.top,tRect.right,tRect.bottom) ;
			SetRect(&mScrollRect,tRect.left+16,tRect.top,tRect.right-16,tRect.bottom) ;
			InsetRect(&mScrollRect,mThumbSize/2,0) ; 
			mStepValue = range/(mScrollRect.right-mScrollRect.left) ;
			SetControlRect() ;
			break ;
		case kVertical:
			SetRect(&mPageUpRect,tRect.left,tRect.top,tRect.right,tRect.top+15) ;
			SetRect(&mPageDownRect,tRect.left,tRect.bottom-15,tRect.right,tRect.bottom) ;
			SetRect(&mScrollRect,tRect.left,tRect.top+16,tRect.right,tRect.bottom-16) ;
			InsetRect(&mScrollRect,0,mThumbSize/2) ;
			mStepValue = range/(mScrollRect.bottom-mScrollRect.top) ;
			SetControlRect() ;
			//fprintf(stderr,"\nTScrollBar::Resize  : Rect = %d,%d,%d,%d",tRect.left,tRect.bottom-15,tRect.right,tRect.bottom) ;
			break ;
		default:
			break ;
	}
}  

void TScrollBar::Draw() {
	
	if (!mParentWindow->IsVisible()) return ;
	RgnHandle clipRgn = NewRgn() ;
	GetClip(clipRgn) ;
	Rect tRect = mRect ;
	SetClip(mRect) ;
	SetFgColor(gGrayPixel) ;
	FillRect(&mRect) ;
	SetFgColor(gWhitePixel) ;	
	InsetRect(&tRect,1,1) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.left,tRect.bottom) ;
	//SetFgColor(gDarkerGray) ;
	DrawLine(tRect.left,tRect.bottom,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	SetFgColor(gBlackPixel) ;
	DrawControl() ;
	DrawPageUp() ;
	DrawPageDown() ;
	SetClip(clipRgn) ;
	DisposeRgn(clipRgn) ;
}


void	TScrollBar::DrawControl()  {

	if (!mParentWindow->IsVisible()) return ;
	Rect tRect = mThumbRect ;
	InsetRect(&tRect,1,1) ;
	SetFgColor(gWhitePixel) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.left,tRect.bottom) ;
	SetFgColor(gBlackPixel) ;
	DrawLine(tRect.left,tRect.bottom,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	
	short	cx,cy ;
	switch(mOrientation) {
		case kHorizontal:
			cx = (mThumbRect.right+mThumbRect.left)/2 ;
			DrawLine(cx-4,tRect.top+3,cx-4,tRect.bottom-3) ;
			DrawLine(cx,tRect.top+3,cx,tRect.bottom-3) ;
			DrawLine(cx+4,tRect.top+3,cx+4,tRect.bottom-3) ;
			SetFgColor(gWhitePixel) ;
			cx += 1 ;
			DrawLine(cx-4,tRect.top+3,cx-4,tRect.bottom-3) ;
			DrawLine(cx,tRect.top+3,cx,tRect.bottom-3) ;
			DrawLine(cx+4,tRect.top+3,cx+4,tRect.bottom-3) ;
			SetFgColor(gBlackPixel) ;
			break ;
		case kVertical:
			cy = (mThumbRect.bottom+mThumbRect.top)/2 ;
			DrawLine(tRect.left+3,cy-4,tRect.right-3,cy-4) ;
			DrawLine(tRect.left+3,cy,tRect.right-3,cy) ;
			DrawLine(tRect.left+3,cy+4,tRect.right-3,cy+4) ;
			SetFgColor(gWhitePixel) ;
			cy += 1 ;
			DrawLine(tRect.left+3,cy-4,tRect.right-3,cy-4) ;
			DrawLine(tRect.left+3,cy,tRect.right-3,cy) ;
			DrawLine(tRect.left+3,cy+4,tRect.right-3,cy+4) ;
			SetFgColor(gBlackPixel) ;
			break ;
		default:
			break ;
	}

}

void	TScrollBar::DrawPageUp()  {
	Rect tRect = mPageUpRect ;
	SetFgColor(gWhitePixel) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.left,tRect.bottom) ;
	SetFgColor(gBlackPixel) ;
	DrawLine(tRect.left,tRect.bottom,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	
	InsetRect(&tRect,2,2) ;
	switch(mOrientation) {
		case kHorizontal:
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,(tRect.bottom+tRect.top)/2,tRect.right,tRect.top) ;
			SetFgColor(gBlackPixel) ;
			DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.right,tRect.bottom,tRect.left,(tRect.bottom+tRect.top)/2) ;
			break ;
		case kVertical:
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.bottom,(tRect.left+tRect.right)/2,tRect.top) ;
			SetFgColor(gBlackPixel) ;
			DrawLine((tRect.left+tRect.right)/2,tRect.top,tRect.right,tRect.bottom) ;
			DrawLine(tRect.left,tRect.bottom,tRect.right,tRect.bottom) ;
			break ;
		default:
			break ;
	}
}

void	TScrollBar::DrawPageDown()  {
	Rect tRect = mPageDownRect ;
	SetFgColor(gWhitePixel) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.left,tRect.bottom) ;
	SetFgColor(gBlackPixel) ;
	DrawLine(tRect.left,tRect.bottom,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	SetFgColor(gBlackPixel) ;
	InsetRect(&tRect,2,2) ;
	switch(mOrientation) {
		case kHorizontal:
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.right,(tRect.bottom+tRect.top)/2,tRect.left,tRect.top) ;
			DrawLine(tRect.left,tRect.top,tRect.left,tRect.bottom) ;
			SetFgColor(gBlackPixel) ;
			DrawLine(tRect.left,tRect.bottom,tRect.right,(tRect.bottom+tRect.top)/2) ;
			break ;
		case kVertical:
			SetFgColor(gWhitePixel) ;
			DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
			DrawLine(tRect.left,tRect.top,(tRect.left+tRect.right)/2,tRect.bottom) ;
			SetFgColor(gBlackPixel) ;
			DrawLine((tRect.left+tRect.right)/2,tRect.bottom,tRect.right,tRect.top) ;
			break ;
		default:
			break ;
	}
	SetFgColor(gBlackPixel) ;
}

void	TScrollBar::EraseControl() {

	Rect tRect = mThumbRect ;
	InsetRect(&tRect,1,1) ;
	SetFgColor(gGrayPixel) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.left,tRect.bottom) ;
	DrawLine(tRect.left,tRect.bottom,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	
	short	cx,cy ;
	switch(mOrientation) {
		case kHorizontal:
			cx = (mThumbRect.right+mThumbRect.left)/2 ;
			DrawLine(cx-4,tRect.top+3,cx-4,tRect.bottom-3) ;
			DrawLine(cx,tRect.top+3,cx,tRect.bottom-3) ;
			DrawLine(cx+4,tRect.top+3,cx+4,tRect.bottom-3) ;
			cx += 1 ;
			DrawLine(cx-4,tRect.top+3,cx-4,tRect.bottom-3) ;
			DrawLine(cx,tRect.top+3,cx,tRect.bottom-3) ;
			DrawLine(cx+4,tRect.top+3,cx+4,tRect.bottom-3) ;
			SetFgColor(gBlackPixel) ;
			break ;
		case kVertical:
			cy = (mThumbRect.bottom+mThumbRect.top)/2 ;
			DrawLine(tRect.left+3,cy-4,tRect.right-3,cy-4) ;
			DrawLine(tRect.left+3,cy,tRect.right-3,cy) ;
			DrawLine(tRect.left+3,cy+4,tRect.right-3,cy+4) ;
			cy += 1 ;
			DrawLine(tRect.left+3,cy-4,tRect.right-3,cy-4) ;
			DrawLine(tRect.left+3,cy,tRect.right-3,cy) ;
			DrawLine(tRect.left+3,cy+4,tRect.right-3,cy+4) ;
			SetFgColor(gBlackPixel) ;
			break ;
		default:
			break ;
	}

}

void	TScrollBar::MoveControl(short delta) {
	float	newValue ;

	newValue = mCurrentValue + mStepValue*delta ;
	if (newValue > mMaxValue) newValue = mMaxValue ;
	if (newValue < mMinValue) newValue = mMinValue ;
	
	if (newValue != mCurrentValue) {				
		EraseControl() ;
		mCurrentValue = newValue ;
		SetControlRect() ;
		DrawControl() ;
		if (mTrackingRoutine != nil)
			(*mTrackingRoutine)(this) ;
	}
}

void	TScrollBar::SetControlRect() {
	Rect tRect = mRect ;
	float	valPosX,valPosY ;
	short	top,left,bottom,right ;
	switch(mOrientation) {
		case kHorizontal:
			valPosX = (mScrollRect.right-mScrollRect.left)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			left = mScrollRect.left + near(valPosX) - mThumbSize/2 ;
			right = left + mThumbSize ;
			SetRect(&mThumbRect,left,mScrollRect.top,right,mScrollRect.bottom) ;
			break ;
		case kVertical:
			valPosY = (mScrollRect.bottom-mScrollRect.top)*(mCurrentValue-mMinValue)/
				(mMaxValue-mMinValue) ;
			top =  mScrollRect.top + near(valPosY) - mThumbSize/2 ;
			bottom = top + mThumbSize ;
			SetRect(&mThumbRect,mScrollRect.left,top,mScrollRect.right,bottom) ;
			break ;
	}
}

Boolean TScrollBar::DoMouseDown(Point *pt) {
	Point	currentPos,lastPos ;
	short	dd ;

	if (!mIsVisible) return false ;
	if (!mIsActive) return false ;
    				
	GetMouse(&currentPos) ;
	lastPos = currentPos ;
	if (PtInRect(&currentPos,&mThumbRect)) {
		while (Button()) {
			GetMouse(&currentPos) ;
			switch(mOrientation) {
				case kHorizontal:
					dd = currentPos.x - lastPos.x ;
					if (dd != 0) {
						MoveControl(dd) ;
						lastPos = currentPos ;
					}
					break ;
				case kVertical:
					dd = currentPos.y - lastPos.y ;
					if (dd != 0) {
						MoveControl(dd) ;
						lastPos = currentPos ;
					}
					break ;
				default:
					break ;
			}
		}
		if (mActionRoutine != nil) {
			(*mActionRoutine)(this) ;
		}		
		Draw() ;
	}
	else if (PtInRect(&currentPos,&mPageUpRect)) {
		switch(mOrientation) {
			case kHorizontal:
				// try to move one 'page' at a time
				// Define a page as the length of the scrollbar
				dd = -near((mScrollRect.right-mScrollRect.left)/mStepValue) ;
				MoveControl(dd) ;
				break ;
			case kVertical:
				// try to move one 'page' at a time
				// Define a page as the length of the scrollbar
				dd = -near((mScrollRect.bottom-mScrollRect.top)/mStepValue) ;
				MoveControl(dd) ;
				break ;
			default:
				break ;
		}
	}
	else if (PtInRect(&currentPos,&mPageDownRect)) {
		switch(mOrientation) {
			case kHorizontal:
				// try to move one 'page' at a time
				// Define a page as the length of the scrollbar
				dd = near((mScrollRect.right-mScrollRect.left)/mStepValue) ;
				MoveControl(dd) ;
				break ;
			case kVertical:
				// try to move one 'page' at a time
				// Define a page as the length of the scrollbar
				dd = near((mScrollRect.bottom-mScrollRect.top)/mStepValue) ;
				MoveControl(dd) ;
				break ;
			default:
				break ;
		}
	}
	return true ;
}
