#include "TCalibrationDialog.h"
#include "TButton.h"
#include <stdio.h>

TCalibrationDialog::TCalibrationDialog(TCalibrationRecord *calibrationData) 
	: TDialog(kDialogPosX,kDialogPosY, 320, 186,"Calibrate Image") {
	
	char	text[256] ;
	TStaticText	*theText ;
	
	mCalibrationRecord = *calibrationData ;
	
	theText = new TStaticText(20,20,100,40,"Length in Pixels") ;
	AddObject(theText) ;
	
	sprintf(text,"%.2f",mCalibrationRecord.pixelLength) ;
	mTextPixelLength = new TStaticText(110,20,170,40,text) ;
	AddObject(mTextPixelLength) ;
	
	theText = new TStaticText(20,45,100,65,"Calibrated Length") ;
	AddObject(theText) ;
	
	sprintf(text,"%.2f",mCalibrationRecord.calibratedLength) ;
	mTextCalibratedLength = new TEditText(110,45,170,65,"",text) ;

	theText = new TStaticText(20,70,100,90,"Inverse Length") ;
	AddObject(theText) ;
	
	sprintf(text,"%f",1./mCalibrationRecord.calibratedLength) ;
	mTextInverseLength = new TEditText(110,70,170,90,"",text) ;
		
	AddTextEditObject(mTextInverseLength) ;
	AddTextEditObject(mTextCalibratedLength) ;
	
	theText = new TStaticText(20,90,190,136,"For inverse units enter the length in the \ncorresponding non inverse units\nwhen changing calibration") ;
	AddObject(theText) ;
	
	TGroupBox *grpBox = new TGroupBox(195,20,302,136,"Unit") ;
	AddObject(grpBox) ;
	
	mPixelUnit = new TRadioButton(210,30,300,46,"Pixel") ;
	AddObject(mPixelUnit) ;
	mInversePixelUnit = new TRadioButton(210,46,300,62,"1/Pixel") ;
	AddObject(mInversePixelUnit) ;
	mAngstromUnit = new TRadioButton(210,62,300,78,"Angstrom") ;
	AddObject(mAngstromUnit) ;
	mInverseAngstromUnit = new TRadioButton(210,78,300,94,"1/Angstrom") ;
	AddObject(mInverseAngstromUnit) ;
	mNanometerUnit = new TRadioButton(210,94,300,110,"nanometer") ;
	AddObject(mNanometerUnit) ;
	mInverseNanometerUnit = new TRadioButton(210,110,300,126,"1/nanometer") ;
	AddObject(mInverseNanometerUnit) ;
	
	mDefaultButton = mOKButton = new TButton(232,150,302,170,"OK") ;
	AddObject(mOKButton) ;
	
	mCancelButton = new TButton(152,150,222,170,"Cancel") ;
	AddObject(mCancelButton) ;
		
	switch(mCalibrationRecord.calibrationUnit) {
		case kPixel:
			mPixelUnit->SetState(1) ;
			mCurrentUnit = mPixelUnit ;
			mTextInverseLength->Activate(false) ;
			mTextCalibratedLength->SelectAll() ;
			mTextCalibratedLength->Activate(false) ;
			break ;
		case kAngstrom:
			mAngstromUnit->SetState(1) ;
			mCurrentUnit = mAngstromUnit ;
			mTextInverseLength->Activate(false) ;
			mTextCalibratedLength->SelectAll() ;
			break ;
		case kNanometer:
			mNanometerUnit->SetState(1) ;
			mCurrentUnit = mNanometerUnit ;
			mTextInverseLength->Activate(false) ;
			mTextCalibratedLength->SelectAll() ;
			break ;
		case kInversePixel:
			mInversePixelUnit->SetState(1) ;
			mCurrentUnit = mInversePixelUnit ;
			mTextCalibratedLength->Activate(false) ;
			mTextInverseLength->SelectAll() ;
			break ;
		case kInverseAngstrom:
			mInverseAngstromUnit->SetState(1) ;
			mCurrentUnit = mInverseAngstromUnit ;
			mTextCalibratedLength->Activate(false) ;
			mTextInverseLength->SelectAll() ;
			break ;
		case kInverseNanometer:
			mInverseNanometerUnit->SetState(1) ;
			mCurrentUnit = mInverseNanometerUnit ;
			mTextCalibratedLength->Activate(false) ;
			mTextInverseLength->SelectAll() ;
			break ;
	}
	
}

void	TCalibrationDialog::DoMouseDownInObject(TObject *obj,Point *pt) {
	if (pt != nil)
		if (!obj->DoMouseDown(pt)) return ;
	if (obj == mOKButton) {
		// Get the length of the line in whatever units
		char text[256] ;
		if (mTextCalibratedLength->IsEnabled()) {
			mTextCalibratedLength->GetEditText(text) ;
			mCalibrationRecord.calibratedLength = atof(text) ;
		}
		else {
			mTextInverseLength->GetEditText(text) ;
			mCalibrationRecord.calibratedLength = 1./atof(text) ;
		}
		mCalibrationRecord.calibrationConstant = mCalibrationRecord.calibratedLength/mCalibrationRecord.pixelLength ;
		mDone = true ;
		mDialogReturnCode = kDialogOK ;
	}
	else if (obj == mCancelButton) {
		mDone = true ;
		mDialogReturnCode = kDialogCancel ;
	}
	if (obj == mPixelUnit && mCurrentUnit != mPixelUnit) {
		mCalibrationRecord.calibrationUnit = kPixel ;
		mCurrentUnit->SetState(0) ;
		mCurrentUnit = mPixelUnit ;
		mCurrentUnit->SetState(1) ;
		Select(mTextCalibratedLength) ;
		if (mTextCalibratedLength->IsEnabled()) mTextCalibratedLength->Disable() ;
		if (mTextInverseLength->IsEnabled()) mTextInverseLength->Disable() ;
		mTextCalibratedLength->SelectAll() ;
	}
	if (obj == mAngstromUnit && mCurrentUnit != mAngstromUnit) {
		mCalibrationRecord.calibrationUnit = kAngstrom ;
		mCurrentUnit->SetState(0) ;
		mCurrentUnit = mAngstromUnit ;
		mCurrentUnit->SetState(1) ;
		mTextInverseLength->SetFocus(false) ;
		if (mTextInverseLength->IsEnabled()) mTextInverseLength->Disable() ;
		if (!mTextCalibratedLength->IsEnabled()) mTextCalibratedLength->Enable() ;
		Select(mTextCalibratedLength) ;
		mTextCalibratedLength->SelectAll() ;
	}
	if (obj == mNanometerUnit && mCurrentUnit != mNanometerUnit) {
		mCalibrationRecord.calibrationUnit = kNanometer ;
		mCurrentUnit->SetState(0) ;
		mCurrentUnit = mNanometerUnit ;
		mCurrentUnit->SetState(1) ;
		mTextInverseLength->SetFocus(false) ;
		if (mTextInverseLength->IsEnabled()) mTextInverseLength->Disable() ;
		if (!mTextCalibratedLength->IsEnabled()) mTextCalibratedLength->Enable() ;
		Select(mTextCalibratedLength) ;
		mTextCalibratedLength->SelectAll() ;
	}
	if (obj == mInversePixelUnit && mCurrentUnit != mInversePixelUnit) {
		mCalibrationRecord.calibrationUnit = kInversePixel ;
		mCurrentUnit->SetState(0) ;
		mCurrentUnit = mInversePixelUnit ;
		mCurrentUnit->SetState(1) ;
		mTextCalibratedLength->SetFocus(false) ;
		if (mTextCalibratedLength->IsEnabled()) mTextCalibratedLength->Disable() ;
		if (!mTextInverseLength->IsEnabled()) mTextInverseLength->Enable() ;
		Select(mTextInverseLength) ;
		mTextInverseLength->SelectAll() ;
	}
	if (obj == mInverseAngstromUnit && mCurrentUnit != mInverseAngstromUnit) {
		mCalibrationRecord.calibrationUnit = kInverseAngstrom ;
		mCurrentUnit->SetState(0) ;
		mCurrentUnit = mInverseAngstromUnit ;
		mCurrentUnit->SetState(1) ;
		mTextCalibratedLength->SetFocus(false) ;
		if (mTextCalibratedLength->IsEnabled()) mTextCalibratedLength->Disable() ;
		if (!mTextInverseLength->IsEnabled()) mTextInverseLength->Enable() ;
		Select(mTextInverseLength) ;
		mTextInverseLength->SelectAll() ;
	}
	if (obj == mInverseNanometerUnit && mCurrentUnit != mInverseNanometerUnit) {
		mCalibrationRecord.calibrationUnit = kInverseNanometer ;
		mCurrentUnit->SetState(0) ;
		mCurrentUnit = mInverseNanometerUnit ;
		mCurrentUnit->SetState(1) ;
		mTextCalibratedLength->SetFocus(false) ;
		if (mTextCalibratedLength->IsEnabled()) mTextCalibratedLength->Disable() ;
		if (!mTextInverseLength->IsEnabled()) mTextInverseLength->Enable() ;
		Select(mTextInverseLength) ;
		mTextInverseLength->SelectAll() ;
	}
}

