#ifndef TCrystal_h
#define TCrystal_h

#include <stdio.h>
#include "TDialog.h"
#include "TWindow.h"
#include "TAtom.h"
#include "TMatrix.h"
#include "TVector.h"
#include "CSelection.h"
#include "TImage.h"
#include "TList.h"


class   TCrystalWindow ;
class	TCrystal;


typedef float 	Matrix[4][4] ;
typedef float 	Vector[4] ;
typedef enum {
	kNoAnglePt,kfirstEndPoint,kVertex
} AnglePt ;
typedef enum {
	kNoPlanePt,kfirstPlanePt,ksecondPlanePt
} PlanePt ;


typedef struct GENERATOR {
         float matr[4][4] ;
       } GENERATOR ;

typedef enum {
	xRotAxis,yRotAxis,zRotAxis
} TypeRotAxis ;
#ifndef ToolTypeDefined
#define ToolTypeDefined
typedef enum  {
			XPointerTool=0,XRotateTool,XMagnifyingGlass,XRuler,XAngleTool,XSelectionTool
} XToolType ;
#endif

typedef struct {
	float cx,cy,cz ;	/* Position of the center */
	FPoint3D	corner[4] ;
	Vector normal ;
} UCellFace ;

typedef struct {
	float AngstrPrPixel ;
	Point Origin ;
	Rect  Boundary ;
} XSection ;

typedef struct RMatrix { 
	float m[3][3] ;
} RMatrix ;
typedef RMatrix *RMatrixPtr ;
typedef struct SMatrix { 
	short m[3][3] ;
} SMatrix ;
typedef SMatrix *SMatrixPtr ;
typedef struct LMatrix { 
	long m[3][3] ;
} LMatrix ;
typedef LMatrix *LMatrixPtr ;

typedef struct SVector {
	short a[3] ;
} SVector ;

typedef struct LVector {
	long a[3] ;
} LVector ;

class TMicroscope  {
	public:
		TMicroscope() ;
		TMicroscope(TMicroscope *microscope) ;
		~TMicroscope() ;

	    void    Update() ;

		char	mName[32] ;
		float	mVoltage ;
		float	mWavelength ;	
		float	mCs ;
		float	mConvergence ;
		float	mSpreadOfFocus ;
		float	m2Astigmatism ;
		float	m2AstigmatismAngle ;
		float	m3Astigmatism ;
		float	m3AstigmatismAngle ;
		float	m5Astigmatism ;
		float	m5AstigmatismAngle ;
		float	mComa ;
		float	mComaAngle ;
		float	mAperture ;
		float	mVibrationX ;
		float	mVibrationY ;
		float	mVibrationAngle ;
		float	mApertureH ;
		float	mApertureK ;
		float	mOpticAxisH ;
		float	mOpticAxisK ;
				
		float	mMinIntensityInLens ;
		
		float	mSigma ;
		float	mStartFocus,mEndFocus,mDeltaFocus ;
		
		TVector	mAperturePosition ;
		TVector mBeamTilt ;
		TVector GetBeamTilt() { return mBeamTilt ; }  ;
		float	GetWaveLength() { return mWavelength ; } ;
		float	GetInteractionConstant() { return mSigma ; } ;
		void	SetVoltage(float voltage) ;
} ;

class HRTEMImageSimulationRecord {
	public:
		HRTEMImageSimulationRecord() ;
		~HRTEMImageSimulationRecord() ;
		
		Boolean	SetCalculationSize(TCrystal *crystal) ;
						
		char	mTitle[256] ;
		float   mA ;
		float   mB ;
		float   mC ;
		float   mGamma ;
		float   mAStar ;
		float   mBStar ;
		float	mGMax ;
		TVector	mZoneAxis ;
		Boolean	m3DSlicing ;
		short	mNumSubSlices ;
		long	mNumUnitCellAtoms ;
		
		void	ShowParameters() ;
		
		TCrystal	*mSimulatedCrystal ;
		TCrystal	*mCrystal ;
		
		float	mStartThickness,mEndThickness,mDeltaThickness ;
		
		Boolean	mStoreReflections ;
		long	mNumReflectionsToStore ;
		
		SVector	*mReflectionsToStore ;
		
		TMicroscope mMicroscope ;
		
		long	mCalculationWidth ;
		long	mCalculationHeight ;
		
		short   mNumThicknesses ;
		short   mNumFocii ;
		
		Boolean mIsModified ;
		long	mCalculationStatus ;
		
}  ;

class	TBasisAtomDisplayList : public TList {
	public:
		TBasisAtomDisplayList(Rect	*theListRect) ;

		short   mSelectedAtom ;
		friend	class	TEditCrystalDialog ;
		
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TUnitCellAtomDisplayList : public TList {
	public:
		TUnitCellAtomDisplayList(Rect	*theListRect) ;
		
		friend	class	TEditCrystalDialog ;

	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TSymmopDisplayList : public TList {
	public:
		TSymmopDisplayList(Rect	*theListRect) ;
		
		friend	class	TEditCrystalDialog ;

	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
} ;

class	TEditCrystalDialog : public TDialog {
	public:
		TEditCrystalDialog(TCrystal *theCrystal) ;
		~TEditCrystalDialog() ;
		
		void	DoMouseDownInObject(TObject *,Point *);
		void	DoKeyDownInObject(TObject *obj) ;
		void	ShowAtom(TAtom &theAtom) ;
		void	RetrieveDialogItems() ;
		
		TCrystal	*GetCrystal() { return mCrystal ; } ;
		Boolean		mCrystalIsModified ;
		
	private:
	
		TCrystal	*mCrystal ;
		TCrystal	*mSourceCrystal ;
		TBasisAtomDisplayList	*mBasisDisplayList ;
		TSymmopDisplayList		*mSymmopDisplayList ;
		TUnitCellAtomDisplayList	*mAtomDisplayList ;
		
		void		UpdateMotif() ;
		void		UpdateSpaceGroup() ;
		
		
		TButton		*mApplySpacegroup ;
		TButton		*mAddAtom ;
		TButton		*mDeleteAtom ;
		TButton		*mModifyAtom ;
		
		TEditText	*mTextA ;
		TEditText	*mTextB ;
		TEditText	*mTextC ;
		TEditText	*mTextAlpha ;
		TEditText	*mTextBeta ;
		TEditText	*mTextGamma ;
		TEditText	*mTextSpacegroup ;

		TEditText	*mTextAtomSymbol ;
		TEditText	*mTextAtomX ;
		TEditText	*mTextAtomY ;
		TEditText	*mTextAtomZ ;
		TEditText	*mTextAtomDW ;
		TEditText	*mTextAtomOCC ;

} ;


class   THRTEMSimulationParameterDialog : public TDialog {
public:
	THRTEMSimulationParameterDialog(HRTEMImageSimulationRecord *simRec) ;
	void	RetrieveDialogItems() ;
protected:
		HRTEMImageSimulationRecord *mHRTEMImageSimulationRecord ;
	
	TEditText   *mZoneU ;
	TEditText   *mZoneV ;
	TEditText   *mZoneW ;
	TEditText   *mThicknessStart ;
	TEditText   *mThicknessInc ;
	TEditText   *mThicknessEnd ;
	
	TEditText   *mGMax ;
	
	TEditText   *mFocusStart ;
	TEditText   *mFocusInc ;
	TEditText   *mFocusEnd ;
	
	TEditText   *mCs ;
	TEditText   *mDiv ;
	TEditText   *mDelta ;
	TEditText   *mVoltage ;
	
} ;

#ifndef _StructureFileType_
#define _StructureFileType_
typedef enum {kNCEMSS_FILE,kMBFIT_FILE} StructureFileType ;
#endif

typedef enum {kMBFIT_SIM_MODE,kMBFIT_DISPERSION_MODE,kMBFIT_BLOCH_MODE} MBFIT_SimulationMode ;
typedef enum {kMBFIT_CALC_KXY_RANGE_0D,kMBFIT_CALC_KXY_RANGE_1D,kMBFIT_CALC_KXY_RANGE_Disk,kMBFIT_CALC_KXY_RANGE_Square } CALC_KXY_RANGE_Type ;
typedef enum {kMBFIT_ManualBeamSearch,kMBFIT_AutoBeamSearch_WG,kMBFIT_AutoBeamSearch_SG} MBFIT_BeamSearchType ;
typedef struct MBFITAutoBeamSearchRec {
	short   num_g_limit ;
	float   g_max ;
	short   lz_min ;
	short   lz_max ;
	float   sg_exact_max ;
	float   sg_bethe_max ;
	float   xi_g_max ;
} MBFITAutoBeamSearchRec ;

class MBFITManualBeamSearchRec {
public:
	MBFITManualBeamSearchRec(short numExact=0, short numBethe=0) ;
	~MBFITManualBeamSearchRec() ;
	short   num_exact ;
	short   num_bethe ;
	LVector *hklExact ;
	LVector *hklBethe ;
}  ;

class MBFITSimulationRecord {
	public:
		MBFITSimulationRecord() ;
		~MBFITSimulationRecord() ;

		void	ShowParameters() ;
			
		MBFIT_SimulationMode mMode ;

		short   mNumCalculationPoints ;
		short   mNumCalculationPointsX ;
		short   mNumCalculationPointsY ;
		float	mMinH ;
		float	mMaxH ;
		float	mMinG ;
		float	mMaxG ;
		float	mMinX ;
		float	mMaxX ;
		float	mMinY ;
		float	mMaxY ;
		
		short	mNumLowOrderG ;
		LVector *mLowOrderG ;

		short	mNumExact ;
		LVector *mExact ;

		short	mNumBethe ;
		LVector *mBethe ;
		
		short	branchBegin ;
		short	branchEnd ;

		short   mNumReflectionsToOutput ;
		LVector *mOutputReflection ;

		LVector	mZoneAxis ;
		LVector	mSurfaceNormal ;
		LVector	mZoneBaseVector ;
		
		float	mTiltH ;
		float	mTiltG ;
		
		short	mNumThicknesses ;
		float	*mThicknesses ;		
		TMicroscope mMicroscope ;

		Boolean mIsModified ;
		
		CALC_KXY_RANGE_Type mCalcType ;
		MBFITAutoBeamSearchRec	mBeamSearchAuto ;
		MBFITManualBeamSearchRec	mBeamSearchManual ;
		
		MBFIT_BeamSearchType mBeamSearchType ;
		
}  ;


class   TMBFITSimulationParameterDialog : public TDialog {
public:
	TMBFITSimulationParameterDialog(MBFITSimulationRecord *simRec) ;
	void	DoMouseDownInObject(TObject *,Point *);
	void	DoKeyDownInObject(TObject *obj) ;
	void	RetrieveDialogItems() ;
protected:
		MBFITSimulationRecord *mMBFITSimulationRecord ;
	
	TEditText   *mZoneU ;
	TEditText   *mZoneV ;
	TEditText   *mZoneW ;

	TEditText   *mSurfaceNormalU ;
	TEditText   *mSurfaceNormalV ;
	TEditText   *mSurfaceNormalW ;

	TEditText   *mBaseVectorU ;
	TEditText   *mBaseVectorV ;
	TEditText   *mBaseVectorW ;

	TEditText   *mTiltH ;
	TEditText   *mTiltK ;
	
	TRadioButton	*m2D_Disk ;
	TRadioButton	*m2D_Square ;
	TRadioButton	*m1D ;
	TRadioButton	*m0D ;
	
	TEditText   *mPixelsAcrossDisk ;
	TEditText   *mH_Min ;
	TEditText   *mH_Max ;	
	TEditText   *mG_Min ;
	TEditText   *mG_Max ;

	TEditText   *mVoltage ;
	
	TEditText   *mThickness ;
	TButton	*mAddThickness ;
	TButton	*mDeleteThickness ;
	
	TList	*mThicknessList ;
	TList	*mBeamList ;

	TRadioButton	*mOutputAllBeams ;
	TRadioButton	*mOutputSelectedBeams ;
	
	TEditText   *mBeam ;
	TButton	*mAddBeam ;
	TButton	*mDeleteBeam ;
	
	TEditText   *mXYRange_XPoints ;
	TEditText   *mXYRange_YPoints ;
	
	TEditText   *mXYRange_XBegin ;
	TEditText   *mXYRange_XEnd ;
	TEditText   *mXYRange_YBegin ;
	TEditText   *mXYRange_YEnd ;
	
	TRadioButton	*mAutomaticBeamSearch ;
	TRadioButton	*mManualBeamSearch ;

	TEditText   *mAutoMaxReflections ;
	TEditText   *mAutoZ_Min ;
	TEditText   *mAutoZ_Max ;
	TEditText   *mAutoGMax ;
	TEditText   *mAutoSG_Exact_Max ;
	TEditText   *mAutoSG_Bethe_Max ;
	TEditText   *mAutoX1_G_Max ;
	
	TList		*mExactList ;
	TList		*mBetheList ;

	TEditText   *mExact ;
	TButton	*mAddExact ;
	TButton	*mDeleteExact ;

	TEditText   *mBethe ;
	TButton	*mAddBethe ;
	TButton	*mDeleteBethe ;

	TEditText   *mBranchStart ;
	TEditText   *mBranchEnd ;
	
	TEditText   *mGMax ;

	
} ;

class TCrystal {
	public:
				
		TCrystal(FSInfo &fsInfo) ;
		TCrystal(TCrystalWindow *crystalWindow,FSInfo &fsInfo) ;
		TCrystal(TCrystal *s) ;
		~TCrystal(void) ;
		
		TCrystal	&operator=(const TCrystal& s) ;
		void	DrawCrystal() ;
		void	DrawCrystal(TVector &zone) ;
		void 	Print() ;
		void 	Rotate(void) ;
		void 	Invalidate(void)  ;
		void	ChangeMagnification(float factor)  ;
		void	SetMagnification(float magnification) ;
		void	PanView(Point &loc) ;
		void	ResetView() ;
		void	SizeAtoms(Boolean reduceSize) ;
		void	SetSelection(CSelection *selection) { mSelection = selection ; } ;
		void	SetSymmops(short numOrigSymops,GENERATOR *symop,RMatrixPtr transfMatrix) ;
		void	SetSymmops(short spacegroupNumber,Boolean secondSetting,RMatrixPtr transfMatrix) ;
		Boolean		mDrawOutline ;
		Boolean		mDrawOrigin;
		Boolean		HasAtomsSelected() { return mHasAtomsSelected ; } ;
		SAtom		mFirstAtom ;
		SAtom		mSecondAtom ;
		void 		MeasureDistance(const Point& where) ;
		void 		MeasureAngle(const Point& where) ;
		void		Inquire(const Point& where) ;
		void		ShowSelectedAtom(TAtom *atom) ;
		void		UpdateWindowSize() ;
		TAtom		*GetAtom(short ndx) { return (mAtomPtr + ndx) ; } ;
		
		HRTEMImageSimulationRecord	mTEMRecord ;
		MBFITSimulationRecord		mMBFITRecord ;
		
		TWindow		*GetWindow() { return (TWindow *)mDocWindow ; } ;

		friend	class TCrystalWindow ;
		friend	class HRTEMImageSimulationRecord ;
		friend	class MBFITSimulationRecord ;
		friend	class TEditCrystalDialog ;

	private:
	
		FSInfo  mFSInfo ;
		Boolean	InitializeFromNCEMSSFile(FSInfo &fsInfo) ;
		Boolean	InitializeFromMBFITFile(FSInfo &fsInfo) ;
		Boolean	InitializeFromNCEMSSPrintFile(FSInfo &fsInfo) ;
		
		void	Edit() ;
		
		void	WriteMBFITControlFile() ;		
		void	WriteNCEMSSControlFile() ;
		
		void	WriteInfo(char *) ;
		void	Draw() ;
		void	ApplySymmops() ;
		void	Assert(void *ptr) ;
		void	SortScreenAtoms(void) ;
		void 	AddViewAtom(float x,float y, float z,float TZ0,float scale,short i,float rad,RGBColor *color) ;
		void 	SetTMatrix() ;
		void 	SetSMatrix() ;
		void 	SetDisRotMatrix(TypeRotAxis whichAxis,float angle) ;
		void 	Draw2DScreenBalls(void) ;
		void 	SetUCellFaces(void) ;
		void	DrawInvisibleUCellFaces(void) ;
		void	DrawVisibleUCellFaces(void) ;
		void 	DrawAtom(SAtom *atom) ;
		Boolean IsFaceVisible(UCellFace face) ;
		void 	DrawFace(UCellFace face) ;
		void 	SortFaces(void) ;
		void	SortAtomsByType() ;
		void	SetAtomTypes() ;
		void	Update() ;
		Boolean FindAtom(Point& mouseLoc,SAtom *whichAtom) ;	
		SAtom	*GetAtom(Point& mouseLoc) ;
		long 	CalculateNumFaceAtoms(void) ;
		void 	CreateView(short na,short nb,short nc) ;
		void	UpdateViewAtoms() ;
		void	SetTransformationMatrix(TVector &view) ;
		void	CalculateRealZoneFromReciprocal(TVector uvw) ; 
		void 	RecipToRlVector(TVector &rcpVec,TVector &rlVec) ;
		void 	RlToRecipVector(TVector &rcpVec,TVector &rlVec) ;
		float 	LengthOfRecipVector(TVector &vec) ;
		float 	LengthOfRealVector(TVector &vec) ;
		void 	RecipToCartVector(TVector &rcVec,TVector &cartVec) ;		// new
		void	SetScale() ;
		void	SetCenter(void) ;
		void	CreateUnitCellViewAtoms() ;
		void	DrawUnitCell() ;
		void	DrawOrigin();  // 9/96
		void	LoadSpacegroup(char *spcgrp,short number) ;
		Point 	VectorToPoint(TVector inV);
		void	TransformGenerator(GENERATOR *symop) ;
		Boolean	NewSymop(GENERATOR *p_symop,long num_symop) ;
		Boolean Equivalent(TAtom *atomList,float x,float y,float z,long icount,long ndx) ;
		void	HiliteAtom(TAtom *theAtom, Point AtomPt);  // bjf 12/96
		void	SelectThisAtom(float x,float y,float z,float TZ0,float scale,TAtom *theAtom) ;		
		
		
		float 	mA ;				/* Unit cell axis in Angstrom	*/
		float 	mB ;				/* Unit cell axis in Angstrom	*/
		float 	mC ;				/* Unit cell axis in Angstrom	*/
		float 	mAlpha ;			/* Unit cell angle in degrees	*/
		float 	mBeta ;				/* Unit cell angle in degrees	*/
		float 	mGamma ;			/* Unit cell angle in degrees	*/
		TVector mZone ;				/* Current Viewing Direction	*/
		
		long		mAllocatedAtoms ;
		Boolean		mHasAtomsSelected ;

		
		float		mLightX ;
		float		mLightY ;
		float		mLightZ ;
		short		mXViewCells ;
		short		mYViewCells ;
		short		mZViewCells ;
		long		mNumFaceAtoms ;
		long		mShiftX ;
		long		mShiftY ;
		
		long		mSnumAtoms ;
		long		mNumViewAtoms ;

		AtomDef		mWAtom ;
		AnglePt		mAnglePt ;
		PlanePt		mPlanePt ;
		Point		mFirstPt ;
		Point		mSecondPt ;
		
		float		mMagnification ;
		float		mAtomScaleFactor ;
				
		UCellFace	mUCellFace[6] ;
		XSection	mXSection ;
		
		TCrystalWindow		*mDocWindow ;
		Rect		mDisplayRect ;
		short		mWidth ;
		short		mHeight ;
		
		short		mSpacegroup ;
		short		mSecondSetting ;
		short		mHexagonalIndices ;
		short		mAlternateSetting ;
		short		mCentroSymmetric ;
		short		mNumTypes ;
				
		CSelection	*mSelection ;
		long		mNumBasisAtoms ;
		long		mNumUnitCellAtoms ;
		short		mNumSymOps ;
		char		mSymopTxt[192][64] ;
		
		TAtom  	*mAtomPtr ;				/* Pointer to the record of 	*/
										/* atoms in the unit cell		*/
		TAtom	*mBasisPtr ;
		TMatrix	*mSymOps ;				/* Holds the symetry operators for this "unit cell"	*/
		SAtom	*mScreenAtom ;			/* Pointer to a list of screen atoms	*/
		TMatrix mXyzToScreenMatrix ;	/* Transformation matrix that 	*/
										/* transforms coordinates in	*/
										/* unit cell coordinates to the	*/
										/* coordinate system that is 	*/
										/* current at a particular time	*/
										/* This would usually be that 	*/
										/* of the interface				*/
		TMatrix mRotMatrix ;			/* Current Rotation Matrix		*/
		TMatrix mTmatrix ;				/* Matrix for transforming between the unit cell 	*/
										/* and the local orthonormal coordinates system  	*/
										/* for the crystal (a,b,c) = T(xl,yl,zl)			*/
		TMatrix	mSmatrix ;				/* (a,b,c) = S (ast,bst,cst) 	for the crystal 	*/
		TMatrix mTst ;					/* The equivalent of T, but for the reciprocal	 	*/
										/* lattice.					 						*/
		TMatrix	mT ;	// The matrix that takes the old axes into the new axes of the zone
		TMatrix	mU ;	// U is just the transpose of the inverse of T
		TVector mO ;	// mO is the origin of the new unit cell if shifted from the original		
} ;

#endif // TCrystal_h
