// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------


#include "TDialog.h"
#include "XDrivers.h"
#include "forms.h"
#include <stdlib.h>
#include "globals.h"

void	AddWindow(TWindow *window) ;
void	RemoveWindow(TWindow *window) ;
void	ProcessModalEvent(XEvent *event) ;
extern	short	gImageDepth ;
extern	Time	gClickTime ;
extern 	Boolean	gIsDoubleClick ;
extern	XFontStruct	*gMenuFont ;
extern	XFontStruct	*gDialogFont ;
extern	XFontStruct	*gWindowFont ;

void	PutMessage(char *) ;
void	PutMessage(char *text) { DisplayMessage(text) ; } ;
extern  void	SetTargetWindow(TWindow *window) ;
extern  Boolean	gFrontWindowIsModal ;

TDialog::TDialog(short xloc,short yloc,short width,short height,char *title) 
	: TWindow(xloc,yloc,width,height,gImageDepth,title,false) {
	
	GetPort(&mPreviousPort) ;

	gFrontWindowIsModal = true ;

	mFont = gDialogFont ;
	XSetFont(gDisplay,mGC,mFont->fid) ;

	mObjectList = new TObjectList ;
	mEditTextList = new TEditTextList ;

	mOKButton = nil ;
	mCancelButton = nil ;
	mDefaultButton = nil ;
	
	DefineDialog() ;
	SetBkColor(gGrayPixel) ;
	mCurrentTextEditField = nil ;
	mDone = false ;
	mIsVisible = false ;
	mWindowKind = 'dilg' ;

}

TDialog::~TDialog() {
	gFrontWindowIsModal = false ;
	SetPort(mPreviousPort) ;
}

void	TDialog::Show() {
	BringToFront(this) ;
	gFrontWindow = this ;
	SetPort(this) ;
}

void	TDialog::DoUpdate(XEvent *) {
	SetPort(this) ;
	Draw() ;
}

void	TDialog::Draw() {
	SetPort(this) ;
	EraseRect(&mPortRect) ;
	TPane::Draw() ; 
	if (mDefaultButton != nil) {
		Rect tRect = mDefaultButton->GetRect() ;
		InsetRect(&tRect,-2,-2) ;
		PenSize(2,2) ;
		FrameRect(&tRect) ;
		PenSize(1,1) ;
	}
}

DialogReturnCode TDialog::DoModalDialog() {
	XEvent event ;
	FL_OBJECT	*theFormsObject ;
	
	Show() ;
	while (!mDone) {
		theFormsObject = fl_do_forms() ;
		if (theFormsObject == FL_EVENT) {
			fl_XNextEvent(&event) ;
			DoEvent(&event) ;
		}
	}
	if (mDialogReturnCode == kDialogOK)
		RetrieveDialogItems() ;
	return mDialogReturnCode ;
}


void	TDialog::DoMouseDown(XEvent *theEvent) {
	Point thePoint ;
	TObject *theObject = nil ;
	TEditText	*editText ;

	if (mObjectList->GetNumberOfObjects() > 0) {
		thePoint.x = theEvent->xbutton.x ;
		thePoint.y = theEvent->xbutton.y ;
		theObject = FindObject(&thePoint) ;
		if (theObject != nil) {
			if ((editText=(TEditText *)mEditTextList->GetObject(&thePoint)) != nil) {
				if (mCurrentTextEditField != editText) {
					mCurrentTextEditField->SetFocus(false) ;
				}
				mCurrentTextEditField = editText ;
				mCurrentTextEditField->SetFocus(true) ;
			}
			DoMouseDownInObject(theObject,&thePoint) ;
		}
	}
}


void	TDialog::DoMouseDownInObject(TObject *obj,Point *pt) {
	if (pt != nil)
		if (!obj->DoMouseDown(pt)) return ;
	if (obj == mOKButton) {
		mDone = true ;
		mDialogReturnCode = kDialogOK ;
	}
	else if (obj == mCancelButton) {
		mDone = true ;
		mDialogReturnCode = kDialogCancel ;
	}
}

void	TDialog::DoKeyDown(XEvent *theEvent) {
	KeySym	theKey ;
	if (mCurrentTextEditField != nil) {
		theKey = mCurrentTextEditField->DoKeyDown(theEvent) ;
		DoKeyDownInObject(mCurrentTextEditField) ;
		if (theKey == XK_Tab) {
			mCurrentTextEditField->SetFocus(false) ;
			mCurrentTextEditField = 
					mEditTextList->GetNextField(mCurrentTextEditField) ;
			mCurrentTextEditField->SelectAll() ;
		}
	}
}

void	TDialog::DoEvent(XEvent *theEvent) {
	// If the event doesn't belong to us
	// send it to ProcessModalEvent so that other windows
	// can get updated
	if (theEvent->xany.window != mXWindow) {
		ProcessModalEvent(theEvent) ;
		return ;
	}
	XButtonEvent *buttEvent = (XButtonEvent *)theEvent ;
	KeySym theKey ;
	switch(theEvent->type) {
		case ButtonPress:
			gIsDoubleClick = false ; 
			if ((buttEvent->time - gClickTime) < 400) gIsDoubleClick = true ;
			gClickTime	= buttEvent->time ;
			DoMouseDown(theEvent) ;
			break ;
		case KeyPress:
			theKey = GetEventKey(theEvent) ;
			if (theKey == XK_Return && mDefaultButton != nil) {
				DoMouseDownInObject(mDefaultButton,nil) ;
				break ;
			}
			DoKeyDown(theEvent) ;
			break ;
		case MapNotify:
			DoMapNotify(theEvent) ;
			break ;
		case UnmapNotify:
			DoUnmapNotify(theEvent) ;
			break ;
		case Expose:
			DoUpdate(theEvent) ;
			break ;
	}
}

TMessageDialog::TMessageDialog(short xloc,short yloc,short width,
	short height,char *text) : TDialog(xloc,yloc,width,height) {

	TStaticText *theText = new TStaticText(10,20,width-10,40,text) ;
	AddObject(theText) ;
	mOKButton = new TButton(width-90,height-35,width-10,height-10,"OK") ;
	AddObject(mOKButton) ;

}


TMessageDialog::TMessageDialog(char *text) : TDialog(kDialogPosX,kDialogPosY,200,100) {
	TStaticText *theText = new TStaticText(10,10,180,30,text) ;
	AddObject(theText) ;
	mOKButton = new TButton(110,65,190,80,"OK") ;
	AddObject(mOKButton) ;
}

void TMessageDialog::DefineDialog() {
}

TCancelOKDialog::TCancelOKDialog(short xloc,short yloc,short width,
	short height,char *text) : TDialog(xloc,yloc,width,height) {

	TStaticText *theText = new TStaticText(10,20,width-10,height-50,text) ;
	AddObject(theText) ;
	mCancelButton = new TButton(width-180,height-45,width-100,height-10,"Cancel") ;
	AddObject(mCancelButton) ;
	mOKButton = new TButton(width-90,height-45,width-10,height-10,"OK") ;
	AddObject(mOKButton) ;

}

TCancelOKDialog::TCancelOKDialog(char *text) : TDialog(kDialogPosX,kDialogPosY,200,100) {
	TStaticText *theText = new TStaticText(10,10,180,30,text) ;
	AddObject(theText) ;
	mCancelButton = new TButton(20,65,100,90,"Cancel") ;
	AddObject(mCancelButton) ;
	mOKButton = new TButton(110,65,190,90,"OK") ;
	AddObject(mOKButton) ;
}

TGetOneRealNumberDialog::TGetOneRealNumberDialog(char *text,float initialValue) 
	: TDialog(kDialogPosX,kDialogPosY,200,100) {
	
	char valueText[256] ;
	sprintf(valueText,"%.2f",initialValue) ;
	
	mTextValue = new TEditText(130,20,180,40,text,valueText) ;
	AddTextEditObject(mTextValue) ;

	mDefaultButton = mOKButton = new TButton(120,60,180,80,"OK") ;
	AddObject(mOKButton) ;

	mCancelButton = new TButton(50,60,110,80,"Cancel") ;
	AddObject(mCancelButton) ;
}

void	TGetOneRealNumberDialog::RetrieveDialogItems() {
	char valueText[256] ;
	mTextValue->GetEditText(valueText) ;
	mValue = atof(valueText) ;
}

TGetOneIntegerNumberDialog::TGetOneIntegerNumberDialog(char *text,long initialValue) 
	: TDialog(kDialogPosX,kDialogPosY,200,100) {
	
	char valueText[256] ;
	sprintf(valueText,"%d",initialValue) ;
	
	mTextValue = new TEditText(130,20,180,40,text,valueText) ;
	AddTextEditObject(mTextValue) ;
	
	mDefaultButton = mOKButton = new TButton(120,60,180,80,"OK") ;
	AddObject(mOKButton) ;

	mCancelButton = new TButton(50,60,110,80,"Cancel") ;
	AddObject(mCancelButton) ;

}

void	TGetOneIntegerNumberDialog::RetrieveDialogItems() {
	char valueText[256] ;
	mTextValue->GetEditText(valueText) ;
	mValue = atoi(valueText) ;
}

Boolean	CancelOKDialog(char *text) {
	Point pos ;
	short	width,height,x,y ;
 
	GetGlobalMouse(&pos) ;
	width = 220 ;
	height = 100 ;
	x = pos.x - width + 35 ;
	y = pos.y - height + 15 ;
	TCancelOKDialog *theDialog = new TCancelOKDialog(x,y,width,height,text) ;
	DialogReturnCode retCode = theDialog->DoModalDialog() ;
	delete theDialog ;
	return (Boolean)retCode ;
}

void DisplayMessage(char *text) {
	Point pos ;
	short	width,height,x,y ;

	GetGlobalMouse(&pos) ;
	width = 280 ;
	height = 100 ;
	x = pos.x - width + 35 ;
	y = pos.y - height + 15 ;
	
	if (x < 20) x = 20 ;
	if (y < 40) y = 40 ;
	if (x+width > 1024) x = 1024 - width ;
	if (y+height > 768) x = 768 - height ;
	TMessageDialog *theDialog = new TMessageDialog(x,y,width,height,text) ;
	theDialog->DoModalDialog() ;
	delete theDialog ;
}

float	GetRealNumberWithPrompt(char *text,float defValue) {
	float	retValue = defValue ;
	TGetOneRealNumberDialog *theDialog = new TGetOneRealNumberDialog(text,defValue) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&retValue) ;
	}
	delete theDialog ;
	return retValue ;
}

long	GetIntegerNumberWithPrompt(char *text,long defValue) {
	long	retValue = defValue ;
	TGetOneIntegerNumberDialog *theDialog = new TGetOneIntegerNumberDialog(text,defValue) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&retValue) ;
	}
	delete theDialog ;
	return retValue ;
}

