// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------

extern	bool gUse_FORMS ;
#include <unistd.h>
#include "TFile.h"

void	ChangeWorkingDirectory() ;
void	ChangeWorkingDirectory(FSInfo &theFSInfo) ;

TFile::TFile() {
	mFP = nil ;
	mSize = 0 ;
	mData = nil ;
	mIsOpen = false ;
}

TFile::TFile(char *fileName) {
	mFP = nil ;
	mSize = 0 ;
	mData = nil ;
	mIsOpen = false ;
	strcpy(mFileName,fileName) ;
}

TFile::TFile(FSInfo &fsInfo) {
	mFP = nil ;
	mSize = 0 ;
	mData = nil ;
	mIsOpen = false ;
	strcpy(mFileName,fsInfo.mFileName) ;
}

TFile::TFile(FSInfo &fsInfo,FileMode	fileMode) {
	mFP = nil ;
	mSize = 0 ;
	mData = nil ;
	mIsOpen = false ;
	strcpy(mFileName,fsInfo.mFileName) ;
	Open(fileMode) ;
}

TFile::TFile(char *fileName,FileMode	fileMode) {
	mFP = nil ;
	mSize = 0 ;
	mData = nil ;
	mIsOpen = false ;
	strcpy(mFileName,fileName) ;
	Open(fileMode) ;
}


TFile::~TFile() {
	if (mIsOpen && mFP != nil)
		fclose(mFP) ;
}

Boolean	TFile::Open(FileMode	fileMode) {
	long	fileStartPos,fileEndPos ;
	switch(fileMode) {
		case kRead:
			mFP = fopen(mFileName,"r") ;
			// Get the file size
			fileStartPos = ftell(mFP) ;
			if (fseek(mFP,0,2) == 0) {
				fileEndPos = ftell(mFP) ;
				mSize = fileEndPos - fileStartPos ;
				fseek(mFP,fileStartPos,0) ;
			}
			break ;
		case kWrite:
			mFP = fopen(mFileName,"w") ;
			break ;
	}
	if (mFP != nil)
		mIsOpen = true ;
        return mIsOpen ;
}

void *TFile::Read() {
	unsigned char *buffer = new unsigned char[mSize+8] ;
	if (Read(buffer,mSize)) {
		// We will null terminate this if we should hav e a stream of data
		// that we need to read as text
		// Take the last byte and make it zero
		unsigned char *pp = (unsigned char *)buffer ;
		*(pp+mSize) = '\0' ;
		return buffer ;
	}
		
	delete[] buffer ;
	return nil ;
}

Boolean	TFile::Write(void *data) {
	return Write(data,mSize) ;
}

Boolean	TFile::Read(void *data,ULong numBytes) {
	ULong numBytesRead ;

	numBytesRead = fread(data,1,numBytes,mFP) ;
	if (numBytesRead == numBytes) {
		return(true) ;
	}
	else 
		return (false) ;
}

Boolean	TFile::Write(void *data,ULong numBytes) {
	ULong numBytesWritten ;
	numBytesWritten = fwrite(data,1,numBytes,mFP) ;
	if (numBytesWritten == numBytes)
		return(true) ;
	else 
		return (false) ;
}

FSInfo::FSInfo() {
	mReply = false ;
	strcpy(mFileName,"") ;
}

// Let us make sure that we keep the last path
char	gCurrentOpenPath[256] = {'.','\0'} ;
void	ChooseFile(char *prompt,char *pattern,FSInfo &fsInfo) {
	char *directoryName ;
	if (gUse_FORMS) {
		directoryName = (char *)fl_show_fselector(prompt,gCurrentOpenPath,pattern,fsInfo.mFileName) ;
		if (directoryName != NULL) {
			strcpy(fsInfo.mFileName,directoryName) ;
			if (strlen(fsInfo.mFileName) == 0)
				fsInfo.mReply = false ;
			else {
				fsInfo.mReply = true ;				
				strcpy(gCurrentOpenPath,directoryName) ;
				// Find the path excluding the file name				
				char *fs = strrchr(gCurrentOpenPath,'/') ;
				if (fs != NULL) *fs = '\0' ;
			}
		}
		else {
			fsInfo.mReply = false ; 
		}
	}
	else {
		strcpy(fsInfo.mFileName,"437Half.tif") ;
		fsInfo.mReply = true ;
	}

}
	
void	ChooseSaveFile(char *prompt,FSInfo &fsInfo) {
	
	char *directoryName = (char *)fl_show_fselector(prompt,gCurrentOpenPath,"",
		fsInfo.mFileName) ;
		
	if (directoryName != NULL) {
		strcpy(fsInfo.mFileName,directoryName) ;
		fsInfo.mReply = true ;
	}
	else {
		fsInfo.mReply = false ; 
	}
}

void	ChangeWorkingDirectory() {
	char	workingDirectory[256] ;

	getcwd(workingDirectory,256) ;	
	char *directoryName = (char *)fl_show_fselector("Set Working Directory (Choose any File in the Dir.)",gCurrentOpenPath,"","") ;
		
	if (directoryName != NULL) {
		strcpy(workingDirectory,directoryName) ;
		// Find the path excluding the file name				
		char *fs = strrchr(workingDirectory,'/') ;
		if (fs != NULL) *fs = '\0' ;
		chdir(workingDirectory) ;
		
		// Read it back
		getcwd(workingDirectory,256) ;
		fprintf(stderr,"\nCurrent working directory is %s  ",workingDirectory) ;
	}	
}

void	ChangeWorkingDirectory(FSInfo &theFSInfo) {
	char	workingDirectory[256] ;		
	strcpy(workingDirectory,theFSInfo.mFileName) ;
	// Find the path excluding the file name				
	char *fs = strrchr(workingDirectory,'/') ;
	if (fs != NULL) *fs = '\0' ;
	chdir(workingDirectory) ;
}
