// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------


#include "TMask.h"
#include "TImageWindow.h"
#include "TImage.h"
#include "globals.h"
#define	hKandleSize	1

extern	long	near(float) ;
extern	void	WriteObjectInfo(char *text) ;

void	TMask::DoMouseDown(Point &pt) {

	if (!mIsSelected) {
		if (InMask(pt)) {
			mIsSelected = true ;
		}
		else 
			return ;
	}
	else {
		if (!InMask(pt)) {
			mIsSelected = false ;
			return ;
		}
	}
	// Allow adjusting the tool
	if (ControlKeyDown())
		Edit(pt) ;
	else  
		Modify(pt) ;
}

TSpotMask::TSpotMask(TImageWindow *window) : TMask(window) {
	mX = window->GetImage()->Width()/2 ;
	mY = window->GetImage()->Height()/2 ;
	mU.a[0] = 10 ;
	mU.a[1] = -10 ;
	mSpotRadius = 5 ;
	char	objectInfo[256] ;
	GetObjectInfo(objectInfo) ;
	::WriteObjectInfo(objectInfo) ;
}

TSpotMask::~TSpotMask() {
}

void	TSpotMask::Draw() {

	SetFgColor(gRedPixel) ;	
	DrawCircle(near(mX+mU.a[0]),near(mY+mU.a[1]),near(mSpotRadius)) ;
	DrawCircle(near(mX-mU.a[0]),near(mY-mU.a[1]),near(mSpotRadius)) ;
	SetFgColor(gBlackPixel) ;
	
	if (mIsSelected)
		DrawHandles() ;
	
}

void	TSpotMask::GetObjectInfo(char *text)  {

	CalibrationUnit calibrationUnit = mWindow->GetImage()->GetCalibrationUnit() ;
	float	calibrationConstant = mWindow->GetImage()->GetCalibration() ;
	float	u = sqrt(mU.a[0]*mU.a[0]+mU.a[1]*mU.a[1])*calibrationConstant ;
	float	ga	= 90.0/asin(1.0)*atan2(-mU.a[1],mU.a[0]) ;
	switch(calibrationUnit) {
		case kPixel:
			sprintf(text,"Spot Mask: [Pixels]\nU=%.1f Ga=%.1f Deg",u,ga) ;
			break ;
		case kInversePixel:
			sprintf(text,"Spot Mask: [1/Pixels]\nU=1/%.1f Ga=%.1f Deg",1/u,ga) ;
			break ;
		case kAngstrom:
			sprintf(text,"Spot Mask: [Angstrom]\nU=%.2f Ga=%.1f Deg",u,ga) ;
			break ;
		case kInverseAngstrom:
			sprintf(text,"Spot Mask: [1/Angstrom]\nU=1/%.2f Ga=%.1f Deg",1/u,ga) ;
			break ;
		case kNanometer:
			sprintf(text,"Spot Mask: [nm]\nU%.2f Ga=%.1f Deg",u,ga) ;
			break ;
		case kInverseNanometer:
			sprintf(text,"Spot Mask: [1/nm]\nU=1/%.2f Ga=%.1f Deg",1/u,ga) ;
			break ;
	}
}

void	TSpotMask::EditMaskparameters() {
	MaskRecord maskRecord ;
	maskRecord.numLatticeVectorsA = kUnusedMaskParameter ;
	maskRecord.numLatticeVectorsB = kUnusedMaskParameter ;
	maskRecord.maskWidth = kUnusedMaskParameter ;
	maskRecord.edgeWidth = mEdgeWidth ;
	maskRecord.maskIsOpaque = mMaskType == kTransparentMask ? 0 : 1 ;
	
	TEditMaskDialog *theDialog = new TEditMaskDialog(&maskRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&maskRecord) ;
		mEdgeWidth = maskRecord.edgeWidth ;
		mMaskType = maskRecord.maskIsOpaque ? kOpaqueMask : kTransparentMask ;
	}
	delete theDialog ;
}

void	TSpotMask::DrawHandles() {

	SetFgColor(gGreenPixel) ;	
	Rect	selectionRect ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]-mSpotRadius),near(mY+mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]+mSpotRadius),near(mY+mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1]+mSpotRadius)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1]-mSpotRadius)) ;
	FrameRect(&selectionRect) ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]-mSpotRadius),near(mY-mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]+mSpotRadius),near(mY-mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]),near(mY-mU.a[1]-mSpotRadius)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]),near(mY-mU.a[1]+mSpotRadius)) ;
	FrameRect(&selectionRect) ;

	SetFgColor(gBlackPixel) ;
}

bool	TSpotMask::InHandles(Point &pt) {
	Rect	selectionRect ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]-mSpotRadius),near(mY+mU.a[1])) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]+mSpotRadius),near(mY+mU.a[1])) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1]+mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1]-mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]-mSpotRadius),near(mY-mU.a[1])) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]+mSpotRadius),near(mY-mU.a[1])) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]),near(mY-mU.a[1]+mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mU.a[0]),near(mY-mU.a[1]-mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	return false ;
}

void	TSpotMask::Modify(Point &pt) {
	Point	where,lastPoint ;
	bool	inNegativePart = false ;
	float	cX,cY ;
	
	Boolean	inHandles = InHandles(pt) ;

	cX = mX ;
	cY = mY ;

	FPoint	cp ;
	FPoint	u,v ;
	cp.x = mX ;
	cp.y = mY ;
	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX - mU.a[0] ;
	v.y = mY - mU.a[1] ;
	
	mWindow->ImageToWindowPoint(cp) ;
	mWindow->ImageToWindowPoint(u) ;
	mWindow->ImageToWindowPoint(v) ;
	mX = cp.x ;
	mY = cp.y ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	
	mSpotRadius *= mWindow->GetMagnification() ;
	long	radsq,tolsq = (mSpotRadius+3)*(mSpotRadius+3) ;
	
	GetMouse(&where) ;
	
	lastPoint = where ;
	mIsSelected = false ;
	PenMode(kXOrMode);
	DrawBaseUnits() ;
	if (!inHandles) {
		// Move the mask
		// Test if positive or negative part
		radsq = near((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
		if (radsq < tolsq) 
			inNegativePart = true ;
		while (Button()) {
			GetMouse(&where) ;
			if (where.x != lastPoint.x || where.y != lastPoint.y) {
				DrawBaseUnits() ;
				if (inNegativePart) {
					mU.a[0] -= (where.x - lastPoint.x) ;
					mU.a[1] -= (where.y - lastPoint.y) ;
				}
				else {
					mU.a[0] += (where.x - lastPoint.x) ;
					mU.a[1] += (where.y - lastPoint.y) ;
				}
				lastPoint = where ;
				DrawBaseUnits() ;
			}
		}
	}
	else {
		// Size the mask
		// Test if positive or negative part
		radsq = near((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
		if (radsq < tolsq) {
			inNegativePart = true ;
		}
		while (Button()) {
			GetMouse(&where) ;
			if (where.x != lastPoint.x || where.y != lastPoint.y) {
				DrawBaseUnits() ;
				if (inNegativePart) {
					mSpotRadius = sqrt((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
				}
				else {
					mSpotRadius = sqrt((where.x-u.x)*(where.x-u.x)+(where.y-u.y)*(where.y-u.y)) ;
				}
				lastPoint = where ;
				DrawBaseUnits() ;
			}
		}
	}
	
	u.x = mU.a[0] + mX ;
	u.y = mU.a[1] + mY ;
	
	mSpotRadius /= mWindow->GetMagnification() ;
	mWindow->WindowToImagePoint(u) ;
	mX = cX ;
	mY = cY ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	PenNormal() ;
	mIsSelected = true ;
}

bool	TSpotMask::InMask(Point &pt) {
	long	radsq,tolsq = (mSpotRadius+3)*(mSpotRadius+3) ;
	
	if (InHandles(pt))
		return true ;
	
	radsq = near((pt.x-(mX+mU.a[0]))*(pt.x-(mX+mU.a[0]))+(pt.y-(mY+mU.a[1]))*(pt.y-(mY+mU.a[1]))) ;
	if (radsq < tolsq) 
		return true ;
	radsq = near((pt.x-(mX-mU.a[0]))*(pt.x-(mX-mU.a[0]))+(pt.y-(mY-mU.a[1]))*(pt.y-(mY-mU.a[1]))) ;
	if (radsq < tolsq) 
		return true ;
	return false ;
}

void	TSpotMask::Apply(TImage *image) {
	TArray	*dataArray = image->GetDataArray() ;
	bool	dataIsComplex = dataArray->IsComplex() ;
	
	// Create the mask
	short	width = dataArray->GetWidth() ;
	short	height = dataArray->GetHeight() ;
	TArray	*mask = new TArray(width,height) ;
	
	short	i,j,ix,iy,radius = near(mSpotRadius+mEdgeWidth) ;
	
	
	radius = mMaskType == kTransparentMask ? near(mSpotRadius+mEdgeWidth) : near(mSpotRadius-mEdgeWidth) ;
	if (radius < 0.) radius = 0. ;

	float	rsq,ysq,r0sq,r1sq,arg,r ;
	r0sq = mSpotRadius*mSpotRadius ;
	r1sq = radius*radius ;
	if (mEdgeWidth > 0) arg = asin(1.0)/mEdgeWidth ;
	
	ix = near(mX + mU.a[0]) ;
	iy = near(mY + mU.a[1]) ;

	switch (mMaskType) {
		case kTransparentMask:
			// Traverse the mask and put down either a sharp circle mask if mEdgeWidth = 0
			// or a cos-square edge if mEdgeWidth > 0
			*mask = 0. ;
			// Check to see if mask is within the image
			// Create the mask at this spot
			if (dataIsComplex) {
				for(j=iy-radius; j <= iy+radius; j++) {
					ysq = (j-iy)*(j-iy) ;
					for(i=ix-radius; i <= ix+radius; i++) {
						rsq = (i-ix)*(i-ix) + ysq ;
						if (rsq <= r0sq) {
							*mask->GetDataHK(i-mX,j-mY) = 1. ;
							*mask->GetDataHK(-i+mX,-j+mY) = 1. ;
						}
						else if (mEdgeWidth > 0 && rsq <= r1sq) {
							r = sqrt(rsq) - mSpotRadius ;
							*mask->GetDataHK(i-mX,j-mY) = cos(arg*r) ;
							*mask->GetDataHK(-i+mX,-j+mY) = cos(arg*r) ;
						}
					}
				}
			}
			else {
				for(j=iy-radius; j <= iy+radius; j++) {
					ysq = (j-iy)*(j-iy) ;
					for(i=ix-radius; i <= ix+radius; i++) {
						rsq = (i-ix)*(i-ix) + ysq ;
						if (rsq <= r0sq) {
							*mask->GetData(i,j) = 1. ;
							*mask->GetData(width-i,height-j) = 1. ;
						}
						else if (mEdgeWidth > 0 && rsq <= r1sq) {
							r = sqrt(rsq) - mSpotRadius ;
							*mask->GetData(i,j) = cos(arg*r) ;
							*mask->GetData(width-i,height-j) = cos(arg*r) ;
						}
					}
				}
			}
			break ;
		case kOpaqueMask:
			*mask = 1. ;
			radius = near(mSpotRadius) ;
			if (dataIsComplex) {
				for(j=iy-radius; j <= iy+radius; j++) {
					ysq = (j-iy)*(j-iy) ;
					for(i=ix-radius; i <= ix+radius; i++) {
						rsq = (i-ix)*(i-ix) + ysq ;
						if (rsq <= r1sq) {
							*mask->GetDataHK(i-mX,j-mY) = 0. ;
							*mask->GetDataHK(-i+mX,-j+mY) = 0. ;
						}
						else if (mEdgeWidth > 0 && rsq <= r0sq) {
							r = mSpotRadius - sqrt(rsq) ;
							*mask->GetDataHK(i-mX,j-mY) = cos(arg*r) ;
							*mask->GetDataHK(-i+mX,-j+mY) = cos(arg*r) ;
						}
					}
				}
			}
			else {
				for(j=iy-radius; j <= iy+radius; j++) {
					ysq = (j-iy)*(j-iy) ;
					for(i=ix-radius; i <= ix+radius; i++) {
						rsq = (i-ix)*(i-ix) + ysq ;
						if (rsq <= r1sq) {
							*mask->GetData(i,j) = 0. ;
							*mask->GetData(width-i,height-j) = 0. ;
						}
						else if (mEdgeWidth > 0 && rsq <= r0sq) {
							r = mSpotRadius - sqrt(rsq) ;
							*mask->GetData(i,j) = cos(arg*r) ;
							*mask->GetData(width-i,height-j) = cos(arg*r) ;
						}
					}
				}
			}
			break ;
	}
	*dataArray *= *mask ;
	delete mask ;
}


TCircleMask::TCircleMask(TImageWindow *window) : TMask(window) {
	mX = window->GetImage()->Width()/2 ;
	mY = window->GetImage()->Height()/2 ;
	mInnerRadius = mX/4 ;
	mOuterRadius = mX/2 ;
	
	char	objectInfo[256] ;
	GetObjectInfo(objectInfo) ;
	::WriteObjectInfo(objectInfo) ;
}

TCircleMask::~TCircleMask() {
}

void	TCircleMask::EditMaskparameters() {
	MaskRecord maskRecord ;
	maskRecord.numLatticeVectorsA = kUnusedMaskParameter ;
	maskRecord.numLatticeVectorsB = kUnusedMaskParameter ;
	maskRecord.maskWidth = kUnusedMaskParameter ;
	maskRecord.edgeWidth = mEdgeWidth ;
	maskRecord.maskIsOpaque = mMaskType == kTransparentMask ? 0 : 1 ;
	
	TEditMaskDialog *theDialog = new TEditMaskDialog(&maskRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&maskRecord) ;
		mEdgeWidth = maskRecord.edgeWidth ;
		mMaskType = maskRecord.maskIsOpaque ? kOpaqueMask : kTransparentMask ;
	}
	delete theDialog ;
}

void	TCircleMask::Draw() {

	SetFgColor(gRedPixel) ;	
	DrawCircle(near(mX),near(mY),near(mInnerRadius)) ;
	DrawCircle(near(mX),near(mY),near(mOuterRadius)) ;
	SetFgColor(gBlackPixel) ;
	if (mIsSelected)
		DrawHandles() ;
}

void	TCircleMask::GetObjectInfo(char *text)  {
	CalibrationUnit calibrationUnit = mWindow->GetImage()->GetCalibrationUnit() ;
	float	calibrationConstant = mWindow->GetImage()->GetCalibration() ;
	float	r1 = mInnerRadius*calibrationConstant ;
	float	r2 = mOuterRadius*calibrationConstant ;
	switch(calibrationUnit) {
		case kPixel:
			sprintf(text,"Circular Mask: [Pixels]\nR1=%.1f R2=%.1f",r1,r2) ;
			break ;
		case kInversePixel:
			sprintf(text,"Circular Mask: [1/Pixels]\nR1=1/%.1f R2=1/%.1f",r1,r2) ;
			break ;
		case kAngstrom:
			sprintf(text,"Circular Mask: [Angstrom]\nR1=%.2f R2=%.2f",r1,r2) ;
			break ;
		case kInverseAngstrom:
			sprintf(text,"Circular Mask: [1/Angstrom]\nR1=1/%.2f R2=1/%.2f",r1,r2) ;
			break ;
		case kNanometer:
			sprintf(text,"Circular Mask: [nm]\nR1=%.2f R2=%.2f",r1,r2) ;
			break ;
		case kInverseNanometer:
			sprintf(text,"Circular Mask: [1/nm]\nR1=1/%.2f R2=1/%.2f",r1,r2) ;
			break ;
	}
}

void	TCircleMask::DrawHandles() {

	SetFgColor(gGreenPixel) ;	
	Rect	selectionRect ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mInnerRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mInnerRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mInnerRadius)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mInnerRadius)) ;
	FrameRect(&selectionRect) ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mOuterRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mOuterRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mOuterRadius)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mOuterRadius)) ;
	FrameRect(&selectionRect) ;

	SetFgColor(gBlackPixel) ;
}

bool	TCircleMask::InHandles(Point &pt) {
	Rect	selectionRect ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mInnerRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mInnerRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mInnerRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mInnerRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mOuterRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mOuterRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mOuterRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mOuterRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	return false ;
}

void	TCircleMask::Modify(Point &pt) {
	Point	where,lastPoint ;
	bool	inInnerCircle = false ;
	long	rad ;
	float	cX,cY ;
	
	// This mask can only be mofified in size, not moved
	
	cX = mX ;
	cY = mY ;

	FPoint	cp ;
	cp.x = mX ;
	cp.y = mY ;
	
	mWindow->ImageToWindowPoint(cp) ;
	mX = cp.x ;
	mY = cp.y ;
	
	mOuterRadius *= mWindow->GetMagnification() ;
	mInnerRadius *= mWindow->GetMagnification() ;
		
	GetMouse(&where) ;

	rad = sqrt((where.x-mX)*(where.x-mX)+(where.y-mY)*(where.y-mY)) ;
	if (fabs(rad - mInnerRadius) < 3.) 
		inInnerCircle = true ;
		
	if (!inInnerCircle) {
		if (fabs(rad - mOuterRadius) > 3.)  {
			mX = cX ;
			mY = cY ;
			mOuterRadius /= mWindow->GetMagnification() ;
			mInnerRadius /= mWindow->GetMagnification() ;
			return ;
		}
	}
	
	lastPoint = where ;
	mIsSelected = false ;
	PenMode(kXOrMode);
	
	
	DrawBaseUnits() ;
	
	// Size the mask
	// Test if inner or outer circle
	while (Button()) {
		GetMouse(&where) ;
		if (where.x != lastPoint.x || where.y != lastPoint.y) {
			DrawBaseUnits() ;
			if (inInnerCircle) {
				mInnerRadius = sqrt((where.x-mX)*(where.x-mX)+(where.y-mY)*(where.y-mY)) ;
			}
			else {
				mOuterRadius = sqrt((where.x-mX)*(where.x-mX)+(where.y-mY)*(where.y-mY)) ;
			}
			lastPoint = where ;
			DrawBaseUnits() ;
		}
	}
	mX = cX ;
	mY = cY ;
	mOuterRadius /= mWindow->GetMagnification() ;
	mInnerRadius /= mWindow->GetMagnification() ;
	if (mOuterRadius < mInnerRadius) {
		float tempradius = mOuterRadius ;
		mOuterRadius = mInnerRadius ;
		mInnerRadius = tempradius ;
	}
	PenNormal() ;
	mIsSelected = true ;

}

bool	TCircleMask::InMask(Point &pt) {
	long	rad ;
	
	if (InHandles(pt))
		return true ;
	rad = sqrt((pt.x-mX)*(pt.x-mX)+(pt.y-mY)*(pt.y-mY)) ;
	if (fabs(rad-mInnerRadius) < 3.)
		return true ;
	if (fabs(rad-mOuterRadius) < 3.)
		return true ;
	return false ;
}

void	TCircleMask::Apply(TImage *image) {
	TArray	*dataArray = image->GetDataArray() ;
	bool	dataIsComplex = dataArray->IsComplex() ;
	
	// Create the mask
	short	width = dataArray->GetWidth() ;
	short	height = dataArray->GetHeight() ;
	TArray	*mask = new TArray(width,height) ;

	float	rsq,ysq,r0sq,r1sq,r00sq,r11sq,arg,r ;
	short	i,j ;
	
	if (mInnerRadius < 3) mInnerRadius = 0 ; 	// It's hard to size the mask properly
	r0sq = mInnerRadius*mInnerRadius ;
	r1sq = mOuterRadius*mOuterRadius ;
	if (mInnerRadius-mEdgeWidth < 0) 
		r00sq = 0. ;
	else 
		r00sq = (mInnerRadius-mEdgeWidth)*(mInnerRadius-mEdgeWidth) ;
		
	r11sq = (mOuterRadius+mEdgeWidth)*(mOuterRadius+mEdgeWidth) ;
	if (mEdgeWidth > 0) arg = asin(1.0)/mEdgeWidth ;
	switch (mMaskType) {
		case kTransparentMask:
			// Traverse the mask and put down either a sharp circle mask if mEdgeWidth = 0
			// or a cos-square edge if mEdgeWidth > 0
			*mask = 0. ;
			// Create the mask at this spot
			if (dataIsComplex) {
				for(j=0; j < height; j++) {
					ysq = (j-mY)*(j-mY) ;
					for(i=0; i < width; i++) {
						rsq = (i-mX)*(i-mX) + ysq ;
						if (rsq >= r0sq && rsq <= r1sq) {
							*mask->GetDataHK(i-mX,j-mY) = 1. ;
						}
						else if (mEdgeWidth > 0) {
							if (rsq < r0sq && rsq > r00sq) {
								r = mInnerRadius - sqrt(rsq) ;
								*mask->GetDataHK(i-mX,j-mY) = cos(arg*r) ;
							}
							else if (rsq > r1sq && rsq < r11sq) {
								r =  sqrt(rsq) - mOuterRadius ;
								*mask->GetDataHK(i-mX,j-mY) = cos(arg*r) ;
							}
						}
					}
				}
			}
			else {
				for(j=0; j < height; j++) {
					ysq = (j-mY)*(j-mY) ;
					for(i=0; i < width; i++) {
						rsq = (i-mX)*(i-mX) + ysq ;
						if (rsq >= r0sq && rsq <= r1sq) {
							*mask->GetData(i,j) = 1. ;
						}
						else if (mEdgeWidth > 0) {
							if (rsq < r0sq && rsq > r00sq) {
								r = mInnerRadius - sqrt(rsq) ;
								*mask->GetData(i,j) = cos(arg*r) ;
							}
							else if (rsq > r1sq && rsq < r11sq) {
								r =  sqrt(rsq) - mOuterRadius ;
								*mask->GetData(i,j) = cos(arg*r) ;
							}
						}
					}
				}
			}
			break ;
		case kOpaqueMask:
			*mask = 1. ;
			// Create the mask at this spot
			if (dataIsComplex) {
				for(j=0; j < height; j++) {
					ysq = (j-mY)*(j-mY) ;
					for(i=0; i < width; i++) {
						rsq = (i-mX)*(i-mX) + ysq ;
						if (rsq >= r0sq && rsq <= r1sq) {
							*mask->GetDataHK(i-mX,j-mY) = 0. ;
						}
						else if (mEdgeWidth > 0) {
							if (rsq < r0sq && rsq > r00sq) {
								r = mInnerRadius - sqrt(rsq) ;
								*mask->GetDataHK(i-mX,j-mY) = 1-cos(arg*r) ;
							}
							else if (rsq > r1sq && rsq < r11sq) {
								r =  sqrt(rsq) - mOuterRadius ;
								*mask->GetDataHK(i-mX,j-mY) = 1-cos(arg*r) ;
							}
						}
					}
				}
			}
			else {
				for(j=0; j < height; j++) {
					ysq = (j-mY)*(j-mY) ;
					for(i=0; i < width; i++) {
						rsq = (i-mX)*(i-mX) + ysq ;
						if (rsq >= r0sq && rsq <= r1sq) {
							*mask->GetData(i,j) = 0. ;
						}
						else if (mEdgeWidth > 0) {
							if (rsq < r0sq && rsq > r00sq) {
								r = mInnerRadius - sqrt(rsq) ;
								*mask->GetData(i,j) = 1-cos(arg*r) ;
							}
							else if (rsq > r1sq && rsq < r11sq) {
								r =  sqrt(rsq) - mOuterRadius ;
								*mask->GetData(i,j) = 1-cos(arg*r) ;
							}
						}
					}
				}
			}
			break ;
	}
	*dataArray *= *mask ;
	delete mask ;
}


TLatticeMask::TLatticeMask(TImageWindow *window)  : TMask(window) {
	mX = window->GetImage()->Width()/2 ;
	mY = window->GetImage()->Height()/2 ;
	mU.a[0] = mX/2 ;
	mU.a[1] = 0 ;
	mV.a[0] = 0 ;
	mV.a[1] = -mY/2 ;
	mSpotRadius = 5 ;
	mNumLaticeVectorsU = 1 ;
	mNumLaticeVectorsV = 1 ;
	char	objectInfo[256] ;
	GetObjectInfo(objectInfo) ;
	::WriteObjectInfo(objectInfo) ;
}

TLatticeMask::~TLatticeMask() {
}

void	TLatticeMask::EditMaskparameters() {
	MaskRecord maskRecord ;
	maskRecord.numLatticeVectorsA = mNumLaticeVectorsU ;
	maskRecord.numLatticeVectorsB = mNumLaticeVectorsV ;
	maskRecord.maskWidth = kUnusedMaskParameter ;
	maskRecord.edgeWidth = mEdgeWidth ;
	maskRecord.maskIsOpaque = mMaskType == kTransparentMask ? 0 : 1 ;
	
	TEditMaskDialog *theDialog = new TEditMaskDialog(&maskRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&maskRecord) ;
		mNumLaticeVectorsU = maskRecord.numLatticeVectorsA ;
		mNumLaticeVectorsV = maskRecord.numLatticeVectorsB ;
		mEdgeWidth = maskRecord.edgeWidth ;
		mMaskType = maskRecord.maskIsOpaque ? kOpaqueMask : kTransparentMask ;
	}
	delete theDialog ;
}

void	TLatticeMask::Draw() {

	SetFgColor(gRedPixel) ;	

	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	
	// Draw All the lattice masks that fit on the image
	// Compute the max h & k index that can possibly fit on the image
	float	ux = mU.a[0]/mNumLaticeVectorsU ;
	float	uy = mU.a[1]/mNumLaticeVectorsU ;
	float	vx = mV.a[0]/mNumLaticeVectorsV ;
	float	vy = mV.a[1]/mNumLaticeVectorsV ;
	
	float	u = sqrt(ux*ux+uy*uy) ;
	float	v = sqrt(vx*vx+vy*vy) ;
	
	float	csalpha = (ux*vx+uy*vy)/(u*v) ;
	float	sialpha = sqrt(1.-csalpha*csalpha) ;
	short	maxH = near(sqrt(mX*mX+mY*mY)/(u*sialpha)) ;
	short	maxK = near(sqrt(mX*mX+mY*mY)/(v*sialpha)) ;
	
	short	h,k,ix,iy,maxX,maxY,radius = near(mSpotRadius) ;
	maxX 	= 2*mX - radius ;
	maxY 	= 2*mY - radius ;
	for(h = -maxH; h < maxH; h++) 
		for(k = -maxK; k < maxK; k++) {
			ix = near(mX + h*ux + k*vx) ;
			iy = near(mY + h*uy + k*vy) ;
			// Check to see if mask is within the image
			if (ix > radius && ix < maxX && iy > radius && iy < maxY) {
				// Draw the mask on this spot
				DrawCircle(ix,iy,radius) ;
			}
		}	
	SetFgColor(gBlackPixel) ;
	DrawBaseUnits() ;	
}

void	TLatticeMask::GetObjectInfo(char *text)  {
	CalibrationUnit calibrationUnit = mWindow->GetImage()->GetCalibrationUnit() ;
	float	calibrationConstant = mWindow->GetImage()->GetCalibration() ;
	float	u = sqrt(mU.a[0]*mU.a[0]+mU.a[1]*mU.a[1]) ;
	float	v = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1]) ;
	float	ga	= 90.0/asin(1.0)*acos((mU.a[0]*mV.a[0]+mU.a[1]*mV.a[1])/(u*v)) ;
	u	*= calibrationConstant ;
	v	*= calibrationConstant ;
	switch(calibrationUnit) {
		case kPixel:
			sprintf(text,"Lattice Mask: [Pixels]\nU=%.1f V=%.1f Ga=%.1f Deg",u,v,ga) ;
			break ;
		case kInversePixel:
			sprintf(text,"Lattice Mask: [1/Pixels]\nU=1/%.1f V=1/%.1f Ga=%.1f Deg",1/u,1/v,ga) ;
			break ;
		case kAngstrom:
			sprintf(text,"Lattice Mask: [Angstrom]\nU=%.2f V=%.2f Ga=%.1f Deg",u,v,ga) ;
			break ;
		case kInverseAngstrom:
			sprintf(text,"Lattice Mask: [1/Angstrom]\nU=1/%.2f V=1/%.2f Ga=%.1f Deg",1/u,1/v,ga) ;
			break ;
		case kNanometer:
			sprintf(text,"Lattice Mask: [nm]\nU%.2f V=%.2f Ga=%.1f Deg",u,v,ga) ;
			break ;
		case kInverseNanometer:
			sprintf(text,"Lattice Mask: [1/nm]\nU=1/%.2f V=1/%.2f Ga=%.1f Deg",1/u,1/v,ga) ;
			break ;
	}
}

void	TLatticeMask::DrawBaseUnits() {
	SetFgColor(gRedPixel) ;	

	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	SetFgColor(gBluePixel) ;	
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mV.a[0]),near(mY+mV.a[1])) ;

	SetFgColor(gRedPixel) ;	
	
	DrawCircle(near(mX),near(mY),mSpotRadius) ;
	
	// Draw arrows
	double	k,b,x0,y0,x1,x2,y1,y2,A,B,L,d,a1,a2,a3,a4,xw1,yw1,xw2,yw2,w ;
	w = 4 ;
	d = 8 ;
	x1 = mX ;
	y1 = mY ;

	x2 = mX + mU.a[0] ;
	y2 = mY + mU.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (mU.a[0] != 0.) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;
		
	SetFgColor(gBluePixel) ;	
	x2 = mX + mV.a[0] ;
	y2 = mY + mV.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (mV.a[0] != 0.) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;

	
	SetFgColor(gBlackPixel) ;
	if (mIsSelected)
		DrawHandles() ;
}

void	TLatticeMask::DrawHandles() {
	

	SetFgColor(gGreenPixel) ;	
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mSpotRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mSpotRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mSpotRadius)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mSpotRadius)) ;
	FrameRect(&selectionRect) ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	FrameRect(&selectionRect) ;
	SetFgColor(gBlackPixel) ;
}

bool	TLatticeMask::InHandles(Point &pt) {
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mSpotRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mSpotRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
		
	long	rad = near(sqrt((pt.x-mX)*(pt.x-mX)+(pt.y-mY)*(pt.y-mY))) ;
	if (rad < mSpotRadius+2) 
		return true ;

	return false ;
}

void	TLatticeMask::Edit(Point &pt) {
	long	radsq,tolsq = 10 ;
	radsq = near((pt.x-(mX+mU.a[0]))*(pt.x-(mX+mU.a[0]))+(pt.y-(mY+mU.a[1]))*(pt.y-(mY+mU.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) {
			mNumLaticeVectorsU -= 1 ;
		}
		else {
			mNumLaticeVectorsU += 1 ;
		}
	}
	radsq = near((pt.x-(mX+mV.a[0]))*(pt.x-(mX+mV.a[0]))+(pt.y-(mY+mV.a[1]))*(pt.y-(mY+mV.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) {
			mNumLaticeVectorsV -= 1 ;
		}
		else {
			mNumLaticeVectorsV += 1 ;
		}
	}
}

void	TLatticeMask::Modify(Point &pt) {
	Point	where,lastPoint ;
	
	GetMouse(&where) ;


	FPoint	cp ;
	FPoint	u,v ;
	cp.x = mX ;
	cp.y = mY ;
	float	cX = mX ;
	float	cY = mY ;
	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	
	mWindow->ImageToWindowPoint(cp) ;
	mWindow->ImageToWindowPoint(u) ;
	mWindow->ImageToWindowPoint(v) ;
	mX = cp.x ;
	mY = cp.y ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	
	mSpotRadius *= mWindow->GetMagnification() ;
	long	rad,radsq,tolsq = mSpotRadius*mSpotRadius ;
	
	lastPoint = where ;
	mIsSelected = false ;
	PenMode(kXOrMode);
	DrawBaseUnits() ;
	if (!InHandles(where)) {
		radsq = near((where.x-u.x)*(where.x-u.x)+(where.y-u.y)*(where.y-u.y)) ;
		if (radsq < tolsq) {
			while (Button()) {
				GetMouse(&where) ;
				if (where.x != lastPoint.x || where.y != lastPoint.y) {
					DrawBaseUnits() ;
					mU.a[0] += (where.x - lastPoint.x) ;
					mU.a[1] += (where.y - lastPoint.y) ;
					lastPoint = where ;
					DrawBaseUnits() ;
				}
			}
		}
		else {
			radsq = near((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
			if (radsq < tolsq) {
				while (Button()) {
					GetMouse(&where) ;
					if (where.x != lastPoint.x || where.y != lastPoint.y) {
						DrawBaseUnits() ;
						mV.a[0] += (where.x - lastPoint.x) ;
						mV.a[1] += (where.y - lastPoint.y) ;
						lastPoint = where ;
						DrawBaseUnits() ;
					}
				}
			}
		}
	}
	else {
		// Size the mask
		while (Button()) {
			GetMouse(&where) ;
			if (where.x != lastPoint.x || where.y != lastPoint.y) {
				DrawBaseUnits() ;
				mSpotRadius = sqrt((where.x-mX)*(where.x-mX)+(where.y-mY)*(where.y-mY)) ;
				lastPoint = where ;
				DrawBaseUnits() ;
			}
		}
	}

	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	mWindow->WindowToImagePoint(u) ;
	mWindow->WindowToImagePoint(v) ;
	mX = cX ;
	mY = cY ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	
	mSpotRadius /= mWindow->GetMagnification() ;

	PenNormal() ;
	mIsSelected = true ;

}

bool	TLatticeMask::InMask(Point &pt) {
	long	radsq,tolsq = (mSpotRadius+3)*(mSpotRadius+3) ;
	
	if (InHandles(pt))
		return true ;
		
	// Check to see if we are close to one of the lines
	float	x,y,x0,y0,a,b,dsq,d1,d2 ;
	
	x = pt.x ;
	y = pt.y ;
	if (mU.a[0] != 0.) {
		a = mU.a[1]/mU.a[0] ;
		b = (mY*(mX+mU.a[0]) - mX*(mY+mU.a[1]))/mU.a[0] ;
		x0 = (x+a*y-a*b)/(1+a*a) ;
		y0 = a*x0+b ;
		dsq = (x-x0)*(x-x0)+(y-y0)*(y-y0) ;
		d1 = fabs(x0 - mX - mU.a[0]) ;
		d2 = fabs(x0 - mX) ;
		if (dsq < 10. && d1 < fabs(mU.a[0])  && d2 < fabs(mU.a[0]))
			return true ;
	}
	else {
		// Line is parallel to the y-axis
		dsq = (x-mX)*(x-mX) ;
		if (dsq < 10.)
			return true ;
	}
		
	if (mV.a[0] != 0.) {
		a = mV.a[1]/mV.a[0] ;
		b = (mY*(mX+mV.a[0]) - mX*(mY+mV.a[1]))/mV.a[0] ;
		x0 = (x+a*y-a*b)/(1+a*a) ;
		y0 = a*x0+b ;
		dsq = (x-x0)*(x-x0)+(y-y0)*(y-y0) ;
		d1 = fabs(x0 - mX - mU.a[0]) ;
		d2 = fabs(x0 - mX) ;
		if (dsq < 10. && d1 < fabs(mV.a[0])  && d2 < fabs(mV.a[0]))
			return true ;
	}
	else {
		// Line is parallel to the y-axis
		dsq = (x-mX)*(x-mX) ;
		d1 = fabs(x0 - mX - mU.a[0]) ;
		d2 = fabs(x0 - mX) ;
		if (dsq < 10.)
			return true ;
	}
	   	
	radsq = near((pt.x-mX)*(pt.x-mX)+(pt.y-mY)*(pt.y-mY)) ;
	if (radsq < tolsq) 
		return true ;
	radsq = near((pt.x-(mX+mU.a[0]))*(pt.x-(mX+mU.a[0]))+(pt.y-(mY+mU.a[1]))*(pt.y-(mY+mU.a[1]))) ;
	if (radsq < tolsq) 
		return true ;
	radsq = near((pt.x-(mX+mV.a[0]))*(pt.x-(mX+mV.a[0]))+(pt.y-(mY+mV.a[1]))*(pt.y-(mY+mV.a[1]))) ;
	if (radsq < tolsq) 
		return true ;
	return false ;
}

void	TLatticeMask::Apply(TImage *image) {
	TArray	*dataArray = image->GetDataArray() ;
	bool	dataIsComplex = dataArray->IsComplex() ;
	
	// Create the mask
	TArray	*mask = new TArray(dataArray->GetWidth(),dataArray->GetHeight()) ;
	
	// Draw All the lattice masks that fit on the image
	// Compute the max h & k index that can possibly fit on the image
	float	ux = mU.a[0]/mNumLaticeVectorsU ;
	float	uy = mU.a[1]/mNumLaticeVectorsU ;
	float	vx = mV.a[0]/mNumLaticeVectorsV ;
	float	vy = mV.a[1]/mNumLaticeVectorsV ;
	
	float	u = sqrt(ux*ux+uy*uy) ;
	float	v = sqrt(vx*vx+vy*vy) ;
	
	float	csalpha = (ux*vx+uy*vy)/(u*v) ;
	float	sialpha = sqrt(1.-csalpha*csalpha) ;
	short	maxH = near(sqrt(mX*mX+mY*mY)/(u*sialpha)) ;
	short	maxK = near(sqrt(mX*mX+mY*mY)/(v*sialpha)) ;
	
	short	i,j,h,k,ix,iy,maxX,maxY,radius = near(mSpotRadius+mEdgeWidth) ;
	maxX 	= 2*mX - radius ;
	maxY 	= 2*mY - radius ;


	float	rsq,ysq,r0sq,r1sq,arg,r ;
	r0sq = mSpotRadius*mSpotRadius ;
	r1sq = radius*radius ;
	if (mEdgeWidth > 0) arg = asin(1.0)/mEdgeWidth ;
	switch (mMaskType) {
		case kTransparentMask:
			// Traverse the mask and put down either a sharp circle mask if mEdgeWidth = 0
			// or a cos-square edge if mEdgeWidth > 0
			*mask = 0. ;
			for(h = -maxH; h < maxH; h++) 
				for(k = -maxK; k < maxK; k++) {
					ix = near(mX + h*ux + k*vx) ;
					iy = near(mY + h*uy + k*vy) ;
					// Check to see if mask is within the image
					if (ix > radius && ix < maxX && iy > radius && iy < maxY) {
						// Create the mask at this spot
						if (dataIsComplex) {
							for(j=iy-radius; j <= iy+radius; j++) {
								ysq = (j-iy)*(j-iy) ;
								for(i=ix-radius; i <= ix+radius; i++) {
									rsq = (i-ix)*(i-ix) + ysq ;
									if (rsq <= r0sq) {
										*mask->GetDataHK(i-mX,j-mY) = 1. ;
									}
									else if (mEdgeWidth > 0 && rsq <= r1sq) {
										r = sqrt(rsq) - mSpotRadius ;
										*mask->GetDataHK(i-mX,j-mY) = cos(arg*r) ;
									}
								}
							}
						}
						else {
							for(j=iy-radius; j <= iy+radius; j++) {
								ysq = (j-iy)*(j-iy) ;
								for(i=ix-radius; i <= ix+radius; i++) {
									rsq = (i-ix)*(i-ix) + ysq ;
									if (rsq <= r0sq) {
										*mask->GetData(i,j) = 1. ;
									}
									else if (mEdgeWidth > 0 && rsq <= r1sq) {
										r = sqrt(rsq) - mSpotRadius ;
										*mask->GetData(i,j) = cos(arg*r) ;
									}
								}
							}
						}
					}
				}	
			break ;
		case kOpaqueMask:
			*mask = 1. ;
			for(h = -maxH; h < maxH; h++) 
				for(k = -maxK; k < maxK; k++) {
					ix = near(mX + h*ux + k*vx) ;
					iy = near(mY + h*uy + k*vy) ;
					// Check to see if mask is within the image
					if (ix > radius && ix < maxX && iy > radius && iy < maxY) {
						// Create the mask at this spot
						if (dataIsComplex) {
							for(j=iy-radius; j <= iy+radius; j++) {
								ysq = (j-iy)*(j-iy) ;
								for(i=ix-radius; i <= ix+radius; i++) {
									rsq = (i-ix)*(i-ix) + ysq ;
									if (rsq <= r0sq) {
										*mask->GetDataHK(i-mX,j-mY) = 0. ;
									}
									else if (mEdgeWidth > 0 && rsq <= r1sq) {
										r = sqrt(rsq) - mSpotRadius ;
										*mask->GetDataHK(i-mX,j-mY) = 1-cos(arg*r) ;
									}
								}
							}
						}
						else {
							for(j=iy-radius; j <= iy+radius; j++) {
								ysq = (j-iy)*(j-iy) ;
								for(i=ix-radius; i <= ix+radius; i++) {
									rsq = (i-ix)*(i-ix) + ysq ;
									if (rsq <= r0sq) {
										*mask->GetData(i,j) = 0. ;
									}
									else if (mEdgeWidth > 0 && rsq <= r1sq) {
										r = sqrt(rsq) - mSpotRadius ;
										*mask->GetData(i,j) = 1-cos(arg*r) ;
									}
								}
							}
						}
					}
				}	
			break ;
	}
	*dataArray *= *mask ;
	delete mask ;
}


TWedgeMask::TWedgeMask(TImageWindow *window) : TMask(window) {
	mX = window->GetImage()->Width()/2 ;
	mY = window->GetImage()->Height()/2 ;
	mU.a[0] = mX/2 ;
	mU.a[1] = -mY/4 ;
	mV.a[0] = mX/4 ;
	mV.a[1] = -mY/2 ;
	char	objectInfo[256] ;
	GetObjectInfo(objectInfo) ;
	::WriteObjectInfo(objectInfo) ;
}

TWedgeMask::~TWedgeMask() {
}

void	TWedgeMask::EditMaskparameters() {
	MaskRecord maskRecord ;
	maskRecord.numLatticeVectorsA = kUnusedMaskParameter ;
	maskRecord.numLatticeVectorsB = kUnusedMaskParameter ;
	maskRecord.maskWidth = kUnusedMaskParameter ;
	maskRecord.edgeWidth = kUnusedMaskParameter ;
	maskRecord.maskIsOpaque = mMaskType == kTransparentMask ? 0 : 1 ;
	
	TEditMaskDialog *theDialog = new TEditMaskDialog(&maskRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&maskRecord) ;
		mMaskType = maskRecord.maskIsOpaque ? kOpaqueMask : kTransparentMask ;
	}
	delete theDialog ;
}

void	TWedgeMask::GetObjectInfo(char *text)  {
	float	ga1	= 90.0/asin(1.0)*atan2(-mU.a[1],mU.a[0]) ;
	float	ga2	= 90.0/asin(1.0)*atan2(-mV.a[1],mV.a[0]) ;
	sprintf(text,"Wedge Mask: \nGa1=%.2f Deg Ga2=%.2f Deg",ga1,ga2) ;
}

void	TWedgeMask::Draw() {
	
	SetFgColor(gRedPixel) ;	

	// Find the intercepts on the line y = 0 and y = Height
	float	a,b,x0,xH,y0,yH,width,height ;
	width 	= 2*mX ;
	height 	= 2*mY ;
	if (mU.a[0] != 0.) {
		a = mU.a[1]/mU.a[0] ;
		b = (mY*(mX+mU.a[0]) - mX*(mY+mU.a[1]))/mU.a[0] ;
		if (a != 0.) {
			x0 = -b/a ;
			if (x0 < 0.) {
				x0 = 0. ;
				y0 = b ;
			}
			else if (x0 >= width) {
				x0 = width-1 ;
				y0 = a*x0+b ;
			}
			else {
				y0 = 0. ;
			}
			xH = (height-1-b)/a ;
			if (xH < 0) {
				xH = 0. ;
				yH = b ;
			}
			else if (xH >= width) {
				xH = width-1 ;
				yH = a*xH+b ;
			}
			else {
				yH = height-1 ;
			}
		}
		else {
			// Line is parallel to the x-axis
			x0 = 0. ;
			xH = width-1 ;
			y0 = b ;
			yH = b ;
		}
	}
	else {
		// Line is parallel to the y-axis
		y0 = 0. ;
		yH = height-1 ;
		x0 = xH = mX ;
	}

	MoveTo(near(x0),near(y0)) ;
	LineTo(near(xH),near(yH)) ;
		
	if (mV.a[0] != 0.) {
		a = mV.a[1]/mV.a[0] ;
		b = (mY*(mX+mV.a[0]) - mX*(mY+mV.a[1]))/mV.a[0] ;
		if (a != 0.) {
			x0 = -b/a ;
			if (x0 < 0.) {
				x0 = 0. ;
				y0 = b ;
			}
			else if (x0 >= width) {
				x0 = width-1 ;
				y0 = a*x0+b ;
			}
			else {
				y0 = 0. ;
			}
			xH = (height-1-b)/a ;
			if (xH < 0) {
				xH = 0. ;
				yH = b ;
			}
			else if (xH >= width) {
				xH = width-1 ;
				yH = a*xH+b ;
			}
			else {
				yH = height-1 ;
			}
		}
		else {
			// Line is parallel to the x-axis
			x0 = 0. ;
			xH = width-1 ;
			y0 = b ;
			yH = b ;
		}
	}
	else {
		// Line is parallel to the y-axis
		y0 = 0. ;
		yH = height-1 ;
		x0 = xH = mX ;
	}

	MoveTo(near(x0),near(y0)) ;
	LineTo(near(xH),near(yH)) ;
	
	
	SetFgColor(gBlackPixel) ;
	DrawBaseUnits() ;
}

void	TWedgeMask::DrawBaseUnits() {

	SetFgColor(gRedPixel) ;	
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mV.a[0]),near(mY+mV.a[1])) ;


	// Draw arrows
	double	k,b,x0,y0,x1,x2,y1,y2,A,B,L,d,a1,a2,a3,a4,xw1,yw1,xw2,yw2,w ;
	w = 4 ;
	d = 8 ;
	x1 = mX ;
	y1 = mY ;

	x2 = mX + mU.a[0] ;
	y2 = mY + mU.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (mU.a[0] != 0.) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;
		
	x2 = mX + mV.a[0] ;
	y2 = mY + mV.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (mV.a[0] != 0.) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;



	SetFgColor(gBlackPixel) ;
	if (mIsSelected)
		DrawHandles() ;
}

void	TWedgeMask::DrawHandles() {
	

	SetFgColor(gGreenPixel) ;	
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	FrameRect(&selectionRect) ;
	SetFgColor(gBlackPixel) ;
}

bool	TWedgeMask::InHandles(Point &pt) {
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	InsetRect(&selectionRect,-3,-3) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	InsetRect(&selectionRect,-3,-3) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	return false ;
}

void	TWedgeMask::Modify(Point &pt) {
	Point	where,lastPoint ;
	long	radsq,tolsq = 16. ;
	
	GetMouse(&where) ;

	FPoint	cp ;
	FPoint	u,v ;
	cp.x = mX ;
	cp.y = mY ;
	float	cX = mX ;
	float	cY = mY ;
	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	
	mWindow->ImageToWindowPoint(cp) ;
	mWindow->ImageToWindowPoint(u) ;
	mWindow->ImageToWindowPoint(v) ;
	mX = cp.x ;
	mY = cp.y ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	
	lastPoint = where ;
	mIsSelected = false ;
	PenMode(kXOrMode);
	DrawBaseUnits() ;
	radsq = near((where.x-u.x)*(where.x-u.x)+(where.y-u.y)*(where.y-u.y)) ;
	if (radsq < tolsq) {
		while (Button()) {
			GetMouse(&where) ;
			if (where.x != lastPoint.x || where.y != lastPoint.y) {
				DrawBaseUnits() ;
				mU.a[0] += (where.x - lastPoint.x) ;
				mU.a[1] += (where.y - lastPoint.y) ;
				lastPoint = where ;
				DrawBaseUnits() ;
			}
		}
	}
	else {
		radsq = near((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
		if (radsq < tolsq) {
			while (Button()) {
				GetMouse(&where) ;
				if (where.x != lastPoint.x || where.y != lastPoint.y) {
					DrawBaseUnits() ;
					mV.a[0] += (where.x - lastPoint.x) ;
					mV.a[1] += (where.y - lastPoint.y) ;
					lastPoint = where ;
					DrawBaseUnits() ;
				}
			}
		}
	}

	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	mWindow->WindowToImagePoint(u) ;
	mWindow->WindowToImagePoint(v) ;
	mX = cX ;
	mY = cY ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	PenNormal() ;
	mIsSelected = true ;

}

bool	TWedgeMask::InMask(Point &pt) {

	if (InHandles(pt))
		return true ;
	// Check to see if we are close to one of the lines
	float	x,y,x0,y0,a,b,dsq ;
	
	x = pt.x ;
	y = pt.y ;
	if (mU.a[0] != 0.) {
		a = mU.a[1]/mU.a[0] ;
		b = (mY*(mX+mU.a[0]) - mX*(mY+mU.a[1]))/mU.a[0] ;
		x0 = (x+a*y-a*b)/(1+a*a) ;
		y0 = a*x0+b ;
		dsq = (x-x0)*(x-x0)+(y-y0)*(y-y0) ;
	}
	else {
		// Line is parallel to the y-axis
		dsq = (x-mX)*(x-mX) ;
	}
	if (dsq < 10.)
		return true ;
		
	if (mV.a[0] != 0.) {
		a = mV.a[1]/mV.a[0] ;
		b = (mY*(mX+mV.a[0]) - mX*(mY+mV.a[1]))/mV.a[0] ;
		x0 = (x+a*y-a*b)/(1+a*a) ;
		y0 = a*x0+b ;
		dsq = (x-x0)*(x-x0)+(y-y0)*(y-y0) ;
	}
	else {
		// Line is parallel to the y-axis
		dsq = (x-mX)*(x-mX) ;
	}
	if (dsq < 10.)
		return true ;
	   
	return false ;
}

void	TWedgeMask::Apply(TImage *image) {
	TArray	*dataArray = image->GetDataArray() ;
	bool	dataIsComplex = dataArray->IsComplex() ;
	
	// Create the mask
	short	width = dataArray->GetWidth() ;
	short	height = dataArray->GetHeight() ;
	TArray	*mask = new TArray(width,height) ;

	float	ux = mU.a[0] ;
	float	uy = mU.a[1] ;
	float	vx = mV.a[0] ;
	float	vy = mV.a[1] ;
	
	float	u = sqrt(ux*ux+uy*uy) ;
	float	v = sqrt(vx*vx+vy*vy) ;
	
	float	csalpha = (ux*vx+uy*vy)/(u*v) ;
	float	alpha = acos(csalpha) ;

	float	ysq,csBeta1,csBeta2,r,beta1,beta2 ;
	short	i,j,ix,iy ;
	switch (mMaskType) {
		case kTransparentMask:
			// Traverse the mask and put down either a sharp circle mask if mEdgeWidth = 0
			// or a cos-square edge if mEdgeWidth > 0
			*mask = 0. ;
			// Create the mask at this spot
			if (dataIsComplex) {
				for(j=0; j < height; j++) {
					iy = j - mY ;
					ysq = iy*iy ;
					for(i=0; i < width; i++) {
						ix = i - mX ;
						r = sqrt(ysq+ix*ix) ;
						csBeta1 = (ix*ux+iy*uy)/(r*u) ;
						csBeta2 = (ix*vx+iy*vy)/(r*v) ;
						beta1 = acos(csBeta1) ;
						beta2 = acos(csBeta2) ;
						if (beta1 <= alpha && beta2 <= alpha) {
							*mask->GetDataHK(i-mX,j-mY) = 1. ;
							*mask->GetDataHK(-i+mX,-j+mY) = 1. ;
						}
					}
				}
			}
			else {
				for(j=0; j < height; j++) {
					iy = j - mY ;
					ysq = iy*iy ;
					for(i=0; i < width; i++) {
						ix = i - mX ;
						r = sqrt(ysq+ix*ix) ;
						csBeta1 = (ix*ux+iy*uy)/(r*u) ;
						csBeta2 = (ix*vx+iy*vy)/(r*v) ;
						beta1 = acos(csBeta1) ;
						beta2 = acos(csBeta2) ;
						if (beta1 <= alpha && beta2 <= alpha) {
							*mask->GetData(i,j) = 1. ;
							*mask->GetData(width-i,height-j) = 1. ;
						}
					}
				}
			}
			break ;
		case kOpaqueMask:
			*mask = 1. ;
			// Create the mask at this spot
			if (dataIsComplex) {
				for(j=0; j < height; j++) {
					iy = j - mY ;
					ysq = iy*iy ;
					for(i=0; i < width; i++) {
						ix = i - mX ;
						r = sqrt(ysq+ix*ix) ;
						csBeta1 = (ix*ux+iy*uy)/(r*u) ;
						csBeta2 = (ix*vx+iy*vy)/(r*v) ;
						beta1 = acos(csBeta1) ;
						beta2 = acos(csBeta2) ;
						if (beta1 <= alpha && beta2 <= alpha) {
							*mask->GetDataHK(i-mX,j-mY) = 0. ;
							*mask->GetDataHK(-i+mX,-j+mY) = 0. ;
						}
					}
				}
			}
			else {
				for(j=0; j < height; j++) {
					iy = j - mY ;
					ysq = iy*iy ;
					for(i=0; i < width; i++) {
						ix = i - mX ;
						r = sqrt(ysq+ix*ix) ;
						csBeta1 = (ix*ux+iy*uy)/(r*u) ;
						csBeta2 = (ix*vx+iy*vy)/(r*v) ;
						beta1 = acos(csBeta1) ;
						beta2 = acos(csBeta2) ;
						if (beta1 <= alpha && beta2 <= alpha) {
							*mask->GetData(i,j) = 0. ;
							*mask->GetData(width-i,height-j) = 0. ;
						}
					}
				}
			}
			break ;
	}
	*dataArray *= *mask ;
	delete mask ;
}


TLineMask::TLineMask(TImageWindow *window) : TMask(window) {
	mX = window->GetImage()->Width()/2 ;
	mY = window->GetImage()->Height()/2 ;
	mLineDistance = mX/5 ;	
	mU.a[0] = mX/2 ;
	mU.a[1] = -mY/4 ;
	float	k = mU.a[1]/mU.a[0] ;
	mV.a[0] = k*mLineDistance/sqrt(1+k*k) ;
	mV.a[1] = -mLineDistance/sqrt(1+k*k) ;
	
	mNumLaticeVectorsV = 1 ;
	mLineWidth = 1 ;
	char	objectInfo[256] ;
	GetObjectInfo(objectInfo) ;
	::WriteObjectInfo(objectInfo) ;
}

TLineMask::~TLineMask() {
}

void	TLineMask::EditMaskparameters() {
	MaskRecord maskRecord ;
	maskRecord.numLatticeVectorsA = mNumLaticeVectorsV ;
	maskRecord.numLatticeVectorsB = kUnusedMaskParameter ;
	maskRecord.maskWidth = mLineWidth ;
	maskRecord.edgeWidth = kUnusedMaskParameter ;
	maskRecord.maskIsOpaque = mMaskType == kTransparentMask ? 0 : 1 ;
	
	TEditMaskDialog *theDialog = new TEditMaskDialog(&maskRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&maskRecord) ;
		mNumLaticeVectorsV = maskRecord.numLatticeVectorsA ;
		if (mNumLaticeVectorsV < 1) mNumLaticeVectorsV = 1 ;	
		mLineDistance = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1])/mNumLaticeVectorsV ;
		mLineWidth = maskRecord.maskWidth ;
		mMaskType = maskRecord.maskIsOpaque ? kOpaqueMask : kTransparentMask ;
	}
	delete theDialog ;
}

void	TLineMask::Draw() {
	
	SetFgColor(gRedPixel) ;	

	// Draw all the lines that fit within the image
	short	width = 2*mX ;
	short	height = 2*mY ;
	float	k,b ;
	float	imageDiagonal = sqrt(mX*mX+mY*mY) ;
	short	n, nMax = near(imageDiagonal/mLineDistance + 1) ;
	short	x1,y1,x2,y2 ; 
	
	PenSize(mLineWidth,mLineWidth) ;
	if (mU.a[0] != 0.) {
		k = mU.a[1]/mU.a[0] ;
		if (k != 0.) {
			for(n = -nMax; n <= nMax; n++) {
				b = (mY+n*mV.a[1]/mNumLaticeVectorsV) - k*(mX+n*mV.a[0]/mNumLaticeVectorsV) ;
				// Draw the line y = kx + b
				x1 = near(-b/k) ;
				if (x1 >= 0 && x1 < width -1) {
					y1 = 0 ;
					x2 = (height-1 - b)/k ;
					if (x2 < 0) x2 = 0 ;
					if (x2 > width-1) x2 = width-1 ;
					y2 = k*x2+b ;
				}
				else if (x1 < 0) {
					x1 = 0 ;
					y1 = b ;
					x2 = (height-1 - b)/k ;
					if (x2 < 0) x2 = 0 ;
					if (x2 > width-1) x2 = width-1 ;
					y2 = k*x2+b ;					 
				}
				else {
					x1 = width-1 ;
					y1 = k*x1 + b ;
					x2 = (height-1 - b)/k ;
					if (x2 < 0) x2 = 0 ;
					if (x2 > width-1) x2 = width-1 ;
					y2 = k*x2+b ;					 
				}
				MoveTo(x1,y1) ;
				LineTo(x2,y2) ;
			}
		}
		else {
			for(n = -nMax; n <= nMax; n++) {
				x1 = 0 ;
				x2 = width-1 ;
				y1 = y2 = mY+n*mV.a[1]/mNumLaticeVectorsV ;
				if (y1 >= 0 && y1 < height) {
					MoveTo(x1,y1) ;
					LineTo(x2,y2) ;
				}					
			}
		}
	}
	else {
		for(n = -nMax; n <= nMax; n++) {
			y1 = 0 ;
			y2 = height-1 ;
			x1 = x2 = mX+n*mV.a[0]/mNumLaticeVectorsV ;
			if (x1 >= 0 && x1 < width) {
				MoveTo(x1,y1) ;
				LineTo(x2,y2) ;
			}					
		}
	}
	PenSize(1,1) ;

	SetFgColor(gBlackPixel) ;
	DrawBaseUnits() ;	
}

void	TLineMask::GetObjectInfo(char *text)  {
	CalibrationUnit calibrationUnit = mWindow->GetImage()->GetCalibrationUnit() ;
	float	calibrationConstant = mWindow->GetImage()->GetCalibration() ;
	float	v = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1])*calibrationConstant/mNumLaticeVectorsV ;
	float	ga	= 90.0/asin(1.0)*atan2(-mU.a[1],mU.a[0]) ;
	switch(calibrationUnit) {
		case kPixel:
			sprintf(text,"Line Mask: [Pixels]\nSpacing=%.1f Ga=%.1f Deg",v,ga) ;
			break ;
		case kInversePixel:
			sprintf(text,"Line Mask: [1/Pixels]\nSpacing=1/%.1f Ga=%.1f Deg",1/v,ga) ;
			break ;
		case kAngstrom:
			sprintf(text,"Line Mask: [Angstrom]\nSpacing=%.2f Ga=%.1f Deg",v,ga) ;
			break ;
		case kInverseAngstrom:
			sprintf(text,"Line Mask: [1/Angstrom]\nSpacing=1/%.2f Ga=%.1f Deg",1/v,ga) ;
			break ;
		case kNanometer:
			sprintf(text,"Line Mask: [nm]\nSpacing%.2f Ga=%.1f Deg",v,ga) ;
			break ;
		case kInverseNanometer:
			sprintf(text,"Line Mask: [1/nm]\nSpacing=1/%.2f Ga=%.1f Deg",1/v,ga) ;
			break ;
	}
}

void	TLineMask::DrawBaseUnits() {
	SetFgColor(gRedPixel) ;	

	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX-mV.a[0]),near(mY-mV.a[1])) ;
		
	// Draw arrows
	double	k,b,x0,y0,x1,x2,y1,y2,A,B,L,d,a1,a2,a3,a4,xw1,yw1,xw2,yw2,w ;
	w = 4 ;
	d = 8 ;
	x1 = mX ;
	y1 = mY ;

	x2 = mX + mU.a[0] ;
	y2 = mY + mU.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (fabs(mU.a[0]) >= 1./mX) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;
		
	x2 = mX + mV.a[0] ;
	y2 = mY + mV.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (fabs(mV.a[0]) >= 1./mX) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;

	x2 = mX - mV.a[0] ;
	y2 = mY - mV.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (fabs(mV.a[0]) >= 1./mX) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;

	
	SetFgColor(gBlackPixel) ;
	if (mIsSelected)
		DrawHandles() ;
}

void	TLineMask::DrawHandles() {
	SetFgColor(gGreenPixel) ;	
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mV.a[0]),near(mY-mV.a[1])) ;
	FrameRect(&selectionRect) ;
	SetFgColor(gBlackPixel) ;
	
}

bool	TLineMask::InHandles(Point &pt) {
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	InsetRect(&selectionRect,-3,-3) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	InsetRect(&selectionRect,-3,-3) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mV.a[0]),near(mY-mV.a[1])) ;
	InsetRect(&selectionRect,-3,-3) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	return false ;
}

void	TLineMask::Edit(Point &pt) {
	long	radsq,tolsq = 16. ;
	
	radsq = near((pt.x-(mX+mV.a[0]))*(pt.x-(mX+mV.a[0]))+(pt.y-(mY+mV.a[1]))*(pt.y-(mY+mV.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) 
			mNumLaticeVectorsV -= 1 ;
		else
			mNumLaticeVectorsV += 1 ;
	}
	
	radsq = near((pt.x-(mX-mV.a[0]))*(pt.x-(mX-mV.a[0]))+(pt.y-(mY-mV.a[1]))*(pt.y-(mY-mV.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) 
			mNumLaticeVectorsV -= 1 ;
		else
			mNumLaticeVectorsV += 1 ;
	}
		
	radsq = near((pt.x-(mX+mU.a[0]))*(pt.x-(mX+mU.a[0]))+(pt.y-(mY+mU.a[1]))*(pt.y-(mY+mU.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) {
			mLineWidth -= 1 ;
			if (mLineWidth < 1) mLineWidth = 1 ;
		}
		else
			mLineWidth += 1 ;
	}
	
	if (mNumLaticeVectorsV < 1) mNumLaticeVectorsV = 1 ;
	
	mLineDistance = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1])/mNumLaticeVectorsV ;
}

void	TLineMask::Modify(Point &pt) {

	Point	where,lastPoint ;
	long	dx,dy,radsq,tolsq = 10. ;
	float	k,vx,vy ;
	
	GetMouse(&where) ;

	FPoint	cp ;
	FPoint	u,v ;
	cp.x = mX ;
	cp.y = mY ;
	float	cX = mX ;
	float	cY = mY ;
	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	mWindow->ImageToWindowPoint(cp) ;
	mWindow->ImageToWindowPoint(u) ;
	mWindow->ImageToWindowPoint(v) ;
	mX = cp.x ;
	mY = cp.y ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	
	lastPoint = where ;
	mIsSelected = false ;
	PenMode(kXOrMode);
	DrawBaseUnits() ;
	radsq = near((where.x-u.x)*(where.x-u.x)+(where.y-u.y)*(where.y-u.y)) ;
	if (radsq < tolsq) {
		while (Button()) {
			GetMouse(&where) ;
			if (where.x != lastPoint.x || where.y != lastPoint.y) {
				DrawBaseUnits() ;
				mU.a[0] += (where.x - lastPoint.x) ;
				mU.a[1] += (where.y - lastPoint.y) ;
				lastPoint = where ;
				// Set the new values
				if (mU.a[0] != 0.) {
					k = mU.a[1]/mU.a[0] ;
					if (fabs(k) < mY) {
						mV.a[0] = k*mLineDistance*mNumLaticeVectorsV/sqrt(1+k*k) ;
						mV.a[1] = -mLineDistance*mNumLaticeVectorsV/sqrt(1+k*k) ;	
					}
					else {
						mV.a[0] = mLineDistance*mNumLaticeVectorsV ;
						mV.a[1] = 0. ;	
					}		
				}
				else {
					mV.a[0] = mLineDistance*mNumLaticeVectorsV ;
					mV.a[1] = 0. ;
				}
				DrawBaseUnits() ;
			}
		}
	}
	else {
		radsq = near((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
		if (radsq < tolsq) {
			// Constrain the move
			vx = mV.a[0] ;
			vy = mV.a[1] ;
			 
			while (Button()) {
				GetMouse(&where) ;
				if (where.x != lastPoint.x || where.y != lastPoint.y) {
					DrawBaseUnits() ;
					dx = where.x - lastPoint.x ;
					dy = where.y - lastPoint.y ;
					if (vx != 0.) {
						k = vy/vx ;
						if (k != 0.) {
							if (fabs(k) < 1.) {
								mV.a[0] += dx ;
								mV.a[1] = k*mV.a[0] ;
							}
							else {
								mV.a[1] += dy ;
								mV.a[0] = mV.a[1]/k ;
							}
						}
						else {
							mV.a[0] += dx ;
						}
					}
					else {
						mV.a[1] += dy ;
					}	
					mLineDistance = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1])/mNumLaticeVectorsV ;
					lastPoint = where ;
					DrawBaseUnits() ;
				}
			}
		}
		else {
			radsq = near((where.x-(mX-mV.a[0]))*(where.x-(mX-mV.a[0]))+(where.y-(mY-mV.a[1]))*(where.y-(mY-mV.a[1]))) ;
			if (radsq < tolsq) {
				// Constrain the move
				vx = mV.a[0] ;
				vy = mV.a[1] ;
				while (Button()) {
					GetMouse(&where) ;
					if (where.x != lastPoint.x || where.y != lastPoint.y) {
						DrawBaseUnits() ;
						dx = where.x - lastPoint.x ;
						dy = where.y - lastPoint.y ;
						if (vx != 0.) {
							k = vy/vx ;
							if (k != 0.) {
								if (fabs(k) < 1.) {
									// Adjust x first and then constrain y
									mV.a[0] -= dx ;
									mV.a[1] = k*mV.a[0] ;
								}
								else {
									mV.a[1] -= dy ;
									mV.a[0] = mV.a[1]/k ;
								}
							}
							else {
								// Parallel to x-axis
								mV.a[0] -= dx ;
							}
						}
						else {
							// Parallel to y-axis
							mV.a[1] -= dy ;
						}	
						mLineDistance = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1])/mNumLaticeVectorsV ;
						lastPoint = where ;
						DrawBaseUnits() ;
					}
				}
			}
		}
	}


	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	mWindow->WindowToImagePoint(u) ;
	mWindow->WindowToImagePoint(v) ;
	mX = cX ;
	mY = cY ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	PenNormal() ;
	mIsSelected = true ;

}

bool	TLineMask::InMask(Point &pt) {

	if (InHandles(pt))
		return true ;

	float	k,b,x1,y1,x2,y2,d ;
	float	imageDiagonal = sqrt(mX*mX+mY*mY) ;
	short	n, nMax = near(imageDiagonal/mLineDistance + 1) ;
	float	vx,vy,kk,tolsq = (mLineWidth+3)*(mLineWidth+3) ;
	
	x1 = pt.x ;
	y1 = pt.y ;
	if (mU.a[0] != 0.) {
		k = mU.a[1]/mU.a[0] ;
		if (k != 0.) {
			kk = 1+k*k ;
			// Test the shortest distance to one of the lines
			for(n = -nMax; n <= nMax; n++) {
				vx = n*mV.a[0]/mNumLaticeVectorsV ;
				vy = n*mV.a[1]/mNumLaticeVectorsV ;
				b = (mY+vy) - k*(mX+vx) ;
				
				// Check distance to line y = kx + b
				// The closest point on the line is (x2,y2)
				x2 = (k*y1+x1-k*b)/kk ;
				y2 = k*x2+b ;
				d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) ;
				if (d <= tolsq) {
					return true ;
				}
			}
		}
		else {
			// Line parallel to x-axis
			// Test the shortest distance to one of the lines
			for(n = -nMax; n <= nMax; n++) {
				y2 = mY+n*mV.a[1]/mNumLaticeVectorsV ;
				d = (y2-y1)*(y2-y1) ;
				if (d <= tolsq) {
					return true ;
				}
			}
		}
	}
	else {
		// Line parallel to y-axis
			// Test the shortest distance to one of the lines
		for(n = -nMax; n <= nMax; n++) {
			x2 = mX+n*mV.a[0]/mNumLaticeVectorsV ;
			d = (x2-x1)*(x2-x1) ;
			if (d <= tolsq) {
				return true ;
			}
		}
	}


	return false ;
}

void	TLineMask::Apply(TImage *image) {
	TArray	*dataArray = image->GetDataArray() ;
	bool	dataIsComplex = dataArray->IsComplex() ;
	
	// Create the mask
	TArray	*mask = new TArray(dataArray->GetWidth(),dataArray->GetHeight()) ;
	
	
	short	width = 2*mX ;
	short	height = 2*mY ;
	float	k,b,x1,y1,x2,y2,d ;
	float	imageDiagonal = sqrt(mX*mX+mY*mY) ;
	short	i,j,n, nMax = near(imageDiagonal/mLineDistance + 1) ;
	float	vx,vy,kk,tolsq = (mLineWidth*mLineWidth/4.) ;
	

	
	switch (mMaskType) {
		case kTransparentMask:
			// Traverse the mask and put down either a sharp circle mask if mEdgeWidth = 0
			// or a cos-square edge if mEdgeWidth > 0
			*mask = 0. ;
			// Create the mask at this spot
			if (dataIsComplex) {
				if (mU.a[0] != 0.) {
					k = mU.a[1]/mU.a[0] ;
					if (k != 0.) {
						kk = 1+k*k ;
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									vx = n*mV.a[0]/mNumLaticeVectorsV ;
									vy = n*mV.a[1]/mNumLaticeVectorsV ;
									b = (mY+vy) - k*(mX+vx) ;
									
									// Check distance to line y = kx + b
									// The closest point on the line is (x2,y2)
									x2 = (k*y1+x1-k*b)/kk ;
									y2 = k*x2+b ;
									d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetDataHK(i-mX,j-mY) = 1. ;
										break ;
									}
								}
							}
						}
					}
					else {
						// Line parallel to x-axis
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									y2 = mY+n*mV.a[1]/mNumLaticeVectorsV ;
									d = (y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetDataHK(i-mX,j-mY) = 1. ;
										break ;
									}
								}
							}
						}
					}
				}
				else {
					// Line parallel to y-axis
					for(i=0;i<width; i++) {
						x1 = i ;
						// Test the shortest distance to one of the lines
						for(n = -nMax; n <= nMax; n++) {
							x2 = mX+n*mV.a[0]/mNumLaticeVectorsV ;
							d = (x2-x1)*(x2-x1) ;
							if (d <= tolsq) {
								for(j=0;j<height;j++) {
									y1 = j ;
									*mask->GetDataHK(i-mX,j-mY) = 1. ;
								}
								break ;
							}
						}
					}
				}
			}
			else {
				if (mU.a[0] != 0.) {
					k = mU.a[1]/mU.a[0] ;
					if (k != 0.) {
						kk = 1+k*k ;
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									vx = n*mV.a[0]/mNumLaticeVectorsV ;
									vy = n*mV.a[1]/mNumLaticeVectorsV ;
									b = (mY+vy) - k*(mX+vx) ;
									
									// Check distance to line y = kx + b
									// The closest point on the line is (x2,y2)
									x2 = (k*y1+x1-k*b)/kk ;
									y2 = k*x2+b ;
									d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetData(i,j) = 1. ;
										break ;
									}
								}
							}
						}
					}
					else {
						// Line parallel to x-axis
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									y2 = mY+n*mV.a[1]/mNumLaticeVectorsV ;
									d = (y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetData(i,j) = 1. ;
										break ;
									}
								}
							}
						}
					}
				}
				else {
					// Line parallel to y-axis
					for(i=0;i<width; i++) {
						x1 = i ;
						// Test the shortest distance to one of the lines
						for(n = -nMax; n <= nMax; n++) {
							x2 = mX+n*mV.a[0]/mNumLaticeVectorsV ;
							d = (x2-x1)*(x2-x1) ;
							if (d <= tolsq) {
								for(j=0;j<height;j++) {
									y1 = j ;
									*mask->GetData(i,j) = 1. ;
								}
								break ;
							}
						}
					}
				}
			}
			break ;
		case kOpaqueMask:
			*mask = 1. ;
			if (dataIsComplex) {
				if (mU.a[0] != 0.) {
					k = mU.a[1]/mU.a[0] ;
					if (k != 0.) {
						kk = 1+k*k ;
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									vx = n*mV.a[0]/mNumLaticeVectorsV ;
									vy = n*mV.a[1]/mNumLaticeVectorsV ;
									b = (mY+vy) - k*(mX+vx) ;
									
									// Check distance to line y = kx + b
									// The closest point on the line is (x2,y2)
									x2 = (k*y1+x1-k*b)/kk ;
									y2 = k*x2+b ;
									d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetDataHK(i-mX,j-mY) = 0. ;
										break ;
									}
								}
							}
						}
					}
					else {
						// Line parallel to x-axis
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									y2 = mY+n*mV.a[1]/mNumLaticeVectorsV ;
									d = (y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetDataHK(i-mX,j-mY) = 0. ;
										break ;
									}
								}
							}
						}
					}
				}
				else {
					// Line parallel to y-axis
					for(i=0;i<width; i++) {
						x1 = i ;
						// Test the shortest distance to one of the lines
						for(n = -nMax; n <= nMax; n++) {
							x2 = mX+n*mV.a[0]/mNumLaticeVectorsV ;
							d = (x2-x1)*(x2-x1) ;
							if (d <= tolsq) {
								for(j=0;j<height;j++) {
									y1 = j ;
									*mask->GetDataHK(i-mX,j-mY) = 0. ;
								}
								break ;
							}
						}
					}
				}
			}
			else {
				if (mU.a[0] != 0.) {
					k = mU.a[1]/mU.a[0] ;
					if (k != 0.) {
						kk = 1+k*k ;
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									vx = n*mV.a[0]/mNumLaticeVectorsV ;
									vy = n*mV.a[1]/mNumLaticeVectorsV ;
									b = (mY+vy) - k*(mX+vx) ;
									
									// Check distance to line y = kx + b
									// The closest point on the line is (x2,y2)
									x2 = (k*y1+x1-k*b)/kk ;
									y2 = k*x2+b ;
									d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetData(i,j) = 0. ;
										break ;
									}
								}
							}
						}
					}
					else {
						// Line parallel to x-axis
						for(j=0;j<height;j++) {
							y1 = j ;
							for(i=0;i<width; i++) {
								x1 = i ;
								// Test the shortest distance to one of the lines
								for(n = -nMax; n <= nMax; n++) {
									y2 = mY+n*mV.a[1]/mNumLaticeVectorsV ;
									d = (y2-y1)*(y2-y1) ;
									if (d <= tolsq) {
										*mask->GetData(i,j) = 0. ;
										break ;
									}
								}
							}
						}
					}
				}
				else {
					// Line parallel to y-axis
					for(i=0;i<width; i++) {
						x1 = i ;
						// Test the shortest distance to one of the lines
						for(n = -nMax; n <= nMax; n++) {
							x2 = mX+n*mV.a[0]/mNumLaticeVectorsV ;
							d = (x2-x1)*(x2-x1) ;
							if (d <= tolsq) {
								for(j=0;j<height;j++) {
									y1 = j ;
									*mask->GetData(i,j) = 0. ;
								}
								break ;
							}
						}
					}
				}
			}
			break ;
	}
	*dataArray *= *mask ;
	delete mask ;
}


TEditMaskDialog::TEditMaskDialog(MaskRecord *maskRecord) 
	: TDialog(kDialogPosX,kDialogPosY, 285, 175,"Mask Parameters") {
	
	char	text[256] ;
	TStaticText	*theText ;
	TFilledRect	*filledRect ;
	
	mMaskRecord = *maskRecord ;
		
	mHasSoftEdge = new TCheckBox(15,10,150,30,"Use soft edge of width") ;	
	if (mMaskRecord.edgeWidth == kUnusedMaskParameter) {
		mHasSoftEdge->SetState(false) ;
		mHasSoftEdge->Disable() ;
	}
	else {
		if (mMaskRecord.edgeWidth > 0) mHasSoftEdge->SetState(true) ;
	}
	AddObject(mHasSoftEdge) ;

	if (mMaskRecord.edgeWidth == kUnusedMaskParameter) {
		sprintf(text,"0") ;
		mTextEdgeWidth = new TEditText(180,10,210,30,"",text) ;
		mTextEdgeWidth->Disable() ;
	}
	else {
		sprintf(text,"%d",mMaskRecord.edgeWidth) ;
		mTextEdgeWidth = new TEditText(180,10,210,30,"",text) ;
	}	
	
	theText = new TStaticText(220,10,260,30,"Pixels") ;
	if (mMaskRecord.edgeWidth == kUnusedMaskParameter) theText->Disable() ;
	AddObject(theText) ;
	
	mIsOpaque = new TCheckBox(15,30,150,50,"Opaque mask") ;	
	if (mMaskRecord.maskIsOpaque == kUnusedMaskParameter) mIsOpaque->Disable() ;
	if (mMaskRecord.maskIsOpaque == 1) mIsOpaque->SetState(true) ;
	AddObject(mIsOpaque) ;

	filledRect = new TFilledRect(20,65,30,75,gRedPixel) ;
	AddObject(filledRect) ;
	
	theText = new TStaticText(40,60,170,80,"Number of lattice vectors") ;
	if (mMaskRecord.numLatticeVectorsA == kUnusedMaskParameter) theText->Disable() ;
	AddObject(theText) ;

	if (mMaskRecord.numLatticeVectorsA == kUnusedMaskParameter) {
		sprintf(text,"1") ;
		mTextNumLatticeVectorsA = new TEditText(180,60,210,80,"",text) ;
		mTextNumLatticeVectorsA->Disable() ;
	}
	else {
		sprintf(text,"%d",mMaskRecord.numLatticeVectorsA) ;
		mTextNumLatticeVectorsA = new TEditText(180,60,210,80,"",text) ;
	}	


	filledRect = new TFilledRect(20,85,30,95,gBluePixel) ;
	AddObject(filledRect) ;
	
	theText = new TStaticText(40,80,170,100,"Number of lattice vectors") ;
	if (mMaskRecord.numLatticeVectorsB == kUnusedMaskParameter) theText->Disable() ;
	AddObject(theText) ;

	if (mMaskRecord.numLatticeVectorsB == kUnusedMaskParameter) {
		sprintf(text,"1") ;
		mTextNumLatticeVectorsB = new TEditText(180,80,210,100,"",text) ;
		mTextNumLatticeVectorsB->Disable() ;
	}
	else {
		sprintf(text,"%d",mMaskRecord.numLatticeVectorsB) ;
		mTextNumLatticeVectorsB = new TEditText(180,80,210,100,"",text) ;
	}	


	theText = new TStaticText(80,110,150,130,"Mask width") ;	
	if (mMaskRecord.maskWidth == kUnusedMaskParameter) theText->Disable() ;
	AddObject(theText) ;

	if (mMaskRecord.maskWidth == kUnusedMaskParameter) {
		sprintf(text,"0") ;
		mTextMaskWidth = new TEditText(180,110,210,130,"",text) ;
		mTextMaskWidth->Disable() ;
	}
	else {
		sprintf(text,"%d",mMaskRecord.maskWidth) ;
		mTextMaskWidth = new TEditText(180,110,210,130,"",text) ;
	}	
	
	theText = new TStaticText(220,110,260,130,"Pixels") ;
	if (mMaskRecord.maskWidth == kUnusedMaskParameter) theText->Disable() ;
	AddObject(theText) ;
	
		
	AddTextEditObject(mTextMaskWidth) ;
	AddTextEditObject(mTextNumLatticeVectorsB) ;
	AddTextEditObject(mTextNumLatticeVectorsA) ;
	AddTextEditObject(mTextEdgeWidth) ;
	
	mDefaultButton = mOKButton = new TButton(200,140,270,160,"OK") ;
	AddObject(mOKButton) ;
	
	mCancelButton = new TButton(110,140,180,160,"Cancel") ;
	AddObject(mCancelButton) ;
	
}

void	TEditMaskDialog::DoKeyDownInObject(TObject *obj)  {
		char 	text[256] ;
		short	width ;
		if (obj == mTextEdgeWidth && mTextEdgeWidth->IsEnabled()) {
			mTextEdgeWidth->GetEditText(text) ;
			width = atoi(text) ;
			if (width > 0) 
				mHasSoftEdge->SetState(true) ;
			else
				mHasSoftEdge->SetState(false) ;
		}

}

void	TEditMaskDialog::DoMouseDownInObject(TObject *obj,Point *pt) {
	if (pt != nil)
		if (!obj->DoMouseDown(pt)) return ;
	if (obj == mOKButton) {
		// Get the appropriate data
		char text[256] ;
		if (mTextMaskWidth->IsEnabled()) {
			mTextMaskWidth->GetEditText(text) ;
			mMaskRecord.maskWidth = atoi(text) ;
		}
		if (mTextEdgeWidth->IsEnabled()) {
			if (mHasSoftEdge->GetState()) {
				mTextEdgeWidth->GetEditText(text) ;
				mMaskRecord.edgeWidth = atoi(text) ;
			}
			else {
				mMaskRecord.edgeWidth = 0 ;
			}
		}
		if (mTextNumLatticeVectorsA->IsEnabled()) {
			mTextNumLatticeVectorsA->GetEditText(text) ;
			mMaskRecord.numLatticeVectorsA = atoi(text) ;
		}
		if (mTextNumLatticeVectorsB->IsEnabled()) {
			mTextNumLatticeVectorsB->GetEditText(text) ;
			mMaskRecord.numLatticeVectorsB = atoi(text) ;
		}
		mMaskRecord.maskIsOpaque = mIsOpaque->GetState() ;
		mDone = true ;
		mDialogReturnCode = kDialogOK ;
	}
	else if (obj == mCancelButton) {
		mDone = true ;
		mDialogReturnCode = kDialogCancel ;
	}
}


TReciprocalVectors::TReciprocalVectors(TImageWindow *window)  : TMask(window) {
	mX = window->GetImage()->Width()/2 ;
	mY = window->GetImage()->Height()/2 ;
	mU.a[0] = mX/2 ;
	mU.a[1] = 0 ;
	mV.a[0] = 0 ;
	mV.a[1] = -mY/2 ;
	mSpotRadius = 5 ;
	mNumLaticeVectorsU = 1 ;
	mNumLaticeVectorsV = 1 ;
	char	objectInfo[256] ;
	GetObjectInfo(objectInfo) ;
	::WriteObjectInfo(objectInfo) ;
}

TReciprocalVectors::~TReciprocalVectors() {
}

void	TReciprocalVectors::EditMaskparameters() {
	MaskRecord maskRecord ;
	maskRecord.numLatticeVectorsA = mNumLaticeVectorsU ;
	maskRecord.numLatticeVectorsB = mNumLaticeVectorsV ;
	maskRecord.maskWidth = kUnusedMaskParameter ;
	maskRecord.edgeWidth = kUnusedMaskParameter ;
	maskRecord.maskIsOpaque = kUnusedMaskParameter ;
	
	TEditMaskDialog *theDialog = new TEditMaskDialog(&maskRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&maskRecord) ;
		mNumLaticeVectorsU = maskRecord.numLatticeVectorsA ;
		mNumLaticeVectorsV = maskRecord.numLatticeVectorsB ;
	}
	delete theDialog ;
}

TLattice	*TReciprocalVectors::GetLattice() {
	FPoint	u,v,o ;
	o.x = mX ;
	o.y = mY ;
	u.x = mU.a[0]/mNumLaticeVectorsU ;
	u.y = mU.a[1]/mNumLaticeVectorsU ;
	v.x = mV.a[0]/mNumLaticeVectorsV ;
	v.y = mV.a[1]/mNumLaticeVectorsV ;
	TLattice *theLattice = new TLattice(o,u,v) ;
	return theLattice ;	
}

void	TReciprocalVectors::SetLattice(TLattice *lattice) {
	FPoint	u,v,o ;
	o = lattice->GetOrigin() ;
	u = lattice->GetU() ;
	v = lattice->GetV() ;
	mX = o.x ;
	mY = o.y ;
	mU.a[0] = u.x*mNumLaticeVectorsU ;
	mU.a[1] = u.y*mNumLaticeVectorsU ;
	mV.a[0] = v.x*mNumLaticeVectorsV ;
	mV.a[1] = v.y*mNumLaticeVectorsV ;
}

void	TReciprocalVectors::Draw() {

	SetFgColor(gRedPixel) ;	

	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	
	// Draw All the lattice masks that fit on the image
	// Compute the max h & k index that can possibly fit on the image
	float	ux = mU.a[0]/mNumLaticeVectorsU ;
	float	uy = mU.a[1]/mNumLaticeVectorsU ;
	float	vx = mV.a[0]/mNumLaticeVectorsV ;
	float	vy = mV.a[1]/mNumLaticeVectorsV ;
	
	float	u = sqrt(ux*ux+uy*uy) ;
	float	v = sqrt(vx*vx+vy*vy) ;
	
	float	csalpha = (ux*vx+uy*vy)/(u*v) ;
	float	sialpha = sqrt(1.-csalpha*csalpha) ;
	short	maxH = near(sqrt(mX*mX+mY*mY)/(u*sialpha)) ;
	short	maxK = near(sqrt(mX*mX+mY*mY)/(v*sialpha)) ;
	
	short	h,k,ix,iy,maxX,maxY,radius = near(mSpotRadius) ;
	maxX 	= 2*mX - radius ;
	maxY 	= 2*mY - radius ;
	for(h = -maxH; h < maxH; h++) 
		for(k = -maxK; k < maxK; k++) {
			ix = near(mX + h*ux + k*vx) ;
			iy = near(mY + h*uy + k*vy) ;
			// Check to see if mask is within the image
			if (ix > radius && ix < maxX && iy > radius && iy < maxY) {
				// Draw the mask on this spot
				DrawCircle(ix,iy,radius) ;
			}
		}	
	SetFgColor(gBlackPixel) ;
	DrawBaseUnits() ;	
}

void	TReciprocalVectors::GetObjectInfo(char *text)  {
	CalibrationUnit calibrationUnit = mWindow->GetImage()->GetCalibrationUnit() ;
	float	calibrationConstant = mWindow->GetImage()->GetCalibration() ;
	float	u = sqrt(mU.a[0]*mU.a[0]+mU.a[1]*mU.a[1]) ;
	float	v = sqrt(mV.a[0]*mV.a[0]+mV.a[1]*mV.a[1]) ;
	float	ga	= 90.0/asin(1.0)*acos((mU.a[0]*mV.a[0]+mU.a[1]*mV.a[1])/(u*v)) ;
	u	*= calibrationConstant ;
	v	*= calibrationConstant ;
	switch(calibrationUnit) {
		case kPixel:
			sprintf(text,"Reciprocal lattice: [Pixels]\nU=%.1f V=%.1f Ga=%.1f Deg",u,v,ga) ;
			break ;
		case kInversePixel:
			sprintf(text,"Reciprocal lattice: [1/Pixels]\nU=1/%.1f V=1/%.1f Ga=%.1f Deg",1/u,1/v,ga) ;
			break ;
		case kAngstrom:
			sprintf(text,"Reciprocal lattice: [Angstrom]\nU=%.2f V=%.2f Ga=%.1f Deg",u,v,ga) ;
			break ;
		case kInverseAngstrom:
			sprintf(text,"Reciprocal lattice: [1/Angstrom]\nU=1/%.2f V=1/%.2f Ga=%.1f Deg",1/u,1/v,ga) ;
			break ;
		case kNanometer:
			sprintf(text,"Reciprocal lattice: [nm]\nU%.2f V=%.2f Ga=%.1f Deg",u,v,ga) ;
			break ;
		case kInverseNanometer:
			sprintf(text,"Reciprocal lattice: [1/nm]\nU=1/%.2f V=1/%.2f Ga=%.1f Deg",1/u,1/v,ga) ;
			break ;
	}
}

void	TReciprocalVectors::DrawBaseUnits() {
	SetFgColor(gRedPixel) ;	

	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	SetFgColor(gBluePixel) ;	
	MoveTo(near(mX),near(mY)) ;
	LineTo(near(mX+mV.a[0]),near(mY+mV.a[1])) ;

	SetFgColor(gRedPixel) ;	
	
	DrawCircle(near(mX),near(mY),mSpotRadius) ;
	
	// Draw arrows
	double	k,b,x0,y0,x1,x2,y1,y2,A,B,L,d,a1,a2,a3,a4,xw1,yw1,xw2,yw2,w ;
	w = 4 ;
	d = 8 ;
	x1 = mX ;
	y1 = mY ;

	x2 = mX + mU.a[0] ;
	y2 = mY + mU.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (mU.a[0] != 0.) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;
		
	SetFgColor(gBluePixel) ;	
	x2 = mX + mV.a[0] ;
	y2 = mY + mV.a[1] ;
	L = sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)) ;
	if (mV.a[0] != 0.) {
		k = (y2-y1)/(x2-x1) ;
		b = (y1*x2 - x1*y2)/(x2-x1) ;
		if (k != 0.) {
			A = k*(y2-y1) + (x2-x1) ;
			B = x2*x2+y2*y2-x1*x2-y1*y2-b*(y2-y1)-L*d ;
			x0 = B/A ;
			y0 = k*x0+b ;
			a1 = 1 + k*k ;
			a2 = -2*a1*x0 ;
			a3 = a1*x0*x0 - k*k*w*w ;
			a4 = a2*a2-4*a1*a3 ;
			if (a4 <= 0.) {
				xw1 = -a2/(2*a1) ;
				xw2 = xw1 ;
			}
			else {
				xw1 = (-a2 + sqrt(a4))/(2*a1) ;
				xw2 = (-a2 - sqrt(a4))/(2*a1) ;
			} 
			yw1 = (-1./k)*xw1 + y0 + (1./k)*x0 ;
			yw2 = (-1./k)*xw2 + y0 + (1./k)*x0 ;
		}
		else {
			// Line is parallel to the x-axis
			xw1 = xw2 = (x2*x2-x1*x2-L*d)/(x2-x1) ;
			yw1 = mY+w ;
			yw2 = mY-w ;
		}
	}
	else {
		// Line is parallel to the y-axis
		xw1 = mX+w ;
		xw2 = mX-w ;
		yw1 = yw2 = (y2*y2-y1*y2-L*d)/(y2-y1) ;
	}

	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw1),near(yw1)) ;
	MoveTo(near(x2),near(y2)) ;
	LineTo(near(xw2),near(yw2)) ;

	
	SetFgColor(gBlackPixel) ;
	if (mIsSelected)
		DrawHandles() ;
}

void	TReciprocalVectors::DrawHandles() {
	

	SetFgColor(gGreenPixel) ;	
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mSpotRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mSpotRadius),near(mY)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mSpotRadius)) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mSpotRadius)) ;
	FrameRect(&selectionRect) ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mU.a[0]),near(mY+mU.a[1])) ;
	FrameRect(&selectionRect) ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mV.a[0]),near(mY+mV.a[1])) ;
	FrameRect(&selectionRect) ;
	SetFgColor(gBlackPixel) ;
}

bool	TReciprocalVectors::InHandles(Point &pt) {
	Rect	selectionRect ;

	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX-mSpotRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX+mSpotRadius),near(mY)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY-mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
	SetRect(&selectionRect,-hKandleSize,-hKandleSize,hKandleSize,hKandleSize) ;
	OffsetRect(&selectionRect,near(mX),near(mY+mSpotRadius)) ;
	InsetRect(&selectionRect,-1,-1) ;
	if (PtInRect(&pt,&selectionRect))
		return true ;
		
	long	rad = near(sqrt((pt.x-mX)*(pt.x-mX)+(pt.y-mY)*(pt.y-mY))) ;
	if (rad < mSpotRadius+2) 
		return true ;

	return false ;
}

void	TReciprocalVectors::Edit(Point &pt) {
	long	radsq,tolsq = 10 ;
	radsq = near((pt.x-(mX+mU.a[0]))*(pt.x-(mX+mU.a[0]))+(pt.y-(mY+mU.a[1]))*(pt.y-(mY+mU.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) {
			mNumLaticeVectorsU -= 1 ;
		}
		else {
			mNumLaticeVectorsU += 1 ;
		}
	}
	radsq = near((pt.x-(mX+mV.a[0]))*(pt.x-(mX+mV.a[0]))+(pt.y-(mY+mV.a[1]))*(pt.y-(mY+mV.a[1]))) ;
	if (radsq < tolsq) {
		if (ShiftKeyDown()) {
			mNumLaticeVectorsV -= 1 ;
		}
		else {
			mNumLaticeVectorsV += 1 ;
		}
	}
}

void	TReciprocalVectors::Modify(Point &pt) {
	Point	where,lastPoint ;
	
	GetMouse(&where) ;


	FPoint	cp ;
	FPoint	u,v ;
	cp.x = mX ;
	cp.y = mY ;
	float	cX = mX ;
	float	cY = mY ;
	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	
	mWindow->ImageToWindowPoint(cp) ;
	mWindow->ImageToWindowPoint(u) ;
	mWindow->ImageToWindowPoint(v) ;
	mX = cp.x ;
	mY = cp.y ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	
	mSpotRadius *= mWindow->GetMagnification() ;
	long	radsq,tolsq = mSpotRadius*mSpotRadius ;
	
	lastPoint = where ;
	mIsSelected = false ;
	PenMode(kXOrMode);
	DrawBaseUnits() ;
	if (!InHandles(where)) {
		radsq = near((where.x-u.x)*(where.x-u.x)+(where.y-u.y)*(where.y-u.y)) ;
		if (radsq < tolsq) {
			while (Button()) {
				GetMouse(&where) ;
				if (where.x != lastPoint.x || where.y != lastPoint.y) {
					DrawBaseUnits() ;
					mU.a[0] += (where.x - lastPoint.x) ;
					mU.a[1] += (where.y - lastPoint.y) ;
					lastPoint = where ;
					DrawBaseUnits() ;
				}
			}
		}
		else {
			radsq = near((where.x-v.x)*(where.x-v.x)+(where.y-v.y)*(where.y-v.y)) ;
			if (radsq < tolsq) {
				while (Button()) {
					GetMouse(&where) ;
					if (where.x != lastPoint.x || where.y != lastPoint.y) {
						DrawBaseUnits() ;
						mV.a[0] += (where.x - lastPoint.x) ;
						mV.a[1] += (where.y - lastPoint.y) ;
						lastPoint = where ;
						DrawBaseUnits() ;
					}
				}
			}
		}
	}
	else {
		// Size the mask
		while (Button()) {
			GetMouse(&where) ;
			if (where.x != lastPoint.x || where.y != lastPoint.y) {
				DrawBaseUnits() ;
				mSpotRadius = sqrt((where.x-mX)*(where.x-mX)+(where.y-mY)*(where.y-mY)) ;
				lastPoint = where ;
				DrawBaseUnits() ;
			}
		}
	}

	u.x = mX + mU.a[0] ;
	u.y = mY + mU.a[1] ;
	v.x = mX + mV.a[0] ;
	v.y = mY + mV.a[1] ;
	mWindow->WindowToImagePoint(u) ;
	mWindow->WindowToImagePoint(v) ;
	mX = cX ;
	mY = cY ;
	mU.a[0] = u.x - mX ;
	mU.a[1] = u.y - mY ;
	mV.a[0] = v.x - mX ;
	mV.a[1] = v.y - mY ;
	
	mSpotRadius /= mWindow->GetMagnification() ;

	PenNormal() ;
	mIsSelected = true ;

}

bool	TReciprocalVectors::InMask(Point &pt) {
	long	radsq,tolsq = (mSpotRadius+3)*(mSpotRadius+3) ;
	
	if (InHandles(pt))
		return true ;
		
	// Check to see if we are close to one of the lines
	float	x,y,x0,y0,a,b,dsq,d1,d2 ;
	
	x = pt.x ;
	y = pt.y ;
	if (mU.a[0] != 0.) {
		a = mU.a[1]/mU.a[0] ;
		b = (mY*(mX+mU.a[0]) - mX*(mY+mU.a[1]))/mU.a[0] ;
		x0 = (x+a*y-a*b)/(1+a*a) ;
		y0 = a*x0+b ;
		dsq = (x-x0)*(x-x0)+(y-y0)*(y-y0) ;
		d1 = fabs(x0 - mX - mU.a[0]) ;
		d2 = fabs(x0 - mX) ;
		if (dsq < 10. && d1 < fabs(mU.a[0])  && d2 < fabs(mU.a[0]))
			return true ;
	}
	else {
		// Line is parallel to the y-axis
		dsq = (x-mX)*(x-mX) ;
		if (dsq < 10.)
			return true ;
	}
		
	if (mV.a[0] != 0.) {
		a = mV.a[1]/mV.a[0] ;
		b = (mY*(mX+mV.a[0]) - mX*(mY+mV.a[1]))/mV.a[0] ;
		x0 = (x+a*y-a*b)/(1+a*a) ;
		y0 = a*x0+b ;
		dsq = (x-x0)*(x-x0)+(y-y0)*(y-y0) ;
		d1 = fabs(x0 - mX - mU.a[0]) ;
		d2 = fabs(x0 - mX) ;
		if (dsq < 10. && d1 < fabs(mV.a[0])  && d2 < fabs(mV.a[0]))
			return true ;
	}
	else {
		// Line is parallel to the y-axis
		dsq = (x-mX)*(x-mX) ;
		d1 = fabs(x0 - mX - mU.a[0]) ;
		d2 = fabs(x0 - mX) ;
		if (dsq < 10.)
			return true ;
	}
	   	
	radsq = near((pt.x-mX)*(pt.x-mX)+(pt.y-mY)*(pt.y-mY)) ;
	if (radsq < tolsq) 
		return true ;
	radsq = near((pt.x-(mX+mU.a[0]))*(pt.x-(mX+mU.a[0]))+(pt.y-(mY+mU.a[1]))*(pt.y-(mY+mU.a[1]))) ;
	if (radsq < tolsq) 
		return true ;
	radsq = near((pt.x-(mX+mV.a[0]))*(pt.x-(mX+mV.a[0]))+(pt.y-(mY+mV.a[1]))*(pt.y-(mY+mV.a[1]))) ;
	if (radsq < tolsq) 
		return true ;
	return false ;
}

