#ifndef __TMask_h
#define __TMask_h
#include "TVector.h"
#include "TButton.h"
#include "XDrivers.h"
#include "TBaseObject.h"
#include "TDialog.h"
#include "globals.h"
#include "TLattice.h"
class	TImageWindow ;
class	TImage ;

#define	kUnusedMaskParameter	-1
typedef struct {
	short			edgeWidth ;
	short			numLatticeVectorsA ;
	short			numLatticeVectorsB ;
	short			maskWidth ;
	short			maskIsOpaque ;
} MaskRecord ;

class	TFilledRect : public TObject {
	public:
		TFilledRect(Rect *rect,unsigned long colorPixel) : TObject(rect),mColor(colorPixel) { mIsActive = false ;} ;
		TFilledRect(short x1,short y1,short x2,short y2,unsigned long colorPixel)
			:TObject(x1,y1,x2,y2),mColor(colorPixel) { mIsActive = false ;};
		void Draw() { SetFgColor(mColor) ; FillRect(&mRect) ; SetFgColor(gBlackPixel) ; FrameRect(&mRect) ; } ;
	private:
		unsigned long mColor ;
} ;

class	TEditMaskDialog : public TDialog {

	public:
		TEditMaskDialog(MaskRecord *maskData) ;
		void	GetDialogData(void *data) { *(MaskRecord *)data = mMaskRecord ; } ;
		void	DoMouseDownInObject(TObject *,Point *) ;
		void	DoKeyDownInObject(TObject *obj)  ;
	private:
		TEditText		*mTextEdgeWidth ;
		TEditText		*mTextNumLatticeVectorsA ;
		TEditText		*mTextNumLatticeVectorsB ;
		TEditText		*mTextMaskWidth ;
		TCheckBox		*mHasSoftEdge ;
		TCheckBox		*mIsOpaque ;
		MaskRecord		mMaskRecord ;
		
} ;


typedef enum {kTransparentMask,kOpaqueMask} MaskType ;
class	TMask  : public TBaseObject {
	public:
		TMask(TImageWindow *window) : mIsSelected(true),mMaskType(kTransparentMask),mEdgeWidth(0),mWindow(window) {mSignature = 'mask' ;} ;
		TMask() {mSignature = 'mask' ; } ;
		virtual 		~TMask() {} ;
		virtual	void	Draw() {} ;
		virtual	void	DrawBaseUnits() { Draw() ; } ;
		virtual	void	DoMouseDown(Point &pt) ;
		virtual	bool	InMask(Point &pt) { return false ; } ;
		virtual	void	Apply(TImage *image) {} ;
		virtual	void	EditMaskparameters() {} ;
		void			Select(bool state) { mIsSelected = state ; } ;
		bool			IsSelected() { return mIsSelected ; } ;
		void			SetMaskType(MaskType type) {mMaskType = type ; } ;
		void			SetEdgeWidth(short width) {mEdgeWidth = width ; } ;
		void			SetWindow(TImageWindow *window) { mWindow = window ; } ;
	protected:
		virtual	void	DrawHandles() {} ;
		virtual	void	Edit(Point &pt) {} ;
		virtual	bool	InHandles(Point &pt) { return false ; } ;
		virtual	void	Modify(Point &pt) {} ;
		float			mX ;
		float			mY ;
		bool			mIsSelected ;
		MaskType		mMaskType ;
		short 			mEdgeWidth ;
		TImageWindow	*mWindow ;
} ;


class	TSpotMask : public TMask {
	public:
		TSpotMask(TImageWindow *window) ;
		TSpotMask() {} ;
		~TSpotMask() ;
		void	Draw() ;
		bool	InMask(Point &pt) ;
		void	Apply(TImage *image) ;
		void	EditMaskparameters() ;
        void	GetObjectInfo(char *text)  ;
        
        // TSpotMask	*Copy() { TSpotMask *mask = new TSpotMask ; *mask = *this ; return mask ; }  ; 
        TBaseObject	*Copy() { TSpotMask *mask = new TSpotMask ; *mask = *this ; return (TBaseObject *)mask ; }  ; 
	protected:
		void	Modify(Point &pt) ;
		void	DrawHandles() ;
		bool	InHandles(Point &pt) ;
		TVector	mU ;
		float	mSpotRadius ;
} ;

class	TCircleMask : public TMask {
	public:
		TCircleMask(TImageWindow *window) ;
		TCircleMask() {} ;
		~TCircleMask() ;
		void	Draw() ;
		bool	InMask(Point &pt) ;
		void	Apply(TImage *image) ;
		void	EditMaskparameters() ;
       	// TCircleMask	*Copy() { TCircleMask *mask = new TCircleMask ; *mask = *this ; return mask ; }  ; 
       	TBaseObject	*Copy() { TCircleMask *mask = new TCircleMask ; *mask = *this ; return (TBaseObject *)mask ; }  ; 
        void	GetObjectInfo(char *text)  ;
        
	protected:
		void	Modify(Point &pt) ;
		void	DrawHandles() ;
		bool	InHandles(Point &pt) ;
		float	mInnerRadius ;
		float	mOuterRadius ;
} ;

class	TLatticeMask : public TMask {
	public:
		TLatticeMask(TImageWindow *window) ;
		TLatticeMask() {} ;
		~TLatticeMask() ;
		void	Draw() ;
		void	DrawBaseUnits() ;
		bool	InMask(Point &pt) ;
		void	Apply(TImage *image) ;
		void	EditMaskparameters() ;
        void	GetObjectInfo(char *text)  ;
        
        // TLatticeMask	*Copy() { TLatticeMask *mask = new TLatticeMask ; *mask = *this ; return mask ; }  ; 
        TBaseObject	*Copy() { TLatticeMask *mask = new TLatticeMask ; *mask = *this ; return (TBaseObject *)mask ; }  ; 
	protected:
		void	Modify(Point &pt) ;
		void	Edit(Point &pt) ;
		void	DrawHandles() ;
		bool	InHandles(Point &pt) ;
		short	mNumLaticeVectorsU ;
		short	mNumLaticeVectorsV ;
		TVector	mU ;
		TVector	mV ;
		float	mSpotRadius ;
} ;

class	TWedgeMask : public TMask {
	public:
		TWedgeMask(TImageWindow *window) ;
		TWedgeMask() {} ;
		~TWedgeMask() ;
		void	Draw() ;
		void	DrawBaseUnits() ;
		bool	InMask(Point &pt) ;
		void	Apply(TImage *image) ;
		void	EditMaskparameters() ;
        void	GetObjectInfo(char *text)  ;
        
        // TWedgeMask	*Copy() { TWedgeMask *mask = new TWedgeMask;  *mask = *this ; return mask ; }  ; 
        TBaseObject	*Copy() { TWedgeMask *mask = new TWedgeMask;  *mask = *this ; return (TBaseObject *)mask ; }  ; 
	protected:
		void	Modify(Point &pt) ;
		void	DrawHandles() ;
		bool	InHandles(Point &pt) ;
		TVector	mU ;
		TVector	mV ;
} ;

class	TLineMask : public TMask {
	public:
		TLineMask(TImageWindow *window) ;
		TLineMask() {} ;
		~TLineMask() ;
		void	Draw() ;
		void	DrawBaseUnits() ;
		bool	InMask(Point &pt) ;
		void	Apply(TImage *image) ;
		void	EditMaskparameters() ;
        void	GetObjectInfo(char *text)  ;
        
       	// TLineMask	*Copy() { TLineMask *mask = new TLineMask ; *mask = *this ; return mask ; }  ; 
       	TBaseObject	*Copy() { TLineMask *mask = new TLineMask ; *mask = *this ; return (TBaseObject *)mask ; }  ; 
	protected:
		void	Modify(Point &pt) ;
		void	Edit(Point &pt) ;
		void	DrawHandles() ;
		bool	InHandles(Point &pt) ;
		TVector	mU ;
		TVector	mV ;
		short	mNumLaticeVectorsV ;
		float	mLineDistance ;
		float	mLineWidth ;
} ;

class	TReciprocalVectors : public TMask {
	public:
		TReciprocalVectors(TImageWindow *window) ;
		TReciprocalVectors() {} ;
		~TReciprocalVectors() ;
		void	Draw() ;
		void	DrawBaseUnits() ;
		bool	InMask(Point &pt) ;
		void	EditMaskparameters() ;
        void	GetObjectInfo(char *text)  ;
        TLattice	*GetLattice() ;
        void	SetLattice(TLattice *lattice) ;
        
        // TReciprocalVectors	*Copy() { TReciprocalVectors *mask = new TReciprocalVectors ; *mask = *this ; return mask ; }  ; 
        TBaseObject	*Copy() { TReciprocalVectors *mask = new TReciprocalVectors ; *mask = *this ; return (TBaseObject *)mask ; }  ; 
	protected:
		void	Modify(Point &pt) ;
		void	Edit(Point &pt) ;
		void	DrawHandles() ;
		bool	InHandles(Point &pt) ;
		short	mNumLaticeVectorsU ;
		short	mNumLaticeVectorsV ;
		TVector	mU ;
		TVector	mV ;
		float	mSpotRadius ;
} ;

#endif
