// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------


#include <stdio.h>
#include <ctype.h>
#include "TMenu.h"
#include "globals.h"
#include "XDrivers.h"
extern	short	gDebugMode ;

extern	UInt32	gLighterGray ;

#define ON 1 
#define max(x,y) (x) > (y) ? (x) : (y)

void	DisableMenuItem(TMenu *menu,short itemnumber) {
	menu->EnableMenuItem(itemnumber,false) ;
}

void	EnableMenuItem(TMenu *menu,short itemnumber) {
	menu->EnableMenuItem(itemnumber,true) ;
}

// -------------------------------
// TMenu
// -------------------------------

TMenu::TMenu(char *heading) {

	strcpy(mTitle,heading) ;
	mTitleWidth = StringWidth(heading)+10 ;
	mHeightPerItem = 20 ;
	mNumMenuItems = 0 ;
	mMenuItems = nil ;
	SetRect(&mWindowRect,0,0,0,0) ;
	mNextMenu = nil ;
	mPopUpWindow = nil ;
	mParentPane = nil ;
	mParentMenu = nil ;
	mEnabled = true ;	// by default ;
	mHighlighted = false ;
	mParentMenuItem = nil ;
	mX = 0 ;
	mY = 0 ; ;
}

TMenu::~TMenu() {
	TMenuItem *menuItem,*nextMenuItem ;
	for( menuItem = mMenuItems; menuItem != nil;) {
    		nextMenuItem = menuItem->mNextItem ;
		delete menuItem ;
		menuItem = nextMenuItem ;
	}
	
	if (mPopUpWindow != nil) delete mPopUpWindow ;
}

void TMenu::Draw() {
	int i,x ;
	TMenuItem *menuItem ;
	extern Display *gDisplay ;
	
	if (mNumMenuItems == 0) return ;

	Point	loc ; 
	loc.x = mX ;
	loc.y = mY ;
	LocalToGlobal(&loc) ;


	AdjustMenuWindowSize() ;

	// The menu draws into its own window, so the menu itself starts at	
	// 0,0 and the menuitems start with respect to this origin.	
	int	width = mWindowRect.right - mWindowRect.left ;
	int	height = mWindowRect.bottom - mWindowRect.top ;
	
	if (width == 0 || height == 0) return ;
	
	if (mPopUpWindow == nil) {
		mPopUpWindow = new TPopUpWindow(mParentPane,mX,mY,width,height) ;
		for( menuItem = mMenuItems; menuItem != nil;menuItem = menuItem->mNextItem) {
				menuItem->mWindow = mPopUpWindow ;
		}
	}
	else {
		// Size the window
		XResizeWindow(gDisplay,mPopUpWindow->GetXWindow(),width,height) ;
	}
	XMoveWindow(gDisplay,mPopUpWindow->GetXWindow(),loc.x ,loc.y) ;

	XMapRaised(gDisplay,mPopUpWindow->GetXWindow()) ;
	x = 10 ;
	menuItem = mMenuItems ;
	
	for (i=0;i<mNumMenuItems;i++) {
		menuItem->Draw(false) ;
		menuItem = menuItem->mNextItem ;
	}

	// Draw the outline of the menu box
	mPopUpWindow->SetFgColor(gWhitePixel) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),0,height-1,0,0) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),0,0,width-1,0) ;
	mPopUpWindow->SetFgColor(gDarkerGray) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),width-1,0,width-1,height-1) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),width-1,height-1,0,height-1) ;
	mPopUpWindow->SetFgColor(gBlackPixel) ;

	XFlush(gDisplay) ;
}

void	TMenu::AdjustMenuWindowSize() {
	short	i,widthOfMenuWindow = 0 ,heightOfMenuWindow = 0,widthOfMenuTitle ;
	TMenuItem *menuItem ;
	char	menuKeyText[32] ;
	Rect	theItemRect ;
	
	for(i=0,menuItem=mMenuItems;i < mNumMenuItems;i++) {
		widthOfMenuTitle = StringWidth(menuItem->mText) + 30 ;
		if (menuItem->mMenuKey != 0) {
			sprintf(menuKeyText,"Ctrl+%c",toupper(menuItem->mMenuKey)) ;
			widthOfMenuTitle += StringWidth(menuKeyText) ;
		}
		if (menuItem->mHierarchicalMenu)
			widthOfMenuTitle += 20 ;
		widthOfMenuWindow = max(widthOfMenuWindow,widthOfMenuTitle) ;
		if (menuItem->mText[0] == '-')
			heightOfMenuWindow += 4 ;
		else 
			heightOfMenuWindow += mHeightPerItem ;
		menuItem = menuItem->mNextItem ;
	}
	SetRect(&mWindowRect,0,0,widthOfMenuWindow,heightOfMenuWindow) ;
	// Adjust all the rectangles of the menu items
	short heightOfMenuItem = 0 ;
	theItemRect.left = 0 ;
	theItemRect.right = widthOfMenuWindow ;
	theItemRect.top = 0 ;
	for(i=0,menuItem=mMenuItems;i < mNumMenuItems;i++) {
		if (menuItem->mText[0] == '-')
			heightOfMenuItem = 4 ;
		else 
			heightOfMenuItem = mHeightPerItem ;
		theItemRect.bottom = theItemRect.top + heightOfMenuItem ;
		menuItem->SetMenuItemRect(&theItemRect) ;
		theItemRect.top = theItemRect.bottom ;
		menuItem = menuItem->mNextItem ;
	}
}

void TMenu::AddMenuItem(char *text,TCommand commandNum,TMenuKey menuKey) {
	int i ;
	TMenuItem *menuItem ;
	int width,lWidth ;
	Rect	rect ;
	char	menuKeyText[32] ;
	

	if (text != nil && *text != '-') {
		lWidth = mWindowRect.right - mWindowRect.left ;
		width = StringWidth(text) + 30 ;
		if (menuKey != 0) {
			sprintf(menuKeyText,"Ctrl+%c",toupper(menuKey)) ;
			width += StringWidth(menuKeyText) ;
		}
		lWidth = (lWidth > width ? lWidth : width) ;
		mWindowRect.right = mWindowRect.left + lWidth ;
		SetRect(&rect,0,mWindowRect.bottom,mWindowRect.right,
			mWindowRect.bottom+mHeightPerItem) ;
		mWindowRect.bottom += mHeightPerItem ;
	}
	else {
		// This is a divider
		SetRect(&rect,0,mWindowRect.bottom,mWindowRect.right,
			mWindowRect.bottom+4) ;
		mWindowRect.bottom += 4 ;
	}	
	// Find the location of the previous menuitem and add a link to this
	if (mNumMenuItems == 0) {
		mMenuItems = new TMenuItem(rect,text,commandNum,menuKey) ;
		menuItem = mMenuItems ;
		menuItem->mParentMenu = this ;
	}
	else {
		for(i=0,menuItem=mMenuItems;menuItem->mNextItem != 0;i++) {
			menuItem->AdjustWidth(lWidth) ;
			menuItem = menuItem->mNextItem ;
		}
		menuItem->mNextItem = new TMenuItem(rect,text,commandNum,menuKey) ;
		menuItem->mParentMenu = this ;
	}
	if (mPopUpWindow != nil) {
		delete mPopUpWindow ;
		mPopUpWindow = nil ;
	}
	//menuItem->mWindow = mPopUpWindow ;
	mNumMenuItems++ ;
}

void TMenu::SetParent(TPane *parent) { 
	mParentPane = parent ; 
	TMenuItem *menuItem ;
	for( menuItem = mMenuItems; menuItem != nil;) {
		if (menuItem->mIsHierarchicalMenu) 
			menuItem->mHierarchicalMenu->SetParent(parent) ;
		menuItem = menuItem->mNextItem ;
	}
}

void TMenu::AddMenuItem(TMenu *theMenu) {
	char text[256] ;
	int i ;
	TMenuItem *menuItem ;
	int width,lWidth ;
	Rect	rect ;
	
	strcpy(text,theMenu->mTitle) ;
	
	lWidth = mWindowRect.right - mWindowRect.left ;
	width = StringWidth(text) + 30 ;
	width += 25 ;
	lWidth = (lWidth > width ? lWidth : width) ;
	mWindowRect.right = mWindowRect.left + lWidth ;
	SetRect(&rect,0,mWindowRect.bottom,mWindowRect.right,
		mWindowRect.bottom+mHeightPerItem) ;
	mWindowRect.bottom += mHeightPerItem ;
	
	theMenu->mParentMenu = this ;
	
	// Find the location of the previous menuitem and add a link to this
	if (mNumMenuItems == 0) {
		mMenuItems = new TMenuItem(rect,text,theMenu) ;
		menuItem = mMenuItems ;
		menuItem->mParentMenu = this ;
		theMenu->mParentMenuItem = menuItem ;
	}
	else {
		for(i=0,menuItem=mMenuItems;menuItem->mNextItem != 0;i++) {
			menuItem->AdjustWidth(lWidth) ;
			menuItem = menuItem->mNextItem ;
		}
		menuItem->mNextItem = new TMenuItem(rect,text,theMenu) ;
		menuItem->mParentMenu = this ;
		theMenu->mParentMenuItem = menuItem->mNextItem ;
	}

	if (mPopUpWindow != nil) {
		delete mPopUpWindow ;
		mPopUpWindow = nil ;
	}
	mNumMenuItems++ ;
}

void	TMenu::RemoveMenuItem(char *text) {
	
	TMenuItem *menuItem,*lastItem = nil ;
	short	i ;

	// item numbers start at 1
	menuItem = mMenuItems ;
	for(i=0;i<mNumMenuItems && menuItem != nil ;i++) {
		if (strcmp(menuItem->mText,text) == 0) {
			// Remove this item
			if (i ==0) {
				mMenuItems = menuItem->mNextItem ;
				delete menuItem ;
				mNumMenuItems-- ;
				mWindowRect.bottom -= mHeightPerItem ;
				delete mPopUpWindow ;
				mPopUpWindow = nil ;
				return ;
				
			}
			else {
				lastItem->mNextItem = menuItem->mNextItem ;
				delete menuItem ;
				mNumMenuItems-- ;
				mWindowRect.bottom -= mHeightPerItem ;
				delete mPopUpWindow ;
				mPopUpWindow = nil ;
				return ;
			}
		}
		else {
			lastItem = menuItem ;
			menuItem = menuItem->mNextItem ;
		}
	}
	// We need to force a recreation of the popup window
}

void	TMenu::RemoveMenuItem(TCommand commandNum) {
	TMenuItem *menuItem,*lastItem = nil ;
	short	i ;

	// item numbers start at 1
	menuItem = mMenuItems ;
	for(i=0;i<mNumMenuItems && menuItem != nil ;i++) {
		if (menuItem->mCommand == commandNum) {
			// Remove this item
			if (i ==0) {
				mMenuItems = menuItem->mNextItem ;
				delete menuItem ;
				mWindowRect.bottom -= mHeightPerItem ;
				mNumMenuItems-- ;
				delete mPopUpWindow ;
				mPopUpWindow = nil ;
				if (gDebugMode) fprintf(stderr,"\nDeleted Menu Item %d",i+1) ;
				return ;
				
			}
			else {
				lastItem->mNextItem = menuItem->mNextItem ;
				delete menuItem ;
				mWindowRect.bottom -= mHeightPerItem ;
				mNumMenuItems-- ;
				delete mPopUpWindow ;
				mPopUpWindow = nil ;
				if (gDebugMode) fprintf(stderr,"\nDeleted Menu Item %d",i+1) ;
				return ;
			}
		}
		else {
			lastItem = menuItem ;
			menuItem = menuItem->mNextItem ;
		}
	}
	if (gDebugMode) fprintf(stderr,"\nFailed to remove window with command number %d",commandNum) ;
}

TMenuItem	*TMenu::GetMenuItem(short itemNumber) {
	short i ;
	TMenuItem *menuItem = nil ;
	// item numbers start at 1
	menuItem = mMenuItems ;
	for(i=0;i<itemNumber-1 && menuItem != nil ;i++) {
		menuItem = menuItem->mNextItem ;
	}
	
	return menuItem ;
}

void	TMenu::EnableMenuItem(short itemNumber,Boolean state) {
	short i ;
	TMenuItem *menuItem ;

	// item numbers start at 1
	menuItem = mMenuItems ;
	for(i=0;i<itemNumber-1 && menuItem != nil ;i++) {
		menuItem = menuItem->mNextItem ;
	}
	if (menuItem != nil) menuItem->Enable(state) ;
}

void	TMenu::CheckMenuItem(short itemNumber,Boolean state) {
	short i ;
	TMenuItem *menuItem ;

	// item numbers start at 1
	menuItem = mMenuItems ;
	for(i=0;i<itemNumber-1 && menuItem != nil ;i++) {
		menuItem = menuItem->mNextItem ;
	}
	if (menuItem != nil) menuItem->Check(state) ;
}

void TMenu::HighlightMenuItem(short itemNumber,Boolean state) {
	short i ;
	TMenuItem *menuItem ;

	// item numbers start at 1
	menuItem = mMenuItems ;
	for(i=0;i<itemNumber-1 && menuItem != nil ;i++) {
		menuItem = menuItem->mNextItem ;
	}
	if (menuItem != nil) menuItem->Draw(state) ;
	XFlush(gDisplay) ;
}

void TMenu::SetMenuItemText(short itemnumber,char *text) {
    short i ;
    TMenuItem *menuItem ;

    /* Number starts at 1 */
    menuItem = mMenuItems ;
    for(i=0;i<itemnumber-1 && menuItem != nil;i++) {
    	menuItem = menuItem->mNextItem ;
    }
    if (menuItem != nil) strcpy(menuItem->mText,text) ;
}

TCommand	TMenu::GetCommand(short itemnumber)  {
	short i ;
	TMenuItem *menuItem ;

	 // Number starts at 1
	menuItem = mMenuItems ;
	for(i=0;i<itemnumber-1 && menuItem != nil;i++) {
    		menuItem = menuItem->mNextItem ;
	}
    	if (menuItem != nil) return menuItem->mCommand  ;
	return 0 ;
}

void TMenu::GetMenuItemText(short itemnumber,char *text) {
    short i ;
    TMenuItem *menuItem ;

    /* Number starts at 1 */
    menuItem = mMenuItems ;
    for(i=0;i<itemnumber-1 && menuItem != nil;i++) {
    	menuItem = menuItem->mNextItem ;
    }
    if (menuItem != nil) strcpy(text,menuItem->mText) ;
}

// -------------------------------
// TPopupMenu
// -------------------------------

TPopupMenu::TPopupMenu(TPane *parent,short x1,short y1,short x2, short y2,char *text,
	char *label,LabelPosition pos) 
		: TMenu(text),TMenuButton(x1,y1,x2,y2,text,label,pos) {
	mParentPane = parent ;
	mSelectedItem = 1 ;	// The menu starts at 1
}

TPopupMenu::TPopupMenu(TPane *parent,Rect *rect,char *text,char *label,LabelPosition pos) 
		: TMenu(text),TMenuButton(rect,text,label,pos) {
	mParentPane = parent ;
	mSelectedItem = 1 ;	// The menu starts at 1
}

TPopupMenu::TPopupMenu(TPane *parent,Rect *rect,char *text[],short choice,char *label,
	LabelPosition pos) 
		: TMenu(text[choice-1]),TMenuButton(rect,text[choice-1],label,pos) {
	mParentPane = parent ;
	mSelectedItem = choice ;	// The menu starts at 1

	for(int i=0;text[i] != nil; i++) {
		// By default all menuitess are anabled so we give them a command number
		AddMenuItem(text[i],999) ;
	}
}

TPopupMenu::TPopupMenu(TPane *parent,short x1,short y1,short x2, short y2,char *text[],
	short choice,char *label,LabelPosition pos) 
		: TMenu(text[choice-1]),TMenuButton(x1,y1,x2,y2,text[choice-1],label,pos) {
	mParentPane = parent ;
	mSelectedItem = choice ;	// The menu starts at 1

	for(int i=0;text[i] != nil; i++) {
		// By default all menuitems are anabled so we give them a command number
		AddMenuItem(text[i],999) ;
	}
}

TPopupMenu::~TPopupMenu() {
}

Boolean TPopupMenu::DoMouseDown(Point *where) {

	if (!mIsActive) return false ;
	if (mState) return false ;


	mState = ON ;
	TMenuButton::Draw() ;
	XFlush(gDisplay) ;

	Point	menuLocation ;
	Point	globalPosition ;
	GetGlobalMouse(&globalPosition) ;
	int dx = globalPosition.x - where->x ;
	int dy = globalPosition.y - where->y ;
	menuLocation.x = where->x ;
	menuLocation.y = (mRect.top+mRect.bottom)/2 - (2*mSelectedItem-1)*mHeightPerItem/2 ;
	menuLocation.x += dx ;
	menuLocation.y += dy ;
	short	selection = Select(&menuLocation) ;
	Boolean	status = false ;
	if (selection != mSelectedItem) status = true ;
	mState = OFF ;
	TMenuButton::Draw() ;
	return status ;
} 

void TPopupMenu::DrawPopped() {
	TMenuItem *menuItem ;
	extern Display *gDisplay ;
	
	if (mNumMenuItems == 0) return ;


	Point	loc ; 
	loc.x = mX ;
	loc.y = mY ;

	// For some reason I have to do a GlobalToLocal first
	// and then a LocalToGlobal to get it right if I change the parent window
	// position between different DrawPopped
	GlobalToLocal(&loc) ;
	LocalToGlobal(&loc) ;
	
	// The menu draws into its own window, so the menu itself starts at	
	// 0,0 and the menuitems start with respect to this origin.	
	int	width = mWindowRect.right - mWindowRect.left ;
	int	height = mWindowRect.bottom - mWindowRect.top ;
	
	if (width == 0 || height == 0) return ;
	
	AdjustMenuWindowSize() ;
	mY = (mNumMenuItems-1)*10 ;
	if (mPopUpWindow == nil) {
		mPopUpWindow = new TPopUpWindow(mParentPane,mX,mY,width,height) ;
		for( menuItem = mMenuItems; menuItem != nil;menuItem = menuItem->mNextItem) {
				menuItem->mWindow = mPopUpWindow ;
		}
	}
	
	XMoveWindow(gDisplay,mPopUpWindow->GetXWindow(),loc.x ,loc.y) ;

	XMapRaised(gDisplay,mPopUpWindow->GetXWindow()) ;
	menuItem = mMenuItems ;
	
	for (int i=0;i<mNumMenuItems;i++) {
		if (mSelectedItem == i+1)
			menuItem->Draw(true) ;
		else
			menuItem->Draw(false) ;
		menuItem = menuItem->mNextItem ;
	}

	// Draw the outline of the menu box
	mPopUpWindow->SetFgColor(gWhitePixel) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),0,height-1,0,0) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),0,0,width-1,0) ;
	mPopUpWindow->SetFgColor(gDarkerGray) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),width-1,0,width-1,height-1) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),width-1,height-1,0,height-1) ;
	mPopUpWindow->SetFgColor(gBlackPixel) ;

	XFlush(gDisplay) ;
}

short	TPopupMenu::Select(Point *where) {
	mX = where->x ;
	mY = where->y ;

	TMenuItem	*menuItem ;
	int 	rx,ry,wx,wy ;
	Rect 	tRect ;
	unsigned int	keyButt ;
	Window	child,root ;
	short	lastmenuitem,selectedItem,j ;
	Point	currentPt ;
	extern	Display	*gDisplay ;


	// Let us sit in an event loop only looking for mouse moved and click events
	Boolean	done = false ;
	Boolean	pseudoMouseIsDown = true ;
	extern	Time	gClickTime ;
	extern	bool	gMouseIsDown ;
	XEvent	event ;
	XButtonEvent *buttEvent = (XButtonEvent *)&event ;

	Boolean	intheMenu		= false ;
	
	XGrabPointer(gDisplay,mParentWindow->GetXWindow(),False,
			ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
			GrabModeAsync,GrabModeAsync,None,None,CurrentTime) ;


	// We have a mouse down event already
	XQueryPointer(gDisplay,mParentWindow->GetXWindow(),&root,&child,&rx,&ry,&wx,&wy,&keyButt) ;
	
	currentPt.x = wx ;
	currentPt.y = wy ;
	if (PtInRect(&currentPt,&mRect)) {	// Within the menu button	
		DrawPopped() ;
	}
	
	lastmenuitem = mSelectedItem ;
	intheMenu = false ;
	
	while (!done) {
		XNextEvent(gDisplay,&event) ;		
		switch(event.type) {
			case ButtonPress:
				if (pseudoMouseIsDown) done = true ;				
				break ;
			case ButtonRelease:
				gMouseIsDown = false ;
				if ((buttEvent->time - gClickTime) < 400) 
					pseudoMouseIsDown = true ;
				else 
					done = true ;
				break ;
			case MotionNotify:
				XQueryPointer(gDisplay,mPopUpWindow->GetDrawable(),
						&root,&child,&rx,&ry,&wx,&wy,&keyButt) ;
				currentPt.x = wx ;
				currentPt.y = wy ;
				menuItem = mMenuItems ;
				intheMenu = false ;
				for (j=0;j<mNumMenuItems;j++) {
					tRect = menuItem->mRect ;
					if (PtInRect(&currentPt,&tRect)) {
						intheMenu = true ;
						selectedItem = j+1 ;
						if (selectedItem != lastmenuitem) {
							if (lastmenuitem != 0) {
								// Unhighlight old
								HighlightMenuItem(lastmenuitem,false) ; 
							}
							lastmenuitem = selectedItem ;
							// Highlight new
							HighlightMenuItem(j+1,true) ;
						}
						break ;
					}
					menuItem = menuItem->mNextItem ;
				}
				if (!intheMenu) {
					if (lastmenuitem != 0) {
						// Unhighlight old
						HighlightMenuItem(lastmenuitem,false) ; 
						selectedItem = 0 ;
						lastmenuitem = 0 ;
					}
				}
				break ;
			default:
				break ;
		}	
	}

	if (lastmenuitem > 0) {
		menuItem = GetMenuItem(lastmenuitem) ;
		if (menuItem != nil)
			if (menuItem->mEnabled) {
				if (mSelectedItem != lastmenuitem) {
					char text[256] ;
					GetMenuItemText(lastmenuitem,text) ;
					SetMenuText(text) ;
				}
				mSelectedItem = lastmenuitem ;
			}
	}
	
	gMouseIsDown = false ;
	
	XUngrabPointer(gDisplay,CurrentTime) ;
	mPopUpWindow->Hide() ;
	return 0 ;
}


// -------------------------------
// TCenteredPopupMenu
// -------------------------------

TCenteredPopupMenu::TCenteredPopupMenu(TPane *parent,short x1,short y1,short x2, short y2,char *text,
	char *label,LabelPosition pos) 
		: TPopupMenu(parent,x1,y1,x2,y2,text,label,pos) {
}

TCenteredPopupMenu::TCenteredPopupMenu(TPane *parent,Rect *rect,char *text,char *label,LabelPosition pos) 
		: TPopupMenu(parent,rect,text,label,pos) {
}

TCenteredPopupMenu::TCenteredPopupMenu(TPane *parent,Rect *rect,char *text[],short choice,char *label,
	LabelPosition pos) 
		: TPopupMenu(parent,rect,text,choice,label,pos) {
}

TCenteredPopupMenu::TCenteredPopupMenu(TPane *parent,short x1,short y1,short x2, short y2,char *text[],
	short choice,char *label,LabelPosition pos) 
		: TPopupMenu(parent,x1,y1,x2,y2,text,choice,label,pos) {
}

void TCenteredPopupMenu::DrawPopped() {
	TMenuItem *menuItem ;
	extern Display *gDisplay ;
	
	if (mNumMenuItems == 0) return ;
	
	// The menu draws into its own window, so the menu itself starts at	
	// 0,0 and the menuitems start with respect to this origin.	
	int	width = mWindowRect.right - mWindowRect.left ;
	int	height = mWindowRect.bottom - mWindowRect.top ;
	
	if (width == 0 || height == 0) return ;
	
	AdjustMenuWindowSize() ;
	mY = (mNumMenuItems-1)*10 ;
	if (mPopUpWindow == nil) {
		mPopUpWindow = new TPopUpWindow(mParentPane,mX,mY,width,height) ;
		for( menuItem = mMenuItems; menuItem != nil;menuItem = menuItem->mNextItem) {
				menuItem->mWindow = mPopUpWindow ;
		}
	}
	
	XMoveWindow(gDisplay,mPopUpWindow->GetXWindow(),mX,mY) ;

	XMapRaised(gDisplay,mPopUpWindow->GetXWindow()) ;
	menuItem = mMenuItems ;
	
	for (int i=0;i<mNumMenuItems;i++) {
		if (mSelectedItem == i+1)
			menuItem->Draw(true) ;
		else
			menuItem->Draw(false) ;
		menuItem = menuItem->mNextItem ;
	}

	// Draw the outline of the menu box
	mPopUpWindow->SetFgColor(gWhitePixel) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),0,height-1,0,0) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),0,0,width-1,0) ;
	mPopUpWindow->SetFgColor(gDarkerGray) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),width-1,0,width-1,height-1) ;
	XDrawLine(gDisplay,mPopUpWindow->GetDrawable(),
			mPopUpWindow->GetGC(),width-1,height-1,0,height-1) ;
	mPopUpWindow->SetFgColor(gBlackPixel) ;

	XFlush(gDisplay) ;
}

// -------------------------------
// TMenuItem
// -------------------------------
TMenuItem::TMenuItem(Rect &rect,char *text,TCommand command,TMenuKey menuKey) {
	mRect = rect ;
	if (text != nil && *text != '-') {
		strcpy(mText,text) ;
		mEnabled = true ; // by default 
		mIsDivider = false ;
	}
	else {
		strcpy(mText,"-") ;
		mIsDivider = true ;
		mEnabled = false ; // always for dividers
	}
	mNextItem = nil ;
	mCommand = command ;
	mMenuKey = menuKey ;
	if (mCommand == 0) mEnabled = false ;
	mChecked = false ; // by default 
	mIsHierarchicalMenu = false ;
	mHierarchicalMenu = nil ;
}

// -------------------------------
// TMenuItem
// -------------------------------
TMenuItem::TMenuItem(Rect &rect,char *title,TMenu *childMenu) {
	mRect = rect ;
	strcpy(mText,title) ;
	mEnabled = true ; // by default 
	mIsDivider = false ;
	mNextItem = nil ;
	mCommand = 0 ;
	mMenuKey = 0 ;
	if (mCommand == 0) mEnabled = false ;
	mChecked = false ; // by default 
	mIsHierarchicalMenu = true ;
	mHierarchicalMenu = childMenu ;
	childMenu->SetParentMenuItem(this) ;
}

TMenuItem::~TMenuItem() {
	if (mHierarchicalMenu != nil) delete mHierarchicalMenu ;
}


void	TMenuItem::SetMenuItemRect(Rect *theRect) { 
	mRect = *theRect ; 
	short x,y ;
	if (mIsHierarchicalMenu) {
		mParentMenu->GetPosition(x,y) ;
		mHierarchicalMenu->AdjustMenuWindowSize() ;
		mHierarchicalMenu->SetPosition(mRect.right+x,mRect.top+y) ;
	}
}

void TMenuItem::DrawCheckMark() {
	short	x = 2 ;
	short	y = mRect.bottom-5 ;


	XDrawLine(gDisplay,mWindow->GetDrawable(),
		mWindow->GetGC(),x,y-4,x+2,y) ;
	XDrawLine(gDisplay,mWindow->GetDrawable(),
		mWindow->GetGC(),x+2,y,x+6,y-8) ;
	
	//MoveTo(x,y-4) ;
	//LineTo(x+2,y) ;
	//LineTo(x+6,y-8) ;
}

void TMenuItem::DrawMenuArrow() {
	short	x = mRect.right-18 ;
	short	y = (mRect.bottom+mRect.top)/2 ;


	XDrawLine(gDisplay,mWindow->GetDrawable(),
		mWindow->GetGC(),x,y-4,x,y+4) ;
	XDrawLine(gDisplay,mWindow->GetDrawable(),
		mWindow->GetGC(),x,y+4,x+8,y) ;
	XDrawLine(gDisplay,mWindow->GetDrawable(),
		mWindow->GetGC(),x+8,y,x,y-4) ;
	
	//MoveTo(x,y-4) ;
	//LineTo(x+2,y) ;
	//LineTo(x+6,y-8) ;
}

void TMenuItem::Draw(Boolean state) {
	Rect	tRect = mRect ;
	short	x = 10 ;
	short	y = mRect.bottom-5 ;
	char	menuKeyText[32] ;

	if (mIsDivider) {
		mWindow->SetFgColor(gDarkerGray) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.left,tRect.top+1,tRect.right,tRect.top+1) ;
		mWindow->SetFgColor(gWhitePixel) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.left,tRect.top+2,tRect.right,tRect.top+2) ;
		mWindow->SetFgColor(gBlackPixel) ;
		return ;
	}

	if (!mEnabled) {
		mWindow->SetFgColor(gDarkerGray) ;
		
		if (mIsHierarchicalMenu) {
			XDrawString(gDisplay,mWindow->GetDrawable(),
				mWindow->GetGC(),x,y,mText,strlen(mText)) ;
			if (state == ON) {
				DrawMenuArrow() ;
				mHierarchicalMenu->Draw() ;
			}
			else {
				mHierarchicalMenu->Hide() ;
				DrawMenuArrow() ;
			}
		}
		else {
			if (mChecked) DrawCheckMark() ;

			XDrawString(gDisplay,mWindow->GetDrawable(),
				mWindow->GetGC(),x,y,mText,strlen(mText)) ;
			if (mMenuKey != 0) {
				sprintf(menuKeyText,"Ctrl+%c",toupper(mMenuKey)) ;
				XDrawString(gDisplay,mWindow->GetDrawable(),
					mWindow->GetGC(),mRect.right-StringWidth(menuKeyText)-10,y,menuKeyText,strlen(menuKeyText)) ;
			}
		}
		mWindow->SetFgColor(gBlackPixel) ;
		return ;
	}
	
	if (state == ON) {

		mWindow->SetFgColor(gLighterGray) ;
		InsetRect(&tRect,1,1) ;
		XFillRectangle(gDisplay,mWindow->GetDrawable(),mWindow->GetGC(),
			tRect.left,tRect.top,tRect.right-tRect.left,
			tRect.bottom-tRect.top) ;
		InsetRect(&tRect,1,1) ;
		mWindow->SetFgColor(gWhitePixel) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.left,tRect.bottom,tRect.left,tRect.top) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.left,tRect.top,tRect.right,tRect.top) ;
		InsetRect(&tRect,1,1) ;
		mWindow->SetFgColor(gDarkerGray) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.right,tRect.top,tRect.right,tRect.bottom) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
		InsetRect(&tRect,-1,-1) ;
		mWindow->SetFgColor(gBlackPixel) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.right,tRect.top,tRect.right,tRect.bottom) ;
		XDrawLine(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;

		if (mIsHierarchicalMenu) {
			XDrawString(gDisplay,mWindow->GetDrawable(),
				mWindow->GetGC(),x,y,mText,strlen(mText)) ;
			DrawMenuArrow() ;
			mHierarchicalMenu->Draw() ;
		}
		else {
			if (mChecked) DrawCheckMark() ;

			XDrawString(gDisplay,mWindow->GetDrawable(),
				mWindow->GetGC(),x,y,mText,strlen(mText)) ;
			if (mMenuKey != 0) {
				sprintf(menuKeyText,"Ctrl+%c",toupper(mMenuKey)) ;
				XDrawString(gDisplay,mWindow->GetDrawable(),
					mWindow->GetGC(),mRect.right-StringWidth(menuKeyText)-10,y,menuKeyText,strlen(menuKeyText)) ;
			}
		}
	}
	else {
		InsetRect(&tRect,1,1) ;
		XClearArea(gDisplay,mWindow->GetDrawable(),tRect.left,tRect.top,
			tRect.right-tRect.left,tRect.bottom-tRect.top,0) ;

		if (mChecked) DrawCheckMark() ;
		if (mIsHierarchicalMenu) {
			mHierarchicalMenu->Hide() ;
			DrawMenuArrow() ;
		}

		XDrawString(gDisplay,mWindow->GetDrawable(),
			mWindow->GetGC(),x,y,mText,strlen(mText)) ;
		if (mMenuKey != 0) {
			sprintf(menuKeyText,"Ctrl+%c",toupper(mMenuKey)) ;
			XDrawString(gDisplay,mWindow->GetDrawable(),
				mWindow->GetGC(),mRect.right-StringWidth(menuKeyText)-10,y,menuKeyText,strlen(menuKeyText)) ;
		}
	}
}


