// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------

#include <stdio.h>

#include "TMotifWindow.h" 
#include "DMMenuCommands.h"
#include "globals.h"
#include "XDrivers.h"
#include "TCrystalDiffractionDataTable.h"
#include "TPotentialMapWindow.h"

extern  long    near(float) ;
extern	double	Pi ;
extern	double	TwoPi ;

extern	char	*gPlaneGroup[] ;
extern	short	gImageDepth ; 
void	SetTargetWindow(TWindow *window) ;
void	SendApplicationCommand(TCommand command) ;
void SaveBinaryImage(TImage *image) ;
void SaveGrayscaleTiffImage(TImage *image) ;

extern	class TDMApplicationWindow *gTheControlWindow ;
extern	void	AddWindow(TWindow *window) ; 

extern	float	gMeanOfMaskedImage ;

TMotifWindow::TMotifWindow(TDiffractionPeaks *diffractionData) 
		: TWindow(190,140,510,316,gImageDepth,"Symmetrize Motif Window",false) {
	short i ;
	short	x,y ;

	mBkColor = gGrayPixel ;
	XSetBackground(gDisplay,mGC,mBkColor) ;
	
	mScaleFactor = 0. ;
	mShowUCOutline = true ;

	mDiffractionData = new TDiffractionPeaks(diffractionData) ;
	
	// I believe the next statement have no effect
	// short xc = 0, yc = 0 ;
	// TArray *dataArray = mDiffractionData->Symmetrize(1,xc,yc) ;
	// delete dataArray ;
		
	mCurrentPlaneGroup = 1 ;
	
	mGamma = 180-diffractionData->mGammaStar ;
		
	mSymmetrizedImage = nil ;
	mPhaseResidualImage = nil ;
	
	for(i=1;i<=17;i++) {
		mPhaseResiduals[i-1] = nil ;
		mMinimumPhaseResidual[i-1] = -1. ;
		mSymmetryOrigin[i-1].x = 0 ;
		mSymmetryOrigin[i-1].y = 0 ;
	}
	mMinimumPhaseResidual[0] = 0. ;
	
	mObjectList = new TObjectList() ;
	TUserRect *theUserRect ;
	
	x = 12 ; 
	y = 24 ;
	mListRectObject = new TUserRect(x,y,x+130,y+280) ;
	mListRectObject->Enable() ;
	mListRect = mListRectObject->GetRect() ;
	AddObject(mListRectObject) ;

	x = mListRect.right + 8 ;
	theUserRect = new TUserRect(x,y,x+170,y+170) ;
	mImageRect = theUserRect->GetRect() ;
	AddObject(theUserRect) ;

	x = mImageRect.right + 8 ;
	mResidualRectObject = new TUserRect(x,y,x+170,y+170) ;
	mResidualRectObject->Enable() ;
	mResidualRect = mResidualRectObject->GetRect() ;
	AddObject(mResidualRectObject) ;
	
	TStaticText *theText ;
	theText = new TStaticText(mListRect.left,mListRect.top-18,mListRect.right,mListRect.top-2,"Plane Group") ;
	theText->SetJustification(kCenterJustified) ;
	AddObject(theText) ;

	theText = new TStaticText(mImageRect.left,mImageRect.top-18,mImageRect.right,mImageRect.top-2,"Symmetrized Image") ;
	theText->SetJustification(kCenterJustified) ;
	AddObject(theText) ;

	theText = new TStaticText(mResidualRect.left,mResidualRect.top-18,mResidualRect.right,mResidualRect.top-2,"Phase residuals") ;
	theText->SetJustification(kCenterJustified) ;
	AddObject(theText) ;
	
	short	buttonHeight = 28 ;
	
	y = mImageRect.bottom + 10  ;
	mTryAllGroups = new TButton(mImageRect.left,y,mImageRect.right,y+buttonHeight,"Calculate phase residuals") ;
	AddObject(mTryAllGroups) ;

	mShowHKs = new TButton(mResidualRect.left,y,mResidualRect.right,y+buttonHeight,"Show Amplitudes & Phases") ;
	AddObject(mShowHKs) ;


	y = mImageRect.bottom + 10 + buttonHeight + 4  ;

	mComputePotentialMap = new TButton(mResidualRect.left,y,mResidualRect.right,y+buttonHeight,"Create 'Potential Map'...") ;
	AddObject(mComputePotentialMap) ;
	
	mCreateImage = new TButton(mImageRect.left,y,mImageRect.right,y+buttonHeight,"Create Symmetrized Image...") ;
	AddObject(mCreateImage) ;

	y += buttonHeight + 4 ;
	mShowOutline = new TCheckBox(mImageRect.left,y,mImageRect.right,y+20,"Show Cell Outline",mShowUCOutline) ;
	AddObject(mShowOutline) ;
	
	mClose = new TButton((mResidualRect.left+mResidualRect.right)/2,mListRect.bottom-buttonHeight,mResidualRect.right,mListRect.bottom,"Close") ;
	AddObject(mClose) ;
	

	// Default is spacegroup 1
	TArray* symmetrizedArray = Symmetrize(mCurrentPlaneGroup,mSymmetryOrigin[mCurrentPlaneGroup-1].x,
			mSymmetryOrigin[mCurrentPlaneGroup-1].y,mMinimumPhaseResidual[mCurrentPlaneGroup-1]) ;
	mSymmetrizedImage = new TImage(symmetrizedArray,mGamma) ;
	mSymmetrizedImage->SetSampling(mDiffractionData->mA/symmetrizedArray->GetWidth(),mDiffractionData->mB/symmetrizedArray->GetHeight()) ;
	mSymmetrizedImage->Repeat(3,3) ;
	mSymmetrizedImage->AdjustGamma() ;
	mSymmetrizedImage->AdjustSampling() ;
	delete 	symmetrizedArray ;				

}


TMotifWindow::~TMotifWindow() {
	short i ;
	delete mDiffractionData ;
	
	if (mPhaseResidualImage != nil) delete mPhaseResidualImage ;
	if (mSymmetrizedImage != nil) delete mSymmetrizedImage ;
	for(i=1;i<=17;i++) {
		if (mPhaseResiduals[i-1] != nil)
			delete mPhaseResiduals[i-1] ;
	}
}

void 	TMotifWindow::CalculatePhaseResiduals() {
	short	i ;
	Rect	theRect ;
	char	text[25] ;
	
	for(i=2;i<=17;i++) {
		mPhaseResiduals[i-1] = mDiffractionData->CalculatePhaseResidual(i,
				mMinimumPhaseResidual[i-1],mSymmetryOrigin[i-1].x,mSymmetryOrigin[i-1].y) ;
				
		if (mMinimumPhaseResidual[i-1] > -1.) {
			SetRect(&theRect,mListRect.left+1,mListRect.top+(i-1)*16+4,mListRect.right-1,mListRect.top+i*16+4) ;
			EraseRect(&theRect) ;
			sprintf(text,"%s",gPlaneGroup[i-1]) ;
			MoveTo(mListRect.left+4,mListRect.top+i*16) ;
			DrawString(text) ;
			sprintf(text,"%.2f",mMinimumPhaseResidual[i-1]) ;
			MoveTo(mListRect.left+80,mListRect.top+i*16) ;
			DrawString(text) ;
			XFlush(gDisplay) ;
		}
	}		
}

void 	TMotifWindow::CreateImage() {
	UnitCellDisplayRecord displayRecord ;
	TImage *theImage = nil ;
	TArray* symmetrizedArray = nil ;
	float	ux,uy,vx,vy ;
	
	FPoint	U = mDiffractionData->mLattice->GetU() ;
	FPoint	V = mDiffractionData->mLattice->GetV() ;
	

	// Adjust the lattice for small round off errors
	if (fabs(U.x) < 0.001) U.x = 0. ;
	if (fabs(U.y) < 0.001) U.y = 0. ;
	if (fabs(V.x) < 0.001) V.x = 0. ;
	if (fabs(V.y) < 0.001) V.y = 0. ;
	
	// Now adjust the lattice so that we have the right symmetry for the plane-group
	// This might distort some things if we are to match the original image
	// We will simplify by letting U stay in place and adjust V since I don't know
	// which is beter at the moment
	
	float	AStar = sqrt(U.x*U.x+U.y*U.y) ;
	float	BStar = sqrt(V.x*V.x+V.y*V.y) ;
	
	float	AStarOld = AStar ;
	float	sinAlpha,cosAlpha ;
	float	sinGammaSt,cosGammaSt ;

	switch(mCurrentPlaneGroup) {
		case 1:	// p1
		case 2: // p2
			break ;
		case 3: // pm -  
		case 4: // pg 
		case 5: // cm 
		case 6: // pmm  -
		case 7: // pmg  
		case 8: // pgg  
		case 9: // cmm  
		case 10: // p4
		case 11: // p4m
		case 12: // p4g
			// mGammaStar = 90.0 ;
			if (mCurrentPlaneGroup > 9) {
				// A* == B*
				cosGammaSt = 0. ;
				sinGammaSt = 1. ;

				AStar = (AStar + BStar)/2. ;
				BStar = AStar ;
				
				U.x = U.x * AStar/AStarOld ;
				U.y = U.y * AStar/AStarOld ;
				
				V.y = V.y > 0. ? fabs(U.x) : -fabs(U.x) ;
				V.x = V.x > 0. ? fabs(U.y) : -fabs(U.y) ;
			}
			break ;
		case 13: // p3
		case 14: // p3m1
		case 15: // p31m
		case 16: // p6
		case 17: // p6m
			// mGammaStar = 60.0 ;
			// A* = B*
			cosGammaSt = 0.5 ;
			sinGammaSt = 0.5*sqrt(3.0) ;
			AStar = (AStar + BStar)/2. ;
			BStar = AStar ;
			U.x = U.x * AStar/AStarOld ;
			U.y = U.y * AStar/AStarOld ;
			
			// Adjust V so that the angle is 60 degrees between U and V
			ux = U.x ;
			uy = -U.y ;	// Since we have negative y going up
			vx = V.x ;
			vy = -V.y ;
			// U * V
			if ((ux*vy - uy*vx) > 0.) {
				// U Cross V Pointing up
				sinAlpha = uy/AStar ;
				cosAlpha = ux/AStar ;
				//vx = U*cos(alpha + gammaStar)
				//vy = U*sin(alpha + gammaStar)
				vx = AStar*(cosAlpha*cosGammaSt - sinAlpha*sinGammaSt) ;
				vy = AStar*(sinAlpha*cosGammaSt + cosAlpha*sinGammaSt) ;				
			}
			else {
				// U Cross V Pointing down
				sinAlpha = uy/AStar ;
				cosAlpha = ux/AStar ;
				//vx = U*cos(alpha - gammaStar)
				//vy = U*sin(alpha - gammaStar)
				vx = AStar*(cosAlpha*cosGammaSt + sinAlpha*sinGammaSt) ;
				vy = AStar*(sinAlpha*cosGammaSt - cosAlpha*sinGammaSt) ;				
			}
			V.x = vx ;
			V.y = -vy ;
			break ;
	}

	long	Width = mDiffractionData->mFourierData->GetWidth() ;
	long	Height = mDiffractionData->mFourierData->GetHeight() ;
		
	TCreateSymmetrizedImageDialog *theDialog = new TCreateSymmetrizedImageDialog(&displayRecord) ;
	if (theDialog->DoModalDialog()) {
		theDialog->GetDialogData(&displayRecord) ;
		delete theDialog ;
		if (displayRecord.createImageMatchingExperiment) {
			// Try to create at least a p1 image to see if it works
			TDiffractionPeaks *symmetrizedPattern = new TDiffractionPeaks(mDiffractionData) ;
			symmetrizedArray = symmetrizedPattern->Symmetrize(mCurrentPlaneGroup,mSymmetryOrigin[mCurrentPlaneGroup-1].x,
								mSymmetryOrigin[mCurrentPlaneGroup-1].y) ;

			TArray *hkArray = symmetrizedPattern->mFourierData ;

			// Put the diffraction data down at the lattice points
			short	hMax = hkArray->GetWidth()/2 ;
			short	kMax = hkArray->GetHeight()/2 ;
			double	amp,phase ;


			ux = U.x ;
			uy = U.y ;
			vx = V.x ;
			vy = V.y ;

			
			float	ax,bx,ay,by ;
			// a = (bst * cst) / Vst
			// b = (cst * ast) / Vst
			
			ux /= mDiffractionData->mImageWidth ;
			vx /= mDiffractionData->mImageWidth ;

			uy /= mDiffractionData->mImageHeight ;
			vy /= mDiffractionData->mImageHeight ;
			
			float	Vst = fabs(ux*vy - uy*vx) ;
			ax = vy/Vst ;
			ay = -vx/Vst ;
			bx = -uy/Vst ;
			by = ux/Vst ;
			

			ux = U.x ;
			uy = U.y ;
			vx = V.x ;
			vy = V.y ;
			
			float	fnx = mDiffractionData->mImageWidth/fabs(ax+bx) ;
			float	fny = mDiffractionData->mImageHeight/fabs(ay+by)  ;
			long nx = fnx ;
			long ny = fny ;
			
			if ((fnx-nx) > 0.01) nx++ ;
			if ((fny-ny) > 0.01) ny++ ;
			
			float scaleFactorX = nx*fabs(ax+bx)/mDiffractionData->mImageWidth ;
			float scaleFactorY = ny*fabs(ay+by)/mDiffractionData->mImageHeight ;
					
			long imageWidth = 2*(near(scaleFactorX*mDiffractionData->mImageWidth)/2) ;
			long imageHeight = 2*(near(scaleFactorY*mDiffractionData->mImageHeight)/2) ;

			ux *= scaleFactorX ;
			uy *= scaleFactorY ;
			vx *= scaleFactorX ;
			vy *= scaleFactorY ;

			ux = near(ux) ;
			uy = near(uy) ;
			vx = near(vx) ;
			vy = near(vy) ;
			
			TArray *imageArray = new TArray(imageWidth,imageHeight,kComplexArray) ;
			*imageArray = 0. ;
			
			register long h,k,ih,ik ;
			
			double phaseShiftCH = mSymmetryOrigin[mCurrentPlaneGroup-1].x * TwoPi/Width ;
			double phaseShiftCK = mSymmetryOrigin[mCurrentPlaneGroup-1].y * TwoPi/Height ;
			double phaseShiftH ;
			double phaseShiftK ;

			for(h = -hMax+1; h < hMax; h++) {
				phaseShiftH = -h*phaseShiftCH ;
				for(k=-kMax+1; k < kMax; k++) {
					amp = *hkArray->GetRealHK(h,k) ;
					if (amp > 0. ) {
						ih = near(h*ux+k*vx) ;
						ik = near(h*uy+k*vy) ;
						phaseShiftK = -k*phaseShiftCK ;
						phase = *hkArray->GetImagHK(h,k) + phaseShiftH + phaseShiftK ;
						*imageArray->GetRealHK(ih,ik) 	= amp*::cos(phase) ;
						*imageArray->GetImagHK(ih,ik) 	= amp*::sin(phase) ;
					}
				}
			}
			
			delete symmetrizedPattern ;	
			imageArray->ifft(false) ;
						
			Rect	imageRect ;
			SetRect(&imageRect,imageWidth-mDiffractionData->mImageWidth,imageHeight-mDiffractionData->mImageHeight,
				imageWidth,imageHeight) ;
					
			theImage = new TImage(*imageArray,&imageRect) ;
			theImage->SetCalibration(1.0) ;
			theImage->SetCalibrationUnit(kPixel) ;
			
			*theImage += gMeanOfMaskedImage ;
			theImage->SetName(gPlaneGroup[mCurrentPlaneGroup-1]) ;
			theImage->Display() ;		
			delete 	imageArray ;	
		}
		else {
			float	lengthA = mDiffractionData->mA ;
			float	lengthB = mDiffractionData->mB ;
			if (mCurrentPlaneGroup > 9) {
				lengthA = (mDiffractionData->mA + mDiffractionData->mB)/2 ;
				lengthB = lengthA ;
			}

			symmetrizedArray = Symmetrize(mCurrentPlaneGroup,mSymmetryOrigin[mCurrentPlaneGroup-1].x,
						mSymmetryOrigin[mCurrentPlaneGroup-1].y,mMinimumPhaseResidual[mCurrentPlaneGroup-1]) ;
			theImage = new TImage(symmetrizedArray,mGamma) ;
			theImage->SetSampling(lengthA/symmetrizedArray->GetWidth(),lengthB/symmetrizedArray->GetHeight()) ;
			theImage->Repeat(displayRecord.numCellsA,displayRecord.numCellsB) ;
			theImage->AdjustGamma() ;
			theImage->AdjustSampling() ;
	 
			theImage->SetName(gPlaneGroup[mCurrentPlaneGroup-1]) ;
			theImage->Display() ;
		}
	}
	else 
		delete theDialog ;
		
	if (symmetrizedArray != nil) delete symmetrizedArray ;
}

void 	TMotifWindow::ShowPotentialMapWindow() {
	TDiffractionPeaks *symmetrizedPattern = new TDiffractionPeaks(mDiffractionData) ;
	TArray* symmetrizedArray = symmetrizedPattern->Symmetrize(mCurrentPlaneGroup,
		mSymmetryOrigin[mCurrentPlaneGroup-1].x,mSymmetryOrigin[mCurrentPlaneGroup-1].y) ;
		
	short 	h,k,hDataMax,kDataMax,mumReflections,hStart ;
	float	amplitude ;
	TArray	*dataArray = symmetrizedPattern->mFourierData ;
	hDataMax = dataArray->GetWidth()/2 -1 ;
	kDataMax = dataArray->GetHeight()/2 -1 ; 
		
	// First find out how many diffraction spots we have
	mumReflections = 0 ;
	for(k = 0; k < kDataMax; k++)  {
		hStart = k == 0 ? 1 : -hDataMax ;
		for(h = hStart; h < hDataMax; h++) {
			if (!(h == 0 && k == 0)) {
				amplitude = *dataArray->GetRealHK(h,k) ;
				if (amplitude > 0.001) mumReflections++ ;
			}
		}
	}
		

	TableAttributes	attr = {mumReflections,5,4,60,20,256,256,true,true,true,false,nil} ;	
	TPotentialDataTable *theTable = new TPotentialDataTable(&attr,symmetrizedPattern,mCurrentPlaneGroup) ;
	TWindow *theWindow = new TPotentialMapWindow(theTable) ;
	theWindow->Show() ;
	theWindow->SetCommander((TWindow *)gTheControlWindow) ;
	AddWindow(theWindow) ;
	delete symmetrizedArray ;
}

void 	TMotifWindow::DisplayHKs() {

	TDiffractionPeaks *symmetrizedPattern = new TDiffractionPeaks(mDiffractionData) ;
	TArray* symmetrizedArray = symmetrizedPattern->Symmetrize(mCurrentPlaneGroup,
		mSymmetryOrigin[mCurrentPlaneGroup-1].x,mSymmetryOrigin[mCurrentPlaneGroup-1].y) ;
	char	tableTitle[256] ;
	sprintf(tableTitle,"Image Fourier components [%s]",gPlaneGroup[mCurrentPlaneGroup-1]) ;
	
	TCrystalDiffractionDataTable *theHKTable = new TCrystalDiffractionDataTable(tableTitle,
		symmetrizedPattern,mCurrentPlaneGroup) ;
		
		
	TWindow *theWindow = (TWindow *)theHKTable->GetTablePane() ;
	theWindow->SetCommander((TWindow *)gTheControlWindow) ;		
	AddWindow(theWindow) ;
	delete symmetrizedArray ;
}

void	TMotifWindow::DoWMRequestDelete() {
	// Send the application the close command for this window
	SetTargetWindow(this) ;
	SendApplicationCommand(kClose) ;
}

TArray 	*TMotifWindow::Symmetrize(short numPlaneGroup,short &xc,short &yc,float &phaseResidual) {

	double	radToDegree = 90.0/asin(1.0) ;
	TDiffractionPeaks *symmetrizedPattern = new TDiffractionPeaks(mDiffractionData) ;
	TArray* symmetrizedArray = symmetrizedPattern->Symmetrize(numPlaneGroup,xc,yc) ;
	symmetrizedArray->ifft(false) ;

	mGamma = 180-mDiffractionData->mGammaStar ;
	
	if (numPlaneGroup < 3) {
		if (fabs(mGamma-90.) < 1.0) mGamma = 90. ;
		if (fabs(mGamma-120.) < 1.0) mGamma = 120. ;
	}
	else if (numPlaneGroup > 2 && numPlaneGroup < 13) 
		mGamma = 90. ;
	else if (numPlaneGroup > 12) 
		mGamma = 120. ;
	
	if (mPhaseResiduals[numPlaneGroup-1] != nil)
		phaseResidual =  *mPhaseResiduals[numPlaneGroup-1]->GetData(xc,yc)*radToDegree ;
	
	delete symmetrizedPattern ;	
	
	return  symmetrizedArray ;
}

void	TMotifWindow::DisplayResiduals() {
	short 	planeGroup ;
	char	text[256] ;
	
	Rect	selectedRect,theRect ;
	for(planeGroup=1;planeGroup<=17;planeGroup++) {
		SetRect(&theRect,mListRect.left+1,mListRect.top+(planeGroup-1)*16+4,mListRect.right-1,mListRect.top+planeGroup*16+4) ;
		EraseRect(&theRect) ;
		if (planeGroup == mCurrentPlaneGroup) {
			SetRect(&selectedRect,mListRect.left+1,mListRect.top+(mCurrentPlaneGroup-1)*16+4,mListRect.right-1,mListRect.top+mCurrentPlaneGroup*16+4) ;
			SetFgColor(gBlackPixel) ;
			FillRect(&selectedRect) ;
			SetFgColor(gWhitePixel) ;
		}
		sprintf(text,"%s",gPlaneGroup[planeGroup-1]) ;
		MoveTo(mListRect.left+4,mListRect.top+planeGroup*16) ;
		DrawString(text) ;
		if (mMinimumPhaseResidual[planeGroup-1] > -1.)
			sprintf(text,"%.2f",mMinimumPhaseResidual[planeGroup-1]) ;
		else
			sprintf(text," - ") ;
		MoveTo(mListRect.left+80,mListRect.top+planeGroup*16) ;
		DrawString(text) ;
		SetFgColor(gBlackPixel) ;
			
	}

	FrameRect(&mListRect) ;
}

void	TMotifWindow::DisplaySymmetrizedImage() {
	if (mSymmetrizedImage != nil) {			
		if (mScaleFactor == 0.) {
			float	fraction1,fraction2 ;
			long	cx,cy,left,top ;	
			cx = (mImageRect.left+mImageRect.right)/2 ;		
			cy = (mImageRect.top+mImageRect.bottom)/2 ;
			// Calculate the actual image in which we are displaying			
			long rectWidth = mImageRect.right - mImageRect.left ;
			long rectHeight = mImageRect.bottom - mImageRect.top ;
			
			fraction1 = ((float)rectWidth)/mSymmetrizedImage->Width() ;
			fraction2 = ((float)rectHeight)/mSymmetrizedImage->Height() ;
			
			mScaleFactor = fraction1 < fraction2 ? fraction1 : fraction2 ;
			
			long imageWidth = near(mScaleFactor*mSymmetrizedImage->Width()) ;
			long imageHeight = near(mScaleFactor*mSymmetrizedImage->Height()) ;
			
			
			left = cx - imageWidth/2 ;
			top = cy - imageHeight/2 ;
			SetRect(&mActualImageRect,left,top,left+imageWidth,top+imageHeight) ;
			
			cx = (mResidualRect.left+mResidualRect.right)/2 ;		
			cy = (mResidualRect.top+mResidualRect.bottom)/2 ;
			left = cx - imageWidth/2 ;
			top = cy - imageHeight/2 ;
			SetRect(&mActualResidualRect,left,top,left+imageWidth,top+imageHeight) ;
		}
		mSymmetrizedImage->Display(this,&mActualImageRect) ;	
		if (mShowUCOutline) {
			TVector A,B,O ;
			float	aa,bb,ga ;
			aa = (mActualImageRect.right-mActualImageRect.left)/3. ;
			bb = aa*mDiffractionData->mB/mDiffractionData->mA ;
			ga = (180-mGamma)*asin(1.0)/90. ;
			if (mGamma > 100.) 
				O.a[0] = mActualImageRect.left+bb*cos(ga) ; 
			else 
				O.a[0] = mActualImageRect.left+aa+bb*cos(ga) ; 
			O.a[1] = mActualImageRect.top+bb*sin(ga) ; 
			A.a[0] = aa ;
			A.a[1] = 0. ;
			B.a[0] = bb*cos(ga) ;
			B.a[1] = bb*sin(ga) ;
			SetFgColor(gRedPixel) ;
			MoveTo(near(O.a[0]),near(O.a[1])) ;
			LineTo(near(O.a[0]+A.a[0]),near(O.a[1]+A.a[1])) ;
			LineTo(near(O.a[0]+A.a[0]+B.a[0]),near(O.a[1]+A.a[1]+B.a[1])) ;
			LineTo(near(O.a[0]+B.a[0]),near(O.a[1]+B.a[1])) ;
			LineTo(near(O.a[0]),near(O.a[1])) ;
			SetFgColor(gBlackPixel) ;
		}
			
	} 
	FrameRect(&mImageRect) ;
}

void	TMotifWindow::DisplayPhaseResidualImage() {
	if (mPhaseResidualImage == nil) {
		EraseRect(&mActualResidualRect) ;
		FrameRect(&mResidualRect) ;
	}
	else if (mMinimumPhaseResidual[mCurrentPlaneGroup-1] > 0.) {
		mPhaseResidualImage->Display(this,&mActualResidualRect) ;
		FrameRect(&mResidualRect) ;
	}
}

void	TMotifWindow::Draw() {
	
	EraseRect(&mPortRect) ;
	DisplayResiduals() ;
	DisplaySymmetrizedImage() ;
	DisplayPhaseResidualImage() ;
	
	TPane::Draw() ;
}

void	TMotifWindow::DoMouseDownInObject(TObject *obj,Point *pt) {	
	SetPort(this) ;
	if (pt != nil)
		if (!obj->DoMouseDown(pt)) return ;
	if (obj == mTryAllGroups) {
		mTryAllGroups->SetState(true) ;
		CalculatePhaseResiduals() ;
		mTryAllGroups->SetState(false) ;
		mTryAllGroups->Disable() ;
	}
	else if (obj == mCreateImage) {
		CreateImage() ; 
	}
	else if (obj == mComputePotentialMap) {
		ShowPotentialMapWindow() ; 
	}
	else if (obj == mShowHKs) {
		DisplayHKs() ;
	}
	else if (obj == mResidualRectObject) {
	}
	else if (obj == mClose) {
		SetTargetWindow(this) ;
		SendApplicationCommand(kClose) ;
		return ;
	}
	else if (obj == mShowOutline) {
		mShowUCOutline = mShowOutline->GetControlValue() ;
		UpdateImages() ;
	}
	else if (obj == mListRectObject) {
		// Find out which plane group we are in
		short row,selectedGroup ;
		Rect	selectedRect ;
		char	text[256] ;
		float	lengthA,lengthB ;
		
		row  = (pt->y - mListRect.top)/16 ;
		if (row >= 0 && row < 17) {
			selectedGroup = row+1 ;
			if (mMinimumPhaseResidual[row] > -1.) {
				if (selectedGroup !=  mCurrentPlaneGroup) {
					SetRect(&selectedRect,mListRect.left+1,mListRect.top+(mCurrentPlaneGroup-1)*16+4,mListRect.right-1,mListRect.top+mCurrentPlaneGroup*16+4) ;
					EraseRect(&selectedRect) ;
					sprintf(text,"%s",gPlaneGroup[mCurrentPlaneGroup-1]) ;
					MoveTo(mListRect.left+4,mListRect.top+mCurrentPlaneGroup*16) ;
					DrawString(text) ;
					if (mMinimumPhaseResidual[mCurrentPlaneGroup-1] > -1.)
						sprintf(text,"%.2f",mMinimumPhaseResidual[mCurrentPlaneGroup-1]) ;
					else
						sprintf(text," - ") ;
					MoveTo(mListRect.left+80,mListRect.top+mCurrentPlaneGroup*16) ;
					DrawString(text) ;
					mCurrentPlaneGroup = selectedGroup ;
					SetRect(&selectedRect,mListRect.left+1,mListRect.top+(mCurrentPlaneGroup-1)*16+4,mListRect.right-1,mListRect.top+mCurrentPlaneGroup*16+4) ;
					FillRect(&selectedRect) ;
					SetFgColor(gWhitePixel) ;
					sprintf(text,"%s",gPlaneGroup[mCurrentPlaneGroup-1]) ;
					MoveTo(mListRect.left+4,mListRect.top+mCurrentPlaneGroup*16) ;
					DrawString(text) ;
					if (mMinimumPhaseResidual[mCurrentPlaneGroup-1] > -1.)
						sprintf(text,"%.2f",mMinimumPhaseResidual[mCurrentPlaneGroup-1]) ;
					else
						sprintf(text," - ") ;
					MoveTo(mListRect.left+80,mListRect.top+mCurrentPlaneGroup*16) ;
					DrawString(text) ;
					SetFgColor(gBlackPixel) ;
					
					lengthA = mDiffractionData->mA ;
					lengthB = mDiffractionData->mB ;
					if (mCurrentPlaneGroup > 9) {
						lengthA = (mDiffractionData->mA + mDiffractionData->mB)/2 ;
						lengthB = lengthA ;
					}
					if (mPhaseResidualImage != nil) delete mPhaseResidualImage ; mPhaseResidualImage = nil ;
					if (mCurrentPlaneGroup > 1) {
						mPhaseResidualImage = new TImage(mPhaseResiduals[mCurrentPlaneGroup-1],mGamma) ;
						mPhaseResidualImage->SetSampling(lengthA/mPhaseResiduals[mCurrentPlaneGroup-1]->GetWidth(),
								lengthB/mPhaseResiduals[mCurrentPlaneGroup-1]->GetHeight()) ;
						mPhaseResidualImage->Repeat(3,3) ;
						mPhaseResidualImage->AdjustGamma() ;
						mPhaseResidualImage->AdjustSampling() ;
					}
					
					if (mSymmetrizedImage != nil) delete mSymmetrizedImage ;
					TArray* symmetrizedArray = Symmetrize(mCurrentPlaneGroup,mSymmetryOrigin[mCurrentPlaneGroup-1].x,
								mSymmetryOrigin[mCurrentPlaneGroup-1].y,mMinimumPhaseResidual[mCurrentPlaneGroup-1]) ;
					mSymmetrizedImage = new TImage(symmetrizedArray,mGamma) ;
					mSymmetrizedImage->SetSampling(lengthA/symmetrizedArray->GetWidth(),lengthB/symmetrizedArray->GetHeight()) ;
					mSymmetrizedImage->Repeat(3,3) ;
					mSymmetrizedImage->AdjustGamma() ;
					mSymmetrizedImage->AdjustSampling() ;

					delete 	symmetrizedArray ;	
					UpdateImages() ;
					
				}
			}
		}
	}
}

TCreateSymmetrizedImageDialog::TCreateSymmetrizedImageDialog(UnitCellDisplayRecord *data)
	: TDialog(kDialogPosX,kDialogPosY,300,140,"Create Symmetrized Image") {
	mData = *data ;
	
	mCreateSymmetrizedUnitCellImage = new TRadioButton(10,10,290,30,"Create symmetrized unit cell image",true) ;
	AddObject(mCreateSymmetrizedUnitCellImage) ;
	
	mCreateSymmetrizedExperimentalImage = new TRadioButton(10,70,290,90,"Create symmetrized 'experimental' image ",false) ;
	mCreateSymmetrizedExperimentalImage->Enable() ; // For now
	AddObject(mCreateSymmetrizedExperimentalImage) ;

	mTextNumA = new TEditText(100,40,140,60,"# Unit Cells Wide","3") ;
	mTextNumB = new TEditText(240,40,280,60,"# Unit Cells High","3") ;

	AddTextEditObject(mTextNumB) ;
	AddTextEditObject(mTextNumA) ;
	
	mDefaultButton = mOKButton = new TButton(200,100,280,120,"OK") ;
	AddObject(mOKButton) ;

	mCancelButton = new TButton(90,100,190,120,"Cancel") ;
	AddObject(mCancelButton) ;
		
}

void	TCreateSymmetrizedImageDialog::DoMouseDownInObject(TObject *obj,Point *pt) {
	
	SetPort(this) ;
	if (pt != nil)
		if (!obj->DoMouseDown(pt)) return ;
	if (obj == mOKButton) {
		mDone = true ;
		mDialogReturnCode = kDialogOK ;
		return ;
	}
	if (obj == mCancelButton) {
		mDone = true ;
		mDialogReturnCode = kDialogCancel ;
		return ;
	}
	if (obj == mCreateSymmetrizedUnitCellImage) {
		mTextNumA->Enable() ;
		mTextNumB->Enable() ;
		mCreateSymmetrizedExperimentalImage->SetControlValue(false) ;
	}
	else if (obj == mCreateSymmetrizedExperimentalImage) {
		mTextNumA->Disable() ;
		mTextNumB->Disable() ;
		mCreateSymmetrizedUnitCellImage->SetControlValue(false) ;
	}
}

void TCreateSymmetrizedImageDialog::RetrieveDialogItems() {
	char text[256] ;
	mTextNumA->GetEditText(text) ;
	mData.numCellsA = atoi(text) ;
	
	mTextNumB->GetEditText(text) ;
	mData.numCellsB = atoi(text) ;
	
	mData.createImageMatchingExperiment = mCreateSymmetrizedExperimentalImage->GetControlValue() ;
}
