/*
 *  TPeaksControlWindow.h
 *  xcode_edm
 *
 *  Created by Roar Kilaas on Wed Oct 13 2004.
 *  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#ifndef __TPeaksControlWindow_H
#define __TPeaksControlWindow_H
#include "FS98ControlRecord.h"
#include "TPotentialMapWindow.h"
#include "TTable.h"
#include "TList.h"
#include "TAtom.h"
#include "TImageWindow.h"
#include "TDialog.h"

typedef long PeakCommand ;
#define kPeakStartup		0 
#define kPeakChangeThreshold	1 
#define kPeakQuit		1000 
#define kPeakAtoms   	2
#define kPeakList    	3		
#define kPeakSave   	4
#define kPeakEnt   	5	
#define kPeakPos   	6
#define kPeakOcc   	7
#define kPeakRef   	8
#define kPeakFix   	9
//#define kPeakGOF   	10
#define kPeakSEED       10
#define kPeakGOF		110
#define kPeakDMap   	11
#define kPeakDProj   	12
#define kPeakModel   	13
#define kPeakTrim   	14
#define kPeakEnter   	15
#define kPeakDMapOne   	16
#define kPeakInquireAtoms			17
#define kPeakFindBestAtomCombination	18
#define kPeakUseAtomList				181
#define kPeakShowAtomAssignments		19
#define kPeakShowCurrentSettings		20
#define kPeakDWRef						120
#define kPeakRefUcmin					121
#define kBabinet						122
#define kPeakRefTwin					123
#define kPeakSendValue	200
#define kPeakTerminal	100

#define kPeakOne	101
#define kPeakTwo	102
#define kPeakEDMT	103
#define kPeakHUNT	104
#define kPeakBOND	105
#define kPeakBUMP	106
#define kPeakFULL	107
#define kPeakERR	108
#define kPeakROB	109
// Adding the command for reading a constraints file 'constraints.lst'
#define kPeakConstraints	111	
//#define kPeakSEED	110
//LDM Added commands
#define kPeakPurgeSeeds     300
#define kPeakReadAtoms      301
#define kPeakUseFHKL		302
#define kPeakSettoModel		303
#define kPeakCycle			304
#define kPeakICons			305
#define kPeakMultiplicity   306

// Seed Refine  SR
// BOND			BOND
// BUMP			BUMP
// All the above76 followed by a list of number (angers), e.g. for a Ti O structure
//Ti-Ti, Ti-O, O-O distances (in angstroms)
// COORD		COOR
// Followed by a min/max for each atom (can be zero min) 
// HUNT			HUNT
// Large Set	FULL (OFF/ON)
// Use Errors   ERR  (OFF/ON)
// ROBUST		ROB  (OFF/ON)

typedef enum {kPeakAddTool,kPeakRemoveTool,kPeakPointer,kPeakZoom,kPeakHand,kPeakRuler } PeakToolType ;

typedef enum {kPeakDisplayGeneticMap,kPeakDisplayCurrentModel,kPeakDisplayCurrentMap,kPeakDisplayDifferenceMap,
	kPeakDisplayExperimentalDP,kPeakDisplayCalculatedDP,kPeakDisplayFourierFit} PeakDisplayMode ;
typedef enum {kRFIT,kRBFIT} HKLCalculatedFileType ;
Boolean	FileExists(char *filename) ;
void	CreateFile(char *filename) ;
void	RemoveFile(char *filename) ;
void	wait(long milliseconds) ;

typedef enum {kUsePeak,kTestPeak,kDoNotUsePeak} PeakUsage ;
class TAtomPeak : public TAtom {
	public:
		TAtomPeak(void) ;
		TAtomPeak(TAtomPeak &s) ;
		short	index ;
		short   baseIndex ;
		PeakUsage	usage ;
		float	peak ;
		char 	symbol[8] ;
		FPoint	screenPos ;
		short	screenRad ;
		TAtomPeak& operator=(const TAtomPeak&) ;
} ;

typedef enum {kHKLDisplayExperimentalDP,kHKLkDisplayCalculatedDP,kHKLDisplayFourierFit} HKLDisplayMode ;

class TAtomPeakEntry {	
	public:
		TAtomPeakEntry() ;
		~TAtomPeakEntry() ;
		friend class TAtomPeakList ;
	private:	
		TAtomPeak		*mAtomPeak ;
		TAtomPeakEntry	*mNextAtomEntry ;
}  ;

class TAtomPeakList {
	public:
		TAtomPeakList() ;
		~TAtomPeakList() ;
		void 			AddAtomPeak(TAtomPeak *theAtomPeak) ;
		void			RemoveAtomPeak(short num) ;
		short			GetNumberOfAtomPeaks(void) { return mNumAtomPeaks ; } ;
		TAtomPeak		*GetAtomPeak(int number) ;
	private:
		short			mNumAtomPeaks ;
		TAtomPeakEntry	*mAtomPeakEntry ;
		TAtomPeakEntry	*mLastEntry ;
	
}  ;


class	TPeakDisplayList : public TList {
	public:
		TPeakDisplayList(Rect	*theListRect) ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TPeakLogList : public TList {
	public:
		TPeakLogList(Rect	*theListRect) ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
} ;


class TPeakSolution {
	public:
		TPeakSolution() { mIsCurrent = false ; } ;
		~TPeakSolution() {} ;
		//TPeakSolution() { mAtomPeakList = new TAtomPeakList ; } ;
		//~TPeakSolution() { delete mAtomPeakList ; } ;
		TPeakSolution& operator=(const TPeakSolution&) ;
		short			mSolutionNumber ;
		float			mRValue ;
		bool			mIsCurrent ;
		TAtomPeakList   *mAtomPeakList ;
} ;

class	TPeakSolutionList : public TList {
	public:
		TPeakSolutionList(Rect	*theListRect) ;
		short   mSelectedSolution ;
		void	Sort() ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

typedef struct {
	long	h ;
	long	k ;
	long	l ;
	float   expVal ;
	float   calcVal ;
	float   error ;
} PeakHKLRecord ;

class	THKLDisplayList : public TList {
	public:
		THKLDisplayList(Rect	*theListRect) ;
		void	SetDisplayMode(HKLDisplayMode mode) { mHKLDisplayMode = mode ; if (mIsVisible) 	Refresh() ; } ;

	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	DrawColLabel(short colNumber) ;
		HKLDisplayMode  mHKLDisplayMode ;
} ;

class BondRecord {
	public:	
		BondRecord() { atom1[0] = '\0' ; atom2[0] = '\0' ; distance = 0. ; } ; 
		char	atom1[8] ;
		char	atom2[8] ;
		float	distance ;
} ;

class StochiometryRecord {
	public:	
		StochiometryRecord() { atom[0] = '\0' ; valence = 1. ; } ; 
		char	atom[8] ;
		float	valence ;
}  ;

class CoordinationRecord {
	public:	
		CoordinationRecord() { atom[0] = '\0' ; minNeighbors = 1  ; maxNeighbors = 1 ; } ; 
		char	atom[8] ;
		long	minNeighbors ;
		long	maxNeighbors ;
} ;

class CellContentRecord {
	public:	
		CellContentRecord() { atom[0] = '\0' ; minNumber = 1 ; maxNumber = 1 ; } ; 
		char	atom[8] ;
		long	minNumber ;
		long	maxNumber ;
}  ;

class	TBonds {
	public:
		TBonds() { mBondRec = nil ; mNumBonds = 0 ; mWeight = 1.0 ; mActive = true ; } ;
		TBonds(short numBonds) { mBondRec = new BondRecord[numBonds] ; mNumBonds = numBonds ;  mWeight = 1.0 ;  mActive = true ; } ;
		~TBonds() { if (mBondRec != nil) delete[] mBondRec ; } ;
		long		mNumBonds ;
		BondRecord	*mBondRec ;
		float		mWeight ;
		Boolean		mActive ;
} ;

class	TStochiometry {
	public:
		TStochiometry() { mStochiometryRecord = nil ; mNumConstraints = 0 ; mWeight = 1.0 ; mActive = true ; } ;
		TStochiometry(short numConstraints) { mStochiometryRecord = new StochiometryRecord[numConstraints] ; mNumConstraints = numConstraints ;  mWeight = 1.0 ;  mActive = true ; } ;
		~TStochiometry() { if (mStochiometryRecord != nil) delete[] mStochiometryRecord ; } ;
		short		mNumConstraints ;
		StochiometryRecord	*mStochiometryRecord ;
		float		mWeight ;
		float		mBase ;
		Boolean		mActive ;
} ;

class	TCellContent {
	public:
		TCellContent() { mCellContentRecord = nil ; mNumConstraints = 0 ; mWeight = 1.0 ; mActive = true ; } ;
		TCellContent(short numConstraints) { mCellContentRecord = new CellContentRecord[numConstraints] ; mNumConstraints = numConstraints ;  mWeight = 1.0 ;  mActive = true ; } ;
		~TCellContent() { if (mCellContentRecord != nil) delete[] mCellContentRecord ; } ;
		long		mNumConstraints ;
		CellContentRecord	*mCellContentRecord ;
		float		mWeight ;
		Boolean		mActive ;
} ;

class	TCoordination {
	public:
		TCoordination() { mCoordinationRecord = nil ; mNumConstraints = 0 ; mWeight = 1.0 ; mActive = true ; } ;
		TCoordination(short numConstraints) { mCoordinationRecord = new CoordinationRecord[numConstraints] ; mNumConstraints = numConstraints ;  mWeight = 1.0 ;  mActive = true ; } ;
		~TCoordination() { if (mCoordinationRecord != nil) delete[] mCoordinationRecord ; } ;
		long		mNumConstraints ;
		CoordinationRecord	*mCoordinationRecord ;
		float		mWeight ;
		Boolean		mActive ;
} ;

class	TConstraints { 
	public:		
		TConstraints() { mBonds = nil ; mBumpingBonds = nil ; mStochiometry = nil ; mCellContent = nil ; mCoordination = nil ; } ;
		~TConstraints() { 
			if (mBonds != nil ) delete mBonds ;
			if (mBumpingBonds != nil ) delete mBumpingBonds ;
			if (mStochiometry != nil ) delete mStochiometry ;
			if (mCellContent != nil ) delete mCellContent ;
			if (mCoordination != nil ) delete mCoordination ;
		}
		TBonds			*mBonds ;
		TBonds			*mBumpingBonds ;
		TStochiometry	*mStochiometry ;
		TCoordination	*mCoordination ;
		TCellContent		*mCellContent ;
} ;
		
class	TBondList : public TList {
	public:
		TBondList(Rect	*theListRect) ;
		friend	class TConstraintsDialog ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TBumpingList : public TList {
	public:
		TBumpingList(Rect	*theListRect) ;
		friend	class TConstraintsDialog ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TStochiometryList : public TList {
	public:
		TStochiometryList(Rect	*theListRect) ;
		friend	class TConstraintsDialog ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TCoordinationList : public TList {
	public:
		TCoordinationList(Rect	*theListRect) ;
		friend	class TConstraintsDialog ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TCellContentList : public TList {
	public:
		TCellContentList(Rect	*theListRect) ;
		friend	class TConstraintsDialog ;
	protected:
		void 	DrawCellValue(ListCellT &inCell,Rect &cellFrame) ;
		void 	ClickCell(ListCellT &inCell) ;
		void 	DrawColLabel(short colNumber) ;
} ;

class	TConstraintsDialog : public TDialog {
	public:
		TConstraintsDialog(TAtomPeakList *atomList,TConstraints *theConstraints) ;
		void	GetConstraints(TConstraints *theConstraints) ;
		void	DoMouseDownInObject(TObject *obj,Point *pt) ;
		
		void	EnableAddBond() { mAddBond->Enable() ; } ;
		void	DisableAddBond() { mAddBond->Disable() ; } ;		
		void	EnableDeleteBond() { mDeleteBond->Enable() ; } ;
		void	DisableDeleteBond() { mDeleteBond->Disable() ; } ;		
		void	EnableModifyBond() { mModifyBond->Enable() ; } ;
		void	DisableModifyBond() { mModifyBond->Disable() ; } ;
		void	ShowBond(BondRecord &theBondRec) ;

		void	EnableAddBumping() { mAddBumping->Enable() ; } ;
		void	DisableAddBumping() { mAddBumping->Disable() ; } ;		
		void	EnableDeleteBumping() { mDeleteBumping->Enable() ; } ;
		void	DisableDeleteBumping() { mDeleteBumping->Disable() ; } ;		
		void	EnableModifyBumping() { mModifyBumping->Enable() ; } ;
		void	DisableModifyBumping() { mModifyBumping->Disable() ; } ;
		void	ShowBumping(BondRecord &theBumpingRec) ;

		void	EnableAddStochiometry() { mAddStochiometry->Enable() ; } ;
		void	DisableAddStochiometry() { mAddStochiometry->Disable() ; } ;		
		void	EnableDeleteStochiometry() { mDeleteStochiometry->Enable() ; } ;
		void	DisableDeleteStochiometry() { mDeleteStochiometry->Disable() ; } ;		
		void	EnableModifyStochiometry() { mModifyStochiometry->Enable() ; } ;
		void	DisableModifyStochiometry() { mModifyStochiometry->Disable() ; } ;
		void	ShowStochiometry(StochiometryRecord &theStochiometryRecord) ;

		void	EnableAddCoordination() { mAddCoordination->Enable() ; } ;
		void	DisableAddCoordination() { mAddCoordination->Disable() ; } ;		
		void	EnableDeleteCoordination() { mDeleteCoordination->Enable() ; } ;
		void	DisableDeleteCoordination() { mDeleteCoordination->Disable() ; } ;		
		void	EnableModifyCoordination() { mModifyCoordination->Enable() ; } ;
		void	DisableModifyCoordination() { mModifyCoordination->Disable() ; } ;
		void	ShowCoordination(CoordinationRecord &theCCoordinationRecord) ;

		void	EnableAddCellContent() { mAddCellContent->Enable() ; } ;
		void	DisableAddCellContent() { mAddCellContent->Disable() ; } ;		
		void	EnableDeleteCellContent() { mDeleteCellContent->Enable() ; } ;
		void	DisableDeleteCellContent() { mDeleteCellContent->Disable() ; } ;		
		void	EnableModifyCellContent() { mModifyCellContent->Enable() ; } ;
		void	DisableModifyCellContent() { mModifyCellContent->Disable() ; } ;
		void	ShowCellContent(CellContentRecord &theCellContentRecord) ;
		
		Boolean	HasFirstBondSelection() { return mBondAtom1->GetSelectedRow() > 0 ; } ;
		Boolean	HasSecondBondSelection() { return mBondAtom2->GetSelectedRow() > 0 ; } ;

		Boolean	HasFirstBumpingSelection() { return mBumpingAtom1->GetSelectedRow() > 0 ; } ;
		Boolean	HasSecondBumpingSelection() { return mBumpingAtom2->GetSelectedRow() > 0 ; } ;

		Boolean	HasStochiometrySelection() { return mStochiometryAtom->GetSelectedRow() > 0 ; } ;
		Boolean	HasCoordinationSelection() { return mCoordinationAtom->GetSelectedRow() > 0 ; } ;
		Boolean	HasContentSelection() { return mCellContentAtom->GetSelectedRow() > 0 ; } ;
		
	protected:
		TList	*mBondAtom1 ;
		TList	*mBondAtom2 ;
		TBondList		*mBondList ;
		TButton	*mAddBond ;
		TButton	*mDeleteBond ;
		TButton	*mModifyBond ;
		TEditText	*mBondDistance ;
		TEditText	*mBondWeight ;
		TCheckBox	*mBondActive ;

		TList	*mBumpingAtom1 ;
		TList	*mBumpingAtom2 ;
		TBumpingList		*mBumpingList ;
		TButton	*mAddBumping ;
		TButton	*mDeleteBumping ;
		TButton	*mModifyBumping ;
		TEditText	*mBumpingDistance ;
		TEditText	*mBumpingWeight ;
		TCheckBox	*mBumpingActive ;

		TList	*mStochiometryAtom ;
		TStochiometryList		*mStochiometryList ;
		TButton	*mAddStochiometry ;
		TButton	*mDeleteStochiometry ;
		TButton	*mModifyStochiometry ;
		TEditText	*mStochiometryValence ;
		TEditText	*mStochiometryBase ;
		TEditText	*mStochiometryWeight ;
		TCheckBox	*mStochiometryActive ;

		TList	*mCoordinationAtom ;
		TCoordinationList		*mCoordinationList ;
		TButton	*mAddCoordination ;
		TButton	*mDeleteCoordination ;
		TButton	*mModifyCoordination ;
		TEditText	*mCoordinationMin ;
		TEditText	*mCoordinationMax ;
		TEditText	*mCoordinationWeight ;
		TCheckBox	*mCoordinationActive ;

		TList	*mCellContentAtom ;
		TCellContentList		*mCellContentList ;
		TButton	*mAddCellContent ;
		TButton	*mDeleteCellContent ;
		TButton	*mModifyCellContent ;
		TEditText	*mCellContentMin ;
		TEditText	*mCellContentMax ;
		TEditText	*mCellContentWeight ;
		TCheckBox	*mCellContentActive ;
		
} ;

class TPeaksControlWindow : public TWindow {
	public:
		TPeaksControlWindow( FS98ControlRecord &theFS98ControlRecord, TImage *image,short whichImage) ;
		~TPeaksControlWindow() ;
		
		void	Draw() ;
		void	DoMouseDownInObject(TObject *obj,Point *pt) ;
		void	DoWMRequestDelete() ;
		
		void	OnCommandHelp() { EDMHelp("TPeaksControlWindow") ; } ;
		
	protected:
	
		friend		class   TPeakSolutionList ;
		friend		class   TPeakDisplayList ;
				
		void				SetAtomList() ;
		void				SetDifferenceAtomList() ;
		void				SetDiffractionList() ;
		void				ReadHKLDataSet() ;
		void				ReadCalculatedHKLDataSet() ;
		void				CalculateCurrentModel() ;
		void				CalculateCurrentMap() ;
		void				CalculateDifferenceMap(PeakCommand cmnd) ;
		void				CalculateExpDiffractionPattern() ;
		void				CalculateCurrentDiffractionPattern() ;
		void				CalculateSplitDiffractionPattern() ;
		void				CreateDisplayAtoms() ;
		void				CreateDisplayDifferencePeaks() ;
		void				SetSymmetryRelatedPeaks() ;
		Boolean				Equivalent(float x,float y) ;
		void				ConvertToImagePos(FPoint &screenPos) ;
		TAtomPeak			*FindAtomPeak(FPoint &imagePos) ;
		float				GetGoodnessOfFit() ;
		void				UpdateCurrentSolution(float gof) ;
		
		Boolean				WriteConstraintsFile() ;
		
		void				FindUnitCellPosition(FPoint &screenPos,FPoint &atomPos) ;
		void				WriteResult() ;
		
		void				DisablePeakCommands() ;
		void				EnablePeakCommands() ;
		void				RestartPeaks() ;
		void				SetOccupancySolutions() ;
		void				ReportOutput() ;
		void				UpdateAtomList(TAtomPeakList *theAtomList) ;
		void				SetSelectedSolutionAsCurrent(short selectedSolution) ;
		Boolean				WriteAtomsToTestFile() ;
		void				ReadAtomTypes() ;
	
		void				PositionImage() ;
		void				DoMouseDownInImage(Point &loc) ;
		void				SendPeaksCommand(PeakCommand command,float value = 0.) ;
		void				SendPeaksTextCommand(char *commandText) ;
		Boolean				WritePeakCommands(PeakCommand command,float value = 0.) ;
		short				PeaksReadyToReceiveCommands() ;
		void				TellPeaksToReceiveCommands() { RemoveFile(".edm_2") ;  CreateFile(".edm_1") ; } ;
		void				WaitForPeaksToCompleteTask() { short stat ; while((stat = PeaksReadyToReceiveCommands()) == 0) { wait(200) ; } if (stat == -1) mPeaksIsRunning = false ; } ;
		TImage				*mCurrentImage ;
		TImage				*mImageMap ;
		TImage				*mCurrentModel ;
		TImage				*mCurrentMap ;
		TImage				*mDifferenceMap ;
		TImage				*mExpDP ;
		TImage				*mCurrentDP ;
		TImage				*mSplitDP ;
		TImage				*mGaussian ;
			
		TRadioButton		*mShowGeneticSolutionMap ;
		TRadioButton		*mShowCurrentModel ;
		TRadioButton		*mShowCurrentMap ; 
		TRadioButton		*mShowDifferenceMap ;
		TRadioButton		*mShowExperimentalDP ;
		TRadioButton		*mShowCalculatedDP ;
		TRadioButton		*mShowFourierFit ;
		TRadioButton		*mSelectedDisplay ;

		TRadioButton		*mShowCurrentPeakList ;
		TRadioButton		*mShowCurrentDifferenceList ;

		TPushButton			*mAddAtomPeak ;
		TPushButton			*mRemoveAtomPeak ;
		TPushButton			*mPointerTool ;
		TPushButton			*mZoomTool ;
		TPushButton			*mHandTool ;
		TPushButton		    *mRulerTool ;
		TPushButton			*mCurrentTool ;
		
		Boolean				mIsInsideActiveRect ;
		Point				mImageCenter ;
		float				mImageMagnification ;
		
		short				mImageNum ;
		
		PeakDisplayMode		mDisplayMode ;
		TCheckBox			*mOverlayAtoms ;
		TCheckBox			*mShowLog ;

		TCheckBox			*mAcceptBestSolution ;
		TCheckBox			*mUseAtomList ;
		
		TEditText			*mFFTGmax ;
		TButton				*mFFTGmaxApply ;

		TEditText			*mFFTDiskRadius ;
		TButton				*mFFTDiskRadiusApply ;

		TEditText			*mPeakThreshold ;
		TButton				*mChangeThreshold ;
		TButton				*mTestPeaks ;
		TButton				*mResetView ;
		
		TButton				*mCreateModelFromPeaks ;
		TButton				*mUpdateCurrentMap ;
		TButton				*mCommandUpdateAtoms ;
		TButton				*mCommandListAtoms ;
		TButton				*mCommandEntropy ;
		TButton				*mCommandPositive ;
		TButton				*mCommandRefine ;
		TButton				*mCommandOccupancy ;
		TButton				*mCommandFix ;
		//TButton				*mCommandGOF ;
		TButton				*mCommandSEED ;
		TButton				*mCommandDiffMap ;
		TButton				*mCommandDiffProj ;
		TButton				*mCommandDiffMapOne ;
		TButton				*mCommandTrim ;
		TButton				*mCommandShowCurrentSettings ;
		TButton				*mCommandSettoModel ;
		TButton				*mCommandPurgeSeeds ;
		TButton				*mCommandReadAtoms ;
		TButton				*mCommandCyclePeaks ;
		TButton				*mBabinet ;
		
		TButton				*mCommandSetConstraints ;
		TRuler				*mRuler ;

		TCheckBox			*mOptionRobust ;
		TCheckBox			*mOptionErrors ;
		TCheckBox           *mOptionUseFHKL ;
        TCheckBox           *mOptionICons ;
        TCheckBox           *mOptionMultiplicity ;
		TCheckBox			*mOptionDWRef ;
		TCheckBox			*mOptionUcmin ;
		TCheckBox		    *mOptionTwin ;
		
		TPeakDisplayList	*mPeakDisplayList ;
		TPeakDisplayList	*mDifferencePeakDisplayList ;
		THKLDisplayList		*mHKLDisplayList ;
		TPeakSolutionList   *mSolutionList ;
		
		TPeakLogList		*mLogList ;
		
		TAtomPeakList		*mAtomPeakList ;
		TAtomPeakList		*mDifferencePeakList ;
		TAtomPeakList		*mAtomTypes ;
		short				mNumSolutions ;
		
		TUserRect			*mImageFrame ;
		
		Rect				mListRect ;
		Rect				mSolutionRect ;
		Rect				mImageRect ;
		Rect				mActualImageRect ;
		void				DisplayMap() ;
		float				mScaleFactor ;
		TButton				*mQuit ;
		
		short				mMapWidth ;
		short				mMapHeight ;
		
		short			mPlaneGroupIndex ;
		
		long			mNumReflections ;
		float			*mAmplitude ;
		float			*mAmplitudeError ;
		float			*mPhase ;
		short			*mH ;
		short			*mK ;
		short			*mL ;

		// I don't know yet if these should be calculated or read from a file
		long			mNumCalculatedReflections ;
		float			*mCalcAmplitude ;
		float			*mCalcDifference ;
		short			*mCalcH ;
		short			*mCalcK ;
		short			*mCalcL ;
		
		TConstraints	*mConstraints ;
		
		HKLCalculatedFileType	mHKLCalculatedFileType ;
		
		float			mGMax ;
		float			mDiskRadius ;
		bool			mPeaksIsRunning ;
		bool			mLogDiagnosticOutput ;
		
		TStaticText		*mInfo;
		void			DrawInfoWindow() ;
		void			WriteMessage(char *text) ;
		Str256			mMessageText ;

		
		FS98ControlRecord   mFS98ControlRecord ;

} ;


#endif


