#ifndef __TRefinementSet
#define __TRefinementSet
#include "TArray.h"
#include "TImage.h"
#include "FS98ControlRecord.h"


class TDiffractionDataTable ;
class TCrystalDiffractionDataTable ;
class TTable ;

typedef enum {kDiffractionDataTable,kCrystalDiffractionDataTable} TableType ;

// We never have more than 4 equivalent reflections in the half-plane
class TRefinementReflection {
	public:
		TRefinementReflection() : mNumEquivalentReflections(0) {} ;
		float	mAmplitude ;
		float	mPhase[6] ;
		long	mPhaseFlag[6] ;
		short	mNumEquivalentReflections ;
		short	mH[6] ;
		short	mK[6] ;
		bool	mIncludeInRefinement ;
		bool	mUsePhase ;
} ;

class	TRefinementSet {
	public:
		TRefinementSet(TDiffractionDataTable *diffractionDataTable) ;
		TRefinementSet(TCrystalDiffractionDataTable *diffractionDataTable) ;
		TRefinementSet(TRefinementSet *set) ;
		~TRefinementSet() ;
		
		TRefinementReflection	*mRefinementReflection ;
		long	SetSymmetryRelatedPhaseFlag(short h,short k,short h1,short k1 ) ;
		
		TArray	*RunGerschbergSaxtonIteration() ;
		void	RunFS98() ;
		
		TTable		*GetDataTable() { return mDataTable ; } ;
		TableType	GetTableType() { return mTableType ; } ;
		
		void	RemoveTable() { mDataTable = nil ; } ;

		void	RunPhaseExtension(FS98ControlRecord &theFS98ControlRecord) ;
		void	RunConvergence(FS98ControlRecord &theFS98ControlRecord) ;
		void	RunGenetic(FS98ControlRecord &theFS98ControlRecord) ;
		void	ShowSolutions(FS98ControlRecord &theFS98ControlRecord) ;
		void	RunPeaks(FS98ControlRecord &theFS98ControlRecord) ;
		
		short	mPlaneGroup ;
		float	mA ;
		float	mB ;
		float	mGamma ;
		float	mUmax ;
		long	mImageWidth ;
		long	mImageHeight ;
		CalibrationUnit	mCalibrationUnit ;
		
		long	mNumReflections ;
		long	mNumIndependentReflections ;
		
		FS98ControlRecord	mFS98ControlRecord ;
		
		TTable	*mDataTable ;
		TableType	mTableType ;
} ;

class	TDMIteration {
	public:
		TDMIteration(TRefinementSet *refinementSet) ;
		~TDMIteration() ;
		
		void			InverseFourierTransform() ;
		void			FourierTransform() ;
		
		virtual	float	RandomPhase(short h,short k ) ;
		virtual	float	SymmetryRelatedPhase(float phase, short h,short k,short h1,short k1 ) ;
		
		virtual	void	ImposeReciprocalSpaceConstraints() ;
		virtual	void	ImposeRealSpaceConstraints() ;
		virtual	void	ImposeSupportConstraints() ;

		virtual	float	CalculateFigureOfMerit() ;
		
		virtual	void	SetSymmetrizedAmplitudePhase(long h,long k) ;
		virtual	float	SetSymmetrizedPhase(long i) ;
		virtual	void	IterationCycle() ;
		virtual	void	AssignAmplitudesAndRandomPhases() ;
		
		TRefinementSet	*mRefinementSet ;
		TArray			*mImageArray ;
		TArray			*mReflectionIndex ;

} ;
#endif
