#ifndef __TWindow_h
#define __TWindow_h

#include <forms.h>

#ifdef VMS
	#include <decw$include/Xlib.h>
	#include <decw$include/Xutil.h>
#else
	#include <X11/Xlib.h>
	#include <X11/Xutil.h>
#endif

#include "TButton.h"
#include "DMMenuCommands.h"

#ifndef __TCommand_h
	#define	__TCommand_h
	typedef unsigned long	TCommand ;
#endif


Boolean	PtInRect(Point *,Rect *) ;

class	TWindow ;
class 	TWindowList ;
class	TPaneList ;
extern	Display 	*gDisplay ;

typedef enum { kNormalWindow, kNonModalDialog, kModalDialog , kFloatingWindow } WindowType ; 
typedef unsigned long	WindowKind ;
typedef unsigned long	WindowContent ;

extern	void	SetTargetWindow(TWindow *) ;
inline	void	BringToFront(TWindow *) ;
extern	void 	SetFocus(TPane *pane) ;
extern	TWindow	*gFrontWindow ;

extern	void	EDMHelp(char *) ;

class	TGrafPort {
	public:
		TGrafPort(short width,short height,short depth = 32) ;
		TGrafPort() ;
		TGrafPort(FL_OBJECT *object) ;	// Create an instance around an xform object
		virtual	~TGrafPort() ;

		Drawable	mDrawable ;
    	GC			mGC ;
    	Rect		mPortRect ;
    	uLong		mFgColor ;
    	uLong		mBkColor ;
    	short		mDepth ;

		RgnHandle	mClipRegion ;
		XFontStruct	*mFont ;
		
		void		SetPortOrigin(FPoint &origin) { mPortOrigin = origin ; } ;
		FPoint		&GetPortOrigin()  { return mPortOrigin ; } ;

		void 		ImageToLocalPoint(Point &inImagePt,Point &outLocalPt) ;
		void 		LocalToImagePoint(Point &inImagePt,Point &outLocalPt) ;

		Drawable 	GetDrawable() { return mDrawable ; } ;
		XFontStruct	*GetFont() { return mFont ; } ;
		void		SetFont(XFontStruct	*font) { 
						mFont = font ; XSetFont(gDisplay,mGC,mFont->fid) ;} ;
		RgnHandle	GetClipRegion() { return mClipRegion ; } ;
		void		SetClipRegion(RgnHandle rgn) ;
		void		SetClipRegion(Rect &rect) ;

		Rect	&GetPortRect() { return mPortRect ; } ;
		GC		GetGC() { return mGC ; } ;
		uLong	GetFgColor() { return mFgColor ; } ;
		uLong	GetBkColor() { return mBkColor ; } ;
		void	SetFgColor(uLong ) ;
		void	SetBkColor(uLong ) ;
		short	GetDepth() { return mDepth ; } ;

		FPoint			mPortOrigin ;
} ;

class TOffscreen : public TGrafPort {
	public:
		TOffscreen(class TGrafPort *,Rect *rect = nil) ;
		~TOffscreen() ;
		Pixmap  GetPixmap(void) { return mPixmap ; } ;

		// It is essential that the order of the members are as follows
		// so that a TOffscreen and TWindow will draw appropriately	
	protected:
		friend	class TWindow ;	
		Pixmap 	mPixmap ;
} ;

class TStOffscreen : public TGrafPort {
	public:
		TStOffscreen(class TGrafPort *,Rect *rect = nil) ;
		~TStOffscreen() ;
		
		Pixmap  	GetPixmap(void) { return mPixmap ; } ;

	protected:
		friend	class TWindow ;	
		
		Pixmap 		mPixmap ;
		TGrafPort	*mCurrentPort ;
		Rect		mRect ;
		Boolean		mHasLocalGC ;
} ;

class TWindow ;

class	TPane : public TGrafPort {
	public:
		TPane() {} ;
		TPane(TPane *parent,short xloc,short yloc,short width,short height) ; 
		TPane(TPane *parent,Rect *rect) ;	// 

		void			InvalPortRgn(Region inUpdateRgn) ;
		inline 			Region	GetLocalUpdateRgn() { return mUpdateRegion ; } ;
		
		virtual			~TPane() ;
		Window  		GetXWindow() { return mXWindow ; } ;
		Boolean			InPane(Point	*thePoint) { return PtInRect(thePoint,&mPaneRect) ; } ;
		Boolean			IsVisible() { return mIsVisible ; } ;
		void			SetVisible(bool isVisible) { mIsVisible = isVisible ; } ;
		virtual void	Draw() {mIsVisible=true ; if(mObjectList!=nil)mObjectList->DrawObjects();} ;
		virtual void	Refresh() { Update(nil) ; } ;
		virtual	void	Show() { XMapWindow(gDisplay,mXWindow) ; XRaiseWindow(gDisplay,mXWindow) ; mIsVisible = true ;} ;
		virtual	void	Hide() { XUnmapWindow(gDisplay,mXWindow) ;  mIsVisible = false ;} ;
		void			AddObject(TObject *obj){if (mObjectList==nil) mObjectList= new TObjectList;mObjectList->AddObject(obj);obj->SetParentWindow(this);};
		void			RemoveObject(TObject *obj){mObjectList->RemoveObject(obj);};
		void			AddTextEditObject(TEditText *obj) ;
		virtual void	Update(XEvent *event) ;
		WindowKind		GetWindowKind() { return mWindowKind ; } ;
		virtual	void	DoMouseDown(XEvent *) ;
		virtual	void	DoKeyDown(XEvent *) ;
		virtual	void	DoKeyUp(XEvent *) {} ;
		virtual	void	DoMouseDownInObject(TObject *obj,Point *pt) {obj->DoMouseDown(pt) ;} ;
		virtual	void	DoKeyDownInObject(TObject *) {} ;
		void			SetParent(TPane *parent) { mParentPane = parent ; } ;
		TPane			*GetParent() { return mParentPane ; } ;
		virtual void	Select(TEditText *theText) ;
		virtual void	SelectAll(TEditText *theText) ;
		TObject			*FindObject(Point *pt) { return mObjectList->GetObject(pt) ; } ;
		
		void			DoEnterNotify(XCrossingEvent *event)  ;
		void			DoLeaveNotify(XCrossingEvent *event) ;

		virtual void	SetCursor() {} ;
		virtual	void	UnsetCursor() {} ;
		
		Rect			&GetRect() { return mPaneRect ; } ;


	protected:

		void			ScrollBits(Int16 inLeftDelta,Int16 inTopDelta) ;

		Time			mCrossingInEventTime ;
		Time			mCrossingOutEventTime ;
		Boolean			mTakeFocus ;

		long			mClickTime ;
		long			mLastClick ;
	
		double			mVScrollPos ;
		double			mHScrollPos ;

		virtual	void	AdjustScrollBars() {} ;

		TScrollBar		*mHorizScrollBar ;
		TScrollBar		*mVertScrollBar ;

		Rect			mCanvasRect ;	// This is the drawable area which depends on if we have scroll bars or not
		Boolean			mPointerInsideWindow ;
		Window			mXWindow ;
		TPane 			*mParentPane ;
		Rect			mPaneRect ;
		Region			mUpdateRegion ;
		Rect			mUpdateRect;
		TObjectList		*mObjectList ;
		TEditTextList	*mEditTextList ;
		TEditText		*mCurrentTextEditField ;
		Boolean			mIsVisible ; 
		WindowKind		mWindowKind ;
} ;

class TPopUpWindow : public TGrafPort {
	public :
		TPopUpWindow(TPane *parent,short xloc,short yloc,short width,short height) ;
		virtual	~TPopUpWindow() ;

		Window  GetXWindow(void) { return mXWindow ; } ;
		virtual	void	Show() { XMapRaised(gDisplay,mXWindow);} ;
		virtual	void	Hide() { XUnmapWindow(gDisplay,mXWindow) ;} ;
		virtual	void	Draw() {} ;
		TPane	*GetParent() { return mParentPane ; } ;
		Boolean	mParentIsRoot ;

	protected:
		Window	mXWindow ;
		TPane 	*mParentPane ;
} ;

typedef TWindow TCommander ; 

class TWindow : public TPane {
	public:
		TWindow() ;
		TWindow(short xloc,short yloc,short width,short height,
			short depth,char *title,Boolean hasOffscreen,Boolean resizable = false) ;
		virtual	~TWindow() ;

		Boolean 		CanReceiveMenuCommands() { return mCanReceiveMenuCommands ; } ;
		virtual	void	SetAsTarget(Boolean isTarget) {
								if(mCanReceiveMenuCommands) {
									mIsTarget = isTarget ;
									if (mIsTarget && mCommander != nil) 
										mCommander->ActivateListener(this) ; 
								}  
							} ; 
		Boolean			IsTarget() { if (mCanReceiveMenuCommands) return mIsTarget ; return false ; } ;


		WindowType		GetWindowType() { return mWindowType ; } ;
		WindowContent	GetWindowContent() { return mWindowContent ; } ;
		void			SetWindowContent(WindowContent content) {mWindowContent = content ; } ;
		void			SetWindowType(WindowType type) {mWindowType = type ; } ;

		void			SetCommander(TCommander *commander) {mCommander = commander; if (mCommander != nil) mCommander->ActivateListener(this) ;} ;
		TCommander		*GetCommander() { return mCommander ; } ;
		void			SetWindowTitle(char *title);
		void			GetWindowTitle(char *title) ;
		char			*GetWindowTitle() { return mWindowTitle ; } ;
		void			SetWindowList(TWindowList *list) { mWindowList = list ; } ;
		void			Show() ;
		TOffscreen 		*GetOffscreen() { return mOffscreen ; } ;
		
		void			ResetPanAndZoom() { mMagnification = 1. ; mPortOrigin.x = 0. ; mPortOrigin.y = 0. ; Update(nil) ; } ;
		float			GetMagnification() { return mMagnification ; } ;

		void			WindowToImagePoint(Point &thePoint) ;
		void			ImageToWindowPoint(Point &thePoint) ;
		void			WindowToImageRect(Rect &theRect) ;
		void			ImageToWindowRect(Rect &theRect) ;
		void			WindowToImagePoint(FPoint &thePoint) ;
		void			ImageToWindowPoint(FPoint &thePoint) ;
		void			WindowToImageRect(FRect &theRect) ;
		void			ImageToWindowRect(FRect &theRect) ;

		void			DoFocusIn(XFocusChangeEvent *theEvent) ;				
		void			DoFocusOut(XFocusChangeEvent *theEvent) ;				

		virtual void	Update(XEvent *event = nil) ;
		void			AddPane(TPane *pane) ; 
		void			DeletePane(TPane *pane) ; 
		
		void			DoMapNotify(XEvent *event) ;
		void			DoUnmapNotify(XEvent *event) ;

		virtual void	ActivateListener(TWindow *) {} ;
		virtual	void	SetControls() {} ;
		virtual	void	DoMouseDown(XEvent *event) ;
		virtual	void	DoKeyDown(XEvent *event) ;
		virtual	void	DoKeyUp(XEvent *event) ;
		virtual void	DoConfigureNotify(XEvent *event) ;
		virtual void	ResizeWindow(long width,long height) ;
		virtual	void	DoIdle() ;
		virtual void	SelectCursor() ;
		virtual	void	DoClientMessage(XClientMessageEvent *event) ;
		virtual	void	Close() ;
		virtual	void	DoWMRequestDelete() ;
		virtual	void	DoCommand(TCommand command,Boolean *hasBeenHandled) ;
		virtual	void	GetCommandStatus(TCommand,Boolean *isEnabled,
							Boolean *isChecked);
		virtual	void	OnCommandHelp() { EDMHelp((char *)"TWindow") ; } ;
		
		Boolean	IsValid() { return mIsValid ; } ;

		friend	void	EraseWindow(TWindow *window) ;
		
	protected:

		TPaneList		*mPaneList ;
		virtual void 	Draw() ;
		virtual	void	Print() {} ;
		virtual void	Erase(Rect *rect = nil) ;
		
		long			mXPosition ;
		long			mYPosition ;
		
		Boolean			mIsValid ;
		
		TOffscreen		*mOffscreen ;			
		TWindowList		*mWindowList ;

		Boolean			mIsTarget ;
		Boolean			mCanReceiveMenuCommands ;
		
		Boolean			mHasFocus ;
		Time			mFocusEventTime ;

		TCommander 		*mCommander ;
		
		WindowType		mWindowType ;
		WindowContent	mWindowContent ;
		char			mWindowTitle[256] ;
						
		float			mMagnification ;
} ;

class	TFrameWindow : public TWindow {
	public:
		TFrameWindow(short xloc,short yloc,short width,short height,
			short depth,char *title) ;
		virtual			~TFrameWindow() ;
		virtual void	ActivateListener(TWindow *) {} ;
		virtual	void	Draw() ;
		virtual	void	DoMouseDown(XEvent *event) ;
		virtual	void	DoKeyDown(XEvent *event) {} ;
		virtual	void	DoKeyUp(XEvent *event) {} ;

	protected:
		Boolean		mHasMenuBar ;
		
} ;

typedef	TGrafPort	*GrafPtr ;
typedef TWindow 	*WindowPtr ;
typedef TOffscreen	*OffScreenPtr ;

extern	void SetFocus(TPane *pane) ;

extern	TWindow	*gFrontWindow ;

inline void BringToFront(TWindow *window) { XMapRaised(gDisplay,window->GetXWindow()) ; gFrontWindow = window ; } ;
inline void EraseWindow(TWindow *window) { window->Erase() ; }  ;

#endif
