// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------


#include <stdio.h>
#include <math.h>
#include "XDrivers.h"
#include "drivers.h"
#include "graphics.h"
#include "char_symbol.h"
#include "header.h"
#include "devices.h"
#include "types.h"
#include "globals.h"
LineRecord LineRc ;
extern CharRecord CharRc ;
extern FILE *gPSFile ;
extern	long near(float) ;
/* define the data structure for the different coordiante systems */
struct FRAMES
       {
        float xmin ;
        float xmax ;
        float ymin ;
        float ymax ;
       } world,window,view ;
/* Declare transformation constants between coordinate systems */

float a_wrld_ndc,b_wrld_ndc,c_wrld_ndc,d_wrld_ndc ;
/* ---------------------- Define constants ----------------------------- */
int INSTRUCT = FALSE ;         /* Flag to tell if the next symbol is an  */
                               /* instruction for decodeing text         */
int SHIFT = FALSE ;            /* Flag to tell if we need to shift text  */
int THICK_CURVE = FALSE ;      /* Flag to indicate thick curve           */
int XPDC_START,YPDC_START ;    /* Starting points in Phys. Device coord. */
int SHIFT_SYMBOL ;             /* Symbol to use in shifting text         */
char  MARKER_SYMBOL ;          /* Symbol to use as marker                */
char INSTRUCTION_SYMBOL ;      /* Symbol to use to inducate an instruct. */
float matr[2][2] ;
char CGS = GS, CUS = US, CESC = ESC ;
/*-----------------------------------------------------------------------*/
void set_ctm(float size,float angle) {
  /* ----------------------------------------------------*/
  /* This routine sets the Coordinate Transformation     */
  /* Matrix to be used                                   */
  /* ----------------------------------------------------*/
    int i,j,k ;
    static float rot[2][2]={1.,0.,0.,1.},
                 scale[2][2]={1.,0.,0.,1.} ;

    angle = angle*3.14159/180. ;
    rot[0][0] = cos(angle) ;
    rot[0][1] = -sin(angle) ;
    rot[1][0] = -rot[0][1] ;
    rot[1][1] = rot[0][0] ;
    scale[0][0] = size ;
    scale[1][1] = size ;
    for (i=0;i<2;i++)
      for (j=0;j<2;j++)
      {
        matr[i][j] = 0. ;
        for (k=0;k<2;k++)
          matr[i][j] = matr[i][j] + rot[i][k]*scale[k][j] ;  
      }
}

void ctm(int *x,int *y) {
  /*-------------------------------------------------------------*/
  /* Apply the coordinate transformation matrix to the point x,y */
  /* all units are in pdc - units , physical device coord.       */
  /*-------------------------------------------------------------*/
    int ix,iy ;
    ix = *x ; iy = *y ;
    *x = near(matr[0][0]*ix + matr[0][1]*iy) ;
    *y = near(matr[1][0]*ix + matr[1][1]*iy) ;
}

void eushft(char symbol) {
  /*--------------------------------------------------*/
  /* This routine sets the symbol to indicate shift   */
  /* in text position and sets the SHIFT flag         */
  /*--------------------------------------------------*/

    SHIFT = TRUE ;
    SHIFT_SYMBOL = symbol ;
}

void instrct(char symbol) {
  /*--------------------------------------------------*/
  /* This routine sets the symbol to indicate an      */
  /* instruction and sets the INSTRUCT flag           */
  /*--------------------------------------------------*/
    INSTRUCT = TRUE ;
    INSTRUCTION_SYMBOL = symbol ;
}

void decode_instruction(char *(*cp),int *y_offset,float old_size,float *new_size) {
  /*--------------------------------------------------------*/
  /* Routine to decode an instruction in a text string      */
  /*--------------------------------------------------------*/

    char c ;

    c = *(*cp) ;
    if (c == 'e' || c == 'E')         /* Raise the text */
      *y_offset = 8 ;
    else if (c == 'L' || c == 'l')    /* Lower the text */
      *y_offset = -3 ;
    c = *(++(*cp)) ;
    if (c == 'x' || c == 'X')         /* Reset the text */
      {
        *y_offset = 0 ;
        c = *(++(*cp)) ;
      }
    if (c == 'h' || c == 'H')         /* Change the size of the text */
      {
        c = *(++(*cp)) ;              /* Change back to old size     */
        if (c == 'x' || c == 'X')
           *new_size = old_size ;
        else
          {
            c = *(++(*cp)) ;          /* Change to new character size */
             *new_size = (int)(c-'0')/10.*old_size ;
          }
       }
     c = *(++(*cp)) ;
}

void messag(float x_ndc,float y_ndc,float size,float angle,char *text) {
  /*--------------------------------------------------------*/
  /* write a string at x,y in normalized device coordinates */
  /*--------------------------------------------------------*/
    write_legend(x_ndc,y_ndc,size,angle,text) ;
}

void write_legend(float x_ndc,float y_ndc,float size,float angle,char *text) {
  /*-----------------------------------------------------------*/
  /* This routine actually write the text on the display after */
  /* transforming the coordinates using the current CTM        */
  /*-----------------------------------------------------------*/
    int x,y ;  /* physical device coordinates */
    int j,i,x_offset,y_offset,old_y_offset,END ;
    int ix,iy,n ;
    char *ptext ;
    float new_size ;
  
    x = x_ndc_pdc(x_ndc) ;
    y = y_ndc_pdc(y_ndc) ;
    if (gOutDevice->type == LP) {
    	y = gOutDevice->yres - 1 - y ;
    }
    set_ctm(size,angle) ; 
    y_offset = 0 ;
    ptext = text ;
    END = FALSE ;
    switch(gOutDevice->type)
	{
	case WS:
    		while(*ptext != '\0')
      			{
        		if (INSTRUCT && *ptext == INSTRUCTION_SYMBOL)
	          		{
            			ptext++ ;
            			decode_instruction(&ptext,&y_offset,size,
						&new_size) ;
			        set_ctm(new_size,angle) ;
            			if (*ptext == '\0') break ;
          			}
        		if (SHIFT && *ptext == SHIFT_SYMBOL)
          			{
            			ptext++ ;
            			n = *ptext - ' ' + 62 ;
          			}
        		else
            			n = *ptext - ' ' ;

        		old_y_offset = y_offset ;
        		if (ascii_char[n].descend) y_offset -= 3 ;
        		for (i=0;i<ascii_char[n].width;i++)
          			for (j=0;j<9;j++) 
            				if (ascii_char[n].pixel[i][j]) 
              					{
                				ix = i ; iy = j+y_offset ;
                				ctm(&ix,&iy) ;
                				gDrawPixel(x+ix,y-iy) ;
              					}
        		x_offset = (n == 0 ? 8 : ascii_char[n].width+2) ;
        		y_offset = 0 ;
        		ctm(&x_offset,&y_offset) ;
        		x += x_offset ;
        		y -= y_offset ;
        		y_offset = old_y_offset ;
        		ptext++ ;
      			}
		break ;
	case LP:
		/* Save the current graphics state */
		fprintf(gPSFile,"\ngsave") ;
		/* Copy the string into a workspace */
		/* First set the size */
		fprintf(gPSFile,"\n/%s findfont %d scalefont setfont",
			CharRc.fontname,(int)(CharRc.fontsize*size)) ;
		fprintf(gPSFile,"\n%d %d moveto",x,y) ;
		if (angle != 0.)
			fprintf(gPSFile,"\n%f rotate",angle) ;
		fprintf(gPSFile,"\n(") ; /* Start on the string */
    		while(*ptext != '\0')
      			{
        		if (INSTRUCT && *ptext == INSTRUCTION_SYMBOL)
	          		{
				/* Write out what we have so far     */
				/* and set the new size and baseline */
				fprintf(gPSFile,") show") ;
            			ptext++ ;
            			decode_instruction(&ptext,&y_offset,size,
						&new_size) ;
				/* Change the baseline, but save the current */
				if (y_offset !=0)
					old_y_offset = y_offset ;
				else
					y_offset = - old_y_offset ;
			        fprintf(gPSFile,"\n0 %d rmoveto",y_offset) ;
				fprintf(gPSFile,
					"\n/%s findfont %d scalefont setfont",
					CharRc.fontname,
					(int)(CharRc.fontsize*new_size)) ;
            			if (*ptext == '\0') {
					END = TRUE ;
					break ;
				}
				/* Start a new substring */
				fprintf(gPSFile,"\n(") ;
          			}
        		if (SHIFT && *ptext == SHIFT_SYMBOL)
          			{
            			ptext++ ;
            			n = *ptext - 65 ;
				/* Hardcode the choices we have presently */
				switch(n)
					{
					case 0: /* Alpha */
						fprintf(gPSFile,"A") ;
						break ;
					case 1:	/* PI */
						fprintf(gPSFile,"P") ;
						break ;
					case 2: /* delta */
						fprintf(gPSFile,"D") ;
						break ;
					case 3: /* Angstrom */
						fprintf(gPSFile,"A") ;
						break ;
					}
          			}
        		else
				fprintf(gPSFile,"%c",*ptext) ;

			ptext++ ;
			}
		/* Terminate the string */
		if (!END)
			fprintf(gPSFile,") show") ;
		fprintf(gPSFile,"\ngrestore") ;
		break ;
	}
}

void physor(float xo,float yo) {
  /*--------------------------------------------------*/
  /* Routine to set the physical origin of the plot   */
  /* on the viewing surface in relative coordinates   */
  /*--------------------------------------------------*/
    view.xmin = xo ;
    view.ymin = yo ;
}

void area2d(float xlength,float ylength) {
  /*--------------------------------------------------*/
  /* Routine to set the 2-d area for the plot         */
  /* on the viewing surface in relative coordinates   */
  /*--------------------------------------------------*/
    view.xmax = view.xmin + xlength ;
    view.ymax = view.ymin + ylength ;
}

void line(float x1,float y1,float x2,float y2) {
  /*---------------------------------------------------- */
  /* Routine to draw a line in WORLD coordinates         */
  /*---------------------------------------------------- */
    line_abs(x1,y1,x2,y2) ;
}

void set_world(float xmin,float xmax,float ymin,float ymax) {
  /*----------------------------------------------------*/
  /* Routine to set the min and max values of the plot  */
  /* in the world coordinate system                     */
  /*----------------------------------------------------*/
    world.xmin = xmin ;
    world.xmax = xmax ;
    world.ymin = ymin ;
    world.ymax = ymax ;
}

void set_window(float xmin,float xmax,float ymin,float ymax) {
    window.xmin = xmin ;
    window.xmax = xmax ;
    window.ymin = ymin ;
    window.ymax = ymax ;
} 

void set_viewport(float xmin,float xmax,float ymin,float ymax) {
    view.xmin = xmin ;
    view.xmax = xmax ;
    view.ymin = ymin ;
    view.ymax = ymax ;
}

void frame() {
  /*-----------------------------------------------*/
  /* Routine to draw a frame around the plot       */
  /*-----------------------------------------------*/

    float xlow,ylow,xhi,yhi ;

    SetLineWidth(3) ; 
    /* ndc : normalized device coordinates */
    xlow = view.xmin ;
    ylow = view.ymin ;
    xhi  = view.xmax ;
    yhi  = view.ymax ;
    gDrawRect(x_ndc_pdc(xlow),y_ndc_pdc(yhi),x_ndc_pdc(xhi),
	     y_ndc_pdc(ylow)) ;
    SetLineWidth(1) ; 
}

void axes() {
  /*---------------------------------------------*/
  /* Routine to draw the axes of the plot with   */
  /* tickmarks                                   */
  /*---------------------------------------------*/

    line_abs(world.xmin,world.ymin,world.xmax,world.ymin) ;
    /* just drew the x-axis in world coord */
    line_abs(world.xmin,world.ymin,world.xmin,world.ymax) ;
    /* just drew the y-axis in world coord */
    ticks_x(world.xmin,world.xmax) ;               /* put tickmarks */
    ticks_y(world.ymin,world.ymax) ;               /* put tickmarks */
}

void move_abs(float x_world,float y_world) {
  /*-----------------------------------------------------*/
  /* Move to the specified position in world coordinates */
  /*-----------------------------------------------------*/
    float x_ndc,y_ndc ;
  
    x_ndc = x_wrld_ndc(x_world) ;
    y_ndc = y_wrld_ndc(y_world) ;
    move_ndc(x_ndc,y_ndc) ;
}

void line_abs(float x1,float y1,float x2,float y2) {
  /*-----------------------------------------*/
  /* Draw a line in world coordinates        */
  /*-----------------------------------------*/

    x1 = x_wrld_ndc(x1) ;
    x2 = x_wrld_ndc(x2) ;
    y1 = y_wrld_ndc(y1) ;
    y2 = y_wrld_ndc(y2) ;
    line_ndc(x1,y1,x2,y2) ;
}

float x_wrld_ndc(float x_world){
  /*-------------------------------------------------------*/
  /* Transform from world to normalized device coordinates */
  /*-------------------------------------------------------*/

    float x ;

    x_world = MAX(x_world,world.xmin) ;
    x_world = MIN(x_world,world.xmax) ;
    x = a_wrld_ndc*x_world + b_wrld_ndc ;
    return(x) ;
}

float y_wrld_ndc(float y_world) {
  /*-------------------------------------------------------*/
  /* Transform from world to normalized device coordinates */
  /*-------------------------------------------------------*/

    float y ;
 
    y_world = MAX(y_world,world.ymin) ;
    y_world = MIN(y_world,world.ymax) ;
    y = c_wrld_ndc*y_world + d_wrld_ndc ;
    return(y) ;
}

void label(float xloc,float yloc,float value,int iaxis) {
  /*-------------------------------------------------------------*/
  /* Write a label onto one of the axis                          */
  /*-------------------------------------------------------------*/
    int n ;
    float fn,xymax ;
    char svalue[80] ;

    xymax = (iaxis == 1 ? world.xmax : world.ymax) ;
    /* 'value' must be converted to a string of ascii characters. */
    /*  we write value = x*10**n where n is determined by x/ymax  */

    if (value == 0.)
      sprintf(svalue,"%.1f",value) ;
    else
      {
        fn = log10((double)ABS(xymax)) ;
        if (fn< 0.)
          {
            if (ABS(fn-(int)fn) < 0.001)
              n = (int)fn  ;
            else
              n = (int)fn - 1 ;
          }
        else
          {
            if (ABS(fn-(int)fn) > 0.999)
              n = (int)fn + 1 ;
            else
              n = (int)fn ;
          }               
        if (n < 4 && n >= 0)
          if (n >= 1)
            sprintf(svalue,"%*.0f",n+3,value) ;
          else
            sprintf(svalue,"%4.2f",value) ;
        else if (n < 0 && n >= -3)
          sprintf(svalue,"%*.*f",ABS(n)+4,ABS(n)+1,value) ;
        else
          {
            value = value/pow((double)10.,(double)n) ;
            sprintf(svalue,"%4.2f",value) ;
          }        
       }
     if (iaxis == 1)     /* X-axis */
        write_legend(xloc,yloc,1.3,0.,svalue) ;
     else                /* Y-axis */
        write_legend(xloc,yloc,1.3,0.,svalue) ;
}

void graph2d(float xmin,float xmax,float ymin,float ymax) {
  /*-----------------------------------------------------*/
  /* Set the world coordinates and draw axis for a plot  */
  /*-----------------------------------------------------*/
    set_world(xmin,xmax,ymin,ymax) ;
    set_window(xmin,xmax,ymin,ymax) ;
    set_mapping() ;
    axes() ;
}

void set_mapping() {   
  /*---------------------------------------------*/
  /* set up the scaling from world to ndc coord. */
  /*---------------------------------------------*/
    a_wrld_ndc = (view.xmax-view.xmin)/(world.xmax-world.xmin) ;
    b_wrld_ndc = view.xmin - world.xmin*a_wrld_ndc ;
    c_wrld_ndc = (view.ymax-view.ymin)/(world.ymax-world.ymin) ;
    d_wrld_ndc = view.ymin - world.ymin*c_wrld_ndc ;
}

void move_ndc(float x_ndc,float y_ndc) {
  /*-----------------------------------------------------*/
  /* Move the "pen" to a specified point in normalize    */
  /* device coordinates                                  */
  /*-----------------------------------------------------*/
    int ix,iy ;
  
    ix = x_ndc_pdc(x_ndc) ;
    iy = y_ndc_pdc(y_ndc) ;
    move_pdc(ix,iy) ; /* pdc : physical device coordinates */
}

void line_ndc(float x1,float y1,float x2,float y2) {
  /*-------------------------------------------------*/
  /* Draw a line in normalized device coordinates    */
  /*-------------------------------------------------*/
    int ix1,iy1,ix2,iy2 ;
  
    ix1 = x_ndc_pdc(x1) ;
    iy1 = y_ndc_pdc(y1) ;
    ix2 = x_ndc_pdc(x2) ;
    iy2 = y_ndc_pdc(y2) ;
    line_pdc(ix1,iy1,ix2,iy2) ; /* pdc : physical device coordinates */
}

int x_ndc_pdc(float x) {
  /*-----------------------------------------------------*/
  /* Transform from normalized device coordinates to     */
  /* physical device coordinates                         */
  /*-----------------------------------------------------*/
    int ix ;
 
    ix = near(x*gOutDevice->xres) ;
    return(ix) ;
}

int y_ndc_pdc(float y) {
  /*-----------------------------------------------------*/
  /* Transform from normalized device coordinates to     */
  /* physical device coordinates                         */
  /*-----------------------------------------------------*/
    int iy ;

    /* iy = y*gOutDevice->xres*Terminal.yres/Terminal.xres ; */
    iy = near(gOutDevice->yres - y*gOutDevice->yres) ; 
    return(iy) ;
}
  
void move_pdc(int ix,int iy) {
  /*----------------------------------------------------------*/
  /* Move the "pen" to a point in physical device coordinates */
  /*----------------------------------------------------------*/
    gMoveTo(ix,iy) ;
}

void line_pdc(int ix1,int iy1,int ix2,int iy2) {
  /*----------------------------------------------------------*/
  /* Draw a line in physical device coordinates. We draw on   */
  /* either the CRT or the Frame-buffer                       */
  /*----------------------------------------------------------*/

    gDrawLine(ix1,iy1,ix2,iy2) ;  
}

void xname(char *cp,float size) {
  /*-----------------------------------------------------------*/
  /* Write a label for the X-axis                              */
  /*-----------------------------------------------------------*/
    float xs,ys ;  /* starting position of the text */
    int nchar ;    /* # of character in the text    */
    char *tp ;  
    float length ;

    /* calculate nchar */
    for (tp=cp,nchar=0;*tp!='$' && *tp != EOS;) 
      {
        if (*tp==INSTRUCTION_SYMBOL) 
          tp += 5 ;
        else 
          {
            tp++ ;
            if (*tp != SHIFT_SYMBOL)
              nchar++ ;
          }
       }
    *tp = '\0' ;       /* append a null character to the string */
    length = size*8.*nchar/gOutDevice->xres ;
    ys = view.ymin - 0.04*size ;
    xs = (view.xmax+view.xmin)/2. - length/2. ;
    write_legend(xs,ys,size,0.,cp) ;
}

void yname(char *cp,float size) {
  /*-----------------------------------------------------------*/
  /* Write a label for the Y-axis                              */
  /*-----------------------------------------------------------*/
    float xs,ys ;  /* starting position of the text */
    int nchar ;    /* # of character in the text    */
    char *tp ;  
    float length ;

    /* calculate nchar */
    for (tp=cp,nchar=0;*tp!='$' && *tp != EOS;) 
      {
        if (*tp==INSTRUCTION_SYMBOL) 
          tp += 5 ;
        else 
          {
            tp++ ;
            if (*tp != SHIFT_SYMBOL)
              nchar++ ;
          }
       } 
    *tp = '\0' ;       /* append a null character to the string */
    length = size*8.*nchar/gOutDevice->yres ;
    xs = view.xmin - 0.06*size ;
    ys = (view.ymax+view.ymin)/2. - length/2. ;
    write_legend(xs,ys,size,90.,cp) ;
}

void headin(char *cp,float size) {
  /*--------------------------------------------------------------*/
  /* Write a heading for the graph centered above the plot        */
  /*--------------------------------------------------------------*/
    float xs,ys ;  /* starting position of the text */
    int nchar ;    /* # of character in the text    */
    char *tp ;  
    float length ;

    /* calculate nchar */
    for (tp=cp,nchar=0;*tp!='$' && *tp != EOS;) 
      {
        if (*tp==INSTRUCTION_SYMBOL) 
          tp += 5 ;
        else 
          {
            tp++ ;
            if (*tp != SHIFT_SYMBOL)
            nchar++ ;
          }
      }
    *tp = '\0' ;       /* append a null character to the string */
    length = size*8.*nchar/gOutDevice->xres ;
    ys = view.ymax + 0.05 ;
    xs = (view.xmax+view.xmin)/2. - length/2. ;
    write_legend(xs,ys,size,0.,cp) ;
}

void ticks_x(float xmin,float xmax) {
  /*-------------------------------------------------------------*/
  /* Draw tick marks on the X-axis and put labels on them        */
  /*-------------------------------------------------------------*/
    int n,ilower,nticks ;
    float fn,delta,loc,length,x,y,ticksize,short_tick,long_tick ;
    double ten=10.,logrange,dlower,range,xs,ys ;
    float dx_small,dx_large ;
    float xs_world,xs_ndc,x_world,test,spacing,labelspacing ;
    char svalue[80] ;

    short_tick = 0.01 ;
    long_tick = 1.5*short_tick ;
    
    range = xmax-xmin ;            /* Range of x_values           */
    length = view.xmax-view.xmin ; /* length of the x-axis in ndc */
    logrange = log10(range) ;                     /* round off to the nearest */
    ilower = logrange<0. ? (int)(logrange-1.): (int)logrange; /* value that is modula     */
    dlower = ilower ;                             /* power of 10              */
    dx_large = pow(ten,dlower) ; /* The increment in x at tickmarks */
    nticks = near(range/dx_large) ;
    /* this gives between 1 and 10 tickmarks.*/
    spacing = length*dx_large/range ; /* spacing between the tickmarks in ndc */
    /* The assumption is that we will label every one of these */
    if (spacing > 0.2) 
      dx_small = dx_large/10. ;
    else if (spacing > 0.1)
      dx_small = dx_large/5. ;
    else if (spacing > 0.05)
      dx_small = dx_large/2. ;
    else 
      dx_small = dx_large ;
    delta = length*dx_small/range ; /* length between small tickmarks in ndc */
  
   /* starting position in world coord */
    xs_world = (int)(xmin/dx_small)*dx_small ;
    if (xs_world < xmin) xs_world += dx_small ;
    xs_ndc = x_wrld_ndc(xs_world) ;      /* starting position in ndc         */
    y = view.ymin ;
    labelspacing = 0. ;
    for (loc=0.,x_world=xs_world;x_world<= xmax;x_world+=dx_small,loc+=delta)
      {
        x = xs_ndc+loc ;
        test = ABS(x_world/dx_large-(int)(x_world/dx_large)) ;
        xs = x ;
        ys = y ;
        if (test < 0.05 || test > 0.95)
          {
            if ( ABS(x_world) < (0.1*dx_large)) x_world = 0. ;
            ticksize = long_tick ;
          }
        else
          {
            ticksize = short_tick ;
          }
        line_ndc(xs,ys,x,y+ticksize) ;
        labelspacing += delta ;
        if (labelspacing > 0.08)
           {
              label(x-0.02,y-0.03,x_world,1) ;
              labelspacing = 0. ;
           }
      }
    if (xmax > 10000. || xmax < 0.001)    /* print out 10 ** n  */
      {
        fn = log10((double)ABS(xmax)) ;
        if (fn< 0.)
          {
            if (ABS(fn-(int)fn) < 0.001)
              n = (int)fn  ;
            else
              n = (int)fn - 1 ;
          }
        else
          {
            if (ABS(fn-(int)fn) > 0.999)
               n = (int)fn + 1 ;
            else
               n = (int)fn ;
          }               
        instrct('#') ;
        sprintf(svalue,"*10#eh.7%d#exhx",n) ;
        write_legend(x_wrld_ndc(xmax),y-0.03,1.5,0.,svalue) ;
     }
}

void ticks_y(float ymin,float ymax) {
  /*-------------------------------------------------------------*/
  /* Draw tick marks on the X-axis and put labels on them        */
  /*-------------------------------------------------------------*/
    int n,ilower,nticks,intpart ;
    float fn,delta,loc,length,x,y,ticksize,short_tick,long_tick ;
    double ten=10.,logrange,dlower,range ;
    float dy_small,dy_large,xs,ys ;
    float ys_world,ys_ndc,y_world,test,spacing,labelspacing ;
    char svalue[10] ;

    short_tick = 0.007 ;
    long_tick = 1.5*short_tick ;
    range = ymax-ymin ;
    length = view.ymax-view.ymin ; /* length of the y-axis in ndc */
    logrange = log10(range) ;                     /* round off to the nearest */
    ilower = logrange<0. ? (int)(logrange-1.): (int)logrange ;/* value that is modula     */
    dlower = ilower ;                             /* power of 10              */
    dy_large = pow(ten,dlower) ; /* The increment in y at tickmarks */
    nticks = near(range/dy_large) ;
    /* this gives between 1 and 10 tickmarks.*/
    spacing = length*dy_large/range ; /* spacing between the tickmarks in ndc */
    /* The assumption is that we will label every one of these */
    if (spacing > 0.2) 
      dy_small = dy_large/10. ;
    else if (spacing > 0.1)
      dy_small = dy_large/5. ;
    else if (spacing > 0.05)
      dy_small = dy_large/2. ;
    else 
      dy_small = dy_large ;
    delta = length*dy_small/range ;  /* length between tickmarks in ndc */
  
   /* starting position in world coord */
    ys_world = (int)(ymin/dy_small)*dy_small ;
    if (ys_world < ymin) ys_world += dy_small ;
    ys_ndc = y_wrld_ndc(ys_world) ;      /* starting position in ndc         */
    x = view.xmin ;
    labelspacing = 0. ;
    for (loc=0.,y_world=ys_world;y_world<= ymax;y_world+=dy_small,loc+=delta)
      {
        y = ys_ndc+loc ;
        xs = x ;
        ys = y ;
        intpart = (int)(y_world/dy_large) ;
        test = ABS(y_world/dy_large-intpart) ;
        if (test < 0.05 || test > 0.95)
          {
            if ( ABS(y_world) < (0.1*dy_large)) y_world = 0. ;
            ticksize = long_tick ;
          }
        else
          {
            ticksize = short_tick ;
          }
        line_ndc(xs,ys,x+ticksize,y) ;
        labelspacing += delta ;
        if (labelspacing > 0.08)
           {
               label(x-0.05,y-0.01,y_world,2) ;
               labelspacing = 0. ;
           }
      }
    if (ymax > 10000. || ymax < 0.001)  /* print out 10**n  */
      {
        fn = log10((double)ABS(ymax)) ;
        if (fn< 0.)
          {
            if (ABS(fn-(int)fn) < 0.001)
              n = (int)fn  ;
            else
              n = (int)fn - 1 ;
          }
        else
          {
            if (ABS(fn-(int)fn) > 0.999)
              n = (int)fn + 1 ;
            else
              n = (int)fn ;
          }               
        instrct('#') ;
        sprintf(svalue,"*10#eh.7%d#exhx",n) ;
        write_legend(x-0.04,y_wrld_ndc(ymax)+0.04,2.0,0.,svalue) ;
      }
}

void curve(float *x,float *y,int pn,int marker_interval) {
  /*-----------------------------------------------------------*/
  /* Plot a curve on the display. The curve is passed in the   */
  /* arrays x and y with *pn giving the number of points       */
  /*-----------------------------------------------------------*/
    int n ;
    float xs,ys ;
    char MARK[2] ;

    MARK[0] = MARKER_SYMBOL ;
    MARK[1] = EOS ;  

    xs = x[0] ;
    ys = y[0] ;
    for (n=1;n<pn;n++) 
      {
        if (marker_interval != 0)
          if (n%marker_interval == 0) 
            {
              write_legend(x_wrld_ndc(x[n]),y_wrld_ndc(y[n])-0.005,
                           1.,0.,&MARKER_SYMBOL) ;
              xs = x[n-1] ;
              ys = y[n-1] ;
            }
          line_abs(xs,ys,x[n],y[n]) ;
          xs = x[n] ;
          ys = y[n] ;
       }
}

void marker(char cp) {
  /*------------------------------------------------*/
  /* Set the marker symbol for later use            */
  /*------------------------------------------------*/
    MARKER_SYMBOL = cp ;
}

void SetLineStyle(int style) {
  /*------------------- */
  /* Set the line style */
  /*------------------- */
    LineRc.style = style ;
    switch(LineRc.style)
	{
	case SOLID: 	switch(gOutDevice->type)
				{
				case WS:  solid() ;
					  break ;
				case LP:  break ;
				}
			break ;
	case DOT:	switch(gOutDevice->type)
				{
				case WS:  dot() ;
					  break ;
				case LP:  break ;
				}
			break ;
	case DASH:	switch(gOutDevice->type)
				{
				case WS:  dash() ;
					  break ;
				case LP:  break ;
				}
			break ;
	case DOTDASH:	switch(gOutDevice->type)
				{
				case WS:  dot_dash() ;
					  break ;
				case LP:  break ;
				}
			break ;
	}
}

int GetLineStyle() {
  /*---------------------------*/
  /* Get the line style        */
  /*---------------------------*/
    return(LineRc.style) ;
}

void SetLineWidth(int width) {
  /*--------------------------------------------------*/
  /* Set flag to indicate that a thick linestyle is   */
  /* requested                                        */
  /*--------------------------------------------------*/
    XGCValues xgc ;

    LineRc.width = width ;
    xgc.line_width = width ;
    switch(gOutDevice->type)
	{
	case WS:
			XChangeGC(gDisplay,gCurrentPort->GetGC(),
				GCLineWidth,&xgc) ;
			break ;
	case LP:	fprintf(gPSFile,"\n%d setlinewidth",(int)LineRc.width) ;
			break ;
	}
}

short GetLineWidth() {
  /*--------------------------------------------------*/
  /* Set flag to indicate that a thick linestyle is   */
  /* requested                                        */
  /*--------------------------------------------------*/
    return(LineRc.width) ;
}

void erase_pdc(int ix1,int iy1,int ix2,int iy2) {
  /*--------------------------------------------------------*/
  /* Erase a portion of the screen in Phys. device coord.   */
  /*--------------------------------------------------------*/
}   

void dash() {
  /*--------------------------------------------------*/
  /* Set the line style to DASH                       */
  /*--------------------------------------------------*/

  XSetLineAttributes(gDisplay,gCurrentPort->GetGC(),
		1,LineOnOffDash,CapButt,JoinMiter) ;
}

void dot_dash() {
  /*--------------------------------------------------*/
  /* Set the line style to DOT-DASH                   */
  /*--------------------------------------------------*/

  XSetLineAttributes(gDisplay,gCurrentPort->GetGC(),
		1,LineDoubleDash,CapButt,JoinMiter) ;
}

void solid() {
  /*--------------------------------------------------*/
  /* Set the line style back to SOLID                 */
  /*--------------------------------------------------*/

    XGCValues xgc ;

    xgc.line_style = LineSolid ;
    XChangeGC(gDisplay,gCurrentPort->GetGC(),GCLineStyle,&xgc) ;
}

void dot() {
  /*--------------------------------------------------*/
  /* Set the line style to DOT                        */
  /*--------------------------------------------------*/
	/*
    XGCValues xgc ;
    static int style=0xf0f0f0f0 ;
	 */
}

void thkcrv() {
  /*--------------------------------------------------*/
  /* Set flag to indicate that a thick linestyle is   */
  /* requested                                        */
  /*--------------------------------------------------*/

    THICK_CURVE = TRUE ;
}

void nrmcrv() {
  /*--------------------------------------------------*/
  /* Set flag to indicate that a normal linestyle is  */
  /* requested                                        */
  /*--------------------------------------------------*/

    THICK_CURVE = FALSE ;
}
