C  This program is free software; you can redistribute it and/or modify
C  it under the terms of the GNU General Public License as published by
C  the Free Software Foundation; either version 2 of the License, or
C  (at your option) any later version.
C
C  This program is distributed in the hope that it will be useful,
C  but WITHOUT ANY WARRANTY; without even the implied warranty of
C  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public License
C  along with this program; if not, write to the Free Software
C  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
C
       SUBROUTINE SORTAG (A, N, TAG )
C///
C       SUBROUTINE SORTAG
C
C        DECK USED - CDC 6500, IBM360/67
C
C        SORTAG
C
C        PURPOSE
C
C           TO SORT A VECTOR INTO INCREASING ORDER FROM A(1) TO A(N),
C           A MAY BE TYPE REAL OR TYPE INTEGER.  VECTOR TAG IS PER-
C           MUTED THE SAME AS VECTOR A.
C
C        USAGE
C
C           CALL SORTAG(A,N,TAG)
C
C        DESCRIPTION OF PARAMETERS
C
C           A - THE NAME OF THE N-VECTOR TO BE SORTED.
C               IF A IS TYPE REAL THEN EACH OF ITS COMPONENTS MUST BE
C               NORMALIZED FORM.
C           N - THE NUMBER OF ELEMENTS IN THE VECTOR TO BE SORTED.
C           TAG - THE NAME OF THE N-VECTOR CONTAINING THE TAG FIELDS.
C
C        REMARKS
C
C           THE PROCEDURE REQUIRES TWO ADDITIONAL ARRAYS IU(K) AND
C           IL(K) WHICH PERMIT SORTING UP TO 2**(K+1)-1 ELEMENTS.
C
C        EXAMPLE
C
C           IF N RANDOM NUMBERS, UPON GENERATION, WERE ASSIGNED
C           CONSECUTIVE TAGS OF 1 THROUGH N, AND THE ARRAY OF RANDOM
C           NUMBERS SORTED, THE TAG ARRAY COULD THEN BE USED TO
C           DETERMINE THAT THE RANDOM NUMBER NOW IN POSITION A(I) WAS
C           ORIGINALLY IN POSITION A(J) BECAUSE TAG(I) = J.
C
C        METHOD
C
C           'AN EFFICIENT ALGORITHM FOR SORTING WITH MINIMAL STORAGE'
C           BY RICHARD C. SINGLETON.  PREPARED FOR INSTITUTE RESEARCH
C           AND DEVELOPMENT.  STANFORD RESEARCH INSTITUTE PROJECT
C           387531-132 SEPTEMBER 1968.
C
C     ..................................................................
C
C      RECEIVED FROM BAUKE DIJKSTR - DEC 1985 - VAX VERSION.
C
C=========================================================================
C      VERSION 23 DECEMBER 1985
C=========================================================================
C\\\
 
         PARAMETER (IKEY = 18)
         DIMENSION A(N), IU(18), IL(18), TAG(N)
         INTEGER A, T, TT, TAG, TG
 
         IF (N .GE. 524288) THEN
           WRITE (*, *) ' TOO MANY ITEMS TO BE SORTED'
           WRITE (*, *)
     X        ' INCREASE PARAMETER IKEY IN CFBLIB-ROUTINE SORTAG'
           STOP
         END IF
 
         M = 1
         I = 1
         J = N
    5    IF (I .GE. J) GO TO 70
   10    K = I
         IJ = (J + I) / 2
         T = A(IJ)
         IF (A(I) .GT. A(IJ)) THEN
           A(IJ) = A(I)
           A(I) = T
           T = A(IJ)
           TG = TAG(IJ)
           TAG(IJ) = TAG(I)
           TAG(I) = TG
         END IF
         L = J
         IF (A(J) .GE. T) GO TO 40
         A(IJ) = A(J)
         A(J) = T
         T = A(IJ)
         TG = TAG(IJ)
         TAG(IJ) = TAG(J)
         TAG(J) = TG
         IF (A(I) .LE. A(IJ)) GO TO 40
         A(IJ) = A(I)
         A(I) = T
         T = A(IJ)
         TG = TAG(IJ)
         TAG(IJ) = TAG(I)
         TAG(I) = TG
         GO TO 40
   30    A(L) = A(K)
         A(K) = TT
         TG = TAG(L)
         TAG(L) = TAG(K)
         TAG(K) = TG
   40    L = L - 1
         IF (A(L) .GT. T) GO TO 40
         TT = A(L)
   50    K = K + 1
         IF (A(K) .LT. T) GO TO 50
         IF (K .LE. L) GO TO 30
         IF (L - I .LE. J - K) GO TO 60
         IL(M) = I
         IU(M) = L
         I = K
         M = M + 1
         GO TO 80
   60    IL(M) = K
         IU(M) = J
         J = L
         M = M + 1
         GO TO 80
   70    M = M - 1
         IF (M .EQ. 0) RETURN
         I = IL(M)
         J = IU(M)
   80    IF (J - I .GE. 1) GO TO 10
         IF (I .EQ. 1) GO TO 5
         I = I - 1
   90    I = I + 1
         IF (I .EQ. J) GO TO 70
         T = A(I+1)
         IF (A(I) .LE. A(I+1)) GO TO 90
         TG = TAG(I+1)
         K = I
  100    A(K+1) = A(K)
         TAG(K+1) = TAG(K)
         K = K - 1
         IF (T .LT. A(K)) GO TO 100
         A(K+1) = T
         TAG(K+1) = TG
         GO TO 90
 
       END
