C  This program is free software; you can redistribute it and/or modify
C  it under the terms of the GNU General Public License as published by
C  the Free Software Foundation; either version 2 of the License, or
C  (at your option) any later version.
C
C  This program is distributed in the hope that it will be useful,
C  but WITHOUT ANY WARRANTY; without even the implied warranty of
C  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public License
C  along with this program; if not, write to the Free Software
C  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
C
C program transl translates the digital code in file 'population'
C back to 'real' parameters.
C note: inclusive of both the upper and the lower bound of parameters
 
       PROGRAM TRANSL
 
C  mparam: maximal number of parameters
C  mnbits: maximal number of bits allowed to describe one parameter
C  mindiv: maximal number of chromosomes per population
C         PARAMETER (MPARAM = 80, MNBITS = 4, MINDIV = 2000)
        INCLUDE 'CrossPars.h'
C
C  Current limits
C     mparam to 30 - may need to be increased
C     mnbits to 10 - Increased from 8 to 10 for multislice use
C     Number of members 100 - Decreased from 120 to 100 to avoid
C                 memory problems after increasing mnbits
 
C  nparam: number of parameters
C  nbits(i): number of bits describing parameter i
C  nndiv: number of chromosomes in one population
C  mbits: maximal number of bits describing one parameter
C  pow2(i): powers of 2
C  bit(i,j): bit no. j in parameter i of a chromosome
C  brange(i), erange(i): beginning and end of parameter range of parameter i
C  actval: help variable
C  param(i): actual value of parameter i
C  dummy: dummy character
         INTEGER NPARAM, NBITS(MPARAM), NNDIV, MBITS, POW2(0:3)
         INTEGER*2 BIT(MPARAM,4)
         REAL BRANGE(MPARAM), ERANGE(MPARAM), ACTVAL, PARAM(MPARAM)
         CHARACTER DUMMY*17
         INTEGER II1, II2, II3
 
         OPEN (1, FILE='control', STATUS='old')
         OPEN (2, FILE='population', STATUS='old')
         OPEN (4, FILE='popw', STATUS='unknown')
 
   10    FORMAT(I4,2F12.8)
   20    FORMAT(A17,200I1)
   30    FORMAT(A3,I3,A2,200F8.4)
 
C======================================================================
C readin needed information from file 'control'
C======================================================================
 
C number of parameters?
         READ (1, 10) NPARAM
 
C number of bits, range of every parameter?
         MBITS = 0
         DO 100 I=1,NPARAM
           READ (1, 10) NBITS(I), BRANGE(I), ERANGE(I)
           IF (NBITS(I) .GT. MBITS) THEN
             MBITS = NBITS(I)
           END IF
  100      CONTINUE
 
C number of chromosomes in one population?
         READ (1, 10) NNDIV
 
C======================================================================
C perform 'translation'
C======================================================================
 
C calculate necessary powers of 2
         CALL POWER2 (POW2,4,MBITS)
 
C run loop over different chromosomes
 
C CHANGE: jump out via end of file
C      do 200 j=1,nndiv
         DO 200 J=1,999999
 
C     read bit chain for present chromosome from file 'population'
           READ (2, 20, ERR=300, END=300) DUMMY, ((BIT(I,K), K=1,NBITS(I
     X        )), I=1,NPARAM)
 
C        loop over different parameters
           DO 210 II1=1,NPARAM
 
             ACTVAL = 0.
             II3 = NBITS(II1)
             DO 220 II2=1,II3
               ACTVAL = ACTVAL + REAL (BIT(II1,II2) * 2 ** (II3 - II2))
  220          CONTINUE
             PARAM(II1) = BRANGE(II1) + ACTVAL * (ERANGE(II1) - BRANGE(
     X          II1)) / (2.D0 ** NBITS(II1) - 1.D0)
 
  210        CONTINUE
 
C======================================================================
C output to file 'popw'
C======================================================================
 
           WRITE (4, 30) 'ind', J, ': ', (PARAM(I), I=1,NPARAM)
 
  200      CONTINUE
  300    CONTINUE
       END
 
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCC   SUBROUTINES   CCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 
C subroutine power2 calculates necessary powers of 2
       SUBROUTINE POWER2 (POW2, MNBITS, MBITS )
         INTEGER POW2(0:MNBITS-1)
         POW2(0) = 1
         DO 100 I=1,MBITS-1
  100      POW2(I) = POW2(I-1) * 2
         RETURN
       END
