#ifndef __TIFF_H
#define __TIFF_H
/* tiff.h
 *
 * made this version on 88-10-31
 * updated 90-05-22
 */

/* TIFF data types
 */
#define TIFFBYTE                1
#define TIFFASCII               2
#define TIFFSHORT               3
#define TIFFLONG                4
#define TIFFRATIONAL    5
#define TIFFSBYTE               6
#define TIFFUNDEFINED   7
#define TIFFSSHORT              8
#define TIFFSLONG               9
#define TIFFSRATIONAL   10
#define TIFFFLOAT               11
#define TIFFDOUBLE              12

/* TIFF tag constants
 */
#define TGNEWSUBFILETYPE                        254
#define TGOLDSUBFILETYPE                        255
#define TGIMAGEWIDTH                            256
#define TGIMAGELENGTH                           257
#define TGBITSPERSAMPLE                         258
#define TGCOMPRESSION                           259

#define TGPHOTOMETRICINTERPRETATION     262
#define TGTHRESHHOLDING                         263
#define TGCELLWIDTH                                     264
#define TGCELLLENGTH                            265
#define TGFILLORDER                                     266

#define TGDOCUMENTNAME                          269
#define TGIMAGEDESCRIPTION                      270
#define TGMAKE                                          271
#define TGMODEL                                         272
#define TGSTRIPOFFSETS                          273
#define TGORIENTATION                           274

#define TGSAMPLESPERPIXEL                       277
#define TGROWSPERSTRIP                          278
#define TGSTRIPBYTECOUNTS                       279
#define TGMINSAMPLEVALUE                        280
#define TGMAXSAMPLEVALUE                        281
#define TGXRESOLUTION                           282
#define TGYRESOLUTION                           283
#define TGPLANARCONFIGURATION           284
#define TGPAGENAME                                      285
#define TGXPOSITION                                     286
#define TGYPOSITION                                     287
#define TGFREEOFFSETS                           288
#define TGFREEBYTECOUNTS                        289
#define TGGRAYUNIT                                      290
#define TGGRAYCURVE                                     291

#define TGRESOLUTIONUNIT                        296             /* 87-12-11 */
#define TGPAGENUMBER                            297

#define TGCOLORRESPONSECURVES           301

#define TGSOFTWARE                                      305
#define TGDATETIME                                      306

#define TGARTIST                                        315
#define TGHOSTCOMPUTER                          316

#define TGPREDICTOR                                     317             /* 88-09-19 */
#define TGWHITEPOINT                            318
#define TGPRIMARYCHROMATICITIES         319
#define TGCOLORMAP                                      320

#define TGHIGHSHADOW                            321             /* 89-11-16 */
#define TGTILEWIDTH                                     322
#define TGTILELENGTH                            323
#define TGTILEOFFSETS                           324
#define TGTILEBYTECOUNTS                        325
#define TGINKSET                                        332
#define TGINKNAMES                                      333
#define TGSAMPLEFORMAT                          339

typedef int     Boolean ;

#define false   0
#define true    1 
#define nil             (void *)0 

/* TIFF "header" (8 bytes)
 * note: GtTiffHdr plays a little loose with this structure.
 */
typedef struct {
                short   thByteOrder;
                short   thVersion;
                int     thIfdOffset;
} TIFFHDR,*PTIFFHDR;

/* IFD entry
 * note: GtTiffEntry plays a little loose with this structure.
 */
typedef struct {
                short  deTag;
                short  deType;
                int     deLength;
                int     deVal;
} DIRENTRY, *PDIRENTRY;

typedef struct {
        short numEntries ;
        PDIRENTRY entry ;
        int nextIFD ;
} IFD,*PIFD ;
typedef enum {kBiLevel,kGrayScale,kPalette,kRGB,kFloatingPoint,kComplexFloatingPoint 
} TiffImageType ;


typedef struct PaletteColor {
        short red ;
        short green ;
        short blue ;
} PaletteColor ;


typedef struct TiffRecord {
        int     width ;
        int height ;
        short bitsPerSample ;
        short samplesPerPixel ;
        short compression ;
        short photoMetricInterpretation ;
        short sampleFormat ;
        short stripOffsetType ;
        int *stripOffset ;
        int rowPerStrip ;
        int numberOfStrips ;
        short stripByteCountType ;
        int *stripByteCount ;
        int     tgxOffset ;
        int tgyOffset ;
        short resolutionUnit ;
        int numPaletteColors ;
        int paletteOffset ;
        PaletteColor    *palette ;
        unsigned char   *data ;
} TiffRecord ;

#define NEBILEVEL       11
#define NEGRAYSCALE     12
#define NEPALETTE       13
#define NERGB           13
#define NEFLPOINT       14
#define NECFLPOINT      14

typedef void *Ptr ;
FILE *CreateTiffFile(char * filename) ;
void CloseTiffFile(FILE *fp) ;
PIFD CreateTiffIFD(short numDE) ;
void WriteTiffDirectory(FILE *fp,TiffImageType imageType,int width,int height) ;
void WriteTiffIFD(FILE *fp,PIFD pifd) ;
void AddTiffTag(PIFD ifd,short TagID,short TagType,int numValues,int value) ;
void WriteTiffPalette( FILE *fp, PaletteColor *colorTable, int numEntries) ;
void WriteTiffStrip( FILE *fp, Ptr data,int nBytes) ;
void WriteTiffHeader(FILE *fp) ;
void WriteTiffTag( short TagID, short TagType, int numValues,int  offset) ;
Boolean ReadTiffFile(char *filename,unsigned char *imagedata) ;
Boolean ReadTiffDirectory(FILE *fp,TIFFHDR *thdr) ;
void SwapDataBytes(unsigned char *data,int width,int height,
                short sampleFormat,short bitsPerSample,short samplesPerPixel) ;
Boolean ProcessTiffDirEntry(FILE *fp,TIFFHDR *thdr,PDIRENTRY dirEntry) ;
Boolean ReadTiffHeader(FILE *fp,TIFFHDR *header) ;
unsigned short ByteSwapShort(unsigned short value) ;
unsigned long ByteSwapLong(unsigned long value) ;
float ByteSwapFloat(unsigned long value) ;



#endif
