      subroutine slmmv(n,m,s,y,rho,scale,mark,v,wa,nhk2)
      integer n, m, mark
      real scale
      real s(nhk2,m), y(nhk2,m), rho(m), v(n), wa(m)
c     **********
c
c     This subroutine computes the matrix-vector product H*v
c     where H is the inverse BFGS approximation.
c
c     The matrix H depends on an initial matrix H0, m steps
c     s(1),...,s(m), and m gradient differences y(1),...,y(m).
c     These vectors are stored in the arrays s and y.
c     The most recent step and gradient difference are stored
c     in columns s(1,mark) and y(1,mark), respectively.
c     The initial matrix H0 is assumed to be scale*I.
c
c     The subroutine statement is
c
c       subroutine slmmv(n,m,s,y,rho,scale,mark,v,wa)
c
c     where
c
c       n is an integer variable.
c         On entry n is the number of variables.
c         On exit n is unchanged.
c
c       m is an integer variable.
c         On entry m specifies the number of steps and gradient
c            differences that are stored.
c         On exit m is unchanged.
c
c       s is a real array of dimension (n,m)
c         On entry s contains the m steps.
c         On exit s is unchanged.
c
c       y is a real array of dimension (n,m)
c         On entry y contains the m gradient differences.
c         On exit y is unchanged.
c
c       rho is a real array of dimension m
c         On entry rho contains the m innerproducts (s(i),y(i)).
c         On exit rho is unchanged.
c
c       scale is a real variable
c         On entry scale specifies the initial matrix H0 = scale*I.
c         On exit scale is unchanged.
c
c       mark is an integer variable.
c         On entry mark points to the current s(i) and y(i).
c         On exit mark is unchanged.
c
c       v is a real array of dimension n.
c         On entry v contains the vector v.
c         On exit v contains the matrix-vector product H*v.
c
c       wa is a real work array of dimension m.
c
c     Subprograms called
c
c       Level 1 BLAS ... saxpy, sdot, sscal
c
c     MINPACK-2 project. November 1993.
c     Argonne National Laboratory and University of Minnesota.
c     Brett M. Averick, Richard G. Carter, and Jorge J. More'.
c
c     **********
      integer i, k
      real beta

C       Note: not using Blas - let HP vectorize this.

C      real sdot
C      external saxpy, sdot, sscal
      k = mark + 1
      do 10 i = 1, m
         k = k - 1
         if (k .eq. 0) k = m
C       Skip the update if rho(k) is negative
        if(rho(k).lt.1e-6)goto 10
                tmp=0
                do j=1,n
                        tmp=tmp+s(j,k)*v(j)
                enddo
                wa(k)=tmp/rho(k)
C         wa(k) = sdot(n,s(1,k),1,v,1)/rho(k)
                do j=1,n
                        v(j)=v(j)-wa(k)*y(j,k)
                enddo
C         call saxpy(n,-wa(k),y(1,k),1,v,1)
   10 continue
        do j=1,n
                v(j)=v(j)*scale
        enddo
C      call sscal(n,scale,v,1)
      do 20 i = 1, m
           if(rho(k).lt.1e-6)goto 11
                beta=0
                do j=1,n
                        beta=beta-y(j,k)*v(j)
                enddo
                beta=wa(k)+beta/rho(k)
C         beta = wa(k) - sdot(n,y(1,k),1,v,1)/rho(k)
                do j=1,n
                        v(j)=v(j)+beta*s(j,k)
                enddo
C         call saxpy(n,beta,s(1,k),1,v,1)
11       k = k + 1
         if (k .eq. m+1) k = 1
   20 continue
      end
