        subroutine matder(found,vals,wts,N,G)
C
C       Evaluate the 2nd derivative (matrix terms)
C       Input:
C             found(n)   logical, beam exists
C             vals(n)    Value
C             wts(n)     Wts
C             scl(n)     Scales
C
C       Output:
C             a(n,n)     2nd derivative terms
C             g(n)       Gradient
C
        implicit real*8 (a-h,o-z)
        include 'msizes.h'
        dimension vals(N),wts(N)
        logical found(N)
        dimension work(20),G1(20),G2(20),G(N)

C       Find 2nd derivative terms numerically
        STEP=1D-8
        STEP2=2.D-8
C       Initialize
        DO I=1,N
               work(I)=scl(i)
               G(I)=0
               do K=1,N
                  AT(I,K)=0
               enddo
        ENDDO
C       Gradient at current scale
        call Rgrad(found,vals,wts,work,N,g)
C       2nd derivates numerically
        DO J=1,N
           IF(found(j))then
            work(j)=scl(j)+STEP
            call Rgrad(found,vals,wts,work,N,g1)
            work(j)=scl(j)-STEP
            call Rgrad(found,vals,wts,work,N,g2)
            work(j)=scl(j)
            DO I=1,N
              IF(found(i))then
                AT(J,I)=(G1(I)-G2(I))/STEP2
              ENDIF
            ENDDO
           endif
        ENDDO
C        call exit(0)
        RETURN
        END
        
        subroutine Rgrad(found,vals,wts,scl,N,grad)
        implicit real*8 (a-h,o-z)
        dimension vals(N),wts(N),scl(N),grad(N)
        dimension work(20)
        logical found(N)
C       Return numerical gradient
        STEP=1D-8
        STEP2=2.D-8
        DO I=1,N
               work(I)=scl(i)
               grad(i)=0
        ENDDO
        DO J=1,N
           IF(found(j))then
            work(j)=scl(j)+STEP
            R1=Rvalue(found,vals,wts,work,N)
            work(j)=scl(j)-STEP
            R2=Rvalue(found,vals,wts,work,N)
            GRAD(J)=(R1-R2)/STEP2
            work(j)=scl(j)
           endif
        ENDDO
        RETURN
        END
            
        real*8 Function Rvalue(found,vals,wts,scl,N)
        implicit real*8 (a-h,o-z)
        dimension vals(N),wts(N),scl(N)
        dimension work(20)
        logical found(N)
C       Find the weighted value
        S1=0
        S2=0
        do J=1,N
           IF(found(j))then
              work(j)=scl(j)*vals(j)
              S1=S1+work(j)*wts(j)
              S2=S2+wts(j)
           ENDIF
        enddo
C       Weighted value
        F=S1/S2
C       Calculate the total error
        Rvalue=0.D0
        DO J=1,N
           IF(found(j))then
              ERR=F-work(j)
              Rvalue=Rvalue+err*err*wts(j)
           ENDIF
        ENDDO
        RETURN
        END
        
