/*
 cnstfunc.c
  Constraint functions
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "mbconfig.h"
#include "fitting.h"
#include "utils.h"
#include "mb_input.h"
#include "data_out.h"
#include "svd.h"

extern DEBUG_INFO dbg;
extern EPS_CTRL	epsilon;

#define PARNAME 	20
#define	LEN_MAX	256
#define STRLEN  256


/* CSD -------------------------------------------------------------------- */
/* ------------------------------------------------------------------------ */
double val_search_csd(char *str, CRYSTAL_STRCT_DATA *csd)
{
  /* !!!!! KT */
  double val;
  int i, n_atom, ia, num_lofg, lof_type;
  
  if ( strcmp(str, csd->lp_f.a.name) == 0 ){
    val = csd->lp.a; return val;
  }
  if ( strcmp(str, csd->lp_f.b.name) == 0 ){
    val = csd->lp.b; return val;
  }
  if ( strcmp(str, csd->lp_f.c.name) == 0 ){
    val = csd->lp.c; return val;
  }
  if ( strcmp(str, csd->lp_f.alpha.name) == 0 ){
    val = csd->lp.alpha; return val;
  }
  if ( strcmp(str, csd->lp_f.beta.name) == 0 ){
    val = csd->lp.beta; return val;
  }
  if ( strcmp(str, csd->lp_f.gamma.name) == 0 ){
    val = csd->lp.gamma; return val;
  }
  
  n_atom = csd->n_atom;
  for(i=0; i<n_atom; i++){
    
    if ( strcmp(str, csd->atm_f[i].x.name) == 0 ){
      val = csd->atm[i].x; return val;
    }
    if ( strcmp(str, csd->atm_f[i].y.name) == 0 ){
      val = csd->atm[i].y; return val;
    }
    if ( strcmp(str, csd->atm_f[i].z.name) == 0 ){
      val = csd->atm[i].z; return val;
    }
    
    if ( csd->atm[i].bf == 0 ){				/* Isotoropic */
      if ( strcmp(str, csd->atm_f[i].b.name) == 0 ){
        val = csd->atm[i].b; return val;
      }
    }
    else if ( csd->atm[i].bf == 1 ){			/* anisotropic */
      if ( strcmp(str, csd->atm_f[i].b11.name) == 0 ){
        val = csd->atm[i].b11; return val;
      }
      if ( strcmp(str, csd->atm_f[i].b22.name) == 0 ){
        val = csd->atm[i].b22; return val;
      }
      if ( strcmp(str, csd->atm_f[i].b33.name) == 0 ){
        val = csd->atm[i].b33; return val;
      }
      if ( strcmp(str, csd->atm_f[i].b12.name) == 0 ){
        val = csd->atm[i].b12; return val;
      }
      if ( strcmp(str, csd->atm_f[i].b13.name) == 0 ){
        val = csd->atm[i].b13; return val;
      }
      if ( strcmp(str, csd->atm_f[i].b23.name) == 0 ){
        val = csd->atm[i].b23; return val;
      }
    }
    else {
      printf("$$ Invalid bf. Abort.\n");
      exit(EXIT_FAILURE);
    }
    
    if ( strcmp(str, csd->atm_f[i].occ.name) == 0 ){
      val = csd->atm[i].occ; return val;
    }
  }
  
  num_lofg = csd->num_lofg;
  lof_type = csd->lof_type;
  
  if (num_lofg > 0){
    if (lof_type == 1){				/* 1: whole part of Fg */
      for(i=0; i<num_lofg; i++){
        if ( strcmp(str, csd->fg_lofg_f[i].re.name) == 0 ){
          val = csd->fg_lofg[i].re; return val;
        }
        if ( strcmp(str, csd->fg_lofg_f[i].im.name) == 0 ){
          val = csd->fg_lofg[i].im; return val;
        }
      }
    }
    else if (lof_type == 2){		/* 2: elastic part of Fg */
      for(i=0; i<num_lofg; i++){
        if ( strcmp(str, csd->fg_lofg_f[i].ela_re.name) == 0 ){
          val = csd->fg_lofg[i].ela_re; return val;
        }
        if ( strcmp(str, csd->fg_lofg_f[i].ela_im.name) == 0 ){
          val = csd->fg_lofg[i].ela_im;	return val;
        }
      }
    }
    else if ( lof_type == 3 || lof_type == 4 ){
      /* 3: both f (elastic and inelastic), 4: elastic f */
      for(i=0; i<num_lofg; i++){
        for(ia=0; ia<n_atom; ia++){
          if ( strcmp(str, csd->fg_lofg_f[i].fe[ia].name) == 0 ){
            val = csd->fg_lofg[i].fe[ia]; return val;
          }
          if ( strcmp(str, csd->fg_lofg_f[i].fi[ia].name) == 0 ){
            val = csd->fg_lofg[i].fi[ia]; return val;
          }
        }
      }
    }
    else if (lof_type == 5){ /* elastic df (deviation from the neutral) */
      printf("$$ Sorry, lof_type=5 is not available. Abort.\n");
      exit(EXIT_FAILURE);
    }
    else {
      printf("$$(fget_low_g_cst) Invalid lof_type(=%d). Abort.\n", lof_type);
      exit(EXIT_FAILURE);
    }
  }
  
  printf("$$ (val_search_csd) constraint %s is not found. Abort.\n", str);
  exit(EXIT_FAILURE);
  return 0; /* dummy */
}


double val_trans_csd(char *str, CRYSTAL_STRCT_DATA *csd)
{
  int length;
  double val = 0.0;
  
  length = strlen(str);
  if (length == 0){	/* null */
    return 0.0;
  }
  
  switch(*(str+0)){
  case '0' : case '1' : case '2' : case '3' :	case '4' : case '5' :
  case '6' : case '7' : case '8' : case '9' :	/* number */
    val = atof(str);
    break;
  default:		/* argument */
    val = val_search_csd(str, csd);
    break;
  }
  
  return val;
}


double constraint_value_csd(char *a, CRYSTAL_STRCT_DATA *csd)
{
  /*
   * Calculations of constrained values
   * available operators: =+-*
   */
  int		i, len;
  int		flag_constraint;
  double	val, tval;
  char	buf[STRLEN];
  
  i = 0;	len = 0;
  flag_constraint = 0;
  val = 0.0;	tval = 1.0;
  while(*(a+i) != '\0'){
    /* printf("%c", *(a+i)); */
    if(*(a+i) == ' '){ /* skip */
      ;
    }
    else if(*(a+i) == '='){ /* constraint begins */
      flag_constraint = 1;
    }
    else if(*(a+i) == '*'){
      *(buf+len) = '\0';			
      tval *= val_trans_csd(buf, csd);
      len = 0;
      /* printf("[%s]", buf);	printf("{%lf}", tval); */
    }
    else if(*(a+i) == '+'){
      *(buf+len) = '\0';
      tval *= val_trans_csd(buf, csd);
      val += tval;
      /* printf("[%s]", buf);	printf("{%lf}", tval);	printf("{{%lf}}", val); */
      len = 0;	tval = 1.0;
      
    }
    else if(*(a+i) == '-'){
      *(buf+len) = '\0';
      tval *= val_trans_csd(buf, csd);
      val += tval;
      /* printf("[%s]", buf);	printf("{%lf}", tval);	printf("{{%lf}}", val); */
      len = 0;	tval = -1.0;
    }
    else{
      if(len < LEN_MAX){
        *(buf+len) = *(a+i);
        len++;
      }
      else{
        *(buf+len-1) = '\0';
        printf("$$ Argument %s is too long. Please rename it.\n", buf);
        exit(EXIT_FAILURE);
      }
    }
    i++;
  }
  *(buf+len) = '\0';
  tval *= val_trans_csd(buf, csd);
  val += tval;
  /* printf("[%s]", buf);	printf("{%lf}", tval);	printf("{{%lf}}", val); */
  
  return val;
}


double constraint_checker_csd( int flag, double val, char *cstr, CRYSTAL_STRCT_DATA *csd )
{
  if (flag == -1){
    return constraint_value_csd(cstr, csd);
  }
  else{
    return val;
  }
}


void constraint_csd_lp(LAT_PAR *lp, LATPAR_FIT *lp_f, CRYSTAL_STRCT_DATA *csd)
{
  lp->a = constraint_checker_csd(lp_f->a.flg, lp->a, lp_f->a.cstr, csd);
  lp->b = constraint_checker_csd(lp_f->b.flg, lp->b, lp_f->b.cstr, csd);
  lp->c = constraint_checker_csd(lp_f->c.flg, lp->c, lp_f->c.cstr, csd);
  lp->alpha = constraint_checker_csd(lp_f->alpha.flg, lp->alpha, lp_f->alpha.cstr, csd);
  lp->beta  = constraint_checker_csd(lp_f->beta.flg,  lp->beta,  lp_f->beta.cstr,  csd);
  lp->gamma = constraint_checker_csd(lp_f->gamma.flg, lp->gamma, lp_f->gamma.cstr, csd);
}


void constraint_csd_atm(
  int n_atom, ATOM *atm, ATOM_FIT *atm_f, CRYSTAL_STRCT_DATA *csd
  )
{
  int i;
  
  for(i=0; i<n_atom; i++){
    atm[i].x =
      constraint_checker_csd(atm_f[i].x.flg, atm[i].x, atm_f[i].x.cstr, csd);
    atm[i].y =
      constraint_checker_csd(atm_f[i].y.flg, atm[i].y, atm_f[i].y.cstr, csd);
    atm[i].z =
      constraint_checker_csd(atm_f[i].z.flg, atm[i].z, atm_f[i].z.cstr, csd);
    
    /* ----- flag: B-factor ----- */
    if ( atm[i].bf == 0 ){				/* Isotoropic */
      atm[i].b =
        constraint_checker_csd(atm_f[i].b.flg, atm[i].b, atm_f[i].b.cstr, csd);
    }
    else if ( atm[i].bf == 1 ){			/* anisotropic */
      atm[i].b11 =
        constraint_checker_csd(
          atm_f[i].b11.flg, atm[i].b11, atm_f[i].b11.cstr, csd
          );
      atm[i].b22 =
        constraint_checker_csd(
          atm_f[i].b22.flg, atm[i].b22, atm_f[i].b22.cstr, csd
          );
      atm[i].b33 =
        constraint_checker_csd(
          atm_f[i].b33.flg, atm[i].b33, atm_f[i].b33.cstr, csd
          );
      atm[i].b12 =
        constraint_checker_csd(
          atm_f[i].b12.flg, atm[i].b12, atm_f[i].b12.cstr, csd
          );
      atm[i].b13 =
        constraint_checker_csd(
          atm_f[i].b13.flg, atm[i].b13, atm_f[i].b13.cstr, csd
          );
      atm[i].b23 =
        constraint_checker_csd(
          atm_f[i].b23.flg, atm[i].b23, atm_f[i].b23.cstr, csd
          );
    }
    else {
      printf("$$ Invalid bf. Abort.\n");
      exit(EXIT_FAILURE);
    }
    
    atm[i].occ =
      constraint_checker_csd(
        atm_f[i].occ.flg, atm[i].occ, atm_f[i].occ.cstr, csd
        );
  }
}


void constraint_csd_lofg(
  int n_atom, int num_lofg, HKL *g_lofg, int lof_type,
  LO_FG *fg_lofg, LO_FG_FIT *fg_lofg_f,
  CRYSTAL_STRCT_DATA *csd
  )
{
  int i, ia, n;
  
  n = num_lofg;
  
  if (lof_type == 1){			/* 1: whole part of Fg */
    for(i=0; i<n; i++){
      fg_lofg[i].re =
        constraint_checker_csd(
          fg_lofg_f[i].re.flg, fg_lofg[i].re, fg_lofg_f[i].re.cstr, csd
          );
      fg_lofg[i].im =
        constraint_checker_csd(
          fg_lofg_f[i].im.flg, fg_lofg[i].im, fg_lofg_f[i].im.cstr, csd
          );
    }
  }
  else if (lof_type == 2){	/* 2: elastic part of Fg */
    for(i=0; i<n; i++){
      fg_lofg[i].ela_re = constraint_checker_csd(
        fg_lofg_f[i].ela_re.flg, fg_lofg[i].ela_re,
        fg_lofg_f[i].ela_re.cstr, csd
        );
      fg_lofg[i].ela_im = constraint_checker_csd(
        fg_lofg_f[i].ela_im.flg, fg_lofg[i].ela_im,
        fg_lofg_f[i].ela_im.cstr, csd
        );
    }
  }	else if ( lof_type == 3 || lof_type == 4 ){
    /* 3: both f (elastic and inelastic), 4: elastic f */
    for(i=0; i<n; i++){
      for(ia=0; ia<n_atom; ia++){
        fg_lofg[i].fe[ia]
          = constraint_checker_csd(
            fg_lofg_f[i].fe[ia].flg, fg_lofg[i].fe[ia],
            fg_lofg_f[i].fe[ia].cstr, csd
            );
        fg_lofg[i].fi[ia]
          = constraint_checker_csd(
            fg_lofg_f[i].fi[ia].flg, fg_lofg[i].fi[ia],
            fg_lofg_f[i].fi[ia].cstr, csd
            );
      }
    }
  }
  else if (lof_type == 5){		/* elastic df (deviation from the neutral) */
    printf("$$ Sorry, lof_type=5 is not available. Abort.\n");
    exit(EXIT_FAILURE);
  }
  else {
    printf("$$(fget_low_g_cst) Invalid lof_type(=%d). Abort.\n", lof_type);
    exit(EXIT_FAILURE);
  }
  
}


void constraint_csd(CRYSTAL_STRCT_DATA *csd)
{
  constraint_csd_lp(&(csd->lp), &(csd->lp_f), csd);
  constraint_csd_atm(csd->n_atom, csd->atm, csd->atm_f, csd);
  if( csd->num_lofg > 0 ){
    constraint_csd_lofg(
      csd->n_atom, csd->num_lofg,
      csd->g_lofg, csd->lof_type, csd->fg_lofg, csd->fg_lofg_f, csd
      );
  }
}


/* FED -------------------------------------------------------------------- */
/* ------------------------------------------------------------------------ */


double val_search_fed(char *str, int num_exp_dataset, FIT_EXP_DATASET *fed)
{
  double	val = 0.0;
  int		i, j, jj;
  
  for (i=0; i<num_exp_dataset; i++){
    if ( strcmp(str, fed[i].H_tilt_f.name) == 0 ){
      val = fed[i].H_tilt;	return val;
    }
    if ( strcmp(str, fed[i].G_tilt_f.name) == 0 ){
      val = fed[i].G_tilt;	return val;
    }
    if ( strcmp(str, fed[i].accel_kvol_f.name) == 0 ){
      val = fed[i].accel_kvol;	return val;
    }
    
    for(j=0; j<fed[i].num_thickness; j++){
      if ( strcmp(str, fed[i].scale_factor_f[j].name) == 0 ){
        val = fed[i].scale_factor[j];	return val;
      }
      if ( strcmp(str, fed[i].thickness_f[j].name) == 0 ){
        val = fed[i].thickness[j];	return val;
      }
      if ( strcmp(str, fed[i].thickwidth_f[j].name) == 0 ){
        val = fed[i].thickwidth[j];	return val;
      }
      for(jj=0; jj<fed[i].num_rg; jj++){
        if ( strcmp(str, fed[i].H_shift_f[j][jj].name) == 0 ){
          val = fed[i].H_shift[j][jj];	return val;
        }
        if ( strcmp(str, fed[i].G_shift_f[j][jj].name) == 0 ){
          val = fed[i].G_shift[j][jj];	return val;
        }
      }
    }
  }
  
  printf("$$ (val_search_fed) constraint %s is not found. Abort.\n", str);
  exit(EXIT_FAILURE);
}


double val_trans_fed(char *str, int num_exp_dataset, FIT_EXP_DATASET *fed)
{
  int		length;
  double	val = 0.0;
  
  length = strlen(str);
  if (length == 0){	/* null */
    return 0.0;
  }
  
  switch(*(str+0)){
  case '0' : case '1' : case '2' : case '3' :	case '4' : case '5' :
  case '6' : case '7' : case '8' : case '9' :	/* number */
    val = atof(str);
    break;
  default:		/* argument */
    val = val_search_fed(str, num_exp_dataset, fed);
    break;
  }
  
  return val;
}


double constraint_value_fed(char *a, int num_exp_dataset, FIT_EXP_DATASET *fed)
{
  /*
    Calculations of constrained values
    available operators: =+-*
   */
  int		i, len;
  int		flag_constraint;
  double	val, tval;
  char	buf[256];
  
  i = 0;	len = 0;
  flag_constraint = 0;
  val = 0.0;	tval = 1.0;
  while(*(a+i) != '\0'){
    /* printf("%c", *(a+i)); */
    if(*(a+i) == ' '){ /* skip */
      ;
    }
    else if(*(a+i) == '='){ /* constraint begins */
      flag_constraint = 1;
    }
    else if(*(a+i) == '*'){
      *(buf+len) = '\0';			
      tval *= val_trans_fed(buf, num_exp_dataset, fed);
      len = 0;
      /* printf("[%s]", buf);	printf("{%lf}", tval); */
    }
    else if(*(a+i) == '+'){
      *(buf+len) = '\0';
      tval *= val_trans_fed(buf, num_exp_dataset, fed);
      val += tval;
      /* printf("[%s]", buf);	printf("{%lf}", tval);	printf("{{%lf}}", val); */
      len = 0;	tval = 1.0;
      
    }
    else if(*(a+i) == '-'){
      *(buf+len) = '\0';
      tval *= val_trans_fed(buf, num_exp_dataset, fed);
      val += tval;
      /* printf("[%s]", buf);	printf("{%lf}", tval);	printf("{{%lf}}", val); */
      len = 0;	tval = -1.0;
    }
    else{
      if(len < LEN_MAX){
        *(buf+len) = *(a+i);
        len++;
      }
      else{
        *(buf+len-1) = '\0';
        printf("$$ Argument %s is too long. Please rename it.\n", buf);
        exit(EXIT_FAILURE);
      }
    }
    i++;
  }
  *(buf+len) = '\0';
  tval *= val_trans_fed(buf, num_exp_dataset, fed);
  val += tval;
  /* printf("[%s]", buf);	printf("{%lf}", tval);	printf("{{%lf}}", val); */
  
  return val;
}


double constraint_checker_fed( int flag, double val, char *cstr, int num_exp_dataset, FIT_EXP_DATASET *fed )
{
  if (flag == -1){
    return constraint_value_fed(cstr, num_exp_dataset, fed);
  }
  else{
    return val;
  }
}


void constraint_fed(int num_exp_dataset, FIT_EXP_DATASET *fed)
{
  int i, j, jj;

  /* printf("$$ constraint_fed 1\n"); */

  for (i=0; i<num_exp_dataset; i++){
    fed[i].H_tilt
      = constraint_checker_fed(
        fed[i].H_tilt_f.flg, fed[i].H_tilt,
        fed[i].H_tilt_f.cstr, num_exp_dataset, fed
        );
    fed[i].G_tilt
      = constraint_checker_fed(
        fed[i].G_tilt_f.flg, fed[i].G_tilt,
        fed[i].G_tilt_f.cstr, num_exp_dataset, fed
        );
    fed[i].accel_kvol
      = constraint_checker_fed(
        fed[i].accel_kvol_f.flg, fed[i].accel_kvol,
        fed[i].accel_kvol_f.cstr, num_exp_dataset, fed
        );
    for(j=0; j<fed[i].num_thickness; j++){
      fed[i].scale_factor[j]
        = constraint_checker_fed(
          fed[i].scale_factor_f[j].flg, fed[i].scale_factor[j],
          fed[i].scale_factor_f[j].cstr, num_exp_dataset, fed
          );
      fed[i].thickness[j]
        = constraint_checker_fed(
          fed[i].thickness_f[j].flg, fed[i].thickness[j],
          fed[i].thickness_f[j].cstr, num_exp_dataset, fed
          );
      fed[i].thickwidth[j]
        = constraint_checker_fed(
          fed[i].thickwidth_f[j].flg, fed[i].thickwidth[j], 
          fed[i].thickwidth_f[j].cstr, num_exp_dataset, fed
          );
      for(jj=0; jj<fed[i].num_rg; jj++){
        fed[i].H_shift[j][jj]
          = constraint_checker_fed(
            fed[i].H_shift_f[j][jj].flg, fed[i].H_shift[j][jj], 
            fed[i].H_shift_f[j][jj].cstr, num_exp_dataset, fed
            );
        fed[i].G_shift[j][jj]
          = constraint_checker_fed(
            fed[i].G_shift_f[j][jj].flg, fed[i].G_shift[j][jj],
            fed[i].G_shift_f[j][jj].cstr, num_exp_dataset, fed
            );
      }
    }
  }

/*  printf("$$ constraint_fed 2\n"); */
}


/*
  Samples of constraint:
  (ND(1)X, ND(1)Y, AU1X ard AU1Y are names of parameters.)
  
  =-1.5*2.0*ND(1)X+0.5*ND(1)Y-0.5
  =+0.5*2*AU1X-0.33333333*AU1Y
  =+0.5*0.5*AU1X-0.33333333*AU1Y
 */


/* ----- */
FIT_CTRL search_csd_fed(
  int ntp, CRYSTAL_STRCT_DATA *csd, int num_exp_dataset, FIT_EXP_DATASET *fed
  )
{
  FIT_CTRL fit;
  int i, j, jj, n_atom, ia, n;

  if( ntp == csd->lp_f.a.ntp ){ fit = csd->lp_f.a; return fit;}
  if( ntp == csd->lp_f.b.ntp ){ fit = csd->lp_f.b; return fit;}
  if( ntp == csd->lp_f.c.ntp ){ fit = csd->lp_f.c; return fit;}
  if( ntp == csd->lp_f.alpha.ntp ){ fit = csd->lp_f.alpha; return fit;}
  if( ntp == csd->lp_f.beta.ntp ){ fit = csd->lp_f.beta; return fit;}
  if( ntp == csd->lp_f.gamma.ntp ){ fit = csd->lp_f.gamma; return fit;}
  
  n_atom = csd->n_atom;
  for(i=0; i<n_atom; i++){
    if( ntp == csd->atm_f[i].x.ntp ){ fit = csd->atm_f[i].x; return fit;}
    if( ntp == csd->atm_f[i].y.ntp ){ fit = csd->atm_f[i].y; return fit;}
    if( ntp == csd->atm_f[i].z.ntp ){ fit = csd->atm_f[i].z; return fit;}
    if ( csd->atm[i].bf == 0 ){
      if( ntp == csd->atm_f[i].b.ntp ){ fit = csd->atm_f[i].b; return fit;}
    }
    else if ( csd->atm[i].bf == 1 ){
      if( ntp == csd->atm_f[i].b11.ntp ){ fit = csd->atm_f[i].b11; return fit;}
      if( ntp == csd->atm_f[i].b22.ntp ){ fit = csd->atm_f[i].b22; return fit;}
      if( ntp == csd->atm_f[i].b33.ntp ){ fit = csd->atm_f[i].b33; return fit;}
      if( ntp == csd->atm_f[i].b12.ntp ){ fit = csd->atm_f[i].b12; return fit;}
      if( ntp == csd->atm_f[i].b13.ntp ){ fit = csd->atm_f[i].b13; return fit;}
      if( ntp == csd->atm_f[i].b23.ntp ){ fit = csd->atm_f[i].b23; return fit;}
    }
    else {
      printf("$$ Invalid bf. Abort.\n");
      exit(EXIT_FAILURE);
    }
    
    if( ntp == csd->atm_f[i].occ.ntp ){ fit = csd->atm_f[i].occ; return fit;}
  }
  
  n = csd->num_lofg;
  
  if ( n > 0 ) {
    if (csd->lof_type == 1){			/* 1: whole part of Fg */
      for(i=0; i<n; i++){
        if( ntp == csd->fg_lofg_f[i].re.ntp ){
          fit = csd->fg_lofg_f[i].re; return fit;
        }
        if( ntp == csd->fg_lofg_f[i].im.ntp ){
          fit = csd->fg_lofg_f[i].im; return fit;
        }
      }
    }
    else if (csd->lof_type == 2){	/* 2: elastic part of Fg */
      for(i=0; i<n; i++){
        if( ntp == csd->fg_lofg_f[i].ela_re.ntp ){
          fit = csd->fg_lofg_f[i].ela_re; return fit;
        }
        if( ntp == csd->fg_lofg_f[i].ela_im.ntp ){
          fit = csd->fg_lofg_f[i].ela_im; return fit;
        }
      }
    }
    else if ( csd->lof_type == 3 || csd->lof_type == 4 ){
      /* 3: both f (elastic and inelastic), 4: elastic f */
      for(i=0; i<n; i++){
        for(ia=0; ia<n_atom; ia++){
          if( ntp == csd->fg_lofg_f[i].fe[ia].ntp ){
            fit = csd->fg_lofg_f[i].fe[ia]; return fit;
          }
          if( ntp == csd->fg_lofg_f[i].fi[ia].ntp ){
            fit = csd->fg_lofg_f[i].fi[ia]; return fit;
          }
        }
      }
    }
    else {
      printf(
        "$$(fget_low_g_cst) Invalid lof_type(=%d). Abort.\n", csd->lof_type
        );
      exit(EXIT_FAILURE);
    }
  }

  for (i=0; i<num_exp_dataset; i++){
    if( ntp == fed[i].H_tilt_f.ntp ){
      fit = fed[i].H_tilt_f; return fit;
    }
    if( ntp == fed[i].G_tilt_f.ntp ){
      fit = fed[i].G_tilt_f; return fit;
    }
    if( ntp == fed[i].accel_kvol_f.ntp ){
      fit = fed[i].accel_kvol_f; return fit;
    }
    for(j=0; j<fed[i].num_thickness; j++){
      if( ntp == fed[i].scale_factor_f[j].ntp ){
        fit = fed[i].scale_factor_f[j]; return fit;
      }
      if( ntp == fed[i].thickness_f[j].ntp ){
        fit = fed[i].thickness_f[j]; return fit;
      }
      if( ntp == fed[i].thickwidth_f[j].ntp ){
        fit = fed[i].thickwidth_f[j]; return fit;
      }
      for(jj=0; jj<fed[i].num_rg; jj++){
        if( ntp == fed[i].H_shift_f[j][jj].ntp ){
          fit = fed[i].H_shift_f[j][jj]; return fit;
        }
        if( ntp == fed[i].G_shift_f[j][jj].ntp ){
          fit = fed[i].G_shift_f[j][jj]; return fit;
        }
      }
    }
  }

  printf("$$ (search_csd_fed) Invalid ntp=%d. Abort.\n", ntp);
  exit(EXIT_FAILURE);
  return fit;  /* dummy */
}


void fitpar_sigma_to_csd_fed(
  CRYSTAL_STRCT_DATA *csd,  int num_exp_dataset, FIT_EXP_DATASET *fed,
  int num_fitpar, double **covar
  )
{
  int  i, j, jj, n_atom, ia, n, ntp;
  double  sigma;

  for(ntp=0; ntp<num_fitpar; ntp++) {
    sigma = sqrt(covar[ntp][ntp]);

    if( ntp == csd->lp_f.a.ntp ){ csd->lp_f.a.esd = sigma; }
    if( ntp == csd->lp_f.b.ntp ){ csd->lp_f.b.esd = sigma; }
    if( ntp == csd->lp_f.c.ntp ){ csd->lp_f.c.esd = sigma; }
    if( ntp == csd->lp_f.alpha.ntp ){ csd->lp_f.alpha.esd = sigma; }
    if( ntp == csd->lp_f.beta.ntp ){ csd->lp_f.beta.esd = sigma; }
    if( ntp == csd->lp_f.gamma.ntp ){ csd->lp_f.gamma.esd = sigma; }

    n_atom = csd->n_atom;
    for(i=0; i<n_atom; i++){
      if( ntp == csd->atm_f[i].x.ntp ){ csd->atm_f[i].x.esd = sigma; }
      if( ntp == csd->atm_f[i].y.ntp ){ csd->atm_f[i].y.esd = sigma; }
      if( ntp == csd->atm_f[i].z.ntp ){ csd->atm_f[i].z.esd = sigma; }
      if ( csd->atm[i].bf == 0 ){
        if( ntp == csd->atm_f[i].b.ntp ){ csd->atm_f[i].b.esd = sigma; }
      }
      else if ( csd->atm[i].bf == 1 ){
        if( ntp == csd->atm_f[i].b11.ntp ){ csd->atm_f[i].b11.esd = sigma; }
        if( ntp == csd->atm_f[i].b22.ntp ){ csd->atm_f[i].b22.esd = sigma; }
        if( ntp == csd->atm_f[i].b33.ntp ){ csd->atm_f[i].b33.esd = sigma; }
        if( ntp == csd->atm_f[i].b12.ntp ){ csd->atm_f[i].b12.esd = sigma; }
        if( ntp == csd->atm_f[i].b13.ntp ){ csd->atm_f[i].b13.esd = sigma; }
        if( ntp == csd->atm_f[i].b23.ntp ){ csd->atm_f[i].b23.esd = sigma; }
      }
      else {
        printf("$$ Invalid bf. Abort.\n");
        exit(EXIT_FAILURE);
      }
      
      if( ntp == csd->atm_f[i].occ.ntp ){ csd->atm_f[i].occ.esd = sigma;}
    }

    n = csd->num_lofg;

    if ( n > 0 ) {
      if (csd->lof_type == 1){			/* 1: whole part of Fg */
        for(i=0; i<n; i++){
          if( ntp == csd->fg_lofg_f[i].re.ntp ){
            csd->fg_lofg_f[i].re.esd = sigma;
          }
          if( ntp == csd->fg_lofg_f[i].im.ntp ){
            csd->fg_lofg_f[i].im.esd = sigma;
          }
        }
      }
      else if (csd->lof_type == 2){	/* 2: elastic part of Fg */
        for(i=0; i<n; i++){
          if( ntp == csd->fg_lofg_f[i].ela_re.ntp ){
            csd->fg_lofg_f[i].ela_re.esd = sigma;
          }
          if( ntp == csd->fg_lofg_f[i].ela_im.ntp ){
            csd->fg_lofg_f[i].ela_im.esd = sigma;
          }
        }
      }
      else if ( csd->lof_type == 3 || csd->lof_type == 4 ){
        /* 3: both f (elastic and inelastic), 4: elastic f */
        for(i=0; i<n; i++){
          for(ia=0; ia<n_atom; ia++){
            if( ntp == csd->fg_lofg_f[i].fe[ia].ntp ){
              csd->fg_lofg_f[i].fe[ia].esd = sigma;
            }
            if( ntp == csd->fg_lofg_f[i].fi[ia].ntp ){
              csd->fg_lofg_f[i].fi[ia].esd = sigma;
            }
          }
        }
      }
      else {
        printf(
          "$$(fget_low_g_cst) Invalid lof_type(=%d). Abort.\n",
          csd->lof_type
          );
        exit(EXIT_FAILURE);
      }
    }

    for (i=0; i<num_exp_dataset; i++){
      if( ntp == fed[i].H_tilt_f.ntp ){ fed[i].H_tilt_f.esd = sigma; }
      if( ntp == fed[i].G_tilt_f.ntp ){ fed[i].G_tilt_f.esd = sigma; }
      if( ntp == fed[i].accel_kvol_f.ntp ){
        fed[i].accel_kvol_f.esd = sigma;
      }
      for(j=0; j<fed[i].num_thickness; j++){
        if( ntp == fed[i].scale_factor_f[j].ntp ){
          fed[i].scale_factor_f[j].esd = sigma;
        }
        if( ntp == fed[i].thickness_f[j].ntp ){
          fed[i].thickness_f[j].esd = sigma;
        }
        if( ntp == fed[i].thickwidth_f[j].ntp ){
          fed[i].thickwidth_f[j].esd = sigma;
        }
        for(jj=0; jj<fed[i].num_rg; jj++){
          if( ntp == fed[i].H_shift_f[j][jj].ntp ){
          fed[i].H_shift_f[j][jj].esd = sigma;
          }
          if( ntp == fed[i].G_shift_f[j][jj].ntp ){
            fed[i].G_shift_f[j][jj].esd = sigma;
          }
        }
      }
    }
  }
}
