/*
  mb_debug.c

  The function names of mb_debug.c begin with "dbg_",
   except for internal functions.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "mbconfig.h"
#include "fitting.h"
#include "mb_input.h"
#include "data_out.h"
#include "mb_mpi.h"

#define  BUFLEN 256

extern DEBUG_INFO dbg;
extern EPS_CTRL	epsilon;

void constraint_csd(CRYSTAL_STRCT_DATA *csd);


int second_int(char *buf)
{
  char *delims=" ,\t", *token;
  token = strtok(buf,delims);
  return atoi( strtok(NULL,delims) );
}


double second_double(char *buf)
{
  char *delims=" ,\t", *token;
  token = strtok(buf,delims);
  return atof( strtok(NULL,delims) );
}


void read_ini_file(void)
     /*
      * read .ini file for getting debug information
      * modified on 2000.12.20
      */
{
  FILE *fi;
  char buf[128];
  
  /*
   * Initialize (set default values)
   */
  dbg.eigval_dump_flag	= 0;
  dbg.eigvec_dump_flag	= 0;
  dbg.reflex_dump_flag	= 0;
  dbg.sg_dump_flag		= 0;
  dbg.betag_dump_flag	= 0;
  dbg.vg_dump_flag		= 0;
  dbg.reflex_only_flag	= 0;
  dbg.a_mat_dump_flag	= 0;
  dbg.b_mat_dump_flag	= 0;
  dbg.b_cond_dump_flag	= 0;
  dbg.all_int_out_flag	= 0;
  dbg.covar_dump_flag	= 0;
  dbg.alpha_dump_flag	= 0;
  dbg.beta_dump_flag	= 0;
  dbg.obs_cal_dump_flag	= 0;
  dbg.ideal_cr_cal_flag	= 0;
  dbg.atm_pos_out_flag	= 0;
  dbg.vg_sel_chk_flag 	= 0;
  dbg.process_mon_flag	= 0;
  dbg.forecast_flag		= 1;
  dbg.wg_xig_list_flag	= 0;
  dbg.print_bij_flag	= 1;
  dbg.chk_data_dump_flag = 0;
  if( (fi = fopen("mbfit.ini","r")) == NULL ){	/* no .ini file */
    mb_fprintf(stdout, "Cannot find mbfit.ini file.  No options.\n");
  }
  else{	/* .ini file exists. */
    mb_fprintf(stdout, "Reading mbfit.ini\n");  mb_fflush(stdout);
    while( comment_skip_fgets(80, buf, fi) != NULL ){
      /* ----- DEBUG_INFO ----- */
      if (strstr(buf,"eigval_dump_flag") != NULL){
        dbg.eigval_dump_flag = second_int(buf);	/* dump eigen values */
      }
      else if (strstr(buf,"eigvec_dump_flag") != NULL){
        dbg.eigvec_dump_flag = second_int(buf);	/* dump eigen values */
      }
      else if (strstr(buf,"sg_dump_flag") != NULL){
        dbg.sg_dump_flag = second_int(buf);		/* dump Sg */
      }
      else if (strstr(buf,"betag_dump_flag") != NULL){
        dbg.betag_dump_flag = second_int(buf);	/* dump beta_g */
      }
      else if (strstr(buf,"reflex_dump_flag") != NULL){
        dbg.reflex_dump_flag = second_int(buf);	/* dump reflections */
      }
      else if (strstr(buf,"reflex_only_flag") != NULL){
        dbg.reflex_only_flag = second_int(buf);	/* dump reflections only (exit) */
      }
      else if (strstr(buf,"vg_dump_flag") != NULL){
        dbg.vg_dump_flag = second_int(buf);		/* dump Vg */
      }
      else if (strstr(buf,"a_mat_dump_flag") != NULL){
        dbg.a_mat_dump_flag = second_int(buf);		/* dump a_matrix */
      }
      else if (strstr(buf,"b_mat_dump_flag") != NULL){
        dbg.b_mat_dump_flag = second_int(buf);		/* dump bethe_matrix */
      }
      else if (strstr(buf,"b_cond_dump_flag") != NULL){
        dbg.b_cond_dump_flag = second_int(buf);		/* dump boundary cond. */
      }
      else if (strstr(buf,"bc_chk_flag") != NULL){
        dbg.bc_chk_flag = second_int(buf);		/* Check boundary cond. */
      }
      else if (strstr(buf,"all_int_out_flag") != NULL){
        dbg.all_int_out_flag = second_int(buf);	/* dump intensities of all cycles */
      }
      else if (strstr(buf,"covar_dump_flag") != NULL){
        dbg.covar_dump_flag = second_int(buf);	/* dump par_step */
      }
      else if (strstr(buf,"alpha_dump_flag") != NULL){
        dbg.alpha_dump_flag = second_int(buf);
      }
      else if (strstr(buf,"beta_dump_flag") != NULL){
        dbg.beta_dump_flag = second_int(buf);
      }
      else if (strstr(buf,"obs_cal_dump_flag") != NULL){
        dbg.obs_cal_dump_flag = second_int(buf);
      }
      else if (strstr(buf,"ideal_cr_cal_flag") != NULL){
        dbg.ideal_cr_cal_flag = second_int(buf);	/* output ideal .cr and .cal */
      }
      else if (strstr(buf,"atm_pos_out_flag") != NULL){
        dbg.atm_pos_out_flag = second_int(buf);		/* dump generated atom positions */
      }
      else if (strstr(buf,"vg_sel_chk_flag") != NULL){
        dbg.vg_sel_chk_flag = second_int(buf);
      }
      else if (strstr(buf,"process_mon_flag") != NULL){
        dbg.process_mon_flag = second_int(buf);
      }
      else if (strstr(buf,"forecast_flag") != NULL){
        dbg.forecast_flag = second_int(buf);
      }
      else if (strstr(buf,"wg_xig_list_flag") != NULL) {
        dbg.wg_xig_list_flag = second_int(buf);
      }
      else if (strstr(buf,"print_bij_flag") != NULL) {
        dbg.print_bij_flag = second_int(buf);
      }
      else if (strstr(buf,"chk_data_dump_flag") != NULL) {
        dbg.chk_data_dump_flag = second_int(buf);
      }
      else if (strstr(buf,"INI_PARAM_END") != NULL) {
        goto NORMAL;
      }
      else{
        mb_fprintf(stdout, "%s is ignored.\n", rm_cr(buf) );
      }
      
    }
    /* ABNORMAL: */
    mb_fprintf(stdout, "\n$$ No INI_PARAM_END command in mbfit.ini. Abort.\n");
    exit(EXIT_FAILURE);
    
  NORMAL:
    fclose(fi);
  }
  
}


void read_eps_ctrl_params( FILE *fi )
     /* 2000.12.20 */
{
  char buf[256];

  /* Initialize (set default values) */
  epsilon.eps				= 1e-10;
  epsilon.eigen			= 1e-8;
  epsilon.inv				= 1e-8;
  epsilon.degeneracy		= 1e-8;
  epsilon.perturb_step	= 1e-3;
  epsilon.tol_chisq_abs	= 1e-2;
  epsilon.tol_chisq_rel	= 1e-3;
  epsilon.tol_parameter	= 1e-3;
  epsilon.intensity_scaling = 1e+4;
  epsilon.corr_threshold	= 0.5;
  /*	epsilon.w_a				= 0.0;
	epsilon.w_b				= 0.0;
	epsilon.w_c				= 0.0;
	epsilon.w_d				= 0.0;
	epsilon.w_e				= 0.0;
	epsilon.w_f				= 0.33333;	*/
  epsilon.w_sq_coef   = 1.0;
  epsilon.w_exp_coef  = 0.0;
  epsilon.w_exp       = 4.0;
  /*
  epsilon.hbgn			= -1.0;
  epsilon.hend			= 1.0;
  epsilon.gbgn			= -1.0;
  epsilon.gend			= 1.0;
  */
  epsilon.bc_accuracy		= 1.0;		/* 1.0: no effect */
  epsilon.vg_select_flag	= 1;		/* vg selection for each tilt */
  epsilon.diagonalize_func	= 0;/* diagonalization func, 0:Eispack, 1:Numpack */
  epsilon.svd_flag = 1;  /* 1:svd, 0:gaussj */
  epsilon.no_fitting_flag = 0; /* 0: fitting, 1: no fitting */
  epsilon.num_thickwidth = 0; /* num_thickwidth * 2 + 1 --> calc. points */
  epsilon.thickwidth_type = 2; /* 1% 0:rectangle, 1:triangle, 2:Gaussian */
  epsilon.cal_range_flag	= 0;		/* <<<<< */
  epsilon.numh			= 20;		/* <<<<< */
  epsilon.auto_multiplicity_off = 0;	/* 0:auto_multiplicity 1:manual*/
  epsilon.command_line_flag = 0;
  epsilon.message_exit_flag = 1;

  chk_data("EPS_CONTROL_PARAMETERS{", fi);

  comment_skip_fgets(128, rm_cr(buf), fi);
  /*	while( comment_skip_fgets(128, rm_cr(buf), fi) != "CTRLPAR_END" ){ */
  while( strstr(buf, "}EPS_CONTROL_PARAMETERS" ) == NULL ){
    /* ----- EPS_CTRL ----- */
    if (strstr(buf,"general_eps") != NULL){
      epsilon.eps	= second_double(buf);
      /*printf("$$ epsilon.eps = %lg\n", epsilon.eps);*/
    }
    else if (strstr(buf,"eigen_eps") != NULL){
      epsilon.eigen	= second_double(buf);
      /*printf("$$ epsilon.eigen = %lg\n", epsilon.eigen);*/
    }
    else if (strstr(buf,"inv_eps") != NULL){
      epsilon.inv	= second_double(buf);
    }
    else if (strstr(buf,"degeneracy_eps") != NULL){
      epsilon.degeneracy	= second_double(buf);
    }
    else if (strstr(buf,"perturb_step") != NULL){
      epsilon.perturb_step	= second_double(buf);
    }
    else if (strstr(buf,"tol_chisq_abs") != NULL){
      epsilon.tol_chisq_abs	= second_double(buf);
    }
    else if (strstr(buf,"tol_chisq_rel") != NULL){
      epsilon.tol_chisq_rel	= second_double(buf);
    }
    else if (strstr(buf,"tol_parameter") != NULL){
      epsilon.tol_parameter	= second_double(buf);
    }
    else if (strstr(buf,"intensity_scaling") != NULL){
      epsilon.intensity_scaling	= second_double(buf);
    }
    else if (strstr(buf,"bc_accuracy") != NULL){
      epsilon.bc_accuracy	= second_double(buf);
    }
    else if (strstr(buf,"bc_accuracy") != NULL){
      epsilon.bc_accuracy	= second_double(buf);
    }
    else if (strstr(buf,"corr_threshold") !=NULL){
      epsilon.corr_threshold = second_double(buf);
    }
    else if (strstr(buf,"w_sq_coef") !=NULL){
      epsilon.w_sq_coef = second_double(buf);
    }
    else if (strstr(buf,"w_exponent_coef") !=NULL){
      epsilon.w_exp_coef = second_double(buf);
    }
    else if (strstr(buf,"w_exponent") !=NULL){
      epsilon.w_exp = second_double(buf);
    }
    else if (strstr(buf,"vg_select_flag") != NULL){
      epsilon.vg_select_flag = second_int(buf);
    }
    else if (strstr(buf,"diagonalize_func") != NULL) {
      epsilon.diagonalize_func = second_int(buf);
    }
    else if (strstr(buf,"svd_flag") != NULL) {
      epsilon.svd_flag = second_int(buf);
    }
    else if (strstr(buf,"no_fitting_flag") != NULL) {
      epsilon.no_fitting_flag = second_int(buf);
    }
    else if (strstr(buf,"thickwidth_type") != NULL) {
      epsilon.thickwidth_type = second_int(buf);
    }
    else if (strstr(buf,"num_thickwidth") != NULL) {
      epsilon.num_thickwidth = second_int(buf);
    }
    else if (strstr(buf,"cal_range_flag") != NULL) {
      epsilon.cal_range_flag = second_int(buf);
    }
    else if (strstr(buf,"numh") != NULL) {
      epsilon.numh = second_int(buf);
    }
    else if (strstr(buf,"auto_multiplicity_off") != NULL) {
      epsilon.auto_multiplicity_off = second_int(buf);
    }
    else{
      mb_fprintf(
        stdout,
        "$$ The next line could not be recognized.\n%s\n", rm_cr(buf)
        );
    }
    comment_skip_fgets(128, rm_cr(buf), fi);
  }
}


void fprintf_eps_ctrl_params( FILE *fo )
{
  /* Updated on 2001.8.31 */
#if 0
  mb_fprintf(fo, "epsilon.eps\t\t%lg", epsilon.eps); mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.eigen\t\t%lg", epsilon.eigen); mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.inv\t\t%lf", epsilon.inv); mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.degeneracy\t\t%lf", epsilon.degeneracy);
  mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.perturb_step\t\t%lf", epsilon.perturb_step);
  mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.tol_chisq_abs\t\t%lf", epsilon.tol_chisq_abs);
  mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.tol_chisq_rel\t\t%lf", epsilon.tol_chisq_rel);
  mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.tol_parameter\t\t%lf", epsilon.tol_parameter);
  mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.intensity_scaling\t\t%lf", epsilon.intensity_scaling);
  mb_fprintf(fo, "\t%%\n");
  mb_fprintf(fo, "epsilon.corr_threshold\t\t%lf", epsilon.corr_threshold);
  mb_fprintf(fo, "\t%%\n");
#endif
  
#if 0
  mb_fprintf(fo, "epsilon.w_sq_coef\t\t%lf", epsilon.w_sq_coef);
  mb_fprintf(fo, "\t\t%%1/(weight_sq_coef*sq(sig)), std=1.0\n");
  mb_fprintf(fo, "epsilon.w_exp_coef\t\t%lf", epsilon.w_exp_coef);
  mb_fprintf(fo, "\t\t%%1/(w_exponent_coef*pow(sig,w_exponent)), std=0.0\n");
  mb_fprintf(fo, "epsilon.w_exp\t\t%lf", epsilon.w_exp);
  mb_fprintf(fo, "\t\t%%std=2.0 or 4.0  (caution: double)\n");
#endif
  /*	mb_fprintf(fo, "epsilon.bc_accuracy\t\t%lf", epsilon.bc_accuracy); 	mb_fprintf(fo, "%%\n");*/
  mb_fprintf(fo, "epsilon.vg_select_flag\t\t%d", epsilon.vg_select_flag);
  mb_fprintf(fo, "\t\t%% vg selection for each tilt (1:on, 0:off)\n");
  
#if 0
  mb_fprintf(fo, "epsilon.diagonalize_func\t\t%d", epsilon.diagonalize_func);
  mb_fprintf(fo, "\t\t%% 0:Eispack, 1:Numpack\n");
#endif
  mb_fprintf(fo, "epsilon.svd_flag\t\t%d", epsilon.svd_flag);
  mb_fprintf(fo, "\t\t%% 1:svd, 0:gaussj\n");
  mb_fprintf(fo, "epsilon.no_fitting_flag\t\t%d", epsilon.no_fitting_flag);
  mb_fprintf(fo, "\t\t%% 1:no fitting, 0:fitting\n");
  mb_fprintf(fo, "epsilon.num_thickwidth\t\t%lg", epsilon.num_thickwidth);
  mb_fprintf(fo, "\t\t%% num_thickwidth*2+1 --> calc. points\n");
  mb_fprintf(fo, "epsilon.thickwidth_type\t\t%d", epsilon.thickwidth_type);
  mb_fprintf(fo, "\t\t%% 0:rectangle, 1:triangle, 2:Gaussian\n");
  mb_fprintf(fo, "epsilon.cal_range_flag\t\t%d", epsilon.cal_range_flag);
  mb_fprintf(fo, "\t\t%% 0:rectangle, 1:triangle, 2:Gaussian\n");
  /*	mb_fprintf(fo, "epsilon.numh\t\t%d", epsilon.numh); */
  mb_fprintf(fo, "epsilon.auto_multiplicity_off\t%d", epsilon.auto_multiplicity_off);
  mb_fprintf(fo, "\t\t%% 0:auto_multiplicity 1:manual.\n");
}


void dbg_dump_ini_fit(void)
{
  /* mb_fprintf(stdout, "\n$$ ***** Check !!! ***** \n"); */
  /* printf("$$ atomf_flag_bk = %d\n", epsilon.atomf_flag_bk); */
  mb_fprintf(stdout, "\n$$ vg_select_flag = %d\n", epsilon.vg_select_flag);
  mb_fprintf(stdout, "$$ svd_flag = %d\n", epsilon.svd_flag);
}


void dbg_dump_ini_sim(void)
{
  /* mb_fprintf(stdout, "\n$$ ***** Check !!! ***** \n"); */
  /*	printf("$$ atomf_flag_bk = %d\n", epsilon.atomf_flag_bk);*/
  mb_fprintf(stdout, "\n$$ vg_select_flag = %d\n", epsilon.vg_select_flag);
}


void exec_time_forecast(long ind)
{
  /*
    Execution time forecast
   */
  static double c1,c2;
  double diff_t;
  
  if(dbg.forecast_flag == 0){return;}
  
  if(ind == -1){ /* Initialize */
    mb_fprintf(stdout, "    estimating exec. time .... ");
    mb_fflush(stdout);
    c1 = clock();
  }
  else if(ind == 0){ /* Elapsed time */
    c2 = clock();
    mb_fprintf(stdout, "100%%\n");
    mb_fprintf(
      stdout, "    elapsed time: %lf [min] = %lf [hour]\n",
      (double) (c2-c1)/ (double) CLOCKS_PER_SEC / 60.0,
      (double) (c2-c1)/ (double) CLOCKS_PER_SEC / 3600.0
      );
  }
  else{ /* Forecast (ind = number of iteration) */
    c2 = clock();
    diff_t = (double) (c2-c1)/ (double) CLOCKS_PER_SEC;
    if( diff_t > 0.1 ){
      mb_fprintf(
        stdout, " %.1lf [min] (1CPU)\n", diff_t * (double) ind / 60.0
        );
      mb_fflush(stdout);
    }
    else{
      mb_fprintf(stdout, " Maybe not so long.\n");
      /* mb_fprintf(stdout, "0%%"); mb_fflush(stdout); */
    }
  }
}


#if 0
void exec_progress(int *progress_ratio, long ith, long num_iter)
{
  int previous_ratio;

  if(dbg.forecast_flag == 0){return;}

  previous_ratio = *progress_ratio;
  *progress_ratio = (int) (100.0 * (double) ith / (double) num_iter );
  /*
	if ( (*progress_ratio != previous_ratio) && (*progress_ratio % 2 == 0) ){
		printf( "*" );	fflush(stdout);
	}
   */
  if ( (*progress_ratio != previous_ratio) && (*progress_ratio % 5 == 0) ){
    mb_fprintf(stdout, "%d%%", *progress_ratio );	fflush(stdout);
  }
}
#endif


void dbg_mon(char *str)
     /* Process monitor */
{
  if(dbg.process_mon_flag == 1){
    mb_fprintf(stdout, "%s",str);	fflush(stdout);
  }
}


void fprint_gen_atom_pos(
  FILE *fp, int num_sym_op, SYM_OP *sym_op, int n_atom, ATOM *atm
  )
{
  /*
   * print atom positions generated by symmetry operations
   */
  int i, j, *num_nthatm;
  XYZ **pos;

  num_nthatm = new_vector(int, n_atom);
  chk_vector(num_nthatm);
  pos = new_matrix(XYZ, n_atom, num_sym_op);		chk_matrix(pos);
  newpos_sym_op(n_atom, atm, num_nthatm, pos, num_sym_op, sym_op);
  mb_fprintf(fp, "* All atom positions\n*\n");

  for(i=0; i<n_atom; i++){
    mb_fprintf(fp, "* Atom[%d]: %s\n", i+1, atm[i].element);
    mb_fprintf(
      fp, "* number of atom positions = %d  (multiplicity = %d/%d = %lf)\n",
      num_nthatm[i],  num_nthatm[i], num_sym_op,
      (double) num_nthatm[i] / (double) num_sym_op
      );
    mb_fprintf(fp, "no:\tx\ty\tz\n");
    for(j=0; j<num_nthatm[i]; j++){
      mb_fprintf(
        fp, "%d\t%lf\t%lf\t%lf\n",
        j, pos[i][j].x, pos[i][j].y, pos[i][j].z);
    }
    mb_fprintf(fp, " Occupancy = %lf\n", atm[i].occ);
  }
  free_vector(num_nthatm);
  free_matrix(pos);
  return;
}


void dbg_atm_pos_out( CRYSTAL_STRCT_DATA *csd, char *infile )
{
  char outfile[128];
  FILE *fp;
  
  if(dbg.atm_pos_out_flag != 1) { return; }

  sprintf(outfile, "%s_aps.txt", infile);
  fp = mb_fopen_chk_wmode(outfile);
  mb_fprintf(stdout, "$$ Generated atom positions ==> %s\n", outfile);
  fprint_gen_atom_pos(
    fp, csd->num_sym_op, csd->sym_op, csd->n_atom, csd->atm
    );
  mb_fclose(fp);
}


void dbg_print_recom_bij(int n_atom, ATOM *atm, CELL_PAR bv)
{
  int i;
  double  b;
  /*	double	aas, bbs, ccs, abs, bcs, acs; */
  
  for(i=0; i<n_atom; i++){
    if(atm[i].bf == 0 && dbg.print_bij_flag == 1){
      mb_fprintf(
        stdout, "$$ ATOM %d: Anisotropic B (beta_{ij}): starting values\n", i
        );
      b = atm[i].b;
      mb_fprintf(stdout, "$$ beta11 = %lf\n", b * bv.aas / 4.0);
      mb_fprintf(stdout, "$$ beta22 = %lf\n", b * bv.bbs / 4.0);
      mb_fprintf(stdout, "$$ beta33 = %lf\n", b * bv.ccs / 4.0);
      mb_fprintf(stdout, "$$ beta12 = %lf\n", b * bv.abs / 4.0);
      mb_fprintf(stdout, "$$ beta23 = %lf\n", b * bv.bcs / 4.0);
      mb_fprintf(stdout, "$$ beta13 = %lf\n", b * bv.acs / 4.0);
    }
    else{
      ;
    }
  }
  
}


#if 0
void dbg_wg_xig_list(
  int num_exact, int num_bethe, HKL g_exact[], HKL g_bethe[],
  CRYSTAL_STRCT_DATA *csd, double accel_kvol
  )
{
  char outfile[128];
  FILE *fo;
  int i, nr, nth, *index, g_table_size, ct_fg_table;
  double *wg, *xig, wave_length, relativistic_correction;
  HKL	*rflx, *g_table;
  complex	*fg_table;
  
  nr = num_exact + num_bethe;
  rflx = new_vector(HKL, nr);		chk_vector(rflx);
  index = new_vector(int, nr);	chk_vector(index);
  wg = new_vector(double, nr);	chk_vector(wg);
  xig = new_vector(double, nr);	chk_vector(xig);
  
  wave_length = wavelength(accel_kvol);
  relativistic_correction = relativ_corr(accel_kvol);
  
  /* ----- Initialization ----- */
  g_table_size = SIZE_GTABLE;
  fg_table = new_vector(complex, g_table_size);	chk_vector(fg_table);
  g_table  = new_vector(HKL, g_table_size);		chk_vector(g_table);
  ct_fg_table = 0;
  for(i=0; i<csd->num_low_order_g; i++){
    stock_fg_tbl_sym(
      csd->g_low_order[i], lofg_to_fg_0(lof_type, csd->fg_low_order[i]), &ct_fg_table,
      g_table_size, g_table, fg_table, csd->num_sym_op, csd->sym_op
      );
  }
  
  for(i=0; i<num_exact; i++){
    rflx[i] = g_exact[i];
    wg[i] = wge[i]; xig[i] = xige[i];
  }
  for(i=0; i<num_bethe; i++){
    rflx[i + num_exact] = g_bethe[i];
    wg[i + num_exact] = wgb[i];
    xig[i + num_exact] = xigb[i];
  }
  
  for(i=0; i<nr; i++){
    fg = structure_factor_ext_tbl(
      /* Structure factor */
      g, csd->num_sym_op, csd->sym_op, csd->n_atom, csd->atm,
      csd->xdyl, csd->ftbl, bv, csd->lattice_type,
      &ct_fg_table, g_table_size, g_table, fg_table
      );
    xi_g = fg_to_xig2(
      fg, relativistic_correction, wave_length, bv
      ); /* Extinction distance */
  }

  d_heapsort_ascend(nr, wg, index);

  strcpy(outfile, dbg.infile);	strcat(outfile, ".wg");
  if((fo = fopen(outfile, "w")) == NULL){
    printf("$$ Cannot open %s. Abort.\n", outfile);
    exit(EXIT_FAILURE);
  }
  printf("($$ wg & xig ==> %s) ..", outfile);

  fprintf(fo,"%% wg and xig list\n");
  fprintf(fo,"    i:  h   k   l\t\twg\t\txig\n");
  for(i=0; i<nr; i++){
    nth = index[i];
    fprintf(
      fo,"%5d: %3d %3d %3d\t%8.1lf\t%10.1lf\n",
      i, rflx[nth].h, rflx[nth].k, rflx[nth].l, wg[i], xig[nth]
      );
  }

  fclose(fo);
  free_vector(rflx);	free_vector(index);
  free_vector(wg);	free_vector(xig);
}
#endif


void reflex_dump(
  int ith_expdata, int num_g_exact, int num_g_bethe, HKL reflex[], int flag_bs,
  BEAM_SEARCH_PAR beam_search, UVW incidence, R_HKL zolzbase1, CELL_PAR bv
  )
{
  /*
   * Dump the reflection list without calculating Fg.
   */
  int i,nlz;
  double x, y, g_abs;
  char rffile[128];
  FILE *fp;
  HKL g;
  
  /* Open file for output */
  sprintf(rffile, "%s.rf%d", dbg.infile, ith_expdata);
  if ( ( fp = fopen(rffile,"w") ) == NULL ) {
    fprintf(stderr,"Cannot open %s\n", rffile);
    return;
  }
  mb_fprintf(stdout, "reflex output ==> %s\n", rffile);

  if(flag_bs == 1){
    mb_fprintf(
      fp,
      "%% beam_search par.: wgbmin=%lg, wgmax=%lg, xigmax=%lg, lz:%d-%d, gmax=%lg\n",
      beam_search.wgbmin, beam_search.wgmax, beam_search.xigmax,
      beam_search.lzmin, beam_search.lzmax, beam_search.gmax
      );
  }
  else{
    mb_fprintf( fp, "%% beam_search --> manual\n" );
  }
  
  mb_fprintf(fp, "%% [%d %d %d]\n",incidence.u, incidence.v, incidence.w);
  mb_fprintf(fp, "%% Exact= %d  Bethe= %d\n",num_g_exact, num_g_bethe);
  mb_fprintf(fp, "  i\th\tk\tl\tnlz\tx\ty\tBethe\tg\n");
  
  for(i=0;i<num_g_exact;i++){
    g = reflex[i];
    nlz	= g.h * incidence.u + g.k * incidence.v + g.l * incidence.w;
    proj_to_zolz(g, incidence, zolzbase1, bv, &x, &y);
    g_abs = sqrt(sq_abs(hkl2rhkl(g),bv));
    mb_fprintf(
      fp, "%3d\t%3d\t%3d\t%3d\t%d\t%lf\t%lf\t%d\t%lf\n",
      i, g.h, g.k, g.l, nlz, x, y, 0, g_abs
      );
  }
  for(i=num_g_exact;i<num_g_exact + num_g_bethe;i++){
    g = reflex[i];
    nlz	= g.h * incidence.u + g.k * incidence.v + g.l * incidence.w;
    proj_to_zolz(g, incidence, zolzbase1, bv, &x, &y);
    g_abs = sqrt(sq_abs(hkl2rhkl(g),bv));
    mb_fprintf(
      fp, "%3d\t%3d\t%3d\t%3d\t%d\t%lf\t%lf\t%d\t%lf\n",
      i, g.h, g.k, g.l, nlz, x, y, 1, g_abs
      );
  }
  fclose(fp);
  
  if (dbg.reflex_only_flag == 1){
    mb_fprintf(stdout, "$$ reflex_only_flag=1. exit.\n");
    exit(EXIT_SUCCESS);
  }
  
}


void reflex_dump_csd(
  int ith_expdata, int num_g_exact, int num_g_bethe, HKL reflex[],
  CRYSTAL_STRCT_DATA *csd, BEAM_SEARCH_PAR beam_search,
  UVW incidence, R_HKL zolzbase1, double kvolts, CELL_PAR bv
  )
{
  /*
   * Dump reflection list with Fg calc.
   */
  int i, nlz, g_table_size, ct_fg_table;
  FILE *fp;
  double x,y, g_abs;
  HKL g, *g_table;
  complex	fg, *fg_table;
  char rffile[128];

  /* Open file for output */
  sprintf(rffile, "%s_fg%d.txt", dbg.infile, ith_expdata);
  fp = mb_fopen_chk_wmode(rffile);
  mb_fprintf(stdout, "reflex output ==> %s\n", rffile);

  /* ----- Initialization ----- */
  g_table_size = SIZE_GTABLE;
  fg_table = new_vector(complex, g_table_size);	chk_vector(fg_table);
  g_table  = new_vector(HKL, g_table_size);		chk_vector(g_table);
  ct_fg_table = 0;
  for(i=0; i<csd->num_lofg; i++){
    stock_fg_tbl_sym(
      csd->g_lofg[i],
      lofg_to_fg(
        csd->g_lofg[i], csd->lof_type, &(csd->fg_lofg[i]),
        csd->num_sym_op, csd->sym_op,
        csd->n_atom, csd->atm, csd->ftbl, csd->xdyl, kvolts, bv
        ),
      &ct_fg_table,
      g_table_size, g_table, fg_table, csd->num_sym_op, csd->sym_op
      );
  }

  if( beam_search.flag_bs == 0 || beam_search.flag_bs == 10 ){	/* wg */
    mb_fprintf( fp, "%% beam_search --> manual\n" );
  }
  else if( beam_search.flag_bs == 1 || beam_search.flag_bs == 11 ){	/* wg */
    mb_fprintf(fp, "%% beam_search par.: \n");
    mb_fprintf(
      fp, " wgbmin=%lg, wgmax=%lg, xigmax=%lg, lz:%d-%d, gmax=%lg\n",
      beam_search.wgbmin, beam_search.wgmax, beam_search.xigmax,
      beam_search.lzmin, beam_search.lzmax, beam_search.gmax
      );
  }
  else if( beam_search.flag_bs == 2 || beam_search.flag_bs == 12 ){	/* sg */
    mb_fprintf(fp, "%% beam_search par.: \n");
    mb_fprintf(
      fp, " sgbmin=%lg, sgmax=%lg, xigmax=%lg, lz:%d-%d, gmax=%lg\n",
      beam_search.sgbmin, beam_search.sgmax, beam_search.xigmax,
      beam_search.lzmin, beam_search.lzmax, beam_search.gmax
      );
  }
  else if( beam_search.flag_bs == 3 || beam_search.flag_bs == 13 ){	/* sg */
    mb_fprintf(fp, "%% beam_search ---> multi sg\n");
  }
  else{
    mb_fprintf(stdout, "$$ beam_search: manual\n");
    mb_fprintf(stdout, "$$ beam_search.flag_bs = %d\n", beam_search.flag_bs );
  }
  
  mb_fprintf(fp, "%% [%d %d %d]\n",incidence.u, incidence.v, incidence.w);
  mb_fprintf(fp, "%% Exact= %d  Bethe= %d\n",num_g_exact, num_g_bethe);
  mb_fprintf(fp, "i\th\tk\tl\tnlz\tx\ty\tBethe\tg\tRe(Fg)\tIm(Fg)\tAbs(Fg)\n");

  for(i=0;i<num_g_exact;i++){
    g = reflex[i];
    nlz = g.h * incidence.u + g.k * incidence.v + g.l * incidence.w;
    proj_to_zolz(g, incidence, zolzbase1, bv, &x, &y);
    fg = structure_factor_ext_tbl(
      /* Structure factor */
      g, csd->num_sym_op, csd->sym_op, csd->n_atom, csd->atm,
      csd->xdyl, csd->ftbl, kvolts, bv, csd->lattice_type,
      &ct_fg_table, g_table_size, g_table, fg_table
      );
    g_abs = sqrt( sq_abs(hkl2rhkl(g),bv) );
    mb_fprintf(
      fp, "%3d\t%3d\t%3d\t%3d\t%d\t%lf\t%lf\t%d\t%lf\t%lf\t%lf\t%lf\n",
      i, g.h, g.k, g.l, nlz,
      x, y, 0, g_abs, fg.re, fg.im, sqrt(sq(fg.re) + sq(fg.im))
      );
  }
  for(i=num_g_exact;i<num_g_exact + num_g_bethe;i++){
    g = reflex[i];
    nlz = g.h * incidence.u + g.k * incidence.v + g.l * incidence.w;
    proj_to_zolz(g, incidence, zolzbase1, bv, &x, &y);
    fg = structure_factor_ext_tbl(
      /* Structure factor */
      g, csd->num_sym_op, csd->sym_op, csd->n_atom, csd->atm,
      csd->xdyl, csd->ftbl, kvolts, bv, csd->lattice_type,
      &ct_fg_table, g_table_size, g_table, fg_table
      );
    g_abs = sqrt(sq_abs(hkl2rhkl(g),bv));
    mb_fprintf(
      fp, "%3d\t%3d\t%3d\t%3d\t%d\t%lf\t%lf\t%d\t%lf\t%lf\t%lf\t%lf\n",
      i, g.h, g.k, g.l, nlz,
      x, y, 1, g_abs, fg.re, fg.im, sqrt(sq(fg.re) + sq(fg.im))
      );
  }
  
  mb_fclose(fp);
  free_vector(fg_table);
  free_vector(g_table);

  if (dbg.reflex_only_flag == 1){
    mb_fprintf(stdout, "$$ reflex_only_flag=1. exit.\n");
    exit(EXIT_SUCCESS);
  }
}


void debug_vg_dump( complex **vg, int num_beam )
{
  char outfile[BUFLEN];
  FILE *fp;
  
  if( dbg.vg_dump_flag == 1 ){	/* debug */
    sprintf(outfile, "%s_vg.txt", dbg.infile);
    /* strcpy(outfile, dbg.infile); strcat(outfile,"vg"); */
    if( (fp = fopen(outfile,"w")) == NULL ){
      printf("$$ Cannot open %s. Abort.\n", outfile);
      exit(EXIT_FAILURE);
    }
    printf("vg_data output ==> %s\n", outfile);
    fprintf(fp,"Vg: [Volt]\n");
    fprint_complexmat(fp, num_beam, vg);
    fclose(fp);
  }
}


void dbg_print_cmat(int flag, char *comment, int n, complex **a)
{
  int i,j;
  if (flag == 0) return;
  printf("$$%s\n", comment);
  for(j=0; j<n; j++){
    for(i=0; i<n; i++){
      printf("%d\t%d\t%lf\t%lf\n", i, j, a[i][j].re, a[i][j].im);
    }
  }
}


void dbg_print_cvec(int flag, char *comment, int n, complex *v)
{
  int i;
  if (flag == 0) return;
  printf("$$%s\n", comment);
  for(i=0; i<n; i++){ printf("%d\t%lf\t%lf\n",i,v[i].re, v[i].im); }
}


void dbg_print_dmat(int flag, char *comment, int n, double **a)
{
  int i,j;
  if (flag == 0) return;
  printf("$$%s\n", comment);
  for(j=0; j<n; j++){
    for(i=0; i<n; i++){
      printf("%d\t%d\t%lf\n", i, j, a[i][j] );
    }
  }
}


void dbg_print_dvec(int flag, char *comment, int n, double *v)
{
  int i;
  if (flag == 0) return;
  printf("$$%s\n", comment);
  for(i=0; i<n; i++){ printf("%d\t%lf\n",i,v[i]); }
}


void dbg_dump_init()
{
  char outfile[128];
  
  if(dbg.vg_sel_chk_flag == 0){ return; }
  
  sprintf(outfile, "%s.dbg", dbg.infile);
  if ( ( dbg.dump = fopen(outfile,"w") ) == NULL ) {
    printf("Cannot open %s\n", outfile);
    return;
  }
  printf("$$ Debug output ==> %s\n",outfile);
}


void dbg_sel_vg_init( int ne, int nb )
{
  int i;
  
  if(dbg.vg_sel_chk_flag == 0){ return; }
  
  printf("$$ chk_sel_vg mode. No intensity calculation.\n");
  fprintf(dbg.dump, "H\tG\th\tk\t\tl\tnesel\tnbsel");
  for(i=0; i<ne+nb; i++){
    fprintf(dbg.dump, "\t%d", i);
  }
  fprintf(dbg.dump, "\n");
}


void dbg_sel_vg_pos(int h, int g)
{
  if(dbg.vg_sel_chk_flag == 1){
    fprintf(dbg.dump, "%d\t%d\t", h, g);
  }
}


void dbg_sel_vg(
  R_HKL k_xy, int ne, int nb, int n_sel_exact, int n_sel_bethe, int *selinfo_nth_reflex
  )
     /* 0:not selected, 1:exact, 2:Bethe */
{
  int i, *table;
  
  table = new_vector(int, ne + nb);	chk_vector(table);
  
  for(i=0; i<ne+nb; i++){ table[i] = 0; }
  for(i=0; i<n_sel_exact; i++){
    table[ selinfo_nth_reflex[i] ] = 1;	/* Exact */
  }
  for(i=n_sel_exact; i<n_sel_exact + n_sel_bethe; i++){
    table[ selinfo_nth_reflex[i] ] = 2;	/* Bethe */
  }
  
  fprintf(dbg.dump, "%lf\t%lf\t%lf", k_xy.h, k_xy.k, k_xy.l);
  fprintf(dbg.dump, "\t%d\t%d", n_sel_exact, n_sel_bethe);
  for(i=0; i<ne + nb; i++){
    fprintf(dbg.dump,"\t%d", table[i]);
  }
  fprintf(dbg.dump,"\n");
  
  free_vector(table);
}


void dbg_dump_term(void)
{
  if(dbg.vg_sel_chk_flag == 1){
    fclose(dbg.dump);
  }
}


#if 0
void dbg_asf_test_level3(void)
{
  FILE *fp;
  char buf[128], element[5];
  ATOMF_BK asftbl;
  complex atomf;
  int i, npoints;
  double s, s_begin, s_end;
  /*	double	fabs0, sq_s; */
  /*	double f_ela, f_inela; */
  double	b_factor, kvolts;
  
  /* Flag chk */
  if(dbg.asf_test_flag != 3) { return; }
  
  printf("***** elastic and inelastic (absorptive) atomic scattering factor *****\n");
  printf(" Bird & King's ATOM\n");
  
  if ((fp = fopen("asf.out","w")) == NULL){
    printf("Cannot open asf.out\n"); exit(EXIT_FAILURE);
  }
  printf("s_begin >>"); fgets(buf,sizeof(buf),stdin);	s_begin = atof(buf);
  printf("s_end >>");   fgets(buf,sizeof(buf),stdin);	s_end = atof(buf);
  printf("kvolts >>");  fgets(buf,sizeof(buf),stdin);	kvolts = atof(buf);
  printf("b_factor >>"); fgets(buf,sizeof(buf),stdin);	
  b_factor = atof(buf);
  printf("number of calculation points >>"); fgets(buf,sizeof(buf),stdin); 
  npoints = atoi(buf);
  printf("element (CAPS) >>");	gets(element,sizeof(element),stdin);
  
  while(strstr(element,"END") == NULL){
    fprintf(fp, "%% element = %s\n", element);
    fprintf(fp, "s\tf_ela(M)\tf_inela(I)\tf_ela(BK)\tf_inela(BK)\n", element);
    asftbl =  read_atom_dat_table(element);
    printf("$$ status = %d\n", atom_bk(asftbl, s = 0.0, b_factor, &atomf, kvolts) );
    
    for(i=0; i<npoints; i++){
      s = s_begin + (s_end - s_begin) * (double) i / (double) (npoints - 1);
      atom_bk(asftbl, s, b_factor, &atomf, kvolts);
      fprintf(
        fp, "%lf\t%lf\t%lf\t%lf\t%lf\n", s, atomf.re, atomf.im
        );
    }
    printf("element (CAPS) (END to quit) >>");	
    fgets(element,sizeof(element),stdin);
  }
  fclose(fp);
  printf("results --> asf.out\n");
  
  printf("\n   Thank you. See you again!!!\n");
  printf("   (Press return key.)\n");
  fgets(buf,sizeof(buf),stdin);
  printf("   (Press command-Q to quit for Macintosh.)\n");
  exit(EXIT_SUCCESS);
}


void dbg_asf_test_level1(void)
{
  FILE *fp;
  char buf[128], element[5];
  XDT xdyl;
  int i, npoints;
  double s, sq_s, s_begin, s_end;
  
  /* Flag chk */
  if(dbg.asf_test_flag != 1) { return; }
  
  printf("***** atomic scattering factor *****\n");
  if ((fp = fopen("asf.out","w")) == NULL){
    printf("Cannot open asf.out\n"); exit(EXIT_FAILURE);
  }
  printf("s_begin >>"); fgets(buf,sizeof(buf),stdin);	
  s_begin = atof(buf);
  printf("s_end >>"); fgets(buf,sizeof(buf),stdin);	
  s_end = atof(buf);
  printf("number of calculation points >>"); fgets(buf,sizeof(buf),stdin); 
  npoints = atoi(buf);
  printf("element (CAPS) (END to quit) >>");	
  fgets(element,sizeof(element),stdin);
  while(strstr(element,"END") == NULL){
    fprintf(fp, "element = %s\n", element);
    xdyl = read_xdt_table(element);
    for(i=0; i<npoints; i++){
      s = s_begin + (s_end - s_begin) * (double) i / (double) (npoints - 1);
      sq_s = sq(s);
      fprintf(fp, "%lf %lf\n", s, atomsf_xdt(sq_s, xdyl) );
    }
    printf("element (CAPS) >>");	fgets(element,sizeof(element),stdin);
  }
  fclose(fp);
  
  
  printf("\n   Thank you. See you again!!!\n");
  printf("   (Press return key.)\n");
  fgets(buf,sizeof(buf),stdin);
  printf("   (Press command-Q to quit for Macintosh.)\n");
  exit(EXIT_SUCCESS);
}


void dbg_asf_test_level2(void)
{
  FILE *fp;
  char buf[128], element[5];
  XDT xdyl;
  ATOMF_BK asftbl;
  complex atomf;
  int i, npoints;
  double s, sq_s, s_begin, s_end, fabs0;
  double f_ela, f_inela, b_factor, kvolts;
  
  /* Flag chk */
  if(dbg.asf_test_flag != 2) { return; }
  
  printf("***** elastic and inelastic (absorptive) atomic scattering factor *****\n");
  printf(" Mott formula + Ichimiya vs. Bird & King's ATOM\n");
  printf(" with Debye-Waller factor exp(-B s2)\n");
  if ((fp = fopen("asf.out","w")) == NULL){
    printf("Cannot open asf.out\n"); exit(EXIT_FAILURE);
  }
  printf("s_begin >>"); fgets(buf,sizeof(buf),stdin);	
  s_begin = atof(buf);
  printf("s_end >>");  fgets(buf,sizeof(buf),stdin);	
  s_end = atof(buf);
  printf("kvolts >>"); fgets(buf,sizeof(buf),stdin);	
  kvolts = atof(buf);
  printf("b_factor >>");	fgets(buf,sizeof(buf),stdin);	
  b_factor = atof(buf);
  printf("number of calculation points >>"); fgets(buf,sizeof(buf),stdin); 
  npoints = atoi(buf);
  printf("element (CAPS) >>");	gets(element,sizeof(element),stdin);
  while(strstr(element,"END") == NULL){
    fprintf(fp, "%% element = %s\n", element);
    fprintf(fp, "s\tf_ela(M)\tf_inela(I)\tf_ela(BK)\tf_inela(BK)\n", element);
    xdyl = read_xdt_table(element);
    asftbl =  read_atom_dat_table(element);
    printf("$$ status = %d\n", atom_bk(asftbl, s = 0.0, b_factor, &atomf, kvolts) );
    fabs0 = atomf.im;
    for(i=0; i<npoints; i++){
      s = s_begin + (s_end - s_begin) * (double) i / (double) (npoints - 1);
      atom_bk(asftbl, s, b_factor, &atomf, kvolts);
      sq_s = sq(s);
      f_ela = atomsf_xdt(sq_s, xdyl);
      f_inela = atom_f_absorption(sq_s, fabs0, b_factor);
      fprintf(
        fp, "%lf\t%lf\t%lf\t%lf\t%lf\n",
        s, f_ela, f_inela, atomf.re, atomf.im
        );
    }
    printf("element (CAPS) (END to quit) >>");	
  fgets(element,sizeof(element),stdin);
  }
  fclose(fp);
  printf("results --> asf.out\n");
  
  printf("\n   Thank you. See you again!!!\n");
  printf("   (Press return key.)\n");
  fgets(buf,sizeof(buf),stdin);
  printf("   (Press command-Q to quit for Macintosh.)\n");
  exit(EXIT_SUCCESS);
}
#endif



void asf_test(void)
     /* 10 Dec. 2000 */
{
  FILE *fp;
  char buf[128], element[5];
  XDT xdyl;
  ATOMF_BK asftbl;
  complex atomf;
  int i, npoints;
  double s, sq_s, s_begin, s_end, fabs0;
  double f_ela, f_inela, b_factor, f_xray_ela, kvolts;
  
  printf("***** elastic and inelastic (absorptive) atomic scattering factor *****\n");
  printf(" Mott formula + Ichimiya vs. Bird & King's ATOM\n");
  printf(" with Debye-Waller factor exp(-B s2)\n");
  if ((fp = fopen("asf.out","w")) == NULL){
    printf("Cannot open asf.out\n"); exit(EXIT_FAILURE);
  }
  printf("s_begin >>"); fgets(buf,sizeof(buf),stdin);	
  s_begin = atof(buf);
  printf("s_end >>");  fgets(buf,sizeof(buf),stdin);	
  s_end = atof(buf);
  printf("kvolts >>"); fgets(buf,sizeof(buf),stdin);	
  kvolts = atof(buf);
  printf("b_factor >>");	fgets(buf,sizeof(buf),stdin);	
  b_factor = atof(buf);
  printf("number of calculation points >>"); fgets(buf,sizeof(buf),stdin); 
  npoints = atoi(buf);
  printf("element (CAPS) >>");	fgets(element,sizeof(element),stdin);
  
  /*	while( strstr(element,"END") != NULL && strlen(element) != 0 ){ */
  while( strlen(element) != 0 ){
    fprintf(fp, "%% element = %s\n", element);
    fprintf(fp, "s\tf_ela(M)\tf_inela(I)\tf_ela(BK)\tf_inela(BK)\tX-ray's_f\n", element);
    
    xdyl	= read_xdt_table(element);			/* Coef. of X-ray's f (Doyle & Turner) */
    asftbl	= read_atom_dat_table(element);		/* Bird & King's table */
    
    /*	printf("$$ status = %d\n", atom_bk(asftbl, s = 0.0, b_factor, &atomf, kvolts) ); */
    printf("$$ status = %d\n", atom_bk(asftbl, 0.0, b_factor, &atomf, kvolts) );
    fabs0 = atomf.im;		/* For atom_f_absorption */
    
    for(i=0; i<npoints; i++){
      s = s_begin + (s_end - s_begin) * (double) i / (double) (npoints - 1);
      sq_s = sq(s);
      
      atom_bk(asftbl, s, b_factor, &atomf, kvolts);		/* Elastic f and inelastic f (electron) by Bird & King's table */
      f_ela = atomsf_xdt(sq_s, xdyl);						/* Elastic f (electron) by Mott formula */
      f_inela = atom_f_absorption(sq_s, fabs0, b_factor);	/* Inelastic f (electron) by Ichimiya's eq. */
      f_xray_ela = xray_asf( sq_s, xdyl );				/* X-ray's elastic f */
      
      fprintf(
        fp, "%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n",
        s, f_ela, f_inela, atomf.re, atomf.im, f_xray_ela
        );
    }
    printf("element (CAPS) (return to quit) >>");	
    fgets(element,sizeof(element),stdin);
  }
  
  fclose(fp);
  printf("results --> asf.out\n");
  
  printf("\n   Thank you. See you again!!!\n");
  exit(EXIT_SUCCESS);
}


void message_exit_faliure(void)
{
  char buf[80];
  
  if(epsilon.message_exit_flag == 1){
    fprintf(stderr, "$$ Mbfit found an error: press any key to quit.\n");
    fgets(buf,sizeof(buf),stdin);
    exit(EXIT_FAILURE);
  }
  else{
    exit(EXIT_FAILURE);
  }
}
