/*
 * mb_input.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "mbconfig.h"
#include "fitting.h"
#include "mb_input.h"
#include "utils.h"
#include "mb_mpi.h"

#define BUFLEN  256
#define GETLEN  256

extern DEBUG_INFO dbg;
extern EPS_CTRL   epsilon;


void chk_data(const char *str, FILE *fp)
{
 /*  Data check by an identifier (str)  */
  char buf[BUFLEN];

  comment_skip_fgets(sizeof buf, buf, fp);
  if( strcmp(str, rm_cr(buf))!=0 ){
    mb_fprintf(
      stdout, "$$ Data format error has been found at %s.\n$$ Abort.\n", str
      );
    mb_fprintf(stdout, "$$ buf=%s", buf);
    exit(EXIT_FAILURE);
  }

  if(dbg.chk_data_dump_flag == 1){
    mb_fprintf(stdout, "$$ chk_data: %s ok.\n", str);
  }
}


void chk_data_info(const char *str, char *info, FILE *fp)
{
  /*
   * Data check by an identifier (str)
   */
  char buf[BUFLEN];

  comment_skip_fgets(sizeof buf, buf, fp);
  if( strcmp(str,rm_cr(buf))!=0 ){
    mb_fprintf(
      stdout, "$$ Data format error has been found at %s.\n$$ Abort.\n", str
      );
    mb_fprintf(stdout, "$$ buf=%s\n", buf);
    mb_fprintf(stdout, "$$%s\n", info);
    exit(EXIT_FAILURE);
  }
}


void fget_element( char *element, FILE *fp )
{
  char buf[BUFLEN];
  char *delims=" ,\t", *token;

  comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf, delims);
  strcpy( element, rm_cr(token) );
}


char *fget_1s( char *str, FILE *fp )
{
  char buf[BUFLEN], *s;
  char *delims=" ,\t", *token;

  s = comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf, delims);
  strcpy( str, rm_cr(token) );
  return s;
}


char *fget_2s( char *str1, char *str2, FILE *fp )
{
  char buf[BUFLEN], *s;
  char *delims=" ,\t", *token;

  s = comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf, delims);
  strcpy( str1, rm_cr(token) );
  token = strtok(NULL, delims);
  strcpy( str2, rm_cr(token) );
  return s;
}


double fget_d(FILE *fp)
{
  char buf[BUFLEN];
  double x;

  comment_skip_fgets(sizeof buf, buf, fp);
  sscanf(buf, "%lf", &x);
  return x;
}


int fget_i(FILE *fp)
{
  char buf[BUFLEN];
  int i;

  comment_skip_fgets(sizeof buf, buf, fp);
  sscanf(buf, "%d", &i);
  return i;
}


void fget_5d( double *v, double *w, double *x, double *y, double *z, FILE *fp )
{
  char buf[BUFLEN];
  double t[5];
  
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(5, buf, t);
  *v = t[0];  *w = t[1];  *x = t[2];  *y = t[3];  *z = t[4];
}


void fget_4d( double *w, double *x, double *y, double *z, FILE *fp )
{
  char buf[BUFLEN];
  double t[4];
  
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(4,buf,t);
  *w = t[0]; *x = t[1];	*y = t[2];	*z = t[3];
}


void fget_3d( double *x, double *y, double *z, FILE *fp )
{
  char buf[BUFLEN];
  double t[3];
  
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(3,buf,t);
  *x = t[0];	*y = t[1];	*z = t[2];
}


void fget_2d( double *x, double *y, FILE *fp )
{
  char buf[BUFLEN];
  double t[2];
  
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(2,buf,t);
  *x = t[0];	*y = t[1];
}


void fget_3i( int *i, int *j, int *k, FILE *fp)
{
  char buf[BUFLEN];
  double t[3];
  
  t[0] = 0.0; t[1] =0.0; t[2] = 0.0;
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(3,buf,t);
  *i = (int) t[0];	*j = (int) t[1];	*k = (int) t[2];
}


void fget_2i( int *i, int *j, FILE *fp)
{
  char buf[BUFLEN];
  double t[2];
  
  t[0] = 0.0; t[1] =0.0;
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(2, buf, t);
  *i = (int) t[0];  *j = (int) t[1];
}


double fget_d_and_i(int *i, FILE *fp)
{
  char buf[BUFLEN];
  double t[2];
  
  t[0] = 0.0; t[1] = 0.0;
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(2,buf,t);
  *i = (int) t[1];
  return t[0];	/* double */
}


/*
void fget_di(double *x, int *i, FILE *fp)
{
  char buf[BUFLEN];
  double t[2];
  int n;
  
  t[0] = 0.0; t[1] = 0.0;
  comment_skip_fgets(sizeof buf, buf, fp);
  n = str_to_fs(2,buf,t);
  if( n != 2 ){
    mb_fprintf(stdout,"$$ Parameter error (n=%d).  Abort.\n", n);
    exit(EXIT_FAILURE);
  }
  *x = t[0]; *i = (int) t[1];
}

  
void fget_did(double *x1, int *i, double *x2, FILE *fp)
{
  char buf[BUFLEN];
  double t[3];
  int n;
  
  t[0] = 0.0; t[1] = 0.0; t[2] = 0.0;
  comment_skip_fgets(sizeof buf, buf, fp);
  n = str_to_fs(3,buf,t);
  if( n != 3 ){ printf("$$ Parameter error (n=%d).  Abort. ", n); exit(EXIT_FAILURE); }
  *x1 = t[0]; *i = (int) t[1]; *x2 = t[2];
}


void fget_didd(double *x1, int *i, double *x2, double *x3, FILE *fp)
{
  char buf[BUFLEN];
  double t[4];
  int n;
  
  t[0] = 0.0; t[1] = 0.0; t[2] = 0.0; t[3] = 0.0;
  comment_skip_fgets(sizeof buf, buf, fp);
  n = str_to_fs(4,buf,t);
  if( n != 4 ){
    printf("$$ Parameter error (n=%d). Abort.", n); exit(EXIT_FAILURE);
  }
  *x1 = t[0]; *i = (int) t[1]; *x2 = t[2]; *x3 = t[3];
}
*/


void fget_str_and_i(char *str, int *i, FILE *fp)
{
  char buf[BUFLEN];
  char *delims=" ,\t", *token;

  comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf,delims);		strcpy(str, token);
  if (token != NULL){
    token = strtok(NULL,delims);
    *i = atoi(token);
  }
  else{
    *i = 0;
  }
}


void fget_str_and_dii(char *str, double *x, int *i, int *j, FILE *fp)
{
  char buf[BUFLEN];
  char *delims=" ,\t", *token;

  comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf,delims);
  strcpy(str, token);
  if (token != NULL){
    token = strtok(NULL,delims);
    *x = atof(token);
  }
  else{
    *x = 0.0;
  }
  if (token != NULL){
    token = strtok(NULL,delims);
    *i = atoi(token);
  }
  else{
    *i = 0;
  }
    if (token != NULL){
    token = strtok(NULL,delims);
    *j = atoi(token);
  }
  else{
    *j = 0;
  }
}


#if 0
void fget_name_cstr_idx(char *name, char *cstr, int *idx, FILE *fp)
{
  /*
   name: name of the parameter
   cstr: constraint
   idx:  fitting flag (0, 1, 2, -1)
   */
  char buf[BUFLEN], *delims=" ,\t", *token;

  comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf, delims);
  strcpy(cstr, token);  /* numerical value or constraint */

  token = strtok(NULL, delims); /* flag for fitting */
  if( strcmp(token, "ADJ") == 0 || strcmp(token, "2") == 0 ){
    *idx = 2;  /* adjustable */
  }
  else if( strcmp(token, "FIX") == 0 || strcmp(token, "0") == 0 ){
    *idx = 0;  /* fix */
  }
  else if( strcmp(token, "CST") == 0 || strcmp(token, "-1") == 0 ){
    *idx = -1;
  }
  /*
  else if( strcmp(token, "RST") == 0 || strcmp(token, "1") == 0 ){
    *idx = 1; 
  }
   */
  else{
    mb_fprintf(stdout, "$$ (fget_name_cstr_idx) Input data error. Abort.\n");
    mb_fprintf(stdout, "$$ buf=%s", buf);
    exit(EXIT_FAILURE);
  }

  token = strtok(NULL, delims);
  if (token != NULL){
    strcpy(name, rm_cr(token)); /* name of the parameter */
  }
  else{
    mb_fprintf(stdout, "$$ (fget_name_cstr_idx) Input data error. Abort.\n");
    mb_fprintf(stdout, "$$ buf=%s", buf);
    exit(EXIT_FAILURE);
  }
}
#endif


void fget_fitpar(FIT_CTRL *fc, double *val, FILE *fp)
{
  /*
   * fget_name_cstr_idx + par_transval
   * modified on 040620
   */
  char buf[BUFLEN], *delims=" ,\t", *token;
  static int num = 0;

  comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf, delims);
  strcpy(fc->cstr, token);   /* a numerical value or a constraint */

  /* ----- Flag for fitting ----- */
  token = strtok(NULL, delims);
  if(token==NULL){  /* To be recognized as the FIX mode */
    fc->flg = 0;
    sprintf(fc->name, "id%d", num);
    num++;
  }
  else if( strcmp(token, "ADJ") == 0 || strcmp(token, "2") == 0 ){
    /* adjustable */
    fc->flg = 2;  
    /* name of the parameter */
    token = strtok(NULL, delims);
    if (token != NULL){
      strcpy(fc->name, rm_cr(token));
    }
    else{
      mb_fprintf(stdout, "$$ (fget_fitpar) Input data error. Abort.\n");
      mb_fprintf(stdout, "$$ buf=%s", buf);  exit(EXIT_FAILURE);
    }
  }
  else if( strcmp(token, "FIX") == 0 || strcmp(token, "0") == 0 ){  /* fix */
    fc->flg = 0;
    /* name of the parameter */
    token = strtok(NULL, delims);
    if (token != NULL){
      strcpy(fc->name, rm_cr(token));
    }
    else{
      mb_fprintf(stdout, "$$ (fget_fitpar) Input data error. Abort.\n");
      mb_fprintf(stdout, "$$ buf=%s", buf);  exit(EXIT_FAILURE);
    }
  }
  else if( strcmp(token, "CST") == 0 || strcmp(token, "-1") == 0 ){
    /* constraint */
    fc->flg = -1;
    /* name of the parameter */
    token = strtok(NULL, delims);
    if (token != NULL){
      strcpy(fc->name, rm_cr(token));
    }
    else{
      mb_fprintf(stdout, "$$ (fget_fitpar) Input data error. Abort.\n");
      mb_fprintf(stdout, "$$ buf=%s", buf);  exit(EXIT_FAILURE);
    }
  }
  /* for restraint inputs */
  /*
  else if( strcmp(token, "RST") == 0 || strcmp(token, "1") == 0 ){
    *idx = 1; 
  }
   */
  else{    /* To be recognized as the FIX mode */
    fc->flg = 0;
    sprintf(fc->name, "id%d", num);
    num++;
  }

  fc->esd = 0.0;   /* initializing esd [031002] */

  /* par_transval */
  if (fc->flg == 2 || fc->flg == 1 || fc->flg == 0) {/* no constraint */
    *val = atof(fc->cstr);
  }
  else if (fc->flg == -1){ /* constraint */
    *val = 0.0;
  }
  else{
    mb_fprintf(stdout, "$$ (fget_fitpar) Invalid idx. Abort.\n");
    exit(EXIT_FAILURE);
  }
}


HKL fget_hkl( FILE *fp )
{
  char buf[BUFLEN];
  double t[3];
  HKL g;

  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(3,buf,t);
  g.h = (int) t[0];  g.k = (int) t[1];  g.l = (int) t[2];
  return g;
}


R_HKL fget_r_hkl( FILE *fp )
{
  char buf[BUFLEN];
  double t[3];
  R_HKL g;

  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(3,buf,t);
  g.h = t[0];  g.k = t[1];  g.l = t[2];

  return g;
}


UVW fget_uvw( FILE *fp )
{
  char buf[BUFLEN];
  double t[3];
  UVW vec;

  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(3,buf,t);
  vec.u = (int) t[0];  vec.v = (int) t[1];  vec.w = (int) t[2];
  return vec;
}


void fget_hkl_weight( HKL *g, double *weight, FILE *fp )
{
  char buf[BUFLEN];
  char *delims=" ,\t", *token;

  comment_skip_fgets(sizeof buf, buf, fp);
  token = strtok(buf, delims);
  if (token != NULL){ g->h = atoi(token); }
  else{
    printf("$$(fget_hkl_weight) Format error. Abort. buf=%s.", buf);
    exit(EXIT_FAILURE);
  }
  token = strtok(NULL, delims);
  if (token != NULL){ g->k = atoi(token); }
  else{
    printf("$$(fget_hkl_weight) Format error. Abort. buf=%s.", buf);
    exit(EXIT_FAILURE);
  }
  token = strtok(NULL, delims);
  if (token != NULL){ g->l = atoi(token); }
  else{
    printf("$$(fget_hkl_weight) Format error. Abort. buf=%s.", buf);
    exit(EXIT_FAILURE);
  }
  token = strtok(NULL, delims);
  if (token != NULL && strncmp(token, "%", 1)){ *weight = atof(token); }
  else{ *weight = 1.0; }
}



/* ======================================================================== */
/* ======================================================================== */
void constraint_csd(CRYSTAL_STRCT_DATA *csd);
/* void fget_latpar_di(FILE *fi, LAT_PAR *lp, LATPAR_IDX *lp_i); */
/* void fget_atmpar_cst(
   FILE *fi, int n_atom, ATOM *atm, ATOM_IDX *atm_i,
   XDT *xdyl, ATOMF_BK *ftbl
); */


#if 0
void par_transval(int idx, char *str, double *arg)
{
  if (idx == 2 || idx == 1 || idx == 0) { /* not constraint str. */
    *arg = atof(str);
  }
  else if (idx == -1){ /* constraint */
    *arg = 0.0;
  }
  else{
    mb_fprintf(stdout, "$$ (par_transval) Invalid idx. Abort.\n");
    exit(EXIT_FAILURE);
  }
}
#endif


void fget_latpar_cstr(FILE *fi, LAT_PAR *lp, LATPAR_FIT *lp_f)
{
  fget_fitpar(&(lp_f->a), &(lp->a), fi);
  fget_fitpar(&(lp_f->b), &(lp->b), fi);
  fget_fitpar(&(lp_f->c), &(lp->c), fi);
  fget_fitpar(&(lp_f->alpha), &(lp->alpha), fi);
  fget_fitpar(&(lp_f->beta), &(lp->beta), fi);
  fget_fitpar(&(lp_f->gamma), &(lp->gamma), fi);
}


void fget_atmpar_cstr(
  FILE *fi, int n_atom, ATOM *atm, ATOM_FIT *atm_f, XDT *xdyl, ATOMF_BK *ftbl
  )
{
  int i;
  char buf[BUFLEN];

  for(i=0; i<n_atom; i++){
    chk_data("SITE{", fi);
    fget_1s(atm[i].element, fi);
    xdyl[i] = read_xdt_table( atm[i].element ); /* Coef. of DT's X-ray f */
    /*  mb_fprintf(stdout, "$$xdyl ok.\n");  */
    ftbl[i] = read_atom_dat_table( atm[i].element );/* Coef. of BK's f */
    /*  mb_fprintf(stdout, "$$ftbl ok.\n");  */

    fget_fitpar(&(atm_f[i].x), &(atm[i].x), fi);
    fget_fitpar(&(atm_f[i].y), &(atm[i].y), fi);
    fget_fitpar(&(atm_f[i].z), &(atm[i].z), fi);
    /* ----- flag: B-factor ----- */
    fget_1s(buf, fi);
    if ( strcmp(buf, "ISO") == 0 || strcmp(buf, "0") == 0){
      /* Isotoropic */
      atm[i].bf = 0;
      fget_fitpar(&(atm_f[i].b), &(atm[i].b), fi);
    }
    else if ( strcmp(buf, "ANISO") == 0 || strcmp(buf, "1") == 0){
      /* Anisotoropic */
      atm[i].bf = 1;
      fget_fitpar(&(atm_f[i].b11), &(atm[i].b11), fi);
      fget_fitpar(&(atm_f[i].b22), &(atm[i].b22), fi);
      fget_fitpar(&(atm_f[i].b33), &(atm[i].b33), fi);
      fget_fitpar(&(atm_f[i].b12), &(atm[i].b12), fi);
      fget_fitpar(&(atm_f[i].b13), &(atm[i].b13), fi);
      fget_fitpar(&(atm_f[i].b23), &(atm[i].b23), fi);
    }
    else {
      printf("$$ Invalid bf. Abort.\n");
      exit(EXIT_FAILURE);
    }

    fget_fitpar(&(atm_f[i].occ), &(atm[i].occ), fi);
    atm[i].af = fget_i(fi);  /* ----- flag: f & f' ----- */
    chk_data("}SITE", fi);
  }
}


BEAM_SEARCH_PAR fget_beam_search(FILE *fi)
{
  char buf[128];
  double t[5];
  BEAM_SEARCH_PAR beam_search;
  
  comment_skip_fgets(90, buf, fi);	str_to_fs(4,buf,t);
  beam_search.nglim = (int) t[0];
  beam_search.gmax	= t[1];
  beam_search.lzmin = (int) t[2];
  beam_search.lzmax = (int) t[3];
  /*beam_search.disksize = t[4];*/
  comment_skip_fgets(90, buf, fi);	str_to_fs(3,buf,t);
  beam_search.wgbmin  = t[0];
  beam_search.wgmax   = t[1];
  beam_search.xigmax  = t[2];
  comment_skip_fgets(90, buf, fi);	str_to_fs(3,buf,t);
  beam_search.wg2bmin = t[0];
  beam_search.wg2max  = t[1];
  beam_search.xig2max = t[2];
  return beam_search;
}


void fget_beam_search_wg(FILE *fi, BEAM_SEARCH_PAR *beam_search)
{
  char buf[128];
  double t[5];
  
  comment_skip_fgets(90, buf, fi);	str_to_fs(4,buf,t);
  beam_search->nglim = (int) t[0];
  beam_search->gmax	= t[1];
  beam_search->lzmin = (int) t[2];
  beam_search->lzmax = (int) t[3];
  /*beam_search->disksize = t[4];*/
  comment_skip_fgets(90, buf, fi);	str_to_fs(3,buf,t);
  beam_search->wgbmin  = t[0];
  beam_search->wgmax   = t[1];
  beam_search->xigmax  = t[2];
}


void fget_beam_search_sg(FILE *fi, BEAM_SEARCH_PAR *beam_search)
{
  char buf[128];
  double t[5];

  comment_skip_fgets(90, buf, fi);	str_to_fs(4,buf,t);
  beam_search->nglim = (int) t[0];
  beam_search->gmax	= t[1];
  beam_search->lzmin = (int) t[2];
  beam_search->lzmax = (int) t[3];
  /*beam_search->disksize = t[4];*/
  comment_skip_fgets(90, buf, fi);	str_to_fs(3,buf,t);
  beam_search->sgbmin  = t[0];
  beam_search->sgmax   = t[1];
  beam_search->xigmax  = t[2];
}


void fget_beam_search_sg_multi(FILE *fi, BEAM_SEARCH_PAR *beam_search)
{
  char buf[128];
  int		i, nlzmax, nc;
  double t[7];

  for(i=0; i<7; i++){ t[i] = 0.0; } /* initialize */

  comment_skip_fgets(90, buf, fi);	nc = str_to_fs(4,buf,t);
  if (nc == 5){
    beam_search->nglim = (int) t[0];
    beam_search->gmax	= t[1];
    beam_search->lzmin = (int) t[2];
    beam_search->lzmax = nlzmax = (int) t[3];
    /*beam_search->disksize = t[4];*/
  }
  /*else if (nc == 6){
    beam_search->nglim = (int) t[0];
    beam_search->gmax	= t[1];
    beam_search->lzmin = (int) t[2];
    beam_search->lzmax = nlzmax = (int) t[3];
    beam_search->disksize = t[4];
    epsilon.vg_select_flag = (int) t[5];
    if(epsilon.vg_select_flag != 0 && epsilon.vg_select_flag != 1){
      printf("$$ Invalid vg_select_flag. Abort.\n");
      exit(EXIT_FAILURE);
    }
  }*/
  else{
    printf("$$ Invalid format for beam_search. Abort.\n");
    exit(EXIT_FAILURE);
  }

  for(i=0; i<=nlzmax; i++){
    comment_skip_fgets(90, buf, fi);	str_to_fs(3,buf,t);
    beam_search->sge_max[i]	= t[0];
    beam_search->sgb_max[i]	= t[1];
    beam_search->xig_max[i] = t[2];
  }
}


void fget_ed_beams(FIT_EXP_DATASET *ed, FILE *fi)
{
  /*
   * Beam search and beam read for ed
   */
  int nr;

  ed->beam_search.flag_bs = fget_i(fi);
  if (
    ed->beam_search.flag_bs == 0 ||
    ed->beam_search.flag_bs == 10
    ){  /* Manual search */
    fget_2i(&(ed->num_exact), &(ed->num_bethe), fi);
    mb_fprintf(
      stdout, "Exact = %d, Bethe = %d\n", ed->num_exact, ed->num_bethe
      );
    nr = ed->num_exact + ed->num_bethe;
    ed->reflex = new_vector(HKL, nr);	chk_vector(ed->reflex);
    fget_beams( ed->num_exact, ed->num_bethe, ed->reflex, fi );
    mb_fprintf(stdout, "$$ vg_select_flag is set to 0.\n");
    epsilon.vg_select_flag = 0;
  }
  else if(
    ed->beam_search.flag_bs == 1 ||
    ed->beam_search.flag_bs == 11
    ){	/* Automatic search (wg) */
    fget_beam_search_wg( fi, &(ed->beam_search) );
    ed->num_exact = 0;	ed->num_bethe = 0;	/* dummy */
  }
  else if(
    ed->beam_search.flag_bs == 2 ||
    ed->beam_search.flag_bs == 12
    ){    /* Automatic search (sg) */
    fget_beam_search_sg(fi, &(ed->beam_search) );
    ed->num_exact = 0;	ed->num_bethe = 0;	/* dummy */
  }
  else if(
    ed->beam_search.flag_bs == 3 ||
    ed->beam_search.flag_bs == 13
    ){	/* Automatic search (sg) */
    fget_beam_search_sg_multi( fi, &(ed->beam_search) );
    ed->num_exact = 0;	ed->num_bethe = 0;	/* dummy */
  }
  else{
    mb_fprintf(stderr,"Invalid flag_bs %d\n", ed->beam_search.flag_bs);
    exit(EXIT_FAILURE);
  }
}


void fget_lofg_cstr(
  FILE *fi, int n_atom, int num_lofg, HKL *g_lofg, int lof_type,
  LO_FG *fg_lofg, LO_FG_FIT *fg_lofg_f
  )
{
  int i, ia, n;

  n = num_lofg;

  if(lof_type == 1){/* 1: whole part of Fg */
    for(i=0; i<n; i++){
      g_lofg[i] = fget_hkl(fi);
      fget_fitpar(&(fg_lofg_f[i].re), &(fg_lofg[i].re), fi); /* Re */
      fget_fitpar(&(fg_lofg_f[i].im), &(fg_lofg[i].im), fi); /* Im */
    }
  }
  else if (lof_type == 2){/* 2: elastic part of Fg */
    for(i=0; i<n; i++){
      g_lofg[i] = fget_hkl(fi);
      fget_fitpar(&(fg_lofg_f[i].ela_re), &(fg_lofg[i].ela_re), fi); /* Re */
      fget_fitpar(&(fg_lofg_f[i].ela_im), &(fg_lofg[i].ela_im), fi); /* Im */
    }
  }
  else if ( lof_type == 3 || lof_type == 4 ){
    /* 3: both f (elastic and inelastic), 4: elastic f */
    for(i=0; i<n; i++){
      g_lofg[i] = fget_hkl(fi);
      for(ia=0; ia<n_atom; ia++){
        fget_fitpar(&(fg_lofg_f[i].fe[ia]), &(fg_lofg[i].fe[ia]), fi); /* Re */
        fget_fitpar(&(fg_lofg_f[i].fi[ia]), &(fg_lofg[i].fi[ia]), fi); /* Im */
      }
    }
  }
  else {
    mb_fprintf(
      stdout, "$$(fget_low_g_cst) Invalid lof_type(=%d). Abort.\n", lof_type
      );
    exit(EXIT_FAILURE);
  }
}


void fget_inc_wave(
  double *accel_kvol, UVW *incidence, UVW *surf_norm,
  R_HKL *zolzbase1, FILE *fp
  )
{
  /*
   * Incident wave
   */
  char buf[BUFLEN];
  double t[3];

  comment_skip_fgets(sizeof buf, buf, fp);	*accel_kvol = atof(buf);
  comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(3,buf,t);
  incidence->u = (int) t[0];
  incidence->v = (int) t[1];
  incidence->w = (int) t[2];

  comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(3,buf,t);
  surf_norm->u = (int) t[0];
  surf_norm->v = (int) t[1];
  surf_norm->w = (int) t[2];

  comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(3,buf,t);
  zolzbase1->h = t[0];
  zolzbase1->k = t[1];
  zolzbase1->l = t[2];
}


void fget_calc_par(
  int *calc_type, int *num_points, int *num_points_x, int *num_points_y,
  double *H_begin, double *H_end, double *G_begin, double *G_end,
  char *hg_file, FILE *fp
  )
{
  char buf[BUFLEN];
  double t[2];
  int		n;
  
  /* Calculation type
    0:0D,1:1D,2:disk,3:square,4:lnp,5:ds-lnp
    7:Arb. incidences
   */
  comment_skip_fgets(BUFLEN, buf, fp);	*calc_type = atoi(buf);

  if( *calc_type == 0 ){
    comment_skip_fgets(BUFLEN, buf, fp);	*num_points = atoi(buf);
    if( *num_points != 1){
      mb_fprintf(
        stdout, "$$ num_points must be 1 for calc_type=0. Abort.\n"
        );
      exit(EXIT_FAILURE);
    }
    comment_skip_fgets(BUFLEN, buf, fp); 	n = str_to_fs(1,buf,t);
    if(n != 1){printf("$$ WARNING: too many H parameters for calc_type=1.\n");}
    *H_begin = t[0];	*H_end   = *H_begin;
  }
  else if( *calc_type == 3 ){ /* lectangle */
    fget_2i(num_points_x, num_points_y, fp);
    fget_2d(H_begin, H_end, fp);
  }
  else{
    comment_skip_fgets(sizeof buf, buf, fp);	*num_points = atoi(buf);
    comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(2,buf,t);
    *H_begin = t[0];	*H_end   = t[1];
  }
  
  if( *calc_type == 0 || *calc_type == 1 || *calc_type == 4 || *calc_type == 5){
    comment_skip_fgets(sizeof buf, buf, fp); 	n = str_to_fs(1,buf,t);
    if(n != 1){
      mb_fprintf(
        stdout, "$$ WARNING: too many G parameters for the calc_type.\n"
        );}
    *G_begin = t[0];	*G_end   = *G_begin;
  }
  else{
    comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(2,buf,t);
    *G_begin = t[0];	*G_end   = t[1];
  }
}


void fget_beams(int num_exact, int num_bethe, HKL *reflex, FILE *fp )
{
  char buf[BUFLEN];
  double t[3];
  int i;
  
  for(i=0; i<num_exact; i++){
    comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(3,buf,t);
    reflex[i].h = (int) t[0];
    reflex[i].k = (int) t[1];
    reflex[i].l = (int) t[2];
  }
  for(i=num_exact; i<(num_exact + num_bethe); i++){
    comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(3,buf,t);
    reflex[i].h = (int) t[0];
    reflex[i].k = (int) t[1];
    reflex[i].l = (int) t[2];
  }
}


void beam_merge(
  int num_exact, int num_bethe, HKL *g_exact, HKL *g_bethe, HKL *reflex
  )
{
  int i;	
  for(i=0;i<num_exact;i++){ reflex[i] = g_exact[i]; }
  for(i=0;i<num_bethe;i++){ reflex[i+num_exact] = g_bethe[i]; }
}


void fget_cal_g(
  int num_cal_g, int num_exact, int num_bethe, int *nth_g, HKL *reflex, FILE *fp
  )
{
  char buf[BUFLEN];
  double t[3];
  int i,nr;
  HKL g;
  
  nr = num_exact + num_bethe;
  for(i=0; i<num_cal_g; i++){
    comment_skip_fgets(sizeof buf, buf, fp); 	str_to_fs(3,buf,t);
    g.h = (int) t[0]; g.k = (int) t[1]; g.l = (int) t[2];
    nth_g[i] = find_n_g( num_exact, g, reflex );
    if (nth_g[i] == -1) {
      mb_fprintf( stdout,
        "$$ cal_g: %d %d %d is not found in the exact beams. Abort.\n",
        g.h, g.k, g.l
        );
      exit(EXIT_FAILURE);
    }
  }
}


void fget_cal_thick(int num_cal_thick, double *thickness, FILE *fp )
{
  char buf[BUFLEN];
  int i;

  for(i=0; i<num_cal_thick; i++){
    comment_skip_fgets(sizeof buf, buf, fp);
    thickness[i] = atof(buf);
  }
}


void marge_cal_gt(
  int num_cal_g, int num_cal_thick,
  int *nth_g, double *thickness, int *gt_nth_g, double *gt_thickness
  )
{
  int i,j,ic;
  ic = 0;
  for(i=0; i<num_cal_thick; i++){
    for(j=0; j<num_cal_g; j++){
      gt_nth_g[ic] = nth_g[j];
      gt_thickness[ic] = thickness[i];
      ic++;
    }
  }
}


void dbg_g_filein(
  FILE *fp, int nglim, int *ne, int *nb, HKL *tg_exact, HKL *tg_bethe
  )
{
  /* for debug */
  int pre_ne, pre_nb, i, j;

  mb_fprintf(stdout, "$$ g read from %s.ggi\n", dbg.infile);
  pre_ne = *ne; pre_nb = *nb;
  fget_2i(&i, &j, fp);	*ne = i;  *nb = j;
  if( *ne > nglim || *nb > nglim ){
    mb_fprintf(
      stdout,
      "$$ Error: Num. of reflections in GFILE is greater than nglim. Abort.\n"
      );
    mb_fprintf(stdout, "$$ Input bigger nglim.\n");
    exit(EXIT_FAILURE);
  }
  else if( *ne < 1 ){
    mb_fprintf(stdout, "$$ Error: Num. of reflections in GFILE is 0. Abort.\n");
    exit(EXIT_FAILURE);
  }
  for(i=0; i<*ne; i++){
    tg_exact[i] = fget_hkl(fp);
    /*printf("%d %d %d\n", tg_exact[i].h, tg_exact[i].k, tg_exact[i].l );*/
  }
  for(i=0; i<*nb; i++){
    tg_bethe[i] = fget_hkl(fp);
    /*printf("%d %d %d\n", tg_bethe[i].h, tg_bethe[i].k, tg_bethe[i].l );*/
  }
  chk_data("RFE",fp);
}


void dbg_vg_filein( FILE *fp, int ne, int nb, HKL *reflex, complex **vg )
     /* for debug */
{
  int i,j;
  double w,x,y,z;
  
  mb_fprintf(stdout, "$$ vg read from %s.vgi\n", dbg.infile);
  fget_2i(&i, &j, fp);
  if( ne != i || nb != j ){
    mb_fprintf(stdout, "$$ Error: Different size of the matrix. Abort.\n");
    exit(EXIT_FAILURE);
  }
  for(j=0; j< ne + nb; j++){
    for(i=0; i< ne + nb; i++){
      fget_4d( &w, &x, &y, &z, fp);
      /*
      if( (int)w != i | (int)x != j ) {
        printf("not match in i,j : %d %d\n", (int)w, (int)x );
        }
        */
      vg[i][j].re = y;  vg[i][j].im = z;
    }
  }
  chk_data("RFE",fp);
}


void input_csd_cstr(CRYSTAL_STRCT_DATA *csd, FILE *fi, char *infile)
{
  /*
   * Input crystal structure data from a file (with constraint)
   * updated on 030221 by KT
   */
  int inversion, natm, nlg; /* <-- dummy */
  CELL_PAR bv;

  chk_data("CRYSTAL_STRUCTURE_DATA{", fi);

  chk_data("LATTICE{", fi);
  csd->space_group = fget_i(fi);
  csd->sym_op = read_sp_group_tbl(
    csd->space_group, &(csd->num_sym_op), &inversion, csd->lattice_type
    ); /* with allocation of sym_op[] */
  fget_latpar_cstr( fi, &(csd->lp), &(csd->lp_f));
  chk_data("}LATTICE", fi);

  chk_data("ATOM{", fi);
  natm = csd->n_atom = fget_i(fi);
  /* mb_fprintf(stdout, "$$ natm=%d\n", natm); */
  csd->atm = new_vector(ATOM, natm);  chk_vector(csd->atm);
  csd->atm_f = new_vector(ATOM_FIT, natm);  chk_vector(csd->atm_f);
  csd->xdyl = new_vector(XDT, natm);  chk_vector(csd->xdyl);
  csd->ftbl = new_vector(ATOMF_BK, natm);  chk_vector(csd->ftbl);
  fget_atmpar_cstr(fi, natm, csd->atm, csd->atm_f, csd->xdyl, csd->ftbl);
  chk_data("}ATOM",fi);

  /* Low-order structure factors */
  chk_data("LOW_ORDER_FG{",fi);
  nlg = csd->num_lofg = fget_i(fi);
  if( nlg > 0 ){
    csd->g_lofg = new_vector(HKL, nlg);  chk_vector(csd->g_lofg);
    csd->fg_lofg = new_vector(LO_FG, nlg);  chk_vector(csd->fg_lofg);
    csd->fg_lofg_f = new_vector(LO_FG_FIT, nlg);  chk_vector(csd->fg_lofg_f);

    csd->lof_type = fget_i(fi);     /* type of fg_lo model */
    alloc_csd_sub_cstr( csd->n_atom, nlg, csd->lof_type, csd );
    fget_lofg_cstr(
      fi, natm, nlg, csd->g_lofg, csd->lof_type,
      csd->fg_lofg, csd->fg_lofg_f
      );
  }
  chk_data("}LOW_ORDER_FG", fi);

  chk_data("}CRYSTAL_STRUCTURE_DATA", fi);

  constraint_csd( csd );
  bv = bravai(csd->lp);
  atom_multiplicity( natm, csd->atm, csd->num_sym_op, csd->sym_op );
  dbg_atm_pos_out( csd, infile );
}
