/*
 * mbutl.c
 *  Utility functions for mbfit
 *  - Visualization of mbfit output data.
 * Programmed by Kenji Tsuda, IMRAM, Tohoku University, Sendai, JAPAN.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "mbconfig.h"
#include "fitting.h"
#include "mb_input.h"
#include "data_out.h"
#include "mb_sim.h"
#include "ts_scan.h"
#include "e_density.h"
#include "mbu_output.h"

#include "mbu_intrct.h"

#define  BUFSIZE  256


DEBUG_INFO	dbg;
EPS_CTRL	epsilon;

/* #define TMP_TEST	*/  /* ifdef only for temp. test */
/*
void fprintf(FILE *fp, char *fmt,...);
void fclose(FILE *fp);
void mb_fflush(FILE *fp);
*/
void print_phys_const(void);
void mb_rfactor(void);
void mb_dfactor(void);
/*void mb_lnprf(void); */
void mb_thinout(void);

/*void tmp_test(void);
void asf_test(void);*/
/*void read_eps_ctrl_params( FILE *fi );*/


#if 0
void mb_HOLZ_lines()
{
  /*
  HOLZ line
  lattice parameters
  uvw, ZOLZ base vector g1
  zolz_ortho_base --> g2
  HOLZ reflex.
  proj_to_zolz,g1 and g2 compo.
  -> gz
  Kz
  HOLZ line, line eq.'s coef.
   */
  char buf[BUFSIZE];
  double  t[3];
  double  acckvol, u000, chi, abs_K;
  double  abs_incidence, abs_zolzbase1, abs_zolzbase2, sqabs_holzg;
  double  holzg_x, holzg_y, holzg_z, K_x, K_y, K_z;
  double  c1, c2, slope, intercept;
  int  nth_lauezone;
  HKL  holzg;
  LAT_PAR  lp;
  UVW  incid;
  R_HKL  zolzbase1, zolzbase2;
  CELL_PAR  bv;
  
  /* on 2001-10-25 */
  printf("*****@mb_HOLZ_lines() test (on 2001-10-25)\n");
  printf("Lattice parameters:\n");
  printf("a, b, c =");	fgets(buf,sizeof(buf),stdin);	
  str_to_fs(3, buf, t);
  lp.a = t[0];	lp.b = t[1];	lp.c = t[2];
  printf("alpha, beta, gamma =");	fgets(buf,sizeof(buf),stdin);	
  str_to_fs(3, buf, t);
  lp.alpha = t[0];	lp.beta = t[1];	lp.gamma = t[2];
  bv = bravai(lp);
  
  printf("Accelerating voltage [kV]:\n");
  printf("acckvol =");	fgets(buf,sizeof(buf),stdin);	
  acckvol = atof(buf);
  printf("U(000) [angstrom^{-2}]:\n");
  printf("u000 =");	fgets(buf,sizeof(buf),stdin);	
  u000 = atof(buf);
  chi = 1.0 / wavelength(acckvol);
  abs_K = sqrt( sq(chi) + u000 );
  K_z = abs_K;
  printf("$$K_z = %lf\n", K_z);
  
  printf("Incident direction:\n");
  printf("u, v, w =");	fgets(buf,sizeof(buf),stdin);	
  str_to_fs(3, buf, t);
  incid.u = (int) t[0];	incid.v = (int) t[1];	incid.w = (int) t[2];
  abs_incidence = abs_uvw(incid, bv);
  
  printf("ZOLZ base vector (perpendicular to the incidence):\n");
  printf("h, k, l = ");	fgets(buf,sizeof(buf),stdin);	
  str_to_fs(3, buf, t);
  zolzbase1.h = t[0];	zolzbase1.k = t[1];	zolzbase1.l = t[2];
  
  /* Check */
  nth_lauezone = incid.u * (int) zolzbase1.h
    + incid.v * (int) zolzbase1.k
      + incid.w * (int) zolzbase1.l;
  if( nth_lauezone != 0 ){
    printf("ZOLZ base vector %d %d %d is not perpendicular to the incidence.\n");
    printf("Abort.\n");
    exit(EXIT_FAILURE);
  }
  abs_zolzbase1 = sqrt( sq_abs(zolzbase1, bv) );
  zolzbase2 = zolz_ort_base(incid, zolzbase1, bv);
  abs_zolzbase2 = sqrt(sq_abs(zolzbase2, bv));
  
  printf(
    "$$ zolzbase1 = %lf %lf %lf\n",
    zolzbase1.h, zolzbase1.k, zolzbase1.l
    );
  printf(
    "$$ zolzbase2 = %lf %lf %lf\n",
    zolzbase2.h, zolzbase2.k, zolzbase2.l
    );
  printf(
    "$$ abs_zolzbase1, abs_zolzbase2 = %lf, %lf\n",
    abs_zolzbase1, abs_zolzbase2
    );
  
  printf("HOLZ reflection: (Enter QUIT to exit)\n");
  printf(" h, k, l =");	fgets(buf,sizeof(buf),stdin);
  
  while( strcmp(buf, "QUIT") != 0 && strcmp(buf, "quit") !=0 ){
    str_to_fs(3, buf, t);
    holzg.h = (int) t[0];
    holzg.k = (int) t[1];
    holzg.l = (int) t[2];
    sqabs_holzg = sq_abs(hkl2rhkl(holzg), bv);
    printf("$$ sqabs_holzg =%lf\n", sqabs_holzg);
    proj_to_zolz2(holzg, incid, zolzbase1, zolzbase2, bv, &c1, &c2);
    printf("$$ c1, c2 = %lf %lf\n", c1, c2);
    printf(
      "$$ proj. holz = %lf %lf %lf\n",
      c1 * zolzbase1.h + c2 * zolzbase2.h,
      c1 * zolzbase1.k + c2 * zolzbase2.k,
      c1 * zolzbase1.l + c2 * zolzbase2.l
      );
    holzg_x = c1 * abs_zolzbase1;
    holzg_y = c2 * abs_zolzbase2;
    
    printf("$$ holzg_x, holzg_y = %lf %lf\n", holzg_x, holzg_y);
    /* holzg_z = calc_g_z( holzg, abs_incidence, incid ); */
    holzg_z = inner_product_uvw_hkl( incid, hkl2rhkl(holzg) ) / abs_incidence;
    printf("$$ holzg_z = %lf\n", holzg_z);
    /*
      Equation of HOLZ line
      Ky = -Kx*gx/gy + Kz*gz/gy - g*g/2/gy;
      Y = k0 * X + c0
     */
    if ( fabs(holzg_y) > EPS ){
      slope = - holzg_x / holzg_y;
      intercept = K_z * holzg_z / holzg_y - sqabs_holzg * 0.5 / holzg_y;
      printf(
        "$$ K_z * holzg_z / holzg_y = %lf\n", K_z * holzg_z / holzg_y
        );
      printf(
        "$$ sqabs_holzg * 0.5 / holzg_y = %lf\n", sqabs_holzg * 0.5 / holzg_y
        );
    }
    else{
      slope = 1e+12;
      intercept = 0.0;
    }
    printf("h k l = %d %d %d\n", holzg.h, holzg.k, holzg.l);
    printf("slope = %lf\n", slope);
    printf("intercept = %lf\n", intercept);
    printf("HOLZ reflection: (Enter QUIT to exit)\n");
    printf(" h, k, l =");	fgets(buf,sizeof(buf),stdin);
  }
  printf("See you.\n");
}
#endif


#if 0
void tmp_test_alloc(void)
     /* malloc test */
{
  int i,j, ***t;
  
  printf("$$ test\n");
  print_phys_const();
  
  t = (void ***) malloc(10 * sizeof(void **));  chk_vector(t);
  /*
  t = new_vector(int, 10);  chk_vector((void *) t);
  ==> not allowed
   */
  for(i=0; i<20; i++){
    t[i] = (void **) malloc(10 * sizeof(void *));	chk_vector(t[i]);
    for(j=0; j<12; j++){
      t[i][j] = new_vector(int, 10);  chk_vector(t[i][j]);
    }
  }
  
  printf("$$ tmp_test\n");
  for(i=0; i<10; i++){
    t[i][i][i] = i;
  }

  for(i=0; i<10; i++){
    printf("%d ", t[i][i][i]);
  }
  free_tensor4(t);
}
#endif


void opening_title(void)
{
  printf("\n*****\n");
  printf("***** MBUTL: visualization utility for mbfit output data.\n");
  printf("*****  programmed by Kenji Tsuda, IMRAM, Tohoku University, Sendai, JAPAN.\n");
  printf("*****\n");
}


void print_phys_const(void)
{
/* Test function */
  double c, e, h, me, pi;
  
  pi = 3.141592654;
  h = 2.0 * pi * 1.05457266e-34;  /* [J s] */
  me = 9.1093897e-31;  /* [kg] */
  e = 1.60217733e-19;  /* [C] */
  c = 2.99792458e+8;  /* [m s^{-1}] */
  
  printf("***** Physical const. *****\n");
  printf(
    " h/sqrt(2*me*e)=%12.8lg, e/(2*me*sq(c))=%12.8lg\n",
    h / sqrt(2 * me * e), e / (2 * me * sq(c))
    );
  printf(
    " GE = 2 * pi * me * e / sq(h) = %12.8lg\n",
    2 * pi * me * e / sq(h)
    );
  printf(" e / me / sq(c) = %12.8lg\n\n", e / me / sq(c));
}


void output_html_header(FILE *fo_html, char *buf)
{
  fprintf(fo_html, "<html>\n");
  fprintf(fo_html, "<title>%s</title>\n", buf);
  fprintf(fo_html, "<body>\n");
}


void exec_gnuplot(char *pltfile)
{
  char buf[BUFSIZE];
  
#if defined(LINUX_MODE) /* for Linux */
  sprintf(buf, "gnuplot %s", pltfile);
  system(buf);
#else   /* win mode */
  sprintf(buf, "wgnuplot %s", pltfile);
  system(buf);
#endif
}


void pltout_dispsurf(
  int branch_begin, int branch_end, int num_rf_amp, int num_thick,
  char *outfile_ds, char *outfile_amp, char *outfile_int,
  char *infile, FILE *fo_html
  )
{
  /*
   * For gnuplot 
   */
  int i, ig, it;
  char outfile_ds_re_plt[BUFSIZE], outfile_ds_im_plt[BUFSIZE];
  char outfile_ds_eigre_plt[BUFSIZE], outfile_ds_eigim_plt[BUFSIZE];
  char outfile_ds_sqeps_plt[BUFSIZE];
  char outfile_amp_plt[BUFSIZE], outfile_int_plt[BUFSIZE];
  FILE *fo_ds_re_plt, *fo_ds_im_plt, *fo_ds_sqeps_plt;
  FILE *fo_ds_eigre_plt, *fo_ds_eigim_plt;
  FILE *fo_amp_plt, *fo_int_plt;
  
  /*
    plt file for Re parts of _ds
   */
  sprintf(outfile_ds_re_plt, "%s_ds_re.plt", infile);
  fo_ds_re_plt = fopen_chk_wmode(outfile_ds_re_plt);
  fprintf(fo_ds_re_plt, "#\n");
  fprintf(fo_ds_re_plt, "set xlabel \"k_xy(H)\"\n");
  fprintf(fo_ds_re_plt, "set ylabel \"Re{k_z}\"\n");
  fprintf(fo_ds_re_plt, "set title \"Dispersion surface\"\n");
  fprintf(fo_ds_re_plt, "#set xrange [0:2]\n");
  fprintf(fo_ds_re_plt, "#set yrange [0:1]\n");
  fprintf(fo_ds_re_plt, "#set xtics 1\n");
  fprintf(fo_ds_re_plt, "#set mxtics 5\n");
  fprintf(fo_ds_re_plt, "#set ytics 0.5\n");
  fprintf(fo_ds_re_plt, "#set mytics 5\n");
  fprintf(
    fo_ds_re_plt,
    "plot \"%s\" using 1:3 title \"Kinematical\" with dots, \\\n",
    outfile_ds
    );
  fprintf(
    fo_ds_re_plt,
    "\"%s\" using 1:%d title \"branch %d\" with lines, \\\n",
    outfile_ds, 4+(branch_begin-1)*5, branch_begin
    );
  for(i=branch_begin+1; i<branch_end; i++){
    fprintf(
      fo_ds_re_plt,
      "\"%s\" using 1:%d title \"%d\" with lines, \\\n",
      outfile_ds, 4+(i-1)*5, i
      );
  }
  fprintf(
    fo_ds_re_plt,
    "\"%s\" using 1:%d title \"%d\" with lines\n",
    outfile_ds, 4+(branch_end-1)*5, branch_end
    );
  fprintf(fo_ds_re_plt, "set term postscript\n");
  fprintf(fo_ds_re_plt, "set output \"%s_ds_re.ps\"\n", infile);
  fprintf(fo_html, "<hr><p><img src=\"%s_ds_re.ps\"></p>\n", infile);
  fprintf(fo_ds_re_plt, "#set term postscript\n");
  fprintf(fo_ds_re_plt, "#set output \"%s_ds_re.ps\"\n", infile);
  fprintf(fo_ds_re_plt, "replot\n");
  fprintf(fo_ds_re_plt, "#pause -1\n");
  fclose(fo_ds_re_plt);
  exec_gnuplot(outfile_ds_re_plt);
  
  /*
    plt file for Im parts of _ds
    */
  sprintf(outfile_ds_im_plt, "%s_ds_im.plt", infile);
  fo_ds_im_plt = fopen_chk_wmode(outfile_ds_im_plt);
  fprintf(fo_ds_im_plt, "#\n");
  fprintf(fo_ds_im_plt, "set xlabel \"k_xy(H)\"\n");
  fprintf(fo_ds_im_plt, "set ylabel \"Im{k_z}\"\n");
  fprintf(fo_ds_im_plt, "set title \"Absorption (Dispersion surface)\"\n");
  fprintf(fo_ds_im_plt, "#set xrange [0:2]\n");
  fprintf(fo_ds_im_plt, "#set yrange [0:1]\n");
  fprintf(fo_ds_im_plt, "#set xtics 1\n");
  fprintf(fo_ds_im_plt, "#set mxtics 5\n");
  fprintf(fo_ds_im_plt, "#set ytics 0.5\n");
  fprintf(fo_ds_im_plt, "#set mytics 5\n");
  fprintf(
    fo_ds_im_plt,
    "plot \"%s\" using 1:%d title \"branch %d\" with lines, \\\n",
    outfile_ds, 5+(branch_begin-1)*5, branch_begin
    );
  for(i=branch_begin+1; i<branch_end; i++){
    fprintf(
      fo_ds_im_plt,
      "\"%s\" using 1:%d title \"%d\" with lines, \\\n",
      outfile_ds, 5+(i-1)*5, i
      );
  }
  fprintf(
    fo_ds_im_plt,
    "\"%s\" using 1:%d title \"%d\" with lines\n",
    outfile_ds, 5+(branch_end-1)*5, branch_end
    );
  fprintf(fo_ds_im_plt, "set term postscript\n");
  fprintf(fo_ds_im_plt, "set output \"%s_ds_im.ps\"\n", infile);
  fprintf(fo_html, "<p><img src=\"%s_ds_im.png\"></p>\n", infile);
  fprintf(fo_ds_im_plt, "#set term postscript\n");
  fprintf(fo_ds_im_plt, "#set output \"%s_ds_im.ps\"\n", infile);
  fprintf(fo_ds_im_plt, "replot\n");
  fprintf(fo_ds_im_plt, "#pause -1\n");
  fclose(fo_ds_im_plt);
  exec_gnuplot(outfile_ds_im_plt);

#if 0
  /*
    plt file of eigre for _ds
    */
  sprintf(outfile_ds_eigre_plt, "%s_ds_eigre.plt", infile);
  fo_ds_eigre_plt = fopen_chk_wmode(outfile_ds_eigre_plt);
  fprintf(fo_ds_eigre_plt, "#\n");
  fprintf(fo_ds_eigre_plt, "set xlabel \"k_xy(H)\"\n");
  fprintf(fo_ds_eigre_plt, "set ylabel \"Re_gamma\"\n");
  fprintf(
    fo_ds_eigre_plt,
    "set title \"Re part of eigen values\"\n"
    );
  fprintf(fo_ds_eigre_plt, "#set xrange [0:2]\n");
  fprintf(fo_ds_eigre_plt, "#set yrange [0:1]\n");
  fprintf(fo_ds_eigre_plt, "#set xtics 1\n");
  fprintf(fo_ds_eigre_plt, "#set mxtics 5\n");
  fprintf(fo_ds_eigre_plt, "#set ytics 0.5\n");
  fprintf(fo_ds_eigre_plt, "#set mytics 5\n");
  fprintf(
    fo_ds_eigre_plt,
    "plot \"%s\" using 1:%d title \"branch %d\" with lines, \\\n",
    outfile_ds, 6+(branch_begin-1)*5, branch_begin
    );
  for(i=branch_begin+1; i<branch_end; i++){
    fprintf(
      fo_ds_eigre_plt,
      "\"%s\" using 1:%d title \"%d\" with lines, \\\n",
      outfile_ds, 6+(i-1)*5, i
      );
  }
  fprintf(
    fo_ds_eigre_plt,
    "\"%s\" using 1:%d title \"%d\" with lines\n",
    outfile_ds, 6+(branch_end-1)*5, branch_end
    );
  fprintf(fo_ds_eigre_plt, "set term postscript\n");
  fprintf(fo_ds_eigre_plt, "set output \"%s_ds_eigre.ps\"\n", infile);
  fprintf(fo_html, "<p><img src=\"%s_ds_eigre.png\"></p>\n", infile);
  fprintf(fo_ds_eigre_plt, "#set term postscript\n");
  fprintf(fo_ds_eigre_plt, "#set output \"%s_ds_eigre.ps\"\n", infile);
  fprintf(fo_ds_eigre_plt, "replot\n");
  fprintf(fo_ds_eigre_plt, "#pause -1\n");
  fclose(fo_ds_eigre_plt);
  exec_gnuplot(outfile_ds_eigre_plt);

  /*
    plt file of eigim for _ds
    */
  sprintf(outfile_ds_eigim_plt, "%s_ds_eigim.plt", infile);
  fo_ds_eigim_plt = fopen_chk_wmode(outfile_ds_eigim_plt);
  fprintf(fo_ds_eigim_plt, "#\n");
  fprintf(fo_ds_eigim_plt, "set xlabel \"k_xy(H)\"\n");
  fprintf(fo_ds_eigim_plt, "set ylabel \"Im{k_z}\"\n");
  fprintf(
    fo_ds_eigim_plt,
    "set title \"Im part of eigen values\"\n"
    );
  fprintf(fo_ds_eigim_plt, "#set xrange [0:2]\n");
  fprintf(fo_ds_eigim_plt, "#set yrange [0:1]\n");
  fprintf(fo_ds_eigim_plt, "#set xtics 1\n");
  fprintf(fo_ds_eigim_plt, "#set mxtics 5\n");
  fprintf(fo_ds_eigim_plt, "#set ytics 0.5\n");
  fprintf(fo_ds_eigim_plt, "#set mytics 5\n");
  fprintf(
    fo_ds_eigim_plt,
    "plot \"%s\" using 1:%d title \"branch %d\" with lines, \\\n",
    outfile_ds, 7+(branch_begin-1)*5, branch_begin
    );
  for(i=branch_begin+1; i<branch_end; i++){
    fprintf(
      fo_ds_eigim_plt,
      "\"%s\" using 1:%d title \"%d\" with lines, \\\n",
      outfile_ds, 7+(i-1)*5, i
      );
  }
  fprintf(
    fo_ds_eigim_plt,
    "\"%s\" using 1:%d title \"%d\" with lines\n",
    outfile_ds, 7+(branch_end-1)*5, branch_end
    );
  fprintf(fo_ds_eigim_plt, "set term postscript\n");
  fprintf(fo_ds_eigim_plt, "set output \"%s_ds_eigim.ps\"\n", infile);
  fprintf(fo_html, "<p><img src=\"%s_ds_eigim.png\"></p>\n", infile);
  fprintf(fo_ds_eigim_plt, "#set term postscript\n");
  fprintf(fo_ds_eigim_plt, "#set output \"%s_ds_eigim.ps\"\n", infile);
  fprintf(fo_ds_eigim_plt, "replot\n");
  fprintf(fo_ds_eigim_plt, "#pause -1\n");
  fclose(fo_ds_eigim_plt);
  exec_gnuplot(outfile_ds_eigim_plt);
#endif
  
  /*
    plt file of sqeps for _ds
    */
  sprintf(outfile_ds_sqeps_plt, "%s_ds_sqeps.plt", infile);
  fo_ds_sqeps_plt = fopen_chk_wmode(outfile_ds_sqeps_plt);
  fprintf(fo_ds_sqeps_plt, "#\n");
  fprintf(fo_ds_sqeps_plt, "set xlabel \"k_xy(H)\"\n");
  fprintf(fo_ds_sqeps_plt, "set ylabel \"eps^2\"\n");
  fprintf(
    fo_ds_sqeps_plt,
    "set title \"Excitation amplitudes of branches (eps^2)\"\n"
    );
  fprintf(fo_ds_sqeps_plt, "#set xrange [0:2]\n");
  fprintf(fo_ds_sqeps_plt, "#set yrange [0:1]\n");
  fprintf(fo_ds_sqeps_plt, "#set xtics 1\n");
  fprintf(fo_ds_sqeps_plt, "#set mxtics 5\n");
  fprintf(fo_ds_sqeps_plt, "#set ytics 0.5\n");
  fprintf(fo_ds_sqeps_plt, "#set mytics 5\n");
  fprintf(
    fo_ds_sqeps_plt,
    "plot \"%s\" using 1:%d title \"branch %d\" with lines, \\\n",
    outfile_ds, 8+(branch_begin-1)*5, branch_begin
    );
  for(i=branch_begin+1; i<branch_end; i++){
    fprintf(
      fo_ds_sqeps_plt,
      "\"%s\" using 1:%d title \"%d\" with lines, \\\n",
      outfile_ds, 8+(i-1)*5, i
      );
  }
  fprintf(
    fo_ds_sqeps_plt,
    "\"%s\" using 1:%d title \"%d\" with lines\n",
    outfile_ds, 8+(branch_end-1)*5, branch_end
    );
  fprintf(fo_ds_sqeps_plt, "set term postscript\n");
  fprintf(fo_ds_sqeps_plt, "set output \"%s_ds_sqeps.ps\"\n", infile);
  fprintf(fo_html, "<p><img src=\"%s_ds_sqeps.png\"></p>\n", infile);
  fprintf(fo_ds_sqeps_plt, "#set term postscript\n");
  fprintf(fo_ds_sqeps_plt, "#set output \"%s_ds_sqeps.ps\"\n", infile);
  fprintf(fo_ds_sqeps_plt, "replot\n");
  fprintf(fo_ds_sqeps_plt, "#pause -1\n");
  fclose(fo_ds_sqeps_plt);
  exec_gnuplot(outfile_ds_sqeps_plt);

#if 0
  /*
    plt file of amplitudes for _amp
    */
  sprintf(outfile_amp_plt, "%s_amp.plt", infile);
  fo_amp_plt = fopen_chk_wmode(outfile_amp_plt);
  fprintf(fo_amp_plt, "#\n");
  fprintf(fo_amp_plt, "set xlabel \"k_xy(H)\"\n");
  fprintf(fo_amp_plt, "set ylabel \"eps^2\"\n");
  fprintf(
    fo_ds_sqeps_plt,
    "set title \"Excitation amplitudes of branches (eps^2)\"\n"
    );
  fprintf(fo_amp_plt, "#set xrange [0:2]\n");
  fprintf(fo_amp_plt, "#set yrange [0:1]\n");
  fprintf(fo_amp_plt, "#set xtics 1\n");
  fprintf(fo_amp_plt, "#set mxtics 5\n");
  fprintf(fo_amp_plt, "#set ytics 0.5\n");
  fprintf(fo_amp_plt, "#set mytics 5\n");
  fprintf(
    fo_amp_plt,
    "plot \"%s\" using 1:%d title \"branch %d\" with lines, \\\n",
    outfile_ds, 8+(branch_begin-1)*5, branch_begin
    );
  for(i=branch_begin+1; i<branch_end; i++){
    fprintf(
      fo_amp_plt,
      "\"%s\" using 1:%d title \"%d\" with lines, \\\n",
      outfile_ds, 8+(i-1)*5, i
      );
  }
  fprintf(
    fo_amp_plt,
    "\"%s\" using 1:%d title \"%d\" with lines\n",
    outfile_ds, 8+(branch_end-1)*5, branch_end
    );
  fprintf(fo_amp_plt, "set term postscript\n");
  fprintf(fo_amp_plt, "set output \"%s_ds_sqeps.ps\"\n", infile);
  fprintf(fo_html, "<p><img src=\"%s_amp.png\"></p>\n", infile);
  fprintf(fo_amp_plt, "#set term postscript\n");
  fprintf(fo_amp_plt, "#set output \"%s_ds_sqeps.ps\"\n", infile);
  fprintf(fo_amp_plt, "replot\n");
  fprintf(fo_amp_plt, "#pause -1\n");
  fclose(fo_amp_plt);
  exec_gnuplot(outfile_amp_plt);
#endif
  /*
    plt file of phases for _amp
    */

  /*
    plt file for _int
    */

  /* fo_int_plt, outfile_int_plt */

}


void job_dispsurf_visual_mode(FILE *fi)
{
  /*
     Output plt files
   */
  int   branch_begin, branch_end, num_rf_amp, num_thick;
  char  buf[BUFSIZE];
  char  htmlfile[BUFSIZE];
  char  outfile_ds[BUFSIZE], outfile_amp[BUFSIZE], outfile_int[BUFSIZE];
  char  infile[BUFSIZE];
  FILE  *fo_html, *fo_plt;

  while( fget_1s(buf, fi) != NULL ){
    if( strcmp(buf, "HTML_OPEN") == 0 ){
      fget_1s(buf, fi);
      strcpy(htmlfile, buf);
      fo_html = fopen_chk_wmode(htmlfile);
      output_html_header(fo_html, htmlfile);
    }
    else if( strcmp(buf, "HTML_CLOSE") == 0 ){
      fprintf(fo_html, "</body>\n</html>\n");
      fclose(fo_html);
      printf("\n==> SEE %s\n\n", htmlfile);
    }
    else if( strcmp(buf, "DISPSURF_VISUAL") == 0 ){
      branch_begin = fget_i(fi);
      branch_end = fget_i(fi);
      num_rf_amp = fget_i(fi);
      num_thick = fget_i(fi);
      fget_1s( infile, fi );
      fget_1s( outfile_ds, fi );
      fget_1s( outfile_amp, fi );
      fget_1s( outfile_int, fi );
      pltout_dispsurf(
        branch_begin, branch_end, num_rf_amp, num_thick, 
        outfile_ds, outfile_amp, outfile_int, infile, fo_html
        );
    }
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}


void pltout_pp( char *pp_file, char *infile, FILE *fo_html )
{
  char  outfile_plt[BUFSIZE];
  FILE  *fo_plt;
  sprintf(outfile_plt, "%s_pp.plt", infile);
  fo_plt = fopen_chk_wmode(outfile_plt);
  fprintf(fo_plt, "set size square\n");
  fprintf(fo_plt, "set view 5,5\n");
  fprintf(fo_plt, "set nosurface\n");
  fprintf(fo_plt, "set pm3d\n");
  fprintf(fo_plt, "set term postscript\n");
  fprintf(fo_plt, "set output \"%s_pp.ps\n", infile);
  fprintf(fo_html, "<hr><p>Projected potential [V]</p>\n");
  fprintf(fo_html, "<p><img src=\"%s_pp.png\"></p>\n", infile);
  fprintf(fo_plt, "splot \"%s\" using 1:2:3 with lines\n", pp_file);
  fprintf(fo_plt, "#pause -1\n");
  fclose(fo_plt);
  exec_gnuplot(outfile_plt);
}


void pltout_bloch(
  char *bl_file, char *infile, int ibranch, double excitation, FILE *fo_html
  )
{
  char  outfile_plt[BUFSIZE];
  FILE  *fo_plt;
  sprintf(outfile_plt, "%s_b%d.plt", infile, ibranch);
  fo_plt = fopen_chk_wmode(outfile_plt);
  fprintf(fo_plt, "set size square\n");
  fprintf(fo_plt, "set view 5,5\n");
  fprintf(fo_plt, "set nosurface\n");
  fprintf(fo_plt, "set pm3d\n");
  fprintf(fo_plt, "set term postscript\n");
  fprintf(fo_plt, "set output \"%s_b%d.ps\n", infile, ibranch);
  fprintf(
    fo_html, "<hr><p>Branch %d, excitation (eps^2) = %g</p>\n", ibranch, excitation
    );
  fprintf(fo_html, "<p><img src=\"%s_b%d.png\"></p>\n", infile, ibranch);
  fprintf(fo_plt, "splot \"%s\" using 1:2:3 with lines\n", bl_file);
  fprintf(fo_plt, "#splot \"%s\" using 1:2:6 with lines\n", bl_file);
  fprintf(fo_plt, "#pause -1\n");
  fclose(fo_plt);
  exec_gnuplot(outfile_plt);
}


void job_bloch_visual_mode(FILE *fi)
{
  /*
     Output plt files
   */
  int   branch_begin, branch_end, ib, ibranch;
  double  excitation;
  char  buf[BUFSIZE], infile[BUFSIZE], htmlfile[BUFSIZE];
  char  blfile[BUFSIZE], ppfile[BUFSIZE];
  FILE  *fo_html, *fo_plt;

  while( fget_1s(buf, fi) != NULL ){
    if( strcmp(buf, "HTML_OPEN") == 0 ){
      fget_1s(buf, fi);
      strcpy(htmlfile, buf);
      fo_html = fopen_chk_wmode(htmlfile);
      output_html_header(fo_html, htmlfile);
      fprintf(
        fo_html, "<p>Probability density distribution of Bloch waves</p>\n"
        );
    }
    else if( strcmp(buf, "HTML_CLOSE") == 0 ){
      fprintf(fo_html, "</body>\n</html>\n");
      fclose(fo_html);
      printf("\n==> SEE %s\n\n", htmlfile);
    }
    else if( strcmp(buf, "BLOCH_VISUAL") == 0 ){
      fget_1s( infile, fi );
      fget_1s( ppfile, fi );
      pltout_pp( ppfile, infile, fo_html );
      branch_begin = fget_i(fi);
      branch_end = fget_i(fi);
      for(ib=branch_begin; ib<=branch_end; ib++){
        fget_1s( blfile, fi );
        ibranch = fget_i(fi); /* should be the same as ib */
        excitation = fget_d(fi); 
        pltout_bloch( blfile, infile, ibranch, excitation, fo_html );
      }
    }
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}


void job_sim_visual_mode(FILE *fi)
{
  /*
     Output hmtl file with shell scripts for SIM_RESULT
   */
  int    mag_coef, cont_rev_flag;
  double  gamma_coef, preset_max_val;
  char  buf[BUFSIZE]; /* tbuf[BUFSIZE]; */
  char  htmlfile[BUFSIZE];
  char  simfile[BUFSIZE], simfile2[BUFSIZE];
  FILE  *fo_html; 

  while( fget_1s(buf, fi) != NULL ){
    if( strcmp(buf, "HTML_OPEN") == 0 ){
      fget_1s(buf, fi); /* html file */
      strcpy(htmlfile, buf);
      fo_html = fopen_chk_wmode(htmlfile);
      output_html_header(fo_html, htmlfile);
    }
    else if( strcmp(buf, "HTML_CLOSE") == 0 ){
      fprintf(fo_html, "</body>\n</html>\n");
      fclose(fo_html);
      printf("\n==> SEE %s\n\n", htmlfile);
    }
    else if(
      strcmp(buf, "SIM_VISUAL") == 0 ||
      strcmp(buf, "VISUAL") == 0
      ){
      fget_str_and_dii(simfile, &gamma_coef, &mag_coef, &cont_rev_flag, fi);
      preset_max_val = -1.0;
      mb_sim_result_visualize(
        simfile, gamma_coef, mag_coef, cont_rev_flag, preset_max_val, fo_html
        );
    }
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}


void job_sim_visual_reflex_mode(FILE *fi)
{
  /*
     Output hmtl file with shell scripts for SIM_RESULT
   */
  int    mag_coef, cont_rev_flag;
  double  gamma_coef, preset_max_val;
  char  buf[BUFSIZE];  /* tbuf[BUFSIZE]; */
  char  htmlfile[BUFSIZE];
  char  simfile[BUFSIZE], simfile2[BUFSIZE];
  FILE  *fo_html;
  
  while( fget_1s(buf, fi) != NULL ){
    if( strcmp(buf, "HTML_OPEN") == 0 ){
      fget_1s(buf, fi);
      strcpy(htmlfile, buf);
      fo_html = fopen_chk_wmode(htmlfile);
      output_html_header(fo_html, htmlfile);
    }
    else if( strcmp(buf, "HTML_CLOSE") == 0 ){
      fprintf(fo_html, "</body>\n</html>\n");
      fclose(fo_html);
      printf("\n==> SEE %s\n\n", htmlfile);
    }
    else if(
      strcmp(buf, "SIM_VISUAL") == 0 ||
      strcmp(buf, "VISUAL") == 0
      ){
      fget_str_and_dii(simfile, &gamma_coef, &mag_coef, &cont_rev_flag, fi);
      preset_max_val = -1.0;
      mb_sim_result_reflex_visualize(
        simfile, gamma_coef, mag_coef, cont_rev_flag, preset_max_val, fo_html
        );   /* --> also output to fo_html! */
    }
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}


void output_gnuplot_file_ts_scan(
  char *pltfile, char *outfile,
  const char *xlabel, const char *ylabel, const char *title
  )
{
  FILE *fo;
  fo = fopen_chk_wmode(pltfile);

  fprintf(fo, "set xlabel \"%s\"\n", xlabel);
  fprintf(fo, "set ylabel \"%s\"\n", ylabel);
  fprintf(fo, "set title \"%s\"\n", title);
  fprintf(fo, "plot \"%s\" using 1:2 with linespoints\n", outfile);
  
  fclose(fo);
}


void job_ts_scan_mode(FILE *fi)
{
  int    ig, num_reflex_preset, num_reflex, *success_flag;
  double  *Hshift_opt, *Gshift_opt, *gof_reflex, *scale_opt;
  double  *Hshift_default, *Gshift_default, *g_weight;
  int  num_Hshift, num_Gshift;
  double  Hshift_min, Hshift_max, Gshift_min, Gshift_max;
  double  Hshift_SIM = 0.0, Gshift_SIM = 0.0;  /* <<<<< */
  int  shift_sim_flag = 0, num_reflex_preset_flag = 0;
  int  shiftscan_width_flag = 0, read_shift_weight_flag = 0;
  double  thickness, gof, scale;
  double  total_gof_shift, total_scale_factor_shift;
  HKL  *g_reflex;
  char  buf[BUFSIZE]; /*, tbuf[BUFSIZE];*/
  char  simfile[BUFSIZE], intfile[BUFSIZE], HGshift_wt_file[BUFSIZE];
  char  t_outfile[BUFSIZE], t_outfile2[BUFSIZE];
  char  t_pltfile[BUFSIZE], t_pltfile2[BUFSIZE];
  char  shift_outfile[BUFSIZE], shift_infile[BUFSIZE];
  FIT_EXP_INTSET  ei;
  SIM_RES  sr;
  FILE  *fo_t_out, *fo_t_out2;
  
  while( fget_1s(buf, fi) != NULL ){
    /*printf("$$buf=%s\n", buf);*/
    if(
      strcmp(buf, "TS_SCAN_OPEN") == 0
      ){
      /* Result of tscan with no shiftscan */
      fget_1s(t_outfile, fi);
      fprintf(stdout, "$$t_outfile=%s\n", t_outfile);
      fget_1s(t_pltfile, fi);
      fprintf(stdout, "$$pltfile=%s\n", t_pltfile);
      fo_t_out = fopen_chk_wmode(t_outfile);
      fprintf(fo_t_out, "# thickness  GOF  scale\n");

      /* Result of tscan with shiftscan */
      fget_1s(t_outfile2, fi);
      fprintf(stdout, "$$t_outfile2=%s\n", t_outfile2);
      fget_1s(t_pltfile2, fi);
      fprintf(stdout, "$$pltfile=%s\n", t_pltfile2);
      fo_t_out2 = fopen_chk_wmode(t_outfile2); 
      fprintf(fo_t_out2, "# thickness  GOF  scale\n");
    }
    else if(
      strcmp(buf, "TS_SCAN_NUM_REFLEX_PRESET") == 0
      ){
      num_reflex_preset = fget_i(fi);
      printf("$$num_reflex_preset = %d\n", num_reflex_preset);
      if(num_reflex_preset > 0 && num_reflex_preset_flag == 0){
        Hshift_default = new_vector(double, num_reflex_preset);
        chk_vector(Hshift_default);
        Gshift_default = new_vector(double, num_reflex_preset);
        chk_vector(Gshift_default);
        g_weight = new_vector(double, num_reflex_preset);
        chk_vector(g_weight);
        Hshift_opt = new_vector(double, num_reflex_preset);
        chk_vector(Hshift_opt);
        Gshift_opt = new_vector(double, num_reflex_preset);
        chk_vector(Gshift_opt);
        g_reflex = new_vector(HKL, num_reflex_preset);
        chk_vector(g_reflex);
        success_flag = new_vector(int, num_reflex_preset);
        chk_vector(success_flag);
        gof_reflex = new_vector(double, num_reflex_preset);
        chk_vector(gof_reflex);
        scale_opt =  new_vector(double, num_reflex_preset);
        chk_vector(scale_opt);
      }
      num_reflex_preset_flag = 1;
      for(ig=0; ig<num_reflex_preset; ig++){ /* default values */
        Hshift_default[ig] = Gshift_default[ig] = 0.0;
        g_weight[ig] = 1.0;
      }
    }
    else if(
      strcmp(buf, "TS_SCAN_INT_FILE") == 0
      ){
      fget_1s(intfile, fi);               /* .int file (experimental data) */
      fprintf(stdout, "$$intfile=%s\n", intfile);
      read_int_file(&ei, intfile);
    }
    else if(
      strcmp(buf, "TS_SCAN_WIDTH") == 0
      ){
      num_Hshift = fget_i(fi);
      fget_2d(&Hshift_min, &Hshift_max, fi);  /* scan width of H */
      num_Gshift = fget_i(fi);
      fget_2d(&Gshift_min, &Gshift_max, fi);  /* scan width of G */
      shiftscan_width_flag = 1;
    }
    else if(
      strcmp(buf, "TS_SCAN_HG_SHIFT_SIM") == 0
      ){
      fget_2d(&Hshift_SIM, &Gshift_SIM, fi);
      printf("$$ Hshift_SIM, Gshift_SIM = %lf, %lf\n", Hshift_SIM, Gshift_SIM );
      shift_sim_flag = 1;
    }
    else if(
      strcmp(buf, "DEFAULT_HG_SHIFT_WEIGHT_FILE") == 0
      ){
      fget_1s(HGshift_wt_file, fi);
      fprintf(stdout, "$$HGshift_wt_file=%s\n", HGshift_wt_file);
      if(num_reflex_preset_flag == 1){
        read_hg_shift_weight_file(
          num_reflex_preset, HGshift_wt_file,
          Hshift_default, Gshift_default, g_weight
          );
        read_shift_weight_flag = 1;
      }
      else{
        printf("$$ no TS_SCAN_NUM_REFLEX_PRESET.  Abort.\n");
        exit(EXIT_FAILURE);
      }
    }
    else if(
      strcmp(buf, "TS_SCAN_SIM_FILE") == 0
      ){
      fget_1s(simfile, fi);               /* SIM_RESULT file */
      fprintf(stdout, "$$simfile=%s\n", simfile);
      if(shift_sim_flag != 1){
        printf("$$ no TS_SCAN_HG_SHIFT_SIM.  Abort.\n");
        exit(EXIT_FAILURE);
      }
      if(shiftscan_width_flag != 1){
        printf("$$ no TS_SCAN_WIDTH.  Abort.\n");
        exit(EXIT_FAILURE);
      }
      fget_1s(shift_outfile, fi);             /* SIM_RESULT file */
      fprintf(stdout, "$$shift_outfile=%s\n", shift_outfile);

      /* ==> Caution: scan area check */

      read_sim_data(&sr, simfile, Hshift_SIM, Gshift_SIM);
      /* GoF with no shift */
      thickness = sr.thickness;
      num_reflex = sr.num_cal_g;
      if(num_reflex != num_reflex_preset){
        printf("$$ Disagreement between num_reflex(=%d)\n", num_reflex);
        printf("$$  and num_reflex_preset(=%d). Abort.\n", num_reflex_preset);
        exit(EXIT_FAILURE);
      }
      /* set_calint_to_ei( &ei, &sr ); */
      set_calint_to_ei_shift(
        &ei, &sr, num_reflex, Hshift_default, Gshift_default
        );
      scale = eval_scale_factor( &ei, num_reflex_preset, g_weight );
      gof = eval_gof( &ei, scale, num_reflex_preset, g_weight );
      fprintf(fo_t_out, "%lf  %lf  %lf\n", thickness, gof, scale);
      /* GoF with shift */
      if (num_Hshift > 1 && num_Gshift > 1){
        set_calint_to_ei_shiftscan(
          &ei, &sr,
          num_Hshift, Hshift_min, Hshift_max,
          num_Gshift, Gshift_min, Gshift_max,
          num_reflex, Hshift_opt, Gshift_opt,
          scale_opt, gof_reflex, success_flag
          );
        set_calint_to_ei_shift(
          &ei, &sr, num_reflex, Hshift_opt, Gshift_opt
          );
        total_scale_factor_shift = eval_scale_factor(
          &ei, num_reflex_preset, g_weight
          );
        total_gof_shift = eval_gof(
          &ei, total_scale_factor_shift, num_reflex_preset, g_weight
          );
        for(ig=0; ig<num_reflex; ig++){
          g_reflex[ig] = ei.obs_rg[ig];
        }
        shiftscan_res_output(
          shift_outfile, num_reflex, thickness, 
          Hshift_opt, Gshift_opt, g_reflex, gof_reflex, success_flag, 
          total_gof_shift, total_scale_factor_shift
          );
        fprintf(
          fo_t_out2, "%lf  %lf  %lf\n",
          thickness, total_gof_shift, total_scale_factor_shift
          );
      }
      free_sr(&sr);
    }
    else if(
      strcmp(buf, "TS_SCAN_SIM_FILE_FIXED_SHIFT") == 0
      ){
      fget_1s(simfile, fi);               /* SIM_RESULT file */
      fprintf(stdout, "$$simfile=%s\n", simfile);
      if(shift_sim_flag != 1){
        printf("$$ no TS_SCAN_HG_SHIFT_SIM.  Abort.\n");
        exit(EXIT_FAILURE);
      }
      fget_1s(shift_infile, fi);             /* SHIFT file */
      fprintf(stdout, "$$shift_infile=%s\n", shift_infile);

      read_sim_data(&sr, simfile, Hshift_SIM, Gshift_SIM);
      read_hg_shift_file(
          num_reflex_preset, shift_infile, Hshift_default, Gshift_default
          );
      thickness = sr.thickness;
      num_reflex = sr.num_cal_g;
      if(num_reflex != num_reflex_preset){
        printf("$$ Disagreement between num_reflex(=%d)\n", num_reflex);
        printf("$$  and num_reflex_preset(=%d). Abort.\n", num_reflex_preset);
        exit(EXIT_FAILURE);
      }
      set_calint_to_ei_shift(
        &ei, &sr, num_reflex, Hshift_default, Gshift_default
        );
      scale = eval_scale_factor( &ei, num_reflex_preset, g_weight );
      gof = eval_gof( &ei, scale, num_reflex_preset, g_weight );
      fprintf(fo_t_out, "%lf  %lf  %lf\n", thickness, gof, scale);
      free_sr(&sr);
    }
    else if(
      strcmp(buf, "TS_SCAN_CLOSE") == 0
      ){ 
      free_ei(&ei);
      fclose(fo_t_out);
      fclose(fo_t_out2);
      output_gnuplot_file_ts_scan(
        t_pltfile, t_outfile, "thickness [A]", "GOF", "TSCAN"
        );
      output_gnuplot_file_ts_scan(
        t_pltfile2, t_outfile2, "thickness [A]", "GOF", "TSCAN with SHIFT"
        );
      if(num_reflex > 0){
        free_vector(Hshift_opt);  free_vector(Gshift_opt);
        free_vector(g_reflex);  free_vector(success_flag);
        free_vector(gof_reflex);  free_vector(scale_opt);
        free_vector(Hshift_default);
        free_vector(Gshift_default);
        free_vector(g_weight);
      }
    }
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}


void job_fit_visual_mode_old_format(FILE *fi)
{
  /*
     Output hmtl file with shell scripts for FIT_RESULT
   */
  int     n, mag_coef, cont_rev_flag;
  int     vis_params_flag = 0, info_file_flag = 0;
  double  t[4], gamma_max, preset_max_val = -1.0;
  char  buf[BUFSIZE];
  char  resfile[BUFSIZE], infofile[BUFSIZE];

  while( fget_1s(buf, fi) != NULL ){
    if( strcmp(buf, "INFO_FILE") == 0 ){
      fget_1s(infofile, fi);
      printf("$$infofile= %s\n", infofile);
      info_file_flag = 1;
    }
    else if( strcmp(buf, "VIS_PARAMS") == 0 ){
      comment_skip_fgets(sizeof buf, buf, fi);
      n = str_to_fs(4, buf, t);
      if(n == 4){
        mag_coef = (int) t[0];  cont_rev_flag = (int) t[1];
        preset_max_val = t[2];  gamma_max = t[3];
      }
      else if(n == 2){
        mag_coef = (int) t[0];  cont_rev_flag = (int) t[1];
        preset_max_val = -1.0;  gamma_max = 1.0;
      }
      else{
        printf("$$ VIS_PARAMS: Invalid parameters. Abort.\n");
        exit(EXIT_FAILURE);
      }
      printf(
        "$$mag_coef, cont_rev_flag = %d %d\n",
        mag_coef, cont_rev_flag
        );
      printf(
        "$$preset_max_val, gamma_max = %lf %lf\n",
        preset_max_val, gamma_max
        );
      vis_params_flag = 1;
    }
    
    else if( strcmp(buf, "FIT_RES_FILE") == 0 ){
      fget_1s(resfile, fi);
      printf("$$resfile= %s\n", resfile);
      if(info_file_flag == 0){
        printf("no INFO_FILE. Abort.\n");
        exit(EXIT_FAILURE);
      }
      else if(vis_params_flag == 0){
        printf("no VIS_PARAMS. Abort.\n");
        exit(EXIT_FAILURE);
      }
      mb_fit_result_visualize(
        resfile, infofile, mag_coef, cont_rev_flag,
        preset_max_val, gamma_max
        );
    }
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}


#if 0
void job_sim_dif_mode(FILE *fi)
{
  int    mag_coef, cont_rev_flag;
  double  gamma_coef, preset_max_val;
  char  buf[BUFSIZE]; /*, tbuf[BUFSIZE]; */
  char  htmlfile[BUFSIZE], batfile[BUFSIZE], shfile[BUFSIZE];
  char  simfile[BUFSIZE], simfile2[BUFSIZE];
  FILE  *fo_html, *fo_bat, *fo_sh;
  
  while( fget_1s(buf, fi) != NULL ){
    /*
      Difference of two SIM_RESULT files
      */
    if( strcmp(buf, "SIM_DIF") == 0 ){
      fget_2s(simfile, simfile2, fi);
      preset_max_val = -1.0;
      mb_sim_result_visualize(
        simfile, gamma_coef, mag_coef, cont_rev_flag, preset_max_val
        );
    }
    /*
      FIT_RESULT
     */
    else if( strcmp(job_identifier, "RWF") == 0){
      mb_rfactor();
    }
    else if( strcmp(job_identifier, "HOLZ") == 0 ){
      mb_HOLZ_lines();
    }
    else if( strcmp(job_identifier, "DFAC") == 0){
      mb_dfactor();
    }
    else if( strcmp(job_identifier, "THINOUT") == 0 ){
      mb_thinout();
    }
    else if( strcmp(infile, "VISDIF") == 0 ){
      mb_visualization_diff(
        command_line_visual = 1, vis_file1, vis_file2,
        gamma_coef, mag_coef,
        cont_rev_flag, preset_max_val
        );
      mb_diff_r_factor( vis_file1, vis_file2 );
    }
    else if( strcmp(job_identifier, "TTEST") == 0){
      printf("***** tmp_test ***** \n");
      tmp_test();
    }
    /*
      Command error
      */
    else{
      printf("$$ Invalid command: %s\n", buf);
      exit(EXIT_FAILURE);
    }
  }
}
#endif


void mbutl_jobs(char *job_file)
{
  char buf[BUFSIZE];
  int  flag;
  FILE *fi;

  fi = fopen_chk_rmode(job_file);

  fget_1s(buf, fi);
  if( strcmp(buf, "MBUTL_SCRIPT") == 0 ){
    printf("\n##### mbutl_script #####\n\n");
  }
  else{
    printf("$$(mbutl) Invalid script file. Abort.\n");
    printf("$$buf=%s\n", buf);
    exit(EXIT_FAILURE);
  }

  /* Job identification */
  fget_1s(buf, fi);
  if( strcmp(buf, "SIM_VISUAL_MODE") == 0 ){
    printf(" 1: entire patterns / 2: individual reflections \n");
    printf(" Input number>>");
    flag = fget_i(stdin);
    if(flag == 1){
      job_sim_visual_mode(fi);
    }
    else if(flag == 2){
      job_sim_visual_reflex_mode(fi);
    }
    else{
      printf("$$(mbutl) Invalid number. Abort.\n");
      printf("$$(flag=%d)\n", flag);  exit(EXIT_FAILURE);
    }
  }
  else if( strcmp(buf, "TS_SCAN_MODE") == 0 ){
    job_ts_scan_mode(fi);
  }
  else if( strcmp(buf, "DISPSURF_VISUAL_MODE") == 0 ){
    job_dispsurf_visual_mode(fi);  /* 040618 */
  }
  else if( strcmp(buf, "BLOCH_VISUAL_MODE") == 0 ){
    job_bloch_visual_mode(fi);  /* 040621 */
  }
  else if( strcmp(buf, "FIT_VISUAL_MODE_MBL") == 0 ){
    job_fit_visual_mode_old_format(fi);
  }
  else if( strcmp(buf, "FIT_VISUAL_MODE") == 0 ){
    job_fit_visual_mode_old_format(fi);  /* tentatively */
  }
  /*
  else if( strcmp(buf, "SIM_DIF_MODE") == 0 ){
    job_sim_dif_mode(fi);
  }
   */
  else{
    printf("$$(mbutl) Invalid script file. Abort.\n");
    printf("$$(buf=%s)\n", buf);
    exit(EXIT_FAILURE);
  }
  fclose(fi);
}


int main(int argc, char *argv[])
{
  char job_file[BUFSIZE];
  
  opening_title();
  printf("***** Ver. 0.2h1 updated on 21 Jun. 2004\n\n");
  
  if (argc == 1){  /* console mode */
    printf("Script-file:\n");
    printf("  (\"interactive\" for the interactive input mode)\n");
    printf(">>");
    fget_1s(job_file, stdin);
  }
  else if(argc == 2){  /* command-line mode */
    strcpy(job_file, argv[1]);
  }
  else{
    fprintf(stdout, "$$ (mbutl) Invalid input.\n");
    fprintf(stdout, "$$ usage: mbutl script_file.\n");
    exit(EXIT_FAILURE);
  }
  
  if (
    strcmp(job_file, "interactive") == 0 ||
    strcmp(job_file, "INTERACTIVE") == 0
    ){
    mbutl_interactive();
  }
  else{
    mbutl_jobs(job_file);
  }
  /* printf("\nThank you. See you again!\n"); */
 return 0;
}
