#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "complex.h"
#include "cmatutil.h"
#include "eig_mat.h"
#include "cg.h"
#include "inv_mat.h"
#include "perturb.h"
#include "vecmat.h"
#include "mbconfig.h"

/* extern DEBUG_INFO dbg; */
extern EPS_CTRL	epsilon;

#define  sq(x)	((x)*(x))

void dbg_mon(char *str);

void c_dagger2(int n, complex **a);
void dbg_mon(char *str);
void print_complexmat(int n, complex **a);


void c_dagger2(int n, complex **a)
{
	int i,j;

	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			a[i][j] = c_conj(a[j][i]);
		}
	}
}


void perturb(
  complex *ec, complex **vec, int n, complex **da,
  double edeg, double eps, complex *p_ec, complex **p_vec
  )
{
  /*
   * Perturbation (for degenerate and nondegenerate)
   *  input >	ec:eigen value,
   *			vec:eigen vector,
   *			n, da
   *  output>	p_ec:perturbed eigen value,
   *			p_vec:perturbed eigen vector
   * {da} = {perturbed_matrix} - {matrix}
   */
  complex **wk, **wk2, **db, *ew;
  complex dtmp2;
  int *iw, *kw, *iw2;
  int i, j, k, ndegen, err;
  
  wk	= new_matrix(complex,n,n);	chk_matrix(wk);
  wk2	= new_matrix(complex,n,n);	chk_matrix(wk2);
  db	= new_matrix(complex,n,n);	chk_matrix(db);
  ew	= new_vector(complex,n);	chk_vector(ew);
  iw	= new_vector(int,n);		chk_vector(iw);
  kw	= new_vector(int,n);		chk_vector(kw);
  iw2	= new_vector(int,n);		chk_vector(iw2);
  
	/*
	 * b_ij --> db(i,j)
	 */
	c_matcopy2(n, da, wk);
	c_matmult2(n, wk, vec);
	c_matcopy2(n, vec, wk2);
	if ( (err = inv_c_mat(n, epsilon.inv, wk2)) != 0 ){
		printf("$$ error (perturb): inv_c_mat = %d.\n", err);
	}
	c_matmult2(n, wk2, wk);
	c_matcopy2(n, wk2, db);		/* db */

	/*
	 * Check degeneracy
	 *  edeg : eps of degeneracy
	 */
	for(i=0; i<n; i++){ iw[i] = 0; }
	for(i=0; i<n; i++){
		ndegen = 1;	/* ndegen : order of degeneracy */
		if( iw[i]==0 ){
			kw[0] = i;		/* <<<<< */
			for(j=i+1; j<n; j++){
				if( fabs(ec[i].re - ec[j].re) < edeg ){
					ndegen++;
					iw[i] = 1;	iw[j] = 1;
           			kw[ndegen] = j;
          		}
			}
			/*
			 * diagonalization
			 */
			if ( ndegen > 1 ){	/* <<<<< */
 				for(k=0; k<ndegen; k++){
 					for(j=0; j<ndegen; j++){
						wk[k][j] = db[kw[k]][kw[j]];
 					}
 				}
				for(j=0; j<ndegen; j++){
					wk[j][j] = c_add( ec[kw[j]], db[kw[j]][kw[j]] );
				}

				/* if( (err = eigen_c_mat(n, epsilon.eigen, ind = 1, wk, ew, wk2 )) !=0){
					printf("$$ (perturb): eigen_c_mat = %d \n", err);
				}*/
				eispack_cg( n, epsilon.eigen, wk, ew, wk2 );

				for(k=0; k<ndegen; k++){
 					for(j=0; j<ndegen; j++){
						wk[kw[k]][kw[j]] = wk2[k][j];
 					}
 				}
				for(j=0; j<ndegen; j++){ p_ec[kw[j]] = ew[j]; }
			}
		}
	}

	/*
	 * p_ec (from lambda)
	 */
	for(i=0; i<n; i++){
		if ( iw[i] == 1 ){	/* degenerate */
			dtmp2 = c_conv(0.0,0.0);
			for(j=0; j<n; j++){
				if(j != i) {
					if( iw[j] == 1
				  	&& fabs( ec[i].re - ec[j].re ) < edeg ){
				  	;}
				  	else {
						dtmp2 = c_add(
								dtmp2,
								c_div( 
									c_mul( db[i][j], c_conj(db[i][j]) ),
									c_sub( ec[i], ec[j] )
								)
							);
					}
				}
			}
			p_ec[i] = c_add( p_ec[i],  dtmp2 );
			/*printf("DEG:ec=%lf %lf, p_ec=%lf %lf\n",
				ec[i].re,ec[i].im, p_ec[i].re, p_ec[i].im);*/
		}
		else{				/* nondegenerate */
          dtmp2 = c_conv(0.0,0.0);
			for(j=0; j<n; j++){
				if(j != i){
					dtmp2 = c_add(
								dtmp2,
								c_div( 
									c_mul( db[i][j], c_conj(db[i][j]) ),
									c_sub( ec[i], ec[j] )
								)
							);
				}
			}
			p_ec[i] = c_add( ec[i], c_add( db[i][i], dtmp2 ) );
			/*printf("ec=%lf %lf, p_ec=%lf %lf, db=%lf %lf\n",
				ec[i].re,ec[i].im, p_ec[i].re, p_ec[i].im,
				db[i][i].re, db[i][i].im);*/
		}
	}

	/*
	 * p_vec (from epsiron)
	 */
	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			if ( i!=j && iw[j]==1 
			&& fabs( ec[i].re - ec[j].re ) < edeg ){
				wk[i][j] = c_conv(0.0, 0.0);
			}
			else if ( i == j ){	/* nondegenerate diagonal terms */
				wk[i][j] = c_conv(1.0,0.0);
			}
			else{				/* nondegenerate off-diagonal terms */
				wk[i][j] = c_div( db[i][j], c_sub( ec[j], ec[i] ) );	/* <<<<< */
			}
		}
	}

	c_matcopy2(n, vec, p_vec);
	c_matmult2(n, p_vec, wk);		/* p_vec */
	free_matrix(wk);	free_matrix(wk2);
	free_matrix(db);	free_vector(ew);
	free_vector(iw);	free_vector(kw);	free_vector(iw2);
}


void perturbation(
	complex *eigen_val, complex **eigen_vec, complex **eigen_vec_inv,
	int n, complex **amat, complex **perturebed_amat,
	double eps_degeneracy, double eps,
	complex *perturbed_eigen_val, complex **perturbed_eigen_vec
)
/*
 * Perturbation (for degenerate and nondegenerate)
 *  input >	ec:eigen value,
 *			vec:eigen vector,
 *			n, da
 *  output>	p_ec:perturbed eigen value,
 *			p_vec:perturbed eigen vector
 */
{
	complex *eg_val, **eg_vec, **wa, *weval, **wevec, **diff_amat, **b;
	int i, j, k, n_degen;

	diff_amat = new_matrix(complex,n,n);	chk_matrix(diff_amat);
	wa		= new_matrix(complex,n,n);		chk_matrix(wa);
	wevec	= new_matrix(complex,n,n);		chk_matrix(wevec);
	b		= new_matrix(complex,n,n);		chk_matrix(b);
	weval	= new_vector(complex,n);		chk_vector(weval);
	eg_vec	= new_matrix(complex,n,n);		chk_matrix(eg_vec);
	eg_val	= new_vector(complex,n);		chk_vector(eg_val);

	for(i=0; i<n; i++){ eg_val[i] = eigen_val[i]; }		/* copy to eg_val */
	c_matcopy2(n, eigen_vec, eg_vec);					/* copy to eg_vec */

	/* Check descending order of eigen_val */
	/* ----- */

	/* difference between perturebed_amat and amat */
	printf("$$ diff_mat\n");
	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			diff_amat[i][j] = c_sub(perturebed_amat[i][j], amat[i][j]);
			/*
			printf("$$ diff_amat[%d][%d] = %lf %lf\n",i,j,diff_amat[i][j].re, diff_amat[i][j].im);
			printf("$$ perturebed_amat[%d][%d] = %lf %lf\n",i,j,perturebed_amat[i][j].re, perturebed_amat[i][j].im);
			printf("$$ amat[%d][%d] = %lf %lf\n",i,j,amat[i][j].re, amat[i][j].im);
			*/
		}
	}
	/* calc. of b_ij */
	printf("$$ b_ij\n");
	c_matmult2(n, diff_amat, eigen_vec);
	/*
	c_matcopy2(n, eigen_vec, b);
	if ( (err = inv_c_mat(n, eps, b)) != 0 ){
		printf("$$ error (perturb): inv_c_mat = %d.\n", err);
	}
	*/
	printf("$$ b_ij2\n");
	c_matcopy2(n, eigen_vec_inv, b);
	printf("$$ b_ij3\n");
	c_matmult2(n, b, diff_amat);	/* --> b */
	/* c_matmult(n, n, n, eigen_vec_inv, diff_amat, b); */
	/* print_complexmat( n, b ); */

	/*
	 * Search degenerated eigen values and remove degeneracies
	 *  by block diagonalization
	 */
	printf("$$ search degen and block diag.\n");
	i = 0;
	while (i < n){
		j = i + 1;	n_degen = 0;
		while (j < n){
			if( fabs(eg_val[i].re - eg_val[j].re) < eps_degeneracy ){	/* degenerate */
				n_degen++;
				printf("$$ degeneracy found %d\n",n_degen);
			}
			j++;
		}
		if( n_degen > 0 ){
			/* block diagonalization */
			for(j=0; j<n_degen; j++) {
				for(k=0; k<n_degen; k++){
					wa[j][k] = b[j + i][k + i];
				}
			}
			for(j=0; j<n_degen; j++){
				wa[j][j] = b[j + i][j + i];
			}	/* wa in the previous line is to be diagonalized. */

			/*if( (err = eigen_c_mat(n_degen, eps, ind = 1, wa, weval, wevec )) !=0){
				printf("$$ (perturb): eigen_c_mat = %d \n", err);
			}*/
			eispack_cg( n_degen, eps, wa, weval, wevec );	/* <<<<< */

			c_matunit(n, wa);	/* wa --> S */
			for(j=0; j<n_degen; j++) {
				for(k=0; k<n_degen; k++){
					wa[j + i][k + i] = wevec[i][k];
					b [j + i][k + i] = c_conv(0.0, 0.0);
				}
			}
			for(j=0; j<n_degen; j++){
				b[j + i][j + i] = weval[j];
				eg_val[j] = c_add(b[j + i][j + i], eigen_val[j]);
			}
			c_matmult2(n, eg_vec, wa);
		}
		i += n_degen + 1;
	}

	/*
	 * perturbed eigen values (from gamma_ij)
	 *  (1st and 2nd order)
	 */
	printf("$$ perturb eigen_val\n");
	for(i=0; i<n; i++){
		perturbed_eigen_val[i] = c_conv(0.0, 0.0);
		for(j=0; j<n; j++){
			if(j == i) {
				perturbed_eigen_val[i]
						 = c_add( perturbed_eigen_val[i], c_add(b[i][i], eg_val[j]) );
			}
			else{
				if( fabs( eg_val[i].re - eg_val[j].re ) < eps_degeneracy ){
				/* degeneracy remains ==> ignore */ ;
				}
				else{	/* nondegenerate */
					perturbed_eigen_val[i]
						 = c_add(
								perturbed_eigen_val[i],
								c_div( 
									c_mul( b[i][j], c_conj(b[i][j]) ),
									c_sub( eg_val[i], eg_val[j] )
								)
						   );
				}
			}
		}
	}

	/*
	 * perturbed eigen vectors (from epsiron_ij)
	 *  (1st order)
	 *   where wa means T.
	 */
	printf("$$ perturb eigen vec.\n");
	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			if (i == j){				/* diagonal terms */
				wa[i][j] = c_conv(1.0,0.0);	/* unperturbed */
			}
			else{
				if( fabs( eg_val[i].re - eg_val[j].re ) < eps_degeneracy ){
				/* degeneracy remains ==> ignore */
					wa[i][j] = c_conv(0.0, 0.0);
				}
				else{				/* nondegenerate off-diagonal terms */
					wa[i][j] = c_div( b[i][j], c_sub( eg_val[j], eg_val[i] ) );
				}
			}
		}
	}
	c_matcopy2(n, eg_vec, perturbed_eigen_vec);
	c_matmult2(n, perturbed_eigen_vec, wa);		/* --> perturbed_eigen_vec */

	printf("$$ perturb end\n");
	free_matrix(diff_amat);
	free_matrix(wa);		free_matrix(wevec);
	free_matrix(b);			free_vector(weval);
	free_matrix(eg_vec);	free_vector(eg_val);
}


void perturbation2(
	complex *eigen_val, complex **eigen_vec, complex **eigen_vec_inv,
	int n, complex **amat, complex **perturbed_amat,
	double eps_degeneracy, double eps,
	complex *perturbed_eigen_val, complex **perturbed_eigen_vec
)
/*
 * Perturbation (for degenerate and nondegenerate)
 *  without block diagonalization ( ==> ignored )
 */
{
	complex **wd, **b;
	int i, j;

	wd		= new_matrix(complex, n, n);		chk_matrix(wd);
	b		= new_matrix(complex, n, n);		chk_matrix(b);

	/* difference between perturebed_amat and amat */
	dbg_mon("$$ p-diff-mat\n");
/*	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			wd[i][j] = c_sub(perturbed_amat[i][j], amat[i][j]);
		}
	} */
	c_matcopy2(n, perturbed_amat, wd);
	c_matsub2(n, wd, amat);
	

	/* calc. of b_ij --> b */
/*	dbg_mon("$$ p-b_ij - 1\n");
	c_matmult2(n, wd, eigen_vec);
	c_matmult(n, n, n, b, eigen_vec_inv, wd);*/

	dbg_mon("$$ p-b_ij\n");
	c_matcopy2(n, eigen_vec_inv, b);
	c_matmult2(n, b, wd);
	c_matmult2(n, b, eigen_vec);

	/* perturbed eigen values  */
	dbg_mon("$$ p-eigen_val\n");
	for(i=0; i<n; i++){		/* 1st order */
		perturbed_eigen_val[i] = c_add(b[i][i], eigen_val[i]); 
	}

#if 0
	for(i=0; i<n; i++){
		perturbed_eigen_val[i] = c_conv(0.0, 0.0);
		for(j=0; j<n; j++){
			if(j == i) {	/* 1st order */
				perturbed_eigen_val[i]
						 = c_add( perturbed_eigen_val[i], c_add(b[i][i], eigen_val[j]) );
			}
			else{	/* 2nd order */
				if( fabs( eigen_val[i].re - eigen_val[j].re ) < eps_degeneracy ){
						/* degenerated ==> ignore */ ;
				}
				else{	/* nondegenerate */
					perturbed_eigen_val[i]
						 = c_add(
								perturbed_eigen_val[i],
								c_div( 
									c_mul( b[i][j], c_conj(b[i][j]) ),
									c_sub( eigen_val[i], eigen_val[j] )
								)
						   );
				}
			}
		}
	}
#endif

	/*
	 * perturbed eigen vectors (from epsiron_ij)
	 *  (1st order),  where tmp matrix wd means T.
	 */
	dbg_mon("$$ p-eigen_vec\n");
	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			if (i == j){				/* diagonal terms */
				wd[i][j] = c_conv(1.0,0.0);	/* unperturbed */
			}
			else{
				if( fabs( eigen_val[i].re - eigen_val[j].re ) < eps_degeneracy ){
				/* degeneracy remains ==> ignore */
					wd[i][j] = c_conv(0.0, 0.0);
				}
				else{				/* nondegenerate off-diagonal terms */
					wd[i][j] = c_div( b[i][j], c_sub( eigen_val[j], eigen_val[i] ) );
				}
			}
		}
	}
	/* --> perturbed_eigen_vec */
	/* c_matmult(n, n, n, perturbed_eigen_vec, eigen_vec, wd);*/
	c_matcopy2(n, eigen_vec, perturbed_eigen_vec);
	c_matmult2(n, perturbed_eigen_vec, wd);

	free_matrix(wd);		free_matrix(b);
}


void eigen_val_perturbation(
	complex *eigen_val, complex **eigen_vec, complex **eigen_vec_inv,
	int n, complex **amat, complex **perturebed_amat, complex *perturbed_eigen_val
)
/*
 * Perturbation (not only for nondegenerate but also degenerate)
 *  of eigen values
 */
{
	complex **diff_amat, *b_diagonal_elm;
	int i, j;

	diff_amat	= new_matrix(complex,n,n);		chk_matrix(diff_amat);
	b_diagonal_elm	= new_vector(complex,n);	chk_vector(b_diagonal_elm);

	/* difference between perturebed_amat and amat */
	/* printf("$$ diff_mat\n"); */
	for(i=0; i<n; i++){
		for(j=0; j<n; j++){
			diff_amat[i][j] = c_sub(perturebed_amat[i][j], amat[i][j]);
		}
	}

	/* calc. of b_ij */
	/* printf("$$ b_ij\n"); */
	c_matmult2(n, diff_amat, eigen_vec);
	/* printf("$$ b_ij2\n"); */
	for(i=0; i<n; i++){
		b_diagonal_elm[i] = c_conv(0.0, 0.0);
		for(j=0; j<n; j++){
			b_diagonal_elm[i] =
				c_add(
					b_diagonal_elm[i],
					c_mul(
						eigen_vec_inv[i][j], diff_amat[j][i]
					)
				);
		}
	}

	/*
	 * perturbed eigen values (from gamma_ij)
	 *  (1st order)
	 */

	/* printf("$$ perturb eigen_val\n"); */
	for(i=0; i<n; i++){
		perturbed_eigen_val[i] = c_add( b_diagonal_elm[i], eigen_val[i] );
		/*
		printf(
			"$$ %d: eigen_val_org =%lf,%lf  p = %lf,%lf\n",
			i, eigen_val[i].re, eigen_val[i].im, perturbed_eigen_val[i].re, perturbed_eigen_val[i].im
		);*/
	}

	/* printf("$$ perturb end\n"); */
	free_matrix(diff_amat);
	free_vector(b_diagonal_elm);
}


#if 0
void perturbation_limited(
	complex *eigen_val, complex **eigen_vec, int n,
	complex **amat, complex **perturebed_amat,
	double eps_degeneracy, double eps,
	complex *perturbed_eigen_val, complex **perturbed_eigen_vec
)

#endif
