/*
 * vecmat.c
 */
#include <stdio.h>
#include <stdlib.h>

#define new_vector(tp, n)  ((tp *) malloc( (n) * sizeof(tp) ))
#define new_matrix(tp,n1,n0)  ((tp **) mmalloc( (n1), (n0) * sizeof(tp) ))
#define new_tensor(tp,n2,n1,n0) \
  ((tp ***) mmmalloc( (n2), (n1), (n0) * sizeof(tp) ))
#define new_tensor4(tp,n3,n2,n1,n0) \
  ((tp ****) mmmmalloc( (n3), (n2), (n1), (n0) * sizeof(tp) ))
#define free_vector(x)  free(x)


void **mmalloc( size_t n0, size_t n1 )
{
  void **x; char *y;
  unsigned int i;

  x =  (void **)malloc(n0 * sizeof(void *));
  if (x==NULL) {return NULL;}
  y = (char *)malloc(n0 * n1 * sizeof(char));
  if (y==NULL) {free(x); return NULL;}
  for(i=0; i<n0; i++) {
    x[i] = (void *)(y);
    y += n1;
  }
  return x;
}


void free_matrix(void **x)
{
  if(x==NULL) return;
  free(x[0]);
  free((void *)x);
}


void ***mmmalloc(size_t n0, size_t n1, size_t n2)
{
  void ***x; void **y; char *z;
  unsigned int i, j;

  x =  (void ***)malloc(n0 * sizeof(void **));
  if (x==NULL) return NULL;
  y =  (void **)malloc(n0 * n1 * sizeof(void *));
  if (y==NULL) {
    free(x);
    return NULL;
  }
  z =  (char *)malloc(n0 * n1 * n2 * sizeof(char));
  if (z==NULL) {
    free(x); free(y);
    return NULL;
  }
  for(i=0; i<n0; i++) {
    x[i] = (void **)(y);
    y += n1;
    for(j=0; j<n1; j++) {
      x[i][j] = (void *)z;
      z += n2;
    }
  }
  return x;
}


void free_tensor(void ***x)
{
  if (x==NULL) return;
  free( (void *)x[0][0] );
  free( (void *)x[0] );
  free( (void *)x );
}


void ****mmmmalloc(size_t n0, size_t n1, size_t n2, size_t n3)
{
  void ****w; void ***x; void **y; char *z;
  unsigned int i, j, k;

  w =  (void ****)malloc(n0 * sizeof(void ***));
  if (w==NULL) return NULL;
  x =  (void ***)malloc(n0 * n1 * sizeof(void **));
  if (x==NULL) {
    free(w);
    return NULL;
  }
  y =  (void * *)malloc(n0 * n1 * n2 * sizeof(void * ));
  if (y==NULL) {
    free(w); free(x);
    return NULL;
  }
  z =  (char *)malloc(n0 * n1 * n2 * n3 * sizeof(char));
  if (z==NULL) {
    free(w); free(x); free(y);
    return NULL;
  }
  for(i=0; i<n0; i++) {
    w[i] = (void ***)(x);
    x += n1;
    for(j=0; j<n1; j++) {
      w[i][j] = (void **)y;
      y += n2;
      for(k=0; k<n2; k++) {
        w[i][j][k] = (void *)z;
        z += n3;
      }
    }
  }
  return w;
}


void free_tensor4(void ****x)
{
  if (x==NULL) return;
  free( (void *)x[0][0][0] );
  free( (void *)x[0][0] );
  free( (void *)x[0] );
  free( (void *)x );
}


void chk_vector( void *x )
{
  if (x==NULL){
    fprintf(stderr, "Not enough memory for vector.\n");
    exit(EXIT_FAILURE);
  }
}


void chk_matrix( void **x )
{
  if (x==NULL){
    fprintf(stderr, "Not enough memory for matrix.\n");
    exit(EXIT_FAILURE);
  }
}


void chk_tensor( void ***x )
{
  if (x==NULL){
    fprintf(stderr, "Not enough memory for tensor.\n");
    exit(EXIT_FAILURE);
  }
}


void chk_tensor4( void ****x )
{
  if (x==NULL){
    fprintf(stderr, "Not enough memory for tensor4.\n");
    exit(EXIT_FAILURE);
  }
}
