// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "ProjCellRecord.h"
#include "PhasegratingRecord.h"
#include "storage.h"
#include "funcdefs.h"

void GetNM(long *width,long *height) ;
void GetOutputImageDimensions(long *width,long *height) ;

void GetOutputImageDimensions(long *width,long *height) {
	long samplingWidth,samplingHeight ;
	float dx,dy ;
	
	GetNM(&samplingWidth,&samplingHeight) ;
	dx = gProjCell->a/samplingWidth ;
	dy = gProjCell->b*gProjCell->siga/samplingHeight ;	// sampling interval in y-dir.
	if (dy < dx) {
		samplingHeight = gProjCell->b*gProjCell->siga/dx ;
	}
	else {
		samplingWidth = gProjCell->a/dy ;
	}
	*width = samplingWidth ;
	*height = samplingHeight ;

}

void GetNM(long *width,long *height) {
	long ierr ;
	if (gPhsgrtRec->nbeam == 0) {
		SetProjectedCell(&ierr) ;
		SetCalculationDimensions(&gProjCell->a,&gProjCell->b,
			&gPhsgrtRec->gmax,&gPhsgrtRec->nbeam,&gPhsgrtRec->mbeam) ;
	}
	*width = gPhsgrtRec->nbeam ;
	*height = gPhsgrtRec->mbeam ;
}

// ----------------------------------------------------------------------- 
//  THIS ROUTINE ADJUSTS THE NUMBER OF SAMPLING POINTS TO BE THE SAME 
//  IN THE X AND Y DIRECTION. IT IS BEING APPLIED AFTER THE VIEWING AREA IF
//  NECCESARY HAS BEEN MADE RECTANGULAR					 
// ------------------------------------------------------------------------
void AdjustSampling(long *np,long *mp,long nx,long ny) {
	float *rp1,*rp2 ,dx,dy,dxnew,dynew,aaxis,baxis,rlo,xpos,ypos,fact ;
	register long j,n,m ;
	long jmax,mlo,nlo,npnew,mpnew,jlo,jhi ;

	jmax = *np*(*mp) ;
	// Store the array into temporary storage
	rp1 = gWorkSpace ;
	rp2 = rp1+jmax ;  	
	memcpy((char *)rp2,(char *)rp1,sizeof(float)*jmax) ;
	
	aaxis = nx*gProjCell->a ;
	baxis = ny*gProjCell->b*gProjCell->siga ;		// axis of interest is the upright 
	dx = aaxis/(*np) ;						// sampling interval in x-dir.
	dy = baxis/(*mp) ;						// sampling interval in y-dir.
	if (dx == dy) return ;					// No adjustment necessary

	rp2 = gWorkSpace + jmax ;				// Position at temp. workspace
	if (dy < dx) {
		dynew = dx ;
		mpnew = baxis/dynew ;
		for (m=0,rp1=gWorkSpace;m<mpnew;m++) {
			ypos = m*dynew ;
			rlo  = ypos/dy ;
			mlo  = (long) rlo ;
			fact = rlo - mlo ;
			for (n=0;n< *np;n++) {
				jlo = mlo*(*np) + n ;
				jhi = jlo + *np ;
				j   = m*(*np) + n ;
				*rp1++ = (1.-fact)*(*(rp2+jlo)) + 
						fact*(*(rp2+jhi)) ;
			}
		}
		*mp = mpnew ;
	}
	else {
		dxnew = dy ;
		npnew = aaxis/dxnew ;
		for (n=0;n<npnew;n++) {
			xpos = n*dxnew ;
			rlo = xpos/dx ;
			nlo = (long)rlo ;
			fact = rlo - nlo ;
			for (m=0;m< *mp;m++) {
				jlo = m*(*np) + nlo ;
				jhi = jlo + 1 ;
				j = m*npnew + n ;
				gWorkSpace[j] = (1.-fact)*(*(rp2+jlo)) + 
							fact*(*(rp2+jhi)) ;
			}
		}
		*np = npnew ;
	}
}
