// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include <stdio.h>
#include <math.h>
#include "funcdefs.h"
#include "files.h"
#include "header.h"
#include "storage.h"
#include "PhasegratingRecord.h"
#include "ProjCellRecord.h"
#include "MslicRecord.h"
#include "layers.h"
#include "errors.h"
extern 	long 	LAYERED ;
extern 	FILE 	*fp_print ;
extern 	char 	stext[] ; 
extern 	short	gSliceThreeD ;
extern	short	gSubMeanPhase ;
#define fabs(x)	((x) < 0. ? -(x) : (x)) 
extern 	struct 	{ float wavel ; long nslice ; float sigma ; } gMSRec ;
/* ---------------------------------------------------------------- */
/* All the work in the multislice part of the calculation takes	*/
/* place in this routine. Thus any speed improvement, etc, should	*/
/* concetrate on this routine and the routines called by this one	*/
/* For the potential calculation, the equivalent routine is 'fcoeff'*/
/* and for the image calculation, the equivalent is 'lens'		*/
/* There are some print statements in this routine that mainly are  */
/* checks on the numbers coming out right.				*/
/* ----------------------------------------------------------------	*/
void CalculatePrecWavefunction(float *zphi,float *zphsgr,float *zprop,long jmax) {
    register 	long i,j ;
    long 	npgold,nt,nphsgr,is,num_thick,num_elements ;
    float 	old_slcth,thickness,tout,rt,t1pr,t2pr,deltatpr ;
    register 	float *rp1 ;
    char	filename[256] ;

    num_elements = 2*jmax ;

    /*--------------- Initialize the wavefunction ------------------- */
    for(i=0,rp1=zphi;i<num_elements;i++) /* Normalize to 1 everywhere */
		{                 /* Normalizing such that SUM( PSI sq)= 1   */
		*rp1++ = 1. ;     /* introduces an undesirable dependence on */
		*rp1++ = 0. ;     /* the number of sampling points           */
		} 

    /* Some kind of initial. routine may work here in the future.  */
    /* init_prop(zprop)    ! Initialize the propagator,            */
    /*                     ! but do not apply the slicethickness   */

    npgold = 0 ;
    old_slcth = 0. ;
    thickness = 0. ;
    if (LAYERED) {
		gMSRec.nslice = gLayerRec.num_slc ;
		tout = gMsliceRec.thick ;
    }
    else {
    	gMSRec.nslice = (long)(gMsliceRec.thick/gPhsgrtRec->slcth+0.5) ;
    	tout = gMSRec.nslice*gPhsgrtRec->slcth ;
    }
    if (gMsliceRec.tts) {
    	num_thick = (gMsliceRec.t2-gMsliceRec.t1)/gMsliceRec.dt + 1 ;
    	/* Find the beginning and ending thickness */
    	rt = gMsliceRec.t1/gPhsgrtRec->slcth ;
    	nt = rt ;
    	if (rt-nt >= 0.5) nt = nt++ ;
    	t1pr = nt*gPhsgrtRec->slcth ; /*  Beginning output thickness based */
    	/*  upon the  unit cell dimensions   */
    	rt = gMsliceRec.t2/gPhsgrtRec->slcth ;
    	nt = rt ;
    	if (rt-nt >= 0.5) nt = nt++ ;
    	t2pr = nt*gPhsgrtRec->slcth ;	/*  Ending output thickness based    */
				/*  upon the unit cell dimensions    */
    	deltatpr = (t2pr-t1pr)/(num_thick-1) ;
    	gMSRec.nslice = (long)(t2pr/gPhsgrtRec->slcth + 0.5) ;
    	tout = t1pr ;
    }
    erase_message() ;
	/* Set the tilts, still to be done */
	for(itheta=0,itheta<NPrec; itheta++) {
	theta = 2*acos(-1.)*itheta/NPrec ;
	gMsliceRec.kxin = RPrec*cos(theta) ;
	gMsliceRec.kyin = RPrec*sin(theta) ;

	for(i=0;i<gMSRec.nslice;i++) {
		sprintf(stext,"STATUS: Exitwave Calculating Slice # %d of %d",i+1,gMSRec.nslice) ;
		write_message(stext) ;

		if (abandn()) {
			erase_message() ;
			InitCursor() ;
			return ;
		}
		if (LAYERED) {	  /*  Layered strucure. Phasegrating and slice */
			/*  thickness (propagator) may change.       */
			nphsgr = gLayerRec.phgr_seq[i] ;
			gPhsgrtRec->slcth = gLayerRec.slcth[gLayerRec.phgr_seq[i]-1] ;
			if (nphsgr != npgold)  {
				strcpy(filename,gLayerRec.pg[nphsgr-1]) ;
				strcat(filename,".pout") ; 
				if(!open_lay_input_file(filename)) {
					sprintf(stext," Input file = %s",filename) ;
					eraser(0,160,1000,200) ;
					temdtx(0,170,stext) ;
					gIOError = ERR_OP ;
					printf("\nOpen Failure: %s on PG #%d in slice %d",
						filename,nphsgr,i) ;
					fflush(stderr) ;
					return ;
				}

				if(!read_lay_input_file((char *)zphsgr,jmax)) {
					gIOError = ERR_RD ;
					close_lay_input_file() ;
					return ;
				}
				close_lay_input_file() ;
				SetPhasegrating(zphsgr,gPhsgrtRec->slcth,gMSRec.sigma,jmax) ;
				npgold = nphsgr ;
			}
			if (gPhsgrtRec->slcth!=old_slcth)  {
				CalculatePropagator(zprop,gPhsgrtRec->slcth) ;
			}
		}
		else if (gPhsgrtRec->nz!=1 && gSliceThreeD) {
			/*  The unit cell has been sub-divided */
			/*  and we have a new PG every time    */
			nphsgr = i%gPhsgrtRec->nz + 1 ;
			if(!read_input_file(nphsgr,(char *)zphsgr,jmax)) {
				gIOError = ERR_RD ;
				return ;
			}
			SetPhasegrating(zphsgr,gPhsgrtRec->slcth,gMSRec.sigma,jmax) ;
			if (i==0) {
				CalculatePropagator(zprop,gPhsgrtRec->slcth) ;
			}
		}
		else if (i==0)  {  /* Normal structure with just 1 phase grating */
			erase_message() ;
			write_message("Reading Input File...") ;
			if(!read_input_file(1,(char *)zphsgr,jmax)) {
				gIOError = ERR_RD ;
				erase_message() ;
				return ;
			}
			erase_message() ;
			fprintf(fp_print,"\n\n POT: %f %f %f %f",
				zphsgr[0],zphsgr[1],
				zphsgr[2],zphsgr[3]) ;
			SetPhasegrating(zphsgr,gPhsgrtRec->slcth,gMSRec.sigma,jmax) ;
			fprintf(fp_print,"\n\n gPhsgrtRec: %f %f %f %f",
				zphsgr[0],zphsgr[1],
				zphsgr[2],zphsgr[3]) ;
			CalculatePropagator(zprop,gPhsgrtRec->slcth) ;
			fprintf(fp_print,"\n\n PROP[dz=%.2f]: %f %f %f %f ",
				gPhsgrtRec->slcth,zprop[0],zprop[1],zprop[2],
			zprop[3]) ;
		}

		ApplyPhasegrating(zphi,zphsgr,jmax) ;
		ApplyPropagator(zphi,zprop,gPhsgrtRec->nbeam,gPhsgrtRec->mbeam,jmax,gMsliceRec.iplot,i);
		thickness += gPhsgrtRec->slcth ;
		if (fabs(thickness-tout) < gPhsgrtRec->slcth/2.)  {
	/* This needs to be edited to read the old data and add the intensities to it, then rewrite it */	
			is = write_output_file((char *)zphi,2*jmax) ;
			fprintf(fp_print,"\n\n ZPHI:[%.2fA] %f %f %f %f ",
				thickness,zphi[0],zphi[1],
				zphi[2],zphi[3]) ;
			if (gMsliceRec.tts) tout += deltatpr ;
		}
	}
	}
    erase_message() ;
}



