// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include <stdio.h>
#include <math.h>
#include "funcdefs.h"
#include "files.h"
#include "MslicRecord.h"
#include "ProjCellRecord.h"
#include "PhasegratingRecord.h"
#include "filenames.h"
#include "xtal.h"
#include "errors.h"
#include "storage.h"
#include "layers.h"
extern 	struct 	{ float wavel ; long nslice ; float sigma ; } gMSRec ;
extern 	FILE 	*fp_print ;
extern 	char 	stext[] ;
extern Boolean gNoGUI ;
void DoMultisliceCalculation() {
    /* --------------------------------------------------------	*/
    /* Main entry point for the start of the calculation of the	*/
    /* exit wave-function for a given thickness. The projected	*/
    /* potential must have been calculated by the 'PHSGRT'	*/
    /* program beforehand. This routine relies on the output	*/
    /* file from the 'PHSGRT' program <filename>.pout to get	*/
    /* its date for the projected potential.			*/
    /* -------------------------------------------------------- */
    float *zphi,*zprop,*zphsgr,*rp ;
    register long i,j ;
    long jmax ;
    char filename[80] ;
	
    /* Because of the way this program was written, the required space 	*/
    /* for the three large arrays used in the calculation was allocated	*/
    /* as a global data structure and not allocated dynamically during	*/
    /* execution. This has the side-effect of limiting the calculation 	*/
    /* to a fixed maximum size during compilation, regardless of the	*/
    /* available memory. The program should really be rewritten such	*/
    /* that neccessary memory is allocated/deallocated as it is needed,	*/
    /* thus requiring less space when the unit cell is small and then	*/
    /* being able to use as large unit cell as the computer could 	*/
    /* handle by allocating data for the arrays and atom-data structures*/
    /* as they are required. This would effect all the routines that	*/
    /* uses 'gWorkSpace' and the structures 'gPhsgrtRec->,'gProjCell->,'atom',	*/
    /* and 'basis'. I'll get to it one day.				*/

    zphi = gWorkSpace ;			/* Divide up the workspace	*/
    zphsgr = zphi+gMaxArraySize/3 ;	/* equally between the arrays	*/
    zprop  = zphsgr+gMaxArraySize/3 ;	/* If we had an array processor	*/
					/* we would utilize the memory	*/
					/* in the AP.			*/
    
    if (!gNoGUI) SetWatchCursor() ;    

    /* Open the print file */
    if ((fp_print=fopen(flname.mprint,"w"))==0) {
	ReportFileOpenError(flname.mprint) ;
    	gIOError = ERR_OP ;
		if (!gNoGUI) InitCursor() ;
       	return ;
    }        
     
    /* Open the input files and the output file 			*/
    if (!gLayerRec.layrd) {
		if(!open_input_file(flname.pout)) {
			fclose(fp_print) ;
			gIOError = ERR_OP ;
			if (!gNoGUI) InitCursor() ;
			return ;
        }
    }

    if (!open_output_file(flname.mout)) {
		if (!gLayerRec.layrd) close_input_file() ;
		fclose(fp_print) ;
    	gIOError = ERR_OP ;
		if (!gNoGUI) InitCursor() ;
    	return ;
    }        

    /* ----------------------------------------------*/
    /* read the information from the input files     */
    /* ----------------------------------------------*/
    SetMsliceParameters() ;
    if(gIOError) {
    	/* ParamText(flname.pout,"","","") ; */
		if (!gLayerRec.layrd) close_input_file() ;
		fclose(fp_print) ;
		close_output_file() ;
		if (!gNoGUI) InitCursor() ;
        return ; 
    }

    /* ------------------------ */
    /* write to the viewfile    */
    /* ------------------------ */
    PrintMsliceOutput() ;
    if(gIOError) {
    	if (!gLayerRec.layrd) close_input_file() ;
    	fclose(fp_print) ;
    	close_output_file() ;
		if (!gNoGUI) InitCursor() ;
    	return ; 
    }

    /* -------------------- */
    /* start multislicing   */
    /*--------------------- */
    jmax = gPhsgrtRec->nbeam*gPhsgrtRec->mbeam ;
    CalculateExitWavefunction(zphi,zphsgr,zprop,jmax) ;

    if(gIOError) {				/* Return with error ? */
	if (!gLayerRec.layrd) close_input_file() ;
    	fclose(fp_print) ;
    	close_output_file() ;
		if (!gNoGUI) InitCursor() ;
    	return ; 
    }

    /*---------------------------------------------------------- */
    /* Write out the amplitudes and phases if iplot = 1          */
    /*---------------------------------------------------------- */
    if (gMsliceRec.iplot) StoreAmplitudes() ;

    /* Close the open files */

    fclose(fp_print) ;
    if (!gLayerRec.layrd) close_input_file() ;
    close_output_file() ;
	if (!gNoGUI) InitCursor() ;
}

void SetMsliceParameters() {
    /* ------------------------------------------------------------- */
    /* This routine doesn't do much any more. The input file(s) used */
    /* to be read from here. Now only a few parameters get set.	     */
    /* ------------------------------------------------------------- */
    register long i ;
    long jmax,ierr ;
    float evolt,ewavel ;
    char text[80] ;
  
    /* Check to see if we have values from the gPhsgrtRec-> program */
    if (gPhsgrtRec->nbeam == 0) {
    	SetProjectedCell(&ierr) ;
    	SetCalculationDimensions(&gProjCell->a,&gProjCell->b,&gPhsgrtRec->gmax,&gPhsgrtRec->nbeam,&gPhsgrtRec->mbeam) ;
    }

    jmax = gPhsgrtRec->nbeam*gPhsgrtRec->mbeam ;
    if (!gLayerRec.layrd) {
    	gPhsgrtRec->slcth = gProjCell->c/gPhsgrtRec->nz ;
    	gMSRec.nslice = (long)(gMsliceRec.thick/gProjCell->c+0.5)*gPhsgrtRec->nz ;
    	gMsliceRec.thick = gMSRec.nslice*gPhsgrtRec->slcth ;
    }
    else {
    	gMSRec.nslice = gLayerRec.num_slc ;
    	gMsliceRec.thick = 0. ;
    	gPhsgrtRec->slcth = gLayerRec.slcth[0] ;
    	for(i=0;i<gLayerRec.num_slc;i++)
    		gMsliceRec.thick += gLayerRec.slcth[gLayerRec.phgr_seq[i]-1] ;
    }

    /* Calculate the wavelength in centimeters */
    evolt = gMsliceRec.volt ;
    ewavel = 1.2263652e-7/sqrt((evolt+9.785025e-4*evolt*evolt)*1000.) ;
    gMSRec.sigma = 2.08886e+6*ewavel*sqrt(1.0+5.886579e-20/(ewavel*ewavel)) ;
    gMSRec.wavel = 1.0e+8*ewavel ;  /* Wavelength in angstroms */

    /* ------------------------------------------------------------------- */
    /* SPACE TO STRIP OFF THE HEADER AS SOON AS I GET AROUND TO WRITE ONE  */
    /* In other words, no header is implemented aas of yet and doesn't	   */
    /* need to be stripped.						   */
    /* ------------------------------------------------------------------- */
}

void PrintMsliceOutput() {
    /* ------------------------------------------------------------------- */
    /* Print some stuff to a file so that the user can check that things   */
    /* went okay. The stuff gets put into the file <structure>.m_prnt	   */
    /* ------------------------------------------------------------------- */
    register long i ;
 
    fprintf(fp_print,"\nThis is output from MSLICE for %s",gPhsgrtRec->title) ;
    fprintf(fp_print,"\n\n Unit cell parameters : ") ;
    fprintf(fp_print,"\n A-axis = %f A",gProjCell->a) ;
    fprintf(fp_print,"\n B-axis = %f A",gProjCell->b) ;

    if (gLayerRec.layrd) {
    	fprintf(fp_print,"\n Slice thicknesses are : ") ;
    	for(i=0;i<gLayerRec.num_pg;fprintf(fp_print," %f A ",
    		gLayerRec.slcth[i++])) ;
    }
    else
    	fprintf(fp_print,"\n Slice thickness is : %f A",gPhsgrtRec->slcth) ;

    fprintf(fp_print,"\n\n Electron Energy Parameters :") ;
    fprintf(fp_print,"\n            Electron Energy %.2f KeV ",gMsliceRec.volt) ;
    fprintf(fp_print,"\n            Wavelength      %f A",gMSRec.wavel) ;
    fprintf(fp_print,"\n            Sigma           %f (A*volt)-1",
    		gMSRec.sigma) ;
    fprintf(fp_print,
        "\n\n The incoming electron beam is oriented as follows:") ;
    fprintf(fp_print,"\n Gx = %f, Gy = %f",gMsliceRec.kxin,gMsliceRec.kyin) ;
    fprintf(fp_print,"\n\n Crystal thickness : %f",gMsliceRec.thick) ;
    fprintf(fp_print,"\n The A-Axis is divided into %d intervals",
    	gPhsgrtRec->nbeam) ;
    fprintf(fp_print,"\n The B-Axis is divided into %d intervals",
    	gPhsgrtRec->mbeam) ;

    if (gMsliceRec.iplot) {
        fprintf(fp_print,"\n\n Following reflections has been stored: ") ;
    	for(i=0;i<gMsliceRec.nw;i++)
    	fprintf(fp_print,"\n %4d %4d %4d ",gMsliceRec.ihw[i],gMsliceRec.ikw[i],
    		gMsliceRec.ilw[i]) ;
    }
}

void StoreAmplitudes() {
    /* ------------------------------------------------------------------ */
    /* Write the complex amplitudes of a selected set of diffracted beams */
    /* to the file <structure>.aout. This file gets used to plot the 	  */
    /* Pendellossung plots 						  */
    /* ------------------------------------------------------------------ */
    FILE *fp_aout ;
    long i ;

    if ((fp_aout=fopen(flname.aout,"w"))==NULL) {
	ReportFileOpenError(flname.aout) ;
 	gIOError = ERR_OP ;
    	return ;
    }
    fwrite(gPhsgrtRec->title,40*sizeof(char),1,fp_aout) ;
    fwrite(&gProjCell->a,sizeof(float),1,fp_aout) ;
    fwrite(&gProjCell->b,sizeof(float),1,fp_aout) ;
    fwrite(&gProjCell->c,sizeof(float),1,fp_aout) ;
    fwrite(&gPhsgrtRec->nbeam,sizeof(long),1,fp_aout) ;
    fwrite(&gPhsgrtRec->mbeam,sizeof(long),1,fp_aout) ;
    fwrite(&gMsliceRec.volt,sizeof(float),1,fp_aout) ;
    fwrite(&gMsliceRec.nw,sizeof(long),1,fp_aout) ;
    fwrite(&gMSRec.nslice,sizeof(long),1,fp_aout) ;
    fwrite(&gPhsgrtRec->slcth,sizeof(float),1,fp_aout) ;
    fwrite(&gPhsgrtRec->gmax,sizeof(float),1,fp_aout) ;
    fwrite(gMsliceRec.ihw,sizeof(long),gMsliceRec.nw,fp_aout) ;
    fwrite(gMsliceRec.ikw,sizeof(long),gMsliceRec.nw,fp_aout) ;
    fwrite(gMsliceRec.ilw,sizeof(long),gMsliceRec.nw,fp_aout) ;
    i = 2*gMSRec.nslice*gMsliceRec.nw ; /* Complex data */
    fwrite(gZplot,sizeof(float),i,fp_aout) ;
    fclose(fp_aout) ;
}

void CalculatePropagator(float *zprop,float dz) {
  /* --------------------------------------------------- */
  /* This subroutine calculates the fourier transform of */
  /* the propagator. Only values for h and k such that   */
  /* usq is less than gmax-squared is calculated         */ 
  /* --------------------------------------------------- */
    register long h,k,j ;
    float ho,ko,csgast,udotk,gmaxsq,cnst,shift,arg,uy,ux ;
    float pi=3.1415927,usq,uxsq ;
    long jmax,maxh,maxk,nh,nk ;

    ho = gMsliceRec.kxin*gProjCell->ast ;  /* Center of the Laue circle */
    ko = gMsliceRec.kyin*gProjCell->bst ;

    jmax = gPhsgrtRec->nbeam*gPhsgrtRec->mbeam ;
    udotk = 0. ;
    csgast = -2.*gProjCell->csga ;
    maxh = gPhsgrtRec->nbeam/2 ;
    maxk = gPhsgrtRec->mbeam/2 ;
    gmaxsq = gPhsgrtRec->gmax*gPhsgrtRec->gmax ;
    cnst = -pi*gMSRec.wavel*dz ;

    for(nh=0;nh<gPhsgrtRec->nbeam;nh++) {
         h = nh ;
         if (h > maxh) h -= gPhsgrtRec->nbeam ;
         ux = h*gProjCell->ast ;
         uxsq = ux*ux ;
         for(nk=0;nk<gPhsgrtRec->mbeam;nk++) {
              k = nk ;
              if (k > maxk) k -= gPhsgrtRec->mbeam ;
              j = 2*(nk*gPhsgrtRec->nbeam+nh) ;
              uy = k*gProjCell->bst ;
              usq = uxsq + uy*uy + ux*uy*csgast ;
              if (usq <gmaxsq) {
                   if (ho==0. && ko==0) 
                        arg = cnst*usq ;
                   else {
                        udotk = ux*ho + uy*ko + 0.5*(ux*ko+uy*ho)*csgast ;
                        arg = cnst*(usq-2.*udotk) ;
                   }
                   zprop[j++] = cos(arg)/jmax ;
                   zprop[j]   = sin(arg)/jmax ;
              }
              else {
                   zprop[j++] = 0. ;
                   zprop[j]   = 0. ;
              }                 
         }
    }
}
