// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TImage.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "TResultTable.h"
#include "TShiftResultTable.h"

#include "TResidualImageDisplay.h" 


#include "funcdefs.h"	
#include "types.h"
#include "PhasegratingRecord.h"
#include "ProjCellRecord.h"
#include "MslicRecord.h"
#include "displ.h"
#include "filenames.h"
#include "files.h"
#include "errors.h"
#include "storage.h"
#include "dfpdspl.h"
#include "tiff.h"
extern struct { float wavel ; short nslice ; float sigma ; } gMSRec ;
extern char stext[] ;
extern short gComplexInputFile,gComplexOutputFile ;
void GetOutputImageDimensions(long *width,long *height) ;
extern long NUM_DEFOC,NUM_THICK ;
extern long CreateOffScreenRect(OffScreenPtr *offScrPtr,GrafPtr orgPort,Rect *rect) ;

void	AddWindow(TWindow *) ;

TImage *ImportRawFile(char *name) ;
TImage *ImportTiffFile(char *name) ;
void	CopyImage(TImage *image1, TImage *image2) ;
float	ImageStandardDeviation(TImage *image,float mean) ;
long	AskSaveResult(char *) ;

Boolean	UpdateCalculation() ;
void DisplayImageInWindow(TImage *image, TWindow *window, 
	Rect *rect,Boolean scaleToExpImageRatio) ;
float RealSpaceXCoeffCompareFromArea(TImage *image1,TImage *image2,FRect *rect) ;

float	gAspectRatio = 1. ;

Boolean IsPowerOfTwo(long value) ;
#define log2(x)	log(x)/log(2.0)

extern short gXRes,gYRes ;
typedef enum {kFileIsText,kFileIsBinary}
ImportFileType ;

typedef enum {kXCF,kChiSquare,kRMSD,kDifferenceImage,kFMAD,kFTXCF}
ComparisonMethod ;

extern	ComparisonMethod gComparisonMethod ;
extern	Boolean gEstimateImageShift ;
extern	Boolean gCalculateExactFTXCF ;
extern	FRect	gComparisonRect ;
extern	Boolean	gCompareEntireImage ;

extern TImage					*gExperimentalImage ;
extern TImage					*gStandardDeviationImage ;
extern  TiffRecord		gTiff ;

extern float gMeanOfExperiment ;
extern float gSigmaOfExperiment ;

extern short				gImportImageWidth ;
extern short				gImportImageHeight ;

void LoadExperimentalImage() ;
void LoadStandardDeviationImage() ;
void CompareWithSimulation() ;
void CompareAndRefine() ;
Boolean AdjustImageSize(TImage **image,long width,long height) ;
float RealSpaceXCoeffCompare(TImage *image1,TImage *image2) ;
Boolean HasExperimentalImage() ;
Boolean HasStandardDeviationImage() ;
float FourierSpaceXCoeffCompare(TImage *image1,TImage *image2,long &shiftN,long &shiftM) ;


TResultTable *ShowImageMatchingResult(TImage *result) ;

TImage *CompareImageWithSavedImages(TImage *expimage,TImage *sigmaImage) {

	long numCalculatedImages ;
	TImage *result,calculatedImage,*residualImage ;
	long i,j,whichImage,numImages,shiftN,shiftM ;
	long	outWidth,outHeight ;
	float	defocus,thickness ;
	char text[64] ;
	
	
	numCalculatedImages = NUM_DEFOC*NUM_THICK ;
	if (gComparisonMethod == kFTXCF && !gCalculateExactFTXCF && gEstimateImageShift) {
		result = CreateNewImage(3*NUM_DEFOC,NUM_THICK) ;
		if (result != nil)
			for(j=0;j<NUM_THICK;j++)
				for(i=0;i<3*NUM_DEFOC;i++) 
					*(result->mData + j*3*NUM_DEFOC+i) = 0. ;
	}
	else {
		result = CreateNewImage(NUM_DEFOC,NUM_THICK) ;
		// Zero in case we abort
		if (result != nil)
			for(j=0;j<NUM_THICK;j++)
				for(i=0;i<NUM_DEFOC;i++) 
					*(result->mData + j*NUM_DEFOC+i) = 0. ;
	}
	
	if (result == nil) {
		return result ;
	}
	
	if(!open_input_file(flname.iout)) {
		return nil ;
	}
		
	numImages = NUM_DEFOC*NUM_THICK	;
	
	FRect actionRect = gComparisonRect ;
	TResidualImageDisplay *theResidualImageDisplay = nil ;
	if (gComparisonMethod == kDifferenceImage) {
		GetOutputImageDimensions(&outWidth,&outHeight) ;
		theResidualImageDisplay = new TResidualImageDisplay(outWidth,outHeight,NUM_DEFOC,NUM_THICK) ;
		if (!theResidualImageDisplay->IsValid()) {
			PutMessage("Memory Failure !") ;
		 	DeleteImage(result) ;
		 	return nil ;
		}
	}
	
	if (gComparisonMethod == kFTXCF) {
		// Perform the Fourier transform of the experimental image
		FourierTransformImage(expimage) ;		
	}
	
	for(j=0;j<NUM_THICK;j++)
		for(i=0;i<NUM_DEFOC;i++) {
			whichImage = j*NUM_DEFOC+i ;
			if (ReadImageFromCalculation(&calculatedImage,whichImage,expimage->mSimulationSampling)) {
				switch(gComparisonMethod) {
					case kXCF:
						*(result->mData + whichImage) =
						 	 RealSpaceXCoeffCompareFromArea(expimage,&calculatedImage,&actionRect) ;
						break ;
					case kChiSquare:
						*(result->mData + whichImage) =
							 ComputeImageChiSquare(expimage,&calculatedImage,sigmaImage,&actionRect,
							 	0) ;
						break ;
					case kRMSD:
						*(result->mData + whichImage) =
							 ComputeMeanSquareDifference(expimage,&calculatedImage,&actionRect) ;
						break ;
					case kDifferenceImage:
						residualImage = CopyImage(expimage) ;
						if (residualImage != nil) {
							*(result->mData + whichImage) =
								 ComputeResidualImage(expimage,&calculatedImage,sigmaImage,&actionRect,
								 	residualImage,0) ;
							
							if (!theResidualImageDisplay->AddImageToDisplay(residualImage)) {
							 	close_input_file() ;
							 	PutMessage("Memory Failure !") ;
							 	DeleteImage(result) ;
							 	DeleteImage(residualImage) ;
							 	return nil ;
							}
							DeleteImage(residualImage) ;
						}
						else {
							close_input_file() ;
							PutMessage("Memory Failure !") ;
							DeleteImage(result) ;
							return nil ;
						}
						break ;
					case kFMAD:
						*(result->mData + whichImage) =
							 ComputeFMAD(expimage,&calculatedImage,&actionRect) ;
						break ;
					case kFTXCF:
						FourierTransformImage(&calculatedImage) ;
						if (!gCalculateExactFTXCF && gEstimateImageShift) {
							*(result->mData + whichImage*3) =
								 FourierSpaceXCoeffCompare(expimage,&calculatedImage,shiftN,shiftM) ;
							*(result->mData + whichImage*3+1) = shiftN ;
							*(result->mData + whichImage*3+2) = shiftM ;
						}
						else {
							*(result->mData + whichImage) =
								 FourierSpaceXCoeffCompare(expimage,&calculatedImage,shiftN,shiftM) ;
						}
						break ;
				}
			}
		}
		
	close_input_file() ;
	if (gComparisonMethod == kDifferenceImage) {
		AddWindow(theResidualImageDisplay) ;
		theResidualImageDisplay->Show() ;
	}

	return result ;
}


TResultTable *ShowImageMatchingResult(TImage *result) {
	TResultTable *theTable ;
	TShiftResultTable *theSTable ;
	TResultTable	*table ;
	
	float defocus,thickness ;
	short i ;

	if (gComparisonMethod == kFTXCF && !gCalculateExactFTXCF && gEstimateImageShift) {
		theSTable = new TShiftResultTable(result) ;
		table = theSTable ;
	}
	else {
		theTable = new TResultTable(result) ;
		table = theTable ;
	}
	
	if (table != nil) {
		switch(gComparisonMethod) {
			case kFTXCF:
					if (gCalculateExactFTXCF) {
						theTable->SetTableTitle("FTXCF") ;
						theTable->SetSuccessValues(1.0,0.9) ;
					}
					else {
						if (!gEstimateImageShift) {
							theTable->SetTableTitle("FTXCF-Amplitudes") ;
							theTable->SetSuccessValues(1.0,0.9) ;
						}
						else {
							theSTable->SetTableTitle("FTXCF-Amplitudes") ;
							theSTable->SetSuccessValues(1.0,0.9) ;
						}
					}
				break ;
			case kXCF:
					theTable->SetTableTitle("XCF") ;
					theTable->SetSuccessValues(1.0,0.9) ;
				break ;
			case kChiSquare:
					theTable->SetTableTitle("Chi-Square") ;
					theTable->SetSuccessValues(0.0,4.0) ;
				break ;
			case kRMSD:
					theTable->SetTableTitle("RMSD") ;
					theTable->SetSuccessValues(0.0,0.1) ;
				break ;
			case kDifferenceImage:
					theTable->SetTableTitle("Chi-Square") ;
					theTable->SetSuccessValues(0.0,4.0) ;
				break ;
			case kFMAD:
					theTable->SetTableTitle("FMAD") ;
					theTable->SetSuccessValues(0.0,0.1) ;
				break ;
		}

		// Need to set defoci 
		for(i=0;i<result->mWidth;i++) {
			// Find the correct defocus
			if (gDisplayRec.tfs)   
				defocus = gDisplayRec.f1 + gDisplayRec.df*i ;
			else
				defocus = gDisplayRec.f1 ;
			
			table->SetDefocusLabel(i+1,defocus) ;
		}
		for(i=0;i<result->mHeight;i++) {
		// Find the correct thickness 
			if (gMsliceRec.tts)   
				thickness = gMsliceRec.t1 + gMsliceRec.dt*i ;
			else
				thickness = gMsliceRec.t1 ;
			table->SetThicknessLabel(i+1,thickness) ;
		}
	}
	return table ;
}

void CompareExperimentalImageWithSimulation() {
	
				
	float cs,div ;
	TImage *sigmaImage = nil ;

	// sort out the number of thicknesses to be calculated
	NUM_THICK = 1 ;
	if (gMsliceRec.tts)		// If thru-thickness series	*/
		NUM_THICK = (gMsliceRec.t2-gMsliceRec.t1)/gMsliceRec.dt + 1 ;
	
	NUM_DEFOC = 1 ;
	if (gDisplayRec.tfs)	// If thru-defocus series
		NUM_DEFOC = fabs(gDisplayRec.f2-gDisplayRec.f1)/fabs(gDisplayRec.df) + 1 ;


	if (!UpdateCalculation()) {
		PutMessage("Failed to update the Calculation") ;
		return ;
	}

	SetLensParameters(&gDisplayRec.cs,&gDisplayRec.div,&cs,&div,&gPhsgrtRec->gmax,
		&gPhsgrtRec->nbeam,&gPhsgrtRec->mbeam,&gPhsgrtRec->nz) ;
	if(!open_input_file(flname.iout)) {
		return  ;
	}
	close_input_file() ;


	TImage *experimentalImage = CopyImage(gExperimentalImage) ;
			
	// Let us see if the image dimensions are different from the simulation sampling.
	// If one of the dimensions is different, adjust to fit the calculated image
	if (experimentalImage->mWidth != gPhsgrtRec->nbeam || 
			experimentalImage->mHeight != gPhsgrtRec->mbeam) {
		if (AdjustImageSize(&experimentalImage, gPhsgrtRec->nbeam,gPhsgrtRec->mbeam)) {
			experimentalImage->mSimulationSampling = true ;
		}
		else {
			// We should return.  We are in memory trouble
			PutMessage("Memory allocation error") ;
			return ;
		}
	}
			
			
	if (HasStandardDeviationImage()) {
		sigmaImage = CopyImage(gStandardDeviationImage) ;
		if (sigmaImage->mWidth != gPhsgrtRec->nbeam || 
				sigmaImage->mHeight != gPhsgrtRec->mbeam) {
			if (AdjustImageSize(&sigmaImage, gPhsgrtRec->nbeam,gPhsgrtRec->mbeam)) {
				sigmaImage->mSimulationSampling = true ;
			}
			else {
				// We should return.  We are in memory trouble
				PutMessage("Memory allocation error") ;
				return ;
			}
		}
	}
				
	// We flip the simulated images in ReadCalculatedImage to compare 
	// with imported images (y-down)
	// so we do not flip imported images		
	TImage *result ;
						
	SetWatchCursor() ;
	result = CompareImageWithSavedImages(experimentalImage,sigmaImage) ;
	InitCursor() ;
	if (result != nil) {
		TResultTable *table = ShowImageMatchingResult(result) ;
		table->Show() ;
		//table->Update() ;
		// We can delete the result image now since we have stored 
		// the values already
		DeleteImage(result) ;
	}

	DeleteImage(experimentalImage) ;
	if (HasStandardDeviationImage()) {
		DeleteImage(sigmaImage) ;
	}

}

float RealSpaceXCoeffCompareFromArea(TImage *image1,TImage *image2,FRect *rect) {
	float xcoeff ;
	register long i,col,row ;
	long lengthOfData ;
	double sum,d1,d2 ;
	float	mean1,mean2 ;
	FRect	rectToCompare ;
	register float *p1,*p2 ;
	long	leftEdge,rightEdge,topEdge,bottomEdge,width,height ;
		
	rectToCompare.left = rect->left ;
	rectToCompare.top = rect->top ;
	rectToCompare.right = rect->right ;
	rectToCompare.bottom = rect->bottom ;
	
	if (rectToCompare.left == 0. && rectToCompare.right == 1. && 
			rectToCompare.top == 0. && rectToCompare.bottom == 1.) {
				
		lengthOfData = image2->mWidth*image2->mHeight ;	// Size of calculated data
		ZeroMeanOfImage(image1) ;
		ZeroMeanOfImage(image2) ;
		
		sum = 0. ;
		d1 = 0. ;
		d2 = 0. ;
		p1 = image1->mData ;
		p2 = image2->mData ;

		for(i=0;i<lengthOfData;i++) {
			sum += *p1 * (*p2) ;
			d1 += *p1 * (*p1) ;
			d2 += *p2 * (*p2) ;
			p1++ ;
			p2++ ;
		}
		xcoeff = sum/(sqrt(d1*d2)) ;
	}
	else {
		leftEdge = rectToCompare.left*image1->mWidth ;
		rightEdge = rectToCompare.right*image1->mWidth ;
		topEdge = rectToCompare.top*image1->mHeight ;
		bottomEdge = rectToCompare.bottom*image1->mHeight ;
		width = rightEdge-leftEdge ;
		height = bottomEdge-topEdge ;
		// Find the mean of the images
		mean1 = ImageMeanOfArea(image1,&rectToCompare) ;
		mean2 = ImageMeanOfArea(image2,&rectToCompare) ;
		sum = 0. ;
		d1 = 0. ;
		d2 = 0. ;
		for(row=topEdge;row <bottomEdge;row++) {
			p1 = image1->mData+row*image1->mWidth + leftEdge;
			p2 = image2->mData+row*image1->mWidth + leftEdge ;
			for(col=leftEdge;col < rightEdge;col++) {
				sum += (*p1-mean1) * (*p2-mean2) ;
				d1 	+= (*p1-mean1) * (*p1-mean1) ;
				d2 	+= (*p2-mean2) * (*p2-mean2) ;
				p1++ ;
				p2++ ;
			}
		}
		xcoeff = sum/(sqrt(d1*d2)) ;
	}
	return xcoeff ;
}

float 
ComputeImageChiSquare(TImage *experimentalImage,TImage *theoreticalImage,TImage *errorImage,
		FRect *rect,long numAdjustableParameters) {
	float chiSquare,normalizationFactor ;
	register long i ;
	long	lengthOfData,row,col ;
	long	leftEdge,topEdge,rightEdge,bottomEdge,offset ;
	double sum,difference,dsq,sigma ;
	register float *p1,*p2,*p3 ;
	
	// Trying with some new scaling factors
	// I(exp) = kI(calc) + b
	float	meanOfCalculation = ImageMean(theoreticalImage) ;
	float 	sigmaOfCalculation	= ImageStandardDeviation(theoreticalImage,meanOfCalculation) ;
	
	float	scaleFactor = gSigmaOfExperiment/sigmaOfCalculation ;  
	float	backGround = gMeanOfExperiment - scaleFactor*meanOfCalculation ;
	
	// float	scaleFactor = gMeanOfExperiment/ImageMean(theoreticalImage) ;  // Old way
	
	if (rect->left == 0. && rect->top == 0. && rect->right == 1. && rect->bottom == 1.) {
		lengthOfData = experimentalImage->mWidth*experimentalImage->mHeight ;
		normalizationFactor = 1./(float)(lengthOfData - numAdjustableParameters) ;
		if (errorImage == nil) {
			sum = 0. ;
			p1 = experimentalImage->mData ;
			p2 = theoreticalImage->mData ;
			for(i=0;i<lengthOfData;i++) {
				/* Original code
				//difference = *p1 - (*p2)*scaleFactor	// to use the same mean
				if (*p1 > 10.0) {
					sigma = 0.1*(*p1) ;	
				}
				else {
					sigma = 1.0 ;
				}
				*/
				difference = *p1 - scaleFactor*(*p2) - backGround ;	// to use the same mean
				sigma = 0.1*gSigmaOfExperiment ;	 // Approximately 10% of the standard deviation
				difference /= sigma ;
				p1++ ;
				p2++ ;
				dsq = difference*difference ;
				sum += dsq ;
			}
			chiSquare = sum*normalizationFactor ;
		}
		else {
			sum = 0. ;
			p1 = experimentalImage->mData ;
			p2 = theoreticalImage->mData ;
			p3 = errorImage->mData ;
			for(i=0;i<lengthOfData;i++) {
				// difference = *p1 - (*p2)*scaleFactor ;
				difference = *p1 - scaleFactor*(*p2) - backGround ;	// to use the same mean
				if (*p3 > 0.01) {
					sigma = *p3 ;
				}
				else {
					sigma = 1.0 ;
				}
				difference /= sigma ;
				p1++ ;
				p2++ ;
				p3++ ;
				dsq = difference*difference ;
				sum += dsq ;
			}
			chiSquare = sum*normalizationFactor ;
		}
	}
	else {
		leftEdge = near(rect->left * experimentalImage->mWidth) ;
		rightEdge = near(rect->right * experimentalImage->mWidth) ;
		topEdge = near(rect->top * experimentalImage->mHeight) ;
		bottomEdge = near(rect->bottom * experimentalImage->mHeight) ;
		lengthOfData = (rightEdge-leftEdge)*(bottomEdge-topEdge) ;
		normalizationFactor = 1./(float)(lengthOfData - numAdjustableParameters) ;
		if (errorImage == nil) {
			sum = 0. ;
			for (row=topEdge;row < bottomEdge; row++) {
				p1 = experimentalImage->mData + row*experimentalImage->mWidth+leftEdge ;
				p2 = theoreticalImage->mData  + row*theoreticalImage->mWidth+leftEdge ;
				for(col=leftEdge;col<rightEdge;col++) {
					/* Original code
					//difference = *p1 - (*p2)*scaleFactor	// to use the same mean
					if (*p1 > 10.0) {
						sigma = 0.1*(*p1) ;	
					}
					else {
						sigma = 1.0 ;
					}
					*/
					difference = *p1 - scaleFactor*(*p2) - backGround ;	// to use the same mean
					sigma = 0.1*gSigmaOfExperiment ;	 // Approximately 10% of the standard deviation

					difference /= sigma ;
					p1++ ;
					p2++ ;
					dsq = difference*difference ;
					sum += dsq ;
				}
			}
			chiSquare = sum*normalizationFactor ;
		}
		else {
			sum = 0. ;
			for (row=topEdge;row < bottomEdge; row++) {
				offset = row*experimentalImage->mWidth+leftEdge ;
				p1 = experimentalImage->mData + offset ;
				p2 = theoreticalImage->mData  + offset ;
				p3 = errorImage->mData  + offset ;
				for(col=leftEdge;col<rightEdge;col++) {
					difference = *p1 - (*p2)*scaleFactor - backGround ;
					if (*p3 > 0.01) {
						sigma = *p3 ;
					}
					else {
						sigma = 1.0 ;
					}
					difference /= sigma ;
					p1++ ;
					p2++ ;
					p3++ ;
					dsq = difference*difference ;
					sum += dsq ;
				}
			}
			chiSquare = sum*normalizationFactor ;
		}
	}		
	return chiSquare ;
}

float FourierSpaceXCoeffCompare(TImage *image1,TImage *image2,long &shiftN,long &shiftM) {
	//Boolean gEstimateImageShift = false ;
	//Boolean gCalculateExactFTXCF = true ;
	float xcoeff ;
	register float x1,x2,y1,y2 ;
	register long i ;
	long lengthOfData,hIndex,kIndex,width,height,wm1,maxHIndex,maxKIndex,shiftNH1,shiftNK1,NHalf,MHalf ;
	double sum,d1,d2,amp1sq,amp2sq,scalesq,thresholdIntensity,thresholdIntensityHK1 ;
	double pi,phase1h,phase2h,phase1k,phase2k,maxIntH,maxIntK,phaseShift ;
	register float *p1,*p2 ;
	Boolean possibleNOver2Ambiguity = false,possibleMOver2Ambiguity = false ;
	Boolean	hasHK0 = false,hasH0K = false ;
	
	// Data is complex and consists of the values for h = 0,1,2,3,.....N/2..-N/2+1....... -1
	// and k = 0,1........ M/2
	// There are 1/2 N*M complex datapoints 
	lengthOfData = image2->mWidth*image2->mHeight ;	// Size of calculated data
	scalesq = 1./(float)lengthOfData ;
	
	thresholdIntensity = 0.00000001*lengthOfData ; // 10**-8
	thresholdIntensityHK1 =  0.0001*lengthOfData ; // 10**-4
	//thresholdIntensity = 0.0 ;
	
	width = image1->mWidth ;
	height = image1->mHeight ;
	wm1 = width-1 ;
	pi = 2.*asin(1.0) ;
	NHalf = gImportImageWidth/2 ;
	MHalf = gImportImageHeight/2 ;
	
	if (gCalculateExactFTXCF) {		
		sum = 0. ;
		d1 = 0. ;
		d2 = 0. ;
		p1 = image1->mData+2 ;
		p2 = image2->mData+2 ;

		for(i=1;i<lengthOfData/2;i++) { // Skip (0,0)
			x1 = *p1++ ;
			y1 = *p1++ ;
			x2 = *p2++ ;
			y2 = *p2++ ;
			if (i< width/2 || i > wm1) {
				amp1sq = (x1*x1+y1*y1) ;
				amp2sq = (x2*x2+y2*y2) ;
				sum += x1*x2+y1*y2 ;
				d1 += amp1sq ;
				d2 += amp2sq ;
			}
		}
		xcoeff = sum/(sqrt(d1*d2)) ;
	}
	else {		
		sum = 0. ;
		d1 = 0. ;
		d2 = 0. ;
		p1 = image1->mData+2 ;
		p2 = image2->mData+2 ;
		
		if (gEstimateImageShift) {	// Estimate image shift
			maxIntH = 0. ;
			maxIntK = 0. ;
			maxHIndex = 0 ;
			maxKIndex = 0 ;
			for(hIndex=1;hIndex < width/2;hIndex++) {
				x1 = *(image1->mData+2*hIndex) ;	// (h,0)
				y1 = *(image1->mData+2*hIndex+1) ;
				x2 = *(image2->mData+2*hIndex) ;
				y2 = *(image2->mData+2*hIndex+1) ;
				amp2sq = (x2*x2+y2*y2) ;
				if (amp2sq > thresholdIntensityHK1) {
					amp1sq = (x1*x1+y1*y1) ; 
					if (amp1sq > maxIntH) {
						maxIntH = amp1sq ;
						maxHIndex = hIndex ;
						phase1h = atan(y1/x1) ;
						phase2h = atan(y2/x2) ;
						if (x1 < 0.) phase1h += pi ;
						if (x2 < 0.) phase2h += pi ;
					}
					hasHK0 = true ;
				}
			}
			
			for(kIndex=1;kIndex < height/2;kIndex++) {
				x1 = *(image1->mData+2*kIndex*width) ;	// (0,k)
				y1 = *(image1->mData+2*kIndex*width+1) ;
				x2 = *(image2->mData+2*kIndex*width) ;
				y2 = *(image2->mData+2*kIndex*width+1) ;
				amp2sq = (x2*x2+y2*y2) ;
				if (amp2sq > thresholdIntensityHK1) {
					amp1sq = (x1*x1+y1*y1) ; 
					if (amp1sq > maxIntK) {
						maxIntK = amp1sq ;
						maxKIndex = kIndex ;
						phase1k = atan(y1/x1) ;
						phase2k = atan(y2/x2) ;
						if (x1 < 0.) phase1k += pi ;
						if (x2 < 0.) phase2k += pi ;
					}
					hasH0K = true ;
				}
			}
			if (hasHK0) {
				// Let us look at the indices returned from the routine
				// It is possible to have shifts that give multiple of 2 
				// n = N*Phase/(2**h)  i*N/h  ; i = 1,2,3... i <= h/2   
				shiftN = near((phase1h-phase2h)*gImportImageWidth/(2.*pi*maxHIndex)) ;
				if (maxHIndex%2 == 0) {	// even
					if (maxHIndex != 2) {
						// Check the phaseshift at h = 2
						hIndex = 2 ;
						x1 = *(image1->mData+2*hIndex) ;	// (h,k)
						y1 = *(image1->mData+2*hIndex+1) ;
						x2 = *(image2->mData+2*hIndex) ;
						y2 = *(image2->mData+2*hIndex+1) ;
						amp2sq = (x2*x2+y2*y2) ;
						if (amp2sq > thresholdIntensityHK1) {
							phase1h = atan(y1/x1) ;
							phase2h = atan(y2/x2) ;
							if (x1 < 0.) phase1h += pi ;
							if (x2 < 0.) phase2h += pi ;
							shiftN = near((phase1h-phase2h)*gImportImageWidth/(2.*pi*2)) ;
						}
					}
					// We may have a translation of N/2 that doesn't show up
					// Try to get the phase shift at h = 1
					hIndex = 1 ;
					x1 = *(image1->mData+2*hIndex) ;	// (h,k)
					y1 = *(image1->mData+2*hIndex+1) ;
					x2 = *(image2->mData+2*hIndex) ;
					y2 = *(image2->mData+2*hIndex+1) ;
					// Since we are looking at simulated images in image 2 it is best to see
					// if we have a non-zero reflection there
					amp2sq = (x2*x2+y2*y2) ;
					if (amp2sq > thresholdIntensityHK1) {
						phase1h = atan(y1/x1) ;
						phase2h = atan(y2/x2) ;
						if (x1 < 0.) phase1h += pi ;
						if (x2 < 0.) phase2h += pi ;
						shiftNH1 = near((phase1h-phase2h)*gImportImageWidth/(2.*pi)) ;
						if (abs(abs(shiftNH1)- NHalf) < 3) 
							shiftN = NHalf ;
					}
					else {
						// We still have an ambiguity
						possibleNOver2Ambiguity = true ;
					}
				}
				else if (maxHIndex != 1) { // Odd, but not equal to 1
					// Check the phaseshift at h = 1
					hIndex = 1 ;
					x1 = *(image1->mData+2*hIndex) ;	// (h,k)
					y1 = *(image1->mData+2*hIndex+1) ;
					x2 = *(image2->mData+2*hIndex) ;
					y2 = *(image2->mData+2*hIndex+1) ;
					amp2sq = (x2*x2+y2*y2) ;
					if (amp2sq > thresholdIntensityHK1) {
						phase1h = atan(y1/x1) ;
						phase2h = atan(y2/x2) ;
						if (x1 < 0.) phase1h += pi ;
						if (x2 < 0.) phase2h += pi ;
						shiftN = near((phase1h-phase2h)*gImportImageWidth/(2.*pi)) ;
					}
					else {
						// We still have an ambiguity
						possibleNOver2Ambiguity = true ;
					}
				}						
			}
			else {
				// I don't know what happens if all (h,0) are forbidden
			}
			
			if (hasH0K) {
				shiftM = near((phase1k-phase2k)*gImportImageHeight/(2.*pi*maxKIndex)) ;
				// Let us look at the indices returned from the routine
				// It is possible to have shifts that give multiple of 2 
				// m = M*Phase/(2**k)  i*M/k  ; i = 1,2,3... i <= k/2  
				//
				if (maxKIndex%2 == 0) {	// even
					if (maxKIndex != 2) {
						// Check the phaseshift at k = 2
						kIndex = 2 ;
						x1 = *(image1->mData+2*kIndex*width) ;	// (h,k)
						y1 = *(image1->mData+2*kIndex*width+1) ;
						x2 = *(image2->mData+2*kIndex*width) ;
						y2 = *(image2->mData+2*kIndex*width+1) ;
						amp2sq = (x2*x2+y2*y2) ;
						if (amp2sq > thresholdIntensityHK1) {
							phase1k = atan(y1/x1) ;
							phase2k = atan(y2/x2) ;
							if (x1 < 0.) phase1k += pi ;
							if (x2 < 0.) phase2k += pi ;
							shiftM = near((phase1k-phase2k)*gImportImageHeight/(2.*pi*2)) ;
						}
					}
					// We may have a translation of N/2 that doesn't show up
					kIndex = 1 ;
					x1 = *(image1->mData+2*kIndex*width) ;	// (h,k)
					y1 = *(image1->mData+2*kIndex*width+1) ;
					x2 = *(image2->mData+2*kIndex*width) ;
					y2 = *(image2->mData+2*kIndex*width+1) ;
					// Since we are looking at simulated images in image 2 it is best to see
					// if we have a non-zero reflection there
					amp2sq = (x2*x2+y2*y2) ;
					if (amp2sq > thresholdIntensityHK1) {
						phase1k = atan(y1/x1) ;
						phase2k = atan(y2/x2) ;
						if (x1 < 0.) phase1k += pi ;
						if (x2 < 0.) phase2k += pi ;
						shiftNK1 = near((phase1k-phase2k)*gImportImageHeight/(2.*pi)) ;
						if (abs(abs(shiftNK1)- MHalf) < 3) 
							shiftM = MHalf ;
					}
					else {
						// We still have an ambiguity
						possibleMOver2Ambiguity = true ;
					}
				}
				else if (maxKIndex != 1) { // Odd, but not equal to 1
					// Check the phaseshift at k = 1
					kIndex = 1 ;
					x1 = *(image1->mData+2*kIndex*width) ;	// (h,k)
					y1 = *(image1->mData+2*kIndex*width+1) ;
					x2 = *(image2->mData+2*kIndex*width) ;
					y2 = *(image2->mData+2*kIndex*width+1) ;
					amp2sq = (x2*x2+y2*y2) ;
					if (amp2sq > thresholdIntensityHK1) {
						phase1k = atan(y1/x1) ;
						phase2k = atan(y2/x2) ;
						if (x1 < 0.) phase1k += pi ;
						if (x2 < 0.) phase2k += pi ;
						shiftM = near((phase1k-phase2k)*gImportImageHeight/(2.*pi)) ;
					}
					else {
						// We still have an ambiguity
						possibleMOver2Ambiguity = true ;
					}
				}
			}
			else {
				// I don't know what happens if all (0,k) are forbidden
			}

			if (possibleNOver2Ambiguity) {
				// Try the reflection (1,2)
				long prevN = shiftN ;
				hIndex = 1 ;
				kIndex = 2 ;
				x1 = *(image1->mData+2*kIndex*width+2*hIndex) ;	// (h,k)
				y1 = *(image1->mData+2*kIndex*width+2*hIndex+1) ;
				x2 = *(image2->mData+2*kIndex*width+2*hIndex) ;
				y2 = *(image2->mData+2*kIndex*width+2*hIndex+1) ;
				// Since we are looking at simulated images in image 2 it is best to see
				// if we have a non-zero reflection there
				amp2sq = (x2*x2+y2*y2) ;
				if (amp2sq > thresholdIntensityHK1) {
					phase1k = atan(y1/x1) ;
					phase2k = atan(y2/x2) ;
					if (x1 < 0.) phase1k += pi ;
					if (x2 < 0.) phase2k += pi ;
					phaseShift = phase1k-phase2k ;
					if (shiftM != 0 && shiftM != MHalf) {
						phaseShift -= 2*pi*shiftM*2/gImportImageHeight ;
					}
					shiftN = near(phaseShift*gImportImageWidth/(2.*pi)) ;
					// test the new N value against the old
					if (abs(abs(shiftN)-abs(prevN)) > gImportImageWidth/4.) {							
						shiftN = shiftN%gImportImageWidth ;
						if (shiftN < 0) shiftN += gImportImageWidth ;
						//shiftNH1 = near(phaseShift*gImportImageWidth/(2.*pi)) ;
						//if (fabs(fabs(shiftNH1)- NHalf) < 3) {
						//	shiftN += NHalf ;
						//	possibleNOver2Ambiguity = false ;
						//}
					}
					else {
						shiftN = prevN ;
					}
				}
			}		
			if (possibleMOver2Ambiguity) {
				// Try the reflection (2,1)
				long prevM = shiftM ;
				hIndex = 2 ;
				kIndex = 1 ;
				x1 = *(image1->mData+2*kIndex*width+2*hIndex) ;	// (h,k)
				y1 = *(image1->mData+2*kIndex*width+2*hIndex+1) ;
				x2 = *(image2->mData+2*kIndex*width+2*hIndex) ;
				y2 = *(image2->mData+2*kIndex*width+2*hIndex+1) ;
				// Since we are looking at simulated images in image 2 it is best to see
				// if we have a non-zero reflection there
				amp2sq = (x2*x2+y2*y2) ;
				if (amp2sq > thresholdIntensityHK1) {
					phase1k = atan(y1/x1) ;
					phase2k = atan(y2/x2) ;
					if (x1 < 0.) phase1k += pi ;
					if (x2 < 0.) phase2k += pi ;
					phaseShift = phase1k-phase2k ;
					if (shiftN != 0 && shiftN != NHalf) {
						phaseShift -= 2*pi*shiftN*2/gImportImageWidth ;
					}
					shiftM = near(phaseShift*gImportImageHeight/(2.*pi)) ;
					// test the new N value against the old
					if (abs(abs(shiftM)-abs(prevM)) > gImportImageHeight/4.) {							
						shiftM = shiftM%gImportImageHeight ;
						if (shiftM < 0) shiftM += gImportImageHeight ;
						//shiftNK1 = near(phaseShift*gImportImageHeight/(2.*pi)) ;
						//if (fabs(fabs(shiftNK1)- MHalf) < 3) {
						//	shiftM = MHalf ;
						//	possibleMOver2Ambiguity = false ;
						//}
					}
					else {
						shiftM = prevM ;
					}
				}
			}		
		} // Estimate image shift
		
		
		for(i=1;i<lengthOfData/2;i++) {	// Skip (0,0)
			x1 = *p1++ ;
			y1 = *p1++ ;
			x2 = *p2++ ;
			y2 = *p2++ ;
			if (i< width/2 || i > wm1) {
				amp1sq = (x1*x1+y1*y1) ;
				amp2sq = (x2*x2+y2*y2) ;
				d1 += amp1sq ;
				d2 += amp2sq ;
				sum += sqrt(amp1sq*amp2sq) ;
			}
		}
		xcoeff = sum/(sqrt(d1*d2)) ;
	}
	return xcoeff ;
}


float RealSpaceXCoeffCompare(TImage *image1,TImage *image2) {
	float xcoeff,xcf ;
	register long i ;
	long lengthOfData ;
	double sum,d1,d2 ;
	TImage *expImage ;
	register float *p1,*p2 ;
		
	// The first image is normally considered the experimental image
	// In case the images are of a different size, ie. the experimental
	// image is greater than the calculated image, we shift the calculated image to cover the
	// are of the experimental image and report the highest value of the cross-correlation
	// coefficient
	
	if (image1->mWidth == image2->mWidth && image1->mHeight == image2->mHeight) {
	
		lengthOfData = image2->mWidth*image2->mHeight ;	// Size of calculated data
		
		ZeroMeanOfImage(image1) ;
		ZeroMeanOfImage(image2) ;
		
		sum = 0. ;
		d1 = 0. ;
		d2 = 0. ;
		p1 = image1->mData ;
		p2 = image2->mData ;

		for(i=0;i<lengthOfData;i++) {
			sum += *p1 * (*p2) ;
			d1 += *p1 * (*p1) ;
			d2 += *p2 * (*p2) ;
			p1++ ;
			p2++ ;
		}
		xcoeff = sum/(sqrt(d1*d2)) ;
	}
	else {
		expImage = CreateNewImage(image2->mWidth,image2->mHeight) ;
		long dx,dy ;
		xcoeff = -1. ;
		for(dy = 0; dy <= image1->mHeight - image2->mHeight; dy++) {
			for(dx = 0; dx <= image1->mWidth - image2->mWidth; dx++) {
				FillImageFromImageArea(image2->mWidth,image2->mHeight,dx,dy,image1,expImage) ;
				xcf = RealSpaceXCoeffCompare(expImage,image2) ;
				if (xcf > xcoeff)
					xcoeff = xcf ;
			}
		}
	}
	return xcoeff ;
}



float ComputeFMAD(TImage *experimentalImage,TImage *theoreticalImage,FRect *rect){
	float fmad ;
	register long i ;
	long	lengthOfData,row,col ;
	long	leftEdge,topEdge,rightEdge,bottomEdge ;
	double sum,difference,length ;
	register float *p1,*p2 ;
	
	
	// Trying with some new scaling factors
	// I(exp) = kI(calc) + b
	float	meanOfCalculation = ImageMean(theoreticalImage) ;
	float 	sigmaOfCalculation	= ImageStandardDeviation(theoreticalImage,meanOfCalculation) ;
	
	float	scaleFactor = gSigmaOfExperiment/sigmaOfCalculation ;  
	float	backGround = gMeanOfExperiment - scaleFactor*meanOfCalculation ;
	
	// float	scaleFactor = gMeanOfExperiment/ImageMean(theoreticalImage) ;  // Old way

	if (rect->left == 0. && rect->top == 0. && rect->right == 1. && rect->bottom == 1.) {
		lengthOfData = experimentalImage->mWidth*experimentalImage->mHeight ;
		sum = 0. ;
		length = 0.0 ;
		p1 = experimentalImage->mData ;
		p2 = theoreticalImage->mData ;
		for(i=0;i<lengthOfData;i++) {
			difference = *p1 - (*p2)*scaleFactor - backGround ;
			length += *p1 ;
			p1++ ;
			p2++ ;
			sum += fabs(difference) ;
		}
		fmad = sum/length ;
	}
	else {
		leftEdge = near(rect->left * experimentalImage->mWidth) ;
		rightEdge = near(rect->right * experimentalImage->mWidth) ;
		topEdge = near(rect->top * experimentalImage->mHeight) ;
		bottomEdge = near(rect->bottom * experimentalImage->mHeight) ;
		lengthOfData = (rightEdge-leftEdge)*(bottomEdge-topEdge) ;
		sum = 0. ;
		length = 0.0 ;
		for (row=topEdge;row < bottomEdge; row++) {
			p1 = experimentalImage->mData + row*experimentalImage->mWidth+leftEdge ;
			p2 = theoreticalImage->mData  + row*theoreticalImage->mWidth+leftEdge ;
			for(col=leftEdge;col<rightEdge;col++) {
				difference = *p1 - (*p2)*scaleFactor - backGround ;
				length += *p1 ;
				p1++ ;
				p2++ ;
				sum += fabs(difference) ;
			}
		}
		fmad = sum/length ;
	}		
	return fmad ;
}

float ComputeMeanSquareDifference(TImage *experimentalImage,TImage *theoreticalImage,FRect *rect) {
	float msd ;
	register long i ;
	long	lengthOfData,row,col ;
	long	leftEdge,topEdge,rightEdge,bottomEdge ;
	double sum,difference,dsq ;
	register float *p1,*p2 ;
	
	// Trying with some new scaling factors
	// I(exp) = kI(calc) + b
	float	meanOfCalculation = ImageMean(theoreticalImage) ;
	float 	sigmaOfCalculation	= ImageStandardDeviation(theoreticalImage,meanOfCalculation) ;
	
	float	scaleFactor = gSigmaOfExperiment/sigmaOfCalculation ;  
	float	backGround = gMeanOfExperiment - scaleFactor*meanOfCalculation ;
	
	// float	scaleFactor = gMeanOfExperiment/ImageMean(theoreticalImage) ;  // Old way
	
	if (rect->left == 0. && rect->top == 0. && rect->right == 1. && rect->bottom == 1.) {
		lengthOfData = experimentalImage->mWidth*experimentalImage->mHeight ;
		sum = 0. ;
		p1 = experimentalImage->mData ;
		p2 = theoreticalImage->mData ;
		for(i=0;i<lengthOfData;i++) {
			// difference = *p1 - (*p2)*scaleFactor ; // Original code
			difference = *p1 - (*p2)*scaleFactor - backGround ;
			p1++ ;
			p2++ ;
			dsq = difference*difference ;
			sum += dsq ;
		}
		msd = sum/lengthOfData ;
	}
	else {
		leftEdge = near(rect->left * experimentalImage->mWidth) ;
		rightEdge = near(rect->right * experimentalImage->mWidth) ;
		topEdge = near(rect->top * experimentalImage->mHeight) ;
		bottomEdge = near(rect->bottom * experimentalImage->mHeight) ;
		lengthOfData = (rightEdge-leftEdge)*(bottomEdge-topEdge) ;
		sum = 0. ;
		for (row=topEdge;row < bottomEdge; row++) {
			p1 = experimentalImage->mData + row*experimentalImage->mWidth+leftEdge ;
			p2 = theoreticalImage->mData  + row*theoreticalImage->mWidth+leftEdge ;
			for(col=leftEdge;col<rightEdge;col++) {
				// difference = *p1 - (*p2)*scaleFactor ; // Original code
				difference = *p1 - (*p2)*scaleFactor - backGround ;
				p1++ ;
				p2++ ;
				dsq = difference*difference ;
				sum += dsq ;
			}
		}
		msd = sum/lengthOfData ;
	}		
	return sqrt(msd) ;
}


float ComputeResidualImage(TImage *experimentalImage,TImage *theoreticalImage,
		TImage *errorImage,FRect *rect,TImage *residualImage,long numAdjustableParameters) {
	float chiSquare,normalizationFactor ;
	register long i ;
	long	lengthOfData,row,col ;
	long	leftEdge,topEdge,rightEdge,bottomEdge ;
	double difference,dsq,sum,sigma ;
	register float *p1,*p2,*p3,*p4 ;
		
	// Trying with some new scaling factors
	// I(exp) = kI(calc) + b
	float	meanOfCalculation = ImageMean(theoreticalImage) ;
	float 	sigmaOfCalculation	= ImageStandardDeviation(theoreticalImage,meanOfCalculation) ;
	
	float	scaleFactor = gSigmaOfExperiment/sigmaOfCalculation ;  
	float	backGround = gMeanOfExperiment - scaleFactor*meanOfCalculation ;
	
	// float	scaleFactor = gMeanOfExperiment/ImageMean(theoreticalImage) ;  // Old way

	// Calculate the residual image from the entire image are
	lengthOfData = experimentalImage->mWidth*experimentalImage->mHeight ;
	if (errorImage == nil) {
		p1 = experimentalImage->mData ;
		p2 = theoreticalImage->mData ;
		p4 = residualImage->mData ;
		for(i=0;i<lengthOfData;i++) {
			/* Original code
			//difference = *p1 - (*p2)*scaleFactor	// to use the same mean
			if (*p1 > 0.0001) {
				sigma = 0.1*(*p1) ;	// Approximately 10% error in the experimental image
			}
			else {
				sigma = 1.0 ;
			}
			*/
			difference = *p1 - scaleFactor*(*p2) - backGround ;	// to use the same mean
			sigma = 0.1*gSigmaOfExperiment ;	 // Approximately 10% of the standard deviation
			difference /= sigma ;	// Approximately 10% error in the experimental image
			p1++ ;
			p2++ ;
			*p4++ = difference;
		}
	}
	else {
		p1 = experimentalImage->mData ;
		p2 = theoreticalImage->mData ;
		p3 = errorImage->mData ;
		p4 = residualImage->mData ;
		for(i=0;i<lengthOfData;i++) {
			if (*p3 > 0.01) {
				sigma = *p3 ;
			}
			else {
				sigma = 1.0 ;
			}
			difference = (*p1 - (*p2)*scaleFactor  - backGround)/sigma ;
			p1++ ;
			p2++ ;
			p3++ ;
			*p4++ = difference;
		}
	}

	// Calculate the chi-square for the active area
	if (rect->left == 0. && rect->top == 0. && rect->right == 1. && rect->bottom == 1.) {
		lengthOfData = experimentalImage->mWidth*experimentalImage->mHeight ;
		normalizationFactor = 1./(float)(lengthOfData - numAdjustableParameters) ;
		p4 = residualImage->mData ;
		sum = 0. ;
		for(i=0;i<lengthOfData;i++) {
			difference = *p4++ ;
			dsq = difference*difference ;
			sum += dsq ;
		}
		chiSquare = sum*normalizationFactor ;
	}
	else {
		leftEdge = near(rect->left * experimentalImage->mWidth) ;
		rightEdge = near(rect->right * experimentalImage->mWidth) ;
		topEdge = near(rect->top * experimentalImage->mHeight) ;
		bottomEdge = near(rect->bottom * experimentalImage->mHeight) ;
		lengthOfData = (rightEdge-leftEdge)*(bottomEdge-topEdge) ;
		normalizationFactor = 1./(float)(lengthOfData - numAdjustableParameters) ;
		sum = 0. ;
		for (row=topEdge;row < bottomEdge; row++) {
			p4 = residualImage->mData  + row*residualImage->mWidth+leftEdge ;
			for(col=leftEdge;col<rightEdge;col++) {
				difference = *p4++ ;
				dsq = difference*difference ;
				sum += dsq ;
			}
		}
		chiSquare = sum*normalizationFactor ;
	}		
	return chiSquare ;
}

long	AskSaveResult(char *text) {
	return 3 ;
}

/* 
** Check to see if the number passed is a power of 2
*/
Boolean IsPowerOfTwo(long value) {
	float fval,rp2,dp2 ;
	long np2 ;
	
	fval = (float)value ;
	rp2 = log2(fval) ;
	np2 = (long)(rp2+0.5) ;
	dp2 = fabs(rp2 - np2) ;
	
	if (dp2 > 0.001) {
		return false ;
	}
		
	return true ;
}
