// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TBondValenceDialog.h"
#include "funcdefs.h"
extern	char *chemsymb[] ;

extern	Boolean		gCalculateBondStrengths ;

typedef struct {
	float top ;
	float left ;
	float bottom ;
	float right ;
} FRect ;

typedef struct	{
	short number ;
	short index[20] ;
} ElementRecord,*ElementPtr ;
extern	ElementPtr MyElements ;

typedef struct {
	short	firstAtomNdx ;
	short	secondAtomNdx ;
	float	maxBondLength ;
	float	valenceSum ;
	float	bFactor ;
	float	weight ;
} BondStrength ;

typedef struct BondStrengthData {
	short	numberOfBonds ;
	float	totalWeight ;
	BondStrength bondStrength[20] ;
} BondStrengthData ;

extern	BondStrengthData gBondStrengthData ;


typedef enum {kCrossCorrelation,kChiSquared,kFTAmplCrossCorrelation } FittingMethod ;

typedef struct {
	float	maxAmplitude ;	// In Angstrom
	Boolean	useActiveRect ;
	FRect	activeRect ;
} RelaxationAtomRecord ;

typedef struct {
	float	start ;			// Starting Temperature
	float	end ;			// Ending Temperature
	float	current ;		// current temperature
	float	dTFraction ;	// Temperature decrement
	long		maxNumTriesPerTemperature ;
	long		maxNumGoodTries ;
} RelaxationTemperatureRecord ;

typedef struct {
	RelaxationAtomRecord		atom ;	// In Angstrom
	RelaxationTemperatureRecord	temperature ;
	float previousEnergy ;
	float thisEnergy ;
	float minEnergy ;
	long  numTriesThisTemperature ;
	FittingMethod fittingMethod ;
} RelaxationParamRecord ;

extern	RelaxationParamRecord gRelaxationParamRecord ;

TBondValenceDialog::TBondValenceDialog() 
	: TXForm("Bond Valence Criteria") {

	DefineForm() ;
	mData = (void *) new BondStrengthData[20] ;
}

TBondValenceDialog::~TBondValenceDialog() {
	delete[] mData ;
}

void TBondValenceDialog::DefineForm() {
	FL_OBJECT	*obj ;
	mXForm->mForm = fl_bgn_form(FL_NO_BOX, 391, 271);
	fl_add_box(FL_UP_BOX,0,0,391,271,"");

	fl_add_labelframe(FL_ENGRAVED_FRAME,190,25,180,110,"Bond Strength Criteria");
	fl_add_labelframe(FL_ENGRAVED_FRAME,190,140,180,45,"Energy contribution");
	fl_add_labelframe(FL_ENGRAVED_FRAME,10,25,170,195,"Bonds");

	mAtomList1 = fl_add_browser(FL_HOLD_BROWSER,15,33,60,110,"From");
	mAtomList2 = fl_add_browser(FL_HOLD_BROWSER,115,33,60,110,"To");
	mBondList = fl_add_browser(FL_HOLD_BROWSER,15,145,160,70,"");

	fl_add_box(FL_UP_BOX,75,90,40,10,"");

	mOKButton = fl_add_button(FL_NORMAL_BUTTON,300,230,70,30,"OK");
	mCancelButton = fl_add_button(FL_NORMAL_BUTTON,190,230,90,30,"Cancel");

	mAdd = fl_add_button(FL_NORMAL_BUTTON,190,195,50,25,"Add");
	mModify = fl_add_button(FL_NORMAL_BUTTON,250,195,60,25,"Modify");
	mDelete = fl_add_button(FL_NORMAL_BUTTON,320,195,50,25,"Delete");

	mRMax = fl_add_input(FL_NORMAL_INPUT,305,30,60,20,"Max. distance  [Ang.]");
	mValence = fl_add_input(FL_NORMAL_INPUT,335,55,30,20,"Atom Valence");
	mBondConstant = fl_add_input(FL_NORMAL_INPUT,305,80,60,20,"Bond Constant");
	mBondWeight = fl_add_input(FL_NORMAL_INPUT,315,105,50,20,"Bond Weight");

	mWeight = obj = fl_add_input(FL_NORMAL_INPUT,295,160,60,20,
		"Total Combined Weight (W)");
	fl_set_object_lalign(obj,FL_ALIGN_TOP_RIGHT);

	fl_end_form();

	fl_set_clipping(0,0,391, 271) ;
	mXForm->mForm->fdui = mXForm;
}

void TBondValenceDialog::DoObject(FL_OBJECT *obj) {

	short 		i,ndx ;
	char 		text[256],*aName,atomName1[8],atomName2[8] ;
	float		distance ;
	float		bondConstant ;
	long		valence ;
	float		weight,bondWeight ;
	long		line1,line2,line3 ;

	BondStrengthData *theData = (BondStrengthData *)mData ;

	if (obj == mAtomList1) {
		if (!((line1 = fl_get_browser(mAtomList1)) > 0 &&
				(line2 = fl_get_browser(mAtomList2)) > 0))
			DeactivateButton(mAdd) ;
		
	}
	if (obj == mAtomList2) {
		if (!((line1 = fl_get_browser(mAtomList1)) > 0 &&
				(line2 = fl_get_browser(mAtomList2)) > 0))
			DeactivateButton(mAdd) ;
	}
	if (obj == mBondList) {
		// If something is selected after the click, activate DELETE and MODIFY atom
		if ((line3 = fl_get_browser(mBondList)) > 0) {				
			i = line3-1 ;	

			strcpy(atomName1,chemsymb[theData->bondStrength[i].firstAtomNdx]) ;
			strcpy(atomName2,chemsymb[theData->bondStrength[i].secondAtomNdx]) ;
			valence = theData->bondStrength[i].valenceSum  ;
			weight = theData->bondStrength[i].weight ;
			bondConstant = theData->bondStrength[i].bFactor ;
			distance = theData->bondStrength[i].maxBondLength ;

			sprintf(text,"%.2f",distance) ;
			fl_set_input(mRMax,text) ;

			sprintf(text,"%.2f",weight) ;
			fl_set_input(mBondWeight,text) ;

			sprintf(text,"%.2f",bondConstant) ;
			fl_set_input(mBondConstant,text) ;

			sprintf(text,"%d",valence) ;
			fl_set_input(mValence,text) ;

			// Find out which items should be selected in the atom lists
			fl_freeze_form(mXForm->mForm) ;
			fl_deselect_browser(mAtomList1) ;
			fl_deselect_browser(mAtomList2) ;
			for (i = 0; i < MyElements->number;i++) {
				ndx = MyElements->index[i] ;
				aName = chemsymb[ndx] ;
				if (strcmp(atomName1,aName)==0) {
					fl_select_browser_line(mAtomList1,i+1) ;
				}
				if (strcmp(atomName2,aName)==0) {
					fl_select_browser_line(mAtomList2,i+1) ;
				}
			}
			fl_unfreeze_form(mXForm->mForm) ;
			ActivateButton(mDelete) ;
			ActivateButton(mModify) ;
		}
		else {
			DeactivateButton(mDelete) ;
			DeactivateButton(mModify) ;
		}	
	}
	if (obj == mAdd) {
		if ((line1 = fl_get_browser(mAtomList1)) > 0) {
			if ((line2 = fl_get_browser(mAtomList2)) > 0) {
				// Get the text in the two lists
				strcpy(atomName1,fl_get_browser_line(mAtomList1,line1)) ;
				strcpy(atomName2,fl_get_browser_line(mAtomList2,line2)) ;
							
				// Get the bond-length and the weight
				strcpy(text,fl_get_input(mRMax)) ;
				distance = strlen(text) == 0 ? 3.0 : atof(text) ;

				strcpy(text,fl_get_input(mBondWeight)) ;
				weight = strlen(text) == 0 ? 0.01 : atof(text) ;

				strcpy(text,fl_get_input(mBondConstant)) ;
				bondConstant = strlen(text) == 0 ? 1.815 : atof(text) ;

				strcpy(text,fl_get_input(mValence)) ;
				valence = strlen(text) == 0 ? 1 : atoi(text) ;
							
				sprintf(text,"%-2.2s [%d] - %-2.2s : %.4f : %.2f",
					atomName1,valence,atomName2,bondConstant,bondWeight) ;
				
				fl_add_browser_line(mBondList,text) ;

				i = theData->numberOfBonds ;
				theData->bondStrength[i].firstAtomNdx = atomno(atomName1)-1 ;
				theData->bondStrength[i].secondAtomNdx = atomno(atomName2)-1 ;
				theData->bondStrength[i].maxBondLength = distance ;
				theData->bondStrength[i].valenceSum = valence ;
				theData->bondStrength[i].weight = weight ;
				theData->bondStrength[i].bFactor = bondConstant ;

				theData->numberOfBonds++ ;
			}
			else 
				DeactivateButton(mAdd) ;
		}
		else 
			DeactivateButton(mAdd) ;
	}
	if (obj == mDelete) {
		// Delete the bond that is defined (highlighted) in the Bond list
		if ((line3 = fl_get_browser(mBondList)) > 0) {
			// Delete this row
			fl_delete_browser_line(mBondList,line3) ;

			// Move the entries down in the bondstrength list
			long	nextEntry ;
			for(i=line3-1,nextEntry=line3; nextEntry < theData->numberOfBonds;
					i++,nextEntry++) {
				theData->bondStrength[i] = theData->bondStrength[nextEntry] ;
			}
			theData->numberOfBonds-- ;
		}
	}
	if (obj == mModify) {
		// Change parameters for the bond that is highlighted in the Bond list
		if ((line3 = fl_get_browser(mBondList)) > 0)
		   	if ((line1 = fl_get_browser(mAtomList1)) > 0)
				if ((line2 = fl_get_browser(mAtomList2)) > 0 ) {
							
			// Get the text in the two lists
			strcpy(atomName1,fl_get_browser_line(mAtomList1,line1)) ;
			strcpy(atomName2,fl_get_browser_line(mAtomList2,line2)) ;
							
			// Get the bond-length and the weight
			strcpy(text,fl_get_input(mRMax)) ;
			distance = strlen(text) == 0 ? 3.0 : atof(text) ;

			strcpy(text,fl_get_input(mBondWeight)) ;
			weight = strlen(text) == 0 ? 0.01 : atof(text) ;

			strcpy(text,fl_get_input(mBondConstant)) ;
			bondConstant = strlen(text) == 0 ? 1.815 : atof(text) ;

			strcpy(text,fl_get_input(mValence)) ;
			valence = strlen(text) == 0 ? 1 : atoi(text) ;
							
			sprintf(text,"%-2.2s [%d] - %-2.2s : %.4f : %.2f",
				atomName1,valence,atomName2,bondConstant,bondWeight) ;
			fl_replace_browser_line(mBondList,line3,text) ;

			i = line3-1 ;
			theData->bondStrength[i].firstAtomNdx = atomno(atomName1)-1 ;
			theData->bondStrength[i].secondAtomNdx = atomno(atomName2)-1 ;
			theData->bondStrength[i].maxBondLength = distance ;
			theData->bondStrength[i].valenceSum = valence ;
			theData->bondStrength[i].weight = weight ;
			theData->bondStrength[i].bFactor = bondConstant ;

		}
	}
}

void TBondValenceDialog::SetFormData(void *data) {
	
	BondStrengthData *theData = (BondStrengthData *)mData ;
	*theData = *((BondStrengthData *)data) ;

	short 		i,ndx ;
	char 		text[256],*aName,atomName1[8],atomName2[8] ;
	float		distance ;
	float		bondWeight,weight ;
	float		bondConstant ;
	long		valence ;

	
	fl_freeze_form(mXForm->mForm);
	fl_clear_browser(mAtomList1);
	fl_clear_browser(mAtomList2);
	fl_clear_browser(mBondList);
	for (i=0;i<MyElements->number;i++) {
		ndx = MyElements->index[i] ;
		aName = chemsymb[ndx] ;
		fl_add_browser_line(mAtomList1,aName) ;
		fl_add_browser_line(mAtomList2,aName) ;
	}
	for(i=0;i<theData->numberOfBonds;i++) {
		strcpy(atomName1,chemsymb[theData->bondStrength[i].firstAtomNdx]) ;
		strcpy(atomName2,chemsymb[theData->bondStrength[i].secondAtomNdx]) ;
		valence = theData->bondStrength[i].valenceSum ;
		bondConstant = theData->bondStrength[i].bFactor ;
		bondWeight = theData->bondStrength[i].weight ;
		sprintf(text,"%-2.2s [%d] - %-2.2s : %.4f : %.2f",
				atomName1,valence,atomName2,bondConstant,bondWeight) ;
		fl_add_browser_line(mBondList,text) ;
	}
	fl_select_browser_line(mAtomList1,1) ;
	fl_select_browser_line(mAtomList2,1) ;
	fl_unfreeze_form(mXForm->mForm);

	DeactivateButton(mModify) ;
	DeactivateButton(mDelete) ;

	if (theData->numberOfBonds == 0) {
		distance = 3.0 ;
		bondWeight = 1.0 ;
		bondConstant = 1.85 ;
		valence = 1 ;
		switch(gRelaxationParamRecord.fittingMethod) {
			case kCrossCorrelation:
			case kFTAmplCrossCorrelation:
				weight = 0.01 ;
				break ;
			case kChiSquared:
				weight = 0.1 ;
				break ;
		}
	}
	else {
		distance = theData->bondStrength[0].maxBondLength ;
		bondWeight = theData->bondStrength[0].weight ;
		weight = theData->totalWeight ;
		bondConstant = theData->bondStrength[0].bFactor ;
		valence = theData->bondStrength[0].valenceSum ;
	}
	sprintf(text,"%.2f",distance) ;
	fl_set_input(mRMax,text) ;
	sprintf(text,"%.3f",weight) ;
	fl_set_input(mWeight,text) ;
	sprintf(text,"%.3f",bondWeight) ;
	fl_set_input(mBondWeight,text) ;
	sprintf(text,"%.3f",bondConstant) ;
	fl_set_input(mBondConstant,text) ;
	sprintf(text,"%.d",valence) ;
	fl_set_input(mValence,text) ;

}

void TBondValenceDialog::GetFormData(void *data) {

	BondStrengthData *theData = (BondStrengthData *)data ;
	*theData = *((BondStrengthData *)mData) ;

	gCalculateBondStrengths = theData->numberOfBonds > 0 ;
}
