// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TMainPage.h"
#include "globals.h"
#include "funcdefs.h"
#include "devices.h"
#include "menudef.h"
#include "menu_index.h"
#include "globals.h"
#include <unistd.h>
#include <time.h>
#include "TWindow.h"

void DrawQuantitativePage() ;
void	ProcessModalEvent(XEvent *event) ;

TMainPage::TMainPage(TWindow *window) : TPage(window) {
		mHasAdvertised = false ;
}

TMainPage::~TMainPage() {
}

void TMainPage::Draw() {
	GrafPtr	tPort ;
	GetPort(&tPort) ;
	SetPort(mWindow) ;
	extern long LAYERED ;
	if (!mHasAdvertised) {
		BringToFront(mWindow) ;
		DoAdvertise() ; 
		return ;
	}
	else {
		EraseRect(&mWindow->GetPortRect()) ; //ClearDisplay() ;
	}

	if (CurrentMenuIs(SEL_OLD_FL)) {
		if (LAYERED)
			set_menu(SEL_LAY_STR) ;
		else
			set_menu(SEL_SNGL_STR) ;
		displ_menu() ;
      		erase_remain_menu() ;
		set_menu(SEL_OLD_FL);
		// displ_menu() ;
		displ_files() ;
	} else if (CurrentMenuIs(SEL_LAY_STR) || CurrentMenuIs(SEL_SNGL_STR)) {
		displ_menu() ;
		displ_files() ;
	} else if (CurrentMenuIs(MAIN_SNGL_STR)) {
		displ_var() ;
		displ_menu() ;
	} else if (CurrentMenuIs(CH_MAIN_PAR)) {
		if (LAYERED)
			set_menu(MAIN_LAY_STR) ;
		else
			set_menu(MAIN_SNGL_STR) ;
		displ_menu() ;
		displ_var() ;
		set_menu(CH_MAIN_PAR) ;
	} else if (CurrentMenuIs(DSPL_SOURCE)) {
		set_menu(DSPL_OPER) ;
		displ_menu() ;
		set_menu(DSPL_SOURCE) ;
		displ_menu() ;
	} else if (CurrentMenuIs(VIEW_ATOMS)) {
		displ_atoms(atom) ;
		displ_menu() ;
	} else if (CurrentMenuIs(VIEW_BASIS)) {
		displ_atoms(basis) ;
		displ_menu() ;
	} else if (CurrentMenuIs(VIEW_SYMOP)) {
		displ_symop() ;
		displ_menu() ;
	} else if (CurrentMenuIs(QUANTITATIVE)) {
		DrawQuantitativePage() ;
		
	}
	else {
		displ_menu() ;
	}
	SetPort(tPort) ;
}


void TMainPage::DoMouseDown(XEvent *event) {
 	long x,y,i ;
	/* This is the control window  				*/
	/* Check to see if the mouse is in one of the controls	*/
	x = event->xbutton.x ;
	y = event->xbutton.y ;
	x *= 2 ;
	y *= 2 ;
	y = Terminal.yres - y ;
	COMMAND	*theButtonPtr ;
   	for (i=0,gCurrentButton = *gCurrentMenu;
			*gCurrentButton !=&null;i++,gCurrentButton++) {
		theButtonPtr = *gCurrentButton ;
		if (!theButtonPtr->disabled)
			if (x >= theButtonPtr->xl && x <= theButtonPtr->xh &&
		    		y >= theButtonPtr->yl && y <= theButtonPtr->yh) {
				// We have to put in a check here to see which
				// button got pressed. We want to provide for
				// a help message to be output as well
				gCurrentCommand = gCurrentButton ;
				if (!theButtonPtr->status) { // Do A Visual Highlight
						on_highlight() ;
						off_highlight() ;
				}
				(*theButtonPtr->action)(i) ;
				break ;
			}
	}
}

void TMainPage::DoUpdate(XEvent *event) {
	GrafPtr tPort ;
	XRectangle updateRect ;
	Rect	thePortRect  ;

	GetPort(&tPort) ;

	updateRect.x = event->xexpose.x ;
	updateRect.y = event->xexpose.y ;
	updateRect.width = event->xexpose.width ;
	updateRect.height = event->xexpose.height ;
	XUnionRectWithRegion(&updateRect,mWindow->mUpdateRegion,mWindow->mUpdateRegion) ;
	if (event->xexpose.count == 0) {
		SetPort(mWindow) ;
		Rect theClipRect ;
		RegionRect(mWindow->mUpdateRegion,&theClipRect) ;
		InsetRect(&theClipRect,-6,-6) ;
		SetClip(theClipRect) ;
		Draw() ;
		thePortRect = mWindow->GetPortRect() ;
		SetClip(thePortRect) ;
		SetPort(tPort) ;
		XDestroyRegion(mWindow->mUpdateRegion) ;
		mWindow->mUpdateRegion = XCreateRegion() ;
	}
}

void TMainPage::WriteMessage(long x,long y, char *message) {
	write_message(message) ;
}

void TMainPage::DoAdvertise() {
  	/* -------------------------------------------------------------- */
  	/* Just write out message saying where the program originated and */
  	/* who wrote it. We will put it in here such that it can not be   */
  	/* removed by not starting up the associated command file         */
  	/* -------------------------------------------------------------- */
  	long y,x1,x2,y1,y2,xc ;
  	Rect tRect ;
  	char text[256] ;
	time_t	startTime,currentTime ;
	static	Boolean isAdvertising = false ;

   	EraseWindow(mWindow) ;
	XFlush(gDisplay) ;
 	x1 = 100 ;
  	x2 = 400 ;
  	y1 = 50 ;
  	y2 = 340 ;
  	y = y1+30 ;
  	strcpy(text,"NCEMSS") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 25 ;
  	strcpy(text,"X-Window Version 2.1   October - 2004") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 25 ;
  	strcpy(text,"This Program is Distributed by The") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"National Center for Electron Microscopy") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"at The Lawrence Berkeley National Laboratory") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"Berkeley CA 94720") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 30 ;
  	strcpy(text,"AUTHOR:") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ; 
  	strcpy(text,"ROAR KILAAS") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"NCEM/MSD LBNL") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"Tel: (510) 486-4618, Fax: (510) 486-5888") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 40 ;
  	strcpy(text,"Acknowledgement") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"The author wishes to thank Dr. M.A.O'Keefe") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;
  	y += 15 ;
  	strcpy(text,"for many useful ideas, suggestions and discussions.") ;
  	xc = (x1+x2)/2 - StringWidth(text)/2 ;
  	DrawText(xc,y,text) ;

	SetFgColor(gWhitePixel) ;
  	SetRect(&tRect,x1,y1,x2,y2) ;
  	FrameRect(&tRect) ;
	OffsetRect(&tRect,2,2) ;
	SetFgColor(gBlackPixel) ;
   	FrameRect(&tRect) ;
  	XFlush(gDisplay) ;
	if (!isAdvertising) {
		XEvent	event ;
		isAdvertising = true ;
		time(&startTime) ;
		FL_OBJECT *obj;
		do {
			obj = fl_do_forms() ;
			if (obj == FL_EVENT) {
				fl_XNextEvent(&event) ;
				ProcessModalEvent(&event) ;
			}
			time(&currentTime) ;		
		} while ((currentTime - startTime) < 3) ;
		mHasAdvertised = 1 ;
		Draw() ;
	}
}
