// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TPage.h"
#include "funcdefs.h"
#include <forms.h>
#include "globals.h"
#include "devices.h"
void	WriteStatus(char *text) ;	

KeySym	GetKey(XEvent *theEvent) ;

TPage * GetCurrentPage() ;
void	WriteStatus(char *text) {
	GetCurrentPage()->WriteStatus(text) ;
}

TPage::TPage(TWindow *window) {
	mObjectList = new TObjectList ;
	mEditTextList = new TEditTextList ;

	mHasOwnWindow = false ;
	mWindow = window ;
	if (mWindow == nil) mHasOwnWindow = true ;
	mCurrentTextEditField = nil ;
	mNumMenus = 0 ;
	mCurrentMenuID = 0 ;
	mDone = false ;
	mIsVisible = false ;
}

TPage::~TPage() {
	delete 	mObjectList ; 
	delete	mEditTextList ;
}

void	TPage::AddTextEditObject(TEditText *obj) {
	mObjectList->AddObject(obj) ; 
	obj->SetParentWindow(this) ;
	mEditTextList->AddObject(obj);
	mCurrentTextEditField = obj ;
}

void	TPage::Show() {

	mIsVisible = true ;
	SetPort(mWindow) ;
	EraseRect(&mWindow->GetPortRect()) ;
	Draw() ;
	XFlush(gDisplay) ;
}

void	TPage::Draw() {
	mObjectList->DrawObjects() ; 
	if (mNumMenus > 0) {
		DrawCurrentMenu() ;
	}
}

void	TPage::DrawCurrentMenu() {
	MENUS	itemPtr ;
	for (itemPtr = mCurrentMenuItemList[mCurrentMenuID] ;*itemPtr!=&null;itemPtr++) {
		display(*itemPtr) ;
		if ((*itemPtr)->status == ON)
			switch_on(*itemPtr) ;
	}
}

void TPage::DoModal() {
	XEvent event ;
	FL_OBJECT	*theFormsObject ;
	while (!mDone) {

		theFormsObject = fl_do_forms() ;
		if (theFormsObject == FL_EVENT) {
			fl_XNextEvent(&event) ;
			DoEvent(&event) ;
		}
	}
}

void	TPage::Select(TEditText *theText) {
	if (mCurrentTextEditField != theText) {
		mCurrentTextEditField->SetFocus(false) ;
	}
	mCurrentTextEditField = theText ;
	mCurrentTextEditField->SetFocus(true) ;
}

void	TPage::SelectAll(TEditText *theText) {
	if (mCurrentTextEditField != theText) {
		mCurrentTextEditField->SetFocus(false) ;
	}
	mCurrentTextEditField = theText ;
	mCurrentTextEditField->SelectAll() ;
	mCurrentTextEditField->SetFocus(true) ;
}

void	TPage::DoMouseDown(XEvent *theEvent) {
	Point thePoint ;
	TObject *theObject = nil ;
	TEditText	*editText ;

	if (mObjectList->GetNumberOfObjects() > 0) {
		thePoint.x = theEvent->xbutton.x ;
		thePoint.y = theEvent->xbutton.y ;
		theObject = FindObject(&thePoint) ;
		if (theObject != nil) {
			if ((editText=(TEditText *)mEditTextList->GetObject(&thePoint)) != nil) {
				if (mCurrentTextEditField != editText) {
					mCurrentTextEditField->SetFocus(false) ;
				}
				mCurrentTextEditField = editText ;
				mCurrentTextEditField->SetFocus(true) ;
			}
			DoMouseDownInObject(theObject,&thePoint) ;
		}
	}
	if (theObject == nil && mNumMenus > 0) {
		long x,y,i ;
		// Check to see if the mouse is in one of the controls
		x = theEvent->xbutton.x ;
		y = theEvent->xbutton.y ;
		x *= 2 ;
		y *= 2 ;
		y = Terminal.yres - y ;
		COMMAND	**theButtonH,*theButtonPtr ;
   		for (i=0,theButtonH = mCurrentMenuItemList[mCurrentMenuID];
				*theButtonH !=&null;i++,theButtonH++) {
			theButtonPtr = *theButtonH ;
			if (x >= theButtonPtr->xl && x <= theButtonPtr->xh &&
		    		y >= theButtonPtr->yl && y <= theButtonPtr->yh) {
				// We have to put in a check here to see which
				// button got pressed. We want to provide for
				// a help message to be output as well
				gCurrentButton = theButtonH ;
				gCurrentCommand = gCurrentButton ;
				if (!theButtonPtr->status) { // Do A Visual Highlight
					on_highlight() ;
					off_highlight() ;
				}
				(*theButtonPtr->action)(i) ;
				break ;
			}
		}
	}
}

void	TPage::DoKeyDown(XEvent *theEvent) {
	KeySym	theKey ;
	if (mCurrentTextEditField != nil) {
		theKey = mCurrentTextEditField->DoKeyDown(theEvent) ;
		DoKeyDownInObject(mCurrentTextEditField) ;
		if (theKey == XK_Tab) {
			mCurrentTextEditField->SetFocus(false) ;
			mCurrentTextEditField = 
					mEditTextList->GetNextField(mCurrentTextEditField) ;
			mCurrentTextEditField->SelectAll() ;
		}
	}
}

void	TPage::DoUpdate(XEvent *theEvent) {
	GrafPtr tPort ;
	XRectangle updateRect,clipRect ;
	Rect	thePortRect ; ;

	GetPort(&tPort) ;

	updateRect.x = theEvent->xexpose.x ;
	updateRect.y = theEvent->xexpose.y ;
	updateRect.width = theEvent->xexpose.width ;
	updateRect.height = theEvent->xexpose.height ;
	XUnionRectWithRegion(&updateRect,mWindow->mUpdateRegion,mWindow->mUpdateRegion) ;
	if (theEvent->xexpose.count == 0) {
		SetPort(mWindow) ;
		Rect theClipRect ;
		RegionRect(mWindow->mUpdateRegion,&theClipRect) ;
		InsetRect(&theClipRect,-6,-6) ;
		SetClip(theClipRect) ;
		Draw() ;
		thePortRect = mWindow->GetPortRect() ;
		SetClip(thePortRect) ;
		SetPort(tPort) ;
		XDestroyRegion(mWindow->mUpdateRegion) ;
		mWindow->mUpdateRegion = XCreateRegion() ;
	}
}

void	TPage::DoEvent(XEvent *theEvent) {
	switch(theEvent->type) {
		case ButtonPress:
			DoMouseDown(theEvent) ;
			break ;
		case KeyPress:
			DoKeyDown(theEvent) ;
			break ;
		case Expose:
			DoUpdate(theEvent) ;
			break ;
	}
}
