// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include <stdlib.h>
#include <math.h>
#include "types.h"
#include "menu_index.h"
#include "TImage.h"
#include "TButton.h"
#include "TMainPage.h"

#include "TRefineParameterProgressPage.h"
#include "globals.h"

void	SetPage(TPage *) ;
extern	TMainPage *gMainPage ;
void	PutMessage(char *) ;
void	ExitFromParameterRefinementProgress() ;

#include "funcdefs.h"	
#include "PhasegratingRecord.h"
#include "ProjCellRecord.h"
#include "displ.h"
#include "MslicRecord.h"
#include "storage.h"
#include "types.h"
#include "files.h"
#include "errors.h"
#include "globals.h"
#include "dfpdspl.h"
#include "symops.h"

void	SetPage(TPage *) ;
extern	TMainPage *gMainPage ;
void	PutMessage(char *) ;
void 	DrawQuantitativePage() ;
void	ExitFromParametersAnnealing() ;
void	CompareAndRefineParameters() ;
void	SaveRefinedParameters() ;

extern	TImage *gStandardDeviationImage ;
extern	TImage *gExperimentalImage ;
Boolean AdjustImageSize(TImage **image,long width,long height) ;

void DisplayImageInWindow(TImage *, TWindow *, Rect *rect,Boolean );


extern	TRefineParameterProgressPage *gRefineParameterProgressPage ;
extern	Boolean	gSaveLogFile ;
extern struct { float wavel ; short nslice ; float sigma ; } gMSRec ;

extern	UInt32	gRedPixel, gBluePixel, gGreenPixel ;
DisplRecord	gSavedDisplRec ;
MslicRecord	gSavedMslicRec ;

long 	gShiftN=0,gShiftM=0 ;
long	gIndex ;
extern	float	gDx,gDy,gDz,gDdw,gDocc ;

typedef enum {kCrossCorrelation,kChiSquared,kFTAmplCrossCorrelation } FittingMethod ;

typedef struct {
	float	start ;			// Starting Temperature
	float	end ;			// Ending Temperature
	float	current ;		// current temperature
	float	dTFraction ;	// Temperature decrement
	long		maxNumTriesPerTemperature ;
	long		maxNumGoodTries ;
} RelaxationTemperatureRecord ;

typedef struct {
	unsigned long	ID ;
	float	minValue ;
	float	maxValue ;
	float	startingValue[2] ;
	float	maxChangeInValue ;
	float	currentValue[3] ;	// We may need to store a magnitude and a direction or possible magnitude and components
} ParameterRecord ;

typedef struct {
	short	numFittingParameters ;
	ParameterRecord parameter[10] ;	// only space for 10 different parameters to fit
	RelaxationTemperatureRecord temperature ;
	float previousEnergy ;
	float thisEnergy ;
	float minEnergy ;
	long  numTriesThisTemperature ;
	FittingMethod fittingMethod ;
} ParameterFittingRecord ;

extern	ParameterFittingRecord gParameterFittingRecord ;

TRefineParameterProgressPage::TRefineParameterProgressPage(TWindow *window)
	: TPage(window) {
	TStaticText	*theText ;
	long	dText = 1 ;

	mEnergyFrame = new TFrame(4,250,506,350);
	AddObject(mEnergyFrame) ;

	mOffscreen = new TOffscreen(mWindow,&mEnergyFrame->GetRect()) ;

	mExperimentalImageFrame = new TUserRect(336,10,506,180);
	AddObject(mExperimentalImageFrame) ;

	mCurrentImageFrame = new TUserRect(160,10,330,180);
	AddObject(mCurrentImageFrame) ;

	mThickness = new TCheckBox(4,10,110,30,"Thickness [A]");
	AddObject(mThickness) ;

	mDefocus = new TCheckBox(4,30,110,50,"Defocus [A]");
	AddObject(mDefocus) ;

	mCrystalTilt = new TCheckBox(4,50,110,70,"Crystal Tilt [mrad]");
	AddObject(mCrystalTilt) ;

	mCs = new TCheckBox(4,90,110,110,"Cs [mm]");
	AddObject(mCs) ;

	mConvergence = new TCheckBox(4,110,110,130,"Conv.Ang. [mrad]");
	AddObject(mConvergence) ;

	mDelta = new TCheckBox(4,130,110,150,"Sread of Def. [A]");
	AddObject(mDelta) ;

	m2FoldAstig = new TCheckBox(4,150,110,170,"2-Fold Astig.");
	AddObject(m2FoldAstig) ;

	m3FoldAstig = new TCheckBox(4,195,110,215,"3-Fold Astig.");
	AddObject(m3FoldAstig) ;

	mCurrentThickness = new TStaticText(110,10-dText,160,30-dText,"text");
	AddObject(mCurrentThickness) ;

	mCurrentDefocus = new TStaticText(110,30-dText,160,50-dText,"text");
	AddObject(mCurrentDefocus) ;

	mCurrentTilt = new TStaticText(110,50-dText,160,70-dText,"text");
	AddObject(mCurrentTilt) ;

	theText = new TStaticText(4,70,45,85,"h',k',0:");
	AddObject(theText) ;

	mHK0 = new TStaticText(45,70,140,85,"text");
	AddObject(mHK0) ;

	//theText = new TStaticText(73,70,98,85,"h,k,l:");
	//AddObject(theText) ;

	//mHKL = new TStaticText(98,70,160,85,"text");
	//AddObject(mHKL) ;

	mCurrentCs = new TStaticText(110,90-dText,160,110-dText,"text");
	AddObject(mCurrentCs) ;

	mCurrentConvergence = new TStaticText(110,110-dText,160,130-dText,"text");
	AddObject(mCurrentConvergence) ;

	mCurrentDelta = new TStaticText(110,130-dText,160,150-dText,"text");
	AddObject(mCurrentDelta) ;

	theText = new TStaticText(4,210,60,230,"Magn. [A]:");
	AddObject(theText) ;

	theText = new TStaticText(70,210,140,230,"Angle with Hor:");
	AddObject(theText) ;

	mCurrent3FAstigValue = new TStaticText(25,225,75,245,"text");
	AddObject(mCurrent3FAstigValue) ;

	mCurrent3FAstigAngle = new TStaticText(85,225,135,245,"text");
	AddObject(mCurrent3FAstigAngle) ;

	theText = new TStaticText(4,165,60,185,"Magn. [A]:");
	AddObject(theText) ;

	theText = new TStaticText(70,165,150,185,"Angle with Hor:");
	AddObject(theText) ;

	mCurrent2FAstigValue = new TStaticText(25,180,75,200,"text");
	AddObject(mCurrent2FAstigValue) ;

	mCurrent2FAstigAngle = new TStaticText(85,180,135,200,"text");
	AddObject(mCurrent2FAstigAngle) ;

	mReturn = new TButton(400,355,500,380,"Return");
	AddObject(mReturn) ;

	mSaveParameters = new TButton(290,355,390,380,"Save Parameters");
	AddObject(mSaveParameters) ;

	theText = new TStaticText(300,230,345,250,"Log File:");
	AddObject(theText) ;

	mLogFileName = new TStaticText(345,230,500,250,"None");
	AddObject(mLogFileName) ;

	theText = new TStaticText(185,180,325,200,"Current Computed Image");
	AddObject(theText) ;

	theText = new TStaticText(360,180,470,200,"Experimental Image");
	AddObject(theText) ;

	mCurrentStatus = new TStaticText(50,355,130,375,"Not Running");
	AddObject(mCurrentStatus) ;

	theText = new TStaticText(160,210,300,228,"\"Energy\" vs \"Time/Temperature\"");
	AddObject(theText) ;

	theText = new TStaticText(300,210,400,228,"Current Temperature:");
	AddObject(theText) ;

	mCurrentTemperature = new TStaticText(400,210,460,228,"");
	AddObject(mCurrentTemperature) ;

	theText = new TStaticText(160,230,240,248,"Current \"Energy\":");
	AddObject(theText) ;

	mCurrentEnergy = new TStaticText(240,230,300,248,"");
	AddObject(mCurrentEnergy) ;

	theText = new TStaticText(140,355,260,375,"Press \"Control C\" to stop");
	AddObject(theText) ;

	theText = new TStaticText(4,355,50,375,"Status:");
	AddObject(theText) ;

	mExperimentalImage = nil ;
	mComputedImage = nil ;

	Initialize() ;
}

TRefineParameterProgressPage::~TRefineParameterProgressPage() {
	if (mExperimentalImage != nil) DeleteImage(mExperimentalImage) ;
	if (mComputedImage != nil) DeleteImage(mComputedImage) ;
	if (mOffscreen != nil) delete mOffscreen ;
}

void	TRefineParameterProgressPage::Draw() {
	EraseRect(&mWindow->GetPortRect()) ;
	TPage::Draw() ;
	DrawExperimentalImage() ;
	DrawCurrentComputedImage() ;
	DrawEnergyWindow() ;
}


void	TRefineParameterProgressPage::DrawEnergyWindow() {

	if (mOffscreen != nil) {
		Rect	evolutionRect = mEnergyFrame->GetRect() ;
		OffsetRect(&evolutionRect,-evolutionRect.left,-evolutionRect.top) ;
		CopyBits(mOffscreen,mWindow,&evolutionRect,&mEnergyFrame->GetRect()) ;
	}
}

void	TRefineParameterProgressPage::DoMouseDownInObject(TObject *obj,Point *pt) {
	if (obj == mReturn) {
		Hide() ;
		gDisplayRec = gSavedDisplRec ;
		gMsliceRec 	= gSavedMslicRec ;
		ExitFromParameterRefinementProgress() ;
	}
	else if (obj == mSaveParameters) {
		Hide() ;
		ExitFromParameterRefinementProgress() ;
		SaveRefinedParameters() ;		
	}

}

void	TRefineParameterProgressPage::Initialize() {
	char	text[256] ;

	if (gDisplayRec.tfs) {
		gDisplayRec.dfocus = (gDisplayRec.f1+gDisplayRec.f2)/2. ;
		gDisplayRec.tfs = false ;
	}
	else 
		gDisplayRec.dfocus = gDisplayRec.f1 ;
		
		
	if (gMsliceRec.tts) {
		gMsliceRec.thick = (gMsliceRec.t1+gMsliceRec.t2)/2. ;
		gMsliceRec.tts = false ;
	}
	else 
		gMsliceRec.thick = gMsliceRec.t1 ;
		
	gMsliceRec.iplot = false ;
	gDisplayRec.f2 = gDisplayRec.f1 ;
	gDisplayRec.df = 0. ;
	
	gMsliceRec.t2 = gMsliceRec.t1 ;
	gMsliceRec.dt = 0. ;

	mThickness->SetControlValue(false) ;
	mDefocus->SetControlValue(false) ;
	mCrystalTilt->SetControlValue(false) ;
	m2FoldAstig->SetControlValue(false) ;
	m3FoldAstig->SetControlValue(false) ;
	mCs->SetControlValue(false) ;
	mConvergence->SetControlValue(false) ;
	mDelta->SetControlValue(false) ;

	for(long k=0; k < gParameterFittingRecord.numFittingParameters; k++) {
		if (gParameterFittingRecord.parameter[k].ID == 'THIC') {
			mThickness->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'DEFO') {
			mDefocus->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'CTLT') {
			mCrystalTilt->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'ASTI') {
			m2FoldAstig->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'AST3') {
			m3FoldAstig->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'CS  ') {
			mCs->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'DIV ') {
			mConvergence->SetControlValue(true) ;
		}
		else if (gParameterFittingRecord.parameter[k].ID == 'DEL ') {
			mDelta->SetControlValue(true) ;
		}
	}
	sprintf(text,"%.2f",gMsliceRec.t1) ;
	mCurrentThickness->SetItemText(text) ;
				
	sprintf(text,"%.2f",gDisplayRec.f1) ;
	mCurrentDefocus->SetItemText(text) ;

	sprintf(text,"%.2f",gDisplayRec.cs) ;
	mCurrentCs->SetItemText(text) ;

	sprintf(text,"%.2f",gDisplayRec.div) ;
	mCurrentConvergence->SetItemText(text) ;

	sprintf(text,"%.2f",gDisplayRec.del) ;
	mCurrentDelta->SetItemText(text) ;
	
	float tiltx,tilty,tilt ;			
	tiltx = gMsliceRec.kxin/gProjCell->a ;
	tilty = -gMsliceRec.kxin*gProjCell->ast*gProjCell->csga+gMsliceRec.kyin*gProjCell->bst ;
	tilt = 1000.0*asin(sqrt(tiltx*tiltx+tilty*tilty)*gMSRec.wavel) ; // mrad
	sprintf(text,"%.2f",tilt) ;
	mCurrentTilt->SetItemText(text) ;
				
	sprintf(text,"%.1f, %.1f, 0",gMsliceRec.kxin,gMsliceRec.kyin) ;
	mHK0->SetItemText(text) ;
				
	// Convert to 3D hkl
	//float	h3=0.,k3=0.,l3=0. ;
	//Convert2DhklTo3Dhkl(gMsliceRec.kxin,gMsliceRec.kyin,0.0,&h3,&k3,&l3) ;
	//sprintf(text,"%.1f,%.1f,%.1f",h3,k3,l3) ;
	//mHKL->SetItemText(text) ;
				
	sprintf(text,"%.2f",gDisplayRec.TwoFoldAstigMag) ;
	mCurrent2FAstigValue->SetItemText(text) ;
	sprintf(text,"%.2f",gDisplayRec.TwoFoldAstigAngle) ;
	mCurrent2FAstigAngle->SetItemText(text) ;
				
	sprintf(text,"%.0f",gDisplayRec.ThreeFoldAstigMag) ;
	mCurrent3FAstigValue->SetItemText(text) ;
	sprintf(text,"%.2f",gDisplayRec.ThreeFoldAstigAngle) ;
	mCurrent3FAstigAngle->SetItemText(text) ;


	short	ix,iy ;
	Rect	evolutionRect = mEnergyFrame->GetRect() ;

	// Initialize the energy window
	if (mOffscreen != nil) {
		OffsetRect(&evolutionRect,-evolutionRect.left,-evolutionRect.top) ;
		SetPort(mOffscreen) ;
	}
	FrameRect(&evolutionRect) ;
	InsetRect(&evolutionRect,1,1) ;
	SetFgColor(gWhitePixel) ;
	PaintRect(&evolutionRect) ;
	SetFgColor(gBlackPixel) ;
				
	// Allow for 10% growth in energy
	// and minimum to be 0
	if (gParameterFittingRecord.thisEnergy > 0.0) {	
		mMaxValueOfEnergy = gParameterFittingRecord.thisEnergy * 1.1 ;
		mMinValueOfEnergy = 0.0 ;
	}
	else {
		mMaxValueOfEnergy = 0.9*gParameterFittingRecord.thisEnergy ;
		mMinValueOfEnergy = 10.0*gParameterFittingRecord.thisEnergy ;  
	}
	mScaleFactor = (evolutionRect.bottom - evolutionRect.top) / 
			(mMaxValueOfEnergy-mMinValueOfEnergy) ;
	ix = evolutionRect.left ;
	iy = evolutionRect.bottom - 
			(gParameterFittingRecord.thisEnergy-mMinValueOfEnergy)*mScaleFactor ;
	if (iy < evolutionRect.top) iy = evolutionRect.top ;
	if (iy > evolutionRect.bottom) iy = evolutionRect.bottom ;
	mCurrentEnergyPt.x = ix ;
	mCurrentEnergyPt.y = iy ;
	mNumDisplayIndex = 1 ;
	if (mOffscreen != nil) {
		SetPort(mWindow) ;
		CopyBits(mOffscreen,mWindow,&evolutionRect,&mEnergyFrame->GetRect()) ;
	}


	mReturn->Disable() ;
	mCurrentEnergy->SetItemText("") ;
	mSaveParameters->Disable() ;

}

void	TRefineParameterProgressPage::DebugCurrentParameters() {
	char 	text[256] ;
		
	// Update the parameter window
					
	mCurrentStatus->SetItemText("Running !") ;
	if (gParameterFittingRecord.parameter[gIndex].ID == 'THIC') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentThickness->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'DEFO') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentDefocus->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'CS  ') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentCs->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'DEL ') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentDelta->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'DIV ') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentConvergence->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'CTLT') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentTilt->SetItemText(text) ;
						
		sprintf(text,"%.2f, %.2f, 0",
			gParameterFittingRecord.parameter[gIndex].currentValue[1],
			gParameterFittingRecord.parameter[gIndex].currentValue[2]) ;
		mHK0->SetItemText(text) ;
						
		// Convert to 3D hkl
		//float h3,k3,l3 ;
		//Convert2DhklTo3Dhkl(gParameterFittingRecord.parameter[gIndex].currentValue[1],gParameterFittingRecord.parameter[gIndex].currentValue[2],0.0,&h3,&k3,&l3) ;
		//sprintf(text,"%.2f,%.2f,%.2f",h3,k3,l3) ;
		//mHKL->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'ASTI') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrent2FAstigValue->SetItemText(text) ;
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[1]) ;
		mCurrent2FAstigAngle->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'AST3') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrent3FAstigValue->SetItemText(text) ;
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[1]) ;
		mCurrent3FAstigAngle->SetItemText(text) ;
	}
	sprintf(text,"%f",gParameterFittingRecord.temperature.current) ;
	mCurrentTemperature->SetItemText(text) ;
	XFlush(gDisplay) ;
}

void	TRefineParameterProgressPage::UpdateCurrentParameters() {
	char 	text[256] ;
		
	// Update the parameter window
					
	mCurrentStatus->SetItemText("Running !") ;
	if (gParameterFittingRecord.parameter[gIndex].ID == 'THIC') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentThickness->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'DEFO') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentDefocus->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'CS  ') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentCs->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'DEL ') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentDelta->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'DIV ') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentConvergence->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'CTLT') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrentTilt->SetItemText(text) ;
						
		sprintf(text,"%.2f, %.2f, 0",
			gParameterFittingRecord.parameter[gIndex].currentValue[1],
			gParameterFittingRecord.parameter[gIndex].currentValue[2]) ;
		mHK0->SetItemText(text) ;
						
		// Convert to 3D hkl
		float h3,k3,l3 ;
		//Convert2DhklTo3Dhkl(gParameterFittingRecord.parameter[gIndex].currentValue[1],gParameterFittingRecord.parameter[gIndex].currentValue[2],0.0,&h3,&k3,&l3) ;
		//sprintf(text,"%.2f,%.2f,%.2f",h3,k3,l3) ;
		//mHKL->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'ASTI') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrent2FAstigValue->SetItemText(text) ;
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[1]) ;
		mCurrent2FAstigAngle->SetItemText(text) ;
	}
	else if (gParameterFittingRecord.parameter[gIndex].ID == 'AST3') {
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[0]) ;
		mCurrent3FAstigValue->SetItemText(text) ;
		sprintf(text,"%.2f",
			gParameterFittingRecord.parameter[gIndex].currentValue[1]) ;
		mCurrent3FAstigAngle->SetItemText(text) ;
	}
															
	sprintf(text,"%f",gParameterFittingRecord.temperature.current) ;
	mCurrentTemperature->SetItemText(text) ;

	// Write current energy to the window
	UpdateCurrentEnergy(gParameterFittingRecord.thisEnergy) ;
	XFlush(gDisplay) ;
}

void	TRefineParameterProgressPage::UpdateCurrentEnergy(float energy) {
	char 	text[256] ;
	short	ix,iy ;
	Rect	evolutionRect = mEnergyFrame->GetRect() ;

	if (mOffscreen != nil) {
		OffsetRect(&evolutionRect,-evolutionRect.left,-evolutionRect.top) ;
		SetPort(mOffscreen) ;
	}

	ix = mNumDisplayIndex%(evolutionRect.right-evolutionRect.left)
		+evolutionRect.left ;
	iy = evolutionRect.bottom - 
		(gParameterFittingRecord.thisEnergy-mMinValueOfEnergy)*mScaleFactor ;
	if (iy > evolutionRect.bottom-3) {
		if (gParameterFittingRecord.thisEnergy > 0.0) {	
			mMaxValueOfEnergy = gParameterFittingRecord.thisEnergy * 1.2 ;
			mMinValueOfEnergy = 0.0 ;
		}
		else {
			mMaxValueOfEnergy = 0.8*gParameterFittingRecord.thisEnergy ;
			mMinValueOfEnergy = 10.0*gParameterFittingRecord.thisEnergy ;  
		}
		mScaleFactor = (evolutionRect.bottom - evolutionRect.top) / 
			(mMaxValueOfEnergy-mMinValueOfEnergy) ;
		iy = evolutionRect.bottom - 
			(gParameterFittingRecord.thisEnergy-mMinValueOfEnergy)*mScaleFactor ;
		SetFgColor(gRedPixel) ;
		MoveTo(ix,evolutionRect.bottom) ;
		LineTo(ix,evolutionRect.top) ;
		mCurrentEnergyPt.x = ix ;
		mCurrentEnergyPt.y = iy ;
	} 
	if (iy < evolutionRect.top) iy = evolutionRect.top ;
	if (iy > evolutionRect.bottom) iy = evolutionRect.bottom ;
					
	if (ix == evolutionRect.left) {	// We just did a jump to go back
		SetFgColor(gWhitePixel) ;
		PaintRect(&evolutionRect) ;
		mCurrentEnergyPt.x = ix ;
		mCurrentEnergyPt.y = iy ;
	}
	else {	
		SetFgColor(gBluePixel) ;
		DrawLine(mCurrentEnergyPt.x,mCurrentEnergyPt.y,ix,iy) ;
		mCurrentEnergyPt.x = ix ;
		mCurrentEnergyPt.y = iy ;
	}
	SetFgColor(gBlackPixel) ;

	if (mOffscreen != nil) {
		SetPort(mWindow) ;
		CopyBits(mOffscreen,mWindow,&evolutionRect,&mEnergyFrame->GetRect()) ;
	}

	sprintf(text,"%f",energy) ;

	if (gParameterFittingRecord.fittingMethod == kFTAmplCrossCorrelation) {
		sprintf(text,"%.4f, Shift: (%d,%d)",
			energy,gShiftN,gShiftM) ;
	}
	else {
		sprintf(text,"%.4f",energy) ;
	}
	mCurrentEnergy->SetItemText(text) ;
	mNumDisplayIndex++ ;
}

void	TRefineParameterProgressPage::DisplayLatestParameters() {
	char 	text[256] ;
	float	tilt,tiltx,tilty,h3,k3,l3 ;

	// Make sure we have the latest values in the window

	sprintf(text,"%.2f",gMsliceRec.t1) ;
	mCurrentThickness->SetItemText(text) ;
		
	sprintf(text,"%.2f",gDisplayRec.f1) ;
	mCurrentDefocus->SetItemText(text) ;
		
	tiltx = gMsliceRec.kxin/gProjCell->a ;
	tilty = -gMsliceRec.kxin*gProjCell->ast*gProjCell->csga + 
		gMsliceRec.kyin*gProjCell->bst ;
	tilt = 1000.0*asin(sqrt(tiltx*tiltx+tilty*tilty)*gMSRec.wavel) ; 
	sprintf(text,"%.2f",tilt) ;
	mCurrentTilt->SetItemText(text) ;
		
	sprintf(text,"%.2f, %.2f, 0",gMsliceRec.kxin,gMsliceRec.kyin) ;
	mHK0->SetItemText(text) ;
		
	// Convert to 3D hkl
	//Convert2DhklTo3Dhkl(gMsliceRec.kxin,gMsliceRec.kyin,0.0,&h3,&k3,&l3) ;
	h3 = 0. ; k3 = 0. ; l3 = 0. ;
	//sprintf(text,"%.2f,%.2f,%.2f",h3,k3,l3) ;
	//mHKL->SetItemText(text) ;

	sprintf(text,"%.2f",gDisplayRec.TwoFoldAstigMag) ;
	mCurrent2FAstigValue->SetItemText(text) ;
	sprintf(text,"%.2f",gDisplayRec.TwoFoldAstigAngle) ;
	mCurrent2FAstigAngle->SetItemText(text) ;

	sprintf(text,"%f",gParameterFittingRecord.temperature.current) ;
	mCurrentTemperature->SetItemText(text) ;
				
	// Write current energy to the window
	UpdateCurrentEnergy(gParameterFittingRecord.thisEnergy) ;

	if (gAbortStatus) {
		mCurrentStatus->SetItemText("Cancelled by User!") ;
	}
	else {
		mCurrentStatus->SetItemText("Finished !") ;
	}
	mReturn->Enable() ;
	mSaveParameters->Enable() ;

}

void	TRefineParameterProgressPage::DrawExperimentalImage() {
	Rect	tRect ;
	extern	unsigned long	gWhitePixel,gBlackPixel ;

	tRect = mExperimentalImageFrame->GetRect() ;
	InsetRect(&tRect,1,1) ;
	SetFgColor(gDarkerGray) ;
	FillRect(&tRect) ;
	PenSize(2,2) ;
	SetFgColor(gBlackPixel) ;
	DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	SetFgColor(gWhitePixel) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
	PenSize(1,1) ;
	SetFgColor(gBlackPixel) ;
	InsetRect(&tRect,4,4) ;
	if (mExperimentalImage == nil) {
		mExperimentalImage = CopyImage(gExperimentalImage) ;
		if (mExperimentalImage != nil) {
			DisplayImageInWindow(mExperimentalImage,mWindow,&tRect,false );
		}
	}
	else {
		DisplayImageInWindow(mExperimentalImage,mWindow,&tRect,false );
	}
	XFlush(gDisplay) ;
}

void	TRefineParameterProgressPage::SetLogFileName(char *text) {
	mLogFileName->SetItemText(text) ;
}

void	TRefineParameterProgressPage::DrawCurrentComputedImage() {
	Rect	tRect ;
	extern	unsigned long	gWhitePixel,gBlackPixel ;

	if (mComputedImage == nil) return ;
	tRect = mCurrentImageFrame->GetRect() ;
	InsetRect(&tRect,1,1) ;
	SetFgColor(gDarkerGray) ;
	FillRect(&tRect) ;
	PenSize(2,2) ;
	SetFgColor(gBlackPixel) ;
	DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	SetFgColor(gWhitePixel) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
	PenSize(1,1) ;
	SetFgColor(gBlackPixel) ;
	InsetRect(&tRect,4,4) ;
	DisplayImageInWindow(mComputedImage,mWindow,&tRect,true );
	XFlush(gDisplay) ;

}

void	TRefineParameterProgressPage::UpdateComputedImage(TImage *image) {
	Rect	tRect ;
	extern	unsigned long	gWhitePixel,gBlackPixel ;

	if (image == nil) return ;
	tRect = mCurrentImageFrame->GetRect() ;
	InsetRect(&tRect,1,1) ;
	if (mComputedImage == nil) {
		SetFgColor(gDarkerGray) ;
		FillRect(&tRect) ;
		PenSize(2,2) ;
		SetFgColor(gBlackPixel) ;
		DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
		DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
		SetFgColor(gWhitePixel) ;
		DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
		DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
		PenSize(1,1) ;
		SetFgColor(gBlackPixel) ;
	}
	InsetRect(&tRect,4,4) ;
	if (mComputedImage != nil) {
		DeleteImage(mComputedImage) ;
		mComputedImage = nil ;
	}
	mComputedImage = CopyImage(image) ;
	if (mComputedImage != nil) {
		DisplayImageInWindow(mComputedImage,mWindow,&tRect,true );
	}
	XFlush(gDisplay) ;
}

void	ExitFromParameterRefinementProgress() {
	delete gRefineParameterProgressPage ;
	gRefineParameterProgressPage = nil ;
	set_menu(QUANTITATIVE) ;
	ClearDisplay() ;
	SetPage(gMainPage) ;
	DrawQuantitativePage() ;
}

