// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TRefineParametersPage.h"
#include <stdlib.h>
#include <math.h>
#include "menu_index.h"
#include "TButton.h"
#include "TMainPage.h"

#include "TRefineParameterProgressPage.h"
#include "TSetImageAreaPage.h" 

#include "funcdefs.h"	
#include "PhasegratingRecord.h"
#include "ProjCellRecord.h"
#include "displ.h"
#include "MslicRecord.h"
#include "storage.h"
#include "types.h"
#include "files.h"
#include "errors.h"
#include "globals.h"
#include "dfpdspl.h"
#include "symops.h"
void 	run_help(char *) ;

extern	DisplRecord	gSavedDisplRec ;
extern	MslicRecord	gSavedMslicRec ;

TSetImageAreaPage *gSetImageAreaPage = nil ;

TRefineParameterProgressPage *gRefineParameterProgressPage = nil ;

void	SetPage(TPage *) ;
extern	TMainPage *gMainPage ;
void	PutMessage(char *) ;
void 	DrawQuantitativePage() ;
void	ExitFromParametersAnnealing() ;
void	CompareAndRefineParameters() ;
void	SaveRefinedParameters() ;


extern	Boolean	gSaveLogFile ;
extern struct { float wavel ; short nslice ; float sigma ; } gMSRec ;

typedef enum {kCrossCorrelation,kChiSquared,kFTAmplCrossCorrelation } FittingMethod ;

typedef struct {
	float	start ;			// Starting Temperature
	float	end ;			// Ending Temperature
	float	current ;		// current temperature
	float	dTFraction ;	// Temperature decrement
	long		maxNumTriesPerTemperature ;
	long		maxNumGoodTries ;
} RelaxationTemperatureRecord ;

typedef struct {
	unsigned long	ID ;
	float	minValue ;
	float	maxValue ;
	float	startingValue[2] ;
	float	maxChangeInValue ;
	float	currentValue[3] ;	// We may need to store a magnitude and a direction or possible magnitude and components
} ParameterRecord ;

typedef struct {
	short	numFittingParameters ;
	ParameterRecord parameter[10] ;	// only space for 10 different parameters to fit
	RelaxationTemperatureRecord temperature ;
	float previousEnergy ;
	float thisEnergy ;
	float minEnergy ;
	long  numTriesThisTemperature ;
	FittingMethod fittingMethod ;
} ParameterFittingRecord ;

extern	ParameterFittingRecord gParameterFittingRecord ;

TRefineParametersPage::TRefineParametersPage(TWindow *window)  
		: TPage(window) {

	TGroupBox	*theGroupBox ;
	TStaticText	*theText ;

	mHelp = new TButton(490,2,510,20,"H") ;
	AddObject(mHelp) ;
	
	theGroupBox = new TGroupBox(15,30,205,180,"Temperature");
	AddObject(theGroupBox) ;

	theGroupBox = new TGroupBox(220,30,500,320,"Parameters to Refine");
	AddObject(theGroupBox) ;

	theGroupBox = new TGroupBox(15,190,205,280,"Matching Method");
	AddObject(theGroupBox) ;

	theGroupBox = new TGroupBox(20,110,200,170,"Per Degree of Freedom");
	AddObject(theGroupBox) ;

	mSetComparisonRegion = new TButton(15,290,205,320,"Set Image Comparison Region");
 	AddObject(mSetComparisonRegion) ;
	mRefine = new TButton(415,350,495,380,"Refine");
	AddObject(mRefine) ;
	mCancel = new TButton(310,350,400,380,"Cancel");
	AddObject(mCancel) ;

	mCreateLogFile = new TCheckBox(15,330,290,350,"Create Log File of \"Energy\" vs. Time");
	AddObject(mCreateLogFile) ;

	mThickness = new TCheckBox(225,50,360,70,"Thickness [A]");
	AddObject(mThickness) ;
	mDefocus = new TCheckBox(225,80,360,100,"Defocus [A]");
	AddObject(mDefocus) ;
	mCs = new TCheckBox(225,110,360,130,"Spherical Abb. [mm]");
	AddObject(mCs) ;
	mDelta = new TCheckBox(225,140,360,160,"Spread Of Defocus [A]");
	AddObject(mDelta) ;
	mConvergence = new TCheckBox(225,170,360,190,"Convergence [mrad]");
	AddObject(mConvergence) ;
	mCrystalTilt = new TCheckBox(225,215,340,235,"Crystal Tilt [mrad]");
	AddObject(mCrystalTilt) ;
	m2FoldAstigmatism = new TCheckBox(225,245,340,265,"2F Astigmatism [A]");
	AddObject(m2FoldAstigmatism) ;
	m3FoldAstigmatism = new TCheckBox(225,275,340,295,"3F Astigmatism [A]");
	AddObject(m3FoldAstigmatism) ;


	mStartingTemp = new TEditText(125,35,195,55,"Starting","0.01000");
	mEndingTemp = new TEditText(125,55,195,75,"Ending","0.00010");
	mFraction = new TEditText(125,75,195,95,"Fractional Change","0.9000");
	mAttemptsPerTemp = new TEditText(130,125,190,145,"# Attempts per Temp","25");
	mSuccesfulAttempts = new TEditText(130,145,190,165,"# Sucessful Attempts","5");

	mStartThickness =new TEditText(370,50,430,70,"");
	mStartDefocus = new TEditText(370,80,430,100,"");
	mStartingCs = new TEditText(370,110,430,130,"");
	mStartingDelta = new TEditText(370,140,430,160,"");
	mStartingConvergence = new TEditText(370,170,430,190,"");
	mDeltaThickness = new TEditText(440,50,490,70,"","5");
	mDeltaDefocus = new TEditText(440,80,490,100,"","500");
	mDeltaCs = new TEditText(440,110,490,130,"",".1");
	mDeltaDelta = new TEditText(440,140,490,160,"","10");
	mDeltaConvergence = new TEditText(440,170,490,190,"",".1");
	mStartCrystalTiltValue = new TEditText(350,215,390,235,"");
	mStartCrystalTiltAngle = new TEditText(390,215,430,235,"");
	mDeltaCrystalTiltValue = new TEditText(440,215,490,235,"",".1");
	mStarting2FAstigValue = new TEditText(350,245,390,265,"");
	mStarting2FAstigAngle = new TEditText(390,245,430,265,"");
	mDelta2FAstigValue = new TEditText(440,245,490,265,"","10");
	mStarting3FAstigValue = new TEditText(350,275,390,295,"");
	mStarting3FAstigAngle = new TEditText(390,275,430,295,"");
	mDelta3FAstigValue = new TEditText(440,275,490,295,"","1000");

	AddTextEditObject(mDelta3FAstigValue) ;
	AddTextEditObject(mStarting3FAstigAngle) ;
	AddTextEditObject(mStarting3FAstigValue) ;
	AddTextEditObject(mDelta2FAstigValue) ;
	AddTextEditObject(mStarting2FAstigAngle) ;
	AddTextEditObject(mStarting2FAstigValue) ;
	AddTextEditObject(mDeltaCrystalTiltValue) ;
	AddTextEditObject(mStartCrystalTiltAngle) ;
	AddTextEditObject(mStartCrystalTiltValue) ;
	AddTextEditObject(mDeltaConvergence) ;
	AddTextEditObject(mStartingConvergence) ;
	AddTextEditObject(mDeltaDelta) ;
	AddTextEditObject(mStartingDelta) ;
	AddTextEditObject(mDeltaCs) ;
	AddTextEditObject(mStartingCs) ;
	AddTextEditObject(mDeltaDefocus) ;
	AddTextEditObject(mStartDefocus) ;
	AddTextEditObject(mDeltaThickness) ;
	AddTextEditObject(mStartThickness) ;
	AddTextEditObject(mSuccesfulAttempts) ;
	AddTextEditObject(mAttemptsPerTemp) ;
	AddTextEditObject(mFraction) ;
	AddTextEditObject(mEndingTemp) ;
	AddTextEditObject(mStartingTemp) ;

	mCCC = new TRadioButton(20,200,160,220,"Cross Correlation");
	AddObject(mCCC) ;
	mChiSquare = new TRadioButton(20,225,160,245,"Chi Square");
	AddObject(mChiSquare) ;
	mFourierAmplitudes = new TRadioButton(20,250,160,270,"Fourier Amplitudes");
	AddObject(mFourierAmplitudes) ;

	theText = new TStaticText(370,30,440,50,"Start Value");
	AddObject(theText) ;
	theText = new TStaticText(440,30,490,50,"Change*");
	AddObject(theText) ;
	theText = new TStaticText(350,195,390,215,"Magn.");
	AddObject(theText) ;
	theText = new TStaticText(390,195,430,215,"Angle");
	AddObject(theText) ;

}

TRefineParametersPage::~TRefineParametersPage() {
}

void	TRefineParametersPage::Initialize() { 
	char	text[256],textAngle[256],textDelta[256] ;
	float	tilt,tiltangle,theta,tiltx,tilty ;
	Boolean	fitThickness,fitTilt,fitDefocus,fitAstigmatism,fitCs,fitDelta,
			fitDiv,fitBackground,fit3Astigmatism ;
	double	pi ;
	
	pi = 2.*asin(1.0) ;
	if (gParameterFittingRecord.numFittingParameters == -1) {
		// First time in. Let us set up some parameters
		gParameterFittingRecord.numFittingParameters = 0 ;
		gParameterFittingRecord.fittingMethod = kCrossCorrelation ;
		gParameterFittingRecord.temperature.start = 0.01 ;
		gParameterFittingRecord.temperature.end = 0.0001 ;
		gParameterFittingRecord.temperature.dTFraction = 0.9 ;
		gParameterFittingRecord.temperature.maxNumTriesPerTemperature = 25 ;
		gParameterFittingRecord.temperature.maxNumGoodTries = 5 ;
	}
 
	gParameterFittingRecord.numFittingParameters = 0 ; // Every time we come in
	mThickness->SetControlValue(false) ;
	mDefocus->SetControlValue(false) ;
	mCrystalTilt->SetControlValue(false) ;
	m2FoldAstigmatism->SetControlValue(false) ;
	m3FoldAstigmatism->SetControlValue(false) ;
	mCs->SetControlValue(false) ;
	mConvergence->SetControlValue(false) ;
	mDelta->SetControlValue(false) ;
	
  	sprintf(text,"%.5f",gParameterFittingRecord.temperature.start) ;
	mStartingTemp->SetEditText(text) ;
	mStartingTemp->SelectAll() ;
	
  	sprintf(text,"%.5f",gParameterFittingRecord.temperature.end) ;
	mEndingTemp->SetEditText(text) ;
		
  	sprintf(text,"%.4f",gParameterFittingRecord.temperature.dTFraction) ;
	mFraction->SetEditText(text) ;
	
  	sprintf(text,"%d",
		gParameterFittingRecord.temperature.maxNumTriesPerTemperature) ;
	mAttemptsPerTemp->SetEditText(text) ;
	
  	sprintf(text,"%d",gParameterFittingRecord.temperature.maxNumGoodTries) ;
	mSuccesfulAttempts->SetEditText(text) ;

	fitThickness = false ;
	fitTilt = false ;
	fitDefocus = false ;
	fitAstigmatism = false ;
	fitCs = false ;
	fitDelta = false ;
	fitDiv = false ;
	fitBackground = false ;
	fit3Astigmatism = false ;
  	for(long i=0;i<gParameterFittingRecord.numFittingParameters;i++) {

		if (fabs(gParameterFittingRecord.parameter[i].startingValue[0]) > 100.) 
			sprintf(text,"%.0f",gParameterFittingRecord.parameter[i].startingValue[0]) ;
		else if (fabs(gParameterFittingRecord.parameter[i].startingValue[0]) > 10.) 
			sprintf(text,"%.1f",gParameterFittingRecord.parameter[i].startingValue[0]) ;
		else 
			sprintf(text,"%.2f",gParameterFittingRecord.parameter[i].startingValue[0]) ;
		if (gParameterFittingRecord.parameter[i].ID == 'CTLT' 
				|| gParameterFittingRecord.parameter[i].ID == 'ASTI'
					|| gParameterFittingRecord.parameter[i].ID == 'AST3') {
			if (fabs(gParameterFittingRecord.parameter[i].startingValue[1]) > 100.) 
				sprintf(textAngle,"%.0f",
					gParameterFittingRecord.parameter[i].startingValue[1]) ;
			else if (fabs(gParameterFittingRecord.parameter[i].startingValue[1]) > 10.) 
				sprintf(textAngle,"%.1f",
					gParameterFittingRecord.parameter[i].startingValue[1]) ;
			else 
				sprintf(textAngle,"%.2f",
					gParameterFittingRecord.parameter[i].startingValue[1]) ;
		}
		if (fabs(gParameterFittingRecord.parameter[i].maxChangeInValue) > 100.) 
			sprintf(textDelta,"%.0f",
				gParameterFittingRecord.parameter[i].maxChangeInValue) ;
		else if (fabs(gParameterFittingRecord.parameter[i].maxChangeInValue) > 10.) 
			sprintf(textDelta,"%.1f",
				gParameterFittingRecord.parameter[i].maxChangeInValue) ;
		else 
			sprintf(textDelta,"%.2f",
				gParameterFittingRecord.parameter[i].maxChangeInValue) ;

		switch (gParameterFittingRecord.parameter[i].ID) {
			case 'THIC':
				mStartThickness->SetEditText(text) ;
				mDeltaThickness->SetEditText(textDelta) ;
				mThickness->SetControlValue(true) ;
				fitThickness = true ;
				break ;
			case 'DEFO':
				mStartDefocus->SetEditText(text) ;
				mDeltaDefocus->SetEditText(textDelta) ;
				mDefocus->SetControlValue(true) ;
				fitDefocus = true ;
				break ;
			case 'CTLT':
				mStartCrystalTiltValue->SetEditText(text) ;
				mStartCrystalTiltAngle->SetEditText(textDelta) ;
				mDeltaCrystalTiltValue->SetEditText(textAngle) ;
				mCrystalTilt->SetControlValue(true) ;
				fitTilt = true ;
				break ;
			case 'ASTI':
				mStarting2FAstigValue->SetEditText(text) ;
				mStarting2FAstigAngle->SetEditText(textAngle) ;
				mDelta2FAstigValue->SetEditText(textDelta) ;
				m2FoldAstigmatism->SetControlValue(true) ;
				fitAstigmatism = true ;
				break ;
			case 'AST3':
				mStarting3FAstigValue->SetEditText(text) ;
				mStarting3FAstigAngle->SetEditText(textAngle) ;
				mDelta3FAstigValue->SetEditText(textDelta) ;
				m3FoldAstigmatism->SetControlValue(true) ;
				fit3Astigmatism = true ;
				break ;
			case 'CS  ':
				mStartingCs->SetEditText(text) ;
				mDeltaCs->SetEditText(textDelta) ;
				mCs->SetControlValue(true) ;
				fitCs = true ;
				break ;
			case 'DIV ':
				mStartingConvergence->SetEditText(text) ;
				mDeltaConvergence->SetEditText(textDelta) ;
				mConvergence->SetControlValue(true) ;
				fitDiv = true ;
				break ;
			case 'DEL ':
				mStartingDelta->SetEditText(text) ;
				mDeltaDelta->SetEditText(textDelta) ;
				mDelta->SetControlValue(true) ;
				fitDelta = true ;
				break ;
			case 'BCKG':
				fitBackground = true ;
				break ;
		}
	}

  	
  	if (!fitThickness) {
  		// Put in current thickness
		sprintf(text,"%.0f",gMsliceRec.t1) ;
		mStartThickness->SetEditText(text) ;
  	}
  	if (!fitDefocus) {
  		// Put in current defocus
		sprintf(text,"%.0f",gDisplayRec.f1) ;
		mStartDefocus->SetEditText(text) ;
  	}
  	if (!fitTilt) {
  		// Put in current tilt
		tilt = sqrt(gMsliceRec.kxin*gProjCell->ast*gMsliceRec.kxin*
			gProjCell->ast+gMsliceRec.kyin*gProjCell->bst*gMsliceRec.kyin
			*gProjCell->bst -
			2.0*gMsliceRec.kxin*gProjCell->ast*gMsliceRec.kyin*
			gProjCell->bst*gProjCell->csga) ;
		tiltx = gMsliceRec.kxin/gProjCell->a ;
		tilty = -gMsliceRec.kxin*gProjCell->ast*gProjCell->csga + 
			gMsliceRec.kyin*gProjCell->bst ;
		tiltangle = 1000.*asin(tilt*gMSRec.wavel) ;		// in mrad	
		// Get the angle with the x-axis
		if (tiltx != 0.) 
			theta = atan(tilty/tiltx) ;
		else {
			if (tilty != 0.)
				theta = pi/2 ;
			else 
				theta = 0. ;
		}
		if (tiltx < 0.)
			theta += pi ;
		theta *= 180./pi ;
		sprintf(text,"%.0f",tiltangle) ;
		mStartCrystalTiltValue->SetEditText(text) ;
		sprintf(text,"%.0f",theta) ;
		mStartCrystalTiltAngle->SetEditText(text) ;
  	}
  	if (!fitAstigmatism) {
  		// Put in current astigmatism
		sprintf(text,"%.2f",gDisplayRec.TwoFoldAstigMag) ;
		mStarting2FAstigValue->SetEditText(text) ;
		sprintf(text,"%.2f",gDisplayRec.TwoFoldAstigAngle) ;
		mStarting2FAstigAngle->SetEditText(text) ;
  	}
  	if (!fit3Astigmatism) {
  		// Put in current astigmatism
		sprintf(text,"%.2f",gDisplayRec.ThreeFoldAstigMag) ;
		mStarting3FAstigValue->SetEditText(text) ;
		sprintf(text,"%.2f",gDisplayRec.ThreeFoldAstigAngle) ;
		mStarting3FAstigAngle->SetEditText(text) ;
  	}
  	if (!fitCs) {
  		// Put in current Spherical abberation
		sprintf(text,"%.2f",gDisplayRec.cs) ;
		mStartingCs->SetEditText(text) ;
  	}
  	if (!fitDelta) {
  		// Put in current spread of defocus
		sprintf(text,"%.0f",gDisplayRec.del) ;
		mStartingDelta->SetEditText(text) ;
  	}
  	if (!fitDiv) {
  		// Put in current convergence angle
		sprintf(text,"%.3f",gDisplayRec.div) ;
		mStartingConvergence->SetEditText(text) ;
  	}
	
	switch (gParameterFittingRecord.fittingMethod) {
		case kChiSquared:
			gParameterFittingRecord.temperature.start = 0.1 ;
			gParameterFittingRecord.temperature.end = 0.001 ;
			mChiSquare->SetControlValue(true) ;
			break ;
		case kCrossCorrelation:
			gParameterFittingRecord.temperature.start = 0.01 ;
			gParameterFittingRecord.temperature.end = 0.0001 ;
			mCCC->SetControlValue(true) ;
			break ;
		case kFTAmplCrossCorrelation:
			gParameterFittingRecord.temperature.start = 0.01 ;
			gParameterFittingRecord.temperature.end = 0.0001 ;
			mFourierAmplitudes->SetControlValue(true) ;
			break ;	
	}

	mCreateLogFile->SetControlValue(gSaveLogFile) ;

}

void	TRefineParametersPage::Draw() {
	EraseRect(&mWindow->GetPortRect()) ;
	TPage::Draw() ;
}

void	TRefineParametersPage::DoMouseDownInObject(TObject *obj,Point *pt) {
	char	text[256] ;
	double	pi ;
	
	pi = 2.*asin(1.0) ;
	if (gParameterFittingRecord.numFittingParameters == -1) {
		// First time in. Let us set up some parameters
		gParameterFittingRecord.numFittingParameters = 0 ;
		gParameterFittingRecord.fittingMethod = kCrossCorrelation ;
		gParameterFittingRecord.temperature.start = 0.01 ;
		gParameterFittingRecord.temperature.end = 0.0001 ;
		gParameterFittingRecord.temperature.dTFraction = 0.9 ;
		gParameterFittingRecord.temperature.maxNumTriesPerTemperature = 25 ;
		gParameterFittingRecord.temperature.maxNumGoodTries = 5 ;
	}
	if (obj->DoMouseDown(pt)) {
		if (obj == mHelp) {
			run_help("TRefineParameters") ;
			return ;
		}
		if (obj == mCCC) {
			mChiSquare->SetControlValue(false) ;
			mFourierAmplitudes->SetControlValue(false) ;
			sprintf(text,"%.5f",0.01) ;
			mStartingTemp->SetEditText(text) ;
			sprintf(text,"%.4f",0.0001) ;
			mEndingTemp->SetEditText(text) ;
		}
		else if (obj == mChiSquare) {
			mCCC->SetControlValue(false) ;
			mFourierAmplitudes->SetControlValue(false) ;
			sprintf(text,"%.5f",0.1) ;
			mStartingTemp->SetEditText(text) ;
			sprintf(text,"%.4f",0.001) ;
			mEndingTemp->SetEditText(text) ;
		}
		else if (obj == mFourierAmplitudes) {
			mCCC->SetControlValue(false) ;
			mChiSquare->SetControlValue(false) ;
			sprintf(text,"%.5f",0.01) ;
			mStartingTemp->SetEditText(text) ;
			sprintf(text,"%.4f",0.0001) ;
			mEndingTemp->SetEditText(text) ;
		}
		else if (obj == mSetComparisonRegion) {
			if (gSetImageAreaPage == nil) {
				gSetImageAreaPage = new TSetImageAreaPage(gControlWindow) ;
			}
			if (gSetImageAreaPage != nil) {
				gSetImageAreaPage->Show() ;
				SetPage(gSetImageAreaPage) ;
			}
		}
		else if (obj == mRefine) {
			// Save the old data in case it needs to be restored later
			gSavedDisplRec = gDisplayRec ;
			gSavedMslicRec = gMsliceRec ;


			// Get and assemble all the relevant parameters and
			// do the refinement

			gSaveLogFile = mCreateLogFile->GetControlValue() ;

			if (mCCC->GetControlValue()) {
				gParameterFittingRecord.fittingMethod = kCrossCorrelation ;
			}
			else if (mChiSquare->GetControlValue()) {
				gParameterFittingRecord.fittingMethod = kChiSquared ;
			}
			else {
				gParameterFittingRecord.fittingMethod = kFTAmplCrossCorrelation ;
			}
			

			mStartingTemp->GetEditText(text) ;
			gParameterFittingRecord.temperature.start = atof(text) ;
		
			mFraction->GetEditText(text) ;
			gParameterFittingRecord.temperature.dTFraction = atof(text) ;
		
			mEndingTemp->GetEditText(text) ;
			gParameterFittingRecord.temperature.end = atof(text) ;
			
			mAttemptsPerTemp->GetEditText(text) ;
			gParameterFittingRecord.temperature.maxNumTriesPerTemperature = atoi(text) ;
		
			mSuccesfulAttempts->GetEditText(text) ;
			gParameterFittingRecord.temperature.maxNumGoodTries = atoi(text) ;

			mStartThickness->GetEditText(text) ;
			gMsliceRec.tts = false ;
			gMsliceRec.thick = gMsliceRec.t1 = atof(text) ;
						
			mStartDefocus->GetEditText(text) ;
			gDisplayRec.tfs = false ;
			gDisplayRec.dfocus = gDisplayRec.f1 = atof(text) ;

			mStartingCs->GetEditText(text) ;
			gDisplayRec.cs = atof(text) ;
			mStartingDelta->GetEditText(text) ;
			gDisplayRec.del = atof(text) ;
			mDeltaConvergence->GetEditText(text) ;
			gDisplayRec.div = atof(text) ;

			gParameterFittingRecord.numFittingParameters = 0 ;
			if (mThickness->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'THIC' ;
			
				mStartThickness->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mDeltaThickness->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;
			
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 1000.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (mDefocus->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'DEFO' ;
			
				mStartDefocus->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mDeltaDefocus->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;

				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = -100000. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 100000.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (mCs->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'CS  ' ;
			
				mStartingCs->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mDeltaCs->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;

				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 100.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (mDelta->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'DEL ' ;
			
				mStartingDelta->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mDeltaDelta->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;

				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 1000.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (mConvergence->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'DIV ' ;
			
				mStartingConvergence->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mDeltaConvergence->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;

				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 10.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (mCrystalTilt->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'CTLT' ;
			
				mStartCrystalTiltValue->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mStartCrystalTiltAngle->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[1] = atof(text) ;
			
				mDeltaCrystalTiltValue->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;
			
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 10000.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (m2FoldAstigmatism->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'ASTI' ;
			
				mStarting2FAstigValue->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mStarting2FAstigAngle->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[1] = atof(text) ;
			
				mDelta2FAstigValue->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;
			
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 100000.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (m3FoldAstigmatism->GetControlValue()) {
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].ID = 'AST3' ; 
			
				mStarting3FAstigAngle->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[0] = atof(text) ;
			
				mStarting3FAstigAngle->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].startingValue[1] = atof(text) ;
			
				mDelta3FAstigValue->GetEditText(text) ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxChangeInValue = atof(text) ;
			
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].minValue = 0. ;
				gParameterFittingRecord.parameter[gParameterFittingRecord.numFittingParameters].maxValue = 100000.0 ;
				gParameterFittingRecord.numFittingParameters++ ;
			}
			if (gParameterFittingRecord.numFittingParameters == 0) {
				PutMessage("No fitting parameters have been specified") ;
				return ;
			}

			Hide() ;
			// Save whatever old data that we have
			CompareAndRefineParameters() ;
		}		
		else if (obj == mCancel) {
			Hide() ;
			ExitFromParametersAnnealing() ;
		}		
	}
} 

void	ExitFromParametersAnnealing() {
	set_menu(QUANTITATIVE) ;
	ClearDisplay() ;
	SetPage(gMainPage) ;
	DrawQuantitativePage() ;
}
