// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TResultTable.h"
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "XDrivers.h"
#include "TFile.h"
void	PutMessage(char *) ;

TResultTable::TResultTable()
{
	mExactMatch = 1.0 ;
	mGoodMatch = 0.9 ;
	mImage = NULL ;
}

TResultTable::TResultTable(Int16 nrows,Int16 ncols,Int16 cellSize,Int16 cellWidth,Int16 cellHeight)
	: TTable(nrows,ncols,cellSize,cellWidth,cellHeight,"XCF Coefficient") 
{
	mImage = NULL ;
}
TResultTable::TResultTable(TImage *result) 
	: TTable(result->mHeight,result->mWidth,4,60,20,"XCF Coefficient") 
{
	long row,col ;
	char	text[5] ;
	TableCellT theCell ;
	float *valPtr ;
	
	for(row=1;row <= result->mHeight;row++) {
		sprintf(text,"T%d",row) ;
		SetRowLabel(row,text) ;
	}
	for(col=1;col <= result->mWidth;col++) {
		sprintf(text,"F%d",col) ;
		SetColLabel(col,text) ;
	}
	
	valPtr = result->mData ;
	for(theCell.row=1;theCell.row <= result->mHeight;theCell.row++) {
		for (theCell.col = 1;theCell.col <= result->mWidth;theCell.col++) {
			SetCellData(theCell,(void *)valPtr++) ;
		}
	}
	mExactMatch = 1.0 ;
	mGoodMatch = 0.9 ;
	mImage = result ;
}

void	
TResultTable::SetSuccessValues(float exactMatch,float goodMatch) {
	float *valPtr,bestValue ;
	TableCellT theCell ;
	
	mExactMatch = exactMatch ;
	mGoodMatch = goodMatch ; 
	
	if (mImage == NULL)
		return ;
		
	valPtr = mImage->mData ;
	bestValue  = mExactMatch > mGoodMatch ? -10000000. : 10000000. ;
	
	for(theCell.row=1;theCell.row <= mImage->mHeight;theCell.row++) {
		for (theCell.col = 1;theCell.col <= mImage->mWidth;theCell.col++) {
			if (mExactMatch > mGoodMatch) {
				if (*valPtr > bestValue) {
					bestValue = *valPtr ;
					mBestCell = theCell ;
				}
			}
			else {
				if (*valPtr < bestValue) {
					bestValue = *valPtr ;
					mBestCell = theCell ;
				}
			}
			valPtr++ ;
		}
	}
}


void	
TResultTable::SetDefocusLabel(short col,float value) {
	char	text[5] ;
	
	sprintf(text,"%.0f",value) ;
	SetColLabel(col,text) ;
}

void	
TResultTable::SetThicknessLabel(short row,float value) {
	char	text[5] ;
	
	sprintf(text,"%.0f",value) ;
	SetRowLabel(row,text) ;
}

void TResultTable::DrawCellValue(
	TableCellT &inCell,
	Rect &cellFrame)
{
	char 	str[64] ;
	float	fvalue,value ;
	RGBColor theFgColor ;
	
	//TextFont(applFont);
	TextSize(10);
	MoveTo(cellFrame.left + 4, cellFrame.bottom - 4);
	
		
	FetchCellData(inCell,(void *)&fvalue) ;
	
	theFgColor.red = theFgColor.green = theFgColor.blue = 0 ;
	if (mExactMatch > mGoodMatch) {	// As in cross-correlation
		if (fvalue > mGoodMatch) {
			theFgColor.red = 30000 + (fvalue-mGoodMatch)/(mExactMatch-mGoodMatch)*25535 ;
			FormRGBForeColor(&theFgColor) ;
		}
	}
	else {	// Lower is better
		if (fvalue < mGoodMatch) {
			theFgColor.red = 30000 + (fvalue-mGoodMatch)/(mExactMatch-mGoodMatch)*25535 ;
			FormRGBForeColor(&theFgColor) ;
		}
	}
	if (inCell.col == mBestCell.col && inCell.row == mBestCell.row) {
			//TextFace(bold) ;
	}
	value = fabs(fvalue) ;
	if (value <= 1.0) {
		sprintf(str,"%.4f",fvalue) ;
	}
	else if (value < 10.0) {
		sprintf(str,"%.3f",fvalue) ;
	}
	else if (value < 100.0) {
		sprintf(str,"%.2f",fvalue) ;
	}
	else if (value < 1000.0) {
		sprintf(str,"%.1f",fvalue) ;
	}
	else {
		sprintf(str,"%6.2E",fvalue) ;
	}
	DrawString(str) ;
	
	theFgColor.red = 0 ;
	TextFace(0) ;
	FormRGBForeColor(&theFgColor) ;
}

void TResultTable::DrawColLabelText(
	Int16 colNumber) 
{
	char str[64] ;
	mColLabels.FetchItemAt(colNumber,(void *)str) ;
	DrawString(str) ;
}
void TResultTable::DrawRowLabelText(
	Int16 rowNumber) 
{
	char str[64] ;
	mRowLabels.FetchItemAt(rowNumber,(void *)str) ;
	DrawString(str) ;
}

void TResultTable::Save() 
{
	// ------------------------------------------------------------------ 
	// Put up the standard file dialog Box to determine the file name 
	// ------------------------------------------------------------------ 
	FSInfo 	theFInfo ;
	
	strcpy(theFInfo.mFileName,"") ;
	ChooseFile("Save Table data as:","*",theFInfo) ;
	
	if (!theFInfo.mReply) 
		return ;

	char buffer[256] ;
	long i ;
	long inByteCount ;
	char str[64] ;
	TableCellT theCell ;
	float	fvalue ;
	FILE	*fp ;

	if ((fp = fopen(theFInfo.mFileName,"w")) == nil) {
		sprintf(buffer,"Unable to open the file %s",theFInfo.mFileName) ;
		PutMessage(buffer) ;
		return ;
	}
	
	for(i=1;i<mCols;i++) {
		mColLabels.FetchItemAt(i,(void *)str) ;
		if (i == 1) {
			sprintf(buffer,"T/df\t%s\t",str) ;
		}
		else {
			sprintf(buffer,"%s\t",str) ;
		}
		inByteCount = strlen(buffer) ;
		fwrite(buffer,inByteCount,1,fp) ;
	}
	mColLabels.FetchItemAt(mCols,(void *)str) ;
	sprintf(buffer,"%s\r",str) ;
	inByteCount = strlen(buffer) ;
	fwrite(buffer,inByteCount,1,fp) ;
	
	for(i=1;i<=mRows;i++) {
		mRowLabels.FetchItemAt(i,(void *)str) ;
		sprintf(buffer,"%s\t",str) ;
		inByteCount = strlen(buffer) ;
		fwrite(buffer,inByteCount,1,fp) ;
		theCell.row = i ;
		for(theCell.col=1;theCell.col<mCols;theCell.col++) {
			FetchCellData(theCell,(void *)&fvalue) ;
			sprintf(buffer,"%.4f\t",fvalue) ;
			inByteCount = strlen(buffer) ;
			fwrite(buffer,inByteCount,1,fp) ;
		}
		theCell.col = mCols ;
		FetchCellData(theCell,(void *)&fvalue) ;
		sprintf(buffer,"%.4f\r",fvalue) ;
		inByteCount = strlen(buffer) ;
		fwrite(buffer,inByteCount,1,fp) ;
	}
	fclose(fp) ;
}


