// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "menu_index.h"
#include "TImage.h"
#include "TButton.h"
#include "TSetImageAreaPage.h"
#include "TMainPage.h"

#include "globals.h"
#include "TRefineParametersPage.h" 

void	SetPage(TPage *) ;
extern	TMainPage *gMainPage ;
void	PutMessage(char *) ;

extern	TRefineParametersPage	*gRefineParametersPage ;

extern	TImage *gExperimentalImage ;
Boolean AdjustImageSize(TImage **image,long width,long height) ;

void DisplayImageInWindow(TImage *, TWindow *, Rect *rect,Boolean );

extern	unsigned char tool_bits[11][60] ;

extern	FRect	gComparisonRect ;
extern	Boolean	gCompareEntireImage ;

void	ExitFromSetComparisonArea() ;

TSetImageAreaPage::TSetImageAreaPage(TWindow *window) 
		: TPage(window) {

	TGroupBox 	*grpBox ;
	TIcon	*icon ;


	icon = new TIcon(20,20,tool_bits[0]) ;
	mArrowButton	= new TPushButton(340,20,364,44,icon) ;
	AddObject(mArrowButton) ;

	icon = new TIcon(20,20,tool_bits[1]) ;
	mRectButton	= new TPushButton(340,60,364,84,icon) ;
	AddObject(mRectButton) ;

	icon = new TIcon(20,20,tool_bits[4]) ;
	mMagnifyButton	= new TPushButton(340,100,364,124,icon) ;
	AddObject(mMagnifyButton) ;

	icon = new TIcon(20,20,tool_bits[9]) ;
	mHandButton	= new TPushButton(340,140,364,164,icon) ;
	AddObject(mHandButton) ;

	mCurrentTool = kComparePointer ;
	mArrowButton->SetState(true) ;

	grpBox = new TGroupBox(340,190,500,300,"Active Comparison Area") ;
	AddObject(grpBox) ;

	mCompareEntireImage = new TRadioButton(350,210,480,230,"Compare Entire Image") ;
	mCompareEntireImage->SetControlValue(true) ;
	AddObject(mCompareEntireImage) ;

	mCompareSelection	= new TRadioButton(350,235,480,255,"Compare Selection") ;
	mCompareSelection->Disable() ;
	AddObject(mCompareSelection) ;

	mSetNewSelection = new TButton(400,260,490,280,"Update Selection") ;
	mSetNewSelection->Disable() ;
	AddObject(mSetNewSelection) ;

	mResetView = new TButton(400,140,490,164,"Reset View") ;
	mResetView->Disable() ;
	AddObject(mResetView) ;

	mExpImageFrame = new TUserRect(20,20,330,330) ;
	AddObject(mExpImageFrame) ;
	mExpImageFrame->Enable() ;

	mImageRect = mExpImageFrame->GetRect() ;

	mOK	= new TButton(410,340,500,370,"OK") ;
	AddObject(mOK) ;

	mCancel = new TButton(300,340,390,370,"Cancel") ;
	AddObject(mCancel) ;

	mExpImage = nil ;

	mImageSelectionRect = gComparisonRect ;

	mImageSelection = nil ;
	mImageCenter.x = (mImageRect.right-mImageRect.left)/2 ;
	mImageCenter.y =  (mImageRect.bottom-mImageRect.top)/2 ;
	mImageMagnification = 1.0 ;
	
	mIsInsideActiveRect = false ;

}
 
TSetImageAreaPage::~TSetImageAreaPage() {
}

void TSetImageAreaPage::Draw() {
	EraseRect(&mWindow->GetPortRect()) ;
	TPage::Draw() ;
	DrawExpImageDisplayArea() ;
}


void 
TSetImageAreaPage::DrawImageComparisonRect() {
	long left,top,right,bottom ;
	float width,height ;
	Rect activeRect ;
	extern	unsigned long	gBlackPixel,gRedPixel ;

	width = mActualImageRect.right - mActualImageRect.left ;
	height = mActualImageRect.bottom - mActualImageRect.top ;
	left = mActualImageRect.left + mImageSelectionRect.left*width ;
	right = mActualImageRect.left + mImageSelectionRect.right*width ;
	top = mActualImageRect.top + mImageSelectionRect.top*height ;
	bottom = mActualImageRect.top + mImageSelectionRect.bottom*height ;
	SetRect(&activeRect,left,top,right,bottom) ;
	
	SetFgColor(gRedPixel) ;
	FrameRect(&activeRect) ;
	SetFgColor(gBlackPixel) ;

}

void TSetImageAreaPage::DrawExpImageDisplayArea() {
	Rect	tRect ;
	extern	unsigned long	gWhitePixel,gBlackPixel ;

	DrawExperimentalImage() ;

	tRect = mExpImageFrame->GetRect() ;
	PenSize(2,2) ;
	SetFgColor(gBlackPixel) ;
	DrawLine(tRect.left,tRect.bottom,tRect.left,tRect.top) ;
	DrawLine(tRect.left,tRect.top,tRect.right,tRect.top) ;
	SetFgColor(gWhitePixel) ;
	DrawLine(tRect.right,tRect.top,tRect.right,tRect.bottom) ;
	DrawLine(tRect.right,tRect.bottom,tRect.left,tRect.bottom) ;
	PenSize(1,1) ;
	SetFgColor(gBlackPixel) ;
}

void	TSetImageAreaPage::DrawExperimentalImage() {
	Rect	tRect ;
	short 	x,y,width,height ;
	extern	unsigned long	gWhitePixel,gBlackPixel ;

	tRect = mExpImageFrame->GetRect() ;

	if (gExperimentalImage == nil) {
		SetFgColor(gDarkerGray) ;
		FillRect(&tRect) ;
		SetFgColor(gWhitePixel) ;
		width = StringWidth("NO IMAGE LOADED") ;
		x = (tRect.left+tRect.right-width)/2 ;
		y = (tRect.top+tRect.bottom)/2 + 10 ;
		DrawText(x,y,"NO IMAGE LOADED") ;
		SetFgColor(gBlackPixel) ;
		return ;
	}

	InsetRect(&tRect,6,6) ;
	if (mExpImage == nil) {
		mExpImage = CopyImage(gExperimentalImage) ;
		if (mExpImage != nil) {
			float	ratio1,ratio2,fraction ;
			short	width = mExpImage->mWidth ;
			short	height = mExpImage->mHeight ;
			ratio1 = tRect.right-tRect.left ;
			ratio1 /= width ;
			ratio2 = tRect.bottom-tRect.top ;
			ratio2 /= height ;
			fraction = ratio2 < ratio1 ? ratio2 : ratio1 ;
			width *= fraction ;
			height *= fraction ;
			mActualImageRect.left = near(mImageCenter.x - width/2) ;	
			mActualImageRect.top = near(mImageCenter.y - height/2) ;
			mActualImageRect.right = mActualImageRect.left + width ;
			mActualImageRect.bottom = mActualImageRect.top + height ;
			if (AdjustImageSize(&mExpImage,width,height)) {
				CreateImageOffscreen(mExpImage,mWindow) ;
			}
		}
	}

	if (mExpImage == nil) return ;

	InsetRect(&mImageRect,1,1) ;
	TStOffscreen offScreen(mWindow,&mImageRect) ;
	SetFgColor(gDarkerGray) ;
	FillRect(&gCurrentPort->GetPortRect()) ;
	SetFgColor(gBlackPixel) ;
	InsetRect(&mImageRect,-1,-1) ;

	DisplayImageInRect(mExpImage,&mActualImageRect);
	DrawImageComparisonRect() ;

}

void	TSetImageAreaPage::DoMouseDownInObject(TObject *obj,Point *pt) {
	if (obj->DoMouseDown(pt)) {
		if (obj == mCompareEntireImage) {
			if (mCompareSelection->GetControlValue()) 
				mCompareSelection->SetControlValue(false) ;
				if (mImageSelection != nil) {
					delete mImageSelection ;
					mImageSelection = nil ;
				}
				mImageSelectionRect.left = 0. ;
				mImageSelectionRect.top = 0. ;
				mImageSelectionRect.right = 1. ;
				mImageSelectionRect.bottom = 1. ;
				DrawExperimentalImage() ;
				mCompareSelection->Disable() ;
				mSetNewSelection->Disable() ;
		}
		else if (obj == mCompareSelection) {
			if (mCompareEntireImage->GetControlValue()) 
				mCompareEntireImage->SetControlValue(false) ;
				if (mImageSelection != nil) {
					Rect selectionRect ;
					mImageSelection->GetSelectionRect(selectionRect) ;
					// Compute the actual rectange mImageSelectionRect
					Rect sizeOfImage = mActualImageRect ;
					OffsetRect(&sizeOfImage,mImageRect.left,mImageRect.top) ;
					float x1,y1,x2,y2,width,height ;
					width = sizeOfImage.right - sizeOfImage.left ;
					height = sizeOfImage.bottom - sizeOfImage.top ;
					x1 = selectionRect.left - sizeOfImage.left ;
					x1 = x1 < 0. ? 0. : x1/width ;
					y1 = selectionRect.top - sizeOfImage.top ;
					y1 = y1 < 0. ? 0. : y1/height ;
					x2 = sizeOfImage.right - selectionRect.right ;
					x2 = x2 < 0. ? 1. : 1.0 - x2/width ;
					y2 = sizeOfImage.bottom - selectionRect.bottom ;
					y2 = y2 < 0. ? 1. : 1.0 - y2/height ;
					mImageSelectionRect.left = x1 ;
					mImageSelectionRect.top = y1 ;
					mImageSelectionRect.right = x2 ;
					mImageSelectionRect.bottom = y2 ;
					
					delete mImageSelection ;
					mImageSelection = nil ;
					DrawExperimentalImage() ;
					mSetNewSelection->Disable() ;
				}
		}
		else if (obj == mSetNewSelection) {
				if (mImageSelection != nil) {
					Rect selectionRect ;
					mImageSelection->GetSelectionRect(selectionRect) ;
					// Compute the actual rectange mImageSelectionRect
					Rect sizeOfImage = mActualImageRect ;
					OffsetRect(&sizeOfImage,mImageRect.left,mImageRect.top) ;
					float x1,y1,x2,y2,width,height ;
					width = sizeOfImage.right - sizeOfImage.left ;
					height = sizeOfImage.bottom - sizeOfImage.top ;
					x1 = selectionRect.left - sizeOfImage.left ;
					x1 = x1 < 0. ? 0. : x1/width ;
					y1 = selectionRect.top - sizeOfImage.top ;
					y1 = y1 < 0. ? 0. : y1/height ;
					x2 = sizeOfImage.right - selectionRect.right ;
					x2 = x2 < 0. ? 1. : 1.0 - x2/width ;
					y2 = sizeOfImage.bottom - selectionRect.bottom ;
					y2 = y2 < 0. ? 1. : 1.0 - y2/height ;
					mImageSelectionRect.left = x1 ;
					mImageSelectionRect.top = y1 ;
					mImageSelectionRect.right = x2 ;
					mImageSelectionRect.bottom = y2 ;
					
					delete mImageSelection ;
					mImageSelection = nil ;
					DrawExperimentalImage() ;
					mSetNewSelection->Disable() ;
					mCompareSelection->Disable() ;
				}
		}
		else if (obj == mArrowButton) {
			switch(mCurrentTool) {
				case kCompareSelection:
					mRectButton->SetState(false) ;
					break ;
				case kCompareMagnify:
					mMagnifyButton->SetState(false) ;
					break ;
				case kCompareHand:
					mHandButton->SetState(false) ;
					break ;
			}
			mCurrentTool = kComparePointer ;
			mArrowButton->SetState(true) ;
		}
		else if (obj == mRectButton) {
			switch(mCurrentTool) {
				case kComparePointer:
					mArrowButton->SetState(false) ;
					break ;
				case kCompareMagnify:
					mMagnifyButton->SetState(false) ;
					break ;
				case kCompareHand:
					mHandButton->SetState(false) ;
					break ;
			}
			mCurrentTool = kCompareSelection ;
			mRectButton->SetState(true) ;
		}
		else if (obj == mMagnifyButton) {
			switch(mCurrentTool) {
				case kCompareSelection:
					mRectButton->SetState(false) ;
					break ;
				case kComparePointer:
					mArrowButton->SetState(false) ;
					break ;
				case kCompareHand:
					mHandButton->SetState(false) ;
					break ;
			}
			mCurrentTool = kCompareMagnify ;
			mMagnifyButton->SetState(true) ;
		}
		else if (obj == mHandButton) {
			switch(mCurrentTool) {
				case kCompareSelection:
					mRectButton->SetState(false) ;
					break ;
				case kCompareMagnify:
					mMagnifyButton->SetState(false) ;
					break ;
				case kComparePointer:
					mArrowButton->SetState(false) ;
					break ;
			}
			mCurrentTool = kCompareHand ;
			mHandButton->SetState(true) ;
		}
		else if (obj == mResetView) {
				mImageCenter.x = (mImageRect.right-mImageRect.left)/2 ;
				mImageCenter.y =  (mImageRect.bottom-mImageRect.top)/2 ;
				mImageMagnification = 1.0 ;
				PositionImage() ;
				DrawExperimentalImage() ;
				mResetView->Disable() ;
		}
		else if (obj == mExpImageFrame) {
			DoMouseDownInImage(*pt) ;
		}
		else if (obj == mOK) {
			// Get and assemble all the relevant parameters and
			// compare the images
			if (mExpImage != nil)
				DeleteImage(mExpImage) ;	
			mExpImage = nil ;

			gCompareEntireImage = mCompareEntireImage->GetControlValue() ;
			gComparisonRect = mImageSelectionRect ;

			Hide() ;
			ExitFromSetComparisonArea() ;
		}		
		else if (obj == mCancel) {
			if (mExpImage != nil)
				DeleteImage(mExpImage) ;	
			mExpImage = nil ;
			Hide() ;
			ExitFromSetComparisonArea() ;
		}		
	}
} 

void	
TSetImageAreaPage::SizeImage() {
}

void
TSetImageAreaPage::PositionImage() {
	
	short width = mExpImage->mWidth*mImageMagnification ;
	short height = mExpImage->mHeight*mImageMagnification ;
	
	mActualImageRect.left = near(mImageCenter.x - width/2) ;	
	mActualImageRect.top = near(mImageCenter.y - height/2) ;
	mActualImageRect.right = mActualImageRect.left + width ;
	mActualImageRect.bottom = mActualImageRect.top + height ;
}

void		
TSetImageAreaPage::DoMouseDownInImage(Point &loc) {
	short 	xc,yc,dx,dy ;
	float	zoomfactor ;
	
	switch (mCurrentTool) {
		case kComparePointer:			
			if (mImageSelection != nil) {
				mImageSelection->Modify(loc) ;
			}
			break ;
		case kCompareMagnify:
			float factor ;
			if (::OptionKeyDown()) {
				if (::ShiftKeyDown()) {
					factor = 2.0 ;
				}
				else {
					factor = 1.1 ;
				}
				
			}
			else {
				if (::ShiftKeyDown()) {
					factor = 0.5 ;
				}
				else {
					factor = 1./1.1 ;
				}
			}
			xc = (mImageRect.right-mImageRect.left)/2 ;
			yc = (mImageRect.bottom-mImageRect.top)/2 ;
			mImageMagnification /= factor ;
			zoomfactor = 1./factor ;
			dx = xc + (mImageCenter.x - xc)/factor - mImageCenter.x ;
			dy = yc + (mImageCenter.y - yc)/factor - mImageCenter.y ;
			mImageCenter.x = xc + (mImageCenter.x - xc)/factor ;
			mImageCenter.y = yc + (mImageCenter.y - yc)/factor ;
			SizeImage() ;
			PositionImage() ;
			DrawExperimentalImage() ;
			if (mImageSelection != nil) {
				mImageSelection->Zoom(zoomfactor,dx,dy) ;
				mImageSelection->Update(&mImageRect) ;
			}			
			if (!mResetView->IsEnabled()) mResetView->Enable() ;
			break ;
		case kCompareHand:	
			// Let us try to pan the view
			Point lastPoint,currentPoint ;
			long dx,dy ;
			lastPoint = loc ;
			currentPoint = loc ;
			while(Button()) {
				GetMouse(&currentPoint) ;
				dx = currentPoint.x - lastPoint.x ;
				dy = currentPoint.y - lastPoint.y ;
				if (dx != 0 || dy != 0) {
					mImageCenter.x += dx ;
					mImageCenter.y += dy ;
					PositionImage() ;
					DrawExperimentalImage() ;
					if (mImageSelection != nil) {
						mImageSelection->Pan(dx,dy) ;
						mImageSelection->Update(&mImageRect) ;
					}			
					lastPoint = currentPoint ;
				}
			}
			if (lastPoint.x != loc.x || lastPoint.y != loc.y) {
				if (!mResetView->IsEnabled()) 	mResetView->Enable() ;
			}
			break ;		
		case kCompareSelection:
			InsetRect(&mImageRect,1,1) ;

			if (mImageSelection != nil) {
				mImageSelection->DoMouseDown(loc) ;
			}
			else {
				mImageSelection = new CSelection(&mImageRect) ;
				mImageSelection->DoMouseDown(loc) ;
			}
			if (mImageSelection->IsNullSelection()) {
				delete mImageSelection ;
				mImageSelection = nil ;
			}
			InsetRect(&mImageRect,-1,-1) ;
			if (mImageSelection != nil) {
				if (mCompareSelection->GetControlValue())
					mSetNewSelection->Enable() ;
				else
					mCompareSelection->Enable() ;
			}
			else 
				mCompareSelection->Disable() ;
			break ;
	}
}

void	ExitFromSetComparisonArea() {
	ClearDisplay() ;
	SetPage(gRefineParametersPage) ;
	gRefineParametersPage->Show() ;
}
