// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include "TTable.h"
#include <stdio.h>
#include "globals.h"
#include "funcdefs.h"
#include "XDrivers.h"
#define	MonitorWidth	1024
#define	MonitorHeight	768 
#include "TFile.h"

extern	uLong	gWhitePixel,gBlackPixel,gDarkerGray,gGrayPixel ;
extern	Atom	gDeleteWindowAtom ;

typedef enum {kSave,kNoSave,kCancelClose} SaveCode ;
SaveCode PromptSave(char *) ;

TTable::TTable(
	Int16 rows,
	Int16 cols,
	Int16 cellSize,
	Int16 colWidth,
	Int16 rowHeight,	
	char *tableTitle) 
	: TXForm(tableTitle),mRowLabels((Int32)(16*sizeof(char))),
	mColLabels((Int32)(16*sizeof(char)))
{
	short theWidth ;
	short theHeight ;
	Int16 xpos,ypos ;
	Int16	maxWindowWidth,tableWidth,tableHeight ;
	Int16	maxWindowHeight ;

	
	mHasColLabels = true ;
	mHasRowLabels = true ;
	mAdornRows = true ;
	mAdornColumns = true ;
	mRows = rows ;
	mCols = cols ;
	mCellDataSize = cellSize ;
	mColWidth = colWidth ;
	mRowHeight = rowHeight ;
	mCellData = nil ;

	mCanvasPort = nil ;
	
	xpos = 20 ;
	ypos = 88 ;
	maxWindowWidth = ((long)((MonitorWidth-xpos-16)/mColWidth))*mColWidth ; // Integer number of columns
	maxWindowHeight = ((long)((MonitorHeight-ypos-16)/mRowHeight))*mRowHeight ; // Integer number of rows
	
	tableWidth = mColWidth*(mCols+1) ;
	tableHeight = mRowHeight*(mRows+1) ;
	theWidth = (tableWidth > maxWindowWidth ? maxWindowWidth : tableWidth) ;
	theHeight = (tableHeight > maxWindowHeight ? maxWindowHeight : tableHeight) ;
	
	SetRect(&mCanvasRect,0,0,theWidth,theHeight) ;
	
	if (tableWidth > maxWindowWidth)	
		theHeight += 16 ;	// Because of scroll bars
	else
		theHeight += 4 ;

	if (tableHeight > maxWindowHeight)	
		theWidth += 16 ;
	else
		theWidth += 4 ;
	
	SetRect(&mWindowRect,0,0,theWidth,theHeight) ;
	DefineForm() ;
	mXForm->mForm->fdui = mXForm;
	fl_set_form_position(mXForm->mForm,xpos,ypos) ;
	
	InitTable() ;
}

TTable::TTable(void) 
	: mRowLabels((Int32)(16*sizeof(char))),
	mColLabels((Int32)(16*sizeof(char)))
{
	short theWidth ;
	short theHeight ;
	Int16	maxWindowWidth,tableWidth,tableHeight ;
	Int16	maxWindowHeight ;
	
	mHasColLabels = true ;
	mHasRowLabels = true ;
	mAdornRows = true ;
	mAdornColumns = true ;
	mRows = 40 ;
	mCols = 5 ;
	mCellDataSize = 4 ;
	mColWidth = 60 ;
	mRowHeight = 20 ;
	
	mCanvasPort = nil ;
	
	maxWindowWidth = ((long)((MonitorWidth-40-16)/mColWidth))*mColWidth ; // Integer # of columns
	maxWindowHeight = ((long)((MonitorHeight-60-16)/mRowHeight))*mRowHeight ; // Integer # of rows
	
	tableWidth = mColWidth*(mCols+1) ;
	tableHeight = mRowHeight*(mRows+1) ;
	theWidth = (tableWidth > maxWindowWidth ? maxWindowWidth : tableWidth) ;
	theHeight = (tableHeight > maxWindowHeight ? maxWindowHeight : tableHeight) ;
	
	SetRect(&mCanvasRect,0,0,theWidth,theHeight) ;

	if (tableWidth > maxWindowWidth)	
		theHeight += 16 ;	// Because of scroll bars
	else
		theHeight += 4 ;

	if (tableHeight > maxWindowHeight)	
		theWidth += 16 ;
	else
		theWidth += 4 ;
	
	SetRect(&mWindowRect,0,0,theWidth,theHeight) ;

	DefineForm() ;

	mXForm->mForm->fdui = mXForm;
	fl_set_form_position(mXForm->mForm,40,60) ;
	fl_bk_color(FL_WHITE) ;
	InitTable() ;
	
}
TTable::TTable(
	TableAttributes *attributes) 
	: mRowLabels((Int32)(16*sizeof(char))),
	mColLabels((Int32)(16*sizeof(char)))
{
	short theWidth ;
	short theHeight ;
	Int16	maxWindowWidth,tableWidth,tableHeight ;
	Int16	maxWindowHeight ;
	
	mHasColLabels = true ;
	mHasRowLabels = true ;
	mAdornRows = true ;
	mAdornColumns = true ;
	mRows = 40 ;
	mCols = 5 ;
	mCellDataSize = 4 ;
	mColWidth = 60 ;
	mRowHeight = 20 ;
	
	mCanvasPort = nil ;
	
	maxWindowWidth = ((long)((MonitorWidth-40-16)/mColWidth))*mColWidth ; // Integer # of columns
	maxWindowHeight = ((long)((MonitorHeight-60-16)/mRowHeight))*mRowHeight ; // Integer # of rows
	
	tableWidth = mColWidth*(mCols+1) ;
	tableHeight = mRowHeight*(mRows+1) ;
	theWidth = (tableWidth > maxWindowWidth ? maxWindowWidth : tableWidth) ;
	theHeight = (tableHeight > maxWindowHeight ? maxWindowHeight : tableHeight) ;
	
	SetRect(&mCanvasRect,0,0,theWidth,theHeight) ;
	
	if (tableWidth > maxWindowWidth)	
		theHeight += 16 ;	// Because of scroll bars
	else
		theHeight += 4 ;

	if (tableHeight > maxWindowHeight)	
		theWidth += 16 ;
	else
		theWidth += 4 ;
	
	SetRect(&mWindowRect,0,0,theWidth,theHeight) ;

	DefineForm() ;

	fl_set_form_position(mXForm->mForm,40,60) ;
	
	fl_set_clipping(0,0,theWidth,theHeight) ;
	InitTable() ;
	
}

void	TTable::DefineForm() {
	long	theWidth 	= mWindowRect.right - mWindowRect.left ;
	long	theHeight	= mWindowRect.bottom - mWindowRect.top ;
	
	long	tableWidth = mColWidth*(mCols+1) ;
	long	tableHeight = mRowHeight*(mRows+1) ;
	
	Boolean	hasVertScrollBar = false ;
	Boolean	hasHorizScrollBar = false ;
	long	canvasWidth = theWidth ;
	long	canvasHeight = theHeight ;
	if (tableWidth > theWidth)	{
		hasHorizScrollBar = true ;
		canvasHeight -= 16 ;
	}
	else	
		canvasHeight -= 4 ;

	if (tableHeight > theHeight) {	
		hasVertScrollBar = true ;
		canvasWidth -= 16 ;
	}
	else	
		canvasWidth -= 4 ;

	XSetWindowAttributes attr ;
	attr.background_pixel = fl_get_pixel(FL_WHITE) ;
	mXForm->mForm	= fl_bgn_form(FL_NO_BOX, theWidth, theHeight);
	mWindowBox = fl_add_box(FL_UP_BOX,0,0,theWidth,theHeight,"");
	mCanvas	= fl_add_canvas(FL_NORMAL_CANVAS,0,0,canvasWidth,canvasHeight,mTitle);
	fl_set_canvas_attributes(mCanvas, CWBackPixel ,&attr);
	if (hasVertScrollBar)
		mVertScrollBar = fl_add_scrollbar(FL_VERT_SCROLLBAR,canvasWidth,0,16,
			canvasHeight,"");
	else 
		mVertScrollBar = nil ;

	if (hasHorizScrollBar)
		mHorizScrollBar = fl_add_scrollbar(FL_HOR_SCROLLBAR,0,canvasHeight,
			canvasWidth,16,"");
	else 
		mHorizScrollBar = nil ;

	fl_end_form() ;
	mXForm->mForm->fdui = mXForm ;
}

void
TTable::InitTable()
{

	double	minVal = 0. ,maxVal ;
	double	theScrollIncrement ;

	mVScrollPos = 0. ;
	mHScrollPos = 0. ;
			
	Int16 vertScrollMax = mRowHeight*mRows - (mCanvasRect.bottom - mRowHeight) ;
	Int16 horizScrollMax = mColWidth*mCols - (mCanvasRect.right - mColWidth) ;

	if (mHorizScrollBar != nil) {
		maxVal	= horizScrollMax ;
		theScrollIncrement = mColWidth ;
		fl_set_scrollbar_bounds(mHorizScrollBar,minVal,maxVal) ;
		fl_set_scrollbar_step(mHorizScrollBar,1.0) ;
		fl_set_scrollbar_increment(mHorizScrollBar,theScrollIncrement,
			theScrollIncrement) ;
		fl_set_scrollbar_value(mHorizScrollBar,minVal) ;
		fl_set_object_callback(mHorizScrollBar,TTable::DoHScrollBar,(long)this) ;
		fl_set_scrollbar_return(mHorizScrollBar,FL_RETURN_END) ;
	}	
	if (mVertScrollBar != nil) {
		maxVal	= vertScrollMax ;
		theScrollIncrement = mRowHeight ;
		fl_set_scrollbar_bounds(mVertScrollBar,minVal,maxVal) ;
		fl_set_scrollbar_step(mHorizScrollBar,1.0) ;
		fl_set_scrollbar_increment(mVertScrollBar,theScrollIncrement,
			theScrollIncrement) ;
		fl_set_scrollbar_value(mVertScrollBar,minVal) ;
		fl_set_object_callback(mVertScrollBar,TTable::DoVScrollBar,(long)this) ;
		fl_set_scrollbar_return(mVertScrollBar,FL_RETURN_END) ;
	}	
	if ((mCellData == nil) && (mCellDataSize > 0)) {
		mCellData = new LArray((UInt32)mCellDataSize);
		mCellData->InsertItemsAt(mRows * mCols, 1, nil);
	}
	
	mRowLabels.InsertItemsAt(mRows, 1, nil);
	mColLabels.InsertItemsAt(mCols, 1, nil);
	
	fl_add_canvas_handler(mCanvas, Expose,TTable::DoCanvasUpdate,this);
	fl_add_canvas_handler(mCanvas, ButtonPress,TTable::DoCanvasClick,this);
	fl_add_canvas_handler(mCanvas, ConfigureNotify,TTable::DoCanvasGrow,this);
	mUpdateRgn = NewRgn() ;
	RectRgn(mUpdateRgn,&mWindowRect) ;
	mImageOrigin.x = 0 ;
	mImageOrigin.y = 0 ;
	mSelectedCell.row = 0 ;
	mSelectedCell.col = 0 ;
	mLastClick = 0 ;

	mPlacement = FL_PLACE_SIZE ;
	mBorder = FL_FULLBORDER ;

}

TTable::~TTable() 
{
	DisposeRgn(mUpdateRgn) ;
	delete	mCanvasPort ;
}

Boolean	TTable::RequestClose() { 
	SaveCode status = PromptSave("Do you want to save the table data?") ;
	switch(status) {
		case kCancelClose:
			return false ;
		case kSave:
			Save() ;
			return true ;
		case kNoSave:
			return true ;
	}
	return true ; 
} 

int		
TTable::DoCanvasClick(FL_OBJECT *ob, Window win, 
					int win_width, int win_height,
					XEvent *event, void *data) {

	TTable *theTable = (TTable *)data ;
	theTable->DoClick(event) ;
	return 1 ;
} 

int		
TTable::DoCanvasGrow(FL_OBJECT *ob, Window win, 
					int win_width, int win_height,
					XEvent *event, void *data) {

	TTable *theTable = (TTable *)data ;
	theTable->DoGrow(event) ;
	return 1 ;
} 

int		
TTable::DoCanvasUpdate(FL_OBJECT *ob, Window win, 
					int win_width, int win_height,
					XEvent *event, void *data) {

	TTable *theTable = (TTable *)data ;
	XRectangle	updateRect,xClipRect ;
	GrafPtr tPort ;
	updateRect.x = event->xexpose.x ;
	updateRect.y = event->xexpose.y ;
	updateRect.width = event->xexpose.width ;
	updateRect.height = event->xexpose.height ;
	XUnionRectWithRegion(&updateRect,theTable->mUpdateRgn,
				theTable->mUpdateRgn) ;
	if (event->xexpose.count == 0) {
		GetPort(&tPort) ;
		Rect	clipRect ;
		RegionRect(theTable->mUpdateRgn,&clipRect) ;
		xClipRect.x = clipRect.left ;
		xClipRect.y = clipRect.top ;
		xClipRect.width = clipRect.right - clipRect.left ;
		xClipRect.height = clipRect.bottom - clipRect.top ;
		XSetClipRectangles(gDisplay,fl_state[fl_get_vclass()].gc[0],0,0,
				&xClipRect,1,Unsorted) ;
		fl_redraw_object(theTable->mWindowBox) ;
		if (theTable->mVertScrollBar != nil) fl_redraw_object(theTable->mVertScrollBar) ;
		if (theTable->mHorizScrollBar != nil) fl_redraw_object(theTable->mHorizScrollBar) ;
		xClipRect.x = theTable->mWindowBox->x ;
		xClipRect.y = theTable->mWindowBox->y ;
		xClipRect.width = theTable->mWindowBox->w ;
		xClipRect.height = theTable->mWindowBox->h ;
		XSetClipRectangles(gDisplay,fl_state[fl_get_vclass()].gc[0],0,0,
				&xClipRect,1,Unsorted) ;
		if (theTable->mCanvasPort == nil) {
			theTable->mCanvasPort = new TGrafPort(theTable->mCanvas) ;
			XStoreName(gDisplay,theTable->mXForm->mForm->window,theTable->mTitle) ;
			SetPort(theTable->mCanvasPort) ;
			SetBkColor(fl_get_pixel(FL_WHITE)) ;
			SetFgColor(fl_get_pixel(FL_BLACK)) ;
		}
		else {
			SetPort(theTable->mCanvasPort) ;
		}
		SetClip(theTable->mUpdateRgn) ;
		theTable->Draw() ;
		SetClip(theTable->mWindowRect) ;
		SetPort(tPort) ;
		XFlush(gDisplay) ;
	}
	return 0 ;
} 

void	
TTable::SetTableTitle(char *title) {
	strcpy(mTitle,title) ;
	fl_set_object_label(mCanvas,title) ;
}

void	
TTable::GetTableTitle(char *title) {
	strcpy(title,mTitle) ;
}

void	
 TTable::SetColLabel(
	Int16 inCol,
	char *title) 
{
	mColLabels.AssignItemsAt(1,inCol,(void *)title,(UInt32)(16*sizeof(char))) ;
}

void	
 TTable::SetRowLabel(
	Int16 inRow,
	char *title)
{
	mRowLabels.AssignItemsAt(1,inRow,(void *)title,(UInt32)(16*sizeof(char))) ;
}

void TTable::SetRowHeight(
	Int16 height) 
{
	mRowHeight = height ; 
}
void TTable::SetColWidth(
	Int16 width) 
{
	mColWidth = width ;
}
void TTable::Draw() 
{
	Int16 row,col ;
	Point	topLeftUpdate,botRightUpdate,topLeft,botRight;
	Rect	totalUpdateRect ;


	if (mCanvasPort == nil) {
		mCanvasPort = new TGrafPort(mCanvas) ;
		XStoreName(gDisplay,mXForm->mForm->window,mTitle) ;
		SetPort(mCanvasPort) ;
		SetBkColor(fl_get_pixel(FL_WHITE)) ;
		SetFgColor(fl_get_pixel(FL_BLACK)) ;
	}

	SetPort(mCanvasPort) ;

	// Get the region associated with the update
	RegionRect(mUpdateRgn,&totalUpdateRect) ;
	
	if (totalUpdateRect.top == 0 && totalUpdateRect.left == 0 &&
		totalUpdateRect.bottom == 0 && totalUpdateRect.right == 0) {
			return ;
	}

	EraseRect(&mCanvasRect) ;

	Int16 rightEdge,bottomEdge,savedLeft ;
	rightEdge = mCanvasRect.right ;
	bottomEdge = mCanvasRect.bottom ;
	
	if (totalUpdateRect.right > rightEdge) {
		totalUpdateRect.right = rightEdge ;
	}
	if (totalUpdateRect.bottom > bottomEdge) {
		totalUpdateRect.bottom = bottomEdge ;
	}
	savedLeft = totalUpdateRect.left ;
	if (mHasColLabels) {
		DrawColLabel(0) ;
		if (totalUpdateRect.left < mColWidth) {
			totalUpdateRect.left = mColWidth ;
		}
	}

	SetClip(totalUpdateRect) ;
	
	if (mHasColLabels) {
		// Find colum label at top left of update rect
		Int16	leftColumn,rightColumn;	
		// Calculate the column intersected by the top left point
		// of the update region
		topLeft.x = totalUpdateRect.left ;
		topLeft.y = totalUpdateRect.top ;
		if (topLeft.y <= mRowHeight) {
			//need to draw  column labels
			leftColumn = (topLeft.x-mImageOrigin.x-1)/mColWidth ;
			if (leftColumn < 0) {
				leftColumn = 0 ;
			}
			
			botRight.x = totalUpdateRect.right ;
			rightColumn = (botRight.x-mImageOrigin.x+1)/mColWidth ;
			if (rightColumn > mCols) {
				rightColumn = mCols ;
			}
			
			for(col=leftColumn;col<=rightColumn;col++) {
				DrawColLabel(col) ;
			}
		}
	}
	
	if (mHasRowLabels) {
		totalUpdateRect.left = savedLeft ;
		if (totalUpdateRect.top < mRowHeight)
			totalUpdateRect.top = mRowHeight ;
	}
	
	SetClip(totalUpdateRect) ;
	
	if (mHasRowLabels) {
		// Find colum label at top left of update rect
		Int16	topRow,bottomRow;	
		// Calculate the column intersected by the top left point
		// of the update region
		topLeft.x = totalUpdateRect.left ;
		topLeft.y = totalUpdateRect.top ;
		if (topLeft.x <= mColWidth) {
			// need to draw  row labels
			topRow = (topLeft.y-mImageOrigin.y-1)/mRowHeight ;
			if (topRow < 0) {
				topRow = 0 ;
			}
			
			botRight.y = totalUpdateRect.bottom ;
			bottomRow = (botRight.y-mImageOrigin.y+1)/mRowHeight ;
			if (bottomRow > mRows) {
				bottomRow = mRows ;
			}
			for(row=topRow;row<=bottomRow;row++) {
				DrawRowLabel(row) ;
			}
		}
	}
	
	RegionRect(mUpdateRgn,&totalUpdateRect) ;
	if (totalUpdateRect.top == 0 && totalUpdateRect.left == 0 &&
		totalUpdateRect.bottom == 0 && totalUpdateRect.right == 0) {
			totalUpdateRect = mCanvasRect ;
	}
	if (totalUpdateRect.right > rightEdge) {
		totalUpdateRect.right = rightEdge ;
	}
	if (totalUpdateRect.bottom > bottomEdge) {
		totalUpdateRect.bottom = bottomEdge ;
	}
	if (totalUpdateRect.top < mRowHeight && mHasColLabels) {
		totalUpdateRect.top = mRowHeight ;
	}
	if (totalUpdateRect.left < mColWidth  && mHasRowLabels) {
		totalUpdateRect.left = mColWidth ;
	}

	SetClip(totalUpdateRect) ;
	// ---------------------------------------
	// Draw the cells that need to be updated
	// ---------------------------------------
	
	// Find cell at top left of update rect
	TableCellT	topLeftCell;	
	// Calculate the cell intersected by the top left point
	// of the update region
	topLeft.x = totalUpdateRect.left - 1 ;
	topLeft.y = totalUpdateRect.top - 1 ;
	LocalToImagePoint(topLeft,topLeftUpdate) ;			
	FetchCellHitBy(topLeftUpdate, topLeftCell);
	topLeftCell.row-- ;
	topLeftCell.col-- ;
	if (topLeftCell.row < 1) {	// Lower bound is cell (1,1)
		topLeftCell.row = 1;
	}
	if (topLeftCell.col < 1) {
		topLeftCell.col = 1;
	}

								// Find cell at bottom right of update rect
	TableCellT	botRightCell;
	botRight.x = totalUpdateRect.right + 1 ;
	botRight.y = totalUpdateRect.bottom + 1 ;
	LocalToImagePoint(botRight, botRightUpdate);
	FetchCellHitBy(botRightUpdate, botRightCell);
	botRightCell.row++ ;
	botRightCell.col++ ;
								// Upper bound is cell (mRows,mCols)
	if (botRightCell.row > mRows) {
		botRightCell.row = mRows;
	}
	if (botRightCell.col > mCols) {
		botRightCell.col = mCols;
	}
	
								// Draw each cell within the update rect
	TableCellT	cell;
	for (cell.row = topLeftCell.row; cell.row <= botRightCell.row; cell.row++) {
		for (cell.col = topLeftCell.col; cell.col <= botRightCell.col; cell.col++) {
			DrawCell(cell);
		}
	}
	
	// Set the clip to the entire window
	SetClip(mWindowRect) ;
		
	//We have done the needed update. zero the mUpdateRgn
	DisposeRgn(mUpdateRgn) ;
	mUpdateRgn = NewRgn() ;

}

void TTable::Print() 
{
	/*
	Int16 row,col ;
	RgnHandle clipRgn,oldRgn ;
	
	clipRgn = NewRgn() ;
	oldRgn = NewRgn() ;
	GetClip(oldRgn) ;
	RectRgn(clipRgn,&qd.thePort->portRect) ;
	SetClip(clipRgn) ;
	
	if (mHasColLabels) {
		DrawColLabel(0) ;
	}
	
	if (mHasColLabels) {
		for(col=1;col<=mCols;col++) {
			DrawColLabel(col) ;
		}
	}
		
	if (mHasRowLabels) {
		for(row=1;row<=mRows;row++) {
				DrawRowLabel(row) ;
		}
	}
	
	TableCellT	cell;
	for (cell.row = 1; cell.row <= mRows; cell.row++) {
		for (cell.col = 1; cell.col <=mCols; cell.col++) {
			DrawCell(cell);
		}
	}
	SetClip(oldRgn) ;
	*/
	
}
void TTable::DrawCell(
	TableCellT &inCell) 
{
	Rect theRect ;
	Point CellImagePt,CellLocalPt ;
	
	CellImagePt.x = (mHasRowLabels ? inCell.col*mColWidth : (inCell.col-1)*mColWidth) ;
	CellImagePt.y = (mHasColLabels ? inCell.row*mRowHeight : (inCell.row-1)*mRowHeight) ;
	ImageToLocalPoint(CellImagePt,CellLocalPt) ;
	SetRect(&theRect,CellLocalPt.x,CellLocalPt.y,CellLocalPt.x+mColWidth,
		CellLocalPt.y+mRowHeight) ;
		
	SetFgColor(fl_get_pixel(FL_COL1)) ;
	MoveTo(theRect.left,theRect.bottom) ;
	LineTo(theRect.left,theRect.top) ;
	LineTo(theRect.right,theRect.top) ;
	if (inCell.col == mCols) {
		LineTo(theRect.right,theRect.bottom) ;
	}
	if (inCell.row == mRows) {
		MoveTo(theRect.left,theRect.bottom) ;
		LineTo(theRect.right,theRect.bottom) ;
	}
	//FrameRect(&theRect) ;
	PenNormal() ;
	SetFgColor(fl_get_pixel(FL_BLACK)) ;
	DrawCellValue(inCell,theRect) ;
}

void TTable::DrawCellValue(
	TableCellT &inCell,
	Rect &cellFrame)
{
	char str[64] ;
	
	//TextFont(applFont);
	TextSize(0);
	MoveTo(cellFrame.left + 4, cellFrame.bottom - 4);
	sprintf(str,"%d,%d",inCell.row,inCell.col) ;
	DrawString(str) ;
}

Boolean
TTable::FetchLocalCellFrame(
	const TableCellT	&inCell,
	Rect				&outCellFrame)
{
								// Get Top-Left in Image coordinates
	Point	cellImage;
	
	if (mHasRowLabels)
		cellImage.x = inCell.col * mColWidth;
	else
		cellImage.x = (inCell.col - 1) * mColWidth;

	if (mHasColLabels)
		cellImage.y = inCell.row * mRowHeight;
	else
		cellImage.y = (inCell.row - 1) * mRowHeight;
	
	// Check if Cell intersects the Frame
	Boolean	intersectsFrame = ImageRectIntersectsFrame(
									cellImage.x, cellImage.y,
									cellImage.x + mColWidth,
									cellImage.y + mRowHeight);
	
	if (intersectsFrame) {		// Convert to Local coordinates
		ImageToLocalPoint(cellImage, topLeft(outCellFrame));
		outCellFrame.right = outCellFrame.left + mColWidth;
		outCellFrame.bottom = outCellFrame.top + mRowHeight;
	}
	
	return intersectsFrame;
}

Boolean 
TTable::ImageRectIntersectsFrame(
	Int16 left,
	Int16 top,
	Int16 right,
	Int16 bottom) 
{
	Point topLeft, bottomRight ;
	
	topLeft.x = left + mImageOrigin.x ;
	topLeft.y = top + mImageOrigin.y ;
	
	bottomRight.x = right + mImageOrigin.x ;
	bottomRight.y = bottom + mImageOrigin.y ;
	
	Rect frame = mCanvasRect ;
	if (mHasRowLabels) {
		frame.left += mColWidth ;
	}
	if (mHasColLabels) {
		frame.top += mRowHeight ;
	}
	frame.right -= 15 ;	// Because of scrollbar
	frame.bottom -= 15 ; // likewise
	
	return (PtInRect(&topLeft,&frame) || PtInRect(&bottomRight,&frame)) ;
}

void 	
TTable::ImageToLocalPoint(
	Point &inImagePt,
	Point &outLocalPt) 
{
	outLocalPt.x = inImagePt.x + mImageOrigin.x ;
	outLocalPt.y = inImagePt.y + mImageOrigin.y ;
}

void 	
TTable::LocalToImagePoint(
	Point &inLocalPt,
	Point &outImagePt) 
{
	outImagePt.x = inLocalPt.x - mImageOrigin.x ;
	outImagePt.y = inLocalPt.y - mImageOrigin.y ;
}


Boolean
TTable::IsValidCell(
	const TableCellT	&inCell) const
{
	return ( (inCell.row > 0) && (inCell.row <= mRows) &&
			 (inCell.col > 0) && (inCell.col <= mCols) );
}


void
TTable::FetchCellHitBy(
	const Point		&inImagePt,
	TableCellT		&outCell)
{
	// Calculate the cell hit by the point (in image coordinates)
	Point CellPoint = inImagePt ;
	
	if (mHasRowLabels) {
		CellPoint.x -= mColWidth ;
	}
	if (mHasColLabels) {
		CellPoint.y -= mRowHeight ;
	}
	outCell.row = (CellPoint.y-1)/ mRowHeight + 1 ;
	outCell.col = (CellPoint.x-1)/ mColWidth + 1 ;
}

 
void TTable::SetCellData(
	TableCellT &inCell, 
	void *data) 
{
	Int32 cellIndex ;
	cellIndex = (inCell.row-1)*mCols+inCell.col ;
	mCellData->AssignItemsAt(1,cellIndex,data) ;
}

Boolean TTable::FetchCellData(
	TableCellT &inCell,
	void *data) 
{
	Int32 cellIndex ;
	cellIndex = (inCell.row-1)*mCols+inCell.col ;
	return(mCellData->FetchItemAt(cellIndex,data)) ;
}

void TTable::SetDataCells(
	TableCellT &inFromCell, 
	TableCellT &inToCell,
	void *data) 
{
	TableCellT cell ;
	char *dataLoc ;
	
	dataLoc = (char *)data ;
	for(cell.row=inFromCell.row;cell.row < inToCell.row;cell.row++) {
		for(cell.col=inFromCell.col;cell.col < inToCell.col;cell.col++) {
			SetCellData(cell,(void *)dataLoc) ;
			dataLoc += mCellDataSize ;
		}
	}
} 
void TTable::FetchDataCells(
	TableCellT &inFromCell, 
	TableCellT &inToCell,
	void *data) 
{
	TableCellT cell ;
	char *dataLoc ;
	
	dataLoc = (char *)data ;
	for(cell.row=inFromCell.row;cell.row < inToCell.row;cell.row++) {
		for(cell.col=inFromCell.col;cell.col < inToCell.col;cell.col++) {
			FetchCellData(cell,(void *)dataLoc) ;
			dataLoc += mCellDataSize ;
		}
	}
} 
void TTable::ScrollTableBy(
	Int16 deltaX,
	Int16 deltaY) 
{
	ScrollBits(deltaX,deltaY) ;
	mImageOrigin.x -= deltaX ;
	if (mImageOrigin.x > 0)
		mImageOrigin.x = 0 ;
	mImageOrigin.y -= deltaY ;
	if (mImageOrigin.y > 0)
		mImageOrigin.y = 0 ;
	Refresh() ;
}

void 
TTable::Update() 
{
	GrafPtr	tPort ;
	GetPort(&tPort) ;
	SetPort(mCanvasPort) ;
	DisposeRgn(mUpdateRgn) ;
	mUpdateRgn = NewRgn() ;
	RectRgn(mUpdateRgn,&mCanvasRect) ;
	Draw() ;
	SetPort(tPort) ;
}

void 
TTable::DoGrow(
	XEvent *inEvent) 
{
	// We don't do grow
}

void
TTable::Refresh() 
{
	GrafPtr	tPort ;
	GetPort(&tPort) ;
	SetPort(mCanvasPort) ;
	DisposeRgn(mUpdateRgn) ;
	mUpdateRgn = NewRgn() ;
	RectRgn(mUpdateRgn,&mCanvasRect) ;
	Draw() ;
	SetPort(tPort) ;
}

void TTable::ClickCell(
	TableCellT &inCell) 
{
	if ((mClickTime-mLastClick) < 100) {
		EditCell(inCell) ;
	}
	mLastClick = mClickTime ;
}

void TTable::EditCell(
	TableCellT &inCell) 
{
	MoveTo(4,10) ;
	char str[16] ;
	sprintf(str,"%d,%d",inCell.row,inCell.col) ;
	DrawString(str) ;
}

void TTable::DoClick(
	XEvent *inEvent) 
{

	Point			theMouseLoc ;
	
	theMouseLoc.x = inEvent->xbutton.x ;
	theMouseLoc.y = inEvent->xbutton.y ;
	mClickTime 		= inEvent->xbutton.time ;
	
	TableCellT theCell ;
	Point imagePoint ;
	LocalToImagePoint(theMouseLoc,imagePoint) ;
	FetchCellHitBy(imagePoint,theCell) ;
	ClickCell(theCell) ;
}
void TTable::HighlightCell(TableCellT &inCell) 
{
}

void
TTable::ScrollBits(
	Int16		inLeftDelta,			// Pixels to scroll horizontally
	Int16		inTopDelta)				// Pixels to scroll vertically
{
										// Get Frame in Port coords
	GrafPtr	tPort ;
	GetPort(&tPort) ;
	GrafPtr	thisPort ;
	SetPort(thisPort = new TGrafPort(mCanvas)) ;
	
	Rect	frame = mCanvasRect ;
	
	if (inLeftDelta != 0) {
		// We want to scroll left/right ;
		if (mHasRowLabels) {
			frame.left += mColWidth ;
		}
	}
	else if (inTopDelta != 0) {
		// We want to scroll up/down
		if (mHasColLabels) {
			frame.top += mRowHeight ;
		}
	}
	
	// Scroll Frame, clipping to the
	//   update region
	//RgnHandle	updateRgnH = NewRgn();
	//RgnHandle	newClip = NewRgn() ;
	//RectRgn(newClip,&frame) ;
	//SetClip(newClip) ;
	//ScrollRect(&frame, -inLeftDelta, -inTopDelta, mUpdateRgn);
	
									// Force redraw of update region
									// Use our InvalPortRgn rather than
									// Toolbox InvalRgn
	//InvalPortRgn(updateRgnH);			// Force a redraw of the content
	//RectRgn(newClip,&thisPort->GetPortRect) ;
	//SetClip(newClip) ;
	//DisposeRgn(updateRgnH);
	//DisposeRgn(newClip);
	SetPort(tPort) ;
	delete	thisPort ;
}

void TTable::DrawColLabel(
	Int16 colNumber) 
{
	Rect	cellFrame;
	
	SetRect(&cellFrame,colNumber*mColWidth,0,(colNumber+1)*mColWidth,mRowHeight) ;
	if (colNumber != 0) 
		::OffsetRect(&cellFrame,mImageOrigin.x,0) ;
	InsetRect(&cellFrame,1,1) ;
	SetFgColor(fl_get_pixel(FL_COL1)) ;
	PaintRect(&cellFrame) ;
	SetFgColor(fl_get_pixel(FL_TOP_BCOL)) ;
	MoveTo(cellFrame.left,cellFrame.bottom) ;
	LineTo(cellFrame.left,cellFrame.top) ;
	LineTo(cellFrame.right,cellFrame.top) ;
	SetFgColor(fl_get_pixel(FL_BLACK)) ;
	MoveTo(cellFrame.right,cellFrame.top) ;
	LineTo(cellFrame.right,cellFrame.bottom) ;
	LineTo(cellFrame.left,cellFrame.bottom) ;
	InsetRect(&cellFrame,-1,-1) ;
	//TextFont(applFont);
	TextSize(0);
	MoveTo(cellFrame.left + 4, cellFrame.bottom - 4);
	if (colNumber > 0) {
		DrawColLabelText(colNumber) ;
	}
}
void TTable::DrawRowLabel(
	Int16 rowNumber) 
{
	Rect	cellFrame;
	
	SetRect(&cellFrame,0,rowNumber*mRowHeight,mColWidth,(rowNumber+1)*mRowHeight) ;
	if (rowNumber != 0)
		::OffsetRect(&cellFrame,0,mImageOrigin.y) ;
	InsetRect(&cellFrame,1,1) ;
	SetFgColor(fl_get_pixel(FL_COL1)) ;
	PaintRect(&cellFrame) ;
	SetFgColor(fl_get_pixel(FL_TOP_BCOL)) ;
	MoveTo(cellFrame.left,cellFrame.bottom) ;
	LineTo(cellFrame.left,cellFrame.top) ;
	LineTo(cellFrame.right,cellFrame.top) ;
	SetFgColor(fl_get_pixel(FL_BLACK)) ;
	MoveTo(cellFrame.right,cellFrame.top) ;
	LineTo(cellFrame.right,cellFrame.bottom) ;
	LineTo(cellFrame.left,cellFrame.bottom) ;
	InsetRect(&cellFrame,-1,-1) ;
	//TextFont(applFont);
	TextSize(0);
	MoveTo(cellFrame.left + 4, cellFrame.bottom - 4);
	if (rowNumber > 0) {
		DrawRowLabelText(rowNumber) ;
	}
}

void 
TTable::DrawRowLabelText(
	Int16 rowNumber) 
{
	char str[64] ;
	sprintf(str,"R%d",rowNumber) ;
	DrawString(str) ;
}

void 
TTable::DrawColLabelText(
	Int16 colNumber) 
{
	char str[64] ;
	sprintf(str,"C%d",colNumber) ;
	DrawString(str) ;
}

void	
TTable::InvalPortRgn(
	RgnHandle inUpdateRgn)
{
	//CopyRgn(inUpdateRgn,mUpdateRgn) ;
	//InvalRgn(inUpdateRgn) ;
}
void 
TTable::DoHScrollBar(FL_OBJECT *object,long arg) {
	float	newVal ;
	Int32	scrollDistance ;
	TTable	*theTable ;
	
	theTable = (TTable *)arg ;
	newVal	= fl_get_scrollbar_value(object) ;

	scrollDistance = (Int32)(newVal - theTable->mHScrollPos) ;
	theTable->ScrollTableBy(scrollDistance,0) ;
	theTable->mHScrollPos = newVal ;
}

void 
TTable::DoVScrollBar(FL_OBJECT *object,long arg) {
	float	newVal ;
	Int32	scrollDistance ;
	TTable	*theTable ;
	
	theTable = (TTable *)arg ;
	newVal	= fl_get_scrollbar_value(object) ;

	scrollDistance = (Int32)(newVal - theTable->mVScrollPos) ;
	theTable->ScrollTableBy(0,scrollDistance) ;
	theTable->mVScrollPos = newVal ;

}	

void TTable::Save() 
{
	// ------------------------------------------------------------------ 
	// Put up the standard file dialog Box to determine the file name 
	// ------------------------------------------------------------------ 
	FSInfo 	theFInfo ;
	
	strcpy(theFInfo.mFileName,"") ;
	ChooseFile("Save Table data as:","*",theFInfo) ;
	
	if (!theFInfo.mReply) 
		return ;

	char buffer[256] ;
	long i ;
	long inByteCount ;
	char str[64] ;
	TableCellT theCell ;
	float	fvalue ;
	FILE	*fp ;

	if ((fp = fopen(theFInfo.mFileName,"w")) == nil) {
		sprintf(buffer,"Unable to open the file %s",theFInfo.mFileName) ;
		PutMessage(buffer) ;
		return ;
	}
	
	
	for(i=1;i<mCols;i++) {
		mColLabels.FetchItemAt(i,(void *)str) ;
		if (i == 1) {
			sprintf(buffer,"Row/Col\t%s\t",str) ;
		}
		else {
			sprintf(buffer,"%s\t",str) ;
		}
		inByteCount = strlen(buffer) ;
		fwrite(buffer,inByteCount,1,fp) ;
	}
	mColLabels.FetchItemAt(mCols,(void *)str) ;
	sprintf(buffer,"%s\r",str) ;
	inByteCount = strlen(buffer) ;
	fwrite(buffer,inByteCount,1,fp) ;
	
	for(i=1;i<=mRows;i++) {
		mRowLabels.FetchItemAt(i,(void *)str) ;
		sprintf(buffer,"%s\t",str) ;
		inByteCount = strlen(buffer) ;
		fwrite(buffer,inByteCount,1,fp) ;
		theCell.row = i ;
		for(theCell.col=1;theCell.col<mCols;theCell.col++) {
			FetchCellData(theCell,(void *)&fvalue) ;
			sprintf(buffer,"%.4f\t",fvalue) ;
			inByteCount = strlen(buffer) ;
			fwrite(buffer,inByteCount,1,fp) ;
		}
		theCell.col = mCols ;
		FetchCellData(theCell,(void *)&fvalue) ;
		sprintf(buffer,"%.4f\r",fvalue) ;
		inByteCount = strlen(buffer) ;
		fwrite(buffer,inByteCount,1,fp) ;
	}
	fclose(fp) ;
}

