#ifndef TTable_h
#define TTable_h
#include "types.h"
#include "TWindow.h"
#include "LArray.h"
#include "UMemoryMgr.h"

#include "TXForm.h"

typedef struct {
	short row ;
	short col ;
} TableCellT ;

/* typedef long	(*FL_HANDLE_CANVAS)(FL_OBJECT *ob, Window win, 
			long win_width, long win_height,
			XEvent *xev, void *user_data) ;
*/
typedef struct {
	Int16	nRows ;
	Int16	nCols ;
	Int16	DataSize ;
	Int16	ColWidth ;
	Int16	RowHeight ;
	Int16	WindowWidth ;
	Int16	WindowHeight ;
	Boolean	AdornColumns ;
	Boolean	AdornRows ;
	Boolean	HasColumnLabels ;
	Boolean	HasRowLabels ;
	char	*WindowTitle ;
}	TableAttributes ;

class TTable : public TXForm {

public:
	TTable(Int16 rows,Int16 cols,Int16 cellSize,Int16 colWidth,Int16 rowHeight,char *tableTitle) ;
	TTable(TableAttributes *attributes) ;
	TTable(void) ;
	virtual	~TTable() ;
	virtual void 	SetRowHeight(Int16 height) ;
	virtual void 	SetColWidth(Int16 width) ;
	virtual void 	Draw() ;
	virtual void	Print() ;
	virtual void	DefineForm() ;
	virtual	void	Save() ;
	virtual	Boolean	RequestClose() ;

	virtual void	SetTableTitle(char *title) ;
	virtual void	GetTableTitle(char *title) ;
	virtual void 	SetCellData(TableCellT &inCell,void *data) ;
	virtual Boolean FetchCellData(TableCellT &inCell,void *data) ;
	virtual void 	SetDataCells(TableCellT &inFromCell, TableCellT &inToCell,void *data) ; 
	virtual void 	FetchDataCells(TableCellT &inFromCell, TableCellT &inToCell,void *data) ; 
	virtual void 	ClickCell(TableCellT &inCell) ;
	virtual void 	EditCell(TableCellT &inCell) ;
	virtual void 	DoClick(XEvent *inEvent) ;
	virtual	void 	Update() ;
	virtual	void 	DoGrow(XEvent *inEvent) ;
	virtual void	SetColLabel(Int16 inCol,char *title) ;
	virtual void	SetRowLabel(Int16 inRow,char *title) ;

	static	void	DoHScrollBar(FL_OBJECT *object,long arg) ;
	static	void	DoVScrollBar(FL_OBJECT *object,long arg) ;

	static	int		DoCanvasUpdate(FL_OBJECT *ob, Window win, 
					int win_width, int win_height,
					XEvent *xev, void *user_data) ; 
	static	int		DoCanvasGrow(FL_OBJECT *ob, Window win, 
					int win_width, int win_height,
					XEvent *xev, void *user_data) ; 
	static	int		DoCanvasClick(FL_OBJECT *ob, Window win, 
					int win_width, int win_height,
					XEvent *xev, void *user_data) ; 
	
protected:

	virtual	void 	DrawCellValue(TableCellT &inCell,Rect &cellFrame) ;
	virtual void 	DrawColLabel(Int16 colNumber) ;
	virtual void 	DrawRowLabel(Int16 rowNumber) ;
	virtual void 	DrawCell(TableCellT &inCell) ;
	virtual void 	ScrollTableBy(Int16 deltaX,Int16 deltaY) ;
	virtual void 	HighlightCell(TableCellT &inCell) ;
	virtual	void 	Refresh() ;
	virtual void	InitTable() ;	
	virtual void	DrawRowLabelText(Int16 rowNumber) ;
	virtual void	DrawColLabelText(Int16 colNumber) ;
	
	Boolean ImageRectIntersectsFrame(Int16 left,Int16 top,Int16 right,Int16 bottom) ;
	void 	ImageToLocalPoint(Point &inImagePt,Point &outLocalPt) ;
	void 	LocalToImagePoint(Point &inImagePt,Point &outLocalPt) ;
	void	FetchCellHitBy(const Point &inImagePt,TableCellT &outCell) ;
	void	InvalPortRgn(Region inUpdateRgn) ;
	Boolean IsValidCell(const TableCellT	&inCell) const ;
	inline 	Region	GetLocalUpdateRgn() { return mUpdateRgn ; } ;
	Boolean FetchLocalCellFrame(const TableCellT &inCell,Rect &outCellFrame) ;
	
	Region			mUpdateRgn ;
	double			mVScrollPos ;
	double			mHScrollPos ;

	char			mTitle[256] ;

	FL_OBJECT 		*mCanvas;
	FL_OBJECT 		*mVertScrollBar;
	FL_OBJECT 		*mHorizScrollBar;

	Boolean			mHasColLabels ;
	Boolean			mHasRowLabels ;
	Boolean			mAdornRows ;
	Boolean			mAdornColumns ;
	Pattern			mAdornPattern;
	Rect			mUpdateRect;
	Rect			mCanvasRect ;
	Point			mImageOrigin ;	// The origin of the table with respect to 
							// the window.  Local Point is Port point.
							// Image Point is with respect to the table origin
	Rect			mRevealedRect ;
	void			ScrollBits(Int16 inLeftDelta,Int16 inTopDelta) ;
	
	// these are arrays of 16 characters (max of labels)
	LArray			mRowLabels;
	LArray			mColLabels;
	
	LArray			*mCellData;
	
	Int16			mRows ;
	Int16			mCols ;
	Int16			mColWidth ;
	Int16			mRowHeight ;
	Int16			mCellDataSize ;
	TableCellT		mSelectedCell ;
	long			mClickTime ;
	long			mLastClick ;
	GrafPtr			mCanvasPort ;
	Rect			mWindowRect ;
} ;

#endif // TTable_h

