#ifndef __TWindow
#define __TWindow
#ifdef VMS
#include <decw$include/Xlib.h>
#include <decw$include/Xutil.h>
#else
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#endif

#include <forms.h>
#include "types.h"
class 	TWindowList ;
class	TPaneList ;
extern	Display 	*gDisplay ;

typedef enum { kNormalWindow, kNonModalDialog, kModalDialog , kFloatingWindow } WindowType ; 

class	TGrafPort {
	public:
		TGrafPort(short width,short height,short depth = 32) ;
		TGrafPort() ;
		TGrafPort(FL_OBJECT *object) ;	// Create an instance around an xform object
		virtual	~TGrafPort() ;

		Drawable	mDrawable ;
	    	GC			mGC ;
	    	Rect		mPortRect ;
	    	uLong		mFgColor ;
	    	uLong		mBkColor ;
	    	short		mDepth ;

		RgnHandle	mClipRegion ;

		Drawable GetDrawable() { return mDrawable ; } ;
		RgnHandle	GetClipRegion() { return mClipRegion ; } ;
		void		SetClipRegion(RgnHandle rgn) ;
		void		SetClipRegion(Rect &rect) ;

		Rect	&GetPortRect() { return mPortRect ; } ;
		GC		GetGC() { return mGC ; } ;
		uLong	GetFgColor() { return mFgColor ; } ;
		uLong	GetBkColor() { return mBkColor ; } ;
		void	SetFgColor(uLong ) ;
		void	SetBkColor(uLong ) ;
		short	GetDepth() { return mDepth ; } ;

} ;

class TOffscreen : public TGrafPort {
	public:
		TOffscreen(class TWindow *,Rect *rect = nil) ;
		~TOffscreen() ;
		Pixmap  GetPixmap(void) { return mPixmap ; } ;

		// It is essential that the order of the members are as follows
		// so that a TOffscreen and TWindow will draw appropriately	
	protected:
		friend	class TWindow ;	
		Pixmap 	mPixmap ;
} ;

class TStOffscreen : public TGrafPort {
	public:
		TStOffscreen(class TWindow *,Rect *rect = nil) ;
		~TStOffscreen() ;
		Pixmap  GetPixmap(void) { return mPixmap ; } ;

	protected:
		friend	class TWindow ;	
		Pixmap 	mPixmap ;
		TGrafPort	*mCurrentPort ;
		Rect	mRect ;
		Boolean	mHasLocalGC ;
} ;

class TWindow ;

class	TPane : public TGrafPort {
	public:
		TPane() {} ;
		TPane(TPane *parent,short xloc,short yloc,short width,short height) ; 
		TPane(TPane *parent,Rect *rect) ;	// 
		virtual	~TPane() ;
		Window  GetXWindow(void) { return mXWindow ; } ;
		virtual void	Draw() {} ;
		virtual	void	Show() { XMapRaised(gDisplay,mXWindow); } ;
		virtual	void	Hide() { XUnmapWindow(gDisplay,mXWindow) ;} ;
		virtual	void	DoMouseDown(XEvent *) {} ;
		virtual	void	DoKeyDown(XEvent *) {} ;
		virtual	void	DoKeyUp(XEvent *) {} ;
		void	SetParent(TPane *parent) { mParentPane = parent ; } ;
		TPane	*GetParent() { return mParentPane ; } ;

	protected:
		Window	mXWindow ;
		TPane 	*mParentPane ;
} ;

class TPopUpWindow : public TGrafPort {
	public :
		TPopUpWindow(TPane *parent,short xloc,short yloc,short width,short height) ;
		virtual	~TPopUpWindow() ;

		Window  GetXWindow(void) { return mXWindow ; } ;
		virtual	void	Show() { XMapRaised(gDisplay,mXWindow); } ;
		virtual	void	Hide() { XUnmapWindow(gDisplay,mXWindow) ;} ;
		virtual	void	Draw() {} ;
		TPane	*GetParent() { return mParentPane ; } ;
		Boolean	mParentIsRoot ;

	protected:
		Window	mXWindow ;
		TPane 	*mParentPane ;
} ;

class TWindow : public TPane {
	public:
		TWindow() {} ;
		TWindow(short xloc,short yloc,short width,short height,
			short depth,char *title,Boolean hasOffscreen) ;
		virtual	~TWindow() ;

		Region		mUpdateRegion ;

		void	SetWindowTitle(char *title);
		void	GetWindowTitle(char *title) ;
		void	SetWindowList(TWindowList *list) { mWindowList = list ; } ;
		void	Show() ;
		TOffscreen *GetOffscreen() { return mOffscreen ; } ;

		
		virtual	void	DoMouseDown(XEvent *event) ;
		virtual	void	DoKeyDown(XEvent *event) ;
		virtual	void	DoKeyUp(XEvent *event) ;
		virtual void	Update(XEvent *event) ;
		virtual void	Resize(XEvent *event) ;
		virtual	void	DoIdle() ;
		virtual void	SelectCursor() ;
		virtual	void	DoClientMessage(XClientMessageEvent *event) ;
		virtual	void	Close() ;
		virtual	void	DoWMRequestDelete() {} ;
		
		Boolean	IsValid() { return mIsValid ; } ;
		Boolean IsVisible() { return mIsVisible ; } ;

		friend	void	EraseWindow(TWindow *window) ;

		
	protected:

		virtual void 	Draw() ;
		virtual	void	Print() {} ;
		virtual void	Erase(Rect *rect = nil) ;
		
		Boolean	mIsValid ;
		Boolean mIsVisible ;
		
		TOffscreen	*mOffscreen ;			
		TWindowList	*mWindowList ;

		
		WindowType	mWindowType ;
		char		mWindowTitle[256] ;
		
} ;

class	TFrameWindow : public TWindow {
	public:
		TFrameWindow(short xloc,short yloc,short width,short height,
			short depth,char *title) ;
		virtual	~TFrameWindow() ;
		void	AddPane(TPane *pane) ; 
		void	DeletePane(TPane *pane) ; 
		virtual	void	Draw() ;
		virtual	void	DoMouseDown(XEvent *event) ;
		virtual	void	DoKeyDown(XEvent *event) {} ;
		virtual	void	DoKeyUp(XEvent *event) {} ;

	protected:
		TPaneList	*mPaneList ;
		Boolean		mHasMenuBar ;
		
} ;

typedef	TGrafPort	*GrafPtr ;
typedef TWindow 	*WindowPtr ;
typedef TOffscreen	*OffScreenPtr ;


inline void BringToFront(TWindow *window) { XMapRaised(gDisplay,window->GetXWindow()) ; } ;
inline void EraseWindow(TWindow *window) { window->Erase() ; }  ;

#endif

