/* ----------------------------------------------------------------- */
/* Draws 3d balls in the appropriate x,y,z positions                 */
/* the x,y,z positions passed have to be scaled by a factor of 1/2   */
/* to fit onto the display and the order of atoms to be drawn has    */
/* to be in the reverse order of the one passed                      */
/* ----------------------------------------------------------------- */
#include <math.h>
#include "storage.h"
#include "devices.h"
#include "globals.h"
#include "errors.h"
#include "funcdefs.h"
#define FALSE 0 
void ball3d(short *atma,short *atmb,short *atmc,short *atmr,long natom){
	Rect uRect ;
	float noise=4 ;        	/* Noise value */
	float dir1=1.,
			dir2=1.,
			dir3=3. ;		/* Direct light	*/
	unsigned char *ballb ;
	long ballps[500],mFact ;
	float pi=3.141592765 ;
	float bot=0.,top ;
	float rnm,v,x,xx,y,yy,z,r,rr,rs,scale,ylast,t1 ;
	register long i,j,na,n ;
	long ptr,jlast,i1,j1,ir,intens,nrows,npixels,xs,ys ;
	long i0=0,iv1=1 ;

	top = OutDevice->range ;
	mFact = (OutDevice->range+1)/32 ;
	ballb = (unsigned char *)gWorkSpace ;
	scale = 1.0 ;         /* To go from the CRT to the FB */
	rnm = 0. ;

	/*  generate appropriately sized/shaded balls for all atom species */

	/* Ballps works as a pointer into a large array containing all the */
	/* rows for each different size ball			       */
	for(i=0;i<natom;i++) {
		ballps[atmr[i]] = -1 ;
	}

	ptr = 0 ;
	for(na=0;na<natom;na++) {
		n = atmr[na] ;
		/* Already defined ? */
		if (ballps[n] == -1) {
			ballps[n] = ptr ;		/* Pointer to beg. of new row */
			/* Generate ball data itself	*/
			/* One long stream of data	*/
			r = atmr[na] ;			/* Radius of the ball	*/
			ir = (long)r ;
			rr = r*r ;			/* Radius squared	*/
			/* Set up for the light */
			rs = r*sqrt(dir1*dir1+dir2*dir2+dir3*dir3) ;
			for(y=r;y >= -r;y -= 1.) {
				yy = y*y ;
				i = (long)sqrt(rr-yy) ;	/* 1/2 pixels in row	*/
				*(ballb+ptr) = i ;	/* Enter into array	*/
				j = 2*i ;		/* Number of pixels	*/
				x = -i ;		/* Starting x-value	*/
				ptr++ ;
				for(i=0;i<j;i++,x += 1.) {
					v = 0. ;
					t1 = (x*dir1+y*dir2+sqrt(rr-yy-x*x)*
					dir3)/rs ;
					if (t1 > 0.)
						v = v + t1*30.99 ;
					if (noise > 0.) {
						rnm = rnm*19.87931+3.48529 ;
						rnm = rnm - (long)rnm ;
						v += (rnm-0.5)*noise ;
						if (v < 0.) v = 0. ;
						if (v > 30.99) v = 30.99 ;
					}
					intens = (long) v*mFact+mFact ;  
					if (intens > OutDevice->range) 
						intens = OutDevice->range ;
					if (OutDevice->type == WS) 
						intens = gLut[intens] ;
					*(ballb+ptr) = intens ;
					ptr++ ;
				}
			}
		}
	}
	/* Loop over all the atoms within the unit cell */
	UpdateImageScreen() ;
	for(na=natom-1; na >= 0; na--) {
		if (abandn()) {
			gAbortStatus = FALSE ;  /* Clear the flag that is set */
			return ;
		}
		x = atma[na]*scale ;
		y = atmb[na]*scale ;
		n = atmr[na] ;

		/* Draw the atom */
		i1 = near(x) ;		/* X location of the centre */
		j1 = near(y) ;		/* Y location of the centre */
		j1 = OutDevice->yres-1-j1 ;/* Because (0,0) is bottom left logically */
		/* in the atom positions		      */
		ptr = ballps[n] ;	/* Starting place for this ball */
		nrows = 2*n ;		/* Number of rows for this ball */
		SetRect(&uRect,i1-n-1,j1-n-1,i1+n+1,j1+n+1) ;
		/* Loop over each row */
		for(i=0,j=j1-n; i<=nrows; i++,j++) {
			n = *(ballb+ptr) ;
			npixels = 2*n ;
			ptr++ ;
			xs = i1 - n ;
			ys = j ;
			stemdro((char *)(ballb+ptr),&iv1,&npixels,&iv1,&i0,&xs,&ys,&bot,&top) ;
			ptr += npixels ;
		}
		if (OutDevice->type == WS) {
			UpdateFromOffScreenRect(gDisplayWindow,gOffscreenPtr,&uRect) ;
			XFlush(gDisplay) ;
		}
	}
}

// ----------------------------------------------------------------
// We need this routine to update the screen when we are using	
// atom3d since we don't have the proper c definitions to call the	
// proper routines directly		
// ----------------------------------------------------------------
void FUpdateRect(long x1,long y1,long x2,long y2){
    Rect uRect ;

    if (OutDevice->type == WS) {
		SetRect(&uRect,x1,y1,x2,y2) ;
		UpdateFromOffScreenRect(gDisplayWindow,gOffscreenPtr,&uRect) ;
		XFlush(gDisplay) ;
    }
}    	
