// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include <string.h>
#include "header.h"
#include "funcdefs.h"
#include "menudef.h"
#include "params.h"
#include "PhasegratingRecord.h"
#include "MslicRecord.h"
#include "displ.h"
#include "layers.h"
#define IDY 5
extern long gNumUnitCellAtoms,gNumSymops,LAYERED ;
extern char stext[] ;
void displ_var() {
  /*******************************************************************/
  /* Displays all the simulation parameters such as structure inform.*/
  /*  multislice parameters and microscope parameters                */
  /*******************************************************************/

    long i,j ;
    char stemp[80] ;

     for (i=0;i<gNumInputParameters;i++)
       {
        switch(i) 
           {
             case 0 : /* Title of the structure */            
                      sprintf(stext,parameter[i].txt) ;
                      sprintf(stemp,"%s",gInputParameter[i].cv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 1 : /* Space group number */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%3d",gInputParameter[i].iv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 2 : /* Length of the A-axis */
             case 3 : /* Length of the B-axis */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%8.2f",gInputParameter[i].fv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 4 : /* Length of the C-axis */
                      if (!LAYERED)
                        {
                          sprintf(stext,parameter[i].txt) ; 
                          sprintf(stemp,"%8.2f",gInputParameter[i].fv) ;
                          strcat(stext,stemp) ;
                          temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                        }
                      break ;
             case 5 : /* ALPHA in degrees   */
             case 6 : /* BETA in degrees    */
             case 7 : /* GAMMA in degrees   */
             case 8 : /* GMAX in rec. Angs. */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%8.2f",gInputParameter[i].fv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 9 : /* ZONE - axis   */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%d %d %d",gPhsgrtRec->iuvw[0],
                              gPhsgrtRec->iuvw[1],gPhsgrtRec->iuvw[2]) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 10: if (!LAYERED)
                      /* Number of symmetry operators */
                        {
                          sprintf(stext,parameter[i].txt) ; 
                          sprintf(stemp,"%3d",gInputParameter[i].iv) ;
                          strcat(stext,stemp) ;
                          temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                        }
                      else
                      /* Number of different Layers in the structure */
                        {
                          sprintf(stext,"Numb. of Different Layers :  %d",
                                  gLayerRec.num_pg) ;
                          temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                        }            
                      break ;
             case 11: if (!LAYERED)
                      /* Number of slices per unit cell */
                        {
                          sprintf(stext,parameter[i].txt) ; 
                          sprintf(stemp,"%3d",gInputParameter[i].iv) ;
                          strcat(stext,stemp) ;
                          temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                        }
                      else
                        {
                          sprintf(stext,"Tot. Numb. of Slices :  %d",
                                  gLayerRec.num_slc) ;
                          temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                        }
                      break ;
             case 12: /* Number of atoms in the basis */
             case 13: /* Number of different types of atoms in the basis */
                      if (!LAYERED)                      
                        {
                          sprintf(stext,parameter[i].txt) ; 
                          sprintf(stemp,"%3d",gInputParameter[i].iv) ;
                          strcat(stext,stemp) ;
                          temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                        }
                      break ;
             case 14: /* Microscope name */
                      sprintf(stext,parameter[i].txt) ;
                      sprintf(stemp,"%s",gInputParameter[i].cv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 15: /* Spherical abberation in mm */
             case 16: /* Spread of defocus in Angst */
             case 17: /* Divergence in mrad (half)  */
             case 18: /* Microscope voltage in kV   */
             case 19: /* Center of the Laue circle in units of H */
             case 20: /* Center of the Laue circle in units of K */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%8.2f",gInputParameter[i].fv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 21: /* Specimen thickness in A */
                      sprintf(stext,parameter[i].txt) ; 
                      if (gMsliceRec.tts)
                          sprintf(stemp,"%.0f(%.0f)%.0f",
                                  gMsliceRec.t1,gMsliceRec.dt,gMsliceRec.t2) ;
                      else
                         sprintf(stemp,"%8.2f",gInputParameter[i].fv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 22: /* Amplitiude output Y/N */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%s",gInputParameter[i].cv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      if (gMsliceRec.iplot== 1)
                        {
                          temdtx(520,450,"The indices are :     h    k    l ") ;
                          for(j=0;j<gMsliceRec.nw;j++)
                            {
                              sprintf(stext,"%3d %3d %3d",
                                      gMsliceRec.ihw[j],gMsliceRec.ikw[j],gMsliceRec.ilw[j]);
                              temdtx(664,420-j*24,stext) ;
                            }
                        }
                      break ;
             case 23: /* Objective lens defocus */
                      sprintf(stext,parameter[i].txt) ; 
                      if (gDisplayRec.tfs)
                          sprintf(stemp,"%.0f(%.0f)%.0f",
                                  gDisplayRec.f1,gDisplayRec.df,gDisplayRec.f2) ;
                      else
                          sprintf(stemp,"%8.2f",gInputParameter[i].fv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
             case 24: /* Size of the objective aperture in rec. Angst. */
             case 25: /* Center of the ojective aperture in units of H */
             case 26: /* Center of the ojective aperture in units of K */
             case 27: /* Center of the optic axis in units of H */
             case 28: /* Center of the optic axis in units of K */
                      sprintf(stext,parameter[i].txt) ; 
                      sprintf(stemp,"%8.2f ",gInputParameter[i].fv) ;
                      strcat(stext,stemp) ;
                      temdtx(parameter[i].xl,parameter[i].yl+IDY,stext) ;
                      break ;
           }
        }
  } 

