// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
/*-------------------------------------------------------------------*/
/* Collection of routines to pick up the requested source            */
/*-------------------------------------------------------------------*/
#include "header.h"
#include "funcdefs.h"
#include "menudef.h"
#include "menu_index.h"
#include "PhasegratingRecord.h"
#include "displ.h"
#include "MslicRecord.h"
#include "dfpdspl.h"
#include "globals.h"
extern MENUS cmndptrs[] ;
extern FPV funcpntrs[] ;
extern long no_cmnd,NUM_THICK,NUM_DEFOC ;
extern long DIFFRPT,IMAGE_MODE,HISTOGRAM,
           SCALED,EXIT_WAVEF ;
extern short gSliceThreeD ;
extern long MONTAGE ;
extern struct {
       long num_pot ;      // Signals which potential to use if nz != 1 */
       long num_wf ;       // Signals which exit wavef. to use when we store
                          //   more than one thickness in a single file, e.g.
                          //   a through thickness series.
                          
       long num_image ;    // Signals which image to use when multiple images
                          //   are stored in a single file. Thru-focus,thru-thick
                          //
       } whichOne ;  
extern char stext[] ;

// --------------------------------------------------------------------------
// Set-up routine for display of diffraction patterns. Invoked by
//  the "DIFFR" command in the DISPLAY menu                      
// --------------------------------------------------------------------------
void r_diff() {

    eraser(0,0,1020,32) ;
    setup_trnsf() ;
    if (no_cmnd == 0) {
		cmndptrs[no_cmnd] = gCurrentButton ;
		funcpntrs[no_cmnd++] = (FPV)diffr ;
		on_highlight() ;
		whichOne.num_wf = 1 ;
		if (gMsliceRec.tts) {
			REVERSE_VIDEO ;
			sprintf(stext," Select which thickness (1-%d) -> ",
			NUM_THICK) ;
			temdtx(0,10,stext) ;
			whichOne.num_wf = get_int(1) ;
			if (whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0)
				do {
					eraser(0,0,1020,32) ;
					BELL ;
					sprintf(stext,
					" Only %d thicknesses available ! Re-enter -> ",NUM_THICK) ;
					temdtx(0,10,stext) ;
					whichOne.num_wf = get_int(1) ;
				} while( whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0) ;
			NORMAL_VIDEO ;
			eraser(0,0,1020,32) ;
		}
		set_menu(DSPL_OPER) ;
		DIFFRPT = TRUE ; IMAGE_MODE = FALSE ; HISTOGRAM = FALSE ;
		SCALED = FALSE ; EXIT_WAVEF = FALSE ;
		dfpdspl.wavef  = TRUE  ;
	}
    else  if (no_cmnd == 1) {
		set_menu(DSPL_SOURCE) ;
		gCurrentCommand = cmndptrs[--no_cmnd] ;
		off_highlight() ;
		cmndptrs[no_cmnd] = gCurrentButton ;
		funcpntrs[no_cmnd++] = (FPV)diffr ;
		on_highlight() ;
		whichOne.num_wf = 1 ;
		if (gMsliceRec.tts) {
			REVERSE_VIDEO ;
			sprintf(stext," Select which thickness (1-%d) -> ",NUM_THICK) ;
			temdtx(0,10,stext) ;
			whichOne.num_wf = get_int(1) ;
			if (whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0)
				do {
					eraser(0,0,1020,32) ;
					BELL ;
					sprintf(stext,
						" Only %d thicknesses available ! Re-enter -> ",NUM_THICK) ;
					temdtx(0,10,stext) ;
					whichOne.num_wf = get_int(1) ;
				} while( whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0) ;
			NORMAL_VIDEO ;
			eraser(0,0,1020,32) ;
		}
		set_menu(DSPL_OPER) ;
		DIFFRPT = TRUE ; IMAGE_MODE = FALSE ; HISTOGRAM = FALSE ;
		SCALED = FALSE ; EXIT_WAVEF = FALSE ;
		dfpdspl.wavef  = TRUE  ;
	}
    else if (no_cmnd > 1) {
		err_disp(2) ;
		return ;
    }
}

// --------------------------------------------------------------------------
// Set up routine for manipulating the projected potential    
//  Invoked from the DISPLAY menu                            
// --------------------------------------------------------------------------
void r_pot() {

	eraser(0,0,1020,32) ;
	setup_trnsf() ;
	if (no_cmnd == 0) {
		cmndptrs[no_cmnd] = gCurrentButton ;
		funcpntrs[no_cmnd++] = (FPV)GetPotential ;
		on_highlight() ;
		whichOne.num_pot = 1 ;
		if (gPhsgrtRec->nz != 1 && gSliceThreeD) {
			REVERSE_VIDEO ;
			sprintf(stext," Select which phase-grating (1-%d) -> ",gPhsgrtRec->nz) ;
			temdtx(0,10,stext) ;
			whichOne.num_pot = get_int(1) ;
			if (whichOne.num_pot > gPhsgrtRec->nz || whichOne.num_pot <= 0)
				do {
					eraser(0,0,1020,32) ;
					BELL ;
					sprintf(stext,
					" Only %d phase-gratings available ! Re-enter -> ",gPhsgrtRec->nz) ;
					temdtx(0,10,stext) ;
					whichOne.num_pot = get_int(1) ;
				} while(whichOne.num_pot > gPhsgrtRec->nz || whichOne.num_pot <= 0) ;
			NORMAL_VIDEO ;
			eraser(0,0,1020,32) ;
		}
		set_menu(DSPL_OPER) ;
		IMAGE_MODE = TRUE ; DIFFRPT = FALSE ; HISTOGRAM = FALSE ;
		SCALED = FALSE ; EXIT_WAVEF = FALSE ;
	}
	else if (no_cmnd == 1) {
		set_menu(DSPL_SOURCE) ;
		gCurrentCommand = cmndptrs[--no_cmnd] ;
		off_highlight() ;
		cmndptrs[no_cmnd] = gCurrentButton ;
		funcpntrs[no_cmnd++] = (FPV)GetPotential ;
		on_highlight() ;
		whichOne.num_pot = 1 ;
		if (gPhsgrtRec->nz != 1 && gSliceThreeD) {
			REVERSE_VIDEO ;
			sprintf(stext," Select which phase-grating (1-%d) -> ",
			gPhsgrtRec->nz) ;
			temdtx(0,10,stext) ;
			whichOne.num_pot = get_int(1) ;
			if (whichOne.num_pot > gPhsgrtRec->nz || whichOne.num_pot <= 0)
				do {
					eraser(0,0,1020,32) ;
					BELL ;
					sprintf(stext,
					" Only %d phase-gratings available ! Re-enter -> ",gPhsgrtRec->nz) ;
					temdtx(0,10,stext) ;
					whichOne.num_pot = get_int(1) ;
				} while(whichOne.num_pot > gPhsgrtRec->nz || whichOne.num_pot <= 0) ;
			NORMAL_VIDEO ;
			eraser(0,0,1020,32) ;
		}
		set_menu(DSPL_OPER) ;
		IMAGE_MODE = TRUE ; DIFFRPT = FALSE ; HISTOGRAM = FALSE ;
		SCALED = FALSE ; EXIT_WAVEF = FALSE ;
	}
	else if (no_cmnd > 1) {
		err_disp(2) ;
		return ;
	}
}

// --------------------------------------------------------------------------
// Set up routine for manipulating the exit wavefunction   
//  Invoked from the DISPLAY menu                         
// --------------------------------------------------------------------------
void r_exwf() {

	eraser(0,0,1020,32) ;
	setup_trnsf() ;
	if (no_cmnd == 0) {
		cmndptrs[no_cmnd] = gCurrentButton ;
		funcpntrs[no_cmnd++] = (FPV)GetExitWave ;
		on_highlight() ;
		set_menu(DSPL_OPER) ;
		whichOne.num_wf = 1 ;
		if (gMsliceRec.tts && !MONTAGE) {
			REVERSE_VIDEO ;
			sprintf(stext," Select which thickness (1-%d) -> ",NUM_THICK) ;
			temdtx(0,10,stext) ;
			whichOne.num_wf = get_int(1) ;
			if (whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0)
				do {
					eraser(0,0,1020,32) ;
					BELL ;
					sprintf(stext,
					" Only %d thicknesses available ! Re-enter -> ",NUM_THICK) ;
					temdtx(0,10,stext) ;
					whichOne.num_wf = get_int(1) ;
				} while( whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0) ;
			NORMAL_VIDEO ;
			eraser(0,0,1020,32) ;
		}
		IMAGE_MODE = TRUE ; DIFFRPT = FALSE ; HISTOGRAM = FALSE ;
		SCALED = FALSE ; EXIT_WAVEF = TRUE ;
		dfpdspl.wavef  = TRUE  ;
	}
	else if (no_cmnd == 1) {
		set_menu(DSPL_SOURCE) ;
		gCurrentCommand = cmndptrs[--no_cmnd] ;
		off_highlight() ;
		cmndptrs[no_cmnd] = gCurrentButton ;
		funcpntrs[no_cmnd++] = (FPV)GetExitWave ;
		on_highlight() ;
		whichOne.num_wf = 1 ;
		if (gMsliceRec.tts && !MONTAGE) {
			REVERSE_VIDEO ;
			sprintf(stext," Select which thickness (1-%d) -> ",NUM_THICK) ;
			temdtx(0,10,stext) ;
			whichOne.num_wf = get_int(1) ;
			if (whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0)
				do {
					eraser(0,0,1020,32) ;
					BELL ;
					sprintf(stext,
					" Only %d thicknesses available ! Re-enter -> ",NUM_THICK) ;
					temdtx(0,10,stext) ;
					whichOne.num_wf = get_int(1) ;
				} while( whichOne.num_wf > NUM_THICK || whichOne.num_wf <= 0) ;
			NORMAL_VIDEO ;
			eraser(0,0,1020,32) ;
		}
		set_menu(DSPL_OPER) ;
		IMAGE_MODE = TRUE ; DIFFRPT = FALSE ; HISTOGRAM = FALSE ;
		SCALED = FALSE ; EXIT_WAVEF = TRUE ;
		dfpdspl.wavef  = TRUE  ;
		}
	else if (no_cmnd > 1) {
		err_disp(2) ;
		return ;
	}
}

// --------------------------------------------------------------------------
// Set up routine for manipulating the computed image  
//  Invoked from the DISPLAY menu                     
// --------------------------------------------------------------------------
void r_image() {
    long total_num_images ;
    eraser(0,0,1020,32) ;
    setup_trnsf() ;
    if (no_cmnd == 0)
    {
      cmndptrs[no_cmnd] = gCurrentButton ;
      funcpntrs[no_cmnd++] = (FPV)GetImage ;
      on_highlight() ;
      whichOne.num_image = 1 ;
      if (gMsliceRec.tts || gDisplayRec.tfs)
        {
         if (!MONTAGE)
          {
           total_num_images = NUM_THICK*NUM_DEFOC ;
           REVERSE_VIDEO ;
           sprintf(stext," Select which image (1-%d) -> ",
                   total_num_images) ;
           temdtx(0,10,stext) ;
           whichOne.num_image = get_int(1) ;
     	   if (whichOne.num_image > total_num_images || whichOne.num_image <= 0)
             do {
                 eraser(0,0,1020,32) ;
                 BELL ;
                 sprintf(stext,
                 " Only %d images available ! Re-enter -> ",total_num_images) ;
                 temdtx(0,10,stext) ;
                 whichOne.num_image = get_int(1) ;
                } while( whichOne.num_image > total_num_images || 
                         whichOne.num_image <= 0) ;
            NORMAL_VIDEO ;
            eraser(0,0,1020,32) ;
          }
	  else if (MONTAGE && gCustomMontage) {
		SetCustomMontageSequence() ;
	  }
        }
      set_menu(DSPL_OPER) ;
      IMAGE_MODE = TRUE ; DIFFRPT = FALSE ; HISTOGRAM = FALSE ;
      SCALED = FALSE ; EXIT_WAVEF = FALSE ;
    }
    else if (no_cmnd == 1)
    {
      set_menu(DSPL_SOURCE) ;
      gCurrentCommand = cmndptrs[--no_cmnd] ;
      off_highlight() ;
      cmndptrs[no_cmnd] = gCurrentButton ;
      funcpntrs[no_cmnd++] = (FPV)GetImage ;
      on_highlight() ;
      whichOne.num_image = 1 ;
      if (gMsliceRec.tts || gDisplayRec.tfs)
        {
         if (!MONTAGE)
          {
           total_num_images = NUM_THICK*NUM_DEFOC ;
           REVERSE_VIDEO ;
           sprintf(stext," Select which image (1-%d) -> ",
                   total_num_images) ;
           temdtx(0,10,stext) ;
           whichOne.num_image = get_int(1) ;
           if (whichOne.num_image > total_num_images || whichOne.num_image <= 0)
             do {
                 eraser(0,0,1020,32) ;
                 BELL ;
                 sprintf(stext,
                 " Only %d images available ! Re-enter -> ",total_num_images) ;
                 temdtx(0,10,stext) ;
                 whichOne.num_image = get_int(1) ;
                } while( whichOne.num_image > total_num_images || 
                         whichOne.num_image <= 0) ;
            NORMAL_VIDEO ;
            eraser(0,0,1020,32) ;
          }
        }
      set_menu(DSPL_OPER) ;
      IMAGE_MODE = TRUE ; DIFFRPT = FALSE ; HISTOGRAM = FALSE ;
      SCALED = FALSE ; EXIT_WAVEF = FALSE ;
    }
    else if (no_cmnd > 1) 
    {
      err_disp(2) ;
      return ;
    }
}

