// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
// --------------------------------------------------------------------------
// This is a collection of routines that reads and writes to the intermed. 
// input and output files. There are also routines to open and close files.
// --------------------------------------------------------------------------
#include <stdio.h>
#include "files.h"
#include "types.h"
FILE 	*finput,*foutput,*finputs[8] ;
FILE	*flinput ;
void ReportFileOpenError(char *) ;
Boolean	FileExists(char *filename) ;

Boolean	FileExists(char *filename) ;

Boolean	FileExists(char *filename) {
	FILE *fp = fopen(filename,"r") ;
	if (fp != NULL) {
		fclose(fp) ;
		return true ;
	}
	return false ;
}

// --------------------------------------------------------------------------
// Open one of the output files. (mout,pout..) 
// --------------------------------------------------------------------------
long open_output_file(char *filename) {
  	foutput = fopen(filename,"w") ;
	if (foutput==NULL) {
		ReportFileOpenError(filename) ;
		return(0) ;
	}
	return(1) ;
}

// --------------------------------------------------------------------------
// Write 'numlong' long words to the output file	
// --------------------------------------------------------------------------
long write_output_file(char *buffer,long numlong) {
  	long num_written ;
	
	num_written = fwrite(buffer,4,numlong,foutput) ;
	if(num_written != numlong)
		return(0) ;
	return(1) ;
}

long close_output_file() {
  	if(!fclose(foutput))
		return(1) ;
	return(0) ;
}

long open_input_file(char *filename) {
  	finput = fopen(filename,"r") ;
	if(finput==NULL) {
		ReportFileOpenError(filename) ;
		return(0) ;
	}
	return(1) ;
}

long read_input_file(long num_image,char *buffer,long numlong) {
  	long num_bytes,num_read ;
	
	/* Position into the file */
	num_bytes = (num_image-1)*numlong*4 ;
	if(!fseek(finput,num_bytes,0)) {
		num_read = fread(buffer,4,numlong,finput) ;
		if(num_read != numlong)
			return(0) ;
		return(1) ;
	}
	else 
		return(0) ;
}

long close_input_file(){
  	if(!fclose(finput))
		return(1) ;
	return(0) ;
}

long open_lay_input_file(char *filename) {
  	if((flinput=fopen(filename,"r"))==NULL) {
		ReportFileOpenError(filename) ;
		return(0) ;
	}
	return(1) ;
}

long open_input_files(char *filename,long numpg) {
  	if((finputs[numpg]=fopen(filename,"r"))==NULL)
		return(0) ;
	return(1) ;
}

long read_lay_input_file(char *buffer,long numlong) {
  	long num_read ;
	num_read = fread(buffer,4,numlong,flinput) ;
	if (num_read != numlong)
		return(0) ;
	return(1) ;
}

long read_input_files(long npg,char *buffer,long numlong) {
  	long num_read ;
	
	/* Position into the file */
	if(!fseek(finputs[npg-1],0,0)) {
		num_read = fread(buffer,4,numlong,finputs[npg-1]) ;
		if (num_read != numlong)
			return(0) ;
	}
	return(1) ;
}

long close_lay_input_file() {
  	if(!fclose(flinput))
		return(1) ;
	return(0) ;
}

long close_input_files(long num_file) {
  	if(!fclose(finputs[num_file]))
		return(1) ;
	return(0) ;
}
	
