// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
#include <math.h>
#include "funcdefs.h"
#include "header.h"
#include "PhasegratingRecord.h"
#include "ProjCellRecord.h"
#include "xtal.h"
#define FALSE 0
#define TRUE 1
extern  long	gSliceThreeD ;
#define fabs(x)		((x) < 0. ? -(x) : (x))
#define abs(x)		((x) < 0  ? -(x) : (x))
void CalculateProjectedCell(float *av,float *bv,float *cv,long ihkl[][3],float *gsq,
	long ic,float rm[3][3],long *inx,long *iny,long *inz) {
    /*-------------------------------------------------------------------- */
    /* This routine finds a new unit cell. A new unit cell can be a	   */
    /* true 3-dimensional unit cell or just a 2-dimensional projected	   */
    /* cell. The way to tell is if the new A and B are an integer sum	   */
    /* of the old A,B and C vectors. The routine has in COMMON the old	   */
    /* lattice parameters and gets passed the array IHKL which is a sorted */
    /* list of g-vectors perpendicular to the ZONE axis, smallest g first. */
    /* The array GSQ contains the length squared of the g-vectors.	   */
    /* RM is the rotation matrix that transforms old positions into new	   */
    /* ones.								   */
    /* --------------------------------------------------------------------*/
    long iav[3],ibv[3],icv[3],iu,iv,iw,ix[8],iy[8],iz[8],isign ;
    float g1[3],g2[3],g3[3],vect[3],uvw[3],astnsq,bstnsq,cstnsq,anewsq,bnewsq,
    	  cnewsq ;
    float ba,ca,bast,cast,asq,bsq,csq,astsq,bstsq,cstsq,zonesq,dot,g1sq,g2sq,
    	  g2m,g1m,csth,gastr,gastnu,rh1,rk1,rl1,rh2,rk2,rl2,rh3,rk3,rl3,
    	  rathl,ratkl,rathk,ratlk,rh,rk,rl,pi=3.14159265 ;
    long i,j,success,lstart,lstop,lstep,kstart,kstop,kstep,nh,nk,nl,i1fact,
    	  i2fact ;
	
    char text[80] ;
	

    ba = gPhsgrtRec->b/gPhsgrtRec->a ;
    ca = gPhsgrtRec->c/gPhsgrtRec->a ;
    bast = xtal.bst/xtal.ast ;
    cast = xtal.cst/xtal.ast ;
    asq = gPhsgrtRec->a*gPhsgrtRec->a ;
    bsq = gPhsgrtRec->b*gPhsgrtRec->b ;
    csq = gPhsgrtRec->c*gPhsgrtRec->c ;
    astsq = xtal.ast*xtal.ast ;
    bstsq = xtal.bst*xtal.bst ;
    cstsq = xtal.cst*xtal.cst ;
    iu = gPhsgrtRec->iuvw[0] ;
    iv = gPhsgrtRec->iuvw[1] ;
    iw = gPhsgrtRec->iuvw[2] ;

    for(i=0;i<3;i++)
	uvw[i] = gPhsgrtRec->iuvw[i] ;	/* Real numbers for accuracy */
		
    zonesq = 	iu*iu*asq+iv*iv*bsq+iw*iw*csq + 
    		2.*iu*iv*gPhsgrtRec->a*gPhsgrtRec->b*xtal.csga +
	 	2.*iu*iw*gPhsgrtRec->a*gPhsgrtRec->c*xtal.csbe + 
    		2.*iv*iw*gPhsgrtRec->b*gPhsgrtRec->c*xtal.csal ;
			 
    for(i=0;i<3;i++)
	g1[i] = ihkl[1][i] ;	/* G1 the first g-vector (0- central) */
		
	
    DotProduct(g1,g1,1.,bast,cast,xtal.csalst,xtal.csbest,xtal.csgast,&g1sq) ;
    g1m = sqrt(g1sq) ;

    /* Go through the list of g-vectors and find the next g-vector 	*/
    /* that makes an angle <= 90 degrees and crossed with G1	        */
    /* gives a vector parallell to the ZONE axis.			*/
	
    for(i=2;i<ic;i++) {
	for(j=0;j<3;j++)
		g2[j] = (float)ihkl[i][j] ;
	CrossProduct(g1,g2,1.,vect) ;			/* G1 * G2 */
	DotProduct(g1,g2,1.,bast,cast,xtal.csalst,xtal.csbest,xtal.csgast,&dot) ;
	DotProduct(g2,g2,1.,bast,cast,xtal.csalst,xtal.csbest,xtal.csgast,&g2sq) ;
	g2m = sqrt(g2sq) ;
	csth = dot/(g1m*g2m) ; /* Get the angle between G2 and G1 */
	if (csth < -1.) csth = -1. ;
	if (csth > 1.)  csth =  1. ;
	gastr = acos(csth) ;
	gastnu = gastr*180./pi ;
	if (gastnu < 90.2 && gastnu > 45. )   {
		DotProduct(uvw,vect,1.,ba,ca,xtal.csal,xtal.csbe,xtal.csga,&dot) ;
		if (fabs(dot) > 0.)
		break ;	/* The ZONE is parallell to g1*G2! We are done */
	}
    }
	
    /* The indices h,k,l of G3 is given by the following		*/
    /* These are not in general integers.				*/
    g3[0] = 	(iu*gPhsgrtRec->a*gPhsgrtRec->a + iv*gPhsgrtRec->a*gPhsgrtRec->b*xtal.csga + 
    		iw*gPhsgrtRec->a*gPhsgrtRec->c*xtal.csbe)/zonesq ;
    g3[1] = 	(iu*gPhsgrtRec->a*gPhsgrtRec->b*xtal.csga + iv*gPhsgrtRec->b*gPhsgrtRec->b + 
    		iw*gPhsgrtRec->b*gPhsgrtRec->c*xtal.csal)/zonesq ;
    g3[2] = 	(iu*gPhsgrtRec->a*gPhsgrtRec->c*xtal.csbe + 
    		iv*gPhsgrtRec->b*gPhsgrtRec->c*xtal.csal + iw*gPhsgrtRec->c*gPhsgrtRec->c)/zonesq ;

    if (dot < 0.) {
       g3[0] = -g3[0] ;			
       g3[1] = -g3[1] ;			
       g3[2] = -g3[2] ;			
    }

    /* Clean up round off errors for entries that should be zero */
    for (i = 0; i < 3; i++) {
	if (fabs(g1[i]) < 0.0001) g1[i] = 0. ;
	if (fabs(g2[i]) < 0.0001) g2[i] = 0. ;
	if (fabs(g3[i]) < 0.0001) g3[i] = 0. ;
    }
	
    rh1 = g1[0] ;		/* H,K,L of vector G1 */
    rk1 = g1[1] ;
    rl1 = g1[2] ;
    rh2 = g2[0] ;		/* H,K,L of vector G2 */
    rk2 = g2[1] ;
    rl2 = g2[2] ;
    rh3 = g3[0] ;		/* H,K,L of vector G3 */
    rk3 = g3[1] ;
    rl3 = g3[2] ;
	
	
    av[0] = rk2*rl3 - rk3*rl2 ;	/* The new lattice vectors A,B and C */
    av[1] = rh3*rl2 - rh2*rl3 ;	/* in units of the old A,B and C     */
    av[2] = rh2*rk3 - rh3*rk2 ;	/* lattice vectors		     */
    bv[0] = rk3*rl1 - rl3*rk1 ;
    bv[1] = rl3*rh1 - rl1*rh3 ;
    bv[2] = rh3*rk1 - rk3*rh1 ;
    cv[0] = rk1*rl2 - rl1*rk2 ;
    cv[1] = rl1*rh2 - rh1*rl2 ;
    cv[2] = rh1*rk2 - rk1*rh2 ;
	
	
    /* At this point we have the option of making it a true unit cell	*/
    /* or just a projected unit cell. Right now it is a projected	*/
    /* cell. To make it into a true periodic cell we need to make	*/
    /* all AV(I),BV(I) and CV(I) integers. Any multiplication factor	*/
    /* applied to AV, BV or CV would have to be a division for the	*/
    /* G1,G2 or G3							*/
	
    success = FALSE ;
    if(gPhsgrtRec->nz > 1 && gSliceThreeD) { /* Try to make it a true */
    				       /* 3-d unit cell      	*/
	/* Work on the A vector */
	/* Make sure we use as denominator something different from 0. */
	i1fact = 1 ;
	i2fact = 1 ;
	if(fabs(av[2]) > 0.001) {		/* We can use L */
		rathl = av[0]/av[2] ;
		ratkl = av[1]/av[2] ;
		/* Start working on a set of guesses for L */
		if (av[2] < 0.) {
			lstart = -1 ;
			lstop  = 10 ;
			lstep  = -1 ;
		}
		else {
			lstart = 1 ;
			lstop  = 10 ;
			lstep  = 1 ;
		}
		for (nl=lstart;abs(nl)<=lstop;nl+=lstep)  {
			rh = rathl*nl ;
			rk = ratkl*nl ;
			nh = near(rh) ;
			nk = near(rk) ;
			if(fabs(rh-nh) < 0.01 && fabs(rk-nk) < 0.01) {
				success = TRUE ;
				i1fact = nl/av[2] ;
				av[0] = nh ;
				av[1] = nk ;
				av[2] = nl ;
				for(i=0;i<3;i++)
					g1[i] = g1[i]/i1fact ;
				break ;
			}
		}
	}
	else if (fabs(av[1]) > 0.001) /* We can use K */ {
		rathk = av[0]/av[1] ;
		ratlk = av[2]/av[1] ;
		/* Start working on a set of guesses for K */
		if (av[1] < 0.) {
			kstart = -1 ;
			kstop  = 10 ;
			kstep  = -1 ;
		}
		else {
			kstart = 1 ;
			kstop  = 10 ;
			kstep  = 1 ;
		}
		for (nk=kstart;abs(nk)<=kstop;nk+=kstep)  {
			rh = rathk*nk ;
			rl = ratlk*nk ;
			nh = near(rh) ;
			nl = near(rl) ;
			if(fabs(rh-nh) < 0.01 && fabs(rl-nl) < 0.01) {
				success = TRUE ;
				i1fact = nl/av[1] ;
				av[0] = nh ;
				av[1] = nk ;
				av[2] = nl ;
				for(i=0;i<3;i++)
					g1[i] = g1[i]/i1fact ;
				break ;
			}
		}
	}  /* H the only non zero component and we should be okay */
			
	/* Start working on the B vector, but only if we had success with A */
	if(success) {
		/* Work on the B vector */
		/* Make sure we use as denominator something different from 0.*/
		i2fact = 1 ;
		if(fabs(bv[2]) > 0.001)		/* We can use L */ {
			rathl = bv[0]/bv[2] ;
			ratkl = bv[1]/bv[2] ;
			/* Start working on a set of guesses for L */
			if (bv[2] < 0.) {
				lstart = -1 ;
				lstop  = 10 ;
				lstep  = -1 ;
			}
			else {
				lstart = 1 ;
				lstop  = 10 ;
				lstep  = 1 ;
			}
			for (nl=lstart;abs(nl)<=lstop;nl+=lstep)  {
				rh = rathl*nl ;
				rk = ratkl*nl ;
				nh = near(rh) ;
				nk = near(rk) ;
				if(fabs(rh-nh) < 0.01 && fabs(rk-nk) < 0.01) {
					success = TRUE ;
					i2fact = nl/bv[2] ;
					bv[0] = nh ;
					bv[1] = nk ;
					bv[2] = nl ;
					for(i=0;i<3;i++)
						g2[i] = g2[i]/i2fact ;
					break ;
				}
			}
		}
		else if (fabs(bv[1]) > 0.001) /* We can use K */ {
			rathk = bv[0]/bv[1] ;
			ratlk = bv[2]/bv[1] ;
			/* Start working on a set of guesses for K */
			if (bv[1] < 0.) {
				kstart = -1 ;
				kstop  = 10 ;
				kstep  = -1 ;
    			}
			else {
				kstart = 1 ;
				kstop  = 10 ;
				kstep  = 1 ;
			}
			for (nk=kstart;abs(nk)<=kstop;nk+=kstep)  {
				rh = rathk*nk ;
				rl = ratlk*nk ;
				nh = near(rh) ;
				nl = near(rl) ;
				if(fabs(rh-nh) < 0.01 && fabs(rl-nl) < 0.01) {
					success = TRUE ;
					i2fact = nl/bv[1] ;
    					bv[0] = nh ;
					bv[1] = nk ;
					bv[2] = nl ;
					for(i=0;i<3;i++)
						g2[i] = g2[i]/i2fact ;
					break ;
				}
			}
		}  
    		/* H the only non zero component and we should be okay */
	}
	if (!success) {
		/* Send off some alarm */
		/* Reset the vectors   */
		for(i=0;i<3;i++) {
			av[i] /= i1fact ;
			bv[i] /= i2fact ;
			g1[i] *= i1fact ;
			g2[i] *= i2fact ;
		}
	}
    }  /* End of NZ > 1 && THREE_D condition */
	
    SetHKLTransformationMatrix(g1,g2,g3,gProjCell->hkltr) ;/* Set up the transformation matrix that */
				    /* transforms new hkl's long old hkl's    */
										
    rotmat(g1,g2,g3,rm) ;	/* Set up the matrix for transforming	 */
				/* coordinates. It is just the transpose */
				/* of the HKLTR matrix 			 */
	
    /* Find the vertices of a supercell that entirely encloses the new cell  */
    /* To do this we need to find the min and max values in each direction   */
    /* of A,B,C of the vectors AV,BV and CV. Get all the corners of the cube.*/
	
    if(success) /* We know that all the indices are integers */ {
	for(i=0;i<3;i++) {
		iav[i] = av[i] ;
		ibv[i] = bv[i] ;
		icv[i] = cv[i] ;
	}
    }
    else /* Not necessarily integers */ {
	for(i=0;i<3;i++) {
		iav[i] = near(av[i]) ;
		if(fabs(av[i]-iav[i]) > 0.001)	/* not an integer	*/ {
			isign = (av[i] < 0. ? -1 : 1)  ;
			iav[i] = (long)(av[i]+isign) ;
		}
		ibv[i] = near(bv[i]) ;
		if(fabs(bv[i]-ibv[i]) > 0.001)	/* not an integer	*/ {
			isign = (bv[i] < 0. ? -1 : 1)  ;
			ibv[i] = (long)(bv[i]+isign) ;
		}
		icv[i] = near(cv[i]) ;
		if(fabs(cv[i]-icv[i]) > 0.001)	/* not an integer	*/ {
			isign = (cv[i] < 0. ? -1 : 1)  ;
			icv[i] = (long)(cv[i]+isign) ;
		}
	}
    }
		
    ix[0] = 0 ;
    ix[1] = iav[0] ;
    ix[2] = ibv[0] ;
    ix[3] = icv[0] ;
    ix[4] = iav[0]+ibv[0] ;
    ix[5] = iav[0]+icv[0] ;
    ix[6] = ibv[0]+icv[0] ;
    ix[7] = iav[0]+ibv[0]+icv[0] ;
    inx[0] = ix[0] ;
    inx[1] = ix[0] ;
    for(i=1;i<8;i++) {
	inx[1] = MAX(inx[1],ix[i]) ;
	inx[0] = MIN(inx[0],ix[i]) ;
    }
    inx[0]++ ;
    iy[0] = 0 ;
    iy[1] = iav[1] ;
    iy[2] = ibv[1] ;
    iy[3] = icv[1] ;
    iy[4] = iav[1]+ibv[1] ;
    iy[5] = iav[1]+icv[1] ;
    iy[6] = ibv[1]+icv[1] ;
    iy[7] = iav[1]+ibv[1]+icv[1] ;
    iny[0] = iy[0] ;
    iny[1] = iy[0] ;
    for(i=1;i<8;i++) {
	iny[1] = MAX(iny[1],iy[i]) ;
	iny[0] = MIN(iny[0],iy[i]) ;
    }
    iny[0]++ ;
    iz[0] = 0 ;
    iz[1] = iav[2] ;
    iz[2] = ibv[2] ;
    iz[3] = icv[2] ;
    iz[4] = iav[2]+ibv[2] ;
    iz[5] = iav[2]+icv[2] ;
    iz[6] = ibv[2]+icv[2] ;
    iz[7] = iav[2]+ibv[2]+icv[2] ;
    inz[0] = iz[0] ;
    inz[1] = iz[0] ;
    for(i=1;i<8;i++) {
	inz[1] = MAX(inz[1],iz[i]) ;
	inz[0] = MIN(inz[0],iz[i]) ;
    }
    inz[0]++ ;
	
    gProjCell->al = 90. ;
    gProjCell->be = 90. ;
    gProjCell->ga = 180. - gastnu ;
    gProjCell->siga = sin(gastr) ;
    gProjCell->csga = - csth ;		/*  -cos(gast) 	*/
    astnsq = 	g1[0]*g1[0]*astsq+g1[1]*g1[1]*bstsq+g1[2]*g1[2]*cstsq +
    		2.*g1[0]*g1[1]*xtal.ast*xtal.bst*xtal.csgast +
    		2.*g1[0]*g1[2]*xtal.ast*xtal.cst*xtal.csbest +
    		2.*g1[1]*g1[2]*xtal.bst*xtal.cst*xtal.csalst ;
    bstnsq = 	g2[0]*g2[0]*astsq+g2[1]*g2[1]*bstsq+g2[2]*g2[2]*cstsq +
    		2.*g2[0]*g2[1]*xtal.ast*xtal.bst*xtal.csgast +
    		2.*g2[0]*g2[2]*xtal.ast*xtal.cst*xtal.csbest +
    		2.*g2[1]*g2[2]*xtal.bst*xtal.cst*xtal.csalst ;
    cstnsq = 	g3[0]*g3[0]*astsq+g3[1]*g3[1]*bstsq+g3[2]*g3[2]*cstsq +
    		2.*g3[0]*g3[1]*xtal.ast*xtal.bst*xtal.csgast +
    		2.*g3[0]*g3[2]*xtal.ast*xtal.cst*xtal.csbest +
    		2.*g3[1]*g3[2]*xtal.bst*xtal.cst*xtal.csalst ;
    gProjCell->ast = sqrt(astnsq) ;
    gProjCell->bst = sqrt(bstnsq) ;
    gProjCell->cst = sqrt(cstnsq) ;
	
    anewsq = av[0]*av[0]*asq+av[1]*av[1]*bsq+av[2]*av[2]*csq+
    		2.*av[0]*av[1]*gPhsgrtRec->a*gPhsgrtRec->b*xtal.csga +
    		2.*av[0]*av[2]*gPhsgrtRec->a*gPhsgrtRec->c*xtal.csbe +
    		2.*av[1]*av[2]*gPhsgrtRec->b*gPhsgrtRec->c*xtal.csal ;
    bnewsq = bv[0]*bv[0]*asq+bv[1]*bv[1]*bsq+bv[2]*bv[2]*csq+
    		2.*bv[0]*bv[1]*gPhsgrtRec->a*gPhsgrtRec->b*xtal.csga +
    		2.*bv[0]*bv[2]*gPhsgrtRec->a*gPhsgrtRec->c*xtal.csbe +
    		2.*bv[1]*bv[2]*gPhsgrtRec->b*gPhsgrtRec->c*xtal.csal ;
    cnewsq = cv[0]*cv[0]*asq+cv[1]*cv[1]*bsq+cv[2]*cv[2]*csq+
    		2.*cv[0]*cv[1]*gPhsgrtRec->a*gPhsgrtRec->b*xtal.csga +
    		2.*cv[0]*cv[2]*gPhsgrtRec->a*gPhsgrtRec->c*xtal.csbe +
    		2.*cv[1]*cv[2]*gPhsgrtRec->b*gPhsgrtRec->c*xtal.csal ;
    gProjCell->a = sqrt(anewsq) ;
    gProjCell->b = sqrt(bnewsq) ;
    gProjCell->c = sqrt(cnewsq) ;
    gProjCell->vol = gProjCell->a*gProjCell->b*gProjCell->c*gProjCell->siga ;
}

void rotmat(float *v1,float *v2,float *v3,float rm[3][3]) {
    /* ---------------------------------------------------------------- */
    /* This routine sets up the rotation matrix that tranforms old	*/
    /* coordinates into the new ones.					*/
    /* ----------------------------------------------------------------	*/
    register long i ;

    for(i=0;i<3;i++) {
	rm[i][0] = v1[i] ;
	rm[i][1] = v2[i] ;
	rm[i][2] = v3[i] ;
    }
}
