// -----------------------------------------------------------------------------
//  Copyright (c) 2001 Roar Kilaas
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// -----------------------------------------------------------------------------
/*  -------------------------------------------------------------------- */
/* This is a selection of routines that displays and modifies the atoms  */
/* in the basis and the atoms in the cell. Also displays and modifies    */
/* the symmetry operators.                                               */
/*  -------------------------------------------------------------------- */
#include "header.h"
#include "funcdefs.h"
#include "menudef.h"
#include "menu_index.h"
#include "atoms.h"
#include "params.h"
#include "symops.h"
#include "globals.h"
#include "devices.h"
extern MENUS gMenuList[] ;
extern long istart,istop,so_start,so_stop ;
extern long DIS_ATOM_INITIAL,DIS_SO_INITIAL ;
extern long gNumBasisAtoms,gNumUnitCellAtoms,gNumSymops ;
extern char stext[],*chemsymb[],name[] ;
extern long gCalculationIsChanged,gPGisChanged,gBasisIsChanged,gSymopsAreChanged ;
extern long LAYERED ;
extern COMMAND par_atoms[],par_symop[] ;
extern struct { long status ; } calculation ;
#define PG_MASK 1
#define IDY 5

// ---------------------------------------------------------------------
// activates the fields containing data of atompositions as menu - 
// items. If they already are activated, turn them off           
// ---------------------------------------------------------------------
void chng_at() {

    eraser(0,60,960,100) ;
    if (CurrentMenuIs(VIEW_ATOMS)) {
		gCurrentCommand = gCurrentButton ;
		on_highlight() ;
		change_cursor() ;
		set_menu(CH_ATOM) ;
	}
    else if (CurrentMenuIs(VIEW_BASIS)) {
        gCurrentCommand = gCurrentButton ;
        change_cursor() ;
        on_highlight() ;
        set_menu(CH_BASIS_AT) ;
	}
    else if (CurrentMenuIs(CH_ATOM))  {
        off_highlight() ;
        reset_cursor() ;
        set_menu(VIEW_ATOMS) ;
	}
    else if (CurrentMenuIs(CH_BASIS_AT))  {
        off_highlight() ;
        reset_cursor() ;
        set_menu(VIEW_BASIS) ;
        if (gBasisIsChanged) ApplySymmetries() ;
	}
} 
 
// ---------------------------------------------------------------------
// activates the fields containing the Symmetry Operators  
// If they already are activated, turn them off           
// ---------------------------------------------------------------------
void chng_so() {

    gCalculationIsChanged = TRUE ;
    gPGisChanged = TRUE ;
    eraser(0,60,960,100) ;
    if (CurrentMenuIs(VIEW_SYMOP)) {
		gCurrentCommand = gCurrentButton ;
		on_highlight() ;
		change_cursor() ;
		set_menu(CH_SYMOP) ;
    }
    else {
		off_highlight() ;
		reset_cursor() ;
		set_menu(VIEW_SYMOP) ;
    }
}  

// ---------------------------------------------------------------------
// Function to modify a Symmetry Operator         
// ---------------------------------------------------------------------
void mod_symop(long i) {
   
    i-- ;
    eraser((*gCurrentButton)->xl,(*gCurrentButton)->yl,
           (*gCurrentButton)->xh,(*gCurrentButton)->yh) ;
    cp_abs((*gCurrentButton)->xl,(*gCurrentButton)->yl+IDY) ;
    read_symop(so_start+i) ;
    eraser((*gCurrentButton)->xl,(*gCurrentButton)->yl,
           (*gCurrentButton)->xh,(*gCurrentButton)->yh) ;
    temdtx((*gCurrentButton)->xl,(*gCurrentButton)->yl+IDY,txtptr_symop[so_start+i]) ;
    gSymopsAreChanged = TRUE ;
    calculation.status &= ~PG_MASK ;
}    

// ---------------------------------------------------------------------
// Displays the atompositions and corresponding menu  
// ---------------------------------------------------------------------
void show_at() {

    set_menu(VIEW_ATOMS) ;
    CLEAR_DISPLAY ;
    displ_menu() ;
    DIS_ATOM_INITIAL = TRUE ;
    displ_atoms(atom) ;
}

// ---------------------------------------------------------------------
// Displays the symmetry operators and corresponding menu 
// ---------------------------------------------------------------------
void show_symop() {

    set_menu(VIEW_SYMOP) ;
    CLEAR_DISPLAY ;
    displ_menu() ;
    DIS_SO_INITIAL = TRUE ;
    displ_symop() ;
}

// ---------------------------------------------------------------------
// Displays the atompositions in the basis and corresponding menu   
// ---------------------------------------------------------------------
void show_basis(){

    set_menu(VIEW_BASIS) ;
    CLEAR_DISPLAY ;
    displ_menu() ;
    DIS_ATOM_INITIAL = TRUE ;
    displ_atoms(basis) ;
}

// ---------------------------------------------------------------------
// Function to display the next set of 20 atompositions  
// ---------------------------------------------------------------------
void fw_at() {
    long total_number ;

    gCurrentCommand = gCurrentButton ;
    if (CurrentMenuIs(VIEW_ATOMS))
		total_number = gNumUnitCellAtoms ;
    else
		total_number = gNumBasisAtoms ;
    if (istop>=total_number) {
		err(1) ;
		return ;
    }
    istart = istop ;
    eraser(0,60,820,764) ;
    if (CurrentMenuIs(VIEW_ATOMS))
		displ_atoms(atom) ;
    else
		displ_atoms(basis) ;
}

// ---------------------------------------------------------------------
// Function to display the next set of 40 symmetry operators 
// ---------------------------------------------------------------------
void fw_so(){

    if (so_stop>=gNumSymops) {
		err(6) ;
		off_highlight() ;
		return ;
    }
    so_start = so_stop ;
    eraser(0,60,820,764) ;
    displ_symop() ;
}

// ---------------------------------------------------------------------
// function to display the previous set of 20 atom positions
// ---------------------------------------------------------------------
void rw_at(){

    if (istart<20) {
		err(2) ;
		off_highlight() ;
		return ;
    }
    istart = istart-20 ;
    eraser(0,60,820,764) ;
    if (CurrentMenuIs(VIEW_ATOMS))
		displ_atoms(atom) ;
    else
		displ_atoms(basis) ;
} 

// ---------------------------------------------------------------------
// function to display atom positions on a specified page 
// ---------------------------------------------------------------------
void r_page() {
    long npage,total_number ;

    gCurrentCommand = gCurrentButton ;
    on_highlight() ;
    eraser(0,60,1020,100) ;
    REVERSE_VIDEO ;
    temdtx(0,70," Display which page -> ") ;
    npage = get_int(1) ;
    NORMAL_VIDEO ;
    eraser(0,60,1020,100) ;
    istart = (npage-1)*20 ;
    if (istart < 0)
		istart = 0 ;
    else {
		if (CurrentMenuIs(VIEW_ATOMS))
			total_number = gNumUnitCellAtoms ;
		else
			total_number = gNumBasisAtoms ;
		if (istart>=total_number)
			istart = (long)(total_number/20)*20  ;  /* Get the top page */
    }
    eraser(0,60,820,764) ;
    if (CurrentMenuIs(VIEW_ATOMS))
		displ_atoms(atom) ;
    else
		displ_atoms(basis) ;
    off_highlight() ;
} 

// ---------------------------------------------------------------------
// Function to display the next set of 40 symmetry operators    
// ---------------------------------------------------------------------
void rw_so(){

    gCurrentCommand = gCurrentButton ;
    if (so_start<40) {
		err(7) ;
		return ;
    }
    so_start -= 40 ;
    eraser(0,60,820,764) ;
    displ_symop() ;
}

// ---------------------------------------------------------------------
// Set up for adding a Symmetry Operator to the list of symm. op.  
// ---------------------------------------------------------------------
void add_so() {

    gCalculationIsChanged = TRUE ;
    gPGisChanged = TRUE ;
    gCurrentCommand = gCurrentButton ;
    on_highlight() ;
    add_symop() ;
    off_highlight() ;
    eraser(0,60,820,764) ;
    displ_symop() ;
}

// ---------------------------------------------------------------------
// Add a Symmetry Operator       
// ---------------------------------------------------------------------
void add_symop() {

    eraser(0,60,960,100) ;
    sprintf(stext," Enter symmetry operator # %d -> ",gNumSymops+1) ;
    temdtx(0,70,stext) ;
    read_symop(gNumSymops++) ;
    txtptr_symop[gNumSymops] = NULL ;
    eraser(0,60,960,100) ;
    gInputParameter[10].iv = gNumSymops ;
    gSymopsAreChanged = TRUE ;
}

// ---------------------------------------------------------------------
// Control routine for deleting an atom from the atom list   
// ---------------------------------------------------------------------
void del_at(){

    gCalculationIsChanged = TRUE ;
    gPGisChanged = TRUE ;
    gCurrentCommand = gCurrentButton ;
    on_highlight() ;
    del_atom() ;
    off_highlight() ;
    eraser(0,60,820,764) ;
    if (gCurrentMenu == &gMenuList[VIEW_ATOMS])
		displ_atoms(atom) ;
    else
		displ_atoms(basis) ;
    gBasisIsChanged = TRUE ;
}

// ---------------------------------------------------------------------
// Control routine for deleting a Symmetry Operators from the list   
// ---------------------------------------------------------------------
void del_so() {

	gCalculationIsChanged = TRUE ;
	gPGisChanged = TRUE ;
	gSymopsAreChanged = TRUE ;
	gCurrentCommand = gCurrentButton ;
	on_highlight() ;
	del_symop() ;
	off_highlight() ;
	eraser(0,60,820,764) ;
	displ_symop() ;
}

// ---------------------------------------------------------------------
// Return from the menu controlling atom coordinates               
// ---------------------------------------------------------------------
void ret_at() {

    istart = 0 ;
    CLEAR_DISPLAY ;
    displ_var() ;
    if (!LAYERED)
		set_menu(MAIN_SNGL_STR) ;
    else
		set_menu(MAIN_LAY_STR) ;
    displ_menu() ;
}

// ---------------------------------------------------------------------
// Return from the menu controlling the display of the symmetry oper. 
// ---------------------------------------------------------------------
void ret_so(){

    so_start = 0 ;
    if (gSymopsAreChanged) {
	ApplySymmetries() ;
 	ViewNewUnitCell() ;
    }
    CLEAR_DISPLAY ;
    displ_var() ;
    set_menu(MAIN_SNGL_STR) ;
    displ_menu() ;
}

// ---------------------------------------------------------------------
// Return from the menu controlling atom coordinates in the basis 
// ---------------------------------------------------------------------
void ret_basis() {

    if (gBasisIsChanged)  {
	ApplySymmetries() ;
	gBasisIsChanged = FALSE ;
	sort(atom,gNumUnitCellAtoms) ;
 	ViewNewUnitCell() ;
   }
    istart = 0 ;
    CLEAR_DISPLAY ;
    displ_var() ;
    set_menu(MAIN_SNGL_STR) ;
    displ_menu() ;
}

// ---------------------------------------------------------------------
// Deletes an atom from the list                
// ---------------------------------------------------------------------
void del_atom() {
	long i,n,ntype,total_number ;

	eraser(0,60,960,100) ;
	temdtx(0,70," Enter the number of the atom to be deleted  -> ") ;
	i = get_int(1) ;
	if (gCurrentMenu == &gMenuList[VIEW_ATOMS]) {
		total_number = --gNumUnitCellAtoms ;
		for (n=i-1;n<total_number;n++) atom[n] = atom[n+1] ;
		ntype = types(atom,total_number) ;
	}
	else {
		total_number = --gNumBasisAtoms ;
		gInputParameter[12].iv = total_number ;
		for (n=i-1;n<total_number;n++) basis[n] = basis[n+1] ;
		ntype = types(basis,total_number) ;
	}
	if (ntype != gInputParameter[13].iv) gInputParameter[13].iv = ntype ;
}

// ---------------------------------------------------------------------
// Does the deleting of a symmetry operator 
// ---------------------------------------------------------------------
void del_symop() {
	long i,j ;

	eraser(0,60,1020,100) ;
	temdtx(0,70,
		" Enter the # of the symmetry element to be deleted -> ") ;
	i = get_int(1) ;
	eraser(0,60,1020,100) ;
	if (i != gNumSymops)
		for (j=i-1;j<gNumSymops-1;j++) txtptr_symop[j] = &symop_txt[j+1][0] ;
	txtptr_symop[--gNumSymops] = NULL ;
	gSymopsAreChanged = TRUE ;
	gInputParameter[10].iv = gNumSymops ;
}

// ---------------------------------------------------------------------
// Displays atom coordinates - up to 20 at the time. The starting # 
//  and ending # is kept track of by istart and istop             
// ---------------------------------------------------------------------
void displ_atoms(AtomRecPtr unit) {

	static long ic,n,yl ;
	long i,total_number,x,y ;
	Rect	theRect ;

	if (DIS_ATOM_INITIAL) {
		istart = 0 ;
		istop = 0 ;
		DIS_ATOM_INITIAL = FALSE ;
	}
	if (gCurrentMenu == &gMenuList[VIEW_ATOMS] || gCurrentMenu == &gMenuList[CH_ATOM])
		total_number = gNumUnitCellAtoms ;
	else
		total_number = gNumBasisAtoms ;
	ic = istart ;
	temdtx(20,720,"#") ;
	temdtx(65,720,"TYPE") ;
	temdtx(par_atoms[0].xl-5,720,"NAME") ;
	temdtx(par_atoms[1].xl,720,"    X") ;
	temdtx(par_atoms[2].xl,720,"    Y") ;
	temdtx(par_atoms[3].xl,720,"    Z") ;
	temdtx(par_atoms[4].xl,720," DW") ;
	temdtx(par_atoms[5].xl,720,"OCC") ;
	n = 0 ;
    	long colorIndex ;
	for (i=0;i<20 && ic<total_number;ic++,i++) { 
		yl = par_atoms[n].yl+IDY ;
		sprintf(stext,"%d",ic+1) ;
		temdtx(20,yl,stext) ;
		sprintf(stext,"%d",(unit+ic)->type.val.iv) ;
		temdtx(85,yl,stext) ;
		x = par_atoms[n].xl/2 - 20 ;
		y = (Terminal.yres - yl)/2 - 8 ;
		SetRect(&theRect,x,y,x+8,y+8) ;
		colorIndex = (unit+ic)->no.val.iv % 39 ;
		SetFgColor(gColors[gNumGrays-1+colorIndex].pixel) ;
		FillRect(&theRect) ;
		SetFgColor(gBlackPixel) ;
		FrameRect(&theRect) ;
		sprintf(stext,"%s",(unit+ic)->name.val.cv) ;
		temdtx(par_atoms[n++].xl,yl,stext) ;
		sprintf(stext,"%.4f",(unit+ic)->xc.val.fv) ;
		temdtx(par_atoms[n++].xl,yl,stext) ;
		sprintf(stext,"%.4f",(unit+ic)->yc.val.fv) ;
		temdtx(par_atoms[n++].xl,yl,stext) ;
		sprintf(stext,"%.4f",(unit+ic)->zc.val.fv) ;
		temdtx(par_atoms[n++].xl,yl,stext) ;
		sprintf(stext,"%.4f",(unit+ic)->dw.val.fv) ;
		temdtx(par_atoms[n++].xl,yl,stext) ;
		sprintf(stext,"%.2f",(unit+ic)->occ.val.fv) ;
		temdtx(par_atoms[n++].xl,yl,stext) ;
	}
	istop = ic ;
}

// ---------------------------------------------------------------------
// Displays all the symmetry operators to be applied to the basis/
// ---------------------------------------------------------------------
void displ_symop() {

	static long ic ;
	long i ;

	if (DIS_SO_INITIAL) {
		so_start = 0 ;
		so_stop  = 0 ;
		DIS_SO_INITIAL = FALSE ;
	}
	ic = so_start ;
	temdtx(10,720,
		"  #   SYMMETRY OPERATOR          #   SYMMETRY OPERATOR ") ;
	for (i=0;i<40 && ic<gNumSymops;ic++,i++) { 
		sprintf(stext,"%3d",ic+1) ;
		temdtx(10+(long)(i/20)*414,par_symop[i].yl+IDY,stext) ;
		temdtx(par_symop[i].xl,par_symop[i].yl+IDY,txtptr_symop[ic]) ;
	}
	so_stop = ic ;
}

// ---------------------------------------------------------------------
// This function handles the input for the addition of an atom 
// ---------------------------------------------------------------------
void add_atom() {

	long iz,n,ntype ;
	float x,y,z,dw,occ ;
	AtomRec *unit ;

	if (CurrentMenuIs(VIEW_ATOMS)) {
		n = gNumUnitCellAtoms++ ;
		unit = &atom[n] ;
	}
	else {
		n = gNumBasisAtoms++ ;
		gInputParameter[12].iv = gNumBasisAtoms ;
		unit = &basis[n] ;
	}
	
	do {
		temdtx(0,70," Enter the chemical symbol  -> ") ;
		ReadString(name,3) ;
		iz = atomno(name) ;
		eraser(0,60,960,100) ;
	} while (iz == -1) ;
	
	unit->no.val.iv = iz ;
	unit->name.val.cv = chemsymb[iz-1] ;
	eraser(0,60,960,100) ;
	temdtx(0,70," Enter x  y  z - coordinates  -> ") ;
	x = get_float(0.) ;
	y = get_float(0.) ;
	z = get_float(0.) ;
	eraser(0,60,960,100) ;
	temdtx(0,70," Enter Debye Waller factor  -> ") ;
	dw = get_float(0.5) ;
	eraser(0,60,960,100) ;
	temdtx(0,70," Enter the occupancy factor -> ") ;
	occ = get_float(1.0) ;
	unit->occ.val.fv = occ ;
	unit->xc.val.fv = x ;
	unit->yc.val.fv = y ;
	unit->zc.val.fv = z ;
	unit->dw.val.fv = dw ;
	ntype = (gCurrentMenu == &gMenuList[VIEW_ATOMS] 
	    ? types(atom,++n) : types(basis,++n)) ;
	if (ntype != gInputParameter[13].iv) gInputParameter[13].iv = ntype ;
	if (gCurrentMenu == &gMenuList[VIEW_ATOMS])
		sort(atom,n--) ;
	else
		sort(basis,n--) ;
	unit->type.xl = 85 ;
	unit->type.xh = 135 ;
	unit->name.xl = 165 ;
	unit->name.xh = 215 ;
	unit->xc.xl = 250 ;
	unit->xc.xh = 330 ;
	unit->yc.xl = 400 ;
	unit->yc.xh = 480 ;
	unit->zc.xl = 530 ;
	unit->zc.xh = 610 ;
	unit->dw.xl = 660 ;
	unit->dw.xh = 740 ;
	unit->occ.xl  = 760 ;
	unit->occ.xh  = 800 ;
	unit->name.yl = unit->type.yl = unit->xc.yl = unit->yc.yl =
	unit->occ.yl = unit->zc.yl = unit->dw.yl = 680-(n%20)*30 ;    
	unit->name.yh = unit->type.yh = unit->xc.yh = unit->yc.yh =
	unit->occ.yh = unit->zc.yh = unit->dw.yh = 680-(n%20)*30+26 ;    
}

// ---------------------------------------------------------------------
// Sets up for adding an atom to the list of atom positions 
// ---------------------------------------------------------------------
void add_at() {

    gCalculationIsChanged = TRUE ;
    gPGisChanged = TRUE ;
    gCurrentCommand = gCurrentButton ;
    on_highlight() ;
    add_atom() ;
    off_highlight() ;
    eraser(0,60,820,764) ;
    if (CurrentMenuIs(VIEW_ATOMS)) {
		displ_atoms(atom) ;
    }
    else
		displ_atoms(basis) ;
    gBasisIsChanged = TRUE ;
}
